@@ -7,13 +7,13 @@
 #include <sstream>
 
 #include "base/files/file_path.h"
-#include "base/macros.h"
+#include "base/stl_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
 
-#if defined(OS_POSIX)
+#if defined(OS_POSIX) || defined(OS_FUCHSIA)
 #include "base/test/scoped_locale.h"
 #endif
 
@@ -21,7 +21,7 @@
 #define FPL(x) FILE_PATH_LITERAL(x)
 
 // This macro constructs strings which can contain NULs.
-#define FPS(x) FilePath::StringType(FPL(x), arraysize(FPL(x)) - 1)
+#define FPS(x) FilePath::StringType(FPL(x), base::size(FPL(x)) - 1)
 
 namespace base {
 
@@ -89,6 +89,7 @@ TEST_F(FilePathTest, DirName) {
     { FPL("{:"),            FPL(".") },
     { FPL("\xB3:"),         FPL(".") },
     { FPL("\xC5:"),         FPL(".") },
+    { FPL("/aa/../bb/cc"),  FPL("/aa/../bb")},
 #if defined(OS_WIN)
     { FPL("\x0143:"),       FPL(".") },
 #endif  // OS_WIN
@@ -128,6 +129,7 @@ TEST_F(FilePathTest, DirName) {
     { FPL("\\\\aa\\bb"),    FPL("\\\\aa") },
     { FPL("\\\\aa\\"),      FPL("\\\\") },
     { FPL("\\\\aa"),        FPL("\\\\") },
+    { FPL("aa\\..\\bb\\c"), FPL("aa\\..\\bb")},
 #if defined(FILE_PATH_USES_DRIVE_LETTERS)
     { FPL("c:\\"),          FPL("c:\\") },
     { FPL("c:\\\\"),        FPL("c:\\\\") },
@@ -140,7 +142,7 @@ TEST_F(FilePathTest, DirName) {
 #endif  // FILE_PATH_USES_WIN_SEPARATORS
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath input(cases[i].input);
     FilePath observed = input.DirName();
     EXPECT_EQ(FilePath::StringType(cases[i].expected), observed.value()) <<
@@ -227,7 +229,7 @@ TEST_F(FilePathTest, BaseName) {
 #endif  // FILE_PATH_USES_WIN_SEPARATORS
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath input(cases[i].input);
     FilePath observed = input.BaseName();
     EXPECT_EQ(FilePath::StringType(cases[i].expected), observed.value()) <<
@@ -305,7 +307,7 @@ TEST_F(FilePathTest, Append) {
 #endif  // FILE_PATH_USES_WIN_SEPARATORS
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath root(cases[i].inputs[0]);
     FilePath::StringType leaf(cases[i].inputs[1]);
     FilePath observed_str = root.Append(leaf);
@@ -319,7 +321,7 @@ TEST_F(FilePathTest, Append) {
     // handle the case when AppendASCII is passed UTF8
 #if defined(OS_WIN)
     std::string ascii = WideToUTF8(leaf);
-#elif defined(OS_POSIX)
+#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
     std::string ascii = leaf;
 #endif
     observed_str = root.AppendASCII(ascii);
@@ -384,7 +386,7 @@ TEST_F(FilePathTest, StripTrailingSeparators) {
 #endif  // FILE_PATH_USES_WIN_SEPARATORS
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath input(cases[i].input);
     FilePath observed = input.StripTrailingSeparators();
     EXPECT_EQ(FilePath::StringType(cases[i].expected), observed.value()) <<
@@ -452,7 +454,7 @@ TEST_F(FilePathTest, IsAbsolute) {
 #endif  // FILE_PATH_USES_WIN_SEPARATORS
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath input(cases[i].input);
     bool observed = input.IsAbsolute();
     EXPECT_EQ(cases[i].expected, observed) <<
@@ -498,15 +500,15 @@ TEST_F(FilePathTest, PathComponentsTest) {
 #endif  // FILE_PATH_USES_WIN_SEPARATORS
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath input(cases[i].input);
     std::vector<FilePath::StringType> comps;
     input.GetComponents(&comps);
 
     FilePath::StringType observed;
-    for (size_t j = 0; j < comps.size(); ++j) {
+    for (const auto& j : comps) {
       observed.append(FILE_PATH_LITERAL("|"), 1);
-      observed.append(comps[j]);
+      observed.append(j);
     }
     EXPECT_EQ(FilePath::StringType(cases[i].expected), observed) <<
               "i: " << i << ", input: " << input.value();
@@ -516,6 +518,7 @@ TEST_F(FilePathTest, PathComponentsTest) {
 TEST_F(FilePathTest, IsParentTest) {
   const struct BinaryBooleanTestData cases[] = {
     { { FPL("/"),             FPL("/foo/bar/baz") },      true},
+    { { FPL("/foo"),          FPL("/foo/bar/baz") },      true},
     { { FPL("/foo/bar"),      FPL("/foo/bar/baz") },      true},
     { { FPL("/foo/bar/"),     FPL("/foo/bar/baz") },      true},
     { { FPL("//foo/bar/"),    FPL("//foo/bar/baz") },     true},
@@ -524,6 +527,7 @@ TEST_F(FilePathTest, IsParentTest) {
     { { FPL("/foo/bar"),      FPL("/foo/bar2/baz") },     false},
     { { FPL("/foo/bar"),      FPL("/foo/bar") },          false},
     { { FPL("/foo/bar/baz"),  FPL("/foo/bar") },          false},
+    { { FPL("foo"),           FPL("foo/bar/baz") },       true},
     { { FPL("foo/bar"),       FPL("foo/bar/baz") },       true},
     { { FPL("foo/bar"),       FPL("foo2/bar/baz") },      false},
     { { FPL("foo/bar"),       FPL("foo/bar2/baz") },      false},
@@ -555,7 +559,7 @@ TEST_F(FilePathTest, IsParentTest) {
 #endif  // FILE_PATH_USES_WIN_SEPARATORS
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath parent(cases[i].inputs[0]);
     FilePath child(cases[i].inputs[1]);
 
@@ -619,7 +623,7 @@ TEST_F(FilePathTest, AppendRelativePathTest) {
 
   const FilePath base(FPL("blah"));
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath parent(cases[i].inputs[0]);
     FilePath child(cases[i].inputs[1]);
     {
@@ -686,7 +690,7 @@ TEST_F(FilePathTest, EqualityTest) {
 #endif  // FILE_PATH_USES_WIN_SEPARATORS
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath a(cases[i].inputs[0]);
     FilePath b(cases[i].inputs[1]);
 
@@ -695,7 +699,7 @@ TEST_F(FilePathTest, EqualityTest) {
       b.value();
   }
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath a(cases[i].inputs[0]);
     FilePath b(cases[i].inputs[1]);
 
@@ -769,7 +773,7 @@ TEST_F(FilePathTest, Extension2) {
     { FPL("foo.user.js"),            FPL(".user.js") },
     { FPL("/foo.tar.bz"),            FPL(".tar.bz") },
   };
-  for (unsigned int i = 0; i < arraysize(cases); ++i) {
+  for (unsigned int i = 0; i < base::size(cases); ++i) {
     FilePath path(cases[i].input);
     FilePath::StringType extension = path.Extension();
     FilePath::StringType final_extension = path.FinalExtension();
@@ -778,7 +782,7 @@ TEST_F(FilePathTest, Extension2) {
     EXPECT_STREQ(cases[i].expected, final_extension.c_str())
         << "i: " << i << ", path: " << path.value();
   }
-  for (unsigned int i = 0; i < arraysize(double_extension_cases); ++i) {
+  for (unsigned int i = 0; i < base::size(double_extension_cases); ++i) {
     FilePath path(double_extension_cases[i].input);
     FilePath::StringType extension = path.Extension();
     EXPECT_STREQ(double_extension_cases[i].expected, extension.c_str())
@@ -846,7 +850,7 @@ TEST_F(FilePathTest, InsertBeforeExtension) {
     { { FPL("/bar/baz/foo.exe"), FPL(" (1)") },   FPL("/bar/baz/foo (1).exe") },
     { { FPL("/bar/baz/..////"), FPL(" (1)") },    FPL("") },
   };
-  for (unsigned int i = 0; i < arraysize(cases); ++i) {
+  for (unsigned int i = 0; i < base::size(cases); ++i) {
     FilePath path(cases[i].inputs[0]);
     FilePath result = path.InsertBeforeExtension(cases[i].inputs[1]);
     EXPECT_EQ(cases[i].expected, result.value()) << "i: " << i <<
@@ -873,7 +877,7 @@ TEST_F(FilePathTest, RemoveExtension) {
     { FPL("/foo.bar/foo"),        FPL("/foo.bar/foo") },
     { FPL("/foo.bar/..////"),     FPL("/foo.bar/..////") },
   };
-  for (unsigned int i = 0; i < arraysize(cases); ++i) {
+  for (unsigned int i = 0; i < base::size(cases); ++i) {
     FilePath path(cases[i].input);
     FilePath removed = path.RemoveExtension();
     FilePath removed_final = path.RemoveFinalExtension();
@@ -922,7 +926,7 @@ TEST_F(FilePathTest, ReplaceExtension) {
     { { FPL("/foo.bar/foo"),        FPL("baz") }, FPL("/foo.bar/foo.baz") },
     { { FPL("/foo.bar/..////"),     FPL("baz") }, FPL("") },
   };
-  for (unsigned int i = 0; i < arraysize(cases); ++i) {
+  for (unsigned int i = 0; i < base::size(cases); ++i) {
     FilePath path(cases[i].inputs[0]);
     FilePath replaced = path.ReplaceExtension(cases[i].inputs[1]);
     EXPECT_EQ(cases[i].expected, replaced.value()) << "i: " << i <<
@@ -960,7 +964,7 @@ TEST_F(FilePathTest, AddExtension) {
     { { FPL("/foo.bar/foo"),        FPL("baz") }, FPL("/foo.bar/foo.baz") },
     { { FPL("/foo.bar/..////"),     FPL("baz") }, FPL("") },
   };
-  for (unsigned int i = 0; i < arraysize(cases); ++i) {
+  for (unsigned int i = 0; i < base::size(cases); ++i) {
     FilePath path(cases[i].inputs[0]);
     FilePath added = path.AddExtension(cases[i].inputs[1]);
     EXPECT_EQ(cases[i].expected, added.value()) << "i: " << i <<
@@ -1004,7 +1008,7 @@ TEST_F(FilePathTest, MatchesExtension) {
 #endif
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath path(cases[i].inputs[0]);
     FilePath::StringType ext(cases[i].inputs[1]);
 
@@ -1081,7 +1085,7 @@ TEST_F(FilePathTest, CompareIgnoreCase) {
 #endif
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath::StringType s1(cases[i].inputs[0]);
     FilePath::StringType s2(cases[i].inputs[1]);
     int result = FilePath::CompareIgnoreCase(s1, s2);
@@ -1116,7 +1120,7 @@ TEST_F(FilePathTest, ReferencesParent) {
     { FPL("a/b/c"),    false },
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath input(cases[i].input);
     bool observed = input.ReferencesParent();
     EXPECT_EQ(cases[i].expected, observed) <<
@@ -1138,7 +1142,7 @@ TEST_F(FilePathTest, FromUTF8Unsafe_And_AsUTF8Unsafe) {
   ScopedLocale locale("en_US.UTF-8");
 #endif
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     // Test FromUTF8Unsafe() works.
     FilePath from_utf8 = FilePath::FromUTF8Unsafe(cases[i].utf8);
     EXPECT_EQ(cases[i].native, from_utf8.value())
@@ -1221,7 +1225,7 @@ TEST_F(FilePathTest, NormalizePathSeparators) {
     { FPL("foo/\\bar/\\"), FPL("foo\\\\bar\\\\") },
     { FPL("/\\foo\\/bar"), FPL("\\\\foo\\\\bar") },
   };
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath input(cases[i].input);
     FilePath observed = input.NormalizePathSeparators();
     EXPECT_EQ(FilePath::StringType(cases[i].expected), observed.value()) <<
@@ -1238,9 +1242,9 @@ TEST_F(FilePathTest, EndsWithSeparator) {
     { FPL("bar"), false },
     { FPL("/foo/bar"), false },
   };
-  for (size_t i = 0; i < arraysize(cases); ++i) {
-    FilePath input = FilePath(cases[i].input).NormalizePathSeparators();
-    EXPECT_EQ(cases[i].expected, input.EndsWithSeparator());
+  for (const auto& i : cases) {
+    FilePath input = FilePath(i.input).NormalizePathSeparators();
+    EXPECT_EQ(i.expected, input.EndsWithSeparator());
   }
 }
 
@@ -1251,9 +1255,9 @@ TEST_F(FilePathTest, AsEndingWithSeparator) {
     { FPL("foo"), FPL("foo/") },
     { FPL("foo/"), FPL("foo/") }
   };
-  for (size_t i = 0; i < arraysize(cases); ++i) {
-    FilePath input = FilePath(cases[i].input).NormalizePathSeparators();
-    FilePath expected = FilePath(cases[i].expected).NormalizePathSeparators();
+  for (const auto& i : cases) {
+    FilePath input = FilePath(i.input).NormalizePathSeparators();
+    FilePath expected = FilePath(i.expected).NormalizePathSeparators();
     EXPECT_EQ(expected.value(), input.AsEndingWithSeparator().value());
   }
 }
@@ -1278,7 +1282,7 @@ TEST_F(FilePathTest, ContentUriTest) {
     { FPL("content%2a%2f%2f"),     false },
   };
 
-  for (size_t i = 0; i < arraysize(cases); ++i) {
+  for (size_t i = 0; i < base::size(cases); ++i) {
     FilePath input(cases[i].input);
     bool observed = input.IsContentUri();
     EXPECT_EQ(cases[i].expected, observed) <<
@@ -1287,12 +1291,11 @@ TEST_F(FilePathTest, ContentUriTest) {
 }
 #endif
 
-// Test the PrintTo overload for FilePath (used when a test fails to compare two
-// FilePaths).
-TEST_F(FilePathTest, PrintTo) {
+// Test the operator<<(ostream, FilePath).
+TEST_F(FilePathTest, PrintToOstream) {
   std::stringstream ss;
   FilePath fp(FPL("foo"));
-  base::PrintTo(fp, &ss);
+  ss << fp;
   EXPECT_EQ("foo", ss.str());
 }
 
