@@ -34,8 +34,9 @@
 #include "base/single_thread_task_runner.h"
 #include "base/stl_util.h"
 #include "base/synchronization/lock.h"
+#include "base/threading/platform_thread.h"
+#include "base/threading/scoped_blocking_call.h"
 #include "base/threading/sequenced_task_runner_handle.h"
-#include "base/threading/thread.h"
 #include "base/trace_event/trace_event.h"
 
 namespace base {
@@ -43,17 +44,32 @@ namespace base {
 namespace {
 
 class FilePathWatcherImpl;
+class InotifyReader;
+
+class InotifyReaderThreadDelegate final : public PlatformThread::Delegate {
+ public:
+  explicit InotifyReaderThreadDelegate(int inotify_fd)
+      : inotify_fd_(inotify_fd) {}
+  ~InotifyReaderThreadDelegate() override = default;
+
+ private:
+  void ThreadMain() override;
+
+  const int inotify_fd_;
+
+  DISALLOW_COPY_AND_ASSIGN(InotifyReaderThreadDelegate);
+};
 
 // Singleton to manage all inotify watches.
 // TODO(tony): It would be nice if this wasn't a singleton.
 // http://crbug.com/38174
 class InotifyReader {
  public:
-  typedef int Watch;  // Watch descriptor used by AddWatch and RemoveWatch.
-  static const Watch kInvalidWatch = -1;
+  using Watch = int;  // Watch descriptor used by AddWatch() and RemoveWatch().
+  static constexpr Watch kInvalidWatch = -1;
 
   // Watch directory |path| for changes. |watcher| will be notified on each
-  // change. Returns kInvalidWatch on failure.
+  // change. Returns |kInvalidWatch| on failure.
   Watch AddWatch(const FilePath& path, FilePathWatcherImpl* watcher);
 
   // Remove |watch| if it's valid.
@@ -65,27 +81,28 @@ class InotifyReader {
  private:
   friend struct LazyInstanceTraitsBase<InotifyReader>;
 
-  typedef std::set<FilePathWatcherImpl*> WatcherSet;
-
   InotifyReader();
   // There is no destructor because |g_inotify_reader| is a
   // base::LazyInstace::Leaky object. Having a destructor causes build
   // issues with GCC 6 (http://crbug.com/636346).
 
-  // We keep track of which delegates want to be notified on which watches.
-  std::unordered_map<Watch, WatcherSet> watchers_;
+  // Returns true on successful thread creation.
+  bool StartThread();
 
-  // Lock to protect watchers_.
+  // Lock to protect |watchers_|.
   Lock lock_;
 
-  // Separate thread on which we run blocking read for inotify events.
-  Thread thread_;
+  // We keep track of which delegates want to be notified on which watches.
+  std::unordered_map<Watch, std::set<FilePathWatcherImpl*>> watchers_;
 
   // File descriptor returned by inotify_init.
   const int inotify_fd_;
 
+  // Thread delegate for the Inotify thread.
+  InotifyReaderThreadDelegate thread_delegate_;
+
   // Flag set to true when startup was successful.
-  bool valid_;
+  bool valid_ = false;
 
   DISALLOW_COPY_AND_ASSIGN(InotifyReader);
 };
@@ -140,7 +157,6 @@ class FilePathWatcherImpl : public FilePathWatcher::PlatformDelegate {
     FilePath::StringType subdir;
     FilePath::StringType linkname;
   };
-  typedef std::vector<WatchEntry> WatchVector;
 
   // Reconfigure to watch for the most specific parent directory of |target_|
   // that exists. Also calls UpdateRecursiveWatches() below.
@@ -177,36 +193,46 @@ class FilePathWatcherImpl : public FilePathWatcher::PlatformDelegate {
   // The file or directory we're supposed to watch.
   FilePath target_;
 
-  bool recursive_;
+  bool recursive_ = false;
 
   // The vector of watches and next component names for all path components,
   // starting at the root directory. The last entry corresponds to the watch for
   // |target_| and always stores an empty next component name in |subdir|.
-  WatchVector watches_;
+  std::vector<WatchEntry> watches_;
 
   std::unordered_map<InotifyReader::Watch, FilePath> recursive_paths_by_watch_;
   std::map<FilePath, InotifyReader::Watch> recursive_watches_by_path_;
 
+  // Read only while INotifyReader::lock_ is held, and used to post asynchronous
+  // notifications to the Watcher on its home task_runner(). Ideally this should
+  // be const, but since it is initialized from |weak_factory_|, which must
+  // appear after it, that is not possible.
+  WeakPtr<FilePathWatcherImpl> weak_ptr_;
+
   WeakPtrFactory<FilePathWatcherImpl> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(FilePathWatcherImpl);
 };
 
-void InotifyReaderCallback(InotifyReader* reader, int inotify_fd) {
-  // Make sure the file descriptors are good for use with select().
-  CHECK_LE(0, inotify_fd);
-  CHECK_GT(FD_SETSIZE, inotify_fd);
+LazyInstance<InotifyReader>::Leaky g_inotify_reader = LAZY_INSTANCE_INITIALIZER;
 
-  trace_event::TraceLog::GetInstance()->SetCurrentThreadBlocksMessageLoop();
+void InotifyReaderThreadDelegate::ThreadMain() {
+  PlatformThread::SetName("inotify_reader");
+
+  // Make sure the file descriptors are good for use with select().
+  CHECK_LE(0, inotify_fd_);
+  CHECK_GT(FD_SETSIZE, inotify_fd_);
 
   while (true) {
     fd_set rfds;
     FD_ZERO(&rfds);
-    FD_SET(inotify_fd, &rfds);
+    FD_SET(inotify_fd_, &rfds);
+
+    ScopedBlockingCall scoped_blocking_call(BlockingType::WILL_BLOCK);
 
     // Wait until some inotify events are available.
     int select_result =
-      HANDLE_EINTR(select(inotify_fd + 1, &rfds, NULL, NULL, NULL));
+        HANDLE_EINTR(select(inotify_fd_ + 1, &rfds, nullptr, nullptr, nullptr));
     if (select_result < 0) {
       DPLOG(WARNING) << "select failed";
       return;
@@ -214,8 +240,7 @@ void InotifyReaderCallback(InotifyReader* reader, int inotify_fd) {
 
     // Adjust buffer size to current event queue size.
     int buffer_size;
-    int ioctl_result = HANDLE_EINTR(ioctl(inotify_fd, FIONREAD,
-                                          &buffer_size));
+    int ioctl_result = HANDLE_EINTR(ioctl(inotify_fd_, FIONREAD, &buffer_size));
 
     if (ioctl_result != 0) {
       DPLOG(WARNING) << "ioctl failed";
@@ -224,8 +249,8 @@ void InotifyReaderCallback(InotifyReader* reader, int inotify_fd) {
 
     std::vector<char> buffer(buffer_size);
 
-    ssize_t bytes_read = HANDLE_EINTR(read(inotify_fd, &buffer[0],
-                                           buffer_size));
+    ssize_t bytes_read =
+        HANDLE_EINTR(read(inotify_fd_, &buffer[0], buffer_size));
 
     if (bytes_read < 0) {
       DPLOG(WARNING) << "read from inotify fd failed";
@@ -237,27 +262,29 @@ void InotifyReaderCallback(InotifyReader* reader, int inotify_fd) {
       inotify_event* event = reinterpret_cast<inotify_event*>(&buffer[i]);
       size_t event_size = sizeof(inotify_event) + event->len;
       DCHECK(i + event_size <= static_cast<size_t>(bytes_read));
-      reader->OnInotifyEvent(event);
+      g_inotify_reader.Get().OnInotifyEvent(event);
       i += event_size;
     }
   }
 }
 
-static LazyInstance<InotifyReader>::Leaky g_inotify_reader =
-    LAZY_INSTANCE_INITIALIZER;
-
 InotifyReader::InotifyReader()
-    : thread_("inotify_reader"),
-      inotify_fd_(inotify_init()),
-      valid_(false) {
-  if (inotify_fd_ < 0)
+    : inotify_fd_(inotify_init()), thread_delegate_(inotify_fd_) {
+  if (inotify_fd_ < 0) {
     PLOG(ERROR) << "inotify_init() failed";
-
-  if (inotify_fd_ >= 0 && thread_.Start()) {
-    thread_.task_runner()->PostTask(
-        FROM_HERE, BindOnce(&InotifyReaderCallback, this, inotify_fd_));
-    valid_ = true;
+    return;
   }
+
+  if (!StartThread())
+    return;
+
+  valid_ = true;
+}
+
+bool InotifyReader::StartThread() {
+  // This object is LazyInstance::Leaky, so thread_delegate_ will outlive the
+  // thread.
+  return PlatformThread::CreateNonJoinable(0, &thread_delegate_);
 }
 
 InotifyReader::Watch InotifyReader::AddWatch(
@@ -267,6 +294,7 @@ InotifyReader::Watch InotifyReader::AddWatch(
 
   AutoLock auto_lock(lock_);
 
+  ScopedBlockingCall scoped_blocking_call(BlockingType::WILL_BLOCK);
   Watch watch = inotify_add_watch(inotify_fd_, path.value().c_str(),
                                   IN_ATTRIB | IN_CREATE | IN_DELETE |
                                   IN_CLOSE_WRITE | IN_MOVE |
@@ -290,6 +318,8 @@ void InotifyReader::RemoveWatch(Watch watch, FilePathWatcherImpl* watcher) {
 
   if (watchers_[watch].empty()) {
     watchers_.erase(watch);
+
+    ScopedBlockingCall scoped_blocking_call(BlockingType::WILL_BLOCK);
     inotify_rm_watch(inotify_fd_, watch);
   }
 }
@@ -301,22 +331,20 @@ void InotifyReader::OnInotifyEvent(const inotify_event* event) {
   FilePath::StringType child(event->len ? event->name : FILE_PATH_LITERAL(""));
   AutoLock auto_lock(lock_);
 
-  for (WatcherSet::iterator watcher = watchers_[event->wd].begin();
-       watcher != watchers_[event->wd].end();
-       ++watcher) {
-    (*watcher)->OnFilePathChanged(event->wd,
-                                  child,
-                                  event->mask & (IN_CREATE | IN_MOVED_TO),
-                                  event->mask & (IN_DELETE | IN_MOVED_FROM),
-                                  event->mask & IN_ISDIR);
+  auto& watcher_set = watchers_[event->wd];
+  for (FilePathWatcherImpl* watcher : watcher_set) {
+    watcher->OnFilePathChanged(
+        event->wd, child, event->mask & (IN_CREATE | IN_MOVED_TO),
+        event->mask & (IN_DELETE | IN_MOVED_FROM), event->mask & IN_ISDIR);
   }
 }
 
-FilePathWatcherImpl::FilePathWatcherImpl()
-    : recursive_(false), weak_factory_(this) {}
+FilePathWatcherImpl::FilePathWatcherImpl() : weak_factory_(this) {
+  weak_ptr_ = weak_factory_.GetWeakPtr();
+}
 
 FilePathWatcherImpl::~FilePathWatcherImpl() {
-  DCHECK(!task_runner() || task_runner()->RunsTasksOnCurrentThread());
+  DCHECK(!task_runner() || task_runner()->RunsTasksInCurrentSequence());
 }
 
 void FilePathWatcherImpl::OnFilePathChanged(InotifyReader::Watch fired_watch,
@@ -324,7 +352,7 @@ void FilePathWatcherImpl::OnFilePathChanged(InotifyReader::Watch fired_watch,
                                             bool created,
                                             bool deleted,
                                             bool is_dir) {
-  DCHECK(!task_runner()->RunsTasksOnCurrentThread());
+  DCHECK(!task_runner()->RunsTasksInCurrentSequence());
 
   // This method is invoked on the Inotify thread. Switch to task_runner() to
   // access |watches_| safely. Use a WeakPtr to prevent the callback from
@@ -332,8 +360,7 @@ void FilePathWatcherImpl::OnFilePathChanged(InotifyReader::Watch fired_watch,
   task_runner()->PostTask(
       FROM_HERE,
       BindOnce(&FilePathWatcherImpl::OnFilePathChangedOnOriginSequence,
-               weak_factory_.GetWeakPtr(), fired_watch, child, created, deleted,
-               is_dir));
+               weak_ptr_, fired_watch, child, created, deleted, is_dir));
 }
 
 void FilePathWatcherImpl::OnFilePathChangedOnOriginSequence(
@@ -342,7 +369,7 @@ void FilePathWatcherImpl::OnFilePathChangedOnOriginSequence(
     bool created,
     bool deleted,
     bool is_dir) {
-  DCHECK(task_runner()->RunsTasksOnCurrentThread());
+  DCHECK(task_runner()->RunsTasksInCurrentSequence());
   DCHECK(!watches_.empty());
   DCHECK(HasValidWatchVector());
 
@@ -451,14 +478,14 @@ void FilePathWatcherImpl::Cancel() {
     return;
   }
 
-  DCHECK(task_runner()->RunsTasksOnCurrentThread());
+  DCHECK(task_runner()->RunsTasksInCurrentSequence());
   DCHECK(!is_cancelled());
 
   set_cancelled();
   callback_.Reset();
 
-  for (size_t i = 0; i < watches_.size(); ++i)
-    g_inotify_reader.Get().RemoveWatch(watches_[i].watch, this);
+  for (const auto& watch : watches_)
+    g_inotify_reader.Get().RemoveWatch(watch.watch, this);
   watches_.clear();
   target_.clear();
   RemoveRecursiveWatches();
@@ -467,13 +494,12 @@ void FilePathWatcherImpl::Cancel() {
 void FilePathWatcherImpl::UpdateWatches() {
   // Ensure this runs on the task_runner() exclusively in order to avoid
   // concurrency issues.
-  DCHECK(task_runner()->RunsTasksOnCurrentThread());
+  DCHECK(task_runner()->RunsTasksInCurrentSequence());
   DCHECK(HasValidWatchVector());
 
   // Walk the list of watches and update them as we go.
   FilePath path(FILE_PATH_LITERAL("/"));
-  for (size_t i = 0; i < watches_.size(); ++i) {
-    WatchEntry& watch_entry = watches_[i];
+  for (WatchEntry& watch_entry : watches_) {
     InotifyReader::Watch old_watch = watch_entry.watch;
     watch_entry.watch = InotifyReader::kInvalidWatch;
     watch_entry.linkname.clear();
@@ -525,9 +551,8 @@ void FilePathWatcherImpl::UpdateRecursiveWatches(
       recursive_paths_by_watch_[fired_watch] :
       target_;
 
-  std::map<FilePath, InotifyReader::Watch>::iterator start_it =
-      recursive_watches_by_path_.lower_bound(changed_dir);
-  std::map<FilePath, InotifyReader::Watch>::iterator end_it = start_it;
+  auto start_it = recursive_watches_by_path_.lower_bound(changed_dir);
+  auto end_it = start_it;
   for (; end_it != recursive_watches_by_path_.end(); ++end_it) {
     const FilePath& cur_path = end_it->first;
     if (!changed_dir.IsParent(cur_path))
@@ -644,7 +669,7 @@ bool FilePathWatcherImpl::HasValidWatchVector() const {
 
 FilePathWatcher::FilePathWatcher() {
   sequence_checker_.DetachFromSequence();
-  impl_ = MakeUnique<FilePathWatcherImpl>();
+  impl_ = std::make_unique<FilePathWatcherImpl>();
 }
 
 }  // namespace base
