@@ -31,12 +31,10 @@
 #include "cc/layers/performance_properties.h"
 #include "cc/layers/render_surface_impl.h"
 #include "cc/layers/touch_action_region.h"
-#include "cc/quads/shared_quad_state.h"
-#include "cc/resources/resource_provider.h"
 #include "cc/tiles/tile_priority.h"
 #include "cc/trees/element_id.h"
-#include "cc/trees/mutator_host_client.h"
 #include "cc/trees/target_property.h"
+#include "components/viz/common/quads/shared_quad_state.h"
 #include "third_party/skia/include/core/SkColor.h"
 #include "ui/gfx/geometry/point3_f.h"
 #include "ui/gfx/geometry/rect.h"
@@ -46,20 +44,22 @@
 
 namespace base {
 namespace trace_event {
-class ConvertableToTraceFormat;
 class TracedValue;
 }
 class DictionaryValue;
 }
 
+namespace viz {
+class ClientResourceProvider;
+class RenderPass;
+}
+
 namespace cc {
 
 class AppendQuadsData;
 class LayerTreeImpl;
 class MicroBenchmarkImpl;
-class MutatorHost;
 class PrioritizedTile;
-class RenderPass;
 class ScrollbarLayerImplBase;
 class SimpleEnclosedRegion;
 class Tile;
@@ -83,8 +83,8 @@ class CC_EXPORT LayerImpl {
 
   int id() const { return layer_id_; }
 
-  // Interactions with attached animations.
-  gfx::ScrollOffset ScrollOffsetForAnimation() const;
+  // Whether this layer is on the active tree, return false if it's on the
+  // pending tree.
   bool IsActive() const;
 
   void SetHasTransformNode(bool val) { has_transform_node_ = val; }
@@ -105,50 +105,49 @@ class CC_EXPORT LayerImpl {
   void SetScrollTreeIndex(int index);
   int scroll_tree_index() const { return scroll_tree_index_; }
 
-  void set_offset_to_transform_parent(const gfx::Vector2dF& offset) {
+  void SetOffsetToTransformParent(const gfx::Vector2dF& offset) {
     offset_to_transform_parent_ = offset;
   }
   gfx::Vector2dF offset_to_transform_parent() const {
     return offset_to_transform_parent_;
   }
 
-  void set_should_flatten_transform_from_property_tree(bool should_flatten) {
-    should_flatten_transform_from_property_tree_ = should_flatten;
+  void SetShouldFlattenScreenSpaceTransformFromPropertyTree(
+      bool should_flatten) {
+    should_flatten_screen_space_transform_from_property_tree_ = should_flatten;
   }
-  bool should_flatten_transform_from_property_tree() const {
-    return should_flatten_transform_from_property_tree_;
+  bool should_flatten_screen_space_transform_from_property_tree() const {
+    return should_flatten_screen_space_transform_from_property_tree_;
   }
 
   bool is_clipped() const { return draw_properties_.is_clipped; }
 
-  void UpdatePropertyTreeScrollOffset();
-
   LayerTreeImpl* layer_tree_impl() const { return layer_tree_impl_; }
 
-  void PopulateSharedQuadState(SharedQuadState* state) const;
-  void PopulateScaledSharedQuadState(SharedQuadState* state,
+  void PopulateSharedQuadState(viz::SharedQuadState* state,
+                               bool contents_opaque) const;
+  void PopulateScaledSharedQuadState(viz::SharedQuadState* state,
                                      float layer_to_content_scale_x,
-                                     float layer_to_content_scale_y) const;
+                                     float layer_to_content_scale_y,
+                                     bool contents_opaque) const;
   // WillDraw must be called before AppendQuads. If WillDraw returns false,
   // AppendQuads and DidDraw will not be called. If WillDraw returns true,
   // DidDraw is guaranteed to be called before another WillDraw or before
-  // the layer is destroyed. To enforce this, any class that overrides
-  // WillDraw/DidDraw must call the base class version only if WillDraw
-  // returns true.
+  // the layer is destroyed.
   virtual bool WillDraw(DrawMode draw_mode,
-                        ResourceProvider* resource_provider);
-  virtual void AppendQuads(RenderPass* render_pass,
+                        viz::ClientResourceProvider* resource_provider);
+  virtual void AppendQuads(viz::RenderPass* render_pass,
                            AppendQuadsData* append_quads_data) {}
-  virtual void DidDraw(ResourceProvider* resource_provider);
+  virtual void DidDraw(viz::ClientResourceProvider* resource_provider);
 
   // Verify that the resource ids in the quad are valid.
-  void ValidateQuadResources(DrawQuad* quad) const {
+  void ValidateQuadResources(viz::DrawQuad* quad) const {
 #if DCHECK_IS_ON()
     ValidateQuadResourcesInternal(quad);
 #endif
   }
 
-  virtual void GetContentsResourceId(ResourceId* resource_id,
+  virtual void GetContentsResourceId(viz::ResourceId* resource_id,
                                      gfx::Size* resource_size,
                                      gfx::SizeF* resource_uv_size) const;
 
@@ -160,8 +159,15 @@ class CC_EXPORT LayerImpl {
   void SetDrawsContent(bool draws_content);
   bool DrawsContent() const { return draws_content_; }
 
-  void SetShouldHitTest(bool should_hit_test);
-  bool should_hit_test() const { return should_hit_test_; }
+  // Make the layer hit test (see: |should_hit_test|) even if !draws_content_.
+  void SetHitTestableWithoutDrawsContent(bool should_hit_test);
+  bool hit_testable_without_draws_content() const {
+    return hit_testable_without_draws_content_;
+  }
+
+  // True if either the layer draws content or has been marked as hit testable
+  // without draws_content.
+  bool ShouldHitTest() const;
 
   LayerImplTestProperties* test_properties() {
     if (!test_properties_)
@@ -176,8 +182,6 @@ class CC_EXPORT LayerImpl {
   // non-opaque color.  Tries to return background_color(), if possible.
   SkColor SafeOpaqueBackgroundColor() const;
 
-  bool HasPotentiallyRunningFilterAnimation() const;
-
   void SetMasksToBounds(bool masks_to_bounds);
   bool masks_to_bounds() const { return masks_to_bounds_; }
 
@@ -185,18 +189,11 @@ class CC_EXPORT LayerImpl {
   bool contents_opaque() const { return contents_opaque_; }
 
   float Opacity() const;
-  const gfx::Transform& Transform() const;
 
   // Stable identifier for clients. See comment in cc/trees/element_id.h.
   void SetElementId(ElementId element_id);
   ElementId element_id() const { return element_id_; }
 
-  void SetMutableProperties(uint32_t properties);
-  uint32_t mutable_properties() const { return mutable_properties_; }
-
-  void SetPosition(const gfx::PointF& position);
-  gfx::PointF position() const { return position_; }
-
   bool IsAffectedByPageScale() const;
 
   bool Is3dSorted() const { return GetSortingContextId() != 0; }
@@ -208,6 +205,9 @@ class CC_EXPORT LayerImpl {
     return use_parent_backface_visibility_;
   }
 
+  bool IsResizedByBrowserControls() const;
+  void SetIsResizedByBrowserControls(bool resized);
+
   void SetShouldCheckBackfaceVisibility(bool should_check_backface_visibility) {
     should_check_backface_visibility_ = should_check_backface_visibility;
   }
@@ -268,6 +268,8 @@ class CC_EXPORT LayerImpl {
   // Viewport bounds delta are only used for viewport layers and account for
   // changes in the viewport layers from browser controls and page scale
   // factors. These deltas are only set on the active tree.
+  // TODO(bokan): These methods should be unneeded now that LTHI sets these
+  // directly on the property trees.
   void SetViewportBoundsDelta(const gfx::Vector2dF& bounds_delta);
   gfx::Vector2dF ViewportBoundsDelta() const;
 
@@ -302,14 +304,6 @@ class CC_EXPORT LayerImpl {
   gfx::Size scroll_container_bounds() const { return scroll_container_bounds_; }
   bool scrollable() const { return scrollable_; }
 
-  void set_main_thread_scrolling_reasons(
-      uint32_t main_thread_scrolling_reasons) {
-    main_thread_scrolling_reasons_ = main_thread_scrolling_reasons;
-  }
-  uint32_t main_thread_scrolling_reasons() const {
-    return main_thread_scrolling_reasons_;
-  }
-
   void SetNonFastScrollableRegion(const Region& region) {
     non_fast_scrollable_region_ = region;
   }
@@ -324,15 +318,15 @@ class CC_EXPORT LayerImpl {
     return touch_action_region_;
   }
 
-  bool HasPotentiallyRunningTransformAnimation() const;
-
-  bool HasFilterAnimationThatInflatesBounds() const;
-  bool HasAnimationThatInflatesBounds() const;
-
-  bool FilterAnimationBoundsForBox(const gfx::BoxF& box,
-                                   gfx::BoxF* bounds) const;
-  bool TransformAnimationBoundsForBox(const gfx::BoxF& box,
-                                      gfx::BoxF* bounds) const;
+  // Set or get the region that contains wheel event handler.
+  // The |wheel_event_handler_region| specify the area where wheel event handler
+  // could block impl scrolling.
+  void SetWheelEventHandlerRegion(const Region& wheel_event_handler_region) {
+    wheel_event_handler_region_ = wheel_event_handler_region;
+  }
+  const Region& wheel_event_handler_region() const {
+    return wheel_event_handler_region_;
+  }
 
   // Note this rect is in layer space (not content space).
   void SetUpdateRect(const gfx::Rect& update_rect);
@@ -341,13 +335,18 @@ class CC_EXPORT LayerImpl {
   void AddDamageRect(const gfx::Rect& damage_rect);
   const gfx::Rect& damage_rect() const { return damage_rect_; }
 
-  virtual std::unique_ptr<base::DictionaryValue> LayerTreeAsJson();
+  virtual std::unique_ptr<base::DictionaryValue> LayerAsJson() const;
+  // TODO(pdr): This should be removed because there is no longer a tree
+  // of layers, only a list.
+  std::unique_ptr<base::DictionaryValue> LayerTreeAsJson();
 
-  // This includes |layer_property_changed_| and property_trees changes.
+  // This includes |layer_property_changed_not_from_property_trees_| and
+  // property_trees changes.
   bool LayerPropertyChanged() const;
-  // Only checks |layer_property_changed_|. Used in damage_tracker to determine
-  // if there is a contributing content damage not from property_trees changes
-  // in animaiton.
+  bool LayerPropertyChangedFromPropertyTrees() const;
+  // Only checks |layer_property_changed_not_from_property_trees_|. Used in
+  // damage_tracker to determine if there is a contributing content damage not
+  // from property_trees changes in animaiton.
   bool LayerPropertyChangedNotFromPropertyTrees() const;
 
   void ResetChangeTracking();
@@ -362,8 +361,13 @@ class CC_EXPORT LayerImpl {
   // that rendered this layer was lost.
   virtual void ReleaseResources();
 
+  // Releases resources in response to memory pressure. The default
+  // implementation just calls ReleaseResources() and subclasses will override
+  // if that's not appropriate.
+  virtual void OnPurgeMemory();
+
   // Release tile resources held by this layer. Called when a rendering mode
-  // switch has occured and tiles are no longer valid.
+  // switch has occurred and tiles are no longer valid.
   virtual void ReleaseTileResources();
 
   // Recreate tile resources held by this layer after they were released by a
@@ -371,21 +375,31 @@ class CC_EXPORT LayerImpl {
   virtual void RecreateTileResources();
 
   virtual std::unique_ptr<LayerImpl> CreateLayerImpl(LayerTreeImpl* tree_impl);
-  virtual bool IsSnapped();
   virtual void PushPropertiesTo(LayerImpl* layer);
 
+  // Internal to property tree construction (which only happens in tests on a
+  // LayerImpl tree. See Layer::IsSnappedToPixelGridInTarget() for explanation,
+  // as this mirrors that method.
+  virtual bool IsSnappedToPixelGridInTarget();
+
   virtual void GetAllPrioritizedTilesForTracing(
       std::vector<PrioritizedTile>* prioritized_tiles) const;
   virtual void AsValueInto(base::trace_event::TracedValue* dict) const;
+  std::string ToString() const;
 
   virtual size_t GPUMemoryUsageInBytes() const;
 
+  // Mark a layer on pending tree that needs to push its properties to the
+  // active tree. These properties should not be changed during pending tree
+  // lifetime, and only changed by being pushed from the main thread. There are
+  // two cases where this function needs to be called: when main thread layer
+  // has properties that need to be pushed, or when a new LayerImpl is created
+  // on pending tree when syncing layers from main thread.
   void SetNeedsPushProperties();
 
   virtual void RunMicroBenchmark(MicroBenchmarkImpl* benchmark);
 
-  void SetDebugInfo(
-      std::unique_ptr<base::trace_event::ConvertableToTraceFormat> debug_info);
+  void SetDebugInfo(std::unique_ptr<base::trace_event::TracedValue> debug_info);
 
   void set_contributes_to_drawn_render_surface(bool is_member) {
     contributes_to_drawn_render_surface_ = is_member;
@@ -395,9 +409,9 @@ class CC_EXPORT LayerImpl {
     return contributes_to_drawn_render_surface_;
   }
 
-  bool IsDrawnScrollbar() {
-    return ToScrollbarLayer() && contributes_to_drawn_render_surface_;
-  }
+  bool is_scrollbar() const { return is_scrollbar_; }
+
+  void set_is_scrollbar(bool is_scrollbar) { is_scrollbar_ = is_scrollbar; }
 
   void set_may_contain_video(bool yes) { may_contain_video_ = yes; }
   bool may_contain_video() const { return may_contain_video_; }
@@ -413,21 +427,18 @@ class CC_EXPORT LayerImpl {
 
   virtual gfx::Rect GetEnclosingRectInTargetSpace() const;
 
-  bool has_copy_requests_in_target_subtree();
-
-  void UpdatePropertyTreeForAnimationIfNeeded();
+  void UpdatePropertyTreeForAnimationIfNeeded(ElementId element_id);
 
   float GetIdealContentsScale() const;
 
   void NoteLayerPropertyChanged();
+  void NoteLayerPropertyChangedFromPropertyTrees();
 
   void SetHasWillChangeTransformHint(bool has_will_change);
   bool has_will_change_transform_hint() const {
     return has_will_change_transform_hint_;
   }
 
-  MutatorHost* GetMutatorHost() const;
-
   ElementListType GetElementTypeForAnimation() const;
 
   void set_needs_show_scrollbars(bool yes) { needs_show_scrollbars_ = yes; }
@@ -440,22 +451,27 @@ class CC_EXPORT LayerImpl {
 
   void EnsureValidPropertyTreeIndices() const;
 
+  // TODO(sunxd): Remove this function and replace it with visitor pattern.
+  virtual bool is_surface_layer() const;
+
  protected:
+  // When |will_always_push_properties| is true, the layer will not itself set
+  // its SetNeedsPushProperties() state, as it expects to be always pushed to
+  // the active tree regardless.
   LayerImpl(LayerTreeImpl* layer_impl,
             int id,
-            scoped_refptr<SyncedScrollOffset> scroll_offset);
-  LayerImpl(LayerTreeImpl* layer_impl, int id);
+            bool will_always_push_properties = false);
 
   // Get the color and size of the layer's debug border.
   virtual void GetDebugBorderProperties(SkColor* color, float* width) const;
 
-  void AppendDebugBorderQuad(RenderPass* render_pass,
-                             const gfx::Size& bounds,
-                             const SharedQuadState* shared_quad_state,
+  void AppendDebugBorderQuad(viz::RenderPass* render_pass,
+                             const gfx::Rect& quad_rect,
+                             const viz::SharedQuadState* shared_quad_state,
                              AppendQuadsData* append_quads_data) const;
-  void AppendDebugBorderQuad(RenderPass* render_pass,
-                             const gfx::Size& bounds,
-                             const SharedQuadState* shared_quad_state,
+  void AppendDebugBorderQuad(viz::RenderPass* render_pass,
+                             const gfx::Rect& quad_rect,
+                             const viz::SharedQuadState* shared_quad_state,
                              AppendQuadsData* append_quads_data,
                              SkColor color,
                              float width) const;
@@ -463,16 +479,13 @@ class CC_EXPORT LayerImpl {
   gfx::Rect GetScaledEnclosingRectInTargetSpace(float scale) const;
 
  private:
-  // This includes all animations, even those that are finished but haven't yet
-  // been deleted.
-  bool HasAnyAnimationTargetingProperty(TargetProperty::Type property) const;
-
-  void ValidateQuadResourcesInternal(DrawQuad* quad) const;
+  void ValidateQuadResourcesInternal(viz::DrawQuad* quad) const;
 
   virtual const char* LayerTypeAsString() const;
 
-  int layer_id_;
-  LayerTreeImpl* layer_tree_impl_;
+  const int layer_id_;
+  LayerTreeImpl* const layer_tree_impl_;
+  const bool will_always_push_properties_ : 1;
 
   std::unique_ptr<LayerImplTestProperties> test_properties_;
 
@@ -480,7 +493,6 @@ class CC_EXPORT LayerImpl {
   gfx::Size bounds_;
 
   gfx::Vector2dF offset_to_transform_parent_;
-  uint32_t main_thread_scrolling_reasons_;
 
   // Size of the scroll container that this layer scrolls in.
   gfx::Size scroll_container_bounds_;
@@ -490,10 +502,17 @@ class CC_EXPORT LayerImpl {
   // |scroll_container_bounds|).
   bool scrollable_ : 1;
 
-  bool should_flatten_transform_from_property_tree_ : 1;
+  bool should_flatten_screen_space_transform_from_property_tree_ : 1;
 
   // Tracks if drawing-related properties have changed since last redraw.
-  bool layer_property_changed_ : 1;
+  // TODO(wutao): We want to distinquish the sources of change so that we can
+  // reuse the cache of render pass. For example, we can reuse the cache when
+  // transform and opacity changing on a surface during animation. Currently
+  // |layer_property_changed_from_property_trees_| does not mean the layer is
+  // damaged from animation. We need better mechanism to explicitly capture
+  // damage from animations. http://crbug.com/755828.
+  bool layer_property_changed_not_from_property_trees_ : 1;
+  bool layer_property_changed_from_property_trees_ : 1;
   bool may_contain_video_ : 1;
 
   bool masks_to_bounds_ : 1;
@@ -502,20 +521,25 @@ class CC_EXPORT LayerImpl {
   bool should_check_backface_visibility_ : 1;
   bool draws_content_ : 1;
   bool contributes_to_drawn_render_surface_ : 1;
-  bool should_hit_test_ : 1;
 
+  // Hit testing depends on draws_content (see: |LayerImpl::should_hit_test|)
+  // and this bit can be set to cause the layer to be hit testable without
+  // draws_content.
+  bool hit_testable_without_draws_content_ : 1;
+  bool is_resized_by_browser_controls_ : 1;
+
+  // TODO(bokan): This can likely be removed after blink-gen-property-trees
+  // is shipped. https://crbug.com/836884.
   static_assert(LAST_VIEWPORT_LAYER_TYPE < (1u << 3),
                 "enough bits for ViewportLayerType (viewport_layer_type_)");
   uint8_t viewport_layer_type_ : 3;  // ViewportLayerType
 
   Region non_fast_scrollable_region_;
   TouchActionRegion touch_action_region_;
+  Region wheel_event_handler_region_;
   SkColor background_color_;
   SkColor safe_opaque_background_color_;
 
-  gfx::PointF position_;
-
-  gfx::Rect clip_rect_in_target_space_;
   int transform_tree_index_;
   int effect_tree_index_;
   int clip_tree_index_;
@@ -534,7 +558,6 @@ class CC_EXPORT LayerImpl {
   TransformTree& GetTransformTree() const;
 
   ElementId element_id_;
-  uint32_t mutable_properties_;
   // Rect indicating what was repainted/updated during update.
   // Note that plugin layers bypass this and leave it empty.
   // This is in the layer's space.
@@ -549,12 +572,12 @@ class CC_EXPORT LayerImpl {
   DrawProperties draw_properties_;
   PerformanceProperties<LayerImpl> performance_properties_;
 
-  std::unique_ptr<base::trace_event::ConvertableToTraceFormat>
-      owned_debug_info_;
-  base::trace_event::ConvertableToTraceFormat* debug_info_;
+  std::unique_ptr<base::trace_event::TracedValue> owned_debug_info_;
+  base::trace_event::TracedValue* debug_info_;
 
   bool has_will_change_transform_hint_ : 1;
   bool needs_push_properties_ : 1;
+  bool is_scrollbar_ : 1;
   bool scrollbars_hidden_ : 1;
 
   // The needs_show_scrollbars_ bit tracks a pending request from Blink to show
