@@ -7,8 +7,8 @@
 #include "base/command_line.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
-#include "base/message_loop/message_loop.h"
 #include "base/strings/stringprintf.h"
+#include "base/test/scoped_task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "components/autofill/core/browser/autofill_test_utils.h"
@@ -17,7 +17,15 @@
 #include "components/autofill/core/browser/personal_data_manager.h"
 #include "components/autofill/core/browser/test_autofill_client.h"
 #include "components/autofill/core/browser/test_autofill_driver.h"
+#include "components/autofill/core/browser/test_personal_data_manager.h"
+#include "components/autofill/core/common/autofill_prefs.h"
+#include "components/prefs/pref_registry_simple.h"
+#include "components/prefs/pref_service.h"
+#include "components/prefs/testing_pref_service.h"
 #include "net/url_request/url_request_test_util.h"
+#include "services/network/public/cpp/shared_url_loader_factory.h"
+#include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
+#include "services/network/test/test_url_loader_factory.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -30,8 +38,10 @@ using testing::_;
 class MockResultDelegate : public FullCardRequest::ResultDelegate,
                            public base::SupportsWeakPtr<MockResultDelegate> {
  public:
-  MOCK_METHOD2(OnFullCardRequestSucceeded,
-               void(const CreditCard&, const base::string16&));
+  MOCK_METHOD3(OnFullCardRequestSucceeded,
+               void(const payments::FullCardRequest&,
+                    const CreditCard&,
+                    const base::string16&));
   MOCK_METHOD0(OnFullCardRequestFailed, void());
 };
 
@@ -48,23 +58,37 @@ class MockUIDelegate : public FullCardRequest::UIDelegate,
 };
 
 // The personal data manager.
-class MockPersonalDataManager : public PersonalDataManager {
+class MockPersonalDataManager : public TestPersonalDataManager {
  public:
-  MockPersonalDataManager() : PersonalDataManager("en-US") {}
+  MockPersonalDataManager() {}
   ~MockPersonalDataManager() override {}
+  MOCK_CONST_METHOD0(IsSyncFeatureEnabled, bool());
   MOCK_METHOD1(UpdateCreditCard, void(const CreditCard& credit_card));
   MOCK_METHOD1(UpdateServerCreditCard, void(const CreditCard& credit_card));
 };
 
+// TODO(crbug.com/881835): Simplify this test setup.
 // The test fixture for full card request.
-class FullCardRequestTest : public testing::Test,
-                            public PaymentsClientDelegate {
+class FullCardRequestTest : public testing::Test {
  public:
   FullCardRequestTest()
       : request_context_(new net::TestURLRequestContextGetter(
             base::ThreadTaskRunnerHandle::Get())),
-        payments_client_(request_context_.get(), this),
-        request_(&autofill_client_, &payments_client_, &personal_data_) {
+        test_shared_loader_factory_(
+            base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
+                &test_url_loader_factory_)) {
+    std::unique_ptr<TestingPrefServiceSimple> pref_service(
+        new TestingPrefServiceSimple());
+    pref_service->registry()->RegisterDoublePref(
+        prefs::kAutofillBillingCustomerNumber, 0.0);
+    autofill_client_.SetPrefs(std::move(pref_service));
+    payments_client_ = std::make_unique<PaymentsClient>(
+        test_shared_loader_factory_, autofill_client_.GetPrefs(),
+        autofill_client_.GetIdentityManager(), &personal_data_);
+    request_ = std::make_unique<FullCardRequest>(
+        &autofill_client_, payments_client_.get(), &personal_data_);
+    personal_data_.SetAccountInfoForPayments(
+        autofill_client_.GetIdentityManager()->GetPrimaryAccountInfo());
     // Silence the warning from PaymentsClient about matching sync and Payments
     // server types.
     base::CommandLine::ForCurrentProcess()->AppendSwitchASCII(
@@ -75,46 +99,32 @@ class FullCardRequestTest : public testing::Test,
 
   MockPersonalDataManager* personal_data() { return &personal_data_; }
 
-  FullCardRequest* request() { return &request_; }
+  FullCardRequest* request() { return request_.get(); }
 
   CardUnmaskDelegate* card_unmask_delegate() {
-    return static_cast<CardUnmaskDelegate*>(&request_);
+    return static_cast<CardUnmaskDelegate*>(request_.get());
   }
 
   MockResultDelegate* result_delegate() { return &result_delegate_; }
 
   MockUIDelegate* ui_delegate() { return &ui_delegate_; }
 
-  // PaymentsClientDelegate:
   void OnDidGetRealPan(AutofillClient::PaymentsRpcResult result,
-                       const std::string& real_pan) override {
-    request_.OnDidGetRealPan(result, real_pan);
+                       const std::string& real_pan) {
+    request_->OnDidGetRealPan(result, real_pan);
   }
 
  private:
-  // PaymentsClientDelegate:
-  IdentityProvider* GetIdentityProvider() override {
-    return autofill_client_.GetIdentityProvider();
-  }
-  void OnDidGetUploadDetails(
-      AutofillClient::PaymentsRpcResult result,
-      const base::string16& context_token,
-      std::unique_ptr<base::DictionaryValue> legal_message) override {
-    ASSERT_TRUE(false) << "No upload details in this test";
-  }
-  void OnDidUploadCard(AutofillClient::PaymentsRpcResult result,
-                       const std::string& server_id) override {
-    ASSERT_TRUE(false) << "No card uploads in this test.";
-  }
-
-  base::MessageLoop message_loop_;
+  base::test::ScopedTaskEnvironment task_environment_;
   MockPersonalDataManager personal_data_;
-  TestAutofillClient autofill_client_;
-  scoped_refptr<net::TestURLRequestContextGetter> request_context_;
-  PaymentsClient payments_client_;
   MockResultDelegate result_delegate_;
   MockUIDelegate ui_delegate_;
-  FullCardRequest request_;
+  TestAutofillClient autofill_client_;
+  scoped_refptr<net::TestURLRequestContextGetter> request_context_;
+  network::TestURLLoaderFactory test_url_loader_factory_;
+  scoped_refptr<network::SharedURLLoaderFactory> test_shared_loader_factory_;
+  std::unique_ptr<PaymentsClient> payments_client_;
+  std::unique_ptr<FullCardRequest> request_;
 
   DISALLOW_COPY_AND_ASSIGN(FullCardRequestTest);
 };
@@ -139,6 +149,7 @@ MATCHER_P4(CardMatches, record_type, number, month, year, "") {
 TEST_F(FullCardRequestTest, GetFullCardPanAndCvcForMaskedServerCard) {
   EXPECT_CALL(*result_delegate(),
               OnFullCardRequestSucceeded(
+                  testing::Ref(*request()),
                   CardMatches(CreditCard::FULL_SERVER_CARD, "4111"),
                   base::ASCIIToUTF16("123")));
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _));
@@ -160,7 +171,8 @@ TEST_F(FullCardRequestTest, GetFullCardPanAndCvcForMaskedServerCard) {
 TEST_F(FullCardRequestTest, GetFullCardPanAndCvcForLocalCard) {
   EXPECT_CALL(
       *result_delegate(),
-      OnFullCardRequestSucceeded(CardMatches(CreditCard::LOCAL_CARD, "4111"),
+      OnFullCardRequestSucceeded(testing::Ref(*request()),
+                                 CardMatches(CreditCard::LOCAL_CARD, "4111"),
                                  base::ASCIIToUTF16("123")));
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _));
   EXPECT_CALL(*ui_delegate(),
@@ -181,6 +193,7 @@ TEST_F(FullCardRequestTest, GetFullCardPanAndCvcForLocalCard) {
 TEST_F(FullCardRequestTest, GetFullCardPanAndCvcForFullServerCard) {
   EXPECT_CALL(*result_delegate(),
               OnFullCardRequestSucceeded(
+                  testing::Ref(*request()),
                   CardMatches(CreditCard::FULL_SERVER_CARD, "4111"),
                   base::ASCIIToUTF16("123")));
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _));
@@ -204,6 +217,7 @@ TEST_F(FullCardRequestTest, GetFullCardPanAndCvcForFullServerCard) {
 TEST_F(FullCardRequestTest,
        GetFullCardPanAndCvcForFullServerCardInExpiredStatus) {
   EXPECT_CALL(*result_delegate(), OnFullCardRequestSucceeded(
+                                      testing::Ref(*request()),
                                       CardMatches(CreditCard::FULL_SERVER_CARD,
                                                   "4111", "12", "2051"),
                                       base::ASCIIToUTF16("123")));
@@ -232,6 +246,7 @@ TEST_F(FullCardRequestTest,
 // expiration date in the past.
 TEST_F(FullCardRequestTest, GetFullCardPanAndCvcForExpiredFullServerCard) {
   EXPECT_CALL(*result_delegate(), OnFullCardRequestSucceeded(
+                                      testing::Ref(*request()),
                                       CardMatches(CreditCard::FULL_SERVER_CARD,
                                                   "4111", "12", "2051"),
                                       base::ASCIIToUTF16("123")));
@@ -280,7 +295,8 @@ TEST_F(FullCardRequestTest, SecondRequestOkAfterFirstFinished) {
   EXPECT_CALL(*result_delegate(), OnFullCardRequestFailed()).Times(0);
   EXPECT_CALL(
       *result_delegate(),
-      OnFullCardRequestSucceeded(CardMatches(CreditCard::LOCAL_CARD, "4111"),
+      OnFullCardRequestSucceeded(testing::Ref(*request()),
+                                 CardMatches(CreditCard::LOCAL_CARD, "4111"),
                                  base::ASCIIToUTF16("123")))
       .Times(2);
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _)).Times(2);
@@ -382,6 +398,7 @@ TEST_F(FullCardRequestTest, TryAgainFailureRetry) {
   EXPECT_CALL(*result_delegate(), OnFullCardRequestFailed()).Times(0);
   EXPECT_CALL(*result_delegate(),
               OnFullCardRequestSucceeded(
+                  testing::Ref(*request()),
                   CardMatches(CreditCard::FULL_SERVER_CARD, "4111"),
                   base::ASCIIToUTF16("123")));
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _));
@@ -407,6 +424,7 @@ TEST_F(FullCardRequestTest, TryAgainFailureRetry) {
 // Verify updating expiration date for a masked server card.
 TEST_F(FullCardRequestTest, UpdateExpDateForMaskedServerCard) {
   EXPECT_CALL(*result_delegate(), OnFullCardRequestSucceeded(
+                                      testing::Ref(*request()),
                                       CardMatches(CreditCard::FULL_SERVER_CARD,
                                                   "4111", "12", "2050"),
                                       base::ASCIIToUTF16("123")));
@@ -430,6 +448,7 @@ TEST_F(FullCardRequestTest, UpdateExpDateForMaskedServerCard) {
 // Verify updating expiration date for an unmasked server card.
 TEST_F(FullCardRequestTest, UpdateExpDateForFullServerCard) {
   EXPECT_CALL(*result_delegate(), OnFullCardRequestSucceeded(
+                                      testing::Ref(*request()),
                                       CardMatches(CreditCard::FULL_SERVER_CARD,
                                                   "4111", "12", "2050"),
                                       base::ASCIIToUTF16("123")));
@@ -456,6 +475,7 @@ TEST_F(FullCardRequestTest, UpdateExpDateForFullServerCard) {
 TEST_F(FullCardRequestTest, UpdateExpDateForLocalCard) {
   EXPECT_CALL(*result_delegate(),
               OnFullCardRequestSucceeded(
+                  testing::Ref(*request()),
                   CardMatches(CreditCard::LOCAL_CARD, "4111", "12", "2051"),
                   base::ASCIIToUTF16("123")));
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _));
@@ -485,6 +505,7 @@ TEST_F(FullCardRequestTest, UpdateExpDateForLocalCard) {
 // Verify saving full PAN on disk.
 TEST_F(FullCardRequestTest, SaveRealPan) {
   EXPECT_CALL(*result_delegate(), OnFullCardRequestSucceeded(
+                                      testing::Ref(*request()),
                                       CardMatches(CreditCard::FULL_SERVER_CARD,
                                                   "4111", "12", "2050"),
                                       base::ASCIIToUTF16("123")));
@@ -513,6 +534,7 @@ TEST_F(FullCardRequestTest, SaveRealPan) {
 TEST_F(FullCardRequestTest, UnmaskForPaymentRequest) {
   EXPECT_CALL(*result_delegate(),
               OnFullCardRequestSucceeded(
+                  testing::Ref(*request()),
                   CardMatches(CreditCard::FULL_SERVER_CARD, "4111"),
                   base::ASCIIToUTF16("123")));
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _));
@@ -535,6 +557,7 @@ TEST_F(FullCardRequestTest, UnmaskForPaymentRequest) {
 TEST_F(FullCardRequestTest, IsGettingFullCardForMaskedServerCard) {
   EXPECT_CALL(*result_delegate(),
               OnFullCardRequestSucceeded(
+                  testing::Ref(*request()),
                   CardMatches(CreditCard::FULL_SERVER_CARD, "4111"),
                   base::ASCIIToUTF16("123")));
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _));
@@ -570,7 +593,8 @@ TEST_F(FullCardRequestTest, IsGettingFullCardForMaskedServerCard) {
 TEST_F(FullCardRequestTest, IsGettingFullCardForLocalCard) {
   EXPECT_CALL(
       *result_delegate(),
-      OnFullCardRequestSucceeded(CardMatches(CreditCard::LOCAL_CARD, "4111"),
+      OnFullCardRequestSucceeded(testing::Ref(*request()),
+                                 CardMatches(CreditCard::LOCAL_CARD, "4111"),
                                  base::ASCIIToUTF16("123")));
   EXPECT_CALL(*ui_delegate(), ShowUnmaskPrompt(_, _, _));
   EXPECT_CALL(*ui_delegate(),
