@@ -11,9 +11,11 @@
 #include "components/autofill/core/browser/autofill_country.h"
 #include "components/autofill/core/browser/autofill_profile.h"
 #include "components/autofill/core/browser/credit_card.h"
+#include "components/autofill/core/browser/payments/payments_customer_data.h"
 #include "components/autofill/core/browser/webdata/autofill_change.h"
 #include "components/autofill/core/browser/webdata/autofill_entry.h"
 #include "components/autofill/core/browser/webdata/autofill_table.h"
+#include "components/autofill/core/browser/webdata/autofill_webdata_service.h"
 #include "components/autofill/core/browser/webdata/autofill_webdata_service_observer.h"
 #include "components/autofill/core/common/form_field_data.h"
 #include "components/webdata/common/web_database_backend.h"
@@ -25,69 +27,110 @@ namespace autofill {
 
 AutofillWebDataBackendImpl::AutofillWebDataBackendImpl(
     scoped_refptr<WebDatabaseBackend> web_database_backend,
-    scoped_refptr<base::SingleThreadTaskRunner> ui_thread,
-    scoped_refptr<base::SingleThreadTaskRunner> db_thread,
+    scoped_refptr<base::SingleThreadTaskRunner> ui_task_runner,
+    scoped_refptr<base::SingleThreadTaskRunner> db_task_runner,
     const base::Closure& on_changed_callback,
     const base::Callback<void(syncer::ModelType)>& on_sync_started_callback)
-    : base::RefCountedDeleteOnSequence<AutofillWebDataBackendImpl>(db_thread),
-      ui_thread_(ui_thread),
-      db_thread_(db_thread),
+    : base::RefCountedDeleteOnSequence<AutofillWebDataBackendImpl>(
+          std::move(db_task_runner)),
+      ui_task_runner_(ui_task_runner),
       web_database_backend_(web_database_backend),
       on_changed_callback_(on_changed_callback),
       on_sync_started_callback_(on_sync_started_callback) {}
 
 void AutofillWebDataBackendImpl::AddObserver(
-    AutofillWebDataServiceObserverOnDBThread* observer) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+    AutofillWebDataServiceObserverOnDBSequence* observer) {
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   db_observer_list_.AddObserver(observer);
 }
 
 void AutofillWebDataBackendImpl::RemoveObserver(
-    AutofillWebDataServiceObserverOnDBThread* observer) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+    AutofillWebDataServiceObserverOnDBSequence* observer) {
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   db_observer_list_.RemoveObserver(observer);
 }
 
 AutofillWebDataBackendImpl::~AutofillWebDataBackendImpl() {
-  DCHECK(!user_data_.get());  // Forgot to call ResetUserData?
+  DCHECK(!user_data_);  // Forgot to call ResetUserData?
+}
+
+void AutofillWebDataBackendImpl::SetAutofillProfileChangedCallback(
+    base::RepeatingCallback<void(const AutofillProfileDeepChange&)> change_cb) {
+  on_autofill_profile_changed_cb_ = std::move(change_cb);
 }
 
 WebDatabase* AutofillWebDataBackendImpl::GetDatabase() {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   return web_database_backend_->database();
 }
 
 void AutofillWebDataBackendImpl::RemoveExpiredFormElements() {
   web_database_backend_->ExecuteWriteTask(
-      Bind(&AutofillWebDataBackendImpl::RemoveExpiredFormElementsImpl,
-           this));
+      Bind(&AutofillWebDataBackendImpl::RemoveExpiredFormElementsImpl, this));
+}
+
+std::unique_ptr<WDTypedResult>
+AutofillWebDataBackendImpl::RemoveExpiredAutocompleteEntries(WebDatabase* db) {
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+  AutofillChangeList changes;
+
+  if (AutofillTable::FromWebDatabase(db)->RemoveExpiredFormElements(&changes)) {
+    if (!changes.empty()) {
+      // Post the notifications including the list of affected keys.
+      // This is sent here so that work resulting from this notification
+      // will be done on the DB sequence, and not the UI sequence.
+      for (auto& db_observer : db_observer_list_)
+        db_observer.AutofillEntriesChanged(changes);
+    }
+  }
+
+  return std::make_unique<WDResult<size_t>>(AUTOFILL_CLEANUP_RESULT,
+                                            changes.size());
+}
+
+void AutofillWebDataBackendImpl::NotifyOfAutofillProfileChanged(
+    const AutofillProfileChange& change) {
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+
+  // DB sequence notification.
+  for (auto& db_observer : db_observer_list_)
+    db_observer.AutofillProfileChanged(change);
+}
+
+void AutofillWebDataBackendImpl::NotifyOfCreditCardChanged(
+    const CreditCardChange& change) {
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+
+  // DB sequence notification.
+  for (auto& db_observer : db_observer_list_)
+    db_observer.CreditCardChanged(change);
 }
 
 void AutofillWebDataBackendImpl::NotifyOfMultipleAutofillChanges() {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
 
-  // DB thread notification.
+  // DB sequence notification.
   for (auto& db_observer : db_observer_list_)
     db_observer.AutofillMultipleChanged();
 
-  // UI thread notification.
-  ui_thread_->PostTask(FROM_HERE, on_changed_callback_);
+  // UI sequence notification.
+  ui_task_runner_->PostTask(FROM_HERE, on_changed_callback_);
 }
 
 void AutofillWebDataBackendImpl::NotifyThatSyncHasStarted(
     syncer::ModelType model_type) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
 
   if (on_sync_started_callback_.is_null())
     return;
 
-  // UI thread notification.
-  ui_thread_->PostTask(FROM_HERE,
-                       base::Bind(on_sync_started_callback_, model_type));
+  // UI sequence notification.
+  ui_task_runner_->PostTask(
+      FROM_HERE, base::BindOnce(on_sync_started_callback_, model_type));
 }
 
 base::SupportsUserData* AutofillWebDataBackendImpl::GetDBUserData() {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (!user_data_)
     user_data_.reset(new SupportsUserDataAggregatable());
   return user_data_.get();
@@ -99,7 +142,7 @@ void AutofillWebDataBackendImpl::ResetUserData() {
 
 WebDatabase::State AutofillWebDataBackendImpl::AddFormElements(
     const std::vector<FormFieldData>& fields, WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   AutofillChangeList changes;
   if (!AutofillTable::FromWebDatabase(db)->AddFormFieldValues(
         fields, &changes)) {
@@ -109,7 +152,7 @@ WebDatabase::State AutofillWebDataBackendImpl::AddFormElements(
 
   // Post the notifications including the list of affected keys.
   // This is sent here so that work resulting from this notification will be
-  // done on the DB thread, and not the UI thread.
+  // done on the DB sequence, and not the UI sequence.
   for (auto& db_observer : db_observer_list_)
     db_observer.AutofillEntriesChanged(changes);
 
@@ -122,19 +165,19 @@ AutofillWebDataBackendImpl::GetFormValuesForElementName(
     const base::string16& prefix,
     int limit,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
-  std::vector<base::string16> values;
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+  std::vector<AutofillEntry> entries;
   AutofillTable::FromWebDatabase(db)->GetFormValuesForElementName(
-      name, prefix, &values, limit);
-  return std::unique_ptr<WDTypedResult>(
-      new WDResult<std::vector<base::string16>>(AUTOFILL_VALUE_RESULT, values));
+      name, prefix, &entries, limit);
+  return std::make_unique<WDResult<std::vector<AutofillEntry>>>(
+      AUTOFILL_VALUE_RESULT, entries);
 }
 
 WebDatabase::State AutofillWebDataBackendImpl::RemoveFormElementsAddedBetween(
     const base::Time& delete_begin,
     const base::Time& delete_end,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   AutofillChangeList changes;
 
   if (AutofillTable::FromWebDatabase(db)->RemoveFormElementsAddedBetween(
@@ -142,7 +185,7 @@ WebDatabase::State AutofillWebDataBackendImpl::RemoveFormElementsAddedBetween(
     if (!changes.empty()) {
       // Post the notifications including the list of affected keys.
       // This is sent here so that work resulting from this notification
-      // will be done on the DB thread, and not the UI thread.
+      // will be done on the DB sequence, and not the UI sequence.
       for (auto& db_observer : db_observer_list_)
         db_observer.AutofillEntriesChanged(changes);
     }
@@ -153,7 +196,7 @@ WebDatabase::State AutofillWebDataBackendImpl::RemoveFormElementsAddedBetween(
 
 WebDatabase::State AutofillWebDataBackendImpl::RemoveFormValueForElementName(
     const base::string16& name, const base::string16& value, WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
 
   if (AutofillTable::FromWebDatabase(db)->RemoveFormElement(name, value)) {
     AutofillChangeList changes;
@@ -171,7 +214,7 @@ WebDatabase::State AutofillWebDataBackendImpl::RemoveFormValueForElementName(
 
 WebDatabase::State AutofillWebDataBackendImpl::AddAutofillProfile(
     const AutofillProfile& profile, WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (!AutofillTable::FromWebDatabase(db)->AddAutofillProfile(profile)) {
     NOTREACHED();
     return WebDatabase::COMMIT_NOT_NEEDED;
@@ -183,12 +226,19 @@ WebDatabase::State AutofillWebDataBackendImpl::AddAutofillProfile(
   for (auto& db_observer : db_observer_list_)
     db_observer.AutofillProfileChanged(change);
 
+  if (!on_autofill_profile_changed_cb_.is_null()) {
+    ui_task_runner_->PostTask(
+        FROM_HERE, base::BindOnce(on_autofill_profile_changed_cb_,
+                                  AutofillProfileDeepChange(
+                                      AutofillProfileChange::ADD, profile)));
+  }
+
   return WebDatabase::COMMIT_NEEDED;
 }
 
 WebDatabase::State AutofillWebDataBackendImpl::UpdateAutofillProfile(
     const AutofillProfile& profile, WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   // Only perform the update if the profile exists.  It is currently
   // valid to try to update a missing profile.  We simply drop the write and
   // the caller will detect this on the next refresh.
@@ -208,12 +258,19 @@ WebDatabase::State AutofillWebDataBackendImpl::UpdateAutofillProfile(
   for (auto& db_observer : db_observer_list_)
     db_observer.AutofillProfileChanged(change);
 
+  if (!on_autofill_profile_changed_cb_.is_null()) {
+    ui_task_runner_->PostTask(
+        FROM_HERE, base::BindOnce(on_autofill_profile_changed_cb_,
+                                  AutofillProfileDeepChange(
+                                      AutofillProfileChange::UPDATE, profile)));
+  }
+
   return WebDatabase::COMMIT_NEEDED;
 }
 
 WebDatabase::State AutofillWebDataBackendImpl::RemoveAutofillProfile(
     const std::string& guid, WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   std::unique_ptr<AutofillProfile> profile =
       AutofillTable::FromWebDatabase(db)->GetAutofillProfile(guid);
   if (!profile) {
@@ -227,16 +284,23 @@ WebDatabase::State AutofillWebDataBackendImpl::RemoveAutofillProfile(
   }
 
   // Send GUID-based notification.
-  AutofillProfileChange change(AutofillProfileChange::REMOVE, guid, NULL);
+  AutofillProfileChange change(AutofillProfileChange::REMOVE, guid, nullptr);
   for (auto& db_observer : db_observer_list_)
     db_observer.AutofillProfileChanged(change);
 
+  if (!on_autofill_profile_changed_cb_.is_null()) {
+    ui_task_runner_->PostTask(
+        FROM_HERE, base::BindOnce(on_autofill_profile_changed_cb_,
+                                  AutofillProfileDeepChange(
+                                      AutofillProfileChange::REMOVE, guid)));
+  }
+
   return WebDatabase::COMMIT_NEEDED;
 }
 
 std::unique_ptr<WDTypedResult> AutofillWebDataBackendImpl::GetAutofillProfiles(
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   std::vector<std::unique_ptr<AutofillProfile>> profiles;
   AutofillTable::FromWebDatabase(db)->GetAutofillProfiles(&profiles);
   return std::unique_ptr<WDTypedResult>(
@@ -246,7 +310,7 @@ std::unique_ptr<WDTypedResult> AutofillWebDataBackendImpl::GetAutofillProfiles(
 
 std::unique_ptr<WDTypedResult> AutofillWebDataBackendImpl::GetServerProfiles(
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   std::vector<std::unique_ptr<AutofillProfile>> profiles;
   AutofillTable::FromWebDatabase(db)->GetServerProfiles(&profiles);
   return std::unique_ptr<WDTypedResult>(
@@ -259,7 +323,7 @@ AutofillWebDataBackendImpl::GetCountOfValuesContainedBetween(
     const base::Time& begin,
     const base::Time& end,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   int value = AutofillTable::FromWebDatabase(db)
       ->GetCountOfValuesContainedBetween(begin, end);
   return std::unique_ptr<WDTypedResult>(
@@ -269,7 +333,7 @@ AutofillWebDataBackendImpl::GetCountOfValuesContainedBetween(
 WebDatabase::State AutofillWebDataBackendImpl::UpdateAutofillEntries(
     const std::vector<AutofillEntry>& autofill_entries,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (!AutofillTable::FromWebDatabase(db)
            ->UpdateAutofillEntries(autofill_entries))
     return WebDatabase::COMMIT_NOT_NEEDED;
@@ -279,7 +343,7 @@ WebDatabase::State AutofillWebDataBackendImpl::UpdateAutofillEntries(
 
 WebDatabase::State AutofillWebDataBackendImpl::AddCreditCard(
     const CreditCard& credit_card, WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (!AutofillTable::FromWebDatabase(db)->AddCreditCard(credit_card)) {
     NOTREACHED();
     return WebDatabase::COMMIT_NOT_NEEDED;
@@ -294,7 +358,7 @@ WebDatabase::State AutofillWebDataBackendImpl::AddCreditCard(
 
 WebDatabase::State AutofillWebDataBackendImpl::UpdateCreditCard(
     const CreditCard& credit_card, WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   // It is currently valid to try to update a missing profile.  We simply drop
   // the write and the caller will detect this on the next refresh.
   std::unique_ptr<CreditCard> original_credit_card =
@@ -316,7 +380,7 @@ WebDatabase::State AutofillWebDataBackendImpl::UpdateCreditCard(
 
 WebDatabase::State AutofillWebDataBackendImpl::RemoveCreditCard(
     const std::string& guid, WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (!AutofillTable::FromWebDatabase(db)->RemoveCreditCard(guid)) {
     NOTREACHED();
     return WebDatabase::COMMIT_NOT_NEEDED;
@@ -332,7 +396,7 @@ WebDatabase::State AutofillWebDataBackendImpl::RemoveCreditCard(
 WebDatabase::State AutofillWebDataBackendImpl::AddFullServerCreditCard(
     const CreditCard& credit_card,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (!AutofillTable::FromWebDatabase(db)->AddFullServerCreditCard(
           credit_card)) {
     NOTREACHED();
@@ -348,7 +412,7 @@ WebDatabase::State AutofillWebDataBackendImpl::AddFullServerCreditCard(
 
 std::unique_ptr<WDTypedResult> AutofillWebDataBackendImpl::GetCreditCards(
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   std::vector<std::unique_ptr<CreditCard>> credit_cards;
   AutofillTable::FromWebDatabase(db)->GetCreditCards(&credit_cards);
   return std::unique_ptr<WDTypedResult>(
@@ -358,7 +422,7 @@ std::unique_ptr<WDTypedResult> AutofillWebDataBackendImpl::GetCreditCards(
 
 std::unique_ptr<WDTypedResult> AutofillWebDataBackendImpl::GetServerCreditCards(
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   std::vector<std::unique_ptr<CreditCard>> credit_cards;
   AutofillTable::FromWebDatabase(db)->GetServerCreditCards(&credit_cards);
   return std::unique_ptr<WDTypedResult>(
@@ -370,7 +434,7 @@ WebDatabase::State AutofillWebDataBackendImpl::UnmaskServerCreditCard(
     const CreditCard& card,
     const base::string16& full_number,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (AutofillTable::FromWebDatabase(db)->UnmaskServerCreditCard(
           card, full_number))
     return WebDatabase::COMMIT_NEEDED;
@@ -381,7 +445,7 @@ WebDatabase::State
     AutofillWebDataBackendImpl::MaskServerCreditCard(
         const std::string& id,
         WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (AutofillTable::FromWebDatabase(db)->MaskServerCreditCard(id))
     return WebDatabase::COMMIT_NEEDED;
   return WebDatabase::COMMIT_NOT_NEEDED;
@@ -390,13 +454,14 @@ WebDatabase::State
 WebDatabase::State AutofillWebDataBackendImpl::UpdateServerCardMetadata(
     const CreditCard& card,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+  DCHECK_NE(CreditCard::LOCAL_CARD, card.record_type());
   if (!AutofillTable::FromWebDatabase(db)->UpdateServerCardMetadata(card))
     return WebDatabase::COMMIT_NOT_NEEDED;
 
   for (auto& db_observer : db_observer_list_) {
     db_observer.CreditCardChanged(
-        CreditCardChange(CreditCardChange::UPDATE, card.guid(), &card));
+        CreditCardChange(CreditCardChange::UPDATE, card.server_id(), &card));
   }
 
   return WebDatabase::COMMIT_NEEDED;
@@ -405,7 +470,8 @@ WebDatabase::State AutofillWebDataBackendImpl::UpdateServerCardMetadata(
 WebDatabase::State AutofillWebDataBackendImpl::UpdateServerAddressMetadata(
     const AutofillProfile& profile,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+  DCHECK_EQ(AutofillProfile::SERVER_PROFILE, profile.record_type());
   if (!AutofillTable::FromWebDatabase(db)->UpdateServerAddressMetadata(
           profile)) {
     return WebDatabase::COMMIT_NOT_NEEDED;
@@ -413,15 +479,24 @@ WebDatabase::State AutofillWebDataBackendImpl::UpdateServerAddressMetadata(
 
   for (auto& db_observer : db_observer_list_) {
     db_observer.AutofillProfileChanged(AutofillProfileChange(
-        AutofillProfileChange::UPDATE, profile.guid(), &profile));
+        AutofillProfileChange::UPDATE, profile.server_id(), &profile));
   }
 
   return WebDatabase::COMMIT_NEEDED;
 }
 
+std::unique_ptr<WDTypedResult>
+AutofillWebDataBackendImpl::GetPaymentsCustomerData(WebDatabase* db) {
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+  std::unique_ptr<PaymentsCustomerData> customer_data;
+  AutofillTable::FromWebDatabase(db)->GetPaymentsCustomerData(&customer_data);
+  return std::make_unique<WDResult<std::unique_ptr<PaymentsCustomerData>>>(
+      AUTOFILL_CUSTOMERDATA_RESULT, std::move(customer_data));
+}
+
 WebDatabase::State AutofillWebDataBackendImpl::ClearAllServerData(
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   if (AutofillTable::FromWebDatabase(db)->ClearAllServerData()) {
     NotifyOfMultipleAutofillChanges();
     return WebDatabase::COMMIT_NEEDED;
@@ -429,12 +504,22 @@ WebDatabase::State AutofillWebDataBackendImpl::ClearAllServerData(
   return WebDatabase::COMMIT_NOT_NEEDED;
 }
 
+WebDatabase::State AutofillWebDataBackendImpl::ClearAllLocalData(
+    WebDatabase* db) {
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+  if (AutofillTable::FromWebDatabase(db)->ClearAllLocalData()) {
+    NotifyOfMultipleAutofillChanges();
+    return WebDatabase::COMMIT_NEEDED;
+  }
+  return WebDatabase::COMMIT_NOT_NEEDED;
+}
+
 WebDatabase::State
     AutofillWebDataBackendImpl::RemoveAutofillDataModifiedBetween(
         const base::Time& delete_begin,
         const base::Time& delete_end,
         WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   std::vector<std::string> profile_guids;
   std::vector<std::string> credit_card_guids;
   if (AutofillTable::FromWebDatabase(db)->RemoveAutofillDataModifiedBetween(
@@ -465,7 +550,7 @@ WebDatabase::State AutofillWebDataBackendImpl::RemoveOriginURLsModifiedBetween(
     const base::Time& delete_begin,
     const base::Time& delete_end,
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   std::vector<std::unique_ptr<AutofillProfile>> profiles;
   if (!AutofillTable::FromWebDatabase(db)->RemoveOriginURLsModifiedBetween(
           delete_begin, delete_end, &profiles)) {
@@ -485,14 +570,14 @@ WebDatabase::State AutofillWebDataBackendImpl::RemoveOriginURLsModifiedBetween(
 
 WebDatabase::State AutofillWebDataBackendImpl::RemoveExpiredFormElementsImpl(
     WebDatabase* db) {
-  DCHECK(db_thread_->BelongsToCurrentThread());
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
   AutofillChangeList changes;
 
   if (AutofillTable::FromWebDatabase(db)->RemoveExpiredFormElements(&changes)) {
     if (!changes.empty()) {
       // Post the notifications including the list of affected keys.
       // This is sent here so that work resulting from this notification
-      // will be done on the DB thread, and not the UI thread.
+      // will be done on the DB sequence, and not the UI sequence.
       for (auto& db_observer : db_observer_list_)
         db_observer.AutofillEntriesChanged(changes);
     }
@@ -501,4 +586,13 @@ WebDatabase::State AutofillWebDataBackendImpl::RemoveExpiredFormElementsImpl(
   return WebDatabase::COMMIT_NOT_NEEDED;
 }
 
+WebDatabase::State AutofillWebDataBackendImpl::RemoveOrphanAutofillTableRows(
+    WebDatabase* db) {
+  DCHECK(owning_task_runner()->RunsTasksInCurrentSequence());
+  if (AutofillTable::FromWebDatabase(db)->RemoveOrphanAutofillTableRows()) {
+    return WebDatabase::COMMIT_NEEDED;
+  }
+  return WebDatabase::COMMIT_NOT_NEEDED;
+}
+
 }  // namespace autofill
