@@ -8,23 +8,31 @@
 #include <stdint.h>
 #include <set>
 #include <string>
+#include <unordered_set>
 #include <utility>
 
+#include <memory>
+#include <unordered_set>
+
 #include "base/base_paths.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
-#include "base/containers/hash_tables.h"
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
+#include "base/files/scoped_temp_dir.h"
+#include "base/run_loop.h"
+#include "base/stl_util.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/test/scoped_task_environment.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "components/bookmarks/browser/bookmark_model_observer.h"
 #include "components/bookmarks/browser/bookmark_undo_delegate.h"
 #include "components/bookmarks/browser/bookmark_utils.h"
+#include "components/bookmarks/browser/titled_url_match.h"
+#include "components/bookmarks/browser/url_and_title.h"
 #include "components/bookmarks/test/bookmark_test_helpers.h"
 #include "components/bookmarks/test/test_bookmark_client.h"
 #include "components/favicon_base/favicon_callback.h"
@@ -131,7 +139,7 @@ void PopulateNodeImpl(const std::vector<std::string>& description,
       // in debugging.
       static int next_folder_id = 1;
       TestNode* new_node = parent->Add(
-          base::MakeUnique<TestNode>(base::IntToString16(next_folder_id++),
+          std::make_unique<TestNode>(base::IntToString16(next_folder_id++),
                                      BookmarkNode::FOLDER),
           parent->child_count());
       PopulateNodeImpl(description, index, new_node);
@@ -145,7 +153,7 @@ void PopulateNodeImpl(const std::vector<std::string>& description,
       // likely means a space was forgotten.
       DCHECK(element.find('[') == std::string::npos);
       DCHECK(element.find(']') == std::string::npos);
-      parent->Add(base::MakeUnique<TestNode>(base::UTF8ToUTF16(element),
+      parent->Add(std::make_unique<TestNode>(base::UTF8ToUTF16(element),
                                              BookmarkNode::URL),
                   parent->child_count());
     }
@@ -215,7 +223,7 @@ void VerifyModelMatchesNode(TestNode* expected, const BookmarkNode* actual) {
 
 void VerifyNoDuplicateIDs(BookmarkModel* model) {
   ui::TreeNodeIterator<const BookmarkNode> it(model->root_node());
-  base::hash_set<int64_t> ids;
+  std::unordered_set<int64_t> ids;
   while (it.has_next())
     ASSERT_TRUE(ids.insert(it.Next()->id()).second);
 }
@@ -225,9 +233,7 @@ class BookmarkModelTest : public testing::Test,
                           public BookmarkUndoDelegate {
  public:
   struct ObserverDetails {
-    ObserverDetails() {
-      Set(NULL, NULL, -1, -1);
-    }
+    ObserverDetails() { Set(nullptr, nullptr, -1, -1); }
 
     void Set(const BookmarkNode* node1,
              const BookmarkNode* node2,
@@ -296,7 +302,7 @@ class BookmarkModelTest : public testing::Test,
                          const BookmarkNode* parent,
                          int index) override {
     ++added_count_;
-    observer_details_.Set(parent, NULL, index, -1);
+    observer_details_.Set(parent, nullptr, index, -1);
   }
 
   void OnWillRemoveBookmarks(BookmarkModel* model,
@@ -314,13 +320,13 @@ class BookmarkModelTest : public testing::Test,
                            const BookmarkNode* node,
                            const std::set<GURL>& removed_urls) override {
     ++removed_count_;
-    observer_details_.Set(parent, NULL, old_index, -1);
+    observer_details_.Set(parent, nullptr, old_index, -1);
   }
 
   void BookmarkNodeChanged(BookmarkModel* model,
                            const BookmarkNode* node) override {
     ++changed_count_;
-    observer_details_.Set(node, NULL, -1, -1);
+    observer_details_.Set(node, nullptr, -1, -1);
   }
 
   void OnWillChangeBookmarkNode(BookmarkModel* model,
@@ -429,7 +435,7 @@ class BookmarkModelTest : public testing::Test,
     model_->RemoveObserver(this);
 
     BookmarkPermanentNodeList extra_nodes;
-    extra_nodes.push_back(base::MakeUnique<BookmarkPermanentNode>(100));
+    extra_nodes.push_back(std::make_unique<BookmarkPermanentNode>(100));
     BookmarkPermanentNode* extra_node = extra_nodes.back().get();
 
     std::unique_ptr<TestBookmarkClient> client(new TestBookmarkClient);
@@ -471,17 +477,17 @@ class BookmarkModelTest : public testing::Test,
 
 TEST_F(BookmarkModelTest, InitialState) {
   const BookmarkNode* bb_node = model_->bookmark_bar_node();
-  ASSERT_TRUE(bb_node != NULL);
+  ASSERT_TRUE(bb_node != nullptr);
   EXPECT_EQ(0, bb_node->child_count());
   EXPECT_EQ(BookmarkNode::BOOKMARK_BAR, bb_node->type());
 
   const BookmarkNode* other_node = model_->other_node();
-  ASSERT_TRUE(other_node != NULL);
+  ASSERT_TRUE(other_node != nullptr);
   EXPECT_EQ(0, other_node->child_count());
   EXPECT_EQ(BookmarkNode::OTHER_NODE, other_node->type());
 
   const BookmarkNode* mobile_node = model_->mobile_node();
-  ASSERT_TRUE(mobile_node != NULL);
+  ASSERT_TRUE(mobile_node != nullptr);
   EXPECT_EQ(0, mobile_node->child_count());
   EXPECT_EQ(BookmarkNode::MOBILE, mobile_node->type());
 
@@ -497,7 +503,7 @@ TEST_F(BookmarkModelTest, AddURL) {
 
   const BookmarkNode* new_node = model_->AddURL(root, 0, title, url);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
 
   ASSERT_EQ(1, root->child_count());
   ASSERT_EQ(title, new_node->GetTitle());
@@ -518,7 +524,7 @@ TEST_F(BookmarkModelTest, AddURLWithUnicodeTitle) {
 
   const BookmarkNode* new_node = model_->AddURL(root, 0, title, url);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
 
   ASSERT_EQ(1, root->child_count());
   ASSERT_EQ(title, new_node->GetTitle());
@@ -532,7 +538,7 @@ TEST_F(BookmarkModelTest, AddURLWithUnicodeTitle) {
 }
 
 TEST_F(BookmarkModelTest, AddURLWithWhitespaceTitle) {
-  for (size_t i = 0; i < arraysize(url_whitespace_test_cases); ++i) {
+  for (size_t i = 0; i < base::size(url_whitespace_test_cases); ++i) {
     const BookmarkNode* root = model_->bookmark_bar_node();
     const base::string16 title(
         ASCIIToUTF16(url_whitespace_test_cases[i].input_title));
@@ -559,7 +565,7 @@ TEST_F(BookmarkModelTest, AddURLWithCreationTimeAndMetaInfo) {
   const BookmarkNode* new_node = model_->AddURLWithCreationTimeAndMetaInfo(
       root, 0, title, url, time, &meta_info);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
 
   ASSERT_EQ(1, root->child_count());
   ASSERT_EQ(title, new_node->GetTitle());
@@ -582,7 +588,7 @@ TEST_F(BookmarkModelTest, AddURLToMobileBookmarks) {
 
   const BookmarkNode* new_node = model_->AddURL(root, 0, title, url);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
 
   ASSERT_EQ(1, root->child_count());
   ASSERT_EQ(title, new_node->GetTitle());
@@ -601,7 +607,7 @@ TEST_F(BookmarkModelTest, AddFolder) {
 
   const BookmarkNode* new_node = model_->AddFolder(root, 0, title);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
 
   ASSERT_EQ(1, root->child_count());
   ASSERT_EQ(title, new_node->GetTitle());
@@ -615,11 +621,11 @@ TEST_F(BookmarkModelTest, AddFolder) {
   ClearCounts();
   model_->AddFolder(root, 0, title);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
 }
 
 TEST_F(BookmarkModelTest, AddFolderWithWhitespaceTitle) {
-  for (size_t i = 0; i < arraysize(title_whitespace_test_cases); ++i) {
+  for (size_t i = 0; i < base::size(title_whitespace_test_cases); ++i) {
     const BookmarkNode* root = model_->bookmark_bar_node();
     const base::string16 title(
         ASCIIToUTF16(title_whitespace_test_cases[i].input_title));
@@ -644,10 +650,10 @@ TEST_F(BookmarkModelTest, RemoveURL) {
   model_->Remove(root->GetChild(0));
   ASSERT_EQ(0, root->child_count());
   AssertObserverCount(0, 0, 1, 0, 0, 1, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
 
   // Make sure there is no mapping for the URL.
-  ASSERT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == NULL);
+  ASSERT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == nullptr);
 }
 
 TEST_F(BookmarkModelTest, RemoveFolder) {
@@ -667,10 +673,10 @@ TEST_F(BookmarkModelTest, RemoveFolder) {
   model_->Remove(root->GetChild(0));
   ASSERT_EQ(0, root->child_count());
   AssertObserverCount(0, 0, 1, 0, 0, 1, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
 
   // Make sure there is no mapping for the URL.
-  ASSERT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == NULL);
+  ASSERT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == nullptr);
 }
 
 TEST_F(BookmarkModelTest, RemoveAllUserBookmarks) {
@@ -727,12 +733,12 @@ TEST_F(BookmarkModelTest, SetTitle) {
   title = ASCIIToUTF16("foo2");
   model_->SetTitle(node, title);
   AssertObserverCount(0, 0, 0, 1, 0, 0, 1, 0, 0);
-  observer_details_.ExpectEquals(node, NULL, -1, -1);
+  observer_details_.ExpectEquals(node, nullptr, -1, -1);
   EXPECT_EQ(title, node->GetTitle());
 }
 
 TEST_F(BookmarkModelTest, SetTitleWithWhitespace) {
-  for (size_t i = 0; i < arraysize(title_whitespace_test_cases); ++i) {
+  for (size_t i = 0; i < base::size(title_whitespace_test_cases); ++i) {
     const BookmarkNode* root = model_->bookmark_bar_node();
     base::string16 title(ASCIIToUTF16("dummy"));
     const GURL url("http://foo.com");
@@ -756,7 +762,7 @@ TEST_F(BookmarkModelTest, SetURL) {
   url = GURL("http://foo2.com");
   model_->SetURL(node, url);
   AssertObserverCount(0, 0, 0, 1, 0, 0, 1, 0, 0);
-  observer_details_.ExpectEquals(node, NULL, -1, -1);
+  observer_details_.ExpectEquals(node, nullptr, -1, -1);
   EXPECT_EQ(url, node->url());
 }
 
@@ -797,8 +803,8 @@ TEST_F(BookmarkModelTest, Move) {
   ClearCounts();
   model_->Remove(root->GetChild(0));
   AssertObserverCount(0, 0, 1, 0, 0, 1, 0, 0, 0);
-  observer_details_.ExpectEquals(root, NULL, 0, -1);
-  EXPECT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == NULL);
+  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  EXPECT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == nullptr);
   EXPECT_EQ(0, root->child_count());
 }
 
@@ -1006,7 +1012,7 @@ TEST_F(BookmarkModelTest, GetBookmarksWithDups) {
   model_->AddURL(model_->bookmark_bar_node(), 0, title, url);
   model_->AddURL(model_->bookmark_bar_node(), 1, title, url);
 
-  std::vector<BookmarkModel::URLAndTitle> bookmarks;
+  std::vector<UrlAndTitle> bookmarks;
   model_->GetBookmarks(&bookmarks);
   ASSERT_EQ(1U, bookmarks.size());
   EXPECT_EQ(url, bookmarks[0].url);
@@ -1186,11 +1192,11 @@ TEST_F(BookmarkModelTest, GetMostRecentlyAddedUserNodeForURLSkipsManagedNodes) {
   const GURL url("http://google.com");
 
   // |url| is not bookmarked yet.
-  EXPECT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == NULL);
+  EXPECT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == nullptr);
 
   // Having a managed node doesn't count.
   model_->AddURL(managed_parent, 0, title, url);
-  EXPECT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == NULL);
+  EXPECT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == nullptr);
 
   // Now add a user node.
   const BookmarkNode* user = model_->AddURL(user_parent, 0, title, url);
@@ -1202,6 +1208,57 @@ TEST_F(BookmarkModelTest, GetMostRecentlyAddedUserNodeForURLSkipsManagedNodes) {
   EXPECT_EQ(user, model_->GetMostRecentlyAddedUserNodeForURL(url));
 }
 
+// Verifies that renaming a bookmark folder does not add the folder node to the
+// autocomplete index. crbug.com/778266
+TEST_F(BookmarkModelTest, RenamedFolderNodeExcludedFromIndex) {
+  // Add a folder.
+  const BookmarkNode* folder =
+      model_->AddFolder(model_->other_node(), 0, ASCIIToUTF16("MyFavorites"));
+
+  // Change the folder title.
+  model_->SetTitle(folder, ASCIIToUTF16("MyBookmarks"));
+
+  // There should be no matching bookmarks.
+  std::vector<TitledUrlMatch> matches;
+  model_->GetBookmarksMatching(ASCIIToUTF16("MyB"), /*max_count = */ 1,
+                               query_parser::MatchingAlgorithm::DEFAULT,
+                               &matches);
+  EXPECT_TRUE(matches.empty());
+}
+
+// Verifies the TitledUrlIndex is probably loaded.
+TEST(BookmarkModelLoadTest, TitledUrlIndexPopulatedOnLoad) {
+  // Create a model with a single url.
+  base::ScopedTempDir tmp_dir;
+  ASSERT_TRUE(tmp_dir.CreateUniqueTempDir());
+  base::test::ScopedTaskEnvironment scoped_task_environment;
+  std::unique_ptr<BookmarkModel> model =
+      std::make_unique<BookmarkModel>(std::make_unique<TestBookmarkClient>());
+  model->Load(nullptr, tmp_dir.GetPath(), base::ThreadTaskRunnerHandle::Get(),
+              base::ThreadTaskRunnerHandle::Get());
+  test::WaitForBookmarkModelToLoad(model.get());
+  const GURL node_url("http://google.com");
+  model->AddURL(model->bookmark_bar_node(), 0, base::ASCIIToUTF16("User"),
+                node_url);
+  // This is necessary to ensure the save completes.
+  base::RunLoop().RunUntilIdle();
+
+  // Recreate the model and ensure GetBookmarksMatching() returns the url that
+  // was added.
+  model =
+      std::make_unique<BookmarkModel>(std::make_unique<TestBookmarkClient>());
+  model->Load(nullptr, tmp_dir.GetPath(), base::ThreadTaskRunnerHandle::Get(),
+              base::ThreadTaskRunnerHandle::Get());
+  test::WaitForBookmarkModelToLoad(model.get());
+
+  std::vector<TitledUrlMatch> matches;
+  model->GetBookmarksMatching(base::ASCIIToUTF16("user"), 1,
+                              query_parser::MatchingAlgorithm::DEFAULT,
+                              &matches);
+  ASSERT_EQ(1u, matches.size());
+  EXPECT_EQ(node_url, matches[0].node->GetTitledUrlNodeUrl());
+}
+
 TEST(BookmarkNodeTest, NodeMetaInfo) {
   GURL url;
   BookmarkNode node(url);
@@ -1254,7 +1311,7 @@ TEST(BookmarkModelTest2, CreateAndRestore) {
     { "a b c [ d e [ f ] ]", "g h i [ j k [ l ] ]"},
   };
   std::unique_ptr<BookmarkModel> model;
-  for (size_t i = 0; i < arraysize(data); ++i) {
+  for (size_t i = 0; i < base::size(data); ++i) {
     model = TestBookmarkClient::CreateModel();
 
     TestNode bbn;
@@ -1297,13 +1354,12 @@ class BookmarkModelFaviconTest : public testing::Test,
       favicon_base::FaviconImageResult image_result;
       image_result.image = image;
       image_result.icon_url = icon_url;
-      model_->OnFaviconDataAvailable(node, favicon_base::IconType::FAVICON,
+      model_->OnFaviconDataAvailable(node, favicon_base::IconType::kFavicon,
                                      image_result);
   }
 
   bool WasNodeUpdated(const BookmarkNode* node) {
-    return std::find(updated_nodes_.begin(), updated_nodes_.end(), node) !=
-           updated_nodes_.end();
+    return base::ContainsValue(updated_nodes_, node);
   }
 
   void ClearUpdatedNodes() {
