@@ -5,16 +5,17 @@
 #include "components/bookmarks/browser/bookmark_utils.h"
 
 #include <stdint.h>
+#include <memory>
+#include <unordered_set>
 #include <utility>
 
 #include "base/bind.h"
-#include "base/containers/hash_tables.h"
 #include "base/files/file_path.h"
 #include "base/i18n/case_conversion.h"
 #include "base/i18n/string_search.h"
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/metrics/user_metrics_action.h"
+#include "base/stl_util.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
@@ -85,8 +86,8 @@ bool MoreRecentlyModified(const BookmarkNode* n1, const BookmarkNode* n2) {
 bool DoesBookmarkTextContainWords(const base::string16& text,
                                   const std::vector<base::string16>& words) {
   for (size_t i = 0; i < words.size(); ++i) {
-    if (!base::i18n::StringSearchIgnoringCaseAndAccents(
-            words[i], text, NULL, NULL)) {
+    if (!base::i18n::StringSearchIgnoringCaseAndAccents(words[i], text, nullptr,
+                                                        nullptr)) {
       return false;
     }
   }
@@ -102,7 +103,7 @@ bool DoesBookmarkContainWords(const BookmarkNode* node,
          DoesBookmarkTextContainWords(
              url_formatter::FormatUrl(
                  node->url(), url_formatter::kFormatUrlOmitNothing,
-                 net::UnescapeRule::NORMAL, NULL, NULL, NULL),
+                 net::UnescapeRule::NORMAL, nullptr, nullptr, nullptr),
              words);
 }
 
@@ -135,7 +136,7 @@ const BookmarkNode* GetNodeByID(const BookmarkNode* node, int64_t id) {
     if (result)
       return result;
   }
-  return NULL;
+  return nullptr;
 }
 
 // Attempts to shorten a URL safely (i.e., by preventing the end of the URL
@@ -255,8 +256,7 @@ void CopyToClipboard(BookmarkModel* model,
     if (!HasSelectedAncestor(model, nodes, nodes[i]->parent()))
       filtered_nodes.push_back(nodes[i]);
 
-  BookmarkNodeData(filtered_nodes).
-      WriteToClipboard(ui::CLIPBOARD_TYPE_COPY_PASTE);
+  BookmarkNodeData(filtered_nodes).WriteToClipboard();
 
   if (remove_nodes) {
     ScopedGroupBookmarkActions group_cut(model);
@@ -274,7 +274,7 @@ void MakeTitleUnique(const BookmarkModel* model,
                      const BookmarkNode* parent,
                      const GURL& url,
                      base::string16* title) {
-  base::hash_set<base::string16> titles;
+  std::unordered_set<base::string16> titles;
   base::string16 original_title_lower = base::i18n::ToLower(*title);
   for (int i = 0; i < parent->child_count(); i++) {
     const BookmarkNode* node = parent->GetChild(i);
@@ -353,9 +353,8 @@ std::vector<const BookmarkNode*> GetMostRecentlyModifiedUserFolders(
       if (max_count == 0) {
         nodes.push_back(parent);
       } else {
-        std::vector<const BookmarkNode*>::iterator i =
-            std::upper_bound(nodes.begin(), nodes.end(), parent,
-                             &MoreRecentlyModified);
+        auto i = std::upper_bound(nodes.begin(), nodes.end(), parent,
+                                  &MoreRecentlyModified);
         if (nodes.size() < max_count || i != nodes.end()) {
           nodes.insert(i, parent);
           while (nodes.size() > max_count)
@@ -374,7 +373,7 @@ std::vector<const BookmarkNode*> GetMostRecentlyModifiedUserFolders(
     for (int i = 0; i < root_node->child_count(); ++i) {
       const BookmarkNode* node = root_node->GetChild(i);
       if (node->IsVisible() && model->client()->CanBeEditedByUser(node) &&
-          std::find(nodes.begin(), nodes.end(), node) == nodes.end()) {
+          !base::ContainsValue(nodes, node)) {
         nodes.push_back(node);
 
         if (nodes.size() == max_count)
@@ -392,9 +391,8 @@ void GetMostRecentlyAddedEntries(BookmarkModel* model,
   while (iterator.has_next()) {
     const BookmarkNode* node = iterator.Next();
     if (node->is_url()) {
-      std::vector<const BookmarkNode*>::iterator insert_position =
-          std::upper_bound(nodes->begin(), nodes->end(), node,
-                           &MoreRecentlyAdded);
+      auto insert_position = std::upper_bound(nodes->begin(), nodes->end(),
+                                              node, &MoreRecentlyAdded);
       if (nodes->size() < count || insert_position != nodes->end()) {
         nodes->insert(insert_position, node);
         while (nodes->size() > count)
@@ -462,11 +460,10 @@ void RegisterManagedBookmarksPrefs(PrefRegistrySimple* registry) {
   // want to sync the expanded state of folders, it should be part of
   // bookmark sync itself (i.e., a property of the sync folder nodes).
   registry->RegisterListPref(prefs::kBookmarkEditorExpandedNodes,
-                             base::MakeUnique<base::ListValue>());
+                             std::make_unique<base::ListValue>());
   registry->RegisterListPref(prefs::kManagedBookmarks);
   registry->RegisterStringPref(
       prefs::kManagedBookmarksFolderName, std::string());
-  registry->RegisterListPref(prefs::kSupervisedBookmarks);
 }
 
 const BookmarkNode* GetParentForNewNodes(
@@ -498,9 +495,7 @@ void DeleteBookmarkFolders(BookmarkModel* model,
                            const std::vector<int64_t>& ids) {
   // Remove the folders that were removed. This has to be done after all the
   // other changes have been committed.
-  for (std::vector<int64_t>::const_iterator iter = ids.begin();
-       iter != ids.end();
-       ++iter) {
+  for (auto iter = ids.begin(); iter != ids.end(); ++iter) {
     const BookmarkNode* node = GetBookmarkNodeByID(model, *iter);
     if (!node)
       continue;
