@@ -5,11 +5,12 @@
 #include "components/bookmarks/browser/bookmark_utils.h"
 
 #include <stddef.h>
+
+#include <memory>
 #include <utility>
 #include <vector>
 
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
+#include "base/stl_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/scoped_task_environment.h"
 #include "build/build_config.h"
@@ -239,7 +240,7 @@ TEST_F(BookmarkUtilsTest, GetBookmarksMatchingPropertiesConjunction) {
                                                &query.url, &query.title};
 
   // Test two fields matching.
-  for (size_t i = 0; i < arraysize(fields); i++) {
+  for (size_t i = 0; i < base::size(fields); i++) {
     std::unique_ptr<base::string16> original_value(fields[i]->release());
     GetBookmarksMatchingProperties(model.get(), query, 100, &nodes);
     ASSERT_EQ(1U, nodes.size());
@@ -249,7 +250,7 @@ TEST_F(BookmarkUtilsTest, GetBookmarksMatchingPropertiesConjunction) {
   }
 
   // Test two fields matching with one non-matching field.
-  for (size_t i = 0; i < arraysize(fields); i++) {
+  for (size_t i = 0; i < base::size(fields); i++) {
     std::unique_ptr<base::string16> original_value(fields[i]->release());
     fields[i]->reset(new base::string16(ASCIIToUTF16("fjdkslafjkldsa")));
     GetBookmarksMatchingProperties(model.get(), query, 100, &nodes);
@@ -422,7 +423,7 @@ TEST_F(BookmarkUtilsTest, PasteNonEditableNodes) {
   // Load a model with an extra node that is not editable.
   std::unique_ptr<TestBookmarkClient> client(new TestBookmarkClient());
   BookmarkPermanentNodeList extra_nodes;
-  extra_nodes.push_back(base::MakeUnique<BookmarkPermanentNode>(100));
+  extra_nodes.push_back(std::make_unique<BookmarkPermanentNode>(100));
   BookmarkPermanentNode* extra_node = extra_nodes.back().get();
   client->SetExtraNodesToLoad(std::move(extra_nodes));
 
@@ -573,7 +574,7 @@ TEST_F(BookmarkUtilsTest, RemoveAllBookmarks) {
   // Load a model with an extra node that is not editable.
   std::unique_ptr<TestBookmarkClient> client(new TestBookmarkClient());
   BookmarkPermanentNodeList extra_nodes;
-  extra_nodes.push_back(base::MakeUnique<BookmarkPermanentNode>(100));
+  extra_nodes.push_back(std::make_unique<BookmarkPermanentNode>(100));
   BookmarkPermanentNode* extra_node = extra_nodes.back().get();
   client->SetExtraNodesToLoad(std::move(extra_nodes));
 
