@@ -6,7 +6,9 @@
 
 #include <utility>
 
+#include "base/token.h"
 #include "net/base/ip_address.h"
+#include "services/service_manager/public/cpp/connector.h"
 
 namespace cast_channel {
 
@@ -29,7 +31,10 @@ MockCastTransportDelegate::~MockCastTransportDelegate() {}
 MockCastSocketObserver::MockCastSocketObserver() {}
 MockCastSocketObserver::~MockCastSocketObserver() {}
 
-MockCastSocketService::MockCastSocketService() {}
+MockCastSocketService::MockCastSocketService(
+    const scoped_refptr<base::SingleThreadTaskRunner>& task_runner) {
+  SetTaskRunnerForTest(task_runner);
+}
 MockCastSocketService::~MockCastSocketService() {}
 
 MockCastSocket::MockCastSocket()
@@ -44,4 +49,15 @@ net::IPEndPoint CreateIPEndPointForTest() {
   return net::IPEndPoint(net::IPAddress(192, 168, 1, 1), 8009);
 }
 
+MockCastMessageHandler::MockCastMessageHandler(
+    MockCastSocketService* socket_service)
+    : CastMessageHandler(socket_service,
+                         /* connector */ nullptr,
+                         base::Token{},
+                         "userAgent",
+                         "1.2.3.4",
+                         "en-US") {}
+
+MockCastMessageHandler::~MockCastMessageHandler() = default;
+
 }  // namespace cast_channel
