@@ -4,13 +4,15 @@
 
 #include "components/certificate_transparency/log_dns_client.h"
 
+#include "base/big_endian.h"
 #include "base/bind.h"
-#include "base/callback_helpers.h"
 #include "base/format_macros.h"
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/stl_util.h"
+#include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
@@ -18,12 +20,14 @@
 #include "base/time/time.h"
 #include "components/base32/base32.h"
 #include "crypto/sha2.h"
+#include "net/base/completion_once_callback.h"
+#include "net/base/sys_addrinfo.h"
 #include "net/cert/merkle_audit_proof.h"
 #include "net/dns/dns_client.h"
-#include "net/dns/dns_config_service.h"
-#include "net/dns/dns_protocol.h"
+#include "net/dns/dns_config.h"
 #include "net/dns/dns_response.h"
 #include "net/dns/dns_transaction.h"
+#include "net/dns/public/dns_protocol.h"
 #include "net/dns/record_parsed.h"
 #include "net/dns/record_rdata.h"
 
@@ -31,26 +35,52 @@ namespace certificate_transparency {
 
 namespace {
 
-// Used by UMA_HISTOGRAM_ENUMERATION to record query success/failures.
-// These values are written to logs.  New enum values can be added, but existing
-// enums must never be renumbered or deleted and reused.
-enum QueryStatus {
-  QUERY_STATUS_SUCCESS = 0,
-  QUERY_STATUS_FAILED_UNKNOWN = 1,
-  QUERY_STATUS_FAILED_NAME_RESOLUTION = 2,
-  QUERY_STATUS_FAILED_LEAF_INDEX_MALFORMED = 3,
-  QUERY_STATUS_FAILED_INCLUSION_PROOF_MALFORMED = 4,
-  QUERY_STATUS_MAX  // Upper bound
-};
-
-void LogQueryStatus(QueryStatus result) {
-  UMA_HISTOGRAM_ENUMERATION("Net.CertificateTransparency.DnsQueryStatus",
-                            result, QUERY_STATUS_MAX);
-}
-
-void LogQueryDuration(const base::TimeDelta& duration) {
+void LogQueryDuration(net::Error error, const base::TimeDelta& duration) {
   UMA_HISTOGRAM_MEDIUM_TIMES("Net.CertificateTransparency.DnsQueryDuration",
                              duration);
+
+  if (error == net::OK) {
+    UMA_HISTOGRAM_MEDIUM_TIMES(
+        "Net.CertificateTransparency.DnsQueryDuration.Success", duration);
+  }
+}
+
+void LogQueryResult(const std::string& name,
+                    net::Error error,
+                    const net::DnsResponse* response) {
+  base::UmaHistogramSparse(
+      base::StrCat({"Net.CertificateTransparency.DnsQuery", name, "Error"}),
+      -error);
+
+  if (response) {
+    base::UmaHistogramSparse(
+        base::StrCat({"Net.CertificateTransparency.DnsQuery", name, "Rcode"}),
+        response->rcode());
+  }
+}
+
+// Returns an EDNS option that disables the client subnet extension, as
+// described in https://tools.ietf.org/html/rfc7871. This is to avoid the
+// privacy issues caused by this extension being enabled in recursive resolvers
+// used by this DNS client (see the "Privacy Note" in RFC7871).
+net::OptRecordRdata::Opt OptToDisableClientSubnetExtension() {
+  const uint16_t kClientSubnetExtensionCode = 8;
+  // https://www.iana.org/assignments/address-family-numbers/address-family-numbers.xhtml
+  const uint16_t kIanaAddressFamilyIpV4 = 1;
+
+  char buf[4];
+  base::BigEndianWriter writer(buf, base::size(buf));
+  // family - address is empty so the value of this is irrelevant, so long as
+  // it's valid (see https://tools.ietf.org/html/rfc7871#section-7.1.2).
+  writer.WriteU16(kIanaAddressFamilyIpV4);
+  // source prefix length - 0 to disable this extension.
+  writer.WriteU8(0);
+  // scope prefix length - must be 0 for queries.
+  writer.WriteU8(0);
+  // no address - don't want a client subnet in the query.
+
+  return net::OptRecordRdata::Opt(kClientSubnetExtensionCode,
+                                  base::StringPiece(buf, base::size(buf)));
 }
 
 // Parses the DNS response and extracts a single string from the TXT RDATA.
@@ -124,24 +154,32 @@ bool ParseAuditPath(const net::DnsResponse& response,
 // Encapsulates the state machine required to get an audit proof from a Merkle
 // leaf hash. This requires a DNS request to obtain the leaf index, then a
 // series of DNS requests to get the nodes of the proof.
-class LogDnsClient::AuditProofQuery {
+class AuditProofQueryImpl : public LogDnsClient::AuditProofQuery {
  public:
-  // The LogDnsClient is guaranteed to outlive the AuditProofQuery, so it's safe
-  // to leave ownership of |dns_client| with LogDnsClient.
-  AuditProofQuery(net::DnsClient* dns_client,
-                  const std::string& domain_for_log,
-                  const net::NetLogWithSource& net_log);
+  // The API contract of LogDnsClient requires that callers make sure the
+  // AuditProofQuery does not outlive LogDnsClient, so it's safe to leave
+  // ownership of |dns_client| with LogDnsClient.
+  AuditProofQueryImpl(net::DnsClient* dns_client,
+                      const std::string& domain_for_log,
+                      const net::NetLogWithSource& net_log);
+
+  ~AuditProofQueryImpl() override;
 
   // Begins the process of getting an audit proof for the CT log entry with a
   // leaf hash of |leaf_hash|. The proof will be for a tree of size |tree_size|.
   // If it cannot be obtained synchronously, net::ERR_IO_PENDING will be
   // returned and |callback| will be invoked when the operation has completed
-  // asynchronously. Ownership of |proof| remains with the caller, and it must
-  // not be deleted until the operation is complete.
+  // asynchronously. If the operation is cancelled (by deleting the
+  // AuditProofQueryImpl), |cancellation_callback| will be invoked.
   net::Error Start(std::string leaf_hash,
                    uint64_t tree_size,
-                   const net::CompletionCallback& callback,
-                   net::ct::MerkleAuditProof* out_proof);
+                   net::CompletionOnceCallback callback,
+                   base::OnceClosure cancellation_callback);
+
+  // Returns the proof that is being obtained by this query.
+  // It is only guaranteed to be populated once either Start() returns net::OK
+  // or the completion callback is invoked with net::OK.
+  const net::ct::MerkleAuditProof& GetProof() const override;
 
  private:
   enum class State {
@@ -202,9 +240,11 @@ class LogDnsClient::AuditProofQuery {
   // The Merkle leaf hash of the CT log entry an audit proof is required for.
   std::string leaf_hash_;
   // The audit proof to populate.
-  net::ct::MerkleAuditProof* proof_;
+  net::ct::MerkleAuditProof proof_;
   // The callback to invoke when the query is complete.
-  net::CompletionCallback callback_;
+  net::CompletionOnceCallback callback_;
+  // The callback to invoke when the query is cancelled.
+  base::OnceClosure cancellation_callback_;
   // The DnsClient to use for sending DNS requests to the CT log.
   net::DnsClient* dns_client_;
   // The most recent DNS request. Null if no DNS requests have been made.
@@ -217,36 +257,40 @@ class LogDnsClient::AuditProofQuery {
   // The time that Start() was last called. Used to measure query duration.
   base::TimeTicks start_time_;
   // Produces WeakPtrs to |this| for binding callbacks.
-  base::WeakPtrFactory<AuditProofQuery> weak_ptr_factory_;
+  base::WeakPtrFactory<AuditProofQueryImpl> weak_ptr_factory_;
 };
 
-LogDnsClient::AuditProofQuery::AuditProofQuery(
-    net::DnsClient* dns_client,
-    const std::string& domain_for_log,
-    const net::NetLogWithSource& net_log)
+AuditProofQueryImpl::AuditProofQueryImpl(net::DnsClient* dns_client,
+                                         const std::string& domain_for_log,
+                                         const net::NetLogWithSource& net_log)
     : next_state_(State::NONE),
       domain_for_log_(domain_for_log),
       dns_client_(dns_client),
+      last_dns_response_(nullptr),
       net_log_(net_log),
       weak_ptr_factory_(this) {
   DCHECK(dns_client_);
   DCHECK(!domain_for_log_.empty());
 }
 
+AuditProofQueryImpl::~AuditProofQueryImpl() {
+  if (next_state_ != State::NONE)
+    std::move(cancellation_callback_).Run();
+}
+
 // |leaf_hash| is not a const-ref to allow callers to std::move that string into
 // the method, avoiding the need to make a copy.
-net::Error LogDnsClient::AuditProofQuery::Start(
-    std::string leaf_hash,
-    uint64_t tree_size,
-    const net::CompletionCallback& callback,
-    net::ct::MerkleAuditProof* proof) {
+net::Error AuditProofQueryImpl::Start(std::string leaf_hash,
+                                      uint64_t tree_size,
+                                      net::CompletionOnceCallback callback,
+                                      base::OnceClosure cancellation_callback) {
   // It should not already be in progress.
   DCHECK_EQ(State::NONE, next_state_);
   start_time_ = base::TimeTicks::Now();
-  proof_ = proof;
-  proof_->tree_size = tree_size;
+  proof_.tree_size = tree_size;
   leaf_hash_ = std::move(leaf_hash);
-  callback_ = callback;
+  callback_ = std::move(callback);
+  cancellation_callback_ = std::move(cancellation_callback);
   // The first step in the query is to request the leaf index corresponding to
   // |leaf_hash| from the CT log.
   next_state_ = State::REQUEST_LEAF_INDEX;
@@ -254,7 +298,11 @@ net::Error LogDnsClient::AuditProofQuery::Start(
   return DoLoop(net::OK);
 }
 
-net::Error LogDnsClient::AuditProofQuery::DoLoop(net::Error result) {
+const net::ct::MerkleAuditProof& AuditProofQueryImpl::GetProof() const {
+  return proof_;
+}
+
+net::Error AuditProofQueryImpl::DoLoop(net::Error result) {
   CHECK_NE(State::NONE, next_state_);
   State state;
   do {
@@ -266,6 +314,8 @@ net::Error LogDnsClient::AuditProofQuery::DoLoop(net::Error result) {
         break;
       case State::REQUEST_LEAF_INDEX_COMPLETE:
         result = RequestLeafIndexComplete(result);
+        if (result == net::OK)
+          LogQueryResult("LeafIndex", net::OK, last_dns_response_);
         break;
       case State::REQUEST_AUDIT_PROOF_NODES:
         result = RequestAuditProofNodes();
@@ -281,31 +331,20 @@ net::Error LogDnsClient::AuditProofQuery::DoLoop(net::Error result) {
 
   if (result != net::ERR_IO_PENDING) {
     // If the query is complete, log some metrics.
-    LogQueryDuration(base::TimeTicks::Now() - start_time_);
-
-    switch (result) {
-      case net::OK:
-        LogQueryStatus(QUERY_STATUS_SUCCESS);
+    LogQueryDuration(result, base::TimeTicks::Now() - start_time_);
+    switch (state) {
+      case State::REQUEST_LEAF_INDEX:
+      case State::REQUEST_LEAF_INDEX_COMPLETE:
+        // An error must have occurred if the query completed in this state.
+        LogQueryResult("LeafIndex", result, last_dns_response_);
         break;
-      case net::ERR_NAME_RESOLUTION_FAILED:
-        LogQueryStatus(QUERY_STATUS_FAILED_NAME_RESOLUTION);
+      case State::REQUEST_AUDIT_PROOF_NODES:
+      case State::REQUEST_AUDIT_PROOF_NODES_COMPLETE:
+        // The query may have completed successfully.
+        LogQueryResult("AuditProof", result, last_dns_response_);
         break;
-      case net::ERR_DNS_MALFORMED_RESPONSE:
-        switch (state) {
-          case State::REQUEST_LEAF_INDEX_COMPLETE:
-            LogQueryStatus(QUERY_STATUS_FAILED_LEAF_INDEX_MALFORMED);
-            break;
-          case State::REQUEST_AUDIT_PROOF_NODES_COMPLETE:
-            LogQueryStatus(QUERY_STATUS_FAILED_INCLUSION_PROOF_MALFORMED);
-            break;
-          default:
-            NOTREACHED();
-            break;
-        }
-        break;
-      default:
-        // Some other error occurred.
-        LogQueryStatus(QUERY_STATUS_FAILED_UNKNOWN);
+      case State::NONE:
+        NOTREACHED();
         break;
     }
   }
@@ -313,7 +352,7 @@ net::Error LogDnsClient::AuditProofQuery::DoLoop(net::Error result) {
   return result;
 }
 
-void LogDnsClient::AuditProofQuery::OnDnsTransactionComplete(
+void AuditProofQueryImpl::OnDnsTransactionComplete(
     net::DnsTransaction* transaction,
     int net_error,
     const net::DnsResponse* response) {
@@ -325,14 +364,14 @@ void LogDnsClient::AuditProofQuery::OnDnsTransactionComplete(
   // callback. OnDnsTransactionComplete() will be invoked again once the I/O
   // is complete, and can invoke the completion callback then if appropriate.
   if (result != net::ERR_IO_PENDING) {
-    // The callback will delete this query (now that it has finished), so copy
+    // The callback may delete this query (now that it has finished), so copy
     // |callback_| before running it so that it is not deleted along with the
     // query, mid-callback-execution (which would result in a crash).
-    base::ResetAndReturn(&callback_).Run(result);
+    std::move(callback_).Run(result);
   }
 }
 
-net::Error LogDnsClient::AuditProofQuery::RequestLeafIndex() {
+net::Error AuditProofQueryImpl::RequestLeafIndex() {
   std::string encoded_leaf_hash = base32::Base32Encode(
       leaf_hash_, base32::Base32EncodePolicy::OMIT_PADDING);
   DCHECK_EQ(encoded_leaf_hash.size(), 52u);
@@ -350,14 +389,13 @@ net::Error LogDnsClient::AuditProofQuery::RequestLeafIndex() {
 
 // Stores the received leaf index in |proof_->leaf_index|.
 // If successful, the audit proof nodes will be requested next.
-net::Error LogDnsClient::AuditProofQuery::RequestLeafIndexComplete(
-    net::Error result) {
+net::Error AuditProofQueryImpl::RequestLeafIndexComplete(net::Error result) {
   if (result != net::OK) {
     return result;
   }
 
   DCHECK(last_dns_response_);
-  if (!ParseLeafIndex(*last_dns_response_, &proof_->leaf_index)) {
+  if (!ParseLeafIndex(*last_dns_response_, &proof_.leaf_index)) {
     return net::ERR_DNS_MALFORMED_RESPONSE;
   }
 
@@ -367,7 +405,7 @@ net::Error LogDnsClient::AuditProofQuery::RequestLeafIndexComplete(
   // b) the wrong leaf hash was provided.
   // c) there is a bug server-side.
   // The first two are more likely, so return ERR_INVALID_ARGUMENT.
-  if (proof_->leaf_index >= proof_->tree_size) {
+  if (proof_.leaf_index >= proof_.tree_size) {
     return net::ERR_INVALID_ARGUMENT;
   }
 
@@ -375,17 +413,17 @@ net::Error LogDnsClient::AuditProofQuery::RequestLeafIndexComplete(
   return net::OK;
 }
 
-net::Error LogDnsClient::AuditProofQuery::RequestAuditProofNodes() {
+net::Error AuditProofQueryImpl::RequestAuditProofNodes() {
   // Test pre-conditions (should be guaranteed by DNS response validation).
-  if (proof_->leaf_index >= proof_->tree_size ||
-      proof_->nodes.size() >= net::ct::CalculateAuditPathLength(
-                                  proof_->leaf_index, proof_->tree_size)) {
+  if (proof_.leaf_index >= proof_.tree_size ||
+      proof_.nodes.size() >= net::ct::CalculateAuditPathLength(
+                                 proof_.leaf_index, proof_.tree_size)) {
     return net::ERR_UNEXPECTED;
   }
 
   std::string qname = base::StringPrintf(
-      "%zu.%" PRIu64 ".%" PRIu64 ".tree.%s.", proof_->nodes.size(),
-      proof_->leaf_index, proof_->tree_size, domain_for_log_.c_str());
+      "%zu.%" PRIu64 ".%" PRIu64 ".tree.%s.", proof_.nodes.size(),
+      proof_.leaf_index, proof_.tree_size, domain_for_log_.c_str());
 
   if (!StartDnsTransaction(qname)) {
     return net::ERR_NAME_RESOLUTION_FAILED;
@@ -395,44 +433,44 @@ net::Error LogDnsClient::AuditProofQuery::RequestAuditProofNodes() {
   return net::ERR_IO_PENDING;
 }
 
-net::Error LogDnsClient::AuditProofQuery::RequestAuditProofNodesComplete(
+net::Error AuditProofQueryImpl::RequestAuditProofNodesComplete(
     net::Error result) {
   if (result != net::OK) {
     return result;
   }
 
   const uint64_t audit_path_length =
-      net::ct::CalculateAuditPathLength(proof_->leaf_index, proof_->tree_size);
+      net::ct::CalculateAuditPathLength(proof_.leaf_index, proof_.tree_size);
 
   // The calculated |audit_path_length| can't ever be greater than 64, so
   // deriving the amount of memory to reserve from the untrusted |leaf_index|
   // is safe.
-  proof_->nodes.reserve(audit_path_length);
+  proof_.nodes.reserve(audit_path_length);
 
   DCHECK(last_dns_response_);
-  if (!ParseAuditPath(*last_dns_response_, proof_)) {
+  if (!ParseAuditPath(*last_dns_response_, &proof_)) {
     return net::ERR_DNS_MALFORMED_RESPONSE;
   }
 
   // Keep requesting more proof nodes until all of them are received.
-  if (proof_->nodes.size() < audit_path_length) {
+  if (proof_.nodes.size() < audit_path_length) {
     next_state_ = State::REQUEST_AUDIT_PROOF_NODES;
   }
 
   return net::OK;
 }
 
-bool LogDnsClient::AuditProofQuery::StartDnsTransaction(
-    const std::string& qname) {
+bool AuditProofQueryImpl::StartDnsTransaction(const std::string& qname) {
   net::DnsTransactionFactory* factory = dns_client_->GetTransactionFactory();
   if (!factory) {
     return false;
   }
 
+  last_dns_response_ = nullptr;
   current_dns_transaction_ = factory->CreateTransaction(
       qname, net::dns_protocol::kTypeTXT,
-      base::Bind(&LogDnsClient::AuditProofQuery::OnDnsTransactionComplete,
-                 weak_ptr_factory_.GetWeakPtr()),
+      base::BindOnce(&AuditProofQueryImpl::OnDnsTransactionComplete,
+                     weak_ptr_factory_.GetWeakPtr()),
       net_log_);
 
   current_dns_transaction_->Start();
@@ -441,11 +479,11 @@ bool LogDnsClient::AuditProofQuery::StartDnsTransaction(
 
 LogDnsClient::LogDnsClient(std::unique_ptr<net::DnsClient> dns_client,
                            const net::NetLogWithSource& net_log,
-                           size_t max_concurrent_queries)
+                           size_t max_in_flight_queries)
     : dns_client_(std::move(dns_client)),
       net_log_(net_log),
-      max_concurrent_queries_(max_concurrent_queries),
-      weak_ptr_factory_(this) {
+      in_flight_queries_(0),
+      max_in_flight_queries_(max_in_flight_queries) {
   CHECK(dns_client_);
   net::NetworkChangeNotifier::AddDNSObserver(this);
   UpdateDnsConfig();
@@ -463,9 +501,9 @@ void LogDnsClient::OnInitialDNSConfigRead() {
   UpdateDnsConfig();
 }
 
-void LogDnsClient::NotifyWhenNotThrottled(const base::Closure& callback) {
-  DCHECK(HasMaxConcurrentQueriesInProgress());
-  not_throttled_callbacks_.push_back(callback);
+void LogDnsClient::NotifyWhenNotThrottled(base::OnceClosure callback) {
+  DCHECK(HasMaxQueriesInFlight());
+  not_throttled_callbacks_.emplace_back(std::move(callback));
 }
 
 // |leaf_hash| is not a const-ref to allow callers to std::move that string into
@@ -474,57 +512,66 @@ net::Error LogDnsClient::QueryAuditProof(
     base::StringPiece domain_for_log,
     std::string leaf_hash,
     uint64_t tree_size,
-    net::ct::MerkleAuditProof* proof,
+    std::unique_ptr<AuditProofQuery>* out_query,
     const net::CompletionCallback& callback) {
-  DCHECK(proof);
+  DCHECK(out_query);
 
   if (domain_for_log.empty() || leaf_hash.size() != crypto::kSHA256Length) {
     return net::ERR_INVALID_ARGUMENT;
   }
 
-  if (HasMaxConcurrentQueriesInProgress()) {
+  if (HasMaxQueriesInFlight()) {
     return net::ERR_TEMPORARILY_THROTTLED;
   }
 
-  AuditProofQuery* query = new AuditProofQuery(
-      dns_client_.get(), domain_for_log.as_string(), net_log_);
-  // Transfers ownership of |query| to |audit_proof_queries_|.
-  audit_proof_queries_.emplace_back(query);
+  auto* query = new AuditProofQueryImpl(dns_client_.get(),
+                                        domain_for_log.as_string(), net_log_);
+  out_query->reset(query);
+
+  ++in_flight_queries_;
 
   return query->Start(std::move(leaf_hash), tree_size,
-                      base::Bind(&LogDnsClient::QueryAuditProofComplete,
-                                 weak_ptr_factory_.GetWeakPtr(),
-                                 base::Unretained(query), callback),
-                      proof);
+                      base::BindOnce(&LogDnsClient::QueryAuditProofComplete,
+                                     base::Unretained(this), callback),
+                      base::BindOnce(&LogDnsClient::QueryAuditProofCancelled,
+                                     base::Unretained(this)));
 }
 
 void LogDnsClient::QueryAuditProofComplete(
-    AuditProofQuery* query,
-    const net::CompletionCallback& callback,
+    const net::CompletionCallback& completion_callback,
     int net_error) {
-  DCHECK(query);
+  --in_flight_queries_;
 
-  // Finished with the query - destroy it.
-  auto query_iterator =
-      std::find_if(audit_proof_queries_.begin(), audit_proof_queries_.end(),
-                   [query](const std::unique_ptr<AuditProofQuery>& p) {
-                     return p.get() == query;
-                   });
-  DCHECK(query_iterator != audit_proof_queries_.end());
-  audit_proof_queries_.erase(query_iterator);
+  // Move the "not throttled" callbacks to a local variable, just in case one of
+  // the callbacks deletes this LogDnsClient.
+  std::list<base::OnceClosure> not_throttled_callbacks =
+      std::move(not_throttled_callbacks_);
 
-  callback.Run(net_error);
+  completion_callback.Run(net_error);
 
   // Notify interested parties that the next query will not be throttled.
-  std::list<base::Closure> callbacks = std::move(not_throttled_callbacks_);
-  for (const base::Closure& callback : callbacks) {
-    callback.Run();
+  for (auto& callback : not_throttled_callbacks) {
+    std::move(callback).Run();
   }
 }
 
-bool LogDnsClient::HasMaxConcurrentQueriesInProgress() const {
-  return max_concurrent_queries_ != 0 &&
-         audit_proof_queries_.size() >= max_concurrent_queries_;
+void LogDnsClient::QueryAuditProofCancelled() {
+  --in_flight_queries_;
+
+  // Move not_throttled_callbacks_ to a local variable, just in case one of the
+  // callbacks deletes this LogDnsClient.
+  std::list<base::OnceClosure> not_throttled_callbacks =
+      std::move(not_throttled_callbacks_);
+
+  // Notify interested parties that the next query will not be throttled.
+  for (auto& callback : not_throttled_callbacks) {
+    std::move(callback).Run();
+  }
+}
+
+bool LogDnsClient::HasMaxQueriesInFlight() const {
+  return max_in_flight_queries_ != 0 &&
+         in_flight_queries_ >= max_in_flight_queries_;
 }
 
 void LogDnsClient::UpdateDnsConfig() {
@@ -532,6 +579,11 @@ void LogDnsClient::UpdateDnsConfig() {
   net::NetworkChangeNotifier::GetDnsConfig(&config);
   if (config.IsValid())
     dns_client_->SetConfig(config);
+
+  net::DnsTransactionFactory* factory = dns_client_->GetTransactionFactory();
+  if (factory) {
+    factory->AddEDNSOption(OptToDisableClientSubnetExtension());
+  }
 }
 
 }  // namespace certificate_transparency
