@@ -5,10 +5,11 @@
 #include "components/content_settings/core/browser/content_settings_pref.h"
 
 #include <memory>
+#include <utility>
+#include <vector>
 
 #include "base/auto_reset.h"
 #include "base/bind.h"
-#include "base/memory/ptr_util.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
@@ -54,7 +55,7 @@ bool IsValueAllowedForType(const base::Value* value, ContentSettingsType type) {
 
   // TODO(raymes): We should permit different types of base::Value for
   // website settings.
-  return value->GetType() == base::Value::Type::DICTIONARY;
+  return value->type() == base::Value::Type::DICTIONARY;
 }
 
 // Extract a timestamp from |dictionary[kLastModifiedPath]|.
@@ -131,10 +132,10 @@ bool ContentSettingsPref::SetWebsiteSetting(
 
   {
     base::AutoLock auto_lock(lock_);
-    if (value.get()) {
+    if (value) {
       map_to_modify->SetValue(primary_pattern, secondary_pattern, content_type_,
                               resource_identifier, modified_time,
-                              value->DeepCopy());
+                              value->Clone());
     } else {
       map_to_modify->DeleteValue(
           primary_pattern,
@@ -222,40 +223,28 @@ void ContentSettingsPref::ReadContentSettingsFromPref() {
   prefs::ScopedDictionaryPrefUpdate update(prefs_, pref_name_);
   base::AutoLock auto_lock(lock_);
 
-  const base::DictionaryValue* all_settings_dictionary =
-      prefs_->GetDictionary(pref_name_);
-
   value_map_.clear();
 
-  // Careful: The returned value could be nullptr if the pref has never been
-  // set.
+  // The returned value could be nullptr if the pref has never been set.
+  const base::DictionaryValue* all_settings_dictionary =
+      prefs_->GetDictionary(pref_name_);
   if (!all_settings_dictionary)
     return;
 
-  const base::DictionaryValue* settings;
+  // Accumulates non-canonical pattern strings found in Prefs for which the
+  // corresponding canonical pattern is also in Prefs. In these cases the
+  // canonical version takes priority, and the non-canonical pattern is removed.
+  std::vector<std::string> non_canonical_patterns_to_remove;
 
-  if (!is_incognito_) {
-    // Convert all Unicode patterns into punycode form, then read.
-    auto mutable_settings = update.Get();
-    CanonicalizeContentSettingsExceptions(mutable_settings.get());
-    settings = mutable_settings->AsConstDictionary();
-  } else {
-    // Canonicalization is unnecessary when |is_incognito_|. Both incognito and
-    // non-incognito read from the same pref and non-incognito reads occur
-    // before incognito reads. Thus, by the time the incognito call to
-    // ReadContentSettingsFromPref() occurs, the non-incognito call will have
-    // canonicalized the stored pref data.
-    settings = all_settings_dictionary;
-  }
+  // Accumulates non-canonical pattern strings found in Prefs for which the
+  // canonical pattern is not found in Prefs. The exception data for these
+  // patterns is to be re-keyed under the canonical pattern.
+  base::StringPairs non_canonical_patterns_to_canonical_pattern;
 
-  size_t cookies_block_exception_count = 0;
-  size_t cookies_allow_exception_count = 0;
-  size_t cookies_session_only_exception_count = 0;
-  for (base::DictionaryValue::Iterator i(*settings); !i.IsAtEnd();
-       i.Advance()) {
+  for (base::DictionaryValue::Iterator i(*all_settings_dictionary);
+       !i.IsAtEnd(); i.Advance()) {
     const std::string& pattern_str(i.key());
-    std::pair<ContentSettingsPattern, ContentSettingsPattern> pattern_pair =
-        ParsePatternString(pattern_str);
+    PatternPair pattern_pair = ParsePatternString(pattern_str);
     if (!pattern_pair.first.IsValid() ||
         !pattern_pair.second.IsValid()) {
       // TODO: Change this to DFATAL when crbug.com/132659 is fixed.
@@ -263,6 +252,23 @@ void ContentSettingsPref::ReadContentSettingsFromPref() {
       continue;
     }
 
+    const std::string canonicalized_pattern_str =
+        CreatePatternString(pattern_pair.first, pattern_pair.second);
+    DCHECK(!canonicalized_pattern_str.empty());
+    if (canonicalized_pattern_str != pattern_str) {
+      if (all_settings_dictionary->HasKey(canonicalized_pattern_str)) {
+        non_canonical_patterns_to_remove.push_back(pattern_str);
+        continue;
+      } else {
+        // Need a container that preserves ordering of insertions, so that if
+        // multiple non-canonical patterns map to the same canonical pattern,
+        // the Preferences updating logic after this loop will preserve the same
+        // value in Prefs that this loop ultimately leaves in |value_map_|.
+        non_canonical_patterns_to_canonical_pattern.push_back(
+            {pattern_str, canonicalized_pattern_str});
+      }
+    }
+
     // Get settings dictionary for the current pattern string, and read
     // settings from the dictionary.
     const base::DictionaryValue* settings_dictionary = nullptr;
@@ -287,7 +293,7 @@ void ContentSettingsPref::ReadContentSettingsFromPref() {
           // Per resource settings store a single timestamps for all resources.
           value_map_.SetValue(pattern_pair.first, pattern_pair.second,
                               content_type_, resource_identifier, last_modified,
-                              setting_ptr->DeepCopy());
+                              setting_ptr->Clone());
         }
       }
     }
@@ -297,37 +303,32 @@ void ContentSettingsPref::ReadContentSettingsFromPref() {
     if (value) {
       base::Time last_modified = GetTimeStamp(settings_dictionary);
       DCHECK(IsValueAllowedForType(value, content_type_));
-      value_map_.SetValue(pattern_pair.first, pattern_pair.second,
-                          content_type_, ResourceIdentifier(), last_modified,
-                          value->DeepCopy());
-      if (content_type_ == CONTENT_SETTINGS_TYPE_COOKIES) {
-        ContentSetting s = ValueToContentSetting(value);
-        switch (s) {
-          case CONTENT_SETTING_ALLOW :
-            ++cookies_allow_exception_count;
-            break;
-          case CONTENT_SETTING_BLOCK :
-            ++cookies_block_exception_count;
-            break;
-          case CONTENT_SETTING_SESSION_ONLY :
-            ++cookies_session_only_exception_count;
-            break;
-          default:
-            NOTREACHED();
-            break;
-        }
-      }
+      value_map_.SetValue(std::move(pattern_pair.first),
+                          std::move(pattern_pair.second), content_type_,
+                          ResourceIdentifier(), last_modified, value->Clone());
     }
-
   }
 
-  if (content_type_ == CONTENT_SETTINGS_TYPE_COOKIES) {
-    UMA_HISTOGRAM_COUNTS("ContentSettings.NumberOfBlockCookiesExceptions",
-                         cookies_block_exception_count);
-    UMA_HISTOGRAM_COUNTS("ContentSettings.NumberOfAllowCookiesExceptions",
-                         cookies_allow_exception_count);
-    UMA_HISTOGRAM_COUNTS("ContentSettings.NumberOfSessionOnlyCookiesExceptions",
-                         cookies_session_only_exception_count);
+  // Canonicalization is unnecessary when |is_incognito_|. Both incognito and
+  // non-incognito read from the same pref and non-incognito reads occur
+  // before incognito reads. Thus, by the time the incognito call to
+  // ReadContentSettingsFromPref() occurs, the regular profile will have
+  // canonicalized the stored pref data.
+  if (!is_incognito_) {
+    auto mutable_settings = update.Get();
+
+    for (const auto& pattern : non_canonical_patterns_to_remove) {
+      mutable_settings.get()->RemoveWithoutPathExpansion(pattern, nullptr);
+    }
+
+    for (const auto& old_to_new_pattern :
+         non_canonical_patterns_to_canonical_pattern) {
+      std::unique_ptr<base::Value> pattern_settings_dictionary;
+      mutable_settings.get()->RemoveWithoutPathExpansion(
+          old_to_new_pattern.first, &pattern_settings_dictionary);
+      mutable_settings.get()->SetWithoutPathExpansion(
+          old_to_new_pattern.second, std::move(pattern_settings_dictionary));
+    }
   }
 }
 
@@ -371,7 +372,7 @@ void ContentSettingsPref::UpdatePref(
     if (!found && value) {
       settings_dictionary =
           pattern_pairs_settings->SetDictionaryWithoutPathExpansion(
-              pattern_str, base::MakeUnique<base::DictionaryValue>());
+              pattern_str, std::make_unique<base::DictionaryValue>());
     }
 
     if (settings_dictionary) {
@@ -386,7 +387,7 @@ void ContentSettingsPref::UpdatePref(
           resource_dictionary =
               settings_dictionary->SetDictionaryWithoutPathExpansion(
                   kPerResourceIdentifierPrefName,
-                  base::MakeUnique<base::DictionaryValue>());
+                  std::make_unique<base::DictionaryValue>());
         }
         // Update resource dictionary.
         if (value == nullptr) {
@@ -402,9 +403,9 @@ void ContentSettingsPref::UpdatePref(
           resource_dictionary->SetWithoutPathExpansion(resource_identifier,
                                                        value->CreateDeepCopy());
           // Update timestamp for whole resource dictionary.
-          settings_dictionary->SetStringWithoutPathExpansion(
-              kLastModifiedPath,
-              base::Int64ToString(last_modified.ToInternalValue()));
+          settings_dictionary->SetKey(kLastModifiedPath,
+                                      base::Value(base::Int64ToString(
+                                          last_modified.ToInternalValue())));
         }
       } else {
         // Update settings dictionary.
@@ -416,9 +417,9 @@ void ContentSettingsPref::UpdatePref(
         } else {
           settings_dictionary->SetWithoutPathExpansion(kSettingPath,
                                                        value->CreateDeepCopy());
-          settings_dictionary->SetStringWithoutPathExpansion(
-              kLastModifiedPath,
-              base::Int64ToString(last_modified.ToInternalValue()));
+          settings_dictionary->SetKey(kLastModifiedPath,
+                                      base::Value(base::Int64ToString(
+                                          last_modified.ToInternalValue())));
         }
       }
       // Remove the settings dictionary if it is empty.
@@ -430,63 +431,6 @@ void ContentSettingsPref::UpdatePref(
   }
 }
 
-// static
-void ContentSettingsPref::CanonicalizeContentSettingsExceptions(
-    prefs::DictionaryValueUpdate* all_settings_dictionary) {
-  DCHECK(all_settings_dictionary);
-
-  std::vector<std::string> remove_items;
-  base::StringPairs move_items;
-  for (base::DictionaryValue::Iterator i(
-           *all_settings_dictionary->AsConstDictionary());
-       !i.IsAtEnd(); i.Advance()) {
-    const std::string& pattern_str(i.key());
-    std::pair<ContentSettingsPattern, ContentSettingsPattern> pattern_pair =
-         ParsePatternString(pattern_str);
-    if (!pattern_pair.first.IsValid() ||
-        !pattern_pair.second.IsValid()) {
-      LOG(ERROR) << "Invalid pattern strings: " << pattern_str;
-      continue;
-    }
-
-    const std::string canonicalized_pattern_str = CreatePatternString(
-        pattern_pair.first, pattern_pair.second);
-
-    if (canonicalized_pattern_str.empty() ||
-        canonicalized_pattern_str == pattern_str) {
-      continue;
-    }
-
-    // Clear old pattern if prefs already have canonicalized pattern.
-    const base::DictionaryValue* new_pattern_settings_dictionary = nullptr;
-    if (all_settings_dictionary->GetDictionaryWithoutPathExpansion(
-            canonicalized_pattern_str, &new_pattern_settings_dictionary)) {
-      remove_items.push_back(pattern_str);
-      continue;
-    }
-
-    // Move old pattern to canonicalized pattern.
-    const base::DictionaryValue* old_pattern_settings_dictionary = nullptr;
-    if (i.value().GetAsDictionary(&old_pattern_settings_dictionary)) {
-      move_items.push_back(
-          std::make_pair(pattern_str, canonicalized_pattern_str));
-    }
-  }
-
-  for (size_t i = 0; i < remove_items.size(); ++i) {
-    all_settings_dictionary->RemoveWithoutPathExpansion(remove_items[i],
-                                                        nullptr);
-  }
-
-  for (size_t i = 0; i < move_items.size(); ++i) {
-    std::unique_ptr<base::Value> pattern_settings_dictionary;
-    all_settings_dictionary->RemoveWithoutPathExpansion(
-        move_items[i].first, &pattern_settings_dictionary);
-    all_settings_dictionary->SetWithoutPathExpansion(
-        move_items[i].second, std::move(pattern_settings_dictionary));
-  }
-}
-
 void ContentSettingsPref::AssertLockNotHeld() const {
 #if !defined(NDEBUG)
   // |Lock::Acquire()| will assert if the lock is held by this thread.
