@@ -14,17 +14,18 @@
 
 #include "base/bind.h"
 #include "base/logging.h"
-#include "base/macros.h"
 #include "base/memory/ptr_util.h"
-#include "base/message_loop/message_loop.h"
 #include "base/metrics/histogram.h"
-#include "base/test/histogram_tester.h"
+#include "base/stl_util.h"
+#include "base/test/metrics/histogram_tester.h"
+#include "base/test/scoped_task_environment.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_config_test_utils.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_configurator.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_io_data.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_network_delegate.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_prefs.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_test_utils.h"
+#include "components/data_reduction_proxy/core/common/data_reduction_proxy_bypass_protocol.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_headers_test_utils.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_params_test_utils.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_pref_names.h"
@@ -35,11 +36,11 @@
 #include "net/base/load_flags.h"
 #include "net/base/net_errors.h"
 #include "net/base/proxy_delegate.h"
+#include "net/base/proxy_server.h"
 #include "net/base/request_priority.h"
 #include "net/dns/mock_host_resolver.h"
 #include "net/http/http_response_headers.h"
 #include "net/http/http_util.h"
-#include "net/proxy/proxy_server.h"
 #include "net/socket/socket_test_util.h"
 #include "net/traffic_annotation/network_traffic_annotation_test_helper.h"
 #include "net/url_request/url_request.h"
@@ -51,6 +52,7 @@
 #include "net/url_request/url_request_status.h"
 #include "net/url_request/url_request_test_job.h"
 #include "net/url_request/url_request_test_util.h"
+#include "services/network/test/test_network_connection_tracker.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -82,6 +84,7 @@ class DataReductionProxyBypassStatsTest : public testing::Test {
 
     test_context_ =
         DataReductionProxyTestContext::Builder().WithMockConfig().Build();
+    test_context_->DisableWarmupURLFetch();
     mock_url_request_ = context_.CreateRequest(GURL(), net::IDLE, &delegate_,
                                                TRAFFIC_ANNOTATION_FOR_TESTS);
   }
@@ -103,14 +106,15 @@ class DataReductionProxyBypassStatsTest : public testing::Test {
     fake_request->Start();
     test_context_->RunUntilIdle();
 
-    EXPECT_TRUE(fake_request->response_headers() != NULL);
+    EXPECT_TRUE(fake_request->response_headers() != nullptr);
     return fake_request;
   }
 
  protected:
   std::unique_ptr<DataReductionProxyBypassStats> BuildBypassStats() {
-    return base::MakeUnique<DataReductionProxyBypassStats>(
-        test_context_->config(), test_context_->unreachable_callback());
+    return std::make_unique<DataReductionProxyBypassStats>(
+        test_context_->config(), test_context_->unreachable_callback(),
+        network::TestNetworkConnectionTracker::GetInstance());
   }
 
   MockDataReductionProxyConfig* config() const {
@@ -118,7 +122,8 @@ class DataReductionProxyBypassStatsTest : public testing::Test {
   }
 
  private:
-  base::MessageLoopForIO message_loop_;
+  base::test::ScopedTaskEnvironment task_environment_{
+      base::test::ScopedTaskEnvironment::MainThreadType::IO};
   net::TestURLRequestContext context_;
   net::TestDelegate delegate_;
   std::unique_ptr<net::URLRequest> mock_url_request_;
@@ -129,122 +134,6 @@ class DataReductionProxyBypassStatsTest : public testing::Test {
 };
 
 
-TEST_F(DataReductionProxyBypassStatsTest, RecordMissingViaHeaderBytes) {
-  const std::string k4xxHistogramName =
-      "DataReductionProxy.MissingViaHeader.Bytes.4xx";
-  const std::string kOtherHistogramName =
-      "DataReductionProxy.MissingViaHeader.Bytes.Other";
-  const int64_t kResponseContentLength = 100;
-
-  struct TestCase {
-    bool was_proxy_used;
-    const char* headers;
-    bool is_4xx_sample_expected;
-    bool is_other_sample_expected;
-  };
-  const TestCase test_cases[] = {
-    // Nothing should be recorded for requests that don't use the proxy.
-    {
-      false,
-      "HTTP/1.1 404 Not Found\n",
-      false,
-      false
-    },
-    {
-      false,
-      "HTTP/1.1 200 OK\n",
-      false,
-      false
-    },
-    // Nothing should be recorded for responses that have the via header.
-    {
-      true,
-      "HTTP/1.1 404 Not Found\n"
-      "Via: 1.1 Chrome-Compression-Proxy\n",
-      false,
-      false
-    },
-    {
-      true,
-      "HTTP/1.1 200 OK\n"
-      "Via: 1.1 Chrome-Compression-Proxy\n",
-      false,
-      false
-    },
-    // 4xx responses that used the proxy and don't have the via header should be
-    // recorded.
-    {
-      true,
-      "HTTP/1.1 404 Not Found\n",
-      true,
-      false
-    },
-    {
-      true,
-      "HTTP/1.1 400 Bad Request\n",
-      true,
-      false
-    },
-    {
-      true,
-      "HTTP/1.1 499 Big Client Error Response Code\n",
-      true,
-      false
-    },
-    // Non-4xx responses that used the proxy and don't have the via header
-    // should be recorded.
-    {
-      true,
-      "HTTP/1.1 200 OK\n",
-      false,
-      true
-    },
-    {
-      true,
-      "HTTP/1.1 399 Big Redirection Response Code\n",
-      false,
-      true
-    },
-    {
-      true,
-      "HTTP/1.1 500 Internal Server Error\n",
-      false,
-      true
-    }
-  };
-
-  for (size_t i = 0; i < arraysize(test_cases); ++i) {
-    base::HistogramTester histogram_tester;
-    std::unique_ptr<DataReductionProxyBypassStats> bypass_stats =
-        BuildBypassStats();
-
-    std::unique_ptr<net::URLRequest> fake_request(
-        CreateURLRequestWithResponseHeaders(GURL("http://www.google.com/"),
-                                            test_cases[i].headers));
-    fake_request->set_received_response_content_length(kResponseContentLength);
-
-    EXPECT_CALL(*config(),
-                WasDataReductionProxyUsed(fake_request.get(), testing::_))
-        .WillRepeatedly(Return(test_cases[i].was_proxy_used));
-
-    bypass_stats->RecordMissingViaHeaderBytes(*fake_request);
-
-    if (test_cases[i].is_4xx_sample_expected) {
-      histogram_tester.ExpectUniqueSample(k4xxHistogramName,
-                                          kResponseContentLength, 1);
-    } else {
-      histogram_tester.ExpectTotalCount(k4xxHistogramName, 0);
-    }
-
-    if (test_cases[i].is_other_sample_expected) {
-      histogram_tester.ExpectUniqueSample(kOtherHistogramName,
-                                          kResponseContentLength, 1);
-    } else {
-      histogram_tester.ExpectTotalCount(kOtherHistogramName, 0);
-    }
-  }
-}
-
 // End-to-end tests for the DataReductionProxy.BypassedBytes histograms.
 class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
  public:
@@ -261,10 +150,10 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
                             .WithURLRequestContext(&context_)
                             .WithMockClientSocketFactory(&mock_socket_factory_)
                             .Build();
+    drp_test_context_->DisableWarmupURLFetch();
     drp_test_context_->AttachToURLRequestContext(&context_storage_);
     context_.set_client_socket_factory(&mock_socket_factory_);
     proxy_delegate_ = drp_test_context_->io_data()->CreateProxyDelegate();
-    context_.set_proxy_delegate(proxy_delegate_.get());
 
     // Only use the primary data reduction proxy in order to make it easier to
     // test bypassed bytes due to proxy fallbacks. This way, a test just needs
@@ -309,7 +198,7 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
         MockRead(net::SYNCHRONOUS, finish_code),
     };
     net::StaticSocketDataProvider initial_socket_data_provider(
-        initial_data_reads, arraysize(initial_data_reads), nullptr, 0);
+        initial_data_reads, base::span<net::MockWrite>());
     mock_socket_factory_.AddSocketDataProvider(&initial_socket_data_provider);
 
     // Prepare for the response from retrying the request, if applicable.
@@ -323,7 +212,7 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
       retry_data_reads.push_back(MockRead(net::SYNCHRONOUS, finish_code));
 
       retry_socket_data_provider.reset(new net::StaticSocketDataProvider(
-          &retry_data_reads.front(), retry_data_reads.size(), nullptr, 0));
+          retry_data_reads, base::span<net::MockWrite>()));
       mock_socket_factory_.AddSocketDataProvider(
           retry_socket_data_provider.get());
     }
@@ -347,7 +236,7 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
         MockRead(""), MockRead(net::SYNCHRONOUS, net::OK),
     };
     net::StaticSocketDataProvider redirect_socket_data_provider_1(
-        redirect_mock_reads_1, arraysize(redirect_mock_reads_1), nullptr, 0);
+        redirect_mock_reads_1, base::span<net::MockWrite>());
     mock_socket_factory_.AddSocketDataProvider(
         &redirect_socket_data_provider_1);
 
@@ -359,7 +248,7 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
         MockRead(""), MockRead(net::SYNCHRONOUS, net::OK),
     };
     net::StaticSocketDataProvider redirect_socket_data_provider_2(
-        redirect_mock_reads_2, arraysize(redirect_mock_reads_2), nullptr, 0);
+        redirect_mock_reads_2, base::span<net::MockWrite>());
     mock_socket_factory_.AddSocketDataProvider(
         &redirect_socket_data_provider_2);
 
@@ -372,7 +261,7 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
         MockRead(""), MockRead(net::SYNCHRONOUS, net::OK),
     };
     net::StaticSocketDataProvider redirect_socket_data_provider_3(
-        redirect_mock_reads_3, arraysize(redirect_mock_reads_3), nullptr, 0);
+        redirect_mock_reads_3, base::span<net::MockWrite>());
     mock_socket_factory_.AddSocketDataProvider(
         &redirect_socket_data_provider_3);
 
@@ -383,7 +272,7 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
         MockRead(net::SYNCHRONOUS, net::OK),
     };
     net::StaticSocketDataProvider response_socket_data_provider(
-        response_mock_reads, arraysize(response_mock_reads), nullptr, 0);
+        response_mock_reads, base::span<net::MockWrite>());
     mock_socket_factory_.AddSocketDataProvider(&response_socket_data_provider);
 
     std::unique_ptr<net::URLRequest> request(
@@ -395,8 +284,8 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
     return request;
   }
 
-  void set_proxy_service(net::ProxyService* proxy_service) {
-    context_.set_proxy_service(proxy_service);
+  void set_proxy_resolution_service(net::ProxyResolutionService* proxy_resolution_service) {
+    context_.set_proxy_resolution_service(proxy_resolution_service);
   }
 
   void set_host_resolver(net::HostResolver* host_resolver) {
@@ -416,11 +305,14 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
   }
 
   void ClearBadProxies() {
-    context_.proxy_service()->ClearBadProxiesCache();
+    context_.proxy_resolution_service()->ClearBadProxiesCache();
   }
 
   void InitializeContext() {
     context_.Init();
+    context_.proxy_resolution_service()->SetProxyDelegate(
+        proxy_delegate_.get());
+    drp_test_context_->DisableWarmupURLFetch();
     drp_test_context_->EnableDataReductionProxyWithSecureProxyCheckSuccess();
   }
 
@@ -490,46 +382,15 @@ class DataReductionProxyBypassStatsEndToEndTest : public testing::Test {
 
   net::TestDelegate* delegate() { return &delegate_; }
 
-  // Marks a data reduction proxy as bypassed if
-  // |bypassed_proxy_server_is_data_reduction_proxy| is true. Then, runs a
-  // request via data reduction proxy if |is_data_reduction_proxy| is true,
-  // and verifies that proxy is unreachable only if |is_unreachable| is true.
-  void VerifyProxyReachablity(
-      bool bypassed_proxy_server_is_data_reduction_proxy,
-      bool is_data_reduction_proxy,
-      bool is_unreachable) {
-    InitializeContext();
-
-    std::string proxy = bypassed_proxy_server_is_data_reduction_proxy
-                            ? "origin.net:80"
-                            : "foo.net:80";
-    net::ProxyServer fallback_proxy_server =
-        net::ProxyServer::FromURI(proxy, net::ProxyServer::SCHEME_HTTP);
-
-    bypass_stats()->OnProxyFallback(fallback_proxy_server,
-                                    net::ERR_PROXY_CONNECTION_FAILED);
-    drp_test_context()->RunUntilIdle();
-
-    if (!is_data_reduction_proxy)
-      config()->SetWasDataReductionProxyNotUsed();
-
-    CreateAndExecuteRequest(GURL("http://bar.com"), net::LOAD_NORMAL, net::OK,
-                            "HTTP/1.1 200 OK\r\n"
-                            "Via: 1.1 Chrome-Compression-Proxy\r\n\r\n",
-                            kNextBody.c_str(), nullptr, nullptr);
-
-    drp_test_context()->RunUntilIdle();
-    EXPECT_EQ(is_unreachable, IsUnreachable());
-  }
-
  private:
-  base::MessageLoopForIO message_loop_;
+  base::test::ScopedTaskEnvironment task_environment_{
+      base::test::ScopedTaskEnvironment::MainThreadType::IO};
   net::TestDelegate delegate_;
   net::MockClientSocketFactory mock_socket_factory_;
   net::TestURLRequestContext context_;
   net::URLRequestContextStorage context_storage_;
-  std::unique_ptr<net::ProxyDelegate> proxy_delegate_;
   std::unique_ptr<DataReductionProxyTestContext> drp_test_context_;
+  std::unique_ptr<net::ProxyDelegate> proxy_delegate_;
 };
 
 TEST_F(DataReductionProxyBypassStatsEndToEndTest, BypassedBytesNoRetry) {
@@ -596,9 +457,10 @@ TEST_F(DataReductionProxyBypassStatsEndToEndTest, URLRedirectCycle) {
 
 TEST_F(DataReductionProxyBypassStatsEndToEndTest,
        BypassedBytesProxyOverridden) {
-  std::unique_ptr<net::ProxyService> proxy_service(
-      net::ProxyService::CreateFixed("http://test.com:80"));
-  set_proxy_service(proxy_service.get());
+  std::unique_ptr<net::ProxyResolutionService> proxy_resolution_service(
+      net::ProxyResolutionService::CreateFixed("http://test.com:80",
+                                               TRAFFIC_ANNOTATION_FOR_TESTS));
+  set_proxy_resolution_service(proxy_resolution_service.get());
   InitializeContext();
 
   base::HistogramTester histogram_tester;
@@ -743,9 +605,6 @@ TEST_F(DataReductionProxyBypassStatsEndToEndTest,
     const char* initial_response_headers;
   };
   const TestCase test_cases[] = {
-    { "DataReductionProxy.BypassedBytes.MissingViaHeaderOther",
-      "HTTP/1.1 200 OK\r\n\r\n",
-    },
     { "DataReductionProxy.BypassedBytes.Malformed407",
       "HTTP/1.1 407 Proxy Authentication Required\r\n\r\n",
     },
@@ -821,64 +680,6 @@ TEST_F(DataReductionProxyBypassStatsEndToEndTest, BypassedBytesNetErrorOther) {
       -net::ERR_PROXY_CONNECTION_FAILED, 1);
 }
 
-TEST_F(DataReductionProxyBypassStatsEndToEndTest,
-       BypassedBytesMissingViaHeader4xx) {
-  InitializeContext();
-  const char* test_case_response_headers[] = {
-      "HTTP/1.1 414 Request-URI Too Long\r\n\r\n",
-      "HTTP/1.1 404 Not Found\r\n\r\n",
-  };
-  for (const char* test_case : test_case_response_headers) {
-    ClearBadProxies();
-    // The first request should be bypassed for missing Via header.
-    {
-      base::HistogramTester histogram_tester;
-      CreateAndExecuteRequest(GURL("http://foo.com"), net::LOAD_NORMAL, net::OK,
-                              test_case, kErrorBody.c_str(),
-                              "HTTP/1.1 200 OK\r\n\r\n", kBody.c_str());
-
-      histogram_tester.ExpectUniqueSample(
-          "DataReductionProxy.BypassedBytes.MissingViaHeader4xx", kBody.size(),
-          1);
-      ExpectOtherBypassedBytesHistogramsEmpty(
-          histogram_tester,
-          "DataReductionProxy.BypassedBytes.MissingViaHeader4xx");
-    }
-    // The second request should be sent via the proxy.
-    {
-      base::HistogramTester histogram_tester;
-      CreateAndExecuteRequest(GURL("http://bar.com"), net::LOAD_NORMAL, net::OK,
-                              "HTTP/1.1 200 OK\r\n"
-                              "Via: 1.1 Chrome-Compression-Proxy\r\n\r\n",
-                              kNextBody.c_str(), nullptr, nullptr);
-      histogram_tester.ExpectUniqueSample(
-          "DataReductionProxy.BypassedBytes.NotBypassed", kNextBody.size(), 1);
-      ExpectOtherBypassedBytesHistogramsEmpty(
-          histogram_tester, "DataReductionProxy.BypassedBytes.NotBypassed");
-    }
-  }
-}
-
-TEST_F(DataReductionProxyBypassStatsEndToEndTest,
-       IsDataReductionProxyUnreachable_Unreachable) {
-  VerifyProxyReachablity(true, false, true);
-}
-
-TEST_F(DataReductionProxyBypassStatsEndToEndTest,
-       IsDataReductionProxyUnreachable_Unreachable_Then_Reachable) {
-  VerifyProxyReachablity(true, true, false);
-}
-
-TEST_F(DataReductionProxyBypassStatsEndToEndTest,
-       IsDataReductionProxyUnreachable_Not_A_data_reduction_proxy_1) {
-  VerifyProxyReachablity(false, true, false);
-}
-
-TEST_F(DataReductionProxyBypassStatsEndToEndTest,
-       IsDataReductionProxyUnreachableNot_A_data_reduction_proxy_2) {
-  VerifyProxyReachablity(false, false, false);
-}
-
 TEST_F(DataReductionProxyBypassStatsEndToEndTest,
        ProxyUnreachableThenReachable) {
   net::ProxyServer fallback_proxy_server =
@@ -955,15 +756,15 @@ TEST_F(DataReductionProxyBypassStatsEndToEndTest,
       {true, "HTTP/1.1 404 Not Found\n", 404, -1},
       {false, "HTTP/1.1 404 Not Found\n", -1, 404}};
 
-  for (size_t i = 0; i < arraysize(test_cases); ++i) {
+  for (size_t i = 0; i < base::size(test_cases); ++i) {
     base::HistogramTester histogram_tester;
     std::string raw_headers(test_cases[i].headers);
     HeadersToRaw(&raw_headers);
     scoped_refptr<net::HttpResponseHeaders> headers(
         new net::HttpResponseHeaders(raw_headers));
 
-    DataReductionProxyBypassStats::DetectAndRecordMissingViaHeaderResponseCode(
-        test_cases[i].is_primary, *headers);
+    DetectAndRecordMissingViaHeaderResponseCode(test_cases[i].is_primary,
+                                                *headers);
 
     if (test_cases[i].expected_primary_sample == -1) {
       histogram_tester.ExpectTotalCount(kPrimaryHistogramName, 0);
@@ -981,72 +782,124 @@ TEST_F(DataReductionProxyBypassStatsEndToEndTest,
   }
 }
 
-TEST_F(DataReductionProxyBypassStatsEndToEndTest, SuccessfulRequestCompletion) {
-  const std::string kPrimaryHistogramName =
+TEST_F(DataReductionProxyBypassStatsEndToEndTest,
+       SuccessfulPrimaryProxyRequestCompletion) {
+  const std::string kHistogramName =
       "DataReductionProxy.SuccessfulRequestCompletionCounts";
-  const std::string kPrimaryMainFrameHistogramName =
+  const std::string kMainFrameHistogramName =
       "DataReductionProxy.SuccessfulRequestCompletionCounts.MainFrame";
 
   InitializeContext();
+
   const struct {
-    bool was_proxy_used;
-    bool is_load_bypass_proxy;
-    size_t proxy_index;
-    bool is_main_frame;
+    int load_flags;
     net::Error net_error;
-  } tests[] = {{false, true, 0, true, net::OK},
-               {false, true, 0, false, net::ERR_TOO_MANY_REDIRECTS},
-               {false, false, 0, true, net::OK},
-               {false, false, 0, false, net::ERR_TOO_MANY_REDIRECTS},
-               {true, false, 0, true, net::OK},
-               {true, false, 0, true, net::ERR_TOO_MANY_REDIRECTS},
-               {true, false, 0, false, net::OK},
-               {true, false, 0, false, net::ERR_TOO_MANY_REDIRECTS},
-               {true, false, 1, true, net::OK},
-               {true, false, 1, true, net::ERR_TOO_MANY_REDIRECTS},
-               {true, false, 1, false, net::OK},
-               {true, false, 1, false, net::ERR_TOO_MANY_REDIRECTS}};
+    bool expect_histogram_sample;
+    bool expect_main_frame_histogram_sample;
+  } tests[] = {
+      {net::LOAD_BYPASS_PROXY | net::LOAD_MAIN_FRAME_DEPRECATED, net::OK, false,
+       false},
+      {net::LOAD_BYPASS_PROXY | net::LOAD_MAIN_FRAME_DEPRECATED,
+       net::ERR_TOO_MANY_REDIRECTS, false, false},
+      {net::LOAD_BYPASS_PROXY, net::OK, false, false},
+      {net::LOAD_BYPASS_PROXY, net::ERR_TOO_MANY_REDIRECTS, false, false},
+      {net::LOAD_MAIN_FRAME_DEPRECATED, net::OK, true, true},
+      {net::LOAD_MAIN_FRAME_DEPRECATED, net::ERR_TOO_MANY_REDIRECTS, false,
+       false},
+      {0, net::OK, true, false},
+      {0, net::ERR_TOO_MANY_REDIRECTS, false, false},
+  };
 
   for (const auto& test : tests) {
-    config()->ResetWasDataReductionProxyUsed();
     base::HistogramTester histogram_tester;
-
-    // Proxy succeeds.
-    int load_flags = net::LOAD_NORMAL;
-    if (test.is_load_bypass_proxy) {
-      load_flags |= net::LOAD_BYPASS_PROXY;
-    }
-    if (test.is_main_frame) {
-      load_flags |= net::LOAD_MAIN_FRAME_DEPRECATED;
-    }
-
-    if (!test.was_proxy_used)
-      config()->SetWasDataReductionProxyNotUsed();
-    else {
-      config()->SetWasDataReductionProxyUsedProxyIndex(test.proxy_index);
-    }
-
-    CreateAndExecuteRequest(GURL("http://bar.com"), load_flags, test.net_error,
+    CreateAndExecuteRequest(GURL("http://foo.com"), test.load_flags,
+                            test.net_error,
                             "HTTP/1.1 200 OK\r\n"
                             "Via: 1.1 Chrome-Compression-Proxy\r\n\r\n",
                             kNextBody.c_str(), nullptr, nullptr);
     drp_test_context()->RunUntilIdle();
 
-    if (test.was_proxy_used && !test.is_load_bypass_proxy &&
-        test.net_error == net::OK) {
-      histogram_tester.ExpectUniqueSample(kPrimaryHistogramName,
-                                          test.proxy_index, 1);
-    } else {
-      histogram_tester.ExpectTotalCount(kPrimaryHistogramName, 0);
-    }
+    if (test.expect_histogram_sample)
+      histogram_tester.ExpectUniqueSample(kHistogramName, 0, 1);
+    else
+      histogram_tester.ExpectTotalCount(kHistogramName, 0);
 
-    if (test.was_proxy_used && !test.is_load_bypass_proxy &&
-        test.is_main_frame && test.net_error == net::OK) {
-      histogram_tester.ExpectUniqueSample(kPrimaryMainFrameHistogramName,
-                                          test.proxy_index, 1);
-    } else {
-      histogram_tester.ExpectTotalCount(kPrimaryMainFrameHistogramName, 0);
-    }
+    if (test.expect_main_frame_histogram_sample)
+      histogram_tester.ExpectUniqueSample(kMainFrameHistogramName, 0, 1);
+    else
+      histogram_tester.ExpectTotalCount(kMainFrameHistogramName, 0);
+  }
+}
+
+TEST_F(DataReductionProxyBypassStatsEndToEndTest,
+       SuccessfulFallbackProxyRequestCompletion) {
+  const std::string kHistogramName =
+      "DataReductionProxy.SuccessfulRequestCompletionCounts";
+  const std::string kMainFrameHistogramName =
+      "DataReductionProxy.SuccessfulRequestCompletionCounts.MainFrame";
+
+  // Explicitly set primary and fallback Data Reduction Proxies to use.
+  config()->test_params()->SetProxiesForHttp(
+      std::vector<DataReductionProxyServer>(
+          {DataReductionProxyServer(
+               net::ProxyServer::FromURI("http://origin.net",
+                                         net::ProxyServer::SCHEME_HTTP),
+               ProxyServer::CORE),
+           DataReductionProxyServer(
+               net::ProxyServer::FromURI("http://fallback.net",
+                                         net::ProxyServer::SCHEME_HTTP),
+               ProxyServer::CORE)}));
+
+  // Make the first Data Reduction Proxy host in the list of Data Reduction
+  // Proxies to use fail to resolve, so that the tests below will use the
+  // fallback proxy.
+  std::unique_ptr<net::MockHostResolver> host_resolver(
+      new net::MockHostResolver());
+  const DataReductionProxyServer& primary_proxy =
+      config()->test_params()->proxies_for_http().front();
+  host_resolver->rules()->AddSimulatedFailure(
+      primary_proxy.proxy_server().host_port_pair().host());
+
+  set_host_resolver(host_resolver.get());
+  InitializeContext();
+
+  const struct {
+    int load_flags;
+    net::Error net_error;
+    bool expect_histogram_sample;
+    bool expect_main_frame_histogram_sample;
+  } tests[] = {
+      {net::LOAD_BYPASS_PROXY | net::LOAD_MAIN_FRAME_DEPRECATED, net::OK, false,
+       false},
+      {net::LOAD_BYPASS_PROXY | net::LOAD_MAIN_FRAME_DEPRECATED,
+       net::ERR_TOO_MANY_REDIRECTS, false, false},
+      {net::LOAD_BYPASS_PROXY, net::OK, false, false},
+      {net::LOAD_BYPASS_PROXY, net::ERR_TOO_MANY_REDIRECTS, false, false},
+      {net::LOAD_MAIN_FRAME_DEPRECATED, net::OK, true, true},
+      {net::LOAD_MAIN_FRAME_DEPRECATED, net::ERR_TOO_MANY_REDIRECTS, false,
+       false},
+      {0, net::OK, true, false},
+      {0, net::ERR_TOO_MANY_REDIRECTS, false, false},
+  };
+
+  for (const auto& test : tests) {
+    base::HistogramTester histogram_tester;
+    CreateAndExecuteRequest(GURL("http://foo.com"), test.load_flags,
+                            test.net_error,
+                            "HTTP/1.1 200 OK\r\n"
+                            "Via: 1.1 Chrome-Compression-Proxy\r\n\r\n",
+                            kNextBody.c_str(), nullptr, nullptr);
+    drp_test_context()->RunUntilIdle();
+
+    if (test.expect_histogram_sample)
+      histogram_tester.ExpectUniqueSample(kHistogramName, 1, 1);
+    else
+      histogram_tester.ExpectTotalCount(kHistogramName, 0);
+
+    if (test.expect_main_frame_histogram_sample)
+      histogram_tester.ExpectUniqueSample(kMainFrameHistogramName, 1, 1);
+    else
+      histogram_tester.ExpectTotalCount(kMainFrameHistogramName, 0);
   }
 }
 
