@@ -14,7 +14,7 @@
 #include "base/memory/ref_counted.h"
 #include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/test/histogram_tester.h"
+#include "base/test/metrics/histogram_tester.h"
 #include "base/test/scoped_task_environment.h"
 #include "base/time/time.h"
 #include "base/values.h"
@@ -142,10 +142,6 @@ class DataReductionProxyCompressionStatsTest : public testing::Test {
     now_delta_ += base::TimeDelta::FromHours(hours);
   }
 
-  DataReductionProxyCompressionStats::SiteUsageMap* DataUsageMap() {
-    return &compression_stats_->data_usage_map_;
-  }
-
   void SetUpPrefs() {
     CreatePrefList(prefs::kDailyHttpOriginalContentLength);
     CreatePrefList(prefs::kDailyHttpReceivedContentLength);
@@ -165,12 +161,12 @@ class DataReductionProxyCompressionStatsTest : public testing::Test {
 
     for (size_t i = 0; i < kNumDaysInHistory; ++i) {
       original_daily_content_length_list->Set(
-          i, base::MakeUnique<base::Value>(base::SizeTToString(i)));
+          i, std::make_unique<base::Value>(base::NumberToString(i)));
     }
 
     received_daily_content_length_list->Clear();
     for (size_t i = 0; i < kNumDaysInHistory / 2; ++i) {
-      received_daily_content_length_list->AppendString(base::SizeTToString(i));
+      received_daily_content_length_list->AppendString(base::NumberToString(i));
     }
   }
 
@@ -179,7 +175,7 @@ class DataReductionProxyCompressionStatsTest : public testing::Test {
     base::ListValue* update = compression_stats_->GetList(pref);
     update->Clear();
     for (size_t i = 0; i < kNumDaysInHistory; ++i) {
-      update->Insert(0, base::MakeUnique<base::Value>(base::Int64ToString(0)));
+      update->Insert(0, std::make_unique<base::Value>(base::Int64ToString(0)));
     }
   }
 
@@ -437,7 +433,9 @@ class DataReductionProxyCompressionStatsTest : public testing::Test {
   }
 
   void ClearDataSavingStatistics() {
-    compression_stats_->ClearDataSavingStatistics();
+    compression_stats_->ClearDataSavingStatistics(
+        DataReductionProxySavingsClearedReason::
+            USER_ACTION_DELETE_BROWSING_HISTORY);
   }
 
   void DeleteBrowsingHistory(const base::Time& start, const base::Time& end) {
@@ -474,6 +472,32 @@ class DataReductionProxyCompressionStatsTest : public testing::Test {
     return drp_test_context_->IsDataReductionProxyEnabled();
   }
 
+  void InitializeWeeklyAggregateDataUse(const base::Time& now) {
+    compression_stats_->InitializeWeeklyAggregateDataUse(now);
+  }
+
+  void RecordWeeklyAggregateDataUse(
+      const base::Time& now,
+      int32_t received_kb,
+      bool is_user_request,
+      data_use_measurement::DataUseUserData::DataUseContentType content_type,
+      int32_t service_hash_code) {
+    compression_stats_->RecordWeeklyAggregateDataUse(
+        now, received_kb, is_user_request, content_type, service_hash_code);
+  }
+
+  void VerifyDictionaryPref(const std::string& pref,
+                            int key,
+                            int expected_value) const {
+    const base::DictionaryValue* dict =
+        compression_stats_->pref_service_->GetDictionary(pref);
+    EXPECT_EQ(expected_value != 0, dict->HasKey(base::IntToString(key)));
+    if (expected_value) {
+      EXPECT_EQ(expected_value,
+                dict->FindKey(base::IntToString(key))->GetInt());
+    }
+  }
+
  private:
   base::test::ScopedTaskEnvironment scoped_task_environment_;
   std::unique_ptr<DataReductionProxyTestContext> drp_test_context_;
@@ -509,50 +533,10 @@ TEST_F(DataReductionProxyCompressionStatsTest, WritePrefsDelayed) {
   VerifyPrefListWasWritten(prefs::kDailyHttpReceivedContentLength);
 }
 
-TEST_F(DataReductionProxyCompressionStatsTest,
-       HistoricNetworkStatsInfoToValue) {
-  const int64_t kOriginalLength = 150;
-  const int64_t kReceivedLength = 100;
-  ResetCompressionStatsWithDelay(
-      base::TimeDelta::FromMinutes(kWriteDelayMinutes));
-
-  base::DictionaryValue* dict = nullptr;
-  std::unique_ptr<base::Value> stats_value(
-      compression_stats()->HistoricNetworkStatsInfoToValue());
-  EXPECT_TRUE(stats_value->GetAsDictionary(&dict));
-  VerifyPrefs(dict);
-
-  SetInt64(prefs::kHttpOriginalContentLength, kOriginalLength);
-  SetInt64(prefs::kHttpReceivedContentLength, kReceivedLength);
-
-  stats_value = compression_stats()->HistoricNetworkStatsInfoToValue();
-  EXPECT_TRUE(stats_value->GetAsDictionary(&dict));
-  VerifyPrefs(dict);
-}
-
-TEST_F(DataReductionProxyCompressionStatsTest,
-       HistoricNetworkStatsInfoToValueDirect) {
-  const int64_t kOriginalLength = 150;
-  const int64_t kReceivedLength = 100;
-
-  base::DictionaryValue* dict = nullptr;
-  std::unique_ptr<base::Value> stats_value(
-      compression_stats()->HistoricNetworkStatsInfoToValue());
-  EXPECT_TRUE(stats_value->GetAsDictionary(&dict));
-  VerifyPrefs(dict);
-
-  SetInt64(prefs::kHttpOriginalContentLength, kOriginalLength);
-  SetInt64(prefs::kHttpReceivedContentLength, kReceivedLength);
-
-  stats_value = compression_stats()->HistoricNetworkStatsInfoToValue();
-  EXPECT_TRUE(stats_value->GetAsDictionary(&dict));
-  VerifyPrefs(dict);
-}
-
 TEST_F(DataReductionProxyCompressionStatsTest, StatsRestoredOnOnRestart) {
   base::ListValue list_value;
   list_value.Insert(0,
-                    base::MakeUnique<base::Value>(base::Int64ToString(1234)));
+                    std::make_unique<base::Value>(base::Int64ToString(1234)));
   pref_service()->Set(prefs::kDailyHttpOriginalContentLength, list_value);
 
   ResetCompressionStatsWithDelay(
@@ -571,7 +555,8 @@ TEST_F(DataReductionProxyCompressionStatsTest, TotalLengths) {
 
   compression_stats()->RecordDataUseWithMimeType(
       kReceivedLength, kOriginalLength, IsDataReductionProxyEnabled(),
-      UNKNOWN_TYPE, std::string());
+      UNKNOWN_TYPE, std::string(), true,
+      data_use_measurement::DataUseUserData::OTHER, 0);
 
   EXPECT_EQ(kReceivedLength,
             GetInt64(data_reduction_proxy::prefs::kHttpReceivedContentLength));
@@ -582,7 +567,8 @@ TEST_F(DataReductionProxyCompressionStatsTest, TotalLengths) {
   // Record the same numbers again, and total lengths should be doubled.
   compression_stats()->RecordDataUseWithMimeType(
       kReceivedLength, kOriginalLength, IsDataReductionProxyEnabled(),
-      UNKNOWN_TYPE, std::string());
+      UNKNOWN_TYPE, std::string(), true,
+      data_use_measurement::DataUseUserData::OTHER, 0);
 
   EXPECT_EQ(kReceivedLength * 2,
             GetInt64(data_reduction_proxy::prefs::kHttpReceivedContentLength));
@@ -613,9 +599,9 @@ TEST_F(DataReductionProxyCompressionStatsTest, MultipleResponses) {
   int64_t received[] = {kReceivedLength};
   RecordContentLengthPrefs(
       kReceivedLength, kOriginalLength, false, UNKNOWN_TYPE, FakeNow());
-  VerifyDailyDataSavingContentLengthPrefLists(original, 1, received, 1, NULL, 0,
-                                              NULL, 0, NULL, 0, NULL, 0,
-                                              kNumDaysInHistory);
+  VerifyDailyDataSavingContentLengthPrefLists(original, 1, received, 1, nullptr,
+                                              0, nullptr, 0, nullptr, 0,
+                                              nullptr, 0, kNumDaysInHistory);
 
   RecordContentLengthPrefs(
       kReceivedLength, kOriginalLength, true, UNKNOWN_TYPE, FakeNow());
@@ -625,7 +611,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, MultipleResponses) {
   int64_t received_proxy_enabled[] = {kReceivedLength};
   VerifyDailyDataSavingContentLengthPrefLists(
       original, 1, received, 1, original_proxy_enabled, 1,
-      received_proxy_enabled, 1, NULL, 0, NULL, 0, kNumDaysInHistory);
+      received_proxy_enabled, 1, nullptr, 0, nullptr, 0, kNumDaysInHistory);
 
   RecordContentLengthPrefs(
       kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
@@ -856,73 +842,6 @@ TEST_F(DataReductionProxyCompressionStatsTest, PartialDayTimeChange) {
       received2, 2, kNumDaysInHistory);
 }
 
-TEST_F(DataReductionProxyCompressionStatsTest, ForwardMultipleDays) {
-  base::HistogramTester histogram_tester;
-  const int64_t kOriginalLength = 200;
-  const int64_t kReceivedLength = 100;
-  RecordContentLengthPrefs(
-      kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
-      FakeNow());
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 1);
-
-  // Forward three days.
-  SetFakeTimeDeltaInHours(3 * 24);
-
-  RecordContentLengthPrefs(
-      kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
-      FakeNow());
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 2);
-
-  int64_t original[] = {kOriginalLength, 0, 0, kOriginalLength};
-  int64_t received[] = {kReceivedLength, 0, 0, kReceivedLength};
-  VerifyDailyDataSavingContentLengthPrefLists(
-      original, 4, received, 4, original, 4, received, 4, original, 4, received,
-      4, kNumDaysInHistory);
-
-  // Forward four more days.
-  AddFakeTimeDeltaInHours(4 * 24);
-  RecordContentLengthPrefs(
-      kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
-      FakeNow());
-  int64_t original2[] = {
-      kOriginalLength, 0, 0, kOriginalLength, 0, 0, 0, kOriginalLength,
-  };
-  int64_t received2[] = {
-      kReceivedLength, 0, 0, kReceivedLength, 0, 0, 0, kReceivedLength,
-  };
-  VerifyDailyDataSavingContentLengthPrefLists(
-      original2, 8, received2, 8, original2, 8, received2, 8, original2, 8,
-      received2, 8, kNumDaysInHistory);
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 3);
-
-  // Forward |kNumDaysInHistory| more days.
-  AddFakeTimeDeltaInHours(kNumDaysInHistory * 24);
-  RecordContentLengthPrefs(
-      kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
-      FakeNow());
-  int64_t original3[] = {kOriginalLength};
-  int64_t received3[] = {kReceivedLength};
-  VerifyDailyDataSavingContentLengthPrefLists(
-      original3, 1, received3, 1, original3, 1, received3, 1, original3, 1,
-      received3, 1, kNumDaysInHistory);
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 4);
-
-  // Forward |kNumDaysInHistory| + 1 more days.
-  AddFakeTimeDeltaInHours((kNumDaysInHistory + 1)* 24);
-  RecordContentLengthPrefs(
-      kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
-      FakeNow());
-  VerifyDailyDataSavingContentLengthPrefLists(
-      original3, 1, received3, 1, original3, 1, received3, 1, original3, 1,
-      received3, 1, kNumDaysInHistory);
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 5);
-}
-
 TEST_F(DataReductionProxyCompressionStatsTest, BackwardAndForwardOneDay) {
   base::HistogramTester histogram_tester;
   const int64_t kOriginalLength = 200;
@@ -933,10 +852,8 @@ TEST_F(DataReductionProxyCompressionStatsTest, BackwardAndForwardOneDay) {
   RecordContentLengthPrefs(
       kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
       FakeNow());
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 1);
 
-  // Backward one day.
+  // Backward one day, expect no count.
   SetFakeTimeDeltaInHours(-24);
   RecordContentLengthPrefs(
       kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
@@ -946,10 +863,10 @@ TEST_F(DataReductionProxyCompressionStatsTest, BackwardAndForwardOneDay) {
   VerifyDailyDataSavingContentLengthPrefLists(
       original, 1, received, 1, original, 1, received, 1, original, 1, received,
       1, kNumDaysInHistory);
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 2);
+  histogram_tester.ExpectTotalCount("DataReductionProxy.SavingsCleared.Reason",
+                                    0);
 
-  // Then, Forward one day
+  // Then forward one day, expect no count.
   AddFakeTimeDeltaInHours(24);
   RecordContentLengthPrefs(
       kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
@@ -959,8 +876,8 @@ TEST_F(DataReductionProxyCompressionStatsTest, BackwardAndForwardOneDay) {
   VerifyDailyDataSavingContentLengthPrefLists(
       original2, 2, received2, 2, original2, 2, received2, 2, original2, 2,
       received2, 2, kNumDaysInHistory);
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 3);
+  histogram_tester.ExpectTotalCount("DataReductionProxy.SavingsCleared.Reason",
+                                    0);
 }
 
 TEST_F(DataReductionProxyCompressionStatsTest, BackwardTwoDays) {
@@ -973,10 +890,8 @@ TEST_F(DataReductionProxyCompressionStatsTest, BackwardTwoDays) {
   RecordContentLengthPrefs(
       kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
       FakeNow());
-  histogram_tester.ExpectUniqueSample(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 1);
 
-  // Backward two days.
+  // Backward two days, expect SYSTEM_CLOCK_MOVED_BACK.
   SetFakeTimeDeltaInHours(-2 * 24);
   RecordContentLengthPrefs(
       kReceivedLength, kOriginalLength, true, VIA_DATA_REDUCTION_PROXY,
@@ -984,30 +899,27 @@ TEST_F(DataReductionProxyCompressionStatsTest, BackwardTwoDays) {
   VerifyDailyDataSavingContentLengthPrefLists(
       original, 1, received, 1, original, 1, received, 1, original, 1, received,
       1, kNumDaysInHistory);
-  histogram_tester.ExpectTotalCount(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", 2);
-  histogram_tester.ExpectBucketCount(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", true, 1);
+  histogram_tester.ExpectUniqueSample(
+      "DataReductionProxy.SavingsCleared.Reason",
+      DataReductionProxySavingsClearedReason::SYSTEM_CLOCK_MOVED_BACK, 1);
   VerifyPrefInt64(prefs::kDataReductionProxySavingsClearedNegativeSystemClock,
                   FakeNow().ToInternalValue());
 
-  // Backward two days.
+  // Backward another two days, expect SYSTEM_CLOCK_MOVED_BACK.
   SetFakeTimeDeltaInHours(-4 * 24);
   RecordContentLengthPrefs(kReceivedLength, kOriginalLength, true,
                            VIA_DATA_REDUCTION_PROXY, FakeNow());
-  histogram_tester.ExpectTotalCount(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", 3);
-  histogram_tester.ExpectBucketCount(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", true, 2);
+  histogram_tester.ExpectUniqueSample(
+      "DataReductionProxy.SavingsCleared.Reason",
+      DataReductionProxySavingsClearedReason::SYSTEM_CLOCK_MOVED_BACK, 2);
 
-  // Forward 10 days.
-  AddFakeTimeDeltaInHours(10 * 24);
+  // Forward 2 days, expect no change.
+  AddFakeTimeDeltaInHours(2 * 24);
   RecordContentLengthPrefs(kReceivedLength, kOriginalLength, true,
                            VIA_DATA_REDUCTION_PROXY, FakeNow());
-  histogram_tester.ExpectTotalCount(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", 4);
-  histogram_tester.ExpectBucketCount(
-      "DataReductionProxy.SavingsCleared.NegativeSystemClock", false, 2);
+  histogram_tester.ExpectUniqueSample(
+      "DataReductionProxy.SavingsCleared.Reason",
+      DataReductionProxySavingsClearedReason::SYSTEM_CLOCK_MOVED_BACK, 2);
 }
 
 TEST_F(DataReductionProxyCompressionStatsTest, NormalizeHostname) {
@@ -1082,7 +994,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, RecordDataUsageSingleSite) {
   RecordDataUsage("https://www.foo.com", 1000, 1250, now);
 
   auto expected_data_usage =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   data_reduction_proxy::PerConnectionDataUsage* connection_usage =
       expected_data_usage->at(kNumExpectedBuckets - 1).add_connection_usage();
@@ -1113,7 +1025,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, DisableDataUsageRecording) {
 #if !defined(OS_ANDROID)
   // Data usage on disk must be deleted.
   auto expected_data_usage1 =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   DataUsageLoadVerifier verifier1(std::move(expected_data_usage1));
   LoadHistoricalDataUsage(base::Bind(&DataUsageLoadVerifier::OnLoadDataUsage,
@@ -1121,7 +1033,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, DisableDataUsageRecording) {
 
   // Public API must return an empty array.
   auto expected_data_usage2 =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>();
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>();
   DataUsageLoadVerifier verifier2(std::move(expected_data_usage2));
   GetHistoricalDataUsage(base::Bind(&DataUsageLoadVerifier::OnLoadDataUsage,
                                     base::Unretained(&verifier2)),
@@ -1129,7 +1041,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, DisableDataUsageRecording) {
 #else
   // For Android don't delete data usage.
   auto expected_data_usage =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   data_reduction_proxy::PerConnectionDataUsage* connection_usage =
       expected_data_usage->at(kNumExpectedBuckets - 1).add_connection_usage();
@@ -1159,7 +1071,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, RecordDataUsageMultipleSites) {
   RecordDataUsage("http://foobar.com", 1002, 1252, now);
 
   auto expected_data_usage =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   data_reduction_proxy::PerConnectionDataUsage* connection_usage =
       expected_data_usage->at(kNumExpectedBuckets - 1).add_connection_usage();
@@ -1200,7 +1112,7 @@ TEST_F(DataReductionProxyCompressionStatsTest,
   RecordDataUsage("https://bar.com", 1001, 1251, now);
 
   auto expected_data_usage =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   data_reduction_proxy::PerConnectionDataUsage* connection_usage =
       expected_data_usage->at(kNumExpectedBuckets - 2).add_connection_usage();
@@ -1271,7 +1183,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, DeleteHistoricalDataUsage) {
   base::RunLoop().RunUntilIdle();
 
   auto expected_data_usage =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   DataUsageLoadVerifier verifier(std::move(expected_data_usage));
 
@@ -1298,10 +1210,10 @@ TEST_F(DataReductionProxyCompressionStatsTest, DeleteBrowsingHistory) {
   DeleteBrowsingHistory(now, now);
   base::RunLoop().RunUntilIdle();
 
-  ASSERT_TRUE(DataUsageMap()->empty());
+  ASSERT_TRUE(compression_stats()->DataUsageMapForTesting().empty());
 
   auto expected_data_usage =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   data_reduction_proxy::PerConnectionDataUsage* connection_usage =
       expected_data_usage->at(kNumExpectedBuckets - 1).add_connection_usage();
@@ -1321,7 +1233,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, DeleteBrowsingHistory) {
   base::RunLoop().RunUntilIdle();
 
   expected_data_usage =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   DataUsageLoadVerifier verifier2(std::move(expected_data_usage));
   LoadHistoricalDataUsage(base::Bind(&DataUsageLoadVerifier::OnLoadDataUsage,
@@ -1356,7 +1268,7 @@ TEST_F(DataReductionProxyCompressionStatsTest, ClearDataSavingStatistics) {
   base::RunLoop().RunUntilIdle();
 
   auto expected_data_usage =
-      base::MakeUnique<std::vector<data_reduction_proxy::DataUsageBucket>>(
+      std::make_unique<std::vector<data_reduction_proxy::DataUsageBucket>>(
           kNumExpectedBuckets);
   DataUsageLoadVerifier verifier(std::move(expected_data_usage));
 
@@ -1370,4 +1282,134 @@ TEST_F(DataReductionProxyCompressionStatsTest, ClearDataSavingStatistics) {
                                               nullptr, 0, 0);
 }
 
+TEST_F(DataReductionProxyCompressionStatsTest, WeeklyAggregateDataUse) {
+  const int32_t kDataUseKB = 100;
+  base::HistogramTester histogram_tester;
+
+  InitializeWeeklyAggregateDataUse(base::Time::Now());
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.ThisWeekAggregateKB.Services.Downstream.Background",
+      0);
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.ThisWeekAggregateKB.Services.Downstream.Foreground",
+      0);
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.LastWeekAggregateKB.Services.Downstream.Background",
+      0);
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.LastWeekAggregateKB.Services.Downstream.Foreground",
+      0);
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.ThisWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      0);
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.LastWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      0);
+
+  RecordWeeklyAggregateDataUse(
+      base::Time::Now(), kDataUseKB, true,
+      data_use_measurement::DataUseUserData::MAIN_FRAME_HTML, 0);
+  VerifyDictionaryPref(prefs::kThisWeekUserTrafficContentTypeDownstreamKB,
+                       data_use_measurement::DataUseUserData::MAIN_FRAME_HTML,
+                       kDataUseKB);
+
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.ThisWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      0);
+
+  InitializeWeeklyAggregateDataUse(base::Time::Now());
+  histogram_tester.ExpectBucketCount(
+      "DataReductionProxy.ThisWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      data_use_measurement::DataUseUserData::MAIN_FRAME_HTML, kDataUseKB);
+  histogram_tester.ExpectBucketCount(
+      "DataReductionProxy.LastWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      data_use_measurement::DataUseUserData::MAIN_FRAME_HTML, 0);
+}
+
+TEST_F(DataReductionProxyCompressionStatsTest, AggregateDataUseForwardWeeks) {
+  const int32_t kMainFrameKB = 100;
+  const int32_t kNonMainFrameKB = 101;
+  base::HistogramTester histogram_tester;
+
+  base::Time fake_time_now = base::Time::Now();
+
+  InitializeWeeklyAggregateDataUse(fake_time_now);
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.ThisWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      0);
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.LastWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      0);
+
+  RecordWeeklyAggregateDataUse(
+      fake_time_now, kMainFrameKB, true,
+      data_use_measurement::DataUseUserData::MAIN_FRAME_HTML, 0);
+  VerifyDictionaryPref(prefs::kThisWeekUserTrafficContentTypeDownstreamKB,
+                       data_use_measurement::DataUseUserData::MAIN_FRAME_HTML,
+                       kMainFrameKB);
+  RecordWeeklyAggregateDataUse(
+      fake_time_now, kNonMainFrameKB, true,
+      data_use_measurement::DataUseUserData::NON_MAIN_FRAME_HTML, 0);
+  VerifyDictionaryPref(
+      prefs::kThisWeekUserTrafficContentTypeDownstreamKB,
+      data_use_measurement::DataUseUserData::NON_MAIN_FRAME_HTML,
+      kNonMainFrameKB);
+
+  // Fast forward 7 days, and verify that the last week histograms are recorded.
+  fake_time_now += base::TimeDelta::FromDays(7);
+  InitializeWeeklyAggregateDataUse(fake_time_now);
+  VerifyDictionaryPref(prefs::kLastWeekUserTrafficContentTypeDownstreamKB,
+                       data_use_measurement::DataUseUserData::MAIN_FRAME_HTML,
+                       kMainFrameKB);
+  VerifyDictionaryPref(
+      prefs::kLastWeekUserTrafficContentTypeDownstreamKB,
+      data_use_measurement::DataUseUserData::NON_MAIN_FRAME_HTML,
+      kNonMainFrameKB);
+
+  histogram_tester.ExpectBucketCount(
+      "DataReductionProxy.LastWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      data_use_measurement::DataUseUserData::MAIN_FRAME_HTML, kMainFrameKB);
+  histogram_tester.ExpectBucketCount(
+      "DataReductionProxy.LastWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      data_use_measurement::DataUseUserData::NON_MAIN_FRAME_HTML,
+      kNonMainFrameKB);
+  histogram_tester.ExpectTotalCount(
+      "DataReductionProxy.ThisWeekAggregateKB.UserTraffic.Downstream."
+      "ContentType",
+      0);
+
+  // Subsequent data use should be recorded to the current week prefs.
+  RecordWeeklyAggregateDataUse(
+      fake_time_now, kMainFrameKB, true,
+      data_use_measurement::DataUseUserData::MAIN_FRAME_HTML, 0);
+  VerifyDictionaryPref(prefs::kThisWeekUserTrafficContentTypeDownstreamKB,
+                       data_use_measurement::DataUseUserData::MAIN_FRAME_HTML,
+                       kMainFrameKB);
+
+  // Fast forward by more than two weeks, and the prefs will be cleared.
+  fake_time_now += base::TimeDelta::FromDays(15);
+  InitializeWeeklyAggregateDataUse(fake_time_now);
+  VerifyDictionaryPref(prefs::kLastWeekUserTrafficContentTypeDownstreamKB,
+                       data_use_measurement::DataUseUserData::MAIN_FRAME_HTML,
+                       0);
+  VerifyDictionaryPref(
+      prefs::kLastWeekUserTrafficContentTypeDownstreamKB,
+      data_use_measurement::DataUseUserData::NON_MAIN_FRAME_HTML, 0);
+  VerifyDictionaryPref(prefs::kThisWeekUserTrafficContentTypeDownstreamKB,
+                       data_use_measurement::DataUseUserData::MAIN_FRAME_HTML,
+                       0);
+  VerifyDictionaryPref(
+      prefs::kThisWeekUserTrafficContentTypeDownstreamKB,
+      data_use_measurement::DataUseUserData::NON_MAIN_FRAME_HTML, 0);
+}
+
 }  // namespace data_reduction_proxy
