@@ -9,13 +9,11 @@
 #include <string>
 #include <vector>
 
+#include "base/callback.h"
 #include "base/command_line.h"
-#include "base/md5.h"
-#include "base/message_loop/message_loop.h"
 #include "base/metrics/field_trial.h"
-#include "base/strings/string16.h"
-#include "base/strings/utf_string_conversions.h"
-#include "base/time/time.h"
+#include "base/strings/string_util.h"
+#include "base/test/scoped_task_environment.h"
 #include "build/build_config.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_config_test_utils.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_test_utils.h"
@@ -25,7 +23,7 @@
 #include "net/base/auth.h"
 #include "net/base/host_port_pair.h"
 #include "net/base/load_flags.h"
-#include "net/proxy/proxy_server.h"
+#include "net/base/proxy_server.h"
 #include "net/url_request/url_request.h"
 #include "net/url_request/url_request_context.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -38,17 +36,11 @@ const char kChromeProxyHeader[] = "chrome-proxy";
 const char kVersion[] = "0.1.2.3";
 const char kExpectedBuild[] = "2";
 const char kExpectedPatch[] = "3";
-const char kExpectedCredentials[] = "96bd72ec4a050ba60981743d41787768";
-const char kExpectedSession[] = "0-1633771873-1633771873-1633771873";
 const char kPageId[] = "1";
 const uint64_t kPageIdValue = 1;
 
 const char kTestKey2[] = "test-key2";
-const char kExpectedCredentials2[] = "c911fdb402f578787562cf7f00eda972";
-const char kExpectedSession2[] = "0-1633771873-1633771873-1633771873";
-const char kDataReductionProxyKey[] = "12345";
 const char kPageId2[] = "f";
-const uint64_t kPageIdValue2 = 15;
 
 const char kSecureSession[] = "TestSecureSessionKey";
 }  // namespace
@@ -92,9 +84,7 @@ const Client kClient = Client::UNKNOWN;
 const char kClientStr[] = "";
 #endif
 
-void SetHeaderExpectations(const std::string& session,
-                           const std::string& credentials,
-                           const std::string& secure_session,
+void SetHeaderExpectations(const std::string& secure_session,
                            const std::string& client,
                            const std::string& build,
                            const std::string& patch,
@@ -102,14 +92,6 @@ void SetHeaderExpectations(const std::string& session,
                            const std::vector<std::string> experiments,
                            std::string* expected_header) {
   std::vector<std::string> expected_options;
-  if (!session.empty()) {
-    expected_options.push_back(
-        std::string(kSessionHeaderOption) + "=" + session);
-  }
-  if (!credentials.empty()) {
-    expected_options.push_back(
-        std::string(kCredentialsHeaderOption) + "=" + credentials);
-  }
   if (!secure_session.empty()) {
     expected_options.push_back(std::string(kSecureSessionHeaderOption) + "=" +
                                secure_session);
@@ -153,6 +135,17 @@ class DataReductionProxyRequestOptionsTest : public testing::Test {
     request_options_->Init();
   }
 
+  void CreateRequestOptionsWithCallback(const std::string& version) {
+    CreateRequestOptions(version);
+    request_options_->SetUpdateHeaderCallback(base::BindRepeating(
+        &DataReductionProxyRequestOptionsTest::UpdateHeaderCallback,
+        base::Unretained(this)));
+  }
+
+  void UpdateHeaderCallback(const net::HttpRequestHeaders& headers) {
+    callback_headers_ = headers;
+  }
+
   TestDataReductionProxyParams* params() {
     return test_context_->config()->test_params();
   }
@@ -161,6 +154,8 @@ class DataReductionProxyRequestOptionsTest : public testing::Test {
     return request_options_.get();
   }
 
+  net::HttpRequestHeaders callback_headers() { return callback_headers_; }
+
   void VerifyExpectedHeader(const std::string& expected_header,
                             uint64_t page_id) {
     test_context_->RunUntilIdle();
@@ -176,31 +171,23 @@ class DataReductionProxyRequestOptionsTest : public testing::Test {
     EXPECT_EQ(expected_header, header_value);
   }
 
-  base::MessageLoopForIO message_loop_;
+  base::test::ScopedTaskEnvironment task_environment_{
+      base::test::ScopedTaskEnvironment::MainThreadType::IO};
   std::unique_ptr<TestDataReductionProxyRequestOptions> request_options_;
   std::unique_ptr<DataReductionProxyTestContext> test_context_;
+  net::HttpRequestHeaders callback_headers_;
 };
 
-TEST_F(DataReductionProxyRequestOptionsTest, AuthHashForSalt) {
-  std::string salt = "8675309"; // Jenny's number to test the hash generator.
-  std::string salted_key = salt + kDataReductionProxyKey + salt;
-  base::string16 expected_hash = base::UTF8ToUTF16(base::MD5String(salted_key));
-  EXPECT_EQ(expected_hash,
-            DataReductionProxyRequestOptions::AuthHashForSalt(
-                8675309, kDataReductionProxyKey));
-}
-
 TEST_F(DataReductionProxyRequestOptionsTest, AuthorizationOnIOThread) {
   std::string expected_header;
-  SetHeaderExpectations(kExpectedSession2, kExpectedCredentials2, std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        std::vector<std::string>(), &expected_header);
+  SetHeaderExpectations(std::string(), kClientStr, kExpectedBuild,
+                        kExpectedPatch, kPageId, std::vector<std::string>(),
+                        &expected_header);
 
   std::string expected_header2;
-  SetHeaderExpectations("86401-1633771873-1633771873-1633771873",
-                        "d7c1c34ef6b90303b01c48a6c1db6419", std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId2,
-                        std::vector<std::string>(), &expected_header2);
+  SetHeaderExpectations(std::string(), kClientStr, kExpectedBuild,
+                        kExpectedPatch, kPageId2, std::vector<std::string>(),
+                        &expected_header2);
 
   CreateRequestOptions(kVersion);
   test_context_->RunUntilIdle();
@@ -210,21 +197,13 @@ TEST_F(DataReductionProxyRequestOptionsTest, AuthorizationOnIOThread) {
 
   // Write headers.
   VerifyExpectedHeader(expected_header, kPageIdValue);
-
-  // Fast forward 24 hours. The header should be the same.
-  request_options()->set_offset(base::TimeDelta::FromSeconds(24 * 60 * 60));
-  VerifyExpectedHeader(expected_header, kPageIdValue);
-
-  // Fast forward one more second. The header should be new.
-  request_options()->set_offset(base::TimeDelta::FromSeconds(24 * 60 * 60 + 1));
-  VerifyExpectedHeader(expected_header2, kPageIdValue2);
 }
 
 TEST_F(DataReductionProxyRequestOptionsTest, AuthorizationIgnoresEmptyKey) {
   std::string expected_header;
-  SetHeaderExpectations(kExpectedSession, kExpectedCredentials, std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        std::vector<std::string>(), &expected_header);
+  SetHeaderExpectations(std::string(), kClientStr, kExpectedBuild,
+                        kExpectedPatch, kPageId, std::vector<std::string>(),
+                        &expected_header);
   CreateRequestOptions(kVersion);
   VerifyExpectedHeader(expected_header, kPageIdValue);
 
@@ -236,15 +215,34 @@ TEST_F(DataReductionProxyRequestOptionsTest, AuthorizationIgnoresEmptyKey) {
 
 TEST_F(DataReductionProxyRequestOptionsTest, SecureSession) {
   std::string expected_header;
-  SetHeaderExpectations(std::string(), std::string(), kSecureSession,
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        std::vector<std::string>(), &expected_header);
+  SetHeaderExpectations(kSecureSession, kClientStr, kExpectedBuild,
+                        kExpectedPatch, kPageId, std::vector<std::string>(),
+                        &expected_header);
 
   CreateRequestOptions(kVersion);
   request_options()->SetSecureSession(kSecureSession);
   VerifyExpectedHeader(expected_header, kPageIdValue);
 }
 
+TEST_F(DataReductionProxyRequestOptionsTest, CallsHeaderCallback) {
+  std::string expected_header;
+  SetHeaderExpectations(kSecureSession, kClientStr, kExpectedBuild,
+                        kExpectedPatch, kPageId, std::vector<std::string>(),
+                        &expected_header);
+
+  CreateRequestOptionsWithCallback(kVersion);
+  request_options()->SetSecureSession(kSecureSession);
+  VerifyExpectedHeader(expected_header, kPageIdValue);
+
+  std::string callback_header;
+  callback_headers().GetHeader(kChromeProxyHeader, &callback_header);
+  // |callback_header| does not include a page id. Since the page id is always
+  // the last element in the header, check that |callback_header| is the prefix
+  // of |expected_header|.
+  EXPECT_TRUE(base::StartsWith(expected_header, callback_header,
+                               base::CompareCase::SENSITIVE));
+}
+
 TEST_F(DataReductionProxyRequestOptionsTest, ParseExperiments) {
   base::CommandLine::ForCurrentProcess()->AppendSwitchASCII(
       data_reduction_proxy::switches::kDataReductionProxyExperiment,
@@ -253,9 +251,9 @@ TEST_F(DataReductionProxyRequestOptionsTest, ParseExperiments) {
   expected_experiments.push_back("staging");
   expected_experiments.push_back("\"foo,bar\"");
   std::string expected_header;
-  SetHeaderExpectations(kExpectedSession, kExpectedCredentials, std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        expected_experiments, &expected_header);
+  SetHeaderExpectations(std::string(), kClientStr, kExpectedBuild,
+                        kExpectedPatch, kPageId, expected_experiments,
+                        &expected_header);
 
   CreateRequestOptions(kVersion);
   VerifyExpectedHeader(expected_header, kPageIdValue);
@@ -302,7 +300,7 @@ TEST_F(DataReductionProxyRequestOptionsTest, ParseExperimentsFromFieldTrial) {
   for (const auto& test : tests) {
     std::vector<std::string> expected_experiments;
 
-    base::CommandLine::ForCurrentProcess()->InitFromArgv(0, NULL);
+    base::CommandLine::ForCurrentProcess()->InitFromArgv(0, nullptr);
 
     base::CommandLine::ForCurrentProcess()->AppendSwitchASCII(
         data_reduction_proxy::switches::kDataReductionProxyExperiment,
@@ -320,9 +318,9 @@ TEST_F(DataReductionProxyRequestOptionsTest, ParseExperimentsFromFieldTrial) {
     if (!test.expected_experiment.empty())
       expected_experiments.push_back(test.expected_experiment);
 
-    SetHeaderExpectations(kExpectedSession, kExpectedCredentials, std::string(),
-                          kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                          expected_experiments, &expected_header);
+    SetHeaderExpectations(std::string(), kClientStr, kExpectedBuild,
+                          kExpectedPatch, kPageId, expected_experiments,
+                          &expected_header);
 
     CreateRequestOptions(kVersion);
     VerifyExpectedHeader(expected_header, kPageIdValue);
@@ -347,23 +345,9 @@ TEST_F(DataReductionProxyRequestOptionsTest, TestExperimentPrecedence) {
   std::vector<std::string> expected_experiments;
   expected_experiments.push_back("foo");
   std::string expected_header;
-  SetHeaderExpectations(kExpectedSession, kExpectedCredentials, std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        expected_experiments, &expected_header);
-  CreateRequestOptions(kVersion);
-  VerifyExpectedHeader(expected_header, kPageIdValue);
-
-  // "force_lite_page" has the next lowest priority.
-  base::CommandLine::ForCurrentProcess()->AppendSwitchASCII(
-      switches::kDataReductionProxyLoFi,
-      switches::kDataReductionProxyLoFiValueAlwaysOn);
-  base::CommandLine::ForCurrentProcess()->AppendSwitch(
-      switches::kEnableDataReductionProxyLitePage);
-  expected_experiments.clear();
-  expected_experiments.push_back(chrome_proxy_experiment_force_lite_page());
-  SetHeaderExpectations(kExpectedSession, kExpectedCredentials, std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        expected_experiments, &expected_header);
+  SetHeaderExpectations(std::string(), kClientStr, kExpectedBuild,
+                        kExpectedPatch, kPageId, expected_experiments,
+                        &expected_header);
   CreateRequestOptions(kVersion);
   VerifyExpectedHeader(expected_header, kPageIdValue);
 
@@ -372,37 +356,9 @@ TEST_F(DataReductionProxyRequestOptionsTest, TestExperimentPrecedence) {
       data_reduction_proxy::switches::kDataReductionProxyExperiment, "bar");
   expected_experiments.clear();
   expected_experiments.push_back("bar");
-  SetHeaderExpectations(kExpectedSession, kExpectedCredentials, std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        expected_experiments, &expected_header);
-  CreateRequestOptions(kVersion);
-  VerifyExpectedHeader(expected_header, kPageIdValue);
-}
-
-TEST_F(DataReductionProxyRequestOptionsTest, TestExperimentOtherLoFiFlags) {
-  std::string expected_header;
-  std::vector<std::string> expected_experiments;
-
-  // No "exp=force_*" is set for SlowConnectionOnly flag.
-  base::CommandLine::ForCurrentProcess()->AppendSwitchASCII(
-      switches::kDataReductionProxyLoFi,
-      switches::kDataReductionProxyLoFiValueSlowConnectionsOnly);
-  expected_experiments.clear();
-  SetHeaderExpectations(kExpectedSession, kExpectedCredentials, std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        expected_experiments, &expected_header);
-  CreateRequestOptions(kVersion);
-  VerifyExpectedHeader(expected_header, kPageIdValue);
-
-  // "exp=force_empty_image" is set for CellularOnly flag.
-  base::CommandLine::ForCurrentProcess()->AppendSwitchASCII(
-      switches::kDataReductionProxyLoFi,
-      switches::kDataReductionProxyLoFiValueAlwaysOn);
-  expected_experiments.clear();
-  expected_experiments.push_back(chrome_proxy_experiment_force_empty_image());
-  SetHeaderExpectations(kExpectedSession, kExpectedCredentials, std::string(),
-                        kClientStr, kExpectedBuild, kExpectedPatch, kPageId,
-                        expected_experiments, &expected_header);
+  SetHeaderExpectations(std::string(), kClientStr, kExpectedBuild,
+                        kExpectedPatch, kPageId, expected_experiments,
+                        &expected_header);
   CreateRequestOptions(kVersion);
   VerifyExpectedHeader(expected_header, kPageIdValue);
 }
@@ -412,20 +368,22 @@ TEST_F(DataReductionProxyRequestOptionsTest, GetSessionKeyFromRequestHeaders) {
     std::string chrome_proxy_header_key;
     std::string chrome_proxy_header_value;
     std::string expected_session_key;
+    bool expect_result;
   } tests[] = {
-      {"chrome-proxy", "something=something_else, s=123, key=value", "123"},
+      {"chrome-proxy", "something=something_else, s=123, key=value", "123",
+       true},
       {"chrome-proxy", "something=something_else, s= 123  456 , key=value",
-       "123  456"},
+       "123  456", true},
       {"chrome-proxy", "something=something_else, s=123456,    key=value",
-       "123456"},
+       "123456", true},
       {"chrome-proxy", "something=something else, s=123456,    key=value",
-       "123456"},
-      {"chrome-proxy", "something=something else, s=123456  ", "123456"},
-      {"chrome-proxy", "something=something_else, s=, key=value", ""},
-      {"chrome-proxy", "something=something_else, key=value", ""},
-      {"chrome-proxy", "s=123", "123"},
-      {"chrome-proxy", " s = 123 ", "123"},
-      {"some_other_header", "s=123", ""},
+       "123456", true},
+      {"chrome-proxy", "something=something else, s=123456  ", "123456", true},
+      {"chrome-proxy", "something=something_else, s=, key=value", "", false},
+      {"chrome-proxy", "something=something_else, key=value", "", false},
+      {"chrome-proxy", "s=123", "123", true},
+      {"chrome-proxy", " s = 123 ", "123", true},
+      {"some_other_header", "s=123", "", false},
   };
 
   for (const auto& test : tests) {
@@ -435,11 +393,55 @@ TEST_F(DataReductionProxyRequestOptionsTest, GetSessionKeyFromRequestHeaders) {
                               test.chrome_proxy_header_value);
     request_headers.SetHeader("some_random_header_after", "some_random_key");
 
-    std::string session_key =
+    base::Optional<std::string> session_key =
         request_options()->GetSessionKeyFromRequestHeaders(request_headers);
-    EXPECT_EQ(test.expected_session_key, session_key)
-        << test.chrome_proxy_header_key << ":"
-        << test.chrome_proxy_header_value;
+    EXPECT_EQ(test.expect_result, session_key.has_value());
+    if (test.expect_result) {
+      EXPECT_EQ(test.expected_session_key, session_key)
+          << test.chrome_proxy_header_key << ":"
+          << test.chrome_proxy_header_value;
+    }
+  }
+}
+
+TEST_F(DataReductionProxyRequestOptionsTest, GetPageIdFromRequestHeaders) {
+  const struct {
+    std::string chrome_proxy_header_key;
+    std::string chrome_proxy_header_value;
+    uint64_t expected_page_id;
+    bool expect_result;
+  } tests[] = {
+      {"chrome-proxy", "something=something_else, pid=123, key=value", 123,
+       true},
+      {"chrome-proxy", "something=something_else, pid= 123 , key=value", 123,
+       true},
+      {"chrome-proxy", "something=something_else, pid=123456,    key=value",
+       123456, true},
+      {"chrome-proxy", "something=something else, pid=123456,    key=value",
+       123456, true},
+      {"chrome-proxy", "something=something else, pid=123456  ", 123456, true},
+      {"chrome-proxy", "something=something_else, pid=, key=value", 0, false},
+      {"chrome-proxy", "something=something_else, key=value", 0, false},
+      {"chrome-proxy", "pid=123", 123, true},
+      {"chrome-proxy", " pid = 123 ", 123, true},
+      {"some_other_header", "pid=123", 0, false},
+  };
+
+  for (const auto& test : tests) {
+    net::HttpRequestHeaders request_headers;
+    request_headers.SetHeader("some_random_header_before", "some_random_key");
+    request_headers.SetHeader(test.chrome_proxy_header_key,
+                              test.chrome_proxy_header_value);
+    request_headers.SetHeader("some_random_header_after", "some_random_key");
+
+    base::Optional<uint64_t> page_id =
+        request_options()->GetPageIdFromRequestHeaders(request_headers);
+    EXPECT_EQ(test.expect_result, page_id.has_value());
+    if (test.expect_result) {
+      EXPECT_EQ(test.expected_page_id, page_id)
+          << test.chrome_proxy_header_key << ":"
+          << test.chrome_proxy_header_value;
+    }
   }
 }
 
