@@ -7,6 +7,7 @@
 #include <map>
 #include <utility>
 
+#include "base/command_line.h"
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/run_loop.h"
@@ -19,31 +20,51 @@
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_io_data.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_mutable_config_values.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_network_delegate.h"
+#include "components/data_reduction_proxy/core/browser/data_reduction_proxy_pingback_client.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_prefs.h"
 #include "components/data_reduction_proxy/core/browser/data_reduction_proxy_settings.h"
 #include "components/data_reduction_proxy/core/browser/data_store.h"
-#include "components/data_reduction_proxy/core/common/data_reduction_proxy_event_creator.h"
-#include "components/data_reduction_proxy/core/common/data_reduction_proxy_event_storage_delegate_test_utils.h"
-#include "components/data_reduction_proxy/core/common/data_reduction_proxy_event_store.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_params.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_params_test_utils.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_pref_names.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_server.h"
+#include "components/data_reduction_proxy/core/common/data_reduction_proxy_switches.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/testing_pref_service.h"
 #include "net/base/network_delegate_impl.h"
-#include "net/proxy/proxy_config.h"
-#include "net/proxy/proxy_info.h"
-#include "net/proxy/proxy_list.h"
+#include "net/proxy_resolution/proxy_config.h"
+#include "net/proxy_resolution/proxy_info.h"
+#include "net/proxy_resolution/proxy_list.h"
 #include "net/socket/socket_test_util.h"
 #include "net/url_request/url_request_context_storage.h"
 #include "net/url_request/url_request_intercepting_job_factory.h"
 #include "net/url_request/url_request_job_factory_impl.h"
 #include "net/url_request/url_request_test_util.h"
+#include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
+#include "services/network/test/test_network_connection_tracker.h"
+#include "services/network/test/test_network_quality_tracker.h"
+#include "services/network/test/test_url_loader_factory.h"
 #include "url/gurl.h"
 
 namespace {
 
+enum TestContextOptions {
+  // Permits mocking of the underlying |DataReductionProxyConfig|.
+  USE_MOCK_CONFIG = 0x1,
+  // Construct, but do not initialize the |DataReductionProxySettings| object.
+  // Primarily used for testing of the |DataReductionProxySettings| object
+  // itself.
+  SKIP_SETTINGS_INITIALIZATION = 0x2,
+  // Permits mocking of the underlying |DataReductionProxyService|.
+  USE_MOCK_SERVICE = 0x4,
+  // Permits mocking of the underlying |DataReductionProxyRequestOptions|.
+  USE_MOCK_REQUEST_OPTIONS = 0x8,
+  // Specifies the use of the |DataReductionProxyConfigServiceClient|.
+  USE_CONFIG_CLIENT = 0x10,
+  // Specifies the use of the |TESTDataReductionProxyConfigServiceClient|.
+  USE_TEST_CONFIG_CLIENT = 0x20,
+};
+
 const char kTestKey[] = "test-key";
 
 // Name of the preference that governs enabling the Data Reduction Proxy.
@@ -74,24 +95,6 @@ std::string TestDataReductionProxyRequestOptions::GetDefaultKey() const {
   return kTestKey;
 }
 
-base::Time TestDataReductionProxyRequestOptions::Now() const {
-  return base::Time::UnixEpoch() + now_offset_;
-}
-
-void TestDataReductionProxyRequestOptions::RandBytes(void* output,
-                                                     size_t length) const {
-  char* c = static_cast<char*>(output);
-  for (size_t i = 0; i < length; ++i) {
-    c[i] = 'a';
-  }
-}
-
-// Time after the unix epoch that Now() reports.
-void TestDataReductionProxyRequestOptions::set_offset(
-    const base::TimeDelta& now_offset) {
-  now_offset_ = now_offset;
-}
-
 MockDataReductionProxyRequestOptions::MockDataReductionProxyRequestOptions(
     Client client,
     DataReductionProxyConfig* config)
@@ -106,18 +109,15 @@ TestDataReductionProxyConfigServiceClient::
         DataReductionProxyRequestOptions* request_options,
         DataReductionProxyMutableConfigValues* config_values,
         DataReductionProxyConfig* config,
-        DataReductionProxyEventCreator* event_creator,
         DataReductionProxyIOData* io_data,
-        net::NetLog* net_log,
+        network::NetworkConnectionTracker* network_connection_tracker,
         ConfigStorer config_storer)
-    : DataReductionProxyConfigServiceClient(std::move(params),
-                                            kTestBackoffPolicy,
+    : DataReductionProxyConfigServiceClient(kTestBackoffPolicy,
                                             request_options,
                                             config_values,
                                             config,
-                                            event_creator,
                                             io_data,
-                                            net_log,
+                                            network_connection_tracker,
                                             config_storer),
 #if defined(OS_ANDROID)
       is_application_state_background_(false),
@@ -173,12 +173,12 @@ TestDataReductionProxyConfigServiceClient::TestTickClock::TestTickClock(
 }
 
 base::TimeTicks
-TestDataReductionProxyConfigServiceClient::TestTickClock::NowTicks() {
+TestDataReductionProxyConfigServiceClient::TestTickClock::NowTicks() const {
   return base::TimeTicks::UnixEpoch() + (time_ - base::Time::UnixEpoch());
 }
 
-base::Time
-TestDataReductionProxyConfigServiceClient::TestTickClock::Now() {
+base::Time TestDataReductionProxyConfigServiceClient::TestTickClock::Now()
+    const {
   return time_;
 }
 
@@ -198,47 +198,68 @@ void TestDataReductionProxyConfigServiceClient::
   OnApplicationStateChange(
       base::android::APPLICATION_STATE_HAS_RUNNING_ACTIVITIES);
 }
-#endif
+#endif  // OS_ANDROID
+
+void TestDataReductionProxyConfigServiceClient::SetRemoteConfigApplied(
+    bool remote_config_applied) {
+  remote_config_applied_ = remote_config_applied;
+}
+
+bool TestDataReductionProxyConfigServiceClient::RemoteConfigApplied() const {
+  if (!remote_config_applied_) {
+    return DataReductionProxyConfigServiceClient::RemoteConfigApplied();
+  }
+  return remote_config_applied_.value();
+}
 
 MockDataReductionProxyService::MockDataReductionProxyService(
     DataReductionProxySettings* settings,
+    network::TestNetworkQualityTracker* test_network_quality_tracker,
     PrefService* prefs,
     net::URLRequestContextGetter* request_context,
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
     const scoped_refptr<base::SingleThreadTaskRunner>& task_runner)
-    : DataReductionProxyService(settings,
-                                prefs,
-                                request_context,
-                                base::MakeUnique<TestDataStore>(),
-                                task_runner,
-                                task_runner,
-                                task_runner,
-                                base::TimeDelta()) {}
+    : DataReductionProxyService(
+          settings,
+          prefs,
+          request_context,
+          std::move(url_loader_factory),
+          std::make_unique<TestDataStore>(),
+          nullptr,
+          test_network_quality_tracker,
+          network::TestNetworkConnectionTracker::GetInstance(),
+          nullptr,
+          task_runner,
+          task_runner,
+          task_runner,
+          base::TimeDelta()) {}
 
 MockDataReductionProxyService::~MockDataReductionProxyService() {
 }
 
 TestDataReductionProxyIOData::TestDataReductionProxyIOData(
+    PrefService* prefs,
     const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
     std::unique_ptr<DataReductionProxyConfig> config,
-    std::unique_ptr<DataReductionProxyEventCreator> event_creator,
     std::unique_ptr<TestDataReductionProxyRequestOptions> request_options,
     std::unique_ptr<DataReductionProxyConfigurator> configurator,
-    net::NetLog* net_log,
+    network::NetworkConnectionTracker* network_connection_tracker,
     bool enabled)
-    : DataReductionProxyIOData(),
+    : DataReductionProxyIOData(prefs, task_runner, task_runner),
       service_set_(false),
       pingback_reporting_fraction_(0.0f),
       test_request_options_(request_options.get()) {
   io_task_runner_ = task_runner;
   ui_task_runner_ = task_runner;
   config_ = std::move(config);
-  event_creator_ = std::move(event_creator);
   request_options_ = std::move(request_options);
   configurator_ = std::move(configurator);
-  net_log_ = net_log;
+  network_connection_tracker_ = network_connection_tracker;
   bypass_stats_.reset(new DataReductionProxyBypassStats(
-      config_.get(), base::Bind(&DataReductionProxyIOData::SetUnreachable,
-                                base::Unretained(this))));
+      config_.get(),
+      base::BindRepeating(&DataReductionProxyIOData::SetUnreachable,
+                          base::Unretained(this)),
+      network_connection_tracker));
   enabled_ = enabled;
 }
 
@@ -250,6 +271,11 @@ void TestDataReductionProxyIOData::SetPingbackReportingFraction(
   pingback_reporting_fraction_ = pingback_reporting_fraction;
 }
 
+void TestDataReductionProxyIOData::SetIgnoreLongTermBlackListRules(
+    bool ignore_long_term_black_list_rules) {
+  ignore_blacklist_ = ignore_long_term_black_list_rules;
+}
+
 void TestDataReductionProxyIOData::SetDataReductionProxyService(
     base::WeakPtr<DataReductionProxyService> data_reduction_proxy_service) {
   if (!service_set_)
@@ -313,6 +339,13 @@ DataReductionProxyTestContext::Builder::WithURLRequestContext(
   return *this;
 }
 
+DataReductionProxyTestContext::Builder&
+DataReductionProxyTestContext::Builder::WithURLLoaderFactory(
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory) {
+  url_loader_factory_ = url_loader_factory;
+  return *this;
+}
+
 DataReductionProxyTestContext::Builder&
 DataReductionProxyTestContext::Builder::WithMockClientSocketFactory(
     net::MockClientSocketFactory* mock_socket_factory) {
@@ -371,6 +404,13 @@ DataReductionProxyTestContext::Builder::WithProxiesForHttp(
   return *this;
 }
 
+DataReductionProxyTestContext::Builder&
+DataReductionProxyTestContext::Builder::WithSettings(
+    std::unique_ptr<DataReductionProxySettings> settings) {
+  settings_ = std::move(settings);
+  return *this;
+}
+
 std::unique_ptr<DataReductionProxyTestContext>
 DataReductionProxyTestContext::Builder::Build() {
   // Check for invalid builder combinations.
@@ -380,11 +420,14 @@ DataReductionProxyTestContext::Builder::Build() {
   scoped_refptr<base::SingleThreadTaskRunner> task_runner =
       base::ThreadTaskRunnerHandle::Get();
   scoped_refptr<net::URLRequestContextGetter> request_context_getter;
+  scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory;
   std::unique_ptr<TestingPrefServiceSimple> pref_service(
       new TestingPrefServiceSimple());
-  std::unique_ptr<net::TestNetLog> net_log(new net::TestNetLog());
+  auto* test_network_connection_tracker =
+      network::TestNetworkConnectionTracker::GetInstance();
   std::unique_ptr<TestConfigStorer> config_storer(
       new TestConfigStorer(pref_service.get()));
+
   if (request_context_) {
     request_context_getter = new net::TrivialURLRequestContextGetter(
         request_context_, task_runner);
@@ -393,17 +436,19 @@ DataReductionProxyTestContext::Builder::Build() {
         new net::TestURLRequestContext(true));
     if (mock_socket_factory_)
       test_request_context->set_client_socket_factory(mock_socket_factory_);
+
     test_request_context->Init();
     request_context_getter = new net::TestURLRequestContextGetter(
         task_runner, std::move(test_request_context));
   }
 
-  std::unique_ptr<TestDataReductionProxyEventStorageDelegate> storage_delegate(
-      new TestDataReductionProxyEventStorageDelegate());
-  std::unique_ptr<DataReductionProxyEventCreator> event_creator(
-      new DataReductionProxyEventCreator(storage_delegate.get()));
+  // In case no |url_loader_factory_| is specified, an instance will be
+  // created in DataReductionProxyTestContext's ctor.
+  if (url_loader_factory_)
+    url_loader_factory = url_loader_factory_;
+
   std::unique_ptr<DataReductionProxyConfigurator> configurator(
-      new DataReductionProxyConfigurator(net_log.get(), event_creator.get()));
+      new DataReductionProxyConfigurator());
 
   std::unique_ptr<TestDataReductionProxyConfig> config;
   std::unique_ptr<DataReductionProxyConfigServiceClient> config_client;
@@ -414,26 +459,25 @@ DataReductionProxyTestContext::Builder::Build() {
   if (use_config_client_) {
     test_context_flags |= USE_CONFIG_CLIENT;
     std::unique_ptr<DataReductionProxyMutableConfigValues> mutable_config =
-        DataReductionProxyMutableConfigValues::CreateFromParams(params.get());
+        std::make_unique<DataReductionProxyMutableConfigValues>();
     if (!proxy_servers_.empty()) {
       mutable_config->UpdateValues(proxy_servers_);
     }
     raw_mutable_config = mutable_config.get();
-    config.reset(new TestDataReductionProxyConfig(
-        std::move(mutable_config), task_runner, net_log.get(),
-        configurator.get(), event_creator.get()));
+    config.reset(new TestDataReductionProxyConfig(std::move(mutable_config),
+                                                  task_runner, task_runner,
+                                                  configurator.get()));
   } else if (use_mock_config_) {
     test_context_flags |= USE_MOCK_CONFIG;
     config.reset(new MockDataReductionProxyConfig(
-        std::move(params), task_runner, net_log.get(), configurator.get(),
-        event_creator.get()));
+        std::move(params), task_runner, task_runner, configurator.get()));
   } else {
+    test_context_flags ^= USE_MOCK_CONFIG;
     if (!proxy_servers_.empty()) {
       params->SetProxiesForHttp(proxy_servers_);
     }
     config.reset(new TestDataReductionProxyConfig(
-        std::move(params), task_runner, net_log.get(), configurator.get(),
-        event_creator.get()));
+        std::move(params), task_runner, task_runner, configurator.get()));
   }
 
   std::unique_ptr<TestDataReductionProxyRequestOptions> request_options;
@@ -447,10 +491,10 @@ DataReductionProxyTestContext::Builder::Build() {
         client_, "1.2.3.4", config.get()));
   }
 
-  std::unique_ptr<DataReductionProxySettings> settings(
-      new DataReductionProxySettings());
+  if (!settings_)
+    settings_ = std::make_unique<DataReductionProxySettings>();
   if (skip_settings_initialization_) {
-    settings->set_data_reduction_proxy_enabled_pref_name_for_test(
+    settings_->set_data_reduction_proxy_enabled_pref_name_for_test(
         kDataReductionProxyEnabled);
     test_context_flags |= SKIP_SETTINGS_INITIALIZATION;
   }
@@ -464,38 +508,35 @@ DataReductionProxyTestContext::Builder::Build() {
 
   std::unique_ptr<TestDataReductionProxyIOData> io_data(
       new TestDataReductionProxyIOData(
-          task_runner, std::move(config), std::move(event_creator),
-          std::move(request_options), std::move(configurator), net_log.get(),
-          true /* enabled */));
-  io_data->SetSimpleURLRequestContextGetter(request_context_getter);
+          pref_service.get(), task_runner, std::move(config),
+          std::move(request_options), std::move(configurator),
+          test_network_connection_tracker, true /* enabled */));
 
   if (use_test_config_client_) {
     test_context_flags |= USE_TEST_CONFIG_CLIENT;
     config_client.reset(new TestDataReductionProxyConfigServiceClient(
         std::move(params), io_data->request_options(), raw_mutable_config,
-        io_data->config(), io_data->event_creator(), io_data.get(),
-        net_log.get(), base::Bind(&TestConfigStorer::StoreSerializedConfig,
-                                  base::Unretained(config_storer.get()))));
+        io_data->config(), io_data.get(), test_network_connection_tracker,
+        base::BindRepeating(&TestConfigStorer::StoreSerializedConfig,
+                            base::Unretained(config_storer.get()))));
   } else if (use_config_client_) {
     config_client.reset(new DataReductionProxyConfigServiceClient(
-        std::move(params), GetBackoffPolicy(), io_data->request_options(),
-        raw_mutable_config, io_data->config(), io_data->event_creator(),
-        io_data.get(), net_log.get(),
+        GetBackoffPolicy(), io_data->request_options(), raw_mutable_config,
+        io_data->config(), io_data.get(), test_network_connection_tracker,
         base::Bind(&TestConfigStorer::StoreSerializedConfig,
                    base::Unretained(config_storer.get()))));
   }
   io_data->set_config_client(std::move(config_client));
 
   io_data->set_proxy_delegate(base::WrapUnique(new DataReductionProxyDelegate(
-      io_data->config(), io_data->configurator(), io_data->event_creator(),
-      io_data->bypass_stats(), net_log.get())));
+      io_data->config(), io_data->configurator(), io_data->bypass_stats())));
 
   std::unique_ptr<DataReductionProxyTestContext> test_context(
       new DataReductionProxyTestContext(
-          task_runner, std::move(pref_service), std::move(net_log),
-          request_context_getter, mock_socket_factory_, std::move(io_data),
-          std::move(settings), std::move(storage_delegate),
-          std::move(config_storer), raw_params, test_context_flags));
+          task_runner, std::move(pref_service), request_context_getter,
+          url_loader_factory, mock_socket_factory_, std::move(io_data),
+          std::move(settings_), std::move(config_storer), raw_params,
+          test_context_flags));
 
   if (!skip_settings_initialization_)
     test_context->InitSettingsWithoutCheck();
@@ -506,27 +547,36 @@ DataReductionProxyTestContext::Builder::Build() {
 DataReductionProxyTestContext::DataReductionProxyTestContext(
     const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
     std::unique_ptr<TestingPrefServiceSimple> simple_pref_service,
-    std::unique_ptr<net::TestNetLog> net_log,
     scoped_refptr<net::URLRequestContextGetter> request_context_getter,
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
     net::MockClientSocketFactory* mock_socket_factory,
     std::unique_ptr<TestDataReductionProxyIOData> io_data,
     std::unique_ptr<DataReductionProxySettings> settings,
-    std::unique_ptr<TestDataReductionProxyEventStorageDelegate>
-        storage_delegate,
     std::unique_ptr<TestConfigStorer> config_storer,
     TestDataReductionProxyParams* params,
     unsigned int test_context_flags)
     : test_context_flags_(test_context_flags),
       task_runner_(task_runner),
       simple_pref_service_(std::move(simple_pref_service)),
-      net_log_(std::move(net_log)),
       request_context_getter_(request_context_getter),
       mock_socket_factory_(mock_socket_factory),
       io_data_(std::move(io_data)),
       settings_(std::move(settings)),
-      storage_delegate_(std::move(storage_delegate)),
       config_storer_(std::move(config_storer)),
-      params_(params) {}
+      test_network_quality_tracker_(
+          std::make_unique<network::TestNetworkQualityTracker>()),
+      params_(params) {
+  if (url_loader_factory) {
+    test_shared_url_loader_factory_ = std::move(url_loader_factory);
+    return;
+  }
+
+  // To be used for EnableDataReductionProxyWithSecureProxyCheckSuccess.
+  test_url_loader_factory_ = std::make_unique<network::TestURLLoaderFactory>();
+  test_shared_url_loader_factory_ =
+      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
+          test_url_loader_factory_.get());
+}
 
 DataReductionProxyTestContext::~DataReductionProxyTestContext() {
   DestroySettings();
@@ -555,15 +605,14 @@ void DataReductionProxyTestContext::RunUntilIdle() {
 }
 
 void DataReductionProxyTestContext::InitSettings() {
-  DCHECK(test_context_flags_ &
-         DataReductionProxyTestContext::SKIP_SETTINGS_INITIALIZATION);
+  DCHECK(test_context_flags_ & SKIP_SETTINGS_INITIALIZATION);
   InitSettingsWithoutCheck();
 }
 
 void DataReductionProxyTestContext::DestroySettings() {
   // Force destruction of |DBDataOwner|, which lives on DB task runner and is
   // indirectly owned by |settings_|.
-  if (settings_.get()) {
+  if (settings_) {
     settings_.reset();
     RunUntilIdle();
   }
@@ -573,37 +622,33 @@ void DataReductionProxyTestContext::InitSettingsWithoutCheck() {
   settings_->InitDataReductionProxySettings(
       kDataReductionProxyEnabled, simple_pref_service_.get(), io_data_.get(),
       CreateDataReductionProxyServiceInternal(settings_.get()));
-  storage_delegate_->SetStorageDelegate(
-      settings_->data_reduction_proxy_service()->event_store());
   io_data_->SetDataReductionProxyService(
       settings_->data_reduction_proxy_service()->GetWeakPtr());
-  if (io_data_->config_client())
-    io_data_->config_client()->InitializeOnIOThread(
-        request_context_getter_.get());
   settings_->data_reduction_proxy_service()->SetIOData(io_data_->GetWeakPtr());
 }
 
 std::unique_ptr<DataReductionProxyService>
 DataReductionProxyTestContext::CreateDataReductionProxyService(
     DataReductionProxySettings* settings) {
-  DCHECK(test_context_flags_ &
-         DataReductionProxyTestContext::SKIP_SETTINGS_INITIALIZATION);
+  DCHECK(test_context_flags_ & SKIP_SETTINGS_INITIALIZATION);
   return CreateDataReductionProxyServiceInternal(settings);
 }
 
 std::unique_ptr<DataReductionProxyService>
 DataReductionProxyTestContext::CreateDataReductionProxyServiceInternal(
     DataReductionProxySettings* settings) {
-  if (test_context_flags_ & DataReductionProxyTestContext::USE_MOCK_SERVICE) {
-    return base::MakeUnique<MockDataReductionProxyService>(
-        settings, simple_pref_service_.get(), request_context_getter_.get(),
-        task_runner_);
-  } else {
-    return base::MakeUnique<DataReductionProxyService>(
-        settings, simple_pref_service_.get(), request_context_getter_.get(),
-        base::WrapUnique(new TestDataStore()), task_runner_, task_runner_,
-        task_runner_, base::TimeDelta());
+  if (test_context_flags_ & USE_MOCK_SERVICE) {
+    return std::make_unique<MockDataReductionProxyService>(
+        settings, test_network_quality_tracker_.get(),
+        simple_pref_service_.get(), request_context_getter_.get(),
+        test_shared_url_loader_factory_, task_runner_);
   }
+  return std::make_unique<DataReductionProxyService>(
+      settings, simple_pref_service_.get(), request_context_getter_.get(),
+      test_shared_url_loader_factory_, base::WrapUnique(new TestDataStore()),
+      nullptr, test_network_quality_tracker_.get(),
+      network::TestNetworkConnectionTracker::GetInstance(), nullptr,
+      task_runner_, task_runner_, task_runner_, base::TimeDelta());
 }
 
 void DataReductionProxyTestContext::AttachToURLRequestContext(
@@ -613,12 +658,12 @@ void DataReductionProxyTestContext::AttachToURLRequestContext(
   // |request_context_storage| takes ownership of the network delegate.
   std::unique_ptr<DataReductionProxyNetworkDelegate> network_delegate =
       io_data()->CreateNetworkDelegate(
-          base::MakeUnique<net::NetworkDelegateImpl>(), true);
+          std::make_unique<net::NetworkDelegateImpl>(), true);
 
   request_context_storage->set_network_delegate(std::move(network_delegate));
 
   request_context_storage->set_job_factory(
-      base::MakeUnique<net::URLRequestInterceptingJobFactory>(
+      std::make_unique<net::URLRequestInterceptingJobFactory>(
           std::unique_ptr<net::URLRequestJobFactory>(
               new net::URLRequestJobFactoryImpl()),
           io_data()->CreateInterceptor()));
@@ -632,25 +677,33 @@ void DataReductionProxyTestContext::
   // check.
   DCHECK(data_reduction_proxy_service());
 
+  // This method should be called in case DataReductionProxyTestContext is
+  // constructed without a valid SharedURLLoaderFactory instance.
+  DCHECK(test_url_loader_factory_);
+
   // Enable the Data Reduction Proxy, simulating a successful secure proxy
   // check.
-  net::MockRead mock_reads[] = {
-      net::MockRead("HTTP/1.1 200 OK\r\n\r\n"),
-      net::MockRead("OK"),
-      net::MockRead(net::SYNCHRONOUS, net::OK),
-  };
-  net::StaticSocketDataProvider socket_data_provider(
-      mock_reads, arraysize(mock_reads), nullptr, 0);
-  mock_socket_factory_->AddSocketDataProvider(&socket_data_provider);
+  test_url_loader_factory_->AddResponse(params::GetSecureProxyCheckURL().spec(),
+                                        "OK");
 
   // Set the pref to cause the secure proxy check to be issued.
   pref_service()->SetBoolean(kDataReductionProxyEnabled, true);
   RunUntilIdle();
 }
 
+void DataReductionProxyTestContext::DisableWarmupURLFetch() {
+  base::CommandLine::ForCurrentProcess()->AppendSwitch(
+      switches::kDisableDataReductionProxyWarmupURLFetch);
+}
+
+void DataReductionProxyTestContext::DisableWarmupURLFetchCallback() {
+  base::CommandLine::ForCurrentProcess()->AppendSwitch(
+      switches::kDisableDataReductionProxyWarmupURLFetchCallback);
+}
+
 MockDataReductionProxyConfig* DataReductionProxyTestContext::mock_config()
     const {
-  DCHECK(test_context_flags_ & DataReductionProxyTestContext::USE_MOCK_CONFIG);
+  DCHECK(test_context_flags_ & USE_MOCK_CONFIG);
   return reinterpret_cast<MockDataReductionProxyConfig*>(io_data_->config());
 }
 
@@ -662,17 +715,15 @@ DataReductionProxyTestContext::data_reduction_proxy_service() const {
 MockDataReductionProxyService*
 DataReductionProxyTestContext::mock_data_reduction_proxy_service()
     const {
-  DCHECK(!(test_context_flags_ &
-           DataReductionProxyTestContext::SKIP_SETTINGS_INITIALIZATION));
-  DCHECK(test_context_flags_ & DataReductionProxyTestContext::USE_MOCK_SERVICE);
+  DCHECK(!(test_context_flags_ & SKIP_SETTINGS_INITIALIZATION));
+  DCHECK(test_context_flags_ & USE_MOCK_SERVICE);
   return reinterpret_cast<MockDataReductionProxyService*>(
       data_reduction_proxy_service());
 }
 
 MockDataReductionProxyRequestOptions*
 DataReductionProxyTestContext::mock_request_options() const {
-  DCHECK(test_context_flags_ &
-         DataReductionProxyTestContext::USE_MOCK_REQUEST_OPTIONS);
+  DCHECK(test_context_flags_ & USE_MOCK_REQUEST_OPTIONS);
   return reinterpret_cast<MockDataReductionProxyRequestOptions*>(
       io_data_->request_options());
 }
@@ -683,16 +734,14 @@ TestDataReductionProxyConfig* DataReductionProxyTestContext::config() const {
 
 DataReductionProxyMutableConfigValues*
 DataReductionProxyTestContext::mutable_config_values() {
-  DCHECK(test_context_flags_ &
-         DataReductionProxyTestContext::USE_CONFIG_CLIENT);
+  DCHECK(test_context_flags_ & USE_CONFIG_CLIENT);
   return reinterpret_cast<DataReductionProxyMutableConfigValues*>(
       config()->config_values());
 }
 
 TestDataReductionProxyConfigServiceClient*
 DataReductionProxyTestContext::test_config_client() {
-  DCHECK(test_context_flags_ &
-         DataReductionProxyTestContext::USE_TEST_CONFIG_CLIENT);
+  DCHECK(test_context_flags_ & USE_TEST_CONFIG_CLIENT);
   return reinterpret_cast<TestDataReductionProxyConfigServiceClient*>(
       io_data_->config_client());
 }
@@ -712,6 +761,8 @@ DataReductionProxyTestContext::TestConfigStorer::TestConfigStorer(
 void DataReductionProxyTestContext::TestConfigStorer::StoreSerializedConfig(
     const std::string& serialized_config) {
   prefs_->SetString(prefs::kDataReductionProxyConfig, serialized_config);
+  prefs_->SetInt64(prefs::kDataReductionProxyLastConfigRetrievalTime,
+                   (base::Time::Now() - base::Time()).InMicroseconds());
 }
 
 std::vector<net::ProxyServer>
