@@ -17,11 +17,11 @@
 #include "base/bind_helpers.h"
 #include "base/location.h"
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
-#include "base/message_loop/message_loop.h"
+#include "base/message_loop/message_loop_current.h"
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
+#include "base/test/scoped_task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/values.h"
 #include "components/dom_distiller/core/article_distillation_update.h"
@@ -30,16 +30,17 @@
 #include "components/dom_distiller/core/proto/distilled_article.pb.h"
 #include "components/dom_distiller/core/proto/distilled_page.pb.h"
 #include "net/url_request/url_request_context_getter.h"
+#include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/dom_distiller_js/dom_distiller.pb.h"
 #include "third_party/dom_distiller_js/dom_distiller_json_converter.h"
 
-using std::vector;
 using std::string;
+using std::vector;
+using ::testing::_;
 using ::testing::Invoke;
 using ::testing::Return;
-using ::testing::_;
 
 using dom_distiller::proto::DomDistillerOptions;
 using dom_distiller::proto::DomDistillerResult;
@@ -50,10 +51,11 @@ namespace {
 const char kTitle[] = "Title";
 const char kContent[] = "Content";
 const char kURL[] = "http://a.com/";
+const size_t kTotalGoodImages = 2;
 const size_t kTotalImages = 3;
-const char* kImageURLs[kTotalImages] = {"http://a.com/img1.jpg",
-                                        "http://a.com/img2.jpg",
-                                        "./bad_url_should_fail"};
+// Good images need to be in the front.
+const char* kImageURLs[kTotalImages] = {
+    "http://a.com/img1.jpg", "http://a.com/img2.jpg", "./bad_url_should_fail"};
 const char* kImageData[kTotalImages] = {"abcde", "12345", "VWXYZ"};
 const char kDebugLog[] = "Debug Log";
 
@@ -100,7 +102,7 @@ struct MultipageDistillerData {
   ~MultipageDistillerData() {}
   vector<string> page_urls;
   vector<string> content;
-  vector<vector<int> > image_ids;
+  vector<vector<int>> image_ids;
   // The Javascript values returned by mock distiller.
   std::vector<std::unique_ptr<base::Value>> distilled_values;
 
@@ -150,36 +152,44 @@ void VerifyIncrementalUpdatesMatch(
   }
 }
 
-string GenerateNextPageUrl(const std::string& url_prefix, size_t page_num,
-    size_t pages_size) {
-  return page_num + 1 < pages_size ?
-      url_prefix + base::SizeTToString(page_num + 1) : "";
+string GenerateNextPageUrl(const std::string& url_prefix,
+                           size_t page_num,
+                           size_t pages_size) {
+  return page_num + 1 < pages_size
+             ? url_prefix + base::NumberToString(page_num + 1)
+             : "";
 }
 
 string GeneratePrevPageUrl(const std::string& url_prefix, size_t page_num) {
-  return page_num > 0 ? url_prefix + base::SizeTToString(page_num - 1) : "";
+  return page_num > 0 ? url_prefix + base::NumberToString(page_num - 1) : "";
+}
+
+std::unique_ptr<MultipageDistillerData> CreateMultipageDistillerDataWithImages(
+    const vector<vector<int>>& image_ids) {
+  size_t pages_size = image_ids.size();
+  std::unique_ptr<MultipageDistillerData> result(new MultipageDistillerData());
+  string url_prefix = kURL;
+  result->image_ids = image_ids;
+  for (size_t page_num = 0; page_num < pages_size; ++page_num) {
+    result->page_urls.push_back(url_prefix + base::NumberToString(page_num));
+    result->content.push_back("Content for page:" +
+                              base::NumberToString(page_num));
+    string next_page_url =
+        GenerateNextPageUrl(url_prefix, page_num, pages_size);
+    string prev_page_url = GeneratePrevPageUrl(url_prefix, page_num);
+    std::unique_ptr<base::Value> distilled_value =
+        CreateDistilledValueReturnedFromJS(kTitle, result->content[page_num],
+                                           image_ids[page_num], next_page_url,
+                                           prev_page_url);
+    result->distilled_values.push_back(std::move(distilled_value));
+  }
+  return result;
 }
 
 std::unique_ptr<MultipageDistillerData>
 CreateMultipageDistillerDataWithoutImages(size_t pages_size) {
-  std::unique_ptr<MultipageDistillerData> result(new MultipageDistillerData());
-  string url_prefix = kURL;
-  for (size_t page_num = 0; page_num < pages_size; ++page_num) {
-    result->page_urls.push_back(url_prefix + base::SizeTToString(page_num));
-    result->content.push_back("Content for page:" +
-                              base::SizeTToString(page_num));
-    result->image_ids.push_back(vector<int>());
-    string next_page_url =
-        GenerateNextPageUrl(url_prefix, page_num, pages_size);
-    string prev_page_url =
-        GeneratePrevPageUrl(url_prefix, page_num);
-    std::unique_ptr<base::Value> distilled_value =
-        CreateDistilledValueReturnedFromJS(kTitle, result->content[page_num],
-                                           result->image_ids[page_num],
-                                           next_page_url, prev_page_url);
-    result->distilled_values.push_back(std::move(distilled_value));
-  }
-  return result;
+  return CreateMultipageDistillerDataWithImages(
+      vector<vector<int>>(pages_size));
 }
 
 void VerifyArticleProtoMatchesMultipageData(
@@ -200,8 +210,12 @@ void VerifyArticleProtoMatchesMultipageData(
               static_cast<size_t>(page.image_size()));
     const vector<int>& image_ids_for_page = distiller_data->image_ids[page_num];
     for (size_t img_num = 0; img_num < image_ids_for_page.size(); ++img_num) {
-      EXPECT_EQ(kImageData[image_ids_for_page[img_num]],
-                page.image(img_num).data());
+      if (dom_distiller::DistillerImpl::DoesFetchImages()) {
+        EXPECT_EQ(kImageData[image_ids_for_page[img_num]],
+                  page.image(img_num).data());
+      } else {
+        EXPECT_EQ("", page.image(img_num).data());
+      }
       EXPECT_EQ(GetImageName(page_num + 1, img_num),
                 page.image(img_num).name());
     }
@@ -225,7 +239,7 @@ using test::MockDistillerPageFactory;
 class TestDistillerURLFetcher : public DistillerURLFetcher {
  public:
   explicit TestDistillerURLFetcher(bool delay_fetch)
-      : DistillerURLFetcher(NULL), delay_fetch_(delay_fetch) {
+      : DistillerURLFetcher(nullptr), delay_fetch_(delay_fetch) {
     responses_[kImageURLs[0]] = string(kImageData[0]);
     responses_[kImageURLs[1]] = string(kImageData[1]);
   }
@@ -241,10 +255,10 @@ class TestDistillerURLFetcher : public DistillerURLFetcher {
   }
 
   void PostCallbackTask() {
-    ASSERT_TRUE(base::MessageLoop::current());
+    ASSERT_TRUE(base::MessageLoopCurrent::Get());
     ASSERT_FALSE(callback_.is_null());
     base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::Bind(callback_, responses_[url_]));
+        FROM_HERE, base::BindOnce(callback_, responses_[url_]));
   }
 
  private:
@@ -256,7 +270,7 @@ class TestDistillerURLFetcher : public DistillerURLFetcher {
 
 class TestDistillerURLFetcherFactory : public DistillerURLFetcherFactory {
  public:
-  TestDistillerURLFetcherFactory() : DistillerURLFetcherFactory(NULL) {}
+  TestDistillerURLFetcherFactory() : DistillerURLFetcherFactory(nullptr) {}
 
   ~TestDistillerURLFetcherFactory() override {}
   DistillerURLFetcher* CreateDistillerURLFetcher() const override {
@@ -266,8 +280,8 @@ class TestDistillerURLFetcherFactory : public DistillerURLFetcherFactory {
 
 class MockDistillerURLFetcherFactory : public DistillerURLFetcherFactory {
  public:
-  MockDistillerURLFetcherFactory() : DistillerURLFetcherFactory(NULL) {}
-  virtual ~MockDistillerURLFetcherFactory() {}
+  MockDistillerURLFetcherFactory() : DistillerURLFetcherFactory(nullptr) {}
+  ~MockDistillerURLFetcherFactory() override {}
 
   MOCK_CONST_METHOD0(CreateDistillerURLFetcher, DistillerURLFetcher*());
 };
@@ -294,6 +308,8 @@ class DistillerTest : public testing::Test {
   }
 
  protected:
+  base::test::ScopedTaskEnvironment task_environment_{
+      base::test::ScopedTaskEnvironment::MainThreadType::UI};
   std::unique_ptr<DistillerImpl> distiller_;
   std::unique_ptr<DistilledArticleProto> article_proto_;
   std::vector<ArticleDistillationUpdate> in_sequence_updates_;
@@ -344,7 +360,6 @@ std::unique_ptr<DistillerPage> CreateMockDistillerPages(
 }
 
 TEST_F(DistillerTest, DistillPage) {
-  base::MessageLoopForUI loop;
   std::unique_ptr<base::Value> result =
       CreateDistilledValueReturnedFromJS(kTitle, kContent, vector<int>(), "");
   distiller_.reset(
@@ -359,7 +374,6 @@ TEST_F(DistillerTest, DistillPage) {
 }
 
 TEST_F(DistillerTest, DistillPageWithDebugInfo) {
-  base::MessageLoopForUI loop;
   DomDistillerResult dd_result;
   dd_result.mutable_debug_info()->set_log(kDebugLog);
   std::unique_ptr<base::Value> result =
@@ -378,17 +392,16 @@ void SetTimingEntry(TimingEntry* entry, const std::string& name, double time) {
 }
 
 TEST_F(DistillerTest, DistillPageWithTimingInfo) {
-  base::MessageLoopForUI loop;
   DomDistillerResult dd_result;
   dd_result.mutable_timing_info()->set_total_time(1.0);
   dd_result.mutable_timing_info()->set_markup_parsing_time(2.0);
   dd_result.mutable_timing_info()->set_document_construction_time(3.0);
   dd_result.mutable_timing_info()->set_article_processing_time(4.0);
   dd_result.mutable_timing_info()->set_formatting_time(5.0);
-  SetTimingEntry(
-      dd_result.mutable_timing_info()->add_other_times(), "time0", 6.0);
-  SetTimingEntry(
-      dd_result.mutable_timing_info()->add_other_times(), "time1", 7.0);
+  SetTimingEntry(dd_result.mutable_timing_info()->add_other_times(), "time0",
+                 6.0);
+  SetTimingEntry(dd_result.mutable_timing_info()->add_other_times(), "time1",
+                 7.0);
   std::unique_ptr<base::Value> result =
       dom_distiller::proto::json::DomDistillerResult::WriteToValue(dd_result);
   distiller_.reset(
@@ -412,7 +425,6 @@ TEST_F(DistillerTest, DistillPageWithTimingInfo) {
 }
 
 TEST_F(DistillerTest, DistillPageWithImages) {
-  base::MessageLoopForUI loop;
   vector<int> image_indices;
   image_indices.push_back(0);
   image_indices.push_back(1);
@@ -429,21 +441,29 @@ TEST_F(DistillerTest, DistillPageWithImages) {
   EXPECT_EQ(kContent, first_page.html());
   EXPECT_EQ(kURL, first_page.url());
   ASSERT_EQ(2, first_page.image_size());
-  EXPECT_EQ(kImageData[0], first_page.image(0).data());
+
+  if (DistillerImpl::DoesFetchImages()) {
+    EXPECT_EQ(kImageData[0], first_page.image(0).data());
+  } else {
+    EXPECT_EQ("", first_page.image(0).data());
+  }
   EXPECT_EQ(kImageURLs[0], first_page.image(0).url());
   EXPECT_EQ(GetImageName(1, 0), first_page.image(0).name());
-  EXPECT_EQ(kImageData[1], first_page.image(1).data());
+
+  if (DistillerImpl::DoesFetchImages()) {
+    EXPECT_EQ(kImageData[1], first_page.image(1).data());
+  } else {
+    EXPECT_EQ("", first_page.image(1).data());
+  }
   EXPECT_EQ(kImageURLs[1], first_page.image(1).url());
   EXPECT_EQ(GetImageName(1, 1), first_page.image(1).name());
 }
 
 TEST_F(DistillerTest, DistillMultiplePages) {
-  base::MessageLoopForUI loop;
   const size_t kNumPages = 8;
-  std::unique_ptr<MultipageDistillerData> distiller_data =
-      CreateMultipageDistillerDataWithoutImages(kNumPages);
 
   // Add images.
+  vector<vector<int>> image_ids;
   int next_image_number = 0;
   for (size_t page_num = 0; page_num < kNumPages; ++page_num) {
     // Each page has different number of images.
@@ -451,11 +471,14 @@ TEST_F(DistillerTest, DistillMultiplePages) {
     vector<int> image_indices;
     for (size_t img_num = 0; img_num < tot_images; img_num++) {
       image_indices.push_back(next_image_number);
-      next_image_number = (next_image_number + 1) % kTotalImages;
+      next_image_number = (next_image_number + 1) % kTotalGoodImages;
     }
-    distiller_data->image_ids.push_back(image_indices);
+    image_ids.push_back(image_indices);
   }
 
+  std::unique_ptr<MultipageDistillerData> distiller_data =
+      CreateMultipageDistillerDataWithImages(image_ids);
+
   distiller_.reset(
       new DistillerImpl(url_fetcher_factory_, DomDistillerOptions()));
   DistillPage(distiller_data->page_urls[0],
@@ -466,7 +489,6 @@ TEST_F(DistillerTest, DistillMultiplePages) {
 }
 
 TEST_F(DistillerTest, DistillLinkLoop) {
-  base::MessageLoopForUI loop;
   // Create a loop, the next page is same as the current page. This could
   // happen if javascript misparses a next page link.
   std::unique_ptr<base::Value> result =
@@ -480,7 +502,6 @@ TEST_F(DistillerTest, DistillLinkLoop) {
 }
 
 TEST_F(DistillerTest, CheckMaxPageLimitExtraPage) {
-  base::MessageLoopForUI loop;
   const size_t kMaxPagesInArticle = 10;
   std::unique_ptr<MultipageDistillerData> distiller_data =
       CreateMultipageDistillerDataWithoutImages(kMaxPagesInArticle);
@@ -511,7 +532,6 @@ TEST_F(DistillerTest, CheckMaxPageLimitExtraPage) {
 }
 
 TEST_F(DistillerTest, CheckMaxPageLimitExactLimit) {
-  base::MessageLoopForUI loop;
   const size_t kMaxPagesInArticle = 10;
   std::unique_ptr<MultipageDistillerData> distiller_data =
       CreateMultipageDistillerDataWithoutImages(kMaxPagesInArticle);
@@ -532,9 +552,8 @@ TEST_F(DistillerTest, CheckMaxPageLimitExactLimit) {
 }
 
 TEST_F(DistillerTest, SinglePageDistillationFailure) {
-  base::MessageLoopForUI loop;
   // To simulate failure return a null value.
-  auto null_value = base::MakeUnique<base::Value>();
+  auto null_value = std::make_unique<base::Value>();
   distiller_.reset(
       new DistillerImpl(url_fetcher_factory_, DomDistillerOptions()));
   DistillPage(kURL, CreateMockDistillerPage(null_value.get(), GURL(kURL)));
@@ -544,7 +563,6 @@ TEST_F(DistillerTest, SinglePageDistillationFailure) {
 }
 
 TEST_F(DistillerTest, MultiplePagesDistillationFailure) {
-  base::MessageLoopForUI loop;
   const size_t kNumPages = 8;
   std::unique_ptr<MultipageDistillerData> distiller_data =
       CreateMultipageDistillerDataWithoutImages(kNumPages);
@@ -556,7 +574,7 @@ TEST_F(DistillerTest, MultiplePagesDistillationFailure) {
       distiller_data->distilled_values.begin() + failed_page_num);
   distiller_data->distilled_values.insert(
       distiller_data->distilled_values.begin() + failed_page_num,
-      base::MakeUnique<base::Value>());
+      std::make_unique<base::Value>());
   // Expect only calls till the failed page number.
   distiller_.reset(
       new DistillerImpl(url_fetcher_factory_, DomDistillerOptions()));
@@ -570,7 +588,6 @@ TEST_F(DistillerTest, MultiplePagesDistillationFailure) {
 }
 
 TEST_F(DistillerTest, DistillMultiplePagesFirstEmpty) {
-  base::MessageLoopForUI loop;
   const size_t kNumPages = 8;
   std::unique_ptr<MultipageDistillerData> distiller_data =
       CreateMultipageDistillerDataWithoutImages(kNumPages);
@@ -579,9 +596,10 @@ TEST_F(DistillerTest, DistillMultiplePagesFirstEmpty) {
   const size_t empty_page_num = 0;
   distiller_data->content[empty_page_num] = "";
   std::unique_ptr<base::Value> distilled_value =
-      CreateDistilledValueReturnedFromJS(kTitle, "", vector<int>(),
-      GenerateNextPageUrl(kURL, empty_page_num, kNumPages),
-      GeneratePrevPageUrl(kURL, empty_page_num));
+      CreateDistilledValueReturnedFromJS(
+          kTitle, "", vector<int>(),
+          GenerateNextPageUrl(kURL, empty_page_num, kNumPages),
+          GeneratePrevPageUrl(kURL, empty_page_num));
   // Reset distilled data of the first page.
   distiller_data->distilled_values.erase(
       distiller_data->distilled_values.begin() + empty_page_num);
@@ -596,12 +614,11 @@ TEST_F(DistillerTest, DistillMultiplePagesFirstEmpty) {
   base::RunLoop().RunUntilIdle();
   // If the first page has no content, stop fetching the next page.
   EXPECT_EQ(1, article_proto_->pages_size());
-  VerifyArticleProtoMatchesMultipageData(
-      article_proto_.get(), distiller_data.get(), 1, 1);
+  VerifyArticleProtoMatchesMultipageData(article_proto_.get(),
+                                         distiller_data.get(), 1, 1);
 }
 
 TEST_F(DistillerTest, DistillMultiplePagesSecondEmpty) {
-  base::MessageLoopForUI loop;
   const size_t kNumPages = 8;
   std::unique_ptr<MultipageDistillerData> distiller_data =
       CreateMultipageDistillerDataWithoutImages(kNumPages);
@@ -610,9 +627,10 @@ TEST_F(DistillerTest, DistillMultiplePagesSecondEmpty) {
   const size_t empty_page_num = 1;
   distiller_data->content[empty_page_num] = "";
   std::unique_ptr<base::Value> distilled_value =
-      CreateDistilledValueReturnedFromJS(kTitle, "", vector<int>(),
-      GenerateNextPageUrl(kURL, empty_page_num, kNumPages),
-      GeneratePrevPageUrl(kURL, empty_page_num));
+      CreateDistilledValueReturnedFromJS(
+          kTitle, "", vector<int>(),
+          GenerateNextPageUrl(kURL, empty_page_num, kNumPages),
+          GeneratePrevPageUrl(kURL, empty_page_num));
   // Reset distilled data of the second page.
   distiller_data->distilled_values.erase(
       distiller_data->distilled_values.begin() + empty_page_num);
@@ -631,7 +649,6 @@ TEST_F(DistillerTest, DistillMultiplePagesSecondEmpty) {
 }
 
 TEST_F(DistillerTest, DistillPreviousPage) {
-  base::MessageLoopForUI loop;
   const size_t kNumPages = 8;
 
   // The page number of the article on which distillation starts.
@@ -650,7 +667,6 @@ TEST_F(DistillerTest, DistillPreviousPage) {
 }
 
 TEST_F(DistillerTest, IncrementalUpdates) {
-  base::MessageLoopForUI loop;
   const size_t kNumPages = 8;
 
   // The page number of the article on which distillation starts.
@@ -668,12 +684,11 @@ TEST_F(DistillerTest, IncrementalUpdates) {
   ASSERT_EQ(kNumPages, static_cast<size_t>(article_proto_->pages_size()));
   EXPECT_EQ(kNumPages, in_sequence_updates_.size());
 
-  VerifyIncrementalUpdatesMatch(
-      distiller_data.get(), kNumPages, in_sequence_updates_, start_page_num);
+  VerifyIncrementalUpdatesMatch(distiller_data.get(), kNumPages,
+                                in_sequence_updates_, start_page_num);
 }
 
 TEST_F(DistillerTest, IncrementalUpdatesDoNotDeleteFinalArticle) {
-  base::MessageLoopForUI loop;
   const size_t kNumPages = 8;
   int start_page_num = 3;
   std::unique_ptr<MultipageDistillerData> distiller_data =
@@ -695,7 +710,6 @@ TEST_F(DistillerTest, IncrementalUpdatesDoNotDeleteFinalArticle) {
 }
 
 TEST_F(DistillerTest, DeletingArticleDoesNotInterfereWithUpdates) {
-  base::MessageLoopForUI loop;
   const size_t kNumPages = 8;
   std::unique_ptr<MultipageDistillerData> distiller_data =
       CreateMultipageDistillerDataWithoutImages(kNumPages);
@@ -714,12 +728,14 @@ TEST_F(DistillerTest, DeletingArticleDoesNotInterfereWithUpdates) {
 
   // Delete the article.
   article_proto_.reset();
-  VerifyIncrementalUpdatesMatch(
-      distiller_data.get(), kNumPages, in_sequence_updates_, start_page_num);
+  VerifyIncrementalUpdatesMatch(distiller_data.get(), kNumPages,
+                                in_sequence_updates_, start_page_num);
 }
 
 TEST_F(DistillerTest, CancelWithDelayedImageFetchCallback) {
-  base::MessageLoopForUI loop;
+  if (!DistillerImpl::DoesFetchImages())
+    return;
+
   vector<int> image_indices;
   image_indices.push_back(0);
   std::unique_ptr<base::Value> distilled_value =
@@ -741,15 +757,13 @@ TEST_F(DistillerTest, CancelWithDelayedImageFetchCallback) {
 }
 
 TEST_F(DistillerTest, CancelWithDelayedJSCallback) {
-  base::MessageLoopForUI loop;
   std::unique_ptr<base::Value> distilled_value =
       CreateDistilledValueReturnedFromJS(kTitle, kContent, vector<int>(), "");
-  MockDistillerPage* distiller_page = NULL;
+  MockDistillerPage* distiller_page = nullptr;
   distiller_.reset(
       new DistillerImpl(url_fetcher_factory_, DomDistillerOptions()));
-  DistillPage(kURL,
-              CreateMockDistillerPageWithPendingJSCallback(&distiller_page,
-                                                           GURL(kURL)));
+  DistillPage(kURL, CreateMockDistillerPageWithPendingJSCallback(
+                        &distiller_page, GURL(kURL)));
   base::RunLoop().RunUntilIdle();
 
   ASSERT_TRUE(distiller_page);
