@@ -12,24 +12,25 @@
 #include <wayland-client-protocol.h>
 
 #include <cmath>
-#include <deque>
 #include <iostream>
 #include <string>
 #include <vector>
 
 #include "base/at_exit.h"
 #include "base/command_line.h"
+#include "base/containers/circular_deque.h"
 #include "base/logging.h"
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/memory/shared_memory.h"
+#include "base/message_loop/message_loop.h"
 #include "base/scoped_generic.h"
+#include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
 #include "base/time/time.h"
 #include "components/exo/wayland/clients/client_base.h"
 #include "components/exo/wayland/clients/client_helper.h"
 #include "third_party/skia/include/core/SkCanvas.h"
+#include "third_party/skia/include/core/SkFont.h"
 #include "third_party/skia/include/core/SkRefCnt.h"
 #include "third_party/skia/include/core/SkSurface.h"
 #include "third_party/skia/include/gpu/GrContext.h"
@@ -46,7 +47,11 @@ const double kRotationSpeed = 360.0;
 // Benchmark warmup frames before starting measurement.
 const int kBenchmarkWarmupFrames = 10;
 
-using EventTimeStack = std::vector<uint32_t>;
+struct EventTimes {
+  std::vector<base::TimeTicks> motion_timestamps;
+  base::TimeTicks pointer_timestamp;
+  base::TimeTicks touch_timestamp;
+};
 
 void PointerEnter(void* data,
                   wl_pointer* pointer,
@@ -65,9 +70,9 @@ void PointerMotion(void* data,
                    uint32_t time,
                    wl_fixed_t x,
                    wl_fixed_t y) {
-  EventTimeStack* stack = static_cast<EventTimeStack*>(data);
+  EventTimes* event_times = static_cast<EventTimes*>(data);
 
-  stack->push_back(time);
+  event_times->motion_timestamps.push_back(event_times->pointer_timestamp);
 }
 
 void PointerButton(void* data,
@@ -118,9 +123,9 @@ void TouchMotion(void* data,
                  int32_t id,
                  wl_fixed_t x,
                  wl_fixed_t y) {
-  EventTimeStack* stack = static_cast<EventTimeStack*>(data);
+  EventTimes* event_times = static_cast<EventTimes*>(data);
 
-  stack->push_back(time);
+  event_times->motion_timestamps.push_back(event_times->touch_timestamp);
 }
 
 void TouchFrame(void* data, wl_touch* touch) {}
@@ -149,7 +154,7 @@ struct Frame {
 };
 
 struct Presentation {
-  std::deque<std::unique_ptr<Frame>> scheduled_frames;
+  base::circular_deque<std::unique_ptr<Frame>> scheduled_frames;
   base::TimeDelta wall_time;
   base::TimeDelta cpu_time;
   base::TimeDelta latency_time;
@@ -195,12 +200,31 @@ void FeedbackDiscarded(void* data,
                        struct wp_presentation_feedback* presentation_feedback) {
   Presentation* presentation = static_cast<Presentation*>(data);
   DCHECK_GT(presentation->scheduled_frames.size(), 0u);
-  std::unique_ptr<Frame> frame =
-      std::move(presentation->scheduled_frames.front());
-  presentation->scheduled_frames.pop_front();
+  auto it =
+      std::find_if(presentation->scheduled_frames.begin(),
+                   presentation->scheduled_frames.end(),
+                   [presentation_feedback](std::unique_ptr<Frame>& frame) {
+                     return frame->feedback.get() == presentation_feedback;
+                   });
+  DCHECK(it != presentation->scheduled_frames.end());
+  presentation->scheduled_frames.erase(it);
   LOG(WARNING) << "Frame discarded";
 }
 
+void InputTimestamp(void* data,
+                    struct zwp_input_timestamps_v1* zwp_input_timestamps_v1,
+                    uint32_t tv_sec_hi,
+                    uint32_t tv_sec_lo,
+                    uint32_t tv_nsec) {
+  auto* timestamp = static_cast<base::TimeTicks*>(data);
+  int64_t seconds = (static_cast<int64_t>(tv_sec_hi) << 32) + tv_sec_lo;
+  int64_t microseconds = seconds * base::Time::kMicrosecondsPerSecond +
+                         tv_nsec / base::Time::kNanosecondsPerMicrosecond;
+
+  *timestamp =
+      base::TimeTicks() + base::TimeDelta::FromMicroseconds(microseconds);
+}
+
 }  // namespace
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -231,7 +255,7 @@ int RectsClient::Run(const ClientBase::InitParams& params,
   if (!ClientBase::Init(params))
     return 1;
 
-  EventTimeStack event_times;
+  EventTimes event_times;
 
   std::unique_ptr<wl_pointer> pointer(
       static_cast<wl_pointer*>(wl_seat_get_pointer(globals_.seat.get())));
@@ -256,12 +280,36 @@ int RectsClient::Run(const ClientBase::InitParams& params,
                                       TouchFrame, TouchCancel};
   wl_touch_add_listener(touch.get(), &touch_listener, &event_times);
 
+  zwp_input_timestamps_v1_listener input_timestamps_listener = {InputTimestamp};
+
+  std::unique_ptr<zwp_input_timestamps_v1> pointer_timestamps(
+      zwp_input_timestamps_manager_v1_get_pointer_timestamps(
+          globals_.input_timestamps_manager.get(), pointer.get()));
+  if (!pointer_timestamps) {
+    LOG(ERROR) << "Can't get pointer timestamps";
+    return 1;
+  }
+  zwp_input_timestamps_v1_add_listener(pointer_timestamps.get(),
+                                       &input_timestamps_listener,
+                                       &event_times.pointer_timestamp);
+
+  std::unique_ptr<zwp_input_timestamps_v1> touch_timestamps(
+      zwp_input_timestamps_manager_v1_get_touch_timestamps(
+          globals_.input_timestamps_manager.get(), touch.get()));
+  if (!touch_timestamps) {
+    LOG(ERROR) << "Can't get touch timestamps";
+    return 1;
+  }
+  zwp_input_timestamps_v1_add_listener(touch_timestamps.get(),
+                                       &input_timestamps_listener,
+                                       &event_times.touch_timestamp);
+
   Schedule schedule;
   std::unique_ptr<wl_callback> frame_callback;
   wl_callback_listener frame_listener = {FrameCallback};
 
   Presentation presentation;
-  std::deque<std::unique_ptr<Frame>> pending_frames;
+  base::circular_deque<std::unique_ptr<Frame>> pending_frames;
 
   size_t num_benchmark_runs_left = num_benchmark_runs;
   base::TimeTicks benchmark_start_time;
@@ -270,8 +318,10 @@ int RectsClient::Run(const ClientBase::InitParams& params,
   wp_presentation_feedback_listener feedback_listener = {
       FeedbackSyncOutput, FeedbackPresented, FeedbackDiscarded};
 
+  SkFont font;
+  font.setSize(32);
+  font.setEdging(SkFont::Edging::kAlias);
   SkPaint text_paint;
-  text_paint.setTextSize(32.0f);
   text_paint.setColor(SK_ColorWHITE);
   text_paint.setStyle(SkPaint::kFill_Style);
 
@@ -281,18 +331,13 @@ int RectsClient::Run(const ClientBase::InitParams& params,
                              ? pending_frames.size() < max_frames_pending
                              : pending_frames.empty();
     if (enqueue_frame) {
-      auto buffer_it =
-          std::find_if(buffers_.begin(), buffers_.end(),
-                       [](const std::unique_ptr<ClientBase::Buffer>& buffer) {
-                         return !buffer->busy;
-                       });
-      if (buffer_it == buffers_.end()) {
+      Buffer* buffer = DequeueBuffer();
+      if (!buffer) {
         LOG(ERROR) << "Can't find free buffer";
         return 1;
       }
-      auto* buffer = buffer_it->get();
 
-      auto frame = base::MakeUnique<Frame>();
+      auto frame = std::make_unique<Frame>();
       frame->buffer = buffer;
 
       base::TimeTicks wall_time_start;
@@ -335,7 +380,7 @@ int RectsClient::Run(const ClientBase::InitParams& params,
       }
 
       SkCanvas* canvas = buffer->sk_surface->getCanvas();
-      if (event_times.empty()) {
+      if (event_times.motion_timestamps.empty()) {
         canvas->clear(transparent_background_ ? SK_ColorTRANSPARENT
                                               : SK_ColorBLACK);
       } else {
@@ -343,26 +388,31 @@ int RectsClient::Run(const ClientBase::InitParams& params,
         // since last frame. Latest event at the top.
         int y = 0;
         // Note: Rounding up to ensure we cover the whole canvas.
-        int h = (height_ + (event_times.size() / 2)) / event_times.size();
-        while (!event_times.empty()) {
-          SkIRect rect = SkIRect::MakeXYWH(0, y, width_, h);
+        int h = (size_.height() + (event_times.motion_timestamps.size() / 2)) /
+                event_times.motion_timestamps.size();
+        while (!event_times.motion_timestamps.empty()) {
+          SkIRect rect = SkIRect::MakeXYWH(0, y, size_.width(), h);
           SkPaint paint;
-          paint.setColor(SkColorSetRGB((event_times.back() & 0x0000ff) >> 0,
-                                       (event_times.back() & 0x00ff00) >> 8,
-                                       (event_times.back() & 0xff0000) >> 16));
+          base::TimeDelta event_time =
+              event_times.motion_timestamps.back() - base::TimeTicks();
+          int64_t event_time_msec = event_time.InMilliseconds();
+          paint.setColor(SkColorSetRGB((event_time_msec & 0x0000ff) >> 0,
+                                       (event_time_msec & 0x00ff00) >> 8,
+                                       (event_time_msec & 0xff0000) >> 16));
           canvas->drawIRect(rect, paint);
-          std::string text = base::UintToString(event_times.back());
-          canvas->drawText(text.c_str(), text.length(), 8, y + 32, text_paint);
-          frame->event_times.push_back(base::TimeTicks::FromInternalValue(
-              event_times.back() * base::Time::kMicrosecondsPerMillisecond));
-          event_times.pop_back();
+          std::string text = base::NumberToString(event_time.InMicroseconds());
+          canvas->drawSimpleText(text.c_str(), text.length(),
+                                 kUTF8_SkTextEncoding, 8, y + 32, font,
+                                 text_paint);
+          frame->event_times.push_back(event_times.motion_timestamps.back());
+          event_times.motion_timestamps.pop_back();
           y += h;
         }
       }
 
       // Draw rotating rects.
-      SkScalar half_width = SkScalarHalf(width_);
-      SkScalar half_height = SkScalarHalf(height_);
+      SkScalar half_width = SkScalarHalf(size_.width());
+      SkScalar half_height = SkScalarHalf(size_.height());
       SkIRect rect = SkIRect::MakeXYWH(-SkScalarHalf(half_width),
                                        -SkScalarHalf(half_height), half_width,
                                        half_height);
@@ -374,7 +424,7 @@ int RectsClient::Run(const ClientBase::InitParams& params,
                                    SK_ColorRED,  SK_ColorYELLOW,
                                    SK_ColorCYAN, SK_ColorMAGENTA};
         SkPaint paint;
-        paint.setColor(SkColorSetA(kColors[i % arraysize(kColors)], 0xA0));
+        paint.setColor(SkColorSetA(kColors[i % base::size(kColors)], 0xA0));
         canvas->rotate(rotation / num_rects);
         canvas->drawIRect(rect, paint);
       }
@@ -382,14 +432,15 @@ int RectsClient::Run(const ClientBase::InitParams& params,
 
       // Draw FPS counter.
       if (show_fps_counter) {
-        canvas->drawText(fps_counter_text.c_str(), fps_counter_text.length(),
-                         width_ - 48, 32, text_paint);
+        canvas->drawSimpleText(fps_counter_text.c_str(),
+                               fps_counter_text.length(), kUTF8_SkTextEncoding,
+                               size_.width() - 48, 32, font, text_paint);
       }
       GrContext* gr_context = gr_context_.get();
       if (gr_context) {
         gr_context->flush();
 
-#if defined(OZONE_PLATFORM_GBM)
+#if defined(USE_GBM)
         if (egl_sync_type_) {
           buffer->egl_sync.reset(new ScopedEglSync(eglCreateSyncKHR(
               eglGetCurrentDisplay(), egl_sync_type_, nullptr)));
@@ -400,8 +451,6 @@ int RectsClient::Run(const ClientBase::InitParams& params,
         glFlush();
       }
 
-      buffer->busy = true;
-
       if (num_benchmark_runs) {
         frame->wall_time = base::TimeTicks::Now() - wall_time_start;
         frame->cpu_time = base::ThreadTicks::Now() - cpu_time_start;
@@ -417,10 +466,12 @@ int RectsClient::Run(const ClientBase::InitParams& params,
 
       wl_surface* surface = surface_.get();
       wl_surface_set_buffer_scale(surface, scale_);
-      wl_surface_damage(surface, 0, 0, width_ / scale_, height_ / scale_);
+      wl_surface_set_buffer_transform(surface_.get(), transform_);
+      wl_surface_damage(surface_.get(), 0, 0, surface_size_.width(),
+                        surface_size_.height());
       wl_surface_attach(surface, frame->buffer->buffer.get(), 0, 0);
 
-#if defined(OZONE_PLATFORM_GBM)
+#if defined(USE_GBM)
       if (frame->buffer->egl_sync) {
         eglClientWaitSyncKHR(eglGetCurrentDisplay(),
                              frame->buffer->egl_sync->get(),
@@ -482,6 +533,7 @@ int main(int argc, char* argv[]) {
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
 
   exo::wayland::clients::ClientBase::InitParams params;
+  params.num_buffers = 8;  // Allow up to 8 buffers by default.
   if (!params.FromCommandLine(*command_line))
     return 1;
 
@@ -520,6 +572,7 @@ int main(int argc, char* argv[]) {
     return 1;
   }
 
+  base::MessageLoopForUI message_loop;
   exo::wayland::clients::RectsClient client;
   return client.Run(params, max_frames_pending, num_rects, num_benchmark_runs,
                     base::TimeDelta::FromMilliseconds(benchmark_interval_ms),
