@@ -13,6 +13,7 @@
 #include "base/bind_helpers.h"
 #include "base/feature_list.h"
 #include "base/memory/ref_counted_memory.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/metrics/histogram_macros.h"
 #include "build/build_config.h"
 #include "components/favicon/core/favicon_service.h"
@@ -27,12 +28,7 @@
 namespace favicon {
 namespace {
 
-const int kNonTouchLargestIconSize = 192;
-
-// Size (along each axis) of a touch icon. This currently corresponds to
-// the apple touch icon for iPad.
-// TODO(crbug.com/736290): Consider changing this to 192x192 for Android.
-const int kTouchIconSize = 144;
+const int kLargestIconSize = 192;
 
 // Return true if |bitmap_result| is expired.
 bool IsExpired(const favicon_base::FaviconRawBitmapResult& bitmap_result) {
@@ -44,36 +40,6 @@ bool IsValid(const favicon_base::FaviconRawBitmapResult& bitmap_result) {
   return bitmap_result.is_valid();
 }
 
-void RecordDownloadAttemptsForHandlerType(
-    FaviconDriverObserver::NotificationIconType handler_type,
-    int attempts) {
-  // If not at least one attempts was recorded or more than 15 attempts were
-  // registered, something went wrong. Underflows are stored in bucket 0 and
-  // overflows in bucket 16.
-  attempts = std::max(0, std::min(attempts, 16));
-  switch (handler_type) {
-    case FaviconDriverObserver::NON_TOUCH_16_DIP:
-      UMA_HISTOGRAM_SPARSE_SLOWLY("Favicons.DownloadAttempts.Favicons",
-                                  attempts);
-      return;
-    case FaviconDriverObserver::NON_TOUCH_LARGEST:
-      UMA_HISTOGRAM_SPARSE_SLOWLY("Favicons.DownloadAttempts.LargeIcons",
-                                  attempts);
-      return;
-    case FaviconDriverObserver::TOUCH_LARGEST:
-      UMA_HISTOGRAM_SPARSE_SLOWLY("Favicons.DownloadAttempts.TouchIcons",
-                                  attempts);
-      return;
-  }
-  NOTREACHED();
-}
-
-void RecordDownloadOutcome(FaviconHandler::DownloadOutcome outcome) {
-  UMA_HISTOGRAM_ENUMERATION(
-      "Favicons.DownloadOutcome", outcome,
-      FaviconHandler::DownloadOutcome::DOWNLOAD_OUTCOME_COUNT);
-}
-
 // Returns true if |bitmap_results| is non-empty and:
 // - At least one of the bitmaps in |bitmap_results| is expired
 // OR
@@ -86,7 +52,7 @@ bool HasExpiredOrIncompleteResult(
     return false;
 
   // Check if at least one of the bitmaps is expired.
-  std::vector<favicon_base::FaviconRawBitmapResult>::const_iterator it =
+  auto it =
       std::find_if(bitmap_results.begin(), bitmap_results.end(), IsExpired);
   if (it != bitmap_results.end())
     return true;
@@ -109,9 +75,8 @@ bool HasExpiredOrIncompleteResult(
   std::vector<float> favicon_scales = favicon_base::GetFaviconScales();
   for (float favicon_scale : favicon_scales) {
     int edge_size_in_pixel = std::ceil(desired_size_in_dip * favicon_scale);
-    auto it = std::find(favicon_sizes.begin(), favicon_sizes.end(),
-                        gfx::Size(edge_size_in_pixel, edge_size_in_pixel));
-    if (it == favicon_sizes.end())
+    gfx::Size value(edge_size_in_pixel, edge_size_in_pixel);
+    if (!base::ContainsValue(favicon_sizes, value))
       return true;
   }
   return false;
@@ -125,12 +90,7 @@ bool HasValidResult(
 }
 
 std::vector<int> GetDesiredPixelSizes(
-    FaviconDriverObserver::NotificationIconType handler_type,
-    bool candidates_from_web_manifest) {
-  // When reading icons from web manifests, prefer kNonTouchLargestIconSize.
-  if (candidates_from_web_manifest)
-    return std::vector<int>(1U, kNonTouchLargestIconSize);
-
+    FaviconDriverObserver::NotificationIconType handler_type) {
   switch (handler_type) {
     case FaviconDriverObserver::NON_TOUCH_16_DIP: {
       std::vector<int> pixel_sizes;
@@ -141,9 +101,8 @@ std::vector<int> GetDesiredPixelSizes(
       return pixel_sizes;
     }
     case FaviconDriverObserver::NON_TOUCH_LARGEST:
-      return std::vector<int>(1U, kNonTouchLargestIconSize);
     case FaviconDriverObserver::TOUCH_LARGEST:
-      return std::vector<int>(1U, kTouchIconSize);
+      return std::vector<int>(1U, kLargestIconSize);
   }
   NOTREACHED();
   return std::vector<int>();
@@ -162,16 +121,30 @@ bool FaviconURLEquals(const FaviconURL& lhs, const FaviconURL& rhs) {
 FaviconHandler::FaviconCandidate
 FaviconHandler::FaviconCandidate::FromFaviconURL(
     const favicon::FaviconURL& favicon_url,
-    const std::vector<int>& desired_pixel_sizes) {
+    const std::vector<int>& desired_pixel_sizes,
+    bool want_largest_icon) {
   FaviconCandidate candidate;
   candidate.icon_url = favicon_url.icon_url;
   candidate.icon_type = favicon_url.icon_type;
-  // TODO(crbug.com/705900): For candidates without explicit size information,
-  // sizes could be inferred for the most common cases. Namely, .ico files tend
-  // to contain the 16x16 bitmap, which would allow to improve the
-  // prioritization on desktop.
-  SelectFaviconFrameIndices(favicon_url.icon_sizes, desired_pixel_sizes,
-                            /*best_indices=*/nullptr, &candidate.score);
+
+  if (!favicon_url.icon_sizes.empty()) {
+    // For candidates with explicit size information, the score is computed
+    // based on similarity with |desired_pixel_sizes|.
+    SelectFaviconFrameIndices(favicon_url.icon_sizes, desired_pixel_sizes,
+                              /*best_indices=*/nullptr, &candidate.score);
+  } else if (want_largest_icon) {
+    // If looking for largest icon (mobile), candidates without explicit size
+    // information are scored low because they are likely small.
+    candidate.score = 0.0f;
+  } else {
+    // If looking for small icons (desktop), candidates without explicit size
+    // information are scored highest, as high as candidates with an ideal
+    // explicit size information. This guarantees all candidates without
+    // explicit size information will be processed until an ideal candidate is
+    // found (if available).
+    candidate.score = 1.0f;
+  }
+
   return candidate;
 }
 
@@ -189,10 +162,11 @@ FaviconHandler::FaviconHandler(
       download_largest_icon_(
           handler_type == FaviconDriverObserver::NON_TOUCH_LARGEST ||
           handler_type == FaviconDriverObserver::TOUCH_LARGEST),
-      notification_icon_type_(favicon_base::INVALID_ICON),
+      candidates_received_(false),
+      error_other_than_404_found_(false),
+      notification_icon_type_(favicon_base::IconType::kInvalid),
       service_(service),
       delegate_(delegate),
-      num_image_download_requests_(0),
       current_candidate_index_(0u) {
   DCHECK(delegate_);
 }
@@ -201,83 +175,111 @@ FaviconHandler::~FaviconHandler() {
 }
 
 // static
-int FaviconHandler::GetIconTypesFromHandlerType(
+favicon_base::IconTypeSet FaviconHandler::GetIconTypesFromHandlerType(
     FaviconDriverObserver::NotificationIconType handler_type) {
   switch (handler_type) {
     case FaviconDriverObserver::NON_TOUCH_16_DIP:
     case FaviconDriverObserver::NON_TOUCH_LARGEST:
-      return favicon_base::FAVICON;
+      return {favicon_base::IconType::kFavicon};
     case FaviconDriverObserver::TOUCH_LARGEST:
-      return favicon_base::TOUCH_ICON | favicon_base::TOUCH_PRECOMPOSED_ICON |
-             favicon_base::WEB_MANIFEST_ICON;
+      return {favicon_base::IconType::kTouchIcon,
+              favicon_base::IconType::kTouchPrecomposedIcon,
+              favicon_base::IconType::kWebManifestIcon};
   }
-  return 0;
+  return {};
 }
 
-void FaviconHandler::FetchFavicon(const GURL& url) {
+void FaviconHandler::FetchFavicon(const GURL& page_url, bool is_same_document) {
   cancelable_task_tracker_for_page_url_.TryCancelAll();
   cancelable_task_tracker_for_candidates_.TryCancelAll();
 
-  url_ = url;
+  // We generally clear |page_urls_| and start clean unless there are obvious
+  // reasons to think URLs share favicons: the navigation must be within the
+  // same document (e.g. fragment navigation) AND it happened so early that no
+  // candidates were received for the previous URL(s) (e.g. redirect-like
+  // history.replaceState() during page load).
+  if (!is_same_document || candidates_received_) {
+    page_urls_.clear();
+  }
+  page_urls_.insert(page_url);
+  last_page_url_ = page_url;
 
   initial_history_result_expired_or_incomplete_ = false;
   redownload_icons_ = false;
   got_favicon_from_history_ = false;
   manifest_download_request_.Cancel();
   image_download_request_.Cancel();
+  candidates_received_ = false;
   manifest_url_ = GURL();
   non_manifest_original_candidates_.clear();
-  candidates_.clear();
+  final_candidates_.reset();
   notification_icon_url_ = GURL();
-  notification_icon_type_ = favicon_base::INVALID_ICON;
-  num_image_download_requests_ = 0;
+  notification_icon_type_ = favicon_base::IconType::kInvalid;
   current_candidate_index_ = 0u;
   best_favicon_ = DownloadedFavicon();
 
   // Request the favicon from the history service. In parallel to this the
   // renderer is going to notify us (well WebContents) when the favicon url is
-  // available.
+  // available. We use |last_page_url_| specifically (regardless of other
+  // possible values in |page_urls_|) because we want to use the most
+  // up-to-date / latest URL for DB lookups, which is the page URL for which
+  // we get <link rel="icon"> candidates (FaviconHandler::OnUpdateCandidates()).
   service_->GetFaviconForPageURL(
-      url_, icon_types_, preferred_icon_size(),
+      last_page_url_, icon_types_, preferred_icon_size(),
       base::Bind(&FaviconHandler::OnFaviconDataForInitialURLFromFaviconService,
                  base::Unretained(this)),
       &cancelable_task_tracker_for_page_url_);
 }
 
-bool FaviconHandler::UpdateFaviconCandidate(
-    const DownloadedFavicon& downloaded_favicon) {
-  if (downloaded_favicon.candidate.score > best_favicon_.candidate.score)
-    best_favicon_ = downloaded_favicon;
+bool FaviconHandler::ShouldDownloadNextCandidate() const {
+  // Stop downloading if the current candidate is the last candidate.
+  if (current_candidate_index_ + 1 >= final_candidates_->size())
+    return false;
 
-  if (download_largest_icon_) {
-    // The size of the downloaded icon may not match the declared size. It's
-    // important to stop downloading if:
-    // - current candidate is only candidate.
-    // - next candidate has sizes attribute and it is not better than the best
-    //   one observed so far, which means any following candidate should also
-    //   be worse or equal too.
-    // - next candidate doesn't have sizes attributes, which means further
-    //   candidates don't have sizes attribute either (because the score lowest
-    //   and hence get sorted last during prioritization). We stop immediately
-    //   to avoid downloading them all, although we don't have the certainty
-    //   that no better favicon is among them.
-    return current_candidate_index_ + 1 >= candidates_.size() ||
-           candidates_[current_candidate_index_ + 1].score <=
-               best_favicon_.candidate.score;
-  } else {
-    return best_favicon_.candidate.score == 1;
-  }
+  // Continue downloading if no valid favicon has been downloaded yet.
+  if (best_favicon_.candidate.icon_type == favicon_base::IconType::kInvalid)
+    return true;
+
+  // |next_candidate_score| is based on the sizes provided in the <link> tag,
+  // see FaviconCandidate::FromFaviconURL().
+  float next_candidate_score =
+      (*final_candidates_)[current_candidate_index_ + 1].score;
+
+  // Continue downloading only if the next candidate is better than the best one
+  // observed so far.
+  return next_candidate_score > best_favicon_.candidate.score;
 }
 
 void FaviconHandler::SetFavicon(const GURL& icon_url,
                                 const gfx::Image& image,
                                 favicon_base::IconType icon_type) {
-  if (ShouldSaveFavicon())
-    service_->SetFavicons(url_, icon_url, icon_type, image);
+  // Associate the icon to all URLs in |page_urls_|, which contains page URLs
+  // within the same site/document that have been considered to reliably share
+  // the same icon candidates.
+  if (!delegate_->IsOffTheRecord())
+    service_->SetFavicons(page_urls_, icon_url, icon_type, image);
 
   NotifyFaviconUpdated(icon_url, icon_type, image);
 }
 
+void FaviconHandler::MaybeDeleteFaviconMappings() {
+  DCHECK(candidates_received_);
+  DCHECK(got_favicon_from_history_);
+
+  // The order of these conditions is important because we want the feature
+  // state to be checked at the very end.
+  if (!error_other_than_404_found_ &&
+      notification_icon_type_ != favicon_base::IconType::kInvalid) {
+    if (!delegate_->IsOffTheRecord())
+      service_->DeleteFaviconMappings(page_urls_, notification_icon_type_);
+
+    delegate_->OnFaviconDeleted(last_page_url_, handler_type_);
+
+    notification_icon_url_ = GURL();
+    notification_icon_type_ = favicon_base::IconType::kInvalid;
+  }
+}
+
 void FaviconHandler::NotifyFaviconUpdated(
     const std::vector<favicon_base::FaviconRawBitmapResult>&
         favicon_bitmap_results) {
@@ -304,7 +306,7 @@ void FaviconHandler::NotifyFaviconUpdated(const GURL& icon_url,
   gfx::Image image_with_adjusted_colorspace = image;
   favicon_base::SetFaviconColorSpace(&image_with_adjusted_colorspace);
 
-  delegate_->OnFaviconUpdated(url_, handler_type_, icon_url,
+  delegate_->OnFaviconUpdated(last_page_url_, handler_type_, icon_url,
                               icon_url != notification_icon_url_,
                               image_with_adjusted_colorspace);
 
@@ -316,32 +318,29 @@ void FaviconHandler::OnUpdateCandidates(
     const GURL& page_url,
     const std::vector<FaviconURL>& candidates,
     const GURL& manifest_url) {
-  if (page_url != url_)
+  if (last_page_url_ != page_url)
     return;
 
-  bool manifests_feature_enabled =
-      base::FeatureList::IsEnabled(kFaviconsFromWebManifest);
-
   // |candidates| or |manifest_url| could have been modified via Javascript. If
   // neither changed, ignore the call.
-  if ((!manifests_feature_enabled || manifest_url_ == manifest_url) &&
-      non_manifest_original_candidates_.size() == candidates.size() &&
-      std::equal(candidates.begin(), candidates.end(),
-                 non_manifest_original_candidates_.begin(),
-                 &FaviconURLEquals)) {
+  if (candidates_received_ && manifest_url_ == manifest_url &&
+      (non_manifest_original_candidates_.size() == candidates.size() &&
+       std::equal(candidates.begin(), candidates.end(),
+                  non_manifest_original_candidates_.begin(),
+                  &FaviconURLEquals))) {
     return;
   }
 
+  candidates_received_ = true;
+  error_other_than_404_found_ = false;
   non_manifest_original_candidates_ = candidates;
+  final_candidates_.reset();
   cancelable_task_tracker_for_candidates_.TryCancelAll();
   manifest_download_request_.Cancel();
   image_download_request_.Cancel();
-  num_image_download_requests_ = 0;
   current_candidate_index_ = 0u;
   best_favicon_ = DownloadedFavicon();
-
-  if (manifests_feature_enabled)
-    manifest_url_ = manifest_url;
+  manifest_url_ = manifest_url;
 
   // Check if the manifest was previously blacklisted (e.g. returned a 404) and
   // ignore the manifest URL if that's the case.
@@ -360,7 +359,7 @@ void FaviconHandler::OnUpdateCandidates(
   // See if there is a cached favicon for the manifest. This will update the DB
   // mappings only if the manifest URL is cached.
   GetFaviconAndUpdateMappingsUnlessIncognito(
-      /*icon_url=*/manifest_url_, favicon_base::WEB_MANIFEST_ICON,
+      /*icon_url=*/manifest_url_, favicon_base::IconType::kWebManifestIcon,
       base::Bind(&FaviconHandler::OnFaviconDataForManifestFromFaviconService,
                  base::Unretained(this)));
 }
@@ -380,7 +379,7 @@ void FaviconHandler::OnFaviconDataForManifestFromFaviconService(
 
   if (has_valid_result &&
       (notification_icon_url_ != manifest_url_ ||
-       notification_icon_type_ != favicon_base::WEB_MANIFEST_ICON)) {
+       notification_icon_type_ != favicon_base::IconType::kWebManifestIcon)) {
     // There is a valid favicon. Notify any observers. It is useful to notify
     // the observers even if the favicon is expired or incomplete (incorrect
     // size) because temporarily showing the user an expired favicon or
@@ -389,7 +388,7 @@ void FaviconHandler::OnFaviconDataForManifestFromFaviconService(
   }
 
   if (has_expired_or_incomplete_result) {
-    manifest_download_request_.Reset(base::Bind(
+    manifest_download_request_.Reset(base::BindOnce(
         &FaviconHandler::OnDidDownloadManifest, base::Unretained(this)));
     delegate_->DownloadManifest(manifest_url_,
                                 manifest_download_request_.callback());
@@ -420,25 +419,26 @@ void FaviconHandler::OnDidDownloadManifest(
 
 void FaviconHandler::OnGotFinalIconURLCandidates(
     const std::vector<FaviconURL>& candidates) {
+  DCHECK(!final_candidates_);
+
   const std::vector<int> desired_pixel_sizes =
-      GetDesiredPixelSizes(handler_type_, !manifest_url_.is_empty());
+      GetDesiredPixelSizes(handler_type_);
 
   std::vector<FaviconCandidate> sorted_candidates;
   for (const FaviconURL& candidate : candidates) {
-    if (!candidate.icon_url.is_empty() && (candidate.icon_type & icon_types_)) {
-      sorted_candidates.push_back(
-          FaviconCandidate::FromFaviconURL(candidate, desired_pixel_sizes));
+    if (!candidate.icon_url.is_empty() &&
+        (icon_types_.count(candidate.icon_type) != 0)) {
+      sorted_candidates.push_back(FaviconCandidate::FromFaviconURL(
+          candidate, desired_pixel_sizes, download_largest_icon_));
     }
   }
 
   std::stable_sort(sorted_candidates.begin(), sorted_candidates.end(),
                    &FaviconCandidate::CompareScore);
 
-  candidates_ = std::move(sorted_candidates);
+  final_candidates_ = std::move(sorted_candidates);
 
-  // TODO(davemoore) Should clear on empty url. Currently we ignore it.
-  // This appears to be what FF does as well.
-  if (current_candidate() && got_favicon_from_history_)
+  if (got_favicon_from_history_)
     OnGotInitialHistoryDataAndIconURLCandidates();
 }
 
@@ -447,14 +447,24 @@ int FaviconHandler::GetMaximalIconSize(
     FaviconDriverObserver::NotificationIconType handler_type,
     bool candidates_from_web_manifest) {
   int max_size = 0;
-  for (int size :
-       GetDesiredPixelSizes(handler_type, candidates_from_web_manifest)) {
+  for (int size : GetDesiredPixelSizes(handler_type)) {
     max_size = std::max(max_size, size);
   }
   return max_size;
 }
 
 void FaviconHandler::OnGotInitialHistoryDataAndIconURLCandidates() {
+  DCHECK(final_candidates_);
+  DCHECK(got_favicon_from_history_);
+  DCHECK_EQ(0U, current_candidate_index_);
+
+  if (final_candidates_->empty()) {
+    // The page lists no candidates that match our target |icon_types_|, so
+    // check if any existing mappings should be deleted.
+    MaybeDeleteFaviconMappings();
+    return;
+  }
+
   if (!initial_history_result_expired_or_incomplete_ &&
       current_candidate()->icon_url == notification_icon_url_ &&
       current_candidate()->icon_type == notification_icon_type_) {
@@ -482,23 +492,21 @@ void FaviconHandler::OnDidDownloadFavicon(
   // Mark download as finished.
   image_download_request_.Cancel();
 
-  if (bitmaps.empty() && http_status_code == 404) {
-    DVLOG(1) << "Failed to Download Favicon:" << image_url;
-    RecordDownloadOutcome(DownloadOutcome::FAILED);
-    service_->UnableToDownloadFavicon(image_url);
-  }
-
-  bool request_next_icon = true;
-  if (!bitmaps.empty()) {
-    RecordDownloadOutcome(DownloadOutcome::SUCCEEDED);
+  if (bitmaps.empty()) {
+    if (http_status_code == 404) {
+      DVLOG(1) << "Failed to Download Favicon:" << image_url;
+      service_->UnableToDownloadFavicon(image_url);
+    } else if (http_status_code != 0) {
+      error_other_than_404_found_ = true;
+    }
+  } else {
     float score = 0.0f;
     gfx::ImageSkia image_skia;
     if (download_largest_icon_) {
       std::vector<size_t> best_indices;
-      SelectFaviconFrameIndices(
-          original_bitmap_sizes,
-          GetDesiredPixelSizes(handler_type_, !manifest_url_.is_empty()),
-          &best_indices, &score);
+      SelectFaviconFrameIndices(original_bitmap_sizes,
+                                GetDesiredPixelSizes(handler_type_),
+                                &best_indices, &score);
       DCHECK_EQ(1U, best_indices.size());
       image_skia =
           gfx::ImageSkia::CreateFrom1xBitmap(bitmaps[best_indices.front()]);
@@ -509,47 +517,43 @@ void FaviconHandler::OnDidDownloadFavicon(
                                           &score);
     }
 
-    if (!image_skia.isNull()) {
-      // The downloaded icon is still valid when there is no FaviconURL update
-      // during the downloading.
-      DownloadedFavicon downloaded_favicon;
-      downloaded_favicon.image = gfx::Image(image_skia);
-      downloaded_favicon.candidate.icon_url = image_url;
-      downloaded_favicon.candidate.icon_type = icon_type;
-      downloaded_favicon.candidate.score = score;
-      request_next_icon = !UpdateFaviconCandidate(downloaded_favicon);
+    if (!image_skia.isNull() && score > best_favicon_.candidate.score) {
+      best_favicon_.image = gfx::Image(image_skia);
+      best_favicon_.candidate.icon_url = image_url;
+      best_favicon_.candidate.icon_type = icon_type;
+      best_favicon_.candidate.score = score;
     }
   }
 
-  if (request_next_icon && current_candidate_index_ + 1 < candidates_.size()) {
+  if (ShouldDownloadNextCandidate()) {
     // Process the next candidate.
     ++current_candidate_index_;
+    DCHECK_LT(current_candidate_index_, final_candidates_->size());
     DownloadCurrentCandidateOrAskFaviconService();
   } else {
-    // OnDidDownloadFavicon() can only be called after requesting a download, so
-    // |num_image_download_requests_| can never be 0.
-    RecordDownloadAttemptsForHandlerType(handler_type_,
-                                         num_image_download_requests_);
-    // We have either found the ideal candidate or run out of candidates.
-    if (best_favicon_.candidate.icon_type != favicon_base::INVALID_ICON) {
+    if (best_favicon_.candidate.icon_type == favicon_base::IconType::kInvalid) {
+      // No valid icon found, so check if mappings should be deleted.
+      MaybeDeleteFaviconMappings();
+    } else {
+      // We have either found the ideal candidate or run out of candidates.
       // No more icons to request, set the favicon from the candidate. The
       // manifest URL, if available, is used instead of the icon URL.
       SetFavicon(manifest_url_.is_empty() ? best_favicon_.candidate.icon_url
                                           : manifest_url_,
                  best_favicon_.image,
-                 manifest_url_.is_empty() ? best_favicon_.candidate.icon_type
-                                          : favicon_base::WEB_MANIFEST_ICON);
+                 manifest_url_.is_empty()
+                     ? best_favicon_.candidate.icon_type
+                     : favicon_base::IconType::kWebManifestIcon);
     }
     // Clear download related state.
-    current_candidate_index_ = candidates_.size();
-    num_image_download_requests_ = 0;
+    current_candidate_index_ = final_candidates_->size();
     best_favicon_ = DownloadedFavicon();
   }
 }
 
 const std::vector<GURL> FaviconHandler::GetIconURLs() const {
   std::vector<GURL> icon_urls;
-  for (const FaviconCandidate& candidate : candidates_)
+  for (const FaviconCandidate& candidate : *final_candidates_)
     icon_urls.push_back(candidate.icon_url);
   return icon_urls;
 }
@@ -561,14 +565,6 @@ bool FaviconHandler::HasPendingTasksForTest() {
          cancelable_task_tracker_for_candidates_.HasTrackedTasks();
 }
 
-bool FaviconHandler::ShouldSaveFavicon() {
-  if (!delegate_->IsOffTheRecord())
-    return true;
-
-  // Always save favicon if the page is bookmarked.
-  return delegate_->IsBookmarked(url_);
-}
-
 void FaviconHandler::OnFaviconDataForInitialURLFromFaviconService(
     const std::vector<favicon_base::FaviconRawBitmapResult>&
         favicon_bitmap_results) {
@@ -582,6 +578,22 @@ void FaviconHandler::OnFaviconDataForInitialURLFromFaviconService(
                       !favicon_bitmap_results.empty();
 
   if (has_valid_result) {
+    // Propagate mappings to all redirects, in case the redirect chain is
+    // different from the one observed the previous time the page URL was
+    // visited.
+    //
+    // Do the propagation now because we want the propagation to occur in all
+    // scenarios and this is an easy way of guaranteeing it. For instance, we
+    // want the propagation to occur when:
+    // - The favicon in the database is expired.
+    // AND
+    // - Redownloading the favicon fails with a non-404 error code.
+    if (!delegate_->IsOffTheRecord() &&
+        base::FeatureList::IsEnabled(kAllowPropagationOfFaviconCacheHits)) {
+      service_->CloneFaviconMappingsForPages(last_page_url_, icon_types_,
+                                             page_urls_);
+    }
+
     // The db knows the favicon (although it may be out of date). Set the
     // favicon now, and if the favicon turns out to be expired (or the wrong
     // url) we'll fetch later on. This way the user doesn't see a flash of the
@@ -589,11 +601,15 @@ void FaviconHandler::OnFaviconDataForInitialURLFromFaviconService(
     NotifyFaviconUpdated(favicon_bitmap_results);
   }
 
-  if (current_candidate())
+  if (final_candidates_)
     OnGotInitialHistoryDataAndIconURLCandidates();
 }
 
 void FaviconHandler::DownloadCurrentCandidateOrAskFaviconService() {
+  DCHECK(image_download_request_.IsCancelled());
+  DCHECK(manifest_download_request_.IsCancelled());
+  DCHECK(current_candidate());
+
   const GURL icon_url = current_candidate()->icon_url;
   const favicon_base::IconType icon_type = current_candidate()->icon_type;
   // If the icons listed in a manifest are being processed, skip the cache
@@ -626,7 +642,7 @@ void FaviconHandler::GetFaviconAndUpdateMappingsUnlessIncognito(
     //    include the mapping between the page url and the favicon url.
     // This is asynchronous. The history service will call back when done.
     service_->UpdateFaviconMappingsAndFetch(
-        url_, icon_url, icon_type, preferred_icon_size(), callback,
+        page_urls_, icon_url, icon_type, preferred_icon_size(), callback,
         &cancelable_task_tracker_for_candidates_);
   }
 }
@@ -649,9 +665,6 @@ void FaviconHandler::OnFaviconData(const std::vector<
   if (has_expired_or_incomplete_result) {
     ScheduleImageDownload(current_candidate()->icon_url,
                           current_candidate()->icon_type);
-  } else if (num_image_download_requests_ > 0) {
-    RecordDownloadAttemptsForHandlerType(handler_type_,
-                                         num_image_download_requests_);
   }
 }
 
@@ -663,15 +676,13 @@ void FaviconHandler::ScheduleImageDownload(const GURL& image_url,
       << "More than one ongoing download";
   if (service_->WasUnableToDownloadFavicon(image_url)) {
     DVLOG(1) << "Skip Failed FavIcon: " << image_url;
-    RecordDownloadOutcome(DownloadOutcome::SKIPPED);
     OnDidDownloadFavicon(icon_type, 0, 0, image_url, std::vector<SkBitmap>(),
                          std::vector<gfx::Size>());
     return;
   }
-  ++num_image_download_requests_;
   image_download_request_.Reset(
-      base::Bind(&FaviconHandler::OnDidDownloadFavicon, base::Unretained(this),
-                 icon_type));
+      base::BindOnce(&FaviconHandler::OnDidDownloadFavicon,
+                     base::Unretained(this), icon_type));
   // A max bitmap size is specified to avoid receiving huge bitmaps in
   // OnDidDownloadFavicon(). See FaviconDriver::StartDownload()
   // for more details about the max bitmap size.
