@@ -7,15 +7,17 @@
 #include <map>
 #include <string>
 
+#include <memory>
+
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
-#include "base/strings/string_util.h"
+#include "base/strings/strcat.h"
 #include "base/synchronization/waitable_event.h"
+#include "build/build_config.h"
 #include "components/grpc_support/include/bidirectional_stream_c.h"
 #include "components/grpc_support/test/get_stream_engine.h"
-#include "components/grpc_support/test/quic_test_server.h"
 #include "net/base/net_errors.h"
+#include "net/test/quic_simple_test_server.h"
 #include "net/test/test_data_directory.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/gurl.h"
@@ -35,23 +37,28 @@ namespace grpc_support {
 class BidirectionalStreamTest : public ::testing::TestWithParam<bool> {
  protected:
   void SetUp() override {
-    StartQuicTestServer();
-    StartTestStreamEngine(GetQuicTestServerPort());
+    net::QuicSimpleTestServer::Start();
+    StartTestStreamEngine(net::QuicSimpleTestServer::GetPort());
+    quic_server_hello_url_ = net::QuicSimpleTestServer::GetHelloURL().spec();
   }
 
   void TearDown() override {
     ShutdownTestStreamEngine();
-    ShutdownQuicTestServer();
+    net::QuicSimpleTestServer::Shutdown();
   }
 
   BidirectionalStreamTest() {}
   ~BidirectionalStreamTest() override {}
 
   stream_engine* engine() {
-    return GetTestStreamEngine(GetQuicTestServerPort());
+    return GetTestStreamEngine(net::QuicSimpleTestServer::GetPort());
   }
 
+  const char* test_hello_url() const { return quic_server_hello_url_.c_str(); }
+
  private:
+  std::string quic_server_hello_url_;
+
   DISALLOW_COPY_AND_ASSIGN(BidirectionalStreamTest);
 };
 
@@ -110,21 +117,19 @@ class TestBidirectionalStreamCallback {
         read_buffer(nullptr),
         net_error(0) {}
 
-  ~TestBidirectionalStreamCallback() {
-    if (read_buffer)
-      delete[] read_buffer;
-  }
+  ~TestBidirectionalStreamCallback() { delete[] read_buffer; }
 
   static TestBidirectionalStreamCallback* FromStream(
       bidirectional_stream* stream) {
     DCHECK(stream);
-    return (TestBidirectionalStreamCallback*)stream->annotation;
+    return reinterpret_cast<TestBidirectionalStreamCallback*>(
+        stream->annotation);
   }
 
   virtual bool MaybeCancel(bidirectional_stream* stream, ResponseStep step) {
     DCHECK_EQ(stream, this->stream);
     response_step = step;
-    DLOG(WARNING) << "Step: " << step;
+    DVLOG(3) << "Step: " << step;
 
     if (step != cancel_from_step)
       return false;
@@ -141,7 +146,7 @@ class TestBidirectionalStreamCallback {
 
   void AddWriteData(const std::string& data) { AddWriteData(data, true); }
   void AddWriteData(const std::string& data, bool flush) {
-    write_data.push_back(base::MakeUnique<WriteData>(data, flush));
+    write_data.push_back(std::make_unique<WriteData>(data, flush));
   }
 
   virtual void MaybeWriteNextData(bidirectional_stream* stream) {
@@ -266,29 +271,40 @@ TestBidirectionalStreamCallback::WriteData::WriteData(const std::string& data,
 TestBidirectionalStreamCallback::WriteData::~WriteData() {}
 
 TEST_P(BidirectionalStreamTest, StartExampleBidiStream) {
-  TestBidirectionalStreamCallback test;
-  test.AddWriteData("Hello, ");
-  test.AddWriteData("world!");
+  TestBidirectionalStreamCallback test_callback;
+  test_callback.AddWriteData("Hello, ");
+  test_callback.AddWriteData("world!");
   // Use small read buffer size to test that response is split properly.
-  test.read_buffer_size = 2;
-  test.stream = bidirectional_stream_create(engine(), &test, test.callback());
-  DCHECK(test.stream);
-  bidirectional_stream_delay_request_headers_until_flush(test.stream,
+  test_callback.read_buffer_size = 2;
+  test_callback.stream = bidirectional_stream_create(engine(), &test_callback,
+                                                     test_callback.callback());
+  DCHECK(test_callback.stream);
+  bidirectional_stream_delay_request_headers_until_flush(test_callback.stream,
                                                          GetParam());
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, false);
-  test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
-  ASSERT_EQ(std::string(kHelloHeaderValue),
-            test.response_headers[kHelloHeaderName]);
-  ASSERT_EQ(TestBidirectionalStreamCallback::ON_SUCCEEDED, test.response_step);
-  ASSERT_EQ(std::string(kHelloBodyValue, 2), test.read_data.front());
+  bidirectional_stream_start(test_callback.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, false);
+  test_callback.BlockForDone();
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test_callback
+          .response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloHeaderValue(),
+      test_callback
+          .response_headers[net::QuicSimpleTestServer::GetHelloHeaderName()]);
+  ASSERT_EQ(TestBidirectionalStreamCallback::ON_SUCCEEDED,
+            test_callback.response_step);
+  ASSERT_EQ(std::string(net::QuicSimpleTestServer::GetHelloBodyValue(), 0, 2),
+            test_callback.read_data.front());
   // Verify that individual read data joined using empty separator match
   // expected body.
-  ASSERT_EQ(std::string(kHelloBodyValue), base::JoinString(test.read_data, ""));
-  ASSERT_EQ(std::string(kHelloTrailerValue),
-            test.response_trailers[kHelloTrailerName]);
-  bidirectional_stream_destroy(test.stream);
+  ASSERT_EQ(net::QuicSimpleTestServer::GetHelloBodyValue(),
+            base::StrCat(test_callback.read_data));
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloTrailerValue(),
+      test_callback
+          .response_trailers[net::QuicSimpleTestServer::GetHelloTrailerName()]);
+  bidirectional_stream_destroy(test_callback.stream);
 }
 
 TEST_P(BidirectionalStreamTest, SimplePutWithEmptyWriteDataAtTheEnd) {
@@ -300,16 +316,21 @@ TEST_P(BidirectionalStreamTest, SimplePutWithEmptyWriteDataAtTheEnd) {
   DCHECK(test.stream);
   bidirectional_stream_delay_request_headers_until_flush(test.stream,
                                                          GetParam());
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "PUT", &kTestHeadersArray, false);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "PUT",
+                             &kTestHeadersArray, false);
   test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
-  ASSERT_EQ(std::string(kHelloHeaderValue),
-            test.response_headers[kHelloHeaderName]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test.response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloHeaderValue(),
+      test.response_headers[net::QuicSimpleTestServer::GetHelloHeaderName()]);
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_SUCCEEDED, test.response_step);
-  ASSERT_EQ(std::string(kHelloBodyValue), test.read_data.front());
-  ASSERT_EQ(std::string(kHelloTrailerValue),
-            test.response_trailers[kHelloTrailerName]);
+  ASSERT_EQ(net::QuicSimpleTestServer::GetHelloBodyValue(),
+            test.read_data.front());
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloTrailerValue(),
+      test.response_trailers[net::QuicSimpleTestServer::GetHelloTrailerName()]);
   bidirectional_stream_destroy(test.stream);
 }
 
@@ -322,16 +343,21 @@ TEST_P(BidirectionalStreamTest, SimpleGetWithFlush) {
                                                          GetParam());
   // Flush before start is ignored.
   bidirectional_stream_flush(test.stream);
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "GET", &kTestHeadersArray, true);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "GET",
+                             &kTestHeadersArray, true);
   test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
-  ASSERT_EQ(std::string(kHelloHeaderValue),
-            test.response_headers[kHelloHeaderName]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test.response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloHeaderValue(),
+      test.response_headers[net::QuicSimpleTestServer::GetHelloHeaderName()]);
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_SUCCEEDED, test.response_step);
-  ASSERT_EQ(std::string(kHelloBodyValue), base::JoinString(test.read_data, ""));
-  ASSERT_EQ(std::string(kHelloTrailerValue),
-            test.response_trailers[kHelloTrailerName]);
+  ASSERT_EQ(net::QuicSimpleTestServer::GetHelloBodyValue(),
+            test.read_data.front());
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloTrailerValue(),
+      test.response_trailers[net::QuicSimpleTestServer::GetHelloTrailerName()]);
   // Flush after done is ignored.
   bidirectional_stream_flush(test.stream);
   bidirectional_stream_destroy(test.stream);
@@ -349,16 +375,21 @@ TEST_P(BidirectionalStreamTest, SimplePostWithFlush) {
                                                          GetParam());
   // Flush before start is ignored.
   bidirectional_stream_flush(test.stream);
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, false);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, false);
   test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
-  ASSERT_EQ(std::string(kHelloHeaderValue),
-            test.response_headers[kHelloHeaderName]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test.response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloHeaderValue(),
+      test.response_headers[net::QuicSimpleTestServer::GetHelloHeaderName()]);
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_SUCCEEDED, test.response_step);
-  ASSERT_EQ(std::string(kHelloBodyValue), base::JoinString(test.read_data, ""));
-  ASSERT_EQ(std::string(kHelloTrailerValue),
-            test.response_trailers[kHelloTrailerName]);
+  ASSERT_EQ(net::QuicSimpleTestServer::GetHelloBodyValue(),
+            base::StrCat(test.read_data));
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloTrailerValue(),
+      test.response_trailers[net::QuicSimpleTestServer::GetHelloTrailerName()]);
   // Flush after done is ignored.
   bidirectional_stream_flush(test.stream);
   bidirectional_stream_destroy(test.stream);
@@ -379,16 +410,21 @@ TEST_P(BidirectionalStreamTest, SimplePostWithFlushTwice) {
                                                          GetParam());
   // Flush before start is ignored.
   bidirectional_stream_flush(test.stream);
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, false);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, false);
   test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
-  ASSERT_EQ(std::string(kHelloHeaderValue),
-            test.response_headers[kHelloHeaderName]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test.response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloHeaderValue(),
+      test.response_headers[net::QuicSimpleTestServer::GetHelloHeaderName()]);
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_SUCCEEDED, test.response_step);
-  ASSERT_EQ(std::string(kHelloBodyValue), base::JoinString(test.read_data, ""));
-  ASSERT_EQ(std::string(kHelloTrailerValue),
-            test.response_trailers[kHelloTrailerName]);
+  ASSERT_EQ(net::QuicSimpleTestServer::GetHelloBodyValue(),
+            base::StrCat(test.read_data));
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloTrailerValue(),
+      test.response_trailers[net::QuicSimpleTestServer::GetHelloTrailerName()]);
   // Flush after done is ignored.
   bidirectional_stream_flush(test.stream);
   bidirectional_stream_destroy(test.stream);
@@ -406,16 +442,21 @@ TEST_P(BidirectionalStreamTest, SimplePostWithFlushAfterOneWrite) {
                                                          GetParam());
   // Flush before start is ignored.
   bidirectional_stream_flush(test.stream);
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, false);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, false);
   test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
-  ASSERT_EQ(std::string(kHelloHeaderValue),
-            test.response_headers[kHelloHeaderName]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test.response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloHeaderValue(),
+      test.response_headers[net::QuicSimpleTestServer::GetHelloHeaderName()]);
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_SUCCEEDED, test.response_step);
-  ASSERT_EQ(std::string(kHelloBodyValue), base::JoinString(test.read_data, ""));
-  ASSERT_EQ(std::string(kHelloTrailerValue),
-            test.response_trailers[kHelloTrailerName]);
+  ASSERT_EQ(net::QuicSimpleTestServer::GetHelloBodyValue(),
+            base::StrCat(test.read_data));
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloTrailerValue(),
+      test.response_trailers[net::QuicSimpleTestServer::GetHelloTrailerName()]);
   // Flush after done is ignored.
   bidirectional_stream_flush(test.stream);
   bidirectional_stream_destroy(test.stream);
@@ -461,8 +502,8 @@ TEST_P(BidirectionalStreamTest, TestDelayedFlush) {
                                                          GetParam());
   // Flush before start is ignored.
   bidirectional_stream_flush(test.stream);
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, false);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, false);
   test.BlockForDone();
   // Flush after done is ignored.
   bidirectional_stream_flush(test.stream);
@@ -476,11 +517,14 @@ TEST_P(BidirectionalStreamTest, CancelOnRead) {
   bidirectional_stream_delay_request_headers_until_flush(test.stream,
                                                          GetParam());
   test.cancel_from_step = TestBidirectionalStreamCallback::ON_READ_COMPLETED;
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, true);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, true);
   test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
-  ASSERT_EQ(std::string(kHelloBodyValue), test.read_data.front());
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test.response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
+  ASSERT_EQ(net::QuicSimpleTestServer::GetHelloBodyValue(),
+            test.read_data.front());
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_CANCELED, test.response_step);
   bidirectional_stream_destroy(test.stream);
 }
@@ -492,10 +536,12 @@ TEST_P(BidirectionalStreamTest, CancelOnResponse) {
   bidirectional_stream_delay_request_headers_until_flush(test.stream,
                                                          GetParam());
   test.cancel_from_step = TestBidirectionalStreamCallback::ON_RESPONSE_STARTED;
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, true);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, true);
   test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test.response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
   ASSERT_TRUE(test.read_data.empty());
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_CANCELED, test.response_step);
   bidirectional_stream_destroy(test.stream);
@@ -508,11 +554,14 @@ TEST_P(BidirectionalStreamTest, CancelOnSucceeded) {
   bidirectional_stream_delay_request_headers_until_flush(test.stream,
                                                          GetParam());
   test.cancel_from_step = TestBidirectionalStreamCallback::ON_SUCCEEDED;
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, true);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, true);
   test.BlockForDone();
-  ASSERT_EQ(std::string(kHelloStatus), test.response_headers[kStatusHeader]);
-  ASSERT_EQ(std::string(kHelloBodyValue), test.read_data.front());
+  ASSERT_EQ(
+      net::QuicSimpleTestServer::GetHelloStatus(),
+      test.response_headers[net::QuicSimpleTestServer::GetStatusHeaderName()]);
+  ASSERT_EQ(net::QuicSimpleTestServer::GetHelloBodyValue(),
+            test.read_data.front());
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_SUCCEEDED, test.response_step);
   bidirectional_stream_destroy(test.stream);
 }
@@ -532,14 +581,22 @@ TEST_P(BidirectionalStreamTest, ReadFailsBeforeRequestStarted) {
   bidirectional_stream_destroy(test.stream);
 }
 
-TEST_P(BidirectionalStreamTest, StreamFailBeforeReadIsExecutedOnNetworkThread) {
+// TODO(https://crbug.com/880474): This test is flaky on fuchsia_x64 builder.
+#if defined(OS_FUCHSIA)
+#define MAYBE_StreamFailBeforeReadIsExecutedOnNetworkThread \
+  DISABLED_StreamFailBeforeReadIsExecutedOnNetworkThread
+#else
+#define MAYBE_StreamFailBeforeReadIsExecutedOnNetworkThread \
+  StreamFailBeforeReadIsExecutedOnNetworkThread
+#endif
+TEST_P(BidirectionalStreamTest,
+       MAYBE_StreamFailBeforeReadIsExecutedOnNetworkThread) {
   class CustomTestBidirectionalStreamCallback
       : public TestBidirectionalStreamCallback {
     bool MaybeCancel(bidirectional_stream* stream, ResponseStep step) override {
       if (step == ResponseStep::ON_READ_COMPLETED) {
-        // Shut down the server, and the stream should error out.
-        // The second call to ShutdownQuicTestServer is no-op.
-        ShutdownQuicTestServer();
+        // Shut down the server dispatcher, and the stream should error out.
+        net::QuicSimpleTestServer::ShutdownDispatcherForTesting();
       }
       return TestBidirectionalStreamCallback::MaybeCancel(stream, step);
     }
@@ -553,8 +610,8 @@ TEST_P(BidirectionalStreamTest, StreamFailBeforeReadIsExecutedOnNetworkThread) {
   DCHECK(test.stream);
   bidirectional_stream_delay_request_headers_until_flush(test.stream,
                                                          GetParam());
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, false);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, false);
   test.BlockForDone();
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_FAILED, test.response_step);
   ASSERT_TRUE(test.net_error == net::ERR_QUIC_PROTOCOL_ERROR ||
@@ -581,9 +638,8 @@ TEST_P(BidirectionalStreamTest, StreamFailAfterStreamReadyCallback) {
       : public TestBidirectionalStreamCallback {
     bool MaybeCancel(bidirectional_stream* stream, ResponseStep step) override {
       if (step == ResponseStep::ON_STREAM_READY) {
-        // Shut down the server, and the stream should error out.
-        // The second call to ShutdownQuicTestServer is no-op.
-        ShutdownQuicTestServer();
+        // Shut down the server dispatcher, and the stream should error out.
+        net::QuicSimpleTestServer::ShutdownDispatcherForTesting();
       }
       return TestBidirectionalStreamCallback::MaybeCancel(stream, step);
     }
@@ -595,8 +651,8 @@ TEST_P(BidirectionalStreamTest, StreamFailAfterStreamReadyCallback) {
   DCHECK(test.stream);
   bidirectional_stream_delay_request_headers_until_flush(test.stream,
                                                          GetParam());
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, false);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, false);
   test.BlockForDone();
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_FAILED, test.response_step);
   ASSERT_TRUE(test.net_error == net::ERR_QUIC_PROTOCOL_ERROR ||
@@ -611,9 +667,8 @@ TEST_P(BidirectionalStreamTest,
       : public TestBidirectionalStreamCallback {
     bool MaybeCancel(bidirectional_stream* stream, ResponseStep step) override {
       if (step == ResponseStep::ON_WRITE_COMPLETED) {
-        // Shut down the server, and the stream should error out.
-        // The second call to ShutdownQuicTestServer is no-op.
-        ShutdownQuicTestServer();
+        // Shut down the server dispatcher, and the stream should error out.
+        net::QuicSimpleTestServer::ShutdownDispatcherForTesting();
       }
       return TestBidirectionalStreamCallback::MaybeCancel(stream, step);
     }
@@ -627,8 +682,8 @@ TEST_P(BidirectionalStreamTest,
   DCHECK(test.stream);
   bidirectional_stream_delay_request_headers_until_flush(test.stream,
                                                          GetParam());
-  bidirectional_stream_start(test.stream, kTestServerUrl, 0,
-                             "POST", &kTestHeadersArray, false);
+  bidirectional_stream_start(test.stream, test_hello_url(), 0, "POST",
+                             &kTestHeadersArray, false);
   test.BlockForDone();
   ASSERT_EQ(TestBidirectionalStreamCallback::ON_FAILED, test.response_step);
   ASSERT_TRUE(test.net_error == net::ERR_QUIC_PROTOCOL_ERROR ||
