@@ -4,11 +4,11 @@
 
 #include "components/guest_view/browser/guest_view_base.h"
 
+#include <memory>
 #include <utility>
 
 #include "base/lazy_instance.h"
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "components/guest_view/browser/guest_view_event.h"
 #include "components/guest_view/browser/guest_view_manager.h"
@@ -16,6 +16,7 @@
 #include "components/guest_view/common/guest_view_messages.h"
 #include "components/zoom/page_zoom.h"
 #include "components/zoom/zoom_controller.h"
+#include "content/public/browser/file_select_listener.h"
 #include "content/public/browser/guest_mode.h"
 #include "content/public/browser/navigation_handle.h"
 #include "content/public/browser/render_frame_host.h"
@@ -27,7 +28,7 @@
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/page_zoom.h"
 #include "content/public/common/url_constants.h"
-#include "third_party/WebKit/public/platform/WebGestureEvent.h"
+#include "third_party/blink/public/platform/web_gesture_event.h"
 
 using content::WebContents;
 
@@ -108,13 +109,6 @@ class GuestViewBase::OwnerContentsObserver : public WebContentsObserver {
     }
   }
 
-  void OnPageScaleFactorChanged(float page_scale_factor) override {
-    if (destroyed_)
-      return;
-
-    guest_->web_contents()->SetPageScale(page_scale_factor);
-  }
-
   void DidUpdateAudioMutingState(bool muted) override {
     if (destroyed_)
       return;
@@ -136,7 +130,16 @@ class GuestViewBase::OwnerContentsObserver : public WebContentsObserver {
     if (destroyed_)
       return;
     destroyed_ = true;
-    guest_->Destroy(true);
+
+    bool also_delete = true;
+    WebContents* guest_web_contents = guest_->web_contents();
+    if (content::GuestMode::IsCrossProcessFrameGuest(guest_web_contents)) {
+      // The outer WebContents have ownership of attached OOPIF-based guests, so
+      // we are not responsible for their deletion.
+      if (guest_web_contents->GetOuterWebContents())
+        also_delete = false;
+    }
+    guest_->Destroy(also_delete);
   }
 
   DISALLOW_COPY_AND_ASSIGN(OwnerContentsObserver);
@@ -187,7 +190,7 @@ GuestViewBase::GuestViewBase(WebContents* owner_web_contents)
 GuestViewBase::~GuestViewBase() {}
 
 void GuestViewBase::Init(const base::DictionaryValue& create_params,
-                         const WebContentsCreatedCallback& callback) {
+                         WebContentsCreatedCallback callback) {
   if (initialized_)
     return;
   initialized_ = true;
@@ -196,16 +199,15 @@ void GuestViewBase::Init(const base::DictionaryValue& create_params,
     // The derived class did not create a WebContents so this class serves no
     // purpose. Let's self-destruct.
     delete this;
-    callback.Run(nullptr);
+    std::move(callback).Run(nullptr);
     return;
   }
 
   std::unique_ptr<base::DictionaryValue> params(create_params.DeepCopy());
   CreateWebContents(create_params,
-                    base::Bind(&GuestViewBase::CompleteInit,
-                               weak_ptr_factory_.GetWeakPtr(),
-                               base::Passed(&params),
-                               callback));
+                    base::BindOnce(&GuestViewBase::CompleteInit,
+                                   weak_ptr_factory_.GetWeakPtr(),
+                                   std::move(params), std::move(callback)));
 }
 
 void GuestViewBase::InitWithWebContents(
@@ -228,7 +230,7 @@ void GuestViewBase::InitWithWebContents(
   // an observer to the owner WebContents. This observer will be responsible
   // for destroying the guest WebContents if the owner goes away.
   owner_contents_observer_ =
-      base::MakeUnique<OwnerContentsObserver>(this, owner_web_contents_);
+      std::make_unique<OwnerContentsObserver>(this, owner_web_contents_);
 
   WebContentsObserver::Observe(guest_web_contents);
   guest_web_contents->SetDelegate(this);
@@ -263,13 +265,13 @@ void GuestViewBase::DispatchOnResizeEvent(const gfx::Size& old_size,
     return;
 
   // Dispatch the onResize event.
-  auto args = base::MakeUnique<base::DictionaryValue>();
+  auto args = std::make_unique<base::DictionaryValue>();
   args->SetInteger(kOldWidth, old_size.width());
   args->SetInteger(kOldHeight, old_size.height());
   args->SetInteger(kNewWidth, new_size.width());
   args->SetInteger(kNewHeight, new_size.height());
   DispatchEventToGuestProxy(
-      base::MakeUnique<GuestViewEvent>(kEventResize, std::move(args)));
+      std::make_unique<GuestViewEvent>(kEventResize, std::move(args)));
 }
 
 gfx::Size GuestViewBase::GetDefaultSize() const {
@@ -299,10 +301,12 @@ void GuestViewBase::SetSize(const SetSizeParams& params) {
   enable_auto_size &= !min_auto_size_.IsEmpty() && !max_auto_size_.IsEmpty() &&
                       IsAutoSizeSupported();
 
-  content::RenderViewHost* rvh = web_contents()->GetRenderViewHost();
+  content::RenderWidgetHostView* rwhv =
+      web_contents()->GetRenderWidgetHostView();
   if (enable_auto_size) {
     // Autosize is being enabled.
-    rvh->EnableAutoResize(min_auto_size_, max_auto_size_);
+    if (rwhv)
+      rwhv->EnableAutoResize(min_auto_size_, max_auto_size_);
     normal_size_.SetSize(0, 0);
   } else {
     // Autosize is being disabled.
@@ -324,7 +328,8 @@ void GuestViewBase::SetSize(const SetSizeParams& params) {
     bool changed_due_to_auto_resize = false;
     if (auto_size_enabled_) {
       // Autosize was previously enabled.
-      rvh->DisableAutoResize(new_size);
+      if (rwhv)
+        rwhv->DisableAutoResize(new_size);
       changed_due_to_auto_resize = true;
     } else {
       // Autosize was already disabled.
@@ -391,8 +396,6 @@ bool GuestViewBase::ZoomPropagatesFromEmbedderToGuest() const {
   return true;
 }
 
-void GuestViewBase::SetContextMenuPosition(const gfx::Point& position) {}
-
 GuestViewManager* GuestViewBase::GetGuestViewManager() {
   return GuestViewManager::FromBrowserContext(browser_context());
 }
@@ -436,8 +439,8 @@ void GuestViewBase::DidAttach(int guest_proxy_routing_id) {
 void GuestViewBase::DidDetach() {
   GuestViewManager::FromBrowserContext(browser_context_)->DetachGuest(this);
   StopTrackingEmbedderZoomLevel();
-  owner_web_contents()->Send(new GuestViewMsg_GuestDetached(
-      element_instance_id_));
+  owner_web_contents()->GetRenderViewHost()->Send(
+      new GuestViewMsg_GuestDetached(element_instance_id_));
   element_instance_id_ = kInstanceIDNone;
   if (ShouldDestroyOnDetach())
     Destroy(true);
@@ -447,10 +450,6 @@ WebContents* GuestViewBase::GetOwnerWebContents() const {
   return owner_web_contents_;
 }
 
-void GuestViewBase::GuestSizeChanged(const gfx::Size& new_size) {
-  UpdateGuestSize(new_size, auto_size_enabled_);
-}
-
 const GURL& GuestViewBase::GetOwnerSiteURL() const {
   return owner_web_contents()->GetLastCommittedURL();
 }
@@ -471,8 +470,6 @@ void GuestViewBase::Destroy(bool also_delete) {
   StopTrackingEmbedderZoomLevel();
   owner_web_contents_ = nullptr;
 
-  element_instance_id_ = kInstanceIDNone;
-
   DCHECK(web_contents());
 
   // Give the derived class an opportunity to perform some cleanup.
@@ -505,7 +502,7 @@ void GuestViewBase::SetOpener(GuestViewBase* guest) {
     opener_ = guest->weak_ptr_factory_.GetWeakPtr();
     if (!attached()) {
       opener_lifetime_observer_ =
-          base::MakeUnique<OpenerLifetimeObserver>(this);
+          std::make_unique<OpenerLifetimeObserver>(this);
     }
     return;
   }
@@ -520,7 +517,16 @@ void GuestViewBase::SetGuestHost(content::GuestHost* guest_host) {
 void GuestViewBase::WillAttach(WebContents* embedder_web_contents,
                                int element_instance_id,
                                bool is_full_page_plugin,
-                               const base::Closure& callback) {
+                               base::OnceClosure completion_callback) {
+  WillAttach(embedder_web_contents, nullptr, element_instance_id,
+             is_full_page_plugin, std::move(completion_callback));
+}
+
+void GuestViewBase::WillAttach(WebContents* embedder_web_contents,
+                               content::RenderFrameHost* outer_contents_frame,
+                               int element_instance_id,
+                               bool is_full_page_plugin,
+                               base::OnceClosure completion_callback) {
   // Stop tracking the old embedder's zoom level.
   if (owner_web_contents())
     StopTrackingEmbedderZoomLevel();
@@ -529,7 +535,7 @@ void GuestViewBase::WillAttach(WebContents* embedder_web_contents,
     DCHECK_EQ(owner_contents_observer_->web_contents(), owner_web_contents_);
     owner_web_contents_ = embedder_web_contents;
     owner_contents_observer_ =
-        base::MakeUnique<OwnerContentsObserver>(this, embedder_web_contents);
+        std::make_unique<OwnerContentsObserver>(this, embedder_web_contents);
     SetOwnerHost();
   }
 
@@ -541,15 +547,27 @@ void GuestViewBase::WillAttach(WebContents* embedder_web_contents,
 
   WillAttachToEmbedder();
 
+  if (content::GuestMode::IsCrossProcessFrameGuest(web_contents())) {
+    web_contents()->AttachToOuterWebContentsFrame(
+        base::WrapUnique<WebContents>(web_contents()), outer_contents_frame);
+    // TODO(ekaramad): MimeHandlerViewGuest might not need this ACK
+    // (https://crbug.com/659750).
+    // We don't ACK until after AttachToOuterWebContentsFrame, so that
+    // |outer_contents_frame| gets swapped before the AttachIframeGuest callback
+    // is run. We also need to send the ACK before queued events are sent in
+    // DidAttach.
+    embedder_web_contents->GetMainFrame()->Send(
+        new GuestViewMsg_AttachToEmbedderFrame_ACK(element_instance_id));
+  }
   // Completing attachment will resume suspended resource loads and then send
   // queued events.
-  SignalWhenReady(callback);
+  SignalWhenReady(std::move(completion_callback));
 }
 
-void GuestViewBase::SignalWhenReady(const base::Closure& callback) {
+void GuestViewBase::SignalWhenReady(base::OnceClosure callback) {
   // The default behavior is to call the |callback| immediately. Derived classes
   // can implement an alternative signal for readiness.
-  callback.Run();
+  std::move(callback).Run();
 }
 
 int GuestViewBase::LogicalPixelsToPhysicalPixels(double logical_pixels) const {
@@ -608,31 +626,30 @@ void GuestViewBase::ActivateContents(WebContents* web_contents) {
 }
 
 void GuestViewBase::ContentsMouseEvent(WebContents* source,
-                                       const gfx::Point& location,
                                        bool motion,
                                        bool exited) {
   if (!attached() || !embedder_web_contents()->GetDelegate())
     return;
 
   embedder_web_contents()->GetDelegate()->ContentsMouseEvent(
-      embedder_web_contents(), location, motion, exited);
+      embedder_web_contents(), motion, exited);
 }
 
 void GuestViewBase::ContentsZoomChange(bool zoom_in) {
-  zoom::PageZoom::Zoom(embedder_web_contents(), zoom_in
-                                                    ? content::PAGE_ZOOM_IN
-                                                    : content::PAGE_ZOOM_OUT);
+  if (!attached() || !embedder_web_contents()->GetDelegate())
+    return;
+  embedder_web_contents()->GetDelegate()->ContentsZoomChange(zoom_in);
 }
 
-void GuestViewBase::HandleKeyboardEvent(
+bool GuestViewBase::HandleKeyboardEvent(
     WebContents* source,
     const content::NativeWebKeyboardEvent& event) {
   if (!attached())
-    return;
+    return false;
 
   // Send the keyboard events back to the embedder to reprocess them.
-  embedder_web_contents()->GetDelegate()->
-      HandleKeyboardEvent(embedder_web_contents(), event);
+  return embedder_web_contents()->GetDelegate()->HandleKeyboardEvent(
+      embedder_web_contents(), event);
 }
 
 void GuestViewBase::LoadingStateChanged(WebContents* source,
@@ -647,7 +664,7 @@ void GuestViewBase::LoadingStateChanged(WebContents* source,
 content::ColorChooser* GuestViewBase::OpenColorChooser(
     WebContents* web_contents,
     SkColor color,
-    const std::vector<content::ColorSuggestion>& suggestions) {
+    const std::vector<blink::mojom::ColorSuggestionPtr>& suggestions) {
   if (!attached() || !embedder_web_contents()->GetDelegate())
     return nullptr;
 
@@ -657,16 +674,20 @@ content::ColorChooser* GuestViewBase::OpenColorChooser(
 
 void GuestViewBase::ResizeDueToAutoResize(WebContents* web_contents,
                                           const gfx::Size& new_size) {
-  guest_host_->GuestResizeDueToAutoResize(new_size);
+  UpdateGuestSize(new_size, auto_size_enabled_);
 }
 
-void GuestViewBase::RunFileChooser(content::RenderFrameHost* render_frame_host,
-                                   const content::FileChooserParams& params) {
-  if (!attached() || !embedder_web_contents()->GetDelegate())
+void GuestViewBase::RunFileChooser(
+    content::RenderFrameHost* render_frame_host,
+    std::unique_ptr<content::FileSelectListener> listener,
+    const blink::mojom::FileChooserParams& params) {
+  if (!attached() || !embedder_web_contents()->GetDelegate()) {
+    listener->FileSelectionCanceled();
     return;
+  }
 
-  embedder_web_contents()->GetDelegate()->RunFileChooser(render_frame_host,
-                                                         params);
+  embedder_web_contents()->GetDelegate()->RunFileChooser(
+      render_frame_host, std::move(listener), params);
 }
 
 bool GuestViewBase::ShouldFocusPageAfterCrash() {
@@ -676,7 +697,13 @@ bool GuestViewBase::ShouldFocusPageAfterCrash() {
 
 bool GuestViewBase::PreHandleGestureEvent(WebContents* source,
                                           const blink::WebGestureEvent& event) {
-  return blink::WebInputEvent::IsPinchGestureEventType(event.GetType());
+  // Pinch events which cause a scale change should not be routed to a guest.
+  // We still allow synthetic wheel events for touchpad pinch to go to the page.
+  DCHECK(!blink::WebInputEvent::IsPinchGestureEventType(event.GetType()) ||
+         (event.SourceDevice() ==
+              blink::WebGestureDevice::kWebGestureDeviceTouchpad &&
+          event.NeedsWheelEvent()));
+  return false;
 }
 
 void GuestViewBase::UpdatePreferredSize(WebContents* target_web_contents,
@@ -725,6 +752,18 @@ content::SiteInstance* GuestViewBase::GetOwnerSiteInstance() {
   return nullptr;
 }
 
+void GuestViewBase::AttachToOuterWebContentsFrame(
+    content::RenderFrameHost* embedder_frame,
+    int32_t element_instance_id,
+    bool is_full_page_plugin) {
+  auto completion_callback =
+      base::BindOnce(&GuestViewBase::DidAttach, weak_ptr_factory_.GetWeakPtr(),
+                     MSG_ROUTING_NONE);
+  WillAttach(WebContents::FromRenderFrameHost(embedder_frame), embedder_frame,
+             element_instance_id, is_full_page_plugin,
+             std::move(completion_callback));
+}
+
 void GuestViewBase::OnZoomChanged(
     const zoom::ZoomController::ZoomChangedEventData& data) {
   if (data.web_contents == embedder_web_contents()) {
@@ -775,17 +814,17 @@ void GuestViewBase::SendQueuedEvents() {
 
 void GuestViewBase::CompleteInit(
     std::unique_ptr<base::DictionaryValue> create_params,
-    const WebContentsCreatedCallback& callback,
+    WebContentsCreatedCallback callback,
     WebContents* guest_web_contents) {
   if (!guest_web_contents) {
     // The derived class did not create a WebContents so this class serves no
     // purpose. Let's self-destruct.
     delete this;
-    callback.Run(nullptr);
+    std::move(callback).Run(nullptr);
     return;
   }
   InitWithWebContents(*create_params, guest_web_contents);
-  callback.Run(guest_web_contents);
+  std::move(callback).Run(guest_web_contents);
 }
 
 double GuestViewBase::GetEmbedderZoomFactor() const {
@@ -872,14 +911,16 @@ void GuestViewBase::StartTrackingEmbedderZoomLevel() {
 }
 
 void GuestViewBase::StopTrackingEmbedderZoomLevel() {
-  if (!attached() || !ZoomPropagatesFromEmbedderToGuest())
-    return;
+  // TODO(wjmaclean): Remove the observer any time the GuestWebView transitions
+  // from propagating to not-propagating the zoom from the embedder.
 
   auto* embedder_zoom_controller =
       zoom::ZoomController::FromWebContents(owner_web_contents());
   // Chrome Apps do not have a ZoomController.
   if (!embedder_zoom_controller)
     return;
+
+  // It is safe to remove an observer that was never registed.
   embedder_zoom_controller->RemoveObserver(this);
 }
 
