@@ -11,6 +11,7 @@
 #include <utility>
 
 #include "base/bind.h"
+#include "base/bits.h"
 #include "base/debug/alias.h"
 #include "base/files/file_util.h"
 #include "base/memory/ref_counted_memory.h"
@@ -50,7 +51,7 @@ namespace history {
 //   id               Unique ID.
 //   url              The URL at which the favicon file is located.
 //   icon_type        The type of the favicon specified in the rel attribute of
-//                    the link tag. The FAVICON type is used for the default
+//                    the link tag. The kFavicon type is used for the default
 //                    favicon.ico favicon.
 //
 // favicon_bitmaps    This table contains the PNG encoded bitmap data of the
@@ -106,7 +107,7 @@ void FillIconMapping(const sql::Statement& statement,
   icon_mapping->mapping_id = statement.ColumnInt64(0);
   icon_mapping->icon_id = statement.ColumnInt64(1);
   icon_mapping->icon_type =
-      static_cast<favicon_base::IconType>(statement.ColumnInt(2));
+      ThumbnailDatabase::FromPersistedIconType(statement.ColumnInt(2));
   icon_mapping->icon_url = GURL(statement.ColumnString(3));
   icon_mapping->page_url = page_url;
 }
@@ -127,9 +128,9 @@ void RecordInvalidStructure(InvalidStructureType invalid_type) {
                             invalid_type, STRUCTURE_EVENT_MAX);
 }
 
-// TODO(shess): If this proves out, move it all into sql::Connection to be
+// TODO(shess): If this proves out, move it all into sql::Database to be
 // shared.
-void GenerateDiagnostics(sql::Connection* db,
+void GenerateDiagnostics(sql::Database* db,
                          int extended_error,
                          sql::Statement* stmt) {
   // Since some/most errors will not resolve themselves, only report
@@ -151,8 +152,8 @@ void GenerateDiagnostics(sql::Connection* db,
 
 // NOTE(shess): Schema modifications must consider initial creation in
 // |InitImpl()| and history pruning in |RetainDataForPageUrls()|.
-bool InitTables(sql::Connection* db) {
-  const char kIconMappingSql[] =
+bool InitTables(sql::Database* db) {
+  static const char kIconMappingSql[] =
       "CREATE TABLE IF NOT EXISTS icon_mapping"
       "("
       "id INTEGER PRIMARY KEY,"
@@ -162,18 +163,18 @@ bool InitTables(sql::Connection* db) {
   if (!db->Execute(kIconMappingSql))
     return false;
 
-  const char kFaviconsSql[] =
+  static const char kFaviconsSql[] =
       "CREATE TABLE IF NOT EXISTS favicons"
       "("
       "id INTEGER PRIMARY KEY,"
       "url LONGVARCHAR NOT NULL,"
-      // default icon_type FAVICON to be consistent with past migration.
+      // default icon_type kFavicon to be consistent with past migration.
       "icon_type INTEGER DEFAULT 1"
       ")";
   if (!db->Execute(kFaviconsSql))
     return false;
 
-  const char kFaviconBitmapsSql[] =
+  static const char kFaviconBitmapsSql[] =
       "CREATE TABLE IF NOT EXISTS favicon_bitmaps"
       "("
       "id INTEGER PRIMARY KEY,"
@@ -194,11 +195,11 @@ bool InitTables(sql::Connection* db) {
 
 // NOTE(shess): Schema modifications must consider initial creation in
 // |InitImpl()| and history pruning in |RetainDataForPageUrls()|.
-bool InitIndices(sql::Connection* db) {
-  const char kIconMappingUrlIndexSql[] =
+bool InitIndices(sql::Database* db) {
+  static const char kIconMappingUrlIndexSql[] =
       "CREATE INDEX IF NOT EXISTS icon_mapping_page_url_idx"
       " ON icon_mapping(page_url)";
-  const char kIconMappingIdIndexSql[] =
+  static const char kIconMappingIdIndexSql[] =
       "CREATE INDEX IF NOT EXISTS icon_mapping_icon_id_idx"
       " ON icon_mapping(icon_id)";
   if (!db->Execute(kIconMappingUrlIndexSql) ||
@@ -206,12 +207,12 @@ bool InitIndices(sql::Connection* db) {
     return false;
   }
 
-  const char kFaviconsIndexSql[] =
+  static const char kFaviconsIndexSql[] =
       "CREATE INDEX IF NOT EXISTS favicons_url ON favicons(url)";
   if (!db->Execute(kFaviconsIndexSql))
     return false;
 
-  const char kFaviconBitmapsIndexSql[] =
+  static const char kFaviconBitmapsIndexSql[] =
       "CREATE INDEX IF NOT EXISTS favicon_bitmaps_icon_id ON "
       "favicon_bitmaps(icon_id)";
   if (!db->Execute(kFaviconBitmapsIndexSql))
@@ -220,7 +221,7 @@ bool InitIndices(sql::Connection* db) {
   return true;
 }
 
-void DatabaseErrorCallback(sql::Connection* db,
+void DatabaseErrorCallback(sql::Database* db,
                            const base::FilePath& db_path,
                            HistoryBackendClient* backend_client,
                            int extended_error,
@@ -262,12 +263,12 @@ void DatabaseErrorCallback(sql::Connection* db,
     // or hardware issues, not coding errors at the client level, so displaying
     // the error would probably lead to confusion.  The ignored call signals the
     // test-expectation framework that the error was handled.
-    ignore_result(sql::Connection::IsExpectedSqliteError(extended_error));
+    ignore_result(sql::Database::IsExpectedSqliteError(extended_error));
     return;
   }
 
   // The default handling is to assert on debug and to ignore on release.
-  if (!sql::Connection::IsExpectedSqliteError(extended_error))
+  if (!sql::Database::IsExpectedSqliteError(extended_error))
     DLOG(FATAL) << db->GetErrorMessage();
 }
 
@@ -357,8 +358,10 @@ void ThumbnailDatabase::ComputeDatabaseMetrics() {
         db_.GetCachedStatement(
             SQL_FROM_HERE,
             "SELECT COUNT(*) FROM favicons WHERE icon_type IN (?, ?)"));
-    touch_icon_count.BindInt64(0, favicon_base::TOUCH_ICON);
-    touch_icon_count.BindInt64(1, favicon_base::TOUCH_PRECOMPOSED_ICON);
+    touch_icon_count.BindInt64(
+        0, ToPersistedIconType(favicon_base::IconType::kTouchIcon));
+    touch_icon_count.BindInt64(
+        1, ToPersistedIconType(favicon_base::IconType::kTouchPrecomposedIcon));
     UMA_HISTOGRAM_COUNTS_10000(
         "History.NumTouchIconsInDB",
         touch_icon_count.Step() ? touch_icon_count.ColumnInt(0) : 0);
@@ -408,15 +411,41 @@ void ThumbnailDatabase::Vacuum() {
   ignore_result(db_.Execute("VACUUM"));
 }
 
-void ThumbnailDatabase::TrimMemory(bool aggressively) {
-  db_.TrimMemory(aggressively);
+void ThumbnailDatabase::TrimMemory() {
+  db_.TrimMemory();
+}
+
+std::map<favicon_base::FaviconID, IconMappingsForExpiry>
+ThumbnailDatabase::GetOldOnDemandFavicons(base::Time threshold) {
+  // Restrict to on-demand bitmaps (i.e. with last_requested != 0).
+  // This is called rarely during history expiration cleanup and hence not worth
+  // caching.
+  sql::Statement old_icons(db_.GetUniqueStatement(
+      "SELECT favicons.id, favicons.url, icon_mapping.page_url "
+      "FROM favicons "
+      "JOIN favicon_bitmaps ON (favicon_bitmaps.icon_id = favicons.id) "
+      "JOIN icon_mapping ON (icon_mapping.icon_id = favicon_bitmaps.icon_id) "
+      "WHERE (favicon_bitmaps.last_requested > 0 AND "
+      "       favicon_bitmaps.last_requested < ?)"));
+  old_icons.BindInt64(0, threshold.ToDeltaSinceWindowsEpoch().InMicroseconds());
+
+  std::map<favicon_base::FaviconID, IconMappingsForExpiry> icon_mappings;
+
+  while (old_icons.Step()) {
+    favicon_base::FaviconID id = old_icons.ColumnInt64(0);
+    icon_mappings[id].icon_url = GURL(old_icons.ColumnString(1));
+    icon_mappings[id].page_urls.push_back(GURL(old_icons.ColumnString(2)));
+  }
+
+  return icon_mappings;
 }
 
 bool ThumbnailDatabase::GetFaviconBitmapIDSizes(
     favicon_base::FaviconID icon_id,
     std::vector<FaviconBitmapIDSize>* bitmap_id_sizes) {
   DCHECK(icon_id);
-  sql::Statement statement(db_.GetCachedStatement(SQL_FROM_HERE,
+  sql::Statement statement(db_.GetCachedStatement(
+      SQL_FROM_HERE,
       "SELECT id, width, height FROM favicon_bitmaps WHERE icon_id=?"));
   statement.BindInt64(0, icon_id);
 
@@ -453,8 +482,8 @@ bool ThumbnailDatabase::GetFaviconBitmaps(
     FaviconBitmap favicon_bitmap;
     favicon_bitmap.bitmap_id = statement.ColumnInt64(0);
     favicon_bitmap.icon_id = icon_id;
-    favicon_bitmap.last_updated =
-        base::Time::FromInternalValue(statement.ColumnInt64(1));
+    favicon_bitmap.last_updated = base::Time::FromDeltaSinceWindowsEpoch(
+        base::TimeDelta::FromMicroseconds(statement.ColumnInt64(1)));
     if (statement.ColumnByteLength(2) > 0) {
       scoped_refptr<base::RefCountedBytes> data(new base::RefCountedBytes());
       statement.ColumnBlobAsVector(2, &data->data());
@@ -462,8 +491,8 @@ bool ThumbnailDatabase::GetFaviconBitmaps(
     }
     favicon_bitmap.pixel_size = gfx::Size(statement.ColumnInt(3),
                                           statement.ColumnInt(4));
-    favicon_bitmap.last_requested =
-        base::Time::FromInternalValue(statement.ColumnInt64(5));
+    favicon_bitmap.last_requested = base::Time::FromDeltaSinceWindowsEpoch(
+        base::TimeDelta::FromMicroseconds(statement.ColumnInt64(5)));
     favicon_bitmaps->push_back(favicon_bitmap);
   }
   return result;
@@ -484,8 +513,10 @@ bool ThumbnailDatabase::GetFaviconBitmap(
   if (!statement.Step())
     return false;
 
-  if (last_updated)
-    *last_updated = base::Time::FromInternalValue(statement.ColumnInt64(0));
+  if (last_updated) {
+    *last_updated = base::Time::FromDeltaSinceWindowsEpoch(
+        base::TimeDelta::FromMicroseconds(statement.ColumnInt64(0)));
+  }
 
   if (png_icon_data && statement.ColumnByteLength(1) > 0) {
     scoped_refptr<base::RefCountedBytes> data(new base::RefCountedBytes());
@@ -498,8 +529,10 @@ bool ThumbnailDatabase::GetFaviconBitmap(
                             statement.ColumnInt(3));
   }
 
-  if (last_requested)
-    *last_requested = base::Time::FromInternalValue(statement.ColumnInt64(4));
+  if (last_requested) {
+    *last_requested = base::Time::FromDeltaSinceWindowsEpoch(
+        base::TimeDelta::FromMicroseconds(statement.ColumnInt64(4)));
+  }
 
   return true;
 }
@@ -528,12 +561,16 @@ FaviconBitmapID ThumbnailDatabase::AddFaviconBitmap(
   // On-visit bitmaps:
   //  - keep track of last_updated: last write time is used for expiration;
   //  - always have last_requested==0: no need to keep track of last read time.
-  statement.BindInt64(2, type == ON_VISIT ? time.ToInternalValue() : 0);
+  statement.BindInt64(2, type == ON_VISIT
+                             ? time.ToDeltaSinceWindowsEpoch().InMicroseconds()
+                             : 0);
   // On-demand bitmaps:
   //  - always have last_updated==0: last write time is not stored as they are
   //    always expired and thus ready to be replaced by ON_VISIT icons;
   //  - keep track of last_requested: last read time is used for cache eviction.
-  statement.BindInt64(3, type == ON_DEMAND ? time.ToInternalValue() : 0);
+  statement.BindInt64(3, type == ON_DEMAND
+                             ? time.ToDeltaSinceWindowsEpoch().InMicroseconds()
+                             : 0);
 
   statement.BindInt(4, pixel_size.width());
   statement.BindInt(5, pixel_size.height());
@@ -561,7 +598,7 @@ bool ThumbnailDatabase::SetFaviconBitmap(
   } else {
     statement.BindNull(0);
   }
-  statement.BindInt64(1, time.ToInternalValue());
+  statement.BindInt64(1, time.ToDeltaSinceWindowsEpoch().InMicroseconds());
   statement.BindInt64(2, 0);
   statement.BindInt64(3, bitmap_id);
 
@@ -579,7 +616,7 @@ bool ThumbnailDatabase::SetFaviconBitmapLastUpdateTime(
       db_.GetCachedStatement(SQL_FROM_HERE,
                              "UPDATE favicon_bitmaps SET last_updated=?, "
                              "last_requested=? WHERE id=?"));
-  statement.BindInt64(0, time.ToInternalValue());
+  statement.BindInt64(0, time.ToDeltaSinceWindowsEpoch().InMicroseconds());
   statement.BindInt64(1, 0);
   statement.BindInt64(2, bitmap_id);
   return statement.Run();
@@ -606,9 +643,10 @@ bool ThumbnailDatabase::TouchOnDemandFavicon(const GURL& icon_url,
         SQL_FROM_HERE,
         "UPDATE favicon_bitmaps SET last_requested=? WHERE icon_id=? AND "
         "last_requested>0 AND last_requested<=?"));
-    statement.BindInt64(0, time.ToInternalValue());
+    statement.BindInt64(0, time.ToDeltaSinceWindowsEpoch().InMicroseconds());
     statement.BindInt64(1, icon_id);
-    statement.BindInt64(2, max_time.ToInternalValue());
+    statement.BindInt64(2,
+                        max_time.ToDeltaSinceWindowsEpoch().InMicroseconds());
     if (!statement.Run())
       return false;
   }
@@ -631,13 +669,35 @@ bool ThumbnailDatabase::SetFaviconOutOfDate(favicon_base::FaviconID icon_id) {
   return statement.Run();
 }
 
+bool ThumbnailDatabase::GetFaviconLastUpdatedTime(
+    favicon_base::FaviconID icon_id,
+    base::Time* last_updated) {
+  sql::Statement statement(db_.GetCachedStatement(
+      SQL_FROM_HERE,
+      "SELECT MAX(last_updated) FROM favicon_bitmaps WHERE icon_id=?"));
+  statement.BindInt64(0, icon_id);
+
+  if (!statement.Step())
+    return false;
+
+  // Return false also if there there is no bitmap with |icon_id|.
+  if (statement.ColumnType(0) == sql::COLUMN_TYPE_NULL)
+    return false;
+
+  if (last_updated) {
+    *last_updated = base::Time::FromDeltaSinceWindowsEpoch(
+        base::TimeDelta::FromMicroseconds(statement.ColumnInt64(0)));
+  }
+  return true;
+}
+
 favicon_base::FaviconID ThumbnailDatabase::GetFaviconIDForFaviconURL(
     const GURL& icon_url,
     favicon_base::IconType icon_type) {
   sql::Statement statement(db_.GetCachedStatement(
       SQL_FROM_HERE, "SELECT id FROM favicons WHERE url=? AND icon_type=?"));
   statement.BindString(0, URLDatabase::GURLToDatabaseURL(icon_url));
-  statement.BindInt(1, icon_type);
+  statement.BindInt(1, ToPersistedIconType(icon_type));
 
   if (!statement.Step())
     return 0;  // not cached
@@ -660,7 +720,7 @@ bool ThumbnailDatabase::GetFaviconHeader(favicon_base::FaviconID icon_id,
   if (icon_url)
     *icon_url = GURL(statement.ColumnString(0));
   if (icon_type)
-    *icon_type = static_cast<favicon_base::IconType>(statement.ColumnInt(1));
+    *icon_type = FromPersistedIconType(statement.ColumnInt(1));
 
   return true;
 }
@@ -672,7 +732,7 @@ favicon_base::FaviconID ThumbnailDatabase::AddFavicon(
   sql::Statement statement(db_.GetCachedStatement(SQL_FROM_HERE,
       "INSERT INTO favicons (url, icon_type) VALUES (?, ?)"));
   statement.BindString(0, URLDatabase::GURLToDatabaseURL(icon_url));
-  statement.BindInt(1, icon_type);
+  statement.BindInt(1, ToPersistedIconType(icon_type));
 
   if (!statement.Run())
     return 0;
@@ -709,16 +769,15 @@ bool ThumbnailDatabase::DeleteFavicon(favicon_base::FaviconID id) {
 
 bool ThumbnailDatabase::GetIconMappingsForPageURL(
     const GURL& page_url,
-    int required_icon_types,
+    const favicon_base::IconTypeSet& required_icon_types,
     std::vector<IconMapping>* filtered_mapping_data) {
   std::vector<IconMapping> mapping_data;
   if (!GetIconMappingsForPageURL(page_url, &mapping_data))
     return false;
 
   bool result = false;
-  for (std::vector<IconMapping>::iterator m = mapping_data.begin();
-       m != mapping_data.end(); ++m) {
-    if (m->icon_type & required_icon_types) {
+  for (auto m = mapping_data.begin(); m != mapping_data.end(); ++m) {
+    if (required_icon_types.count(m->icon_type) != 0) {
       result = true;
       if (!filtered_mapping_data)
         return result;
@@ -755,10 +814,39 @@ bool ThumbnailDatabase::GetIconMappingsForPageURL(
   return result;
 }
 
+base::Optional<GURL> ThumbnailDatabase::FindFirstPageURLForHost(
+    const GURL& url,
+    const favicon_base::IconTypeSet& required_icon_types) {
+  if (url.host().empty())
+    return base::nullopt;
+
+  sql::Statement statement(
+      db_.GetCachedStatement(SQL_FROM_HERE,
+                             "SELECT icon_mapping.page_url, favicons.icon_type "
+                             "FROM icon_mapping "
+                             "INNER JOIN favicons "
+                             "ON icon_mapping.icon_id = favicons.id "
+                             "WHERE icon_mapping.page_url LIKE ? "
+                             "ORDER BY favicons.icon_type DESC"));
+
+  // Bind the host with a prefix of "://" and suffix of "/" to ensure the entire
+  // host name is matched.
+  statement.BindString(0, base::StringPrintf("%%://%s/%%", url.host().c_str()));
+
+  while (statement.Step()) {
+    favicon_base::IconType icon_type =
+        ThumbnailDatabase::FromPersistedIconType(statement.ColumnInt(1));
+
+    if (required_icon_types.count(icon_type) != 0)
+      return base::make_optional(GURL(statement.ColumnString(0)));
+  }
+  return base::nullopt;
+}
+
 IconMappingID ThumbnailDatabase::AddIconMapping(
     const GURL& page_url,
     favicon_base::FaviconID icon_id) {
-  const char kSql[] =
+  static const char kSql[] =
       "INSERT INTO icon_mapping (page_url, icon_id) VALUES (?, ?)";
   sql::Statement statement(db_.GetCachedStatement(SQL_FROM_HERE, kSql));
   statement.BindString(0, URLDatabase::GURLToDatabaseURL(page_url));
@@ -778,6 +866,16 @@ bool ThumbnailDatabase::DeleteIconMappings(const GURL& page_url) {
   return statement.Run();
 }
 
+bool ThumbnailDatabase::DeleteIconMappingsForFaviconId(
+    favicon_base::FaviconID id) {
+  // This is called rarely during history expiration cleanup and hence not
+  // worth caching.
+  sql::Statement statement(
+      db_.GetUniqueStatement("DELETE FROM icon_mapping WHERE icon_id=?"));
+  statement.BindInt64(0, id);
+  return statement.Run();
+}
+
 bool ThumbnailDatabase::DeleteIconMapping(IconMappingID mapping_id) {
   sql::Statement statement(db_.GetCachedStatement(SQL_FROM_HERE,
       "DELETE FROM icon_mapping WHERE id=?"));
@@ -806,7 +904,7 @@ bool ThumbnailDatabase::InitIconMappingEnumerator(
          "FROM icon_mapping JOIN favicons ON ("
               "icon_mapping.icon_id = favicons.id) "
          "WHERE favicons.icon_type = ?"));
-  enumerator->statement_.BindInt(0, type);
+  enumerator->statement_.BindInt(0, ToPersistedIconType(type));
   return enumerator->statement_.is_valid();
 }
 
@@ -818,12 +916,12 @@ bool ThumbnailDatabase::RetainDataForPageUrls(
 
   // Populate temp.retained_urls with |urls_to_keep|.
   {
-    const char kCreateRetainedUrls[] =
+    static const char kCreateRetainedUrls[] =
         "CREATE TEMP TABLE retained_urls (url LONGVARCHAR PRIMARY KEY)";
     if (!db_.Execute(kCreateRetainedUrls))
       return false;
 
-    const char kRetainedUrlSql[] =
+    static const char kRetainedUrlSql[] =
         "INSERT OR IGNORE INTO temp.retained_urls (url) VALUES (?)";
     sql::Statement statement(db_.GetUniqueStatement(kRetainedUrlSql));
     for (const GURL& url : urls_to_keep) {
@@ -837,7 +935,7 @@ bool ThumbnailDatabase::RetainDataForPageUrls(
   // temp.icon_id_mapping generates new icon ids as consecutive
   // integers starting from 1, and maps them to the old icon ids.
   {
-    const char kIconMappingCreate[] =
+    static const char kIconMappingCreate[] =
         "CREATE TEMP TABLE icon_id_mapping "
         "("
         "new_icon_id INTEGER PRIMARY KEY,"
@@ -847,7 +945,7 @@ bool ThumbnailDatabase::RetainDataForPageUrls(
       return false;
 
     // Insert the icon ids for retained urls, skipping duplicates.
-    const char kIconMappingSql[] =
+    static const char kIconMappingSql[] =
         "INSERT OR IGNORE INTO temp.icon_id_mapping (old_icon_id) "
         "SELECT icon_id FROM icon_mapping "
         "JOIN temp.retained_urls "
@@ -856,9 +954,9 @@ bool ThumbnailDatabase::RetainDataForPageUrls(
       return false;
   }
 
-  const char kRenameIconMappingTable[] =
+  static const char kRenameIconMappingTable[] =
       "ALTER TABLE icon_mapping RENAME TO old_icon_mapping";
-  const char kCopyIconMapping[] =
+  static const char kCopyIconMapping[] =
       "INSERT INTO icon_mapping (page_url, icon_id) "
       "SELECT temp.retained_urls.url, mapping.new_icon_id "
       "FROM temp.retained_urls "
@@ -866,17 +964,17 @@ bool ThumbnailDatabase::RetainDataForPageUrls(
       "ON (temp.retained_urls.url = old.page_url) "
       "JOIN temp.icon_id_mapping AS mapping "
       "ON (old.icon_id = mapping.old_icon_id)";
-  const char kDropOldIconMappingTable[] = "DROP TABLE old_icon_mapping";
+  static const char kDropOldIconMappingTable[] = "DROP TABLE old_icon_mapping";
 
-  const char kRenameFaviconsTable[] =
+  static const char kRenameFaviconsTable[] =
       "ALTER TABLE favicons RENAME TO old_favicons";
-  const char kCopyFavicons[] =
+  static const char kCopyFavicons[] =
       "INSERT INTO favicons (id, url, icon_type) "
       "SELECT mapping.new_icon_id, old.url, old.icon_type "
       "FROM old_favicons AS old "
       "JOIN temp.icon_id_mapping AS mapping "
       "ON (old.id = mapping.old_icon_id)";
-  const char kDropOldFaviconsTable[] = "DROP TABLE old_favicons";
+  static const char kDropOldFaviconsTable[] = "DROP TABLE old_favicons";
 
   // Set the retained favicon bitmaps to be expired (last_updated == 0).
   // The user may be deleting their favicon bitmaps because the favicon bitmaps
@@ -884,9 +982,9 @@ bool ThumbnailDatabase::RetainDataForPageUrls(
   // the user visits a page associated with the favicon bitmap. See
   // crbug.com/474421 for an example of a bug which caused favicon bitmaps to
   // become incorrect.
-  const char kRenameFaviconBitmapsTable[] =
+  static const char kRenameFaviconBitmapsTable[] =
       "ALTER TABLE favicon_bitmaps RENAME TO old_favicon_bitmaps";
-  const char kCopyFaviconBitmaps[] =
+  static const char kCopyFaviconBitmaps[] =
       "INSERT INTO favicon_bitmaps "
       "  (icon_id, last_updated, image_data, width, height, last_requested) "
       "SELECT mapping.new_icon_id, 0, old.image_data, old.width, old.height,"
@@ -894,7 +992,7 @@ bool ThumbnailDatabase::RetainDataForPageUrls(
       "FROM old_favicon_bitmaps AS old "
       "JOIN temp.icon_id_mapping AS mapping "
       "ON (old.icon_id = mapping.old_icon_id)";
-  const char kDropOldFaviconBitmapsTable[] =
+  static const char kDropOldFaviconBitmapsTable[] =
       "DROP TABLE old_favicon_bitmaps";
 
   // Rename existing tables to new location.
@@ -930,15 +1028,35 @@ bool ThumbnailDatabase::RetainDataForPageUrls(
   if (!InitIndices(&db_))
     return false;
 
-  const char kIconMappingDrop[] = "DROP TABLE temp.icon_id_mapping";
-  const char kRetainedUrlsDrop[] = "DROP TABLE temp.retained_urls";
+  static const char kIconMappingDrop[] = "DROP TABLE temp.icon_id_mapping";
+  static const char kRetainedUrlsDrop[] = "DROP TABLE temp.retained_urls";
   if (!db_.Execute(kIconMappingDrop) || !db_.Execute(kRetainedUrlsDrop))
     return false;
 
   return transaction.Commit();
 }
 
-sql::InitStatus ThumbnailDatabase::OpenDatabase(sql::Connection* db,
+// static
+int ThumbnailDatabase::ToPersistedIconType(favicon_base::IconType icon_type) {
+  if (icon_type == favicon_base::IconType::kInvalid)
+    return 0;
+
+  return 1 << (static_cast<int>(icon_type) - 1);
+}
+
+// static
+favicon_base::IconType ThumbnailDatabase::FromPersistedIconType(int icon_type) {
+  if (icon_type == 0)
+    return favicon_base::IconType::kInvalid;
+
+  int val = 1 + base::bits::Log2Floor(icon_type);
+  if (val > static_cast<int>(favicon_base::IconType::kMax))
+    return favicon_base::IconType::kInvalid;
+
+  return static_cast<favicon_base::IconType>(val);
+}
+
+sql::InitStatus ThumbnailDatabase::OpenDatabase(sql::Database* db,
                                                 const base::FilePath& db_name) {
   db->set_histogram_tag("Thumbnail");
   db->set_error_callback(base::Bind(&DatabaseErrorCallback,
@@ -1078,14 +1196,16 @@ sql::InitStatus ThumbnailDatabase::CantUpgradeToVersion(int cur_version) {
 bool ThumbnailDatabase::UpgradeToVersion7() {
   // Sizes column was never used, remove it.
   bool success =
-      db_.Execute("CREATE TABLE temp_favicons ("
-                  "id INTEGER PRIMARY KEY,"
-                  "url LONGVARCHAR NOT NULL,"
-                  // default icon_type FAVICON to be consistent with
-                  // past migration.
-                  "icon_type INTEGER DEFAULT 1)") &&
-      db_.Execute("INSERT INTO temp_favicons (id, url, icon_type) "
-                  "SELECT id, url, icon_type FROM favicons") &&
+      db_.Execute(
+          "CREATE TABLE temp_favicons ("
+          "id INTEGER PRIMARY KEY,"
+          "url LONGVARCHAR NOT NULL,"
+          // default icon_type kFavicon to be consistent with
+          // past migration.
+          "icon_type INTEGER DEFAULT 1)") &&
+      db_.Execute(
+          "INSERT INTO temp_favicons (id, url, icon_type) "
+          "SELECT id, url, icon_type FROM favicons") &&
       db_.Execute("DROP TABLE favicons") &&
       db_.Execute("ALTER TABLE temp_favicons RENAME TO favicons") &&
       db_.Execute("CREATE INDEX IF NOT EXISTS favicons_url ON favicons(url)");
@@ -1100,7 +1220,7 @@ bool ThumbnailDatabase::UpgradeToVersion7() {
 
 bool ThumbnailDatabase::UpgradeToVersion8() {
   // Add the last_requested column to the favicon_bitmaps table.
-  const char kFaviconBitmapsAddLastRequestedSql[] =
+  static const char kFaviconBitmapsAddLastRequestedSql[] =
       "ALTER TABLE favicon_bitmaps ADD COLUMN last_requested INTEGER DEFAULT 0";
   if (!db_.Execute(kFaviconBitmapsAddLastRequestedSql))
     return false;
