@@ -4,21 +4,26 @@
 
 #include "components/metrics/file_metrics_provider.h"
 
+#include <memory>
+
 #include "base/command_line.h"
+#include "base/containers/flat_map.h"
+#include "base/feature_list.h"
 #include "base/files/file.h"
 #include "base/files/file_enumerator.h"
 #include "base/files/file_util.h"
 #include "base/files/memory_mapped_file.h"
 #include "base/logging.h"
-#include "base/memory/ptr_util.h"
 #include "base/metrics/histogram_base.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/metrics/persistent_histogram_allocator.h"
 #include "base/metrics/persistent_memory_allocator.h"
+#include "base/stl_util.h"
 #include "base/strings/string_piece.h"
+#include "base/task/post_task.h"
+#include "base/task/task_traits.h"
 #include "base/task_runner.h"
-#include "base/task_scheduler/post_task.h"
-#include "base/task_scheduler/task_traits.h"
+#include "base/task_runner_util.h"
 #include "base/time/time.h"
 #include "components/metrics/metrics_pref_names.h"
 #include "components/metrics/metrics_service.h"
@@ -30,6 +35,9 @@ namespace metrics {
 
 namespace {
 
+const base::Feature kBackgroundIndependentMetrics = {
+    "BackgoundIndependentMetrics", base::FEATURE_ENABLED_BY_DEFAULT};
+
 // These structures provide values used to define how files are opened and
 // accessed. It obviates the need for multiple code-paths within several of
 // the methods.
@@ -105,7 +113,7 @@ scoped_refptr<base::TaskRunner> CreateBackgroundTaskRunner() {
     return scoped_refptr<base::TaskRunner>(g_task_runner_for_testing);
 
   return base::CreateTaskRunnerWithTraits(
-      {base::MayBlock(), base::TaskPriority::BACKGROUND,
+      {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
        base::TaskShutdownBehavior::SKIP_ON_SHUTDOWN});
 }
 
@@ -114,10 +122,34 @@ scoped_refptr<base::TaskRunner> CreateBackgroundTaskRunner() {
 // This structure stores all the information about the sources being monitored
 // and their current reporting state.
 struct FileMetricsProvider::SourceInfo {
-  SourceInfo(SourceType source_type, SourceAssociation source_association)
-      : type(source_type), association(source_association) {}
+  SourceInfo(const Params& params)
+      : type(params.type),
+        association(params.association),
+        prefs_key(params.prefs_key),
+        filter(params.filter),
+        max_age(params.max_age),
+        max_dir_kib(params.max_dir_kib),
+        max_dir_files(params.max_dir_files) {
+    switch (type) {
+      case SOURCE_HISTOGRAMS_ACTIVE_FILE:
+        DCHECK(prefs_key.empty());
+        FALLTHROUGH;
+      case SOURCE_HISTOGRAMS_ATOMIC_FILE:
+        path = params.path;
+        break;
+      case SOURCE_HISTOGRAMS_ATOMIC_DIR:
+        directory = params.path;
+        break;
+    }
+  }
   ~SourceInfo() {}
 
+  struct FoundFile {
+    base::FilePath path;
+    base::FileEnumerator::FileInfo info;
+  };
+  using FoundFiles = base::flat_map<base::Time, FoundFile>;
+
   // How to access this source (file/dir, atomic/active).
   const SourceType type;
 
@@ -128,6 +160,9 @@ struct FileMetricsProvider::SourceInfo {
   // a directory is being monitored.
   base::FilePath directory;
 
+  // The files found in the above directory, ordered by last-modified.
+  std::unique_ptr<FoundFiles> found_files;
+
   // Where on disk the file is located. If a directory is being monitored,
   // this will be updated for whatever file is being read.
   base::FilePath path;
@@ -135,6 +170,18 @@ struct FileMetricsProvider::SourceInfo {
   // Name used inside prefs to persistent metadata.
   std::string prefs_key;
 
+  // The filter callback for determining what to do with found files.
+  FilterCallback filter;
+
+  // The maximum allowed age of a file.
+  base::TimeDelta max_age;
+
+  // The maximum allowed bytes in a directory.
+  size_t max_dir_kib;
+
+  // The maximum allowed files in a directory.
+  size_t max_dir_files;
+
   // The last-seen time of this source to detect change.
   base::Time last_seen;
 
@@ -149,6 +196,14 @@ struct FileMetricsProvider::SourceInfo {
   DISALLOW_COPY_AND_ASSIGN(SourceInfo);
 };
 
+FileMetricsProvider::Params::Params(const base::FilePath& path,
+                                    SourceType type,
+                                    SourceAssociation association,
+                                    base::StringPiece prefs_key)
+    : path(path), type(type), association(association), prefs_key(prefs_key) {}
+
+FileMetricsProvider::Params::~Params() {}
+
 FileMetricsProvider::FileMetricsProvider(PrefService* local_state)
     : task_runner_(CreateBackgroundTaskRunner()),
       pref_service_(local_state),
@@ -159,39 +214,23 @@ FileMetricsProvider::FileMetricsProvider(PrefService* local_state)
 
 FileMetricsProvider::~FileMetricsProvider() {}
 
-void FileMetricsProvider::RegisterSource(const base::FilePath& path,
-                                         SourceType type,
-                                         SourceAssociation source_association,
-                                         const base::StringPiece prefs_key) {
+void FileMetricsProvider::RegisterSource(const Params& params) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
   // Ensure that kSourceOptions has been filled for this type.
-  DCHECK_GT(arraysize(kSourceOptions), static_cast<size_t>(type));
+  DCHECK_GT(base::size(kSourceOptions), static_cast<size_t>(params.type));
 
-  std::unique_ptr<SourceInfo> source(new SourceInfo(type, source_association));
-  source->prefs_key = prefs_key.as_string();
-
-  switch (source->type) {
-    case SOURCE_HISTOGRAMS_ACTIVE_FILE:
-      DCHECK(prefs_key.empty());
-    // fall through
-    case SOURCE_HISTOGRAMS_ATOMIC_FILE:
-      source->path = path;
-      break;
-    case SOURCE_HISTOGRAMS_ATOMIC_DIR:
-      source->directory = path;
-      break;
-  }
+  std::unique_ptr<SourceInfo> source(new SourceInfo(params));
 
   // |prefs_key| may be empty if the caller does not wish to persist the
   // state across instances of the program.
-  if (pref_service_ && !prefs_key.empty()) {
+  if (pref_service_ && !params.prefs_key.empty()) {
     source->last_seen = base::Time::FromInternalValue(
         pref_service_->GetInt64(metrics::prefs::kMetricsLastSeenPrefix +
                                 source->prefs_key));
   }
 
-  switch (source_association) {
+  switch (params.association) {
     case ASSOCIATE_CURRENT_RUN:
     case ASSOCIATE_INTERNAL_PROFILE:
       sources_to_check_.push_back(std::move(source));
@@ -214,80 +253,115 @@ void FileMetricsProvider::RegisterPrefs(PrefRegistrySimple* prefs,
 // static
 void FileMetricsProvider::SetTaskRunnerForTesting(
     const scoped_refptr<base::TaskRunner>& task_runner) {
-  DCHECK(!g_task_runner_for_testing);
+  DCHECK(!g_task_runner_for_testing || !task_runner);
   g_task_runner_for_testing = task_runner.get();
 }
 
+// static
+void FileMetricsProvider::RecordAccessResult(AccessResult result) {
+  UMA_HISTOGRAM_ENUMERATION("UMA.FileMetricsProvider.AccessResult", result,
+                            ACCESS_RESULT_MAX);
+}
+
 // static
 bool FileMetricsProvider::LocateNextFileInDirectory(SourceInfo* source) {
   DCHECK_EQ(SOURCE_HISTOGRAMS_ATOMIC_DIR, source->type);
   DCHECK(!source->directory.empty());
 
-  // Open the directory and find all the files, remembering the oldest that
-  // has not been read. They can be removed and/or ignored if they're older
-  // than the last-check time.
-  base::Time oldest_file_time = base::Time::Now();
-  base::FilePath oldest_file_path;
-  base::FilePath file_path;
-  int file_count = 0;
-  int delete_count = 0;
-  base::FileEnumerator file_iter(source->directory, /*recursive=*/false,
-                                 base::FileEnumerator::FILES);
-  for (file_path = file_iter.Next(); !file_path.empty();
-       file_path = file_iter.Next()) {
-    base::FileEnumerator::FileInfo file_info = file_iter.GetInfo();
+  // Cumulative directory stats. These will remain zero if the directory isn't
+  // scanned but that's okay since any work they would cause to be done below
+  // would have been done during the first call where the directory was fully
+  // scanned.
+  size_t total_size_kib = 0;  // Using KiB allows 4TiB even on 32-bit builds.
+  size_t file_count = 0;
 
-    // Ignore directories and zero-sized files.
-    if (file_info.IsDirectory() || file_info.GetSize() == 0)
-      continue;
+  base::Time now_time = base::Time::Now();
+  if (!source->found_files) {
+    source->found_files = std::make_unique<SourceInfo::FoundFiles>();
+    base::FileEnumerator file_iter(source->directory, /*recursive=*/false,
+                                   base::FileEnumerator::FILES);
+    SourceInfo::FoundFile found_file;
 
-    // Ignore temporary files.
-    base::FilePath::CharType first_character =
-        file_path.BaseName().value().front();
-    if (first_character == FILE_PATH_LITERAL('.') ||
-        first_character == FILE_PATH_LITERAL('_')) {
-      continue;
-    }
+    // Open the directory and find all the files, remembering the last-modified
+    // time of each.
+    for (found_file.path = file_iter.Next(); !found_file.path.empty();
+         found_file.path = file_iter.Next()) {
+      found_file.info = file_iter.GetInfo();
 
-    // Ignore non-PMA (Persistent Memory Allocator) files.
-    if (file_path.Extension() !=
-        base::PersistentMemoryAllocator::kFileExtension) {
-      continue;
-    }
+      // Ignore directories.
+      if (found_file.info.IsDirectory())
+        continue;
 
-    // Process real files.
-    base::Time modified = file_info.GetLastModifiedTime();
-    if (modified > source->last_seen) {
-      // This file hasn't been read. Remember it if it is older than others.
-      if (modified < oldest_file_time) {
-        oldest_file_path = std::move(file_path);
-        oldest_file_time = modified;
+      // Ignore temporary files.
+      base::FilePath::CharType first_character =
+          found_file.path.BaseName().value().front();
+      if (first_character == FILE_PATH_LITERAL('.') ||
+          first_character == FILE_PATH_LITERAL('_')) {
+        continue;
+      }
+
+      // Ignore non-PMA (Persistent Memory Allocator) files.
+      if (found_file.path.Extension() !=
+          base::PersistentMemoryAllocator::kFileExtension) {
+        continue;
+      }
+
+      // Process real files.
+      total_size_kib += found_file.info.GetSize() >> 10;
+      base::Time modified = found_file.info.GetLastModifiedTime();
+      if (modified > source->last_seen) {
+        // This file hasn't been read. Remember it (unless from the future).
+        if (modified <= now_time)
+          source->found_files->emplace(modified, std::move(found_file));
+        ++file_count;
+      } else {
+        // This file has been read. Try to delete it. Ignore any errors because
+        // the file may be un-removeable by this process. It could, for example,
+        // have been created by a privileged process like setup.exe. Even if it
+        // is not removed, it will continue to be ignored bacuse of the older
+        // modification time.
+        base::DeleteFile(found_file.path, /*recursive=*/false);
       }
-      ++file_count;
-    } else {
-      // This file has been read. Try to delete it. Ignore any errors because
-      // the file may be un-removeable by this process. It could, for example,
-      // have been created by a privileged process like setup.exe. Even if it
-      // is not removed, it will continue to be ignored bacuse of the older
-      // modification time.
-      base::DeleteFile(file_path, /*recursive=*/false);
-      ++delete_count;
     }
   }
 
-  UMA_HISTOGRAM_COUNTS_100("UMA.FileMetricsProvider.DirectoryFiles",
-                           file_count);
-  UMA_HISTOGRAM_COUNTS_100("UMA.FileMetricsProvider.DeletedFiles",
-                           delete_count);
+  // Filter files from the front until one is found for processing.
+  bool have_file = false;
+  while (!source->found_files->empty()) {
+    SourceInfo::FoundFile found =
+        std::move(source->found_files->begin()->second);
+    source->found_files->erase(source->found_files->begin());
 
-  // Stop now if there are no files to read.
-  if (oldest_file_path.empty())
-    return false;
+    bool too_many =
+        source->max_dir_files > 0 && file_count > source->max_dir_files;
+    bool too_big =
+        source->max_dir_kib > 0 && total_size_kib > source->max_dir_kib;
+    bool too_old =
+        source->max_age != base::TimeDelta() &&
+        now_time - found.info.GetLastModifiedTime() > source->max_age;
+    if (too_many || too_big || too_old) {
+      base::DeleteFile(found.path, /*recursive=*/false);
+      --file_count;
+      total_size_kib -= found.info.GetSize() >> 10;
+      RecordAccessResult(too_many ? ACCESS_RESULT_TOO_MANY_FILES
+                                  : too_big ? ACCESS_RESULT_TOO_MANY_BYTES
+                                            : ACCESS_RESULT_TOO_OLD);
+      continue;
+    }
 
-  // Set the active file to be the oldest modified file that has not yet
-  // been read.
-  source->path = std::move(oldest_file_path);
-  return true;
+    AccessResult result = HandleFilterSource(source, found.path);
+    if (result == ACCESS_RESULT_SUCCESS) {
+      source->path = std::move(found.path);
+      have_file = true;
+      break;
+    }
+
+    // Record the result. Success will be recorded by the caller.
+    if (result != ACCESS_RESULT_THIS_PID)
+      RecordAccessResult(result);
+  }
+
+  return have_file;
 }
 
 // static
@@ -303,7 +377,9 @@ void FileMetricsProvider::FinishedWithSource(SourceInfo* source,
       // accumulating or also being recorded by different instances of
       // the browser.
       if (result == ACCESS_RESULT_SUCCESS ||
-          result == ACCESS_RESULT_NOT_MODIFIED) {
+          result == ACCESS_RESULT_NOT_MODIFIED ||
+          result == ACCESS_RESULT_MEMORY_DELETED ||
+          result == ACCESS_RESULT_TOO_OLD) {
         DeleteFileWhenPossible(source->path);
       }
       break;
@@ -320,28 +396,44 @@ void FileMetricsProvider::CheckAndMergeMetricSourcesOnTaskRunner(
   // This method has all state information passed in |sources| and is intended
   // to run on a worker thread rather than the UI thread.
   for (std::unique_ptr<SourceInfo>& source : *sources) {
-    AccessResult result = CheckAndMapMetricSource(source.get());
+    AccessResult result;
+    do {
+      result = CheckAndMapMetricSource(source.get());
 
-    // Some results are not reported in order to keep the dashboard clean.
-    if (result != ACCESS_RESULT_DOESNT_EXIST &&
-        result != ACCESS_RESULT_NOT_MODIFIED) {
-      UMA_HISTOGRAM_ENUMERATION(
-          "UMA.FileMetricsProvider.AccessResult", result, ACCESS_RESULT_MAX);
-    }
+      // Some results are not reported in order to keep the dashboard clean.
+      if (result != ACCESS_RESULT_DOESNT_EXIST &&
+          result != ACCESS_RESULT_NOT_MODIFIED &&
+          result != ACCESS_RESULT_THIS_PID) {
+        RecordAccessResult(result);
+      }
 
-    // Metrics associated with internal profiles have to be fetched directly
-    // so just keep the mapping for use by the main thread.
-    if (source->association == ASSOCIATE_INTERNAL_PROFILE)
-      continue;
+      // If there are no files (or no more files) in this source, stop now.
+      if (result == ACCESS_RESULT_DOESNT_EXIST)
+        break;
 
-    // Mapping was successful. Merge it.
-    if (result == ACCESS_RESULT_SUCCESS) {
-      MergeHistogramDeltasFromSource(source.get());
-      DCHECK(source->read_complete);
-    }
+      // Mapping was successful. Merge it.
+      if (result == ACCESS_RESULT_SUCCESS) {
+        // Metrics associated with internal profiles have to be fetched directly
+        // so just keep the mapping for use by the main thread.
+        if (source->association == ASSOCIATE_INTERNAL_PROFILE)
+          break;
 
-    // All done with this source.
-    FinishedWithSource(source.get(), result);
+        MergeHistogramDeltasFromSource(source.get());
+        DCHECK(source->read_complete);
+      }
+
+      // All done with this source.
+      FinishedWithSource(source.get(), result);
+
+      // If it's a directory, keep trying until a file is successfully opened.
+      // When there are no more files, ACCESS_RESULT_DOESNT_EXIST will be
+      // returned and the loop will exit above.
+    } while (result != ACCESS_RESULT_SUCCESS && !source->directory.empty());
+
+    // If the set of known files is empty, clear the object so the next run
+    // will do a fresh scan of the directory.
+    if (source->found_files && source->found_files->empty())
+      source->found_files.reset();
   }
 }
 
@@ -370,6 +462,17 @@ FileMetricsProvider::AccessResult FileMetricsProvider::CheckAndMapMetricSource(
 
   if (source->last_seen >= info.last_modified)
     return ACCESS_RESULT_NOT_MODIFIED;
+  if (source->max_age != base::TimeDelta() &&
+      base::Time::Now() - info.last_modified > source->max_age) {
+    return ACCESS_RESULT_TOO_OLD;
+  }
+
+  // Non-directory files still need to be filtered.
+  if (source->directory.empty()) {
+    AccessResult result = HandleFilterSource(source, source->path);
+    if (result != ACCESS_RESULT_SUCCESS)
+      return result;
+  }
 
   // A new file of metrics has been found.
   base::File file(source->path, kSourceOptions[source->type].file_open_flags);
@@ -393,24 +496,43 @@ FileMetricsProvider::AccessResult FileMetricsProvider::CheckAndMapMetricSource(
   }
 
   // Map the file and validate it.
-  std::unique_ptr<base::PersistentMemoryAllocator> memory_allocator =
-      base::MakeUnique<base::FilePersistentMemoryAllocator>(
+  std::unique_ptr<base::FilePersistentMemoryAllocator> memory_allocator =
+      std::make_unique<base::FilePersistentMemoryAllocator>(
           std::move(mapped), 0, 0, base::StringPiece(), read_only);
   if (memory_allocator->GetMemoryState() ==
       base::PersistentMemoryAllocator::MEMORY_DELETED) {
     return ACCESS_RESULT_MEMORY_DELETED;
   }
+  if (memory_allocator->IsCorrupt())
+    return ACCESS_RESULT_DATA_CORRUPTION;
+
+  // Cache the file data while running in a background thread so that there
+  // shouldn't be any I/O when the data is accessed from the main thread.
+  // Files with an internal profile, those from previous runs that include
+  // a full system profile and are fetched via ProvideIndependentMetrics(),
+  // are loaded on a background task and so there's no need to cache the
+  // data in advance.
+  if (source->association != ASSOCIATE_INTERNAL_PROFILE)
+    memory_allocator->Cache();
 
   // Create an allocator for the mapped file. Ownership passes to the allocator.
-  source->allocator = base::MakeUnique<base::PersistentHistogramAllocator>(
+  source->allocator = std::make_unique<base::PersistentHistogramAllocator>(
       std::move(memory_allocator));
 
+  // Check that an "independent" file has the necessary information present.
+  if (source->association == ASSOCIATE_INTERNAL_PROFILE &&
+      !PersistentSystemProfile::GetSystemProfile(
+          *source->allocator->memory_allocator(), nullptr)) {
+    return ACCESS_RESULT_NO_PROFILE;
+  }
+
   return ACCESS_RESULT_SUCCESS;
 }
 
 // static
 void FileMetricsProvider::MergeHistogramDeltasFromSource(SourceInfo* source) {
   DCHECK(source->allocator);
+  SCOPED_UMA_HISTOGRAM_TIMER("UMA.FileMetricsProvider.SnapshotTime.File");
   base::PersistentHistogramAllocator::Iterator histogram_iter(
       source->allocator.get());
 
@@ -458,8 +580,91 @@ void FileMetricsProvider::RecordHistogramSnapshotsFromSource(
            << source->path.value();
 }
 
+FileMetricsProvider::AccessResult FileMetricsProvider::HandleFilterSource(
+    SourceInfo* source,
+    const base::FilePath& path) {
+  if (!source->filter)
+    return ACCESS_RESULT_SUCCESS;
+
+  // Alternatively, pass a Params object to the filter like what was originally
+  // used to configure the source.
+  // Params params(path, source->type, source->association, source->prefs_key);
+  FilterAction action = source->filter.Run(path);
+  switch (action) {
+    case FILTER_PROCESS_FILE:
+      // Process the file.
+      return ACCESS_RESULT_SUCCESS;
+
+    case FILTER_ACTIVE_THIS_PID:
+    // Even the file for the current process has to be touched or its stamp
+    // will be less than "last processed" and thus skipped on future runs,
+    // even those done by new instances of the browser if a pref key is
+    // provided so that the last-uploaded stamp is recorded.
+    case FILTER_TRY_LATER: {
+      // Touch the file with the current timestamp making it (presumably) the
+      // newest file in the directory.
+      base::Time now = base::Time::Now();
+      base::TouchFile(path, /*accessed=*/now, /*modified=*/now);
+      if (action == FILTER_ACTIVE_THIS_PID)
+        return ACCESS_RESULT_THIS_PID;
+      return ACCESS_RESULT_FILTER_TRY_LATER;
+    }
+
+    case FILTER_SKIP_FILE:
+      switch (source->type) {
+        case SOURCE_HISTOGRAMS_ATOMIC_FILE:
+        case SOURCE_HISTOGRAMS_ATOMIC_DIR:
+          // Only "atomic" files are deleted (best-effort).
+          DeleteFileWhenPossible(path);
+          break;
+        case SOURCE_HISTOGRAMS_ACTIVE_FILE:
+          // File will presumably get modified elsewhere and thus tried again.
+          break;
+      }
+      return ACCESS_RESULT_FILTER_SKIP_FILE;
+  }
+
+  // Code never gets here but some compilers don't realize that and so complain
+  // that "not all control paths return a value".
+  NOTREACHED();
+  return ACCESS_RESULT_SUCCESS;
+}
+
+/* static */
+bool FileMetricsProvider::ProvideIndependentMetricsOnTaskRunner(
+    SourceInfo* source,
+    SystemProfileProto* system_profile_proto,
+    base::HistogramSnapshotManager* snapshot_manager) {
+  RecordEmbeddedProfileResult(EMBEDDED_PROFILE_ATTEMPT);
+  base::Time start_time = base::Time::Now();
+  if (PersistentSystemProfile::GetSystemProfile(
+          *source->allocator->memory_allocator(), system_profile_proto)) {
+    system_profile_proto->mutable_stability()->set_from_previous_run(true);
+    RecordHistogramSnapshotsFromSource(snapshot_manager, source);
+    UMA_HISTOGRAM_TIMES("UMA.FileMetricsProvider.EmbeddedProfile.RecordTime",
+                        base::Time::Now() - start_time);
+    RecordEmbeddedProfileResult(EMBEDDED_PROFILE_FOUND);
+    return true;
+  }
+
+  RecordEmbeddedProfileResult(EMBEDDED_PROFILE_DROPPED);
+
+  // TODO(bcwhite): Remove these once crbug/695880 is resolved.
+  int histogram_count = 0;
+  base::PersistentHistogramAllocator::Iterator histogram_iter(
+      source->allocator.get());
+  while (histogram_iter.GetNext()) {
+    ++histogram_count;
+  }
+  UMA_HISTOGRAM_COUNTS_10000(
+      "UMA.FileMetricsProvider.EmbeddedProfile.DroppedHistogramCount",
+      histogram_count);
+  return false;
+}
+
 void FileMetricsProvider::ScheduleSourcesCheck() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
+
   if (sources_to_check_.empty())
     return;
 
@@ -471,10 +676,11 @@ void FileMetricsProvider::ScheduleSourcesCheck() {
   std::swap(sources_to_check_, *check_list);
   task_runner_->PostTaskAndReply(
       FROM_HERE,
-      base::Bind(&FileMetricsProvider::CheckAndMergeMetricSourcesOnTaskRunner,
-                 base::Unretained(check_list)),
-      base::Bind(&FileMetricsProvider::RecordSourcesChecked,
-                 weak_factory_.GetWeakPtr(), base::Owned(check_list)));
+      base::BindOnce(
+          &FileMetricsProvider::CheckAndMergeMetricSourcesOnTaskRunner,
+          base::Unretained(check_list)),
+      base::BindOnce(&FileMetricsProvider::RecordSourcesChecked,
+                     weak_factory_.GetWeakPtr(), base::Owned(check_list)));
 }
 
 void FileMetricsProvider::RecordSourcesChecked(SourceInfoList* checked) {
@@ -515,7 +721,8 @@ void FileMetricsProvider::RecordSourcesChecked(SourceInfoList* checked) {
 }
 
 void FileMetricsProvider::DeleteFileAsync(const base::FilePath& path) {
-  task_runner_->PostTask(FROM_HERE, base::Bind(DeleteFileWhenPossible, path));
+  task_runner_->PostTask(FROM_HERE,
+                         base::BindOnce(DeleteFileWhenPossible, path));
 }
 
 void FileMetricsProvider::RecordSourceAsRead(SourceInfo* source) {
@@ -533,10 +740,10 @@ void FileMetricsProvider::RecordSourceAsRead(SourceInfo* source) {
 void FileMetricsProvider::OnDidCreateMetricsLog() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
-  // Schedule a check to see if there are new metrics to load. If so, they
-  // will be reported during the next collection run after this one. The
-  // check is run off of the worker-pool so as to not cause delays on the
-  // main UI thread (which is currently where metric collection is done).
+  // Schedule a check to see if there are new metrics to load. If so, they will
+  // be reported during the next collection run after this one. The check is run
+  // off of a MayBlock() TaskRunner so as to not cause delays on the main UI
+  // thread (which is currently where metric collection is done).
   ScheduleSourcesCheck();
 
   // Clear any data for initial metrics since they're always reported
@@ -549,61 +756,66 @@ void FileMetricsProvider::OnDidCreateMetricsLog() {
   sources_for_previous_run_.clear();
 }
 
-bool FileMetricsProvider::ProvideIndependentMetrics(
+bool FileMetricsProvider::HasIndependentMetrics() {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
+  return !sources_with_profile_.empty();
+}
+
+void FileMetricsProvider::ProvideIndependentMetrics(
+    base::OnceCallback<void(bool)> done_callback,
     SystemProfileProto* system_profile_proto,
     base::HistogramSnapshotManager* snapshot_manager) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
-  while (!sources_with_profile_.empty()) {
-    SourceInfo* source = sources_with_profile_.begin()->get();
-    DCHECK(source->allocator);
-
-    bool success = false;
-    RecordEmbeddedProfileResult(EMBEDDED_PROFILE_ATTEMPT);
-    if (PersistentSystemProfile::GetSystemProfile(
-            *source->allocator->memory_allocator(), system_profile_proto)) {
-      RecordHistogramSnapshotsFromSource(snapshot_manager, source);
-      success = true;
-      RecordEmbeddedProfileResult(EMBEDDED_PROFILE_FOUND);
-    } else {
-      RecordEmbeddedProfileResult(EMBEDDED_PROFILE_DROPPED);
-
-      // TODO(bcwhite): Remove these once crbug/695880 is resolved.
-
-      int histogram_count = 0;
-      base::PersistentHistogramAllocator::Iterator histogram_iter(
-          source->allocator.get());
-      while (histogram_iter.GetNext()) {
-        ++histogram_count;
-      }
-      UMA_HISTOGRAM_COUNTS_10000(
-          "UMA.FileMetricsProvider.EmbeddedProfile.DroppedHistogramCount",
-          histogram_count);
-    }
-
-    // Regardless of whether this source was successfully recorded, it is never
-    // read again.
-    source->read_complete = true;
-    RecordSourceAsRead(source);
-    sources_to_check_.splice(sources_to_check_.end(), sources_with_profile_,
-                             sources_with_profile_.begin());
-    ScheduleSourcesCheck();
-
-    if (success)
-      return true;
+  if (sources_with_profile_.empty()) {
+    std::move(done_callback).Run(false);
+    return;
   }
 
-  return false;
+  std::unique_ptr<SourceInfo> source =
+      std::move(*sources_with_profile_.begin());
+  sources_with_profile_.pop_front();
+  SourceInfo* source_ptr = source.get();
+  DCHECK(source->allocator);
+
+  if (base::FeatureList::IsEnabled(kBackgroundIndependentMetrics)) {
+    // Do the actual work as a background task.
+    base::PostTaskAndReplyWithResult(
+        task_runner_.get(), FROM_HERE,
+        base::BindOnce(
+            &FileMetricsProvider::ProvideIndependentMetricsOnTaskRunner,
+            source_ptr, system_profile_proto, snapshot_manager),
+        base::BindOnce(&FileMetricsProvider::ProvideIndependentMetricsCleanup,
+                       weak_factory_.GetWeakPtr(), std::move(done_callback),
+                       std::move(source)));
+  } else {
+    // Do the actual work now, inline (for performance comparisons).
+    bool success = ProvideIndependentMetricsOnTaskRunner(
+        source_ptr, system_profile_proto, snapshot_manager);
+    ProvideIndependentMetricsCleanup(std::move(done_callback),
+                                     std::move(source), success);
+  }
 }
 
-bool FileMetricsProvider::HasInitialStabilityMetrics() {
+void FileMetricsProvider::ProvideIndependentMetricsCleanup(
+    base::OnceCallback<void(bool)> done_callback,
+    std::unique_ptr<SourceInfo> source,
+    bool success) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
-  // Measure the total time spent checking all sources as well as the time
-  // per individual file. This method is called during startup and thus blocks
-  // the initial showing of the browser window so it's important to know the
-  // total delay.
-  SCOPED_UMA_HISTOGRAM_TIMER("UMA.FileMetricsProvider.InitialCheckTime.Total");
+  // Regardless of whether this source was successfully recorded, it is
+  // never read again.
+  source->read_complete = true;
+  RecordSourceAsRead(source.get());
+  sources_to_check_.push_back(std::move(source));
+  ScheduleSourcesCheck();
+
+  // Execute the chained callback.
+  std::move(done_callback).Run(success);
+}
+
+bool FileMetricsProvider::HasPreviousSessionData() {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
   // Check all sources for previous run to see if they need to be read.
   for (auto iter = sources_for_previous_run_.begin();
@@ -655,13 +867,6 @@ void FileMetricsProvider::RecordInitialHistogramSnapshots(
     base::HistogramSnapshotManager* snapshot_manager) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
-  // Measure the total time spent processing all sources as well as the time
-  // per individual file. This method is called during startup and thus blocks
-  // the initial showing of the browser window so it's important to know the
-  // total delay.
-  SCOPED_UMA_HISTOGRAM_TIMER(
-      "UMA.FileMetricsProvider.InitialSnapshotTime.Total");
-
   for (const std::unique_ptr<SourceInfo>& source : sources_for_previous_run_) {
     SCOPED_UMA_HISTOGRAM_TIMER(
         "UMA.FileMetricsProvider.InitialSnapshotTime.File");
@@ -682,13 +887,7 @@ void FileMetricsProvider::RecordInitialHistogramSnapshots(
 void FileMetricsProvider::MergeHistogramDeltas() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
-  // Measure the total time spent processing all sources as well as the time
-  // per individual file. This method is called on the UI thread so it's
-  // important to know how much total "jank" may be introduced.
-  SCOPED_UMA_HISTOGRAM_TIMER("UMA.FileMetricsProvider.SnapshotTime.Total");
-
   for (std::unique_ptr<SourceInfo>& source : sources_mapped_) {
-    SCOPED_UMA_HISTOGRAM_TIMER("UMA.FileMetricsProvider.SnapshotTime.File");
     MergeHistogramDeltasFromSource(source.get());
   }
 }
