@@ -4,11 +4,14 @@
 
 #include "components/metrics/metrics_service_client.h"
 
+#include "base/command_line.h"
+#include "base/strings/string_util.h"
+#include "components/metrics/metrics_switches.h"
 #include "components/metrics/url_constants.h"
 
 namespace metrics {
 
-MetricsServiceClient::MetricsServiceClient() : update_running_services_() {}
+MetricsServiceClient::MetricsServiceClient() {}
 
 MetricsServiceClient::~MetricsServiceClient() {}
 
@@ -32,10 +35,30 @@ std::string MetricsServiceClient::GetMetricsServerUrl() {
   return kNewMetricsServerUrl;
 }
 
-bool MetricsServiceClient::IsHistorySyncEnabledOnAllProfiles() {
+std::string MetricsServiceClient::GetInsecureMetricsServerUrl() {
+  return kNewMetricsServerUrlInsecure;
+}
+
+bool MetricsServiceClient::SyncStateAllowsUkm() {
   return false;
 }
 
+bool MetricsServiceClient::SyncStateAllowsExtensionUkm() {
+  return false;
+}
+
+bool MetricsServiceClient::AreNotificationListenersEnabledOnAllProfiles() {
+  return false;
+}
+
+std::string MetricsServiceClient::GetAppPackageName() {
+  return std::string();
+}
+
+std::string MetricsServiceClient::GetUploadSigningKey() {
+  return std::string();
+}
+
 void MetricsServiceClient::SetUpdateRunningServicesCallback(
     const base::Closure& callback) {
   update_running_services_ = callback;
@@ -46,4 +69,9 @@ void MetricsServiceClient::UpdateRunningServices() {
     update_running_services_.Run();
 }
 
+bool MetricsServiceClient::IsMetricsReportingForceEnabled() {
+  return base::CommandLine::ForCurrentProcess()->HasSwitch(
+      switches::kForceEnableMetricsReporting);
+}
+
 }  // namespace metrics
