@@ -14,13 +14,16 @@
 #include "base/command_line.h"
 #include "base/macros.h"
 #include "base/strings/string16.h"
+#include "base/strings/string_number_conversions.h"
+#include "base/strings/string_util.h"
+#include "base/test/metrics/histogram_tester.h"
 #include "components/metrics/client_info.h"
+#include "components/metrics/metrics_log.h"
 #include "components/metrics/metrics_pref_names.h"
 #include "components/metrics/metrics_service.h"
 #include "components/metrics/metrics_switches.h"
 #include "components/metrics/test_enabled_state_provider.h"
 #include "components/prefs/testing_pref_service.h"
-#include "components/variations/caching_permuted_entropy_provider.h"
 #include "components/variations/pref_names.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -159,48 +162,105 @@ TEST_F(MetricsStateManagerTest, LowEntropySource0NotReset) {
   EXPECT_EQ(0, state_manager->GetLowEntropySource());
 }
 
-TEST_F(MetricsStateManagerTest,
-       PermutedEntropyCacheClearedWhenLowEntropyReset) {
-  const PrefService::Preference* low_entropy_pref =
-      prefs_.FindPreference(prefs::kMetricsLowEntropySource);
-  const char* kCachePrefName =
-      variations::prefs::kVariationsPermutedEntropyCache;
-  int low_entropy_value = -1;
+TEST_F(MetricsStateManagerTest, HaveNoLowEntropySource) {
+  std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
+  state_manager->client_id_ = "AAAAAAAA-BBBB-CCCC-DDDD-EEEEEEEEEEEF";
+  // If we have neither the new nor old low entropy sources in prefs, then the
+  // new source should be created...
+  int new_low_source = state_manager->GetLowEntropySource();
+  EXPECT_TRUE(MetricsStateManager::IsValidLowEntropySource(new_low_source))
+      << new_low_source;
+  // ...but the old source should not...
+  EXPECT_EQ(MetricsStateManager::kLowEntropySourceNotSet,
+            state_manager->GetOldLowEntropySource());
+  // ...and the high entropy source should include the *new* low entropy source.
+  std::string high_source = state_manager->GetHighEntropySource();
+  EXPECT_TRUE(base::EndsWith(high_source, base::IntToString(new_low_source),
+                             base::CompareCase::SENSITIVE))
+      << high_source;
+}
 
-  // First, generate an initial low entropy source value.
-  {
-    EXPECT_TRUE(low_entropy_pref->IsDefaultValue());
+TEST_F(MetricsStateManagerTest, HaveOnlyNewLowEntropySource) {
+  std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
+  state_manager->client_id_ = "AAAAAAAA-BBBB-CCCC-DDDD-EEEEEEEEEEEF";
+  // If we have the new low entropy sources in prefs, but not the old one...
+  const int new_low_source = 1234;
+  prefs_.SetInteger(prefs::kMetricsLowEntropySource, new_low_source);
+  // ...then the new source should be loaded...
+  EXPECT_EQ(new_low_source, state_manager->GetLowEntropySource());
+  // ...but the old source should not be created...
+  EXPECT_EQ(MetricsStateManager::kLowEntropySourceNotSet,
+            state_manager->GetOldLowEntropySource());
+  // ...and the high entropy source should include the *new* low entropy source.
+  std::string high_source = state_manager->GetHighEntropySource();
+  EXPECT_TRUE(base::EndsWith(high_source, base::IntToString(new_low_source),
+                             base::CompareCase::SENSITIVE))
+      << high_source;
+}
 
-    std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
-    state_manager->GetLowEntropySource();
+TEST_F(MetricsStateManagerTest, HaveOnlyOldLowEntropySource) {
+  std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
+  state_manager->client_id_ = "AAAAAAAA-BBBB-CCCC-DDDD-EEEEEEEEEEEF";
+  // If we have the old low entropy sources in prefs, but not the new one...
+  const int old_low_source = 5678;
+  prefs_.SetInteger(prefs::kMetricsOldLowEntropySource, old_low_source);
+  // ...then the new source should be created...
+  int new_low_source = state_manager->GetLowEntropySource();
+  EXPECT_TRUE(MetricsStateManager::IsValidLowEntropySource(new_low_source))
+      << new_low_source;
+  // ...and the old source should be loaded...
+  EXPECT_EQ(old_low_source, state_manager->GetOldLowEntropySource());
+  // ...and the high entropy source should include the *old* low entropy source.
+  std::string high_source = state_manager->GetHighEntropySource();
+  EXPECT_TRUE(base::EndsWith(high_source, base::IntToString(old_low_source),
+                             base::CompareCase::SENSITIVE))
+      << high_source;
+}
 
-    EXPECT_FALSE(low_entropy_pref->IsDefaultValue());
-    EXPECT_TRUE(low_entropy_pref->GetValue()->GetAsInteger(&low_entropy_value));
+TEST_F(MetricsStateManagerTest, HaveBothLowEntropySources) {
+  std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
+  state_manager->client_id_ = "AAAAAAAA-BBBB-CCCC-DDDD-EEEEEEEEEEEF";
+  // If we have the new and old low entropy sources in prefs...
+  const int new_low_source = 1234;
+  const int old_low_source = 5678;
+  prefs_.SetInteger(prefs::kMetricsLowEntropySource, new_low_source);
+  prefs_.SetInteger(prefs::kMetricsOldLowEntropySource, old_low_source);
+  // ...then both should be loaded...
+  EXPECT_EQ(new_low_source, state_manager->GetLowEntropySource());
+  EXPECT_EQ(old_low_source, state_manager->GetOldLowEntropySource());
+  // ...and the high entropy source should include the *old* low entropy source.
+  std::string high_source = state_manager->GetHighEntropySource();
+  EXPECT_TRUE(base::EndsWith(high_source, base::IntToString(old_low_source),
+                             base::CompareCase::SENSITIVE))
+      << high_source;
+}
+
+TEST_F(MetricsStateManagerTest, CorruptNewLowEntropySources) {
+  std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
+  const int corrupt_sources[] = {-12345, -1, 8000, 12345};
+  for (int corrupt_source : corrupt_sources) {
+    // If the new low entropy source has been corrupted...
+    EXPECT_FALSE(MetricsStateManager::IsValidLowEntropySource(corrupt_source))
+        << corrupt_source;
+    prefs_.SetInteger(prefs::kMetricsLowEntropySource, corrupt_source);
+    // ...then a new source should be created.
+    int loaded_source = state_manager->GetLowEntropySource();
+    EXPECT_TRUE(MetricsStateManager::IsValidLowEntropySource(loaded_source))
+        << loaded_source;
   }
+}
 
-  // Now, set a dummy value in the permuted entropy cache pref and verify that
-  // another call to GetLowEntropySource() doesn't clobber it when
-  // --reset-variation-state wasn't specified.
-  {
-    prefs_.SetString(kCachePrefName, "test");
-
-    std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
-    state_manager->GetLowEntropySource();
-
-    EXPECT_EQ("test", prefs_.GetString(kCachePrefName));
-    EXPECT_EQ(low_entropy_value,
-              prefs_.GetInteger(prefs::kMetricsLowEntropySource));
-  }
-
-  // Verify that the cache does get reset if --reset-variations-state is passed.
-  {
-    base::CommandLine::ForCurrentProcess()->AppendSwitch(
-        switches::kResetVariationState);
-
-    std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
-    state_manager->GetLowEntropySource();
-
-    EXPECT_TRUE(prefs_.GetString(kCachePrefName).empty());
+TEST_F(MetricsStateManagerTest, CorruptOldLowEntropySources) {
+  std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
+  const int corrupt_sources[] = {-12345, -1, 8000, 12345};
+  for (int corrupt_source : corrupt_sources) {
+    // If the old low entropy source has been corrupted...
+    EXPECT_FALSE(MetricsStateManager::IsValidLowEntropySource(corrupt_source))
+        << corrupt_source;
+    prefs_.SetInteger(prefs::kMetricsOldLowEntropySource, corrupt_source);
+    // ...then it should be ignored.
+    EXPECT_EQ(MetricsStateManager::kLowEntropySourceNotSet,
+              state_manager->GetOldLowEntropySource());
   }
 }
 
@@ -218,6 +278,7 @@ TEST_F(MetricsStateManagerTest, ResetMetricsIDs) {
     std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
     state_manager->ForceClientIdCreation();
     EXPECT_EQ(kInitialClientId, state_manager->client_id());
+    EXPECT_FALSE(state_manager->metrics_ids_were_reset_);
   }
 
   // Set the reset pref to cause the IDs to be reset.
@@ -228,6 +289,8 @@ TEST_F(MetricsStateManagerTest, ResetMetricsIDs) {
     std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
     state_manager->ForceClientIdCreation();
     EXPECT_NE(kInitialClientId, state_manager->client_id());
+    EXPECT_TRUE(state_manager->metrics_ids_were_reset_);
+    EXPECT_EQ(kInitialClientId, state_manager->previous_client_id_);
 
     state_manager->GetLowEntropySource();
 
@@ -378,6 +441,8 @@ TEST_F(MetricsStateManagerTest, ResetBackup) {
     // A brand new client id should have been generated.
     EXPECT_NE(std::string(), state_manager->client_id());
     EXPECT_NE(client_info.client_id, state_manager->client_id());
+    EXPECT_TRUE(state_manager->metrics_ids_were_reset_);
+    EXPECT_EQ(client_info.client_id, state_manager->previous_client_id_);
     EXPECT_TRUE(stored_client_info_backup_);
 
     // The installation date should not have been affected.
@@ -390,4 +455,69 @@ TEST_F(MetricsStateManagerTest, ResetBackup) {
   }
 }
 
+TEST_F(MetricsStateManagerTest, CheckProvider) {
+  int64_t kInstallDate = 1373051956;
+  int64_t kInstallDateExpected = 1373050800;  // Computed from kInstallDate.
+  int64_t kEnabledDate = 1373001211;
+  int64_t kEnabledDateExpected = 1373000400;  // Computed from kEnabledDate.
+
+  ClientInfo client_info;
+  client_info.client_id = "AAAAAAAA-BBBB-CCCC-DDDD-EEEEEEEEEEEE";
+  client_info.installation_date = kInstallDate;
+  client_info.reporting_enabled_date = kEnabledDate;
+
+  SetFakeClientInfoBackup(client_info);
+  SetClientInfoPrefs(client_info);
+
+  std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
+  std::unique_ptr<MetricsProvider> provider = state_manager->GetProvider();
+  SystemProfileProto system_profile;
+  provider->ProvideSystemProfileMetrics(&system_profile);
+  EXPECT_EQ(system_profile.install_date(), kInstallDateExpected);
+  EXPECT_EQ(system_profile.uma_enabled_date(), kEnabledDateExpected);
+
+  base::HistogramTester histogram_tester;
+  ChromeUserMetricsExtension uma_proto;
+  provider->ProvidePreviousSessionData(&uma_proto);
+  // The client_id field in the proto should not be overwritten.
+  EXPECT_FALSE(uma_proto.has_client_id());
+  // Nothing should have been emitted to the cloned install histogram.
+  histogram_tester.ExpectTotalCount("UMA.IsClonedInstall", 0);
+}
+
+TEST_F(MetricsStateManagerTest, CheckProviderResetIds) {
+  int64_t kInstallDate = 1373051956;
+  int64_t kInstallDateExpected = 1373050800;  // Computed from kInstallDate.
+  int64_t kEnabledDate = 1373001211;
+  int64_t kEnabledDateExpected = 1373000400;  // Computed from kEnabledDate.
+
+  ClientInfo client_info;
+  client_info.client_id = "AAAAAAAA-BBBB-CCCC-DDDD-EEEEEEEEEEEE";
+  client_info.installation_date = kInstallDate;
+  client_info.reporting_enabled_date = kEnabledDate;
+
+  SetFakeClientInfoBackup(client_info);
+  SetClientInfoPrefs(client_info);
+
+  // Set the reset pref to cause the IDs to be reset.
+  prefs_.SetBoolean(prefs::kMetricsResetIds, true);
+  std::unique_ptr<MetricsStateManager> state_manager(CreateStateManager());
+  EXPECT_NE(client_info.client_id, state_manager->client_id());
+  EXPECT_TRUE(state_manager->metrics_ids_were_reset_);
+  EXPECT_EQ(client_info.client_id, state_manager->previous_client_id_);
+
+  std::unique_ptr<MetricsProvider> provider = state_manager->GetProvider();
+  SystemProfileProto system_profile;
+  provider->ProvideSystemProfileMetrics(&system_profile);
+  EXPECT_EQ(system_profile.install_date(), kInstallDateExpected);
+  EXPECT_EQ(system_profile.uma_enabled_date(), kEnabledDateExpected);
+
+  base::HistogramTester histogram_tester;
+  ChromeUserMetricsExtension uma_proto;
+  provider->ProvidePreviousSessionData(&uma_proto);
+  EXPECT_EQ(MetricsLog::Hash(state_manager->previous_client_id_),
+            uma_proto.client_id());
+  histogram_tester.ExpectUniqueSample("UMA.IsClonedInstall", 1, 1);
+}
+
 }  // namespace metrics
