@@ -16,19 +16,25 @@
 #include "base/files/scoped_temp_dir.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_string_value_serializer.h"
+#include "base/synchronization/waitable_event.h"
 #include "base/test/scoped_task_environment.h"
-#include "base/threading/thread.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "base/values.h"
 #include "build/build_config.h"
-#include "components/net_log/chrome_net_log.h"
+#include "mojo/public/cpp/bindings/strong_binding.h"
+#include "net/base/network_change_notifier.h"
 #include "net/base/test_completion_callback.h"
-#include "net/http/http_network_session.h"
 #include "net/log/net_log_capture_mode.h"
 #include "net/log/net_log_event_type.h"
+#include "net/test/embedded_test_server/default_handlers.h"
+#include "net/test/embedded_test_server/embedded_test_server.h"
+#include "net/test/embedded_test_server/http_request.h"
+#include "net/test/embedded_test_server/http_response.h"
 #include "net/traffic_annotation/network_traffic_annotation_test_helper.h"
-#include "net/url_request/url_request_context_getter.h"
-#include "net/url_request/url_request_test_util.h"
+#include "services/network/network_context.h"
+#include "services/network/network_service.h"
+#include "services/network/public/cpp/resource_request.h"
+#include "services/network/public/cpp/simple_url_loader.h"
+#include "services/network/test/test_network_context.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace {
@@ -54,6 +60,38 @@ const char kStateStoppingLogString[] = "STOPPING_LOG";
 
 namespace net_log {
 
+class FakeNetLogExporter : public network::mojom::NetLogExporter {
+ public:
+  FakeNetLogExporter() {}
+  ~FakeNetLogExporter() override {}
+
+  void Start(base::File destination,
+             base::Value extra_constants,
+             network::mojom::NetLogCaptureMode capture_mode,
+             uint64_t max_file_size,
+             StartCallback callback) override {
+    std::move(callback).Run(net::OK);
+  }
+
+  void Stop(base::Value polled_values, StopCallback callback) override {
+    std::move(callback).Run(net::OK);
+  }
+};
+
+class FakeNetworkContext : public network::TestNetworkContext {
+ public:
+  void CreateNetLogExporter(
+      network::mojom::NetLogExporterRequest request) override {
+    binding_ = mojo::StrongBinding<network::mojom::NetLogExporter>::Create(
+        std::make_unique<FakeNetLogExporter>(), std::move(request));
+  }
+
+  void Disconnect() { binding_->Close(); }
+
+ private:
+  mojo::StrongBindingPtr<network::mojom::NetLogExporter> binding_;
+};
+
 // Sets |path| to |path_to_return| and always returns true. This function is
 // used to override NetExportFileWriter's usual getter for the default log base
 // directory.
@@ -162,44 +200,6 @@ WARN_UNUSED_RESULT ::testing::AssertionResult ReadCompleteLogFile(
   return ::testing::AssertionSuccess();
 }
 
-void SetUpTestContextGetterWithQuicTimeoutInfo(
-    net::NetLog* net_log,
-    int quic_idle_connection_timeout_seconds,
-    scoped_refptr<net::TestURLRequestContextGetter>* context_getter) {
-  std::unique_ptr<net::TestURLRequestContext> context =
-      base::MakeUnique<net::TestURLRequestContext>(true);
-  context->set_net_log(net_log);
-
-  std::unique_ptr<net::HttpNetworkSession::Params> params(
-      new net::HttpNetworkSession::Params);
-  params->quic_idle_connection_timeout_seconds =
-      quic_idle_connection_timeout_seconds;
-
-  context->set_http_network_session_params(std::move(params));
-  context->Init();
-
-  *context_getter = new net::TestURLRequestContextGetter(
-      base::ThreadTaskRunnerHandle::Get(), std::move(context));
-}
-
-void SetUpTestContextGetterWithRequest(
-    net::NetLog* net_log,
-    const GURL& url,
-    net::URLRequest::Delegate* delegate,
-    scoped_refptr<net::TestURLRequestContextGetter>* context_getter,
-    std::unique_ptr<net::URLRequest>* request) {
-  auto context = base::MakeUnique<net::TestURLRequestContext>(true);
-  context->set_net_log(net_log);
-  context->Init();
-
-  *request = context->CreateRequest(url, net::IDLE, delegate,
-                                    TRAFFIC_ANNOTATION_FOR_TESTS);
-  (*request)->Start();
-
-  *context_getter = new net::TestURLRequestContextGetter(
-      base::ThreadTaskRunnerHandle::Get(), std::move(context));
-}
-
 // An implementation of NetExportFileWriter::StateObserver that allows waiting
 // until it's notified of a new state.
 class TestStateObserver : public NetExportFileWriter::StateObserver {
@@ -251,17 +251,24 @@ class TestFilePathCallback {
 
 class NetExportFileWriterTest : public ::testing::Test {
  public:
-  using URLRequestContextGetterList =
-      std::vector<scoped_refptr<net::URLRequestContextGetter>>;
-
   NetExportFileWriterTest()
-      : file_writer_(&net_log_),
-        file_thread_("NetLogFileWriter file thread"),
-        net_thread_("NetLogFileWriter net thread") {}
+      : scoped_task_environment_(
+            base::test::ScopedTaskEnvironment::MainThreadType::IO),
+        network_change_notifier_(net::NetworkChangeNotifier::CreateMock()),
+        network_service_(network::NetworkService::CreateForTesting()) {}
 
   // ::testing::Test implementation
   void SetUp() override {
     ASSERT_TRUE(log_temp_dir_.CreateUniqueTempDir());
+    network::mojom::NetworkContextParamsPtr params =
+        network::mojom::NetworkContextParams::New();
+    // Use a fixed proxy config, to avoid dependencies on local network
+    // configuration.
+    params->initial_proxy_config =
+        net::ProxyConfigWithAnnotation::CreateDirect();
+    network_context_ = std::make_unique<network::NetworkContext>(
+        network_service_.get(), mojo::MakeRequest(&network_context_ptr_),
+        std::move(params));
 
     // Override |file_writer_|'s default-log-base-directory-getter to
     // a getter that returns the temp dir created for the test.
@@ -270,9 +277,6 @@ class NetExportFileWriterTest : public ::testing::Test {
 
     default_log_path_ = log_temp_dir_.GetPath().Append(kLogRelativePath);
 
-    ASSERT_TRUE(file_thread_.Start());
-    ASSERT_TRUE(net_thread_.Start());
-
     file_writer_.AddObserver(&test_state_observer_);
 
     ASSERT_TRUE(VerifyState(file_writer_.GetState(), kStateUninitializedString,
@@ -294,8 +298,7 @@ class NetExportFileWriterTest : public ::testing::Test {
   WARN_UNUSED_RESULT ::testing::AssertionResult InitializeThenVerifyNewState(
       bool expected_initialize_success,
       bool expected_log_exists) {
-    file_writer_.Initialize(file_thread_.task_runner(),
-                            net_thread_.task_runner());
+    file_writer_.Initialize();
     std::unique_ptr<base::DictionaryValue> state =
         test_state_observer_.WaitForNewState();
     ::testing::AssertionResult result =
@@ -329,10 +332,10 @@ class NetExportFileWriterTest : public ::testing::Test {
       const base::FilePath& custom_log_path,
       net::NetLogCaptureMode capture_mode,
       const std::string& expected_capture_mode_string,
-      const URLRequestContextGetterList& context_getters) {
+      network::mojom::NetworkContext* network_context) {
     file_writer_.StartNetLog(custom_log_path, capture_mode, kMaxLogSizeBytes,
                              base::CommandLine::StringType(), kChannelString,
-                             context_getters);
+                             network_context);
     std::unique_ptr<base::DictionaryValue> state =
         test_state_observer_.WaitForNewState();
     ::testing::AssertionResult result =
@@ -370,9 +373,8 @@ class NetExportFileWriterTest : public ::testing::Test {
   WARN_UNUSED_RESULT ::testing::AssertionResult StopThenVerifyNewStateAndFile(
       const base::FilePath& custom_log_path,
       std::unique_ptr<base::DictionaryValue> polled_data,
-      scoped_refptr<net::URLRequestContextGetter> context_getter,
       const std::string& expected_capture_mode_string) {
-    file_writer_.StopNetLog(std::move(polled_data), context_getter);
+    file_writer_.StopNetLog(std::move(polled_data));
     std::unique_ptr<base::DictionaryValue> state =
         test_state_observer_.WaitForNewState();
     ::testing::AssertionResult result =
@@ -419,11 +421,34 @@ class NetExportFileWriterTest : public ::testing::Test {
     return ::testing::AssertionSuccess();
   }
 
- protected:
-  ChromeNetLog net_log_;
+  net::NetLog* net_log() { return network_service_->net_log(); }
 
-  // |file_writer_| is initialized after |net_log_| so that it can stop
-  // obvserving on destruction.
+  NetExportFileWriter* file_writer() { return &file_writer_; }
+
+  const base::FilePath& GetLogTempDirPath() const {
+    return log_temp_dir_.GetPath();
+  }
+
+  const base::FilePath& default_log_path() const { return default_log_path_; }
+
+  TestStateObserver* test_state_observer() { return &test_state_observer_; }
+
+  network::mojom::NetworkContext* network_context() {
+    return network_context_ptr_.get();
+  }
+
+ private:
+  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  // Use a mock NetworkChangeNotifier so the real one can't add any logging.
+  std::unique_ptr<net::NetworkChangeNotifier> network_change_notifier_;
+  std::unique_ptr<network::NetworkService> network_service_;
+
+  network::mojom::NetworkContextPtr network_context_ptr_;
+  std::unique_ptr<network::NetworkContext> network_context_;
+
+  // |file_writer_| is initialized after |network_context_ptr_| since
+  // |file_writer_| destructor can talk to mojo objects owned by
+  // |network_context_|.
   NetExportFileWriter file_writer_;
 
   base::ScopedTempDir log_temp_dir_;
@@ -431,19 +456,13 @@ class NetExportFileWriterTest : public ::testing::Test {
   // The default log path that |file_writer_| will use is cached here.
   base::FilePath default_log_path_;
 
-  base::Thread file_thread_;
-  base::Thread net_thread_;
-
   TestStateObserver test_state_observer_;
-
- private:
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
 };
 
 TEST_F(NetExportFileWriterTest, InitFail) {
   // Override file_writer_'s default log base directory getter to always
   // fail.
-  file_writer_.SetDefaultLogBaseDirectoryGetterForTest(
+  file_writer()->SetDefaultLogBaseDirectoryGetterForTest(
       base::Bind([](base::FilePath* path) -> bool { return false; }));
 
   // Initialization should fail due to the override.
@@ -466,14 +485,14 @@ TEST_F(NetExportFileWriterTest, InitWithExistingLog) {
   // Create and close an empty log file to simulate existence of a previous log
   // file.
   ASSERT_TRUE(
-      base::CreateDirectoryAndGetError(default_log_path_.DirName(), nullptr));
-  base::ScopedFILE empty_file(base::OpenFile(default_log_path_, "w"));
+      base::CreateDirectoryAndGetError(default_log_path().DirName(), nullptr));
+  base::ScopedFILE empty_file(base::OpenFile(default_log_path(), "w"));
   ASSERT_TRUE(empty_file.get());
   empty_file.reset();
 
   ASSERT_TRUE(InitializeThenVerifyNewState(true, true));
 
-  EXPECT_EQ(default_log_path_, FileWriterGetFilePathToCompletedLog());
+  EXPECT_EQ(default_log_path(), FileWriterGetFilePathToCompletedLog());
 }
 
 TEST_F(NetExportFileWriterTest, StartAndStopWithAllCaptureModes) {
@@ -493,38 +512,38 @@ TEST_F(NetExportFileWriterTest, StartAndStopWithAllCaptureModes) {
     // StartNetLog(), should result in state change.
     ASSERT_TRUE(StartThenVerifyNewState(base::FilePath(), capture_modes[i],
                                         capture_mode_strings[i],
-                                        URLRequestContextGetterList()));
+                                        network_context()));
 
     // Calling StartNetLog() again should be a no-op. Try doing StartNetLog()
     // with various capture modes; they should all be ignored and result in no
     // state change.
-    file_writer_.StartNetLog(base::FilePath(), capture_modes[i],
-                             kMaxLogSizeBytes, base::CommandLine::StringType(),
-                             kChannelString, URLRequestContextGetterList());
-    file_writer_.StartNetLog(base::FilePath(), capture_modes[(i + 1) % 3],
-                             kMaxLogSizeBytes, base::CommandLine::StringType(),
-                             kChannelString, URLRequestContextGetterList());
-    file_writer_.StartNetLog(base::FilePath(), capture_modes[(i + 2) % 3],
-                             kMaxLogSizeBytes, base::CommandLine::StringType(),
-                             kChannelString, URLRequestContextGetterList());
+    file_writer()->StartNetLog(
+        base::FilePath(), capture_modes[i], kMaxLogSizeBytes,
+        base::CommandLine::StringType(), kChannelString, network_context());
+    file_writer()->StartNetLog(
+        base::FilePath(), capture_modes[(i + 1) % 3], kMaxLogSizeBytes,
+        base::CommandLine::StringType(), kChannelString, network_context());
+    file_writer()->StartNetLog(
+        base::FilePath(), capture_modes[(i + 2) % 3], kMaxLogSizeBytes,
+        base::CommandLine::StringType(), kChannelString, network_context());
 
     // StopNetLog(), should result in state change. The capture mode should
     // match that of the first StartNetLog() call (called by
     // StartThenVerifyNewState()).
-    ASSERT_TRUE(StopThenVerifyNewStateAndFile(
-        base::FilePath(), nullptr, nullptr, capture_mode_strings[i]));
+    ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr,
+                                              capture_mode_strings[i]));
 
     // Stopping a second time should be a no-op.
-    file_writer_.StopNetLog(nullptr, nullptr);
+    file_writer()->StopNetLog(nullptr);
   }
 
   // Start and stop one more time just to make sure the last StopNetLog() call
   // was properly ignored and left |file_writer_| in a valid state.
   ASSERT_TRUE(StartThenVerifyNewState(base::FilePath(), capture_modes[0],
                                       capture_mode_strings[0],
-                                      URLRequestContextGetterList()));
+                                      network_context()));
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr, nullptr,
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr,
                                             capture_mode_strings[0]));
 }
 
@@ -535,34 +554,34 @@ TEST_F(NetExportFileWriterTest, StartClearsFile) {
 
   ASSERT_TRUE(StartThenVerifyNewState(
       base::FilePath(), net::NetLogCaptureMode::Default(),
-      kCaptureModeDefaultString, URLRequestContextGetterList()));
+      kCaptureModeDefaultString, network_context()));
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr, nullptr,
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr,
                                             kCaptureModeDefaultString));
 
   int64_t stop_file_size;
-  EXPECT_TRUE(base::GetFileSize(default_log_path_, &stop_file_size));
+  EXPECT_TRUE(base::GetFileSize(default_log_path(), &stop_file_size));
 
   // Add some junk at the end of the file.
   std::string junk_data("Hello");
-  EXPECT_TRUE(base::AppendToFile(default_log_path_, junk_data.c_str(),
+  EXPECT_TRUE(base::AppendToFile(default_log_path(), junk_data.c_str(),
                                  junk_data.size()));
 
   int64_t junk_file_size;
-  EXPECT_TRUE(base::GetFileSize(default_log_path_, &junk_file_size));
+  EXPECT_TRUE(base::GetFileSize(default_log_path(), &junk_file_size));
   EXPECT_GT(junk_file_size, stop_file_size);
 
   // Start and stop again and make sure the file is back to the size it was
   // before adding the junk data.
   ASSERT_TRUE(StartThenVerifyNewState(
       base::FilePath(), net::NetLogCaptureMode::Default(),
-      kCaptureModeDefaultString, URLRequestContextGetterList()));
+      kCaptureModeDefaultString, network_context()));
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr, nullptr,
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr,
                                             kCaptureModeDefaultString));
 
   int64_t new_stop_file_size;
-  EXPECT_TRUE(base::GetFileSize(default_log_path_, &new_stop_file_size));
+  EXPECT_TRUE(base::GetFileSize(default_log_path(), &new_stop_file_size));
 
   EXPECT_EQ(stop_file_size, new_stop_file_size);
 }
@@ -574,27 +593,27 @@ TEST_F(NetExportFileWriterTest, AddEvent) {
 
   ASSERT_TRUE(StartThenVerifyNewState(
       base::FilePath(), net::NetLogCaptureMode::Default(),
-      kCaptureModeDefaultString, URLRequestContextGetterList()));
+      kCaptureModeDefaultString, network_context()));
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr, nullptr,
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr,
                                             kCaptureModeDefaultString));
 
   // Get file size without the event.
   int64_t stop_file_size;
-  EXPECT_TRUE(base::GetFileSize(default_log_path_, &stop_file_size));
+  EXPECT_TRUE(base::GetFileSize(default_log_path(), &stop_file_size));
 
   ASSERT_TRUE(StartThenVerifyNewState(
       base::FilePath(), net::NetLogCaptureMode::Default(),
-      kCaptureModeDefaultString, URLRequestContextGetterList()));
+      kCaptureModeDefaultString, network_context()));
 
-  net_log_.AddGlobalEntry(net::NetLogEventType::CANCELLED);
+  net_log()->AddGlobalEntry(net::NetLogEventType::CANCELLED);
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr, nullptr,
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr,
                                             kCaptureModeDefaultString));
 
   // Get file size after adding the event and make sure it's larger than before.
   int64_t new_stop_file_size;
-  EXPECT_TRUE(base::GetFileSize(default_log_path_, &new_stop_file_size));
+  EXPECT_TRUE(base::GetFileSize(default_log_path(), &new_stop_file_size));
   EXPECT_GE(new_stop_file_size, stop_file_size);
 }
 
@@ -606,15 +625,15 @@ TEST_F(NetExportFileWriterTest, AddEventCustomPath) {
   base::FilePath::CharType kCustomRelativePath[] =
       FILE_PATH_LITERAL("custom/custom/chrome-net-export-log.json");
   base::FilePath custom_log_path =
-      log_temp_dir_.GetPath().Append(kCustomRelativePath);
+      GetLogTempDirPath().Append(kCustomRelativePath);
   EXPECT_TRUE(
       base::CreateDirectoryAndGetError(custom_log_path.DirName(), nullptr));
 
   ASSERT_TRUE(StartThenVerifyNewState(
       custom_log_path, net::NetLogCaptureMode::Default(),
-      kCaptureModeDefaultString, URLRequestContextGetterList()));
+      kCaptureModeDefaultString, network_context()));
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(custom_log_path, nullptr, nullptr,
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(custom_log_path, nullptr,
                                             kCaptureModeDefaultString));
 
   // Get file size without the event.
@@ -623,11 +642,11 @@ TEST_F(NetExportFileWriterTest, AddEventCustomPath) {
 
   ASSERT_TRUE(StartThenVerifyNewState(
       custom_log_path, net::NetLogCaptureMode::Default(),
-      kCaptureModeDefaultString, URLRequestContextGetterList()));
+      kCaptureModeDefaultString, network_context()));
 
-  net_log_.AddGlobalEntry(net::NetLogEventType::CANCELLED);
+  net_log()->AddGlobalEntry(net::NetLogEventType::CANCELLED);
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(custom_log_path, nullptr, nullptr,
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(custom_log_path, nullptr,
                                             kCaptureModeDefaultString));
 
   // Get file size after adding the event and make sure it's larger than before.
@@ -636,38 +655,27 @@ TEST_F(NetExportFileWriterTest, AddEventCustomPath) {
   EXPECT_GE(new_stop_file_size, stop_file_size);
 }
 
-TEST_F(NetExportFileWriterTest, StopWithPolledDataAndContextGetter) {
+TEST_F(NetExportFileWriterTest, StopWithPolledData) {
   ASSERT_TRUE(InitializeThenVerifyNewState(true, false));
 
   // Create dummy polled data
   const char kDummyPolledDataPath[] = "dummy_path";
   const char kDummyPolledDataString[] = "dummy_info";
   std::unique_ptr<base::DictionaryValue> dummy_polled_data =
-      base::MakeUnique<base::DictionaryValue>();
+      std::make_unique<base::DictionaryValue>();
   dummy_polled_data->SetString(kDummyPolledDataPath, kDummyPolledDataString);
 
-  // Create test context getter on |net_thread_| and wait for it to finish.
-  scoped_refptr<net::TestURLRequestContextGetter> context_getter;
-  const int kDummyQuicParam = 75;
-  net::TestClosure init_done;
-  net_thread_.task_runner()->PostTaskAndReply(
-      FROM_HERE,
-      base::Bind(&SetUpTestContextGetterWithQuicTimeoutInfo, &net_log_,
-                 kDummyQuicParam, &context_getter),
-      init_done.closure());
-  init_done.WaitForResult();
-
   ASSERT_TRUE(StartThenVerifyNewState(
       base::FilePath(), net::NetLogCaptureMode::Default(),
-      kCaptureModeDefaultString, URLRequestContextGetterList()));
+      kCaptureModeDefaultString, network_context()));
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(
-      base::FilePath(), std::move(dummy_polled_data), context_getter,
-      kCaptureModeDefaultString));
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(),
+                                            std::move(dummy_polled_data),
+                                            kCaptureModeDefaultString));
 
   // Read polledData from log file.
   std::unique_ptr<base::DictionaryValue> root;
-  ASSERT_TRUE(ReadCompleteLogFile(default_log_path_, &root));
+  ASSERT_TRUE(ReadCompleteLogFile(default_log_path(), &root));
   base::DictionaryValue* polled_data;
   ASSERT_TRUE(root->GetDictionary("polledData", &polled_data));
 
@@ -676,52 +684,86 @@ TEST_F(NetExportFileWriterTest, StopWithPolledDataAndContextGetter) {
   ASSERT_TRUE(polled_data->GetString(kDummyPolledDataPath, &dummy_string));
   EXPECT_EQ(kDummyPolledDataString, dummy_string);
 
-  // Check that it also contains the field from the URLRequestContext that was
-  // passed in.
-  base::DictionaryValue* quic_info;
-  ASSERT_TRUE(polled_data->GetDictionary("quicInfo", &quic_info));
-  base::Value* timeout_value = nullptr;
-  int timeout;
-  ASSERT_TRUE(
-      quic_info->Get("idle_connection_timeout_seconds", &timeout_value));
-  ASSERT_TRUE(timeout_value->GetAsInteger(&timeout));
-  EXPECT_EQ(kDummyQuicParam, timeout);
+  // Check that it also contains something from net::GetNetInfo.
+  base::DictionaryValue* http_cache_info;
+  ASSERT_TRUE(polled_data->GetDictionary("httpCacheInfo", &http_cache_info));
 }
 
-TEST_F(NetExportFileWriterTest, StartWithContextGetters) {
+// Test with requests in flight. This is done by going through a sequence of a
+// redirect --- at which point the log is started --- and then a fetch of
+// destination that's blocked on an event in EmbeddedTestServer.
+TEST_F(NetExportFileWriterTest, StartWithNetworkContextActive) {
+  net::EmbeddedTestServer test_server;
+  net::test_server::RegisterDefaultHandlers(&test_server);
+
+  base::WaitableEvent block_fetch(
+      base::WaitableEvent::ResetPolicy::MANUAL,
+      base::WaitableEvent::InitialState::NOT_SIGNALED);
+
+  test_server.RegisterRequestHandler(base::BindRepeating(
+      [](base::WaitableEvent* block_fetch,
+         const net::test_server::HttpRequest& request)
+          -> std::unique_ptr<net::test_server::HttpResponse> {
+        if (request.relative_url == "/block")
+          block_fetch->Wait();
+        return nullptr;
+      },
+      &block_fetch));
+
+  ASSERT_TRUE(test_server.Start());
+
   ASSERT_TRUE(InitializeThenVerifyNewState(true, false));
 
-  // Create test context getter and request on |net_thread_| and wait for it to
-  // finish.
-  const std::string kDummyUrl = "blah:blah";
-  scoped_refptr<net::TestURLRequestContextGetter> context_getter;
-  std::unique_ptr<net::URLRequest> request;
-  net::TestDelegate delegate;
-  delegate.set_quit_on_complete(false);
+  network::mojom::URLLoaderFactoryPtr url_loader_factory;
+  auto url_loader_factory_params =
+      network::mojom::URLLoaderFactoryParams::New();
+  url_loader_factory_params->process_id = network::mojom::kBrowserProcessId;
+  url_loader_factory_params->is_corb_enabled = false;
+  network_context()->CreateURLLoaderFactory(
+      mojo::MakeRequest(&url_loader_factory),
+      std::move(url_loader_factory_params));
 
-  net::TestClosure init_done;
-  net_thread_.task_runner()->PostTaskAndReply(
-      FROM_HERE,
-      base::Bind(&SetUpTestContextGetterWithRequest, &net_log_, GURL(kDummyUrl),
-                 &delegate, &context_getter, &request),
-      init_done.closure());
-  init_done.WaitForResult();
+  const char kRedirectURL[] = "/server-redirect?/block";
+  std::unique_ptr<network::ResourceRequest> request =
+      std::make_unique<network::ResourceRequest>();
+  request->url = test_server.GetURL(kRedirectURL);
 
+  std::unique_ptr<network::SimpleURLLoader> simple_loader =
+      network::SimpleURLLoader::Create(std::move(request),
+                                       TRAFFIC_ANNOTATION_FOR_TESTS);
+  base::RunLoop run_loop, run_loop2;
+  simple_loader->SetOnRedirectCallback(base::BindRepeating(
+      [](base::RepeatingClosure notify_log,
+         const net::RedirectInfo& redirect_info,
+         const network::ResourceResponseHead& response_head,
+         std::vector<std::string>* to_be_removed_headers) { notify_log.Run(); },
+      run_loop.QuitClosure()));
+  simple_loader->DownloadToStringOfUnboundedSizeUntilCrashAndDie(
+      url_loader_factory.get(),
+      base::BindOnce(
+          [](base::OnceClosure quit_closure,
+             std::unique_ptr<std::string> response_body) {
+            std::move(quit_closure).Run();
+          },
+          run_loop2.QuitClosure()));
+
+  // Wait for fetch to get some bytes accross. It will not be the entire
+  // thing since the post-redirect URL will get blocked by the custom handler.
+  run_loop.Run();
   ASSERT_TRUE(StartThenVerifyNewState(
       base::FilePath(), net::NetLogCaptureMode::Default(),
-      kCaptureModeDefaultString, {context_getter}));
+      kCaptureModeDefaultString, network_context()));
 
-  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr, nullptr,
+  ASSERT_TRUE(StopThenVerifyNewStateAndFile(base::FilePath(), nullptr,
                                             kCaptureModeDefaultString));
-
   // Read events from log file.
   std::unique_ptr<base::DictionaryValue> root;
-  ASSERT_TRUE(ReadCompleteLogFile(default_log_path_, &root));
+  ASSERT_TRUE(ReadCompleteLogFile(default_log_path(), &root));
   base::ListValue* events;
   ASSERT_TRUE(root->GetList("events", &events));
 
   // Check there is at least one event as a result of the ongoing request.
-  EXPECT_GE(events->GetSize(), 1u);
+  ASSERT_GE(events->GetSize(), 1u);
 
   // Check the URL in the params of the first event.
   base::DictionaryValue* event;
@@ -730,32 +772,32 @@ TEST_F(NetExportFileWriterTest, StartWithContextGetters) {
   EXPECT_TRUE(event->GetDictionary("params", &event_params));
   std::string event_url;
   EXPECT_TRUE(event_params->GetString("url", &event_url));
-  EXPECT_EQ(kDummyUrl, event_url);
+  EXPECT_EQ(test_server.GetURL(kRedirectURL), event_url);
 
-  net_thread_.task_runner()->DeleteSoon(FROM_HERE, request.release());
+  block_fetch.Signal();
+  run_loop2.Run();
 }
 
 TEST_F(NetExportFileWriterTest, ReceiveStartWhileInitializing) {
   // Trigger initialization of |file_writer_|.
-  file_writer_.Initialize(file_thread_.task_runner(),
-                          net_thread_.task_runner());
+  file_writer()->Initialize();
 
   // Before running the main message loop, tell |file_writer_| to start
   // logging. Not running the main message loop prevents the initialization
   // process from completing, so this ensures that StartNetLog() is received
   // before |file_writer_| finishes initialization, which means this
   // should be a no-op.
-  file_writer_.StartNetLog(base::FilePath(), net::NetLogCaptureMode::Default(),
-                           kMaxLogSizeBytes, base::CommandLine::StringType(),
-                           kChannelString, URLRequestContextGetterList());
+  file_writer()->StartNetLog(
+      base::FilePath(), net::NetLogCaptureMode::Default(), kMaxLogSizeBytes,
+      base::CommandLine::StringType(), kChannelString, network_context());
 
   // Now run the main message loop. Make sure StartNetLog() was ignored by
   // checking that the next two states are "initializing" followed by
   // "not-logging".
   std::unique_ptr<base::DictionaryValue> state =
-      test_state_observer_.WaitForNewState();
+      test_state_observer()->WaitForNewState();
   ASSERT_TRUE(VerifyState(std::move(state), kStateInitializingString));
-  state = test_state_observer_.WaitForNewState();
+  state = test_state_observer()->WaitForNewState();
   ASSERT_TRUE(
       VerifyState(std::move(state), kStateNotLoggingString, false, false, ""));
 }
@@ -766,30 +808,46 @@ TEST_F(NetExportFileWriterTest, ReceiveStartWhileStoppingLog) {
   // Call StartNetLog() on |file_writer_| and wait for the state change.
   ASSERT_TRUE(StartThenVerifyNewState(
       base::FilePath(), net::NetLogCaptureMode::IncludeSocketBytes(),
-      kCaptureModeIncludeSocketBytesString, URLRequestContextGetterList()));
+      kCaptureModeIncludeSocketBytesString, network_context()));
 
   // Tell |file_writer_| to stop logging.
-  file_writer_.StopNetLog(nullptr, nullptr);
+  file_writer()->StopNetLog(nullptr);
 
   // Before running the main message loop, tell |file_writer_| to start
   // logging. Not running the main message loop prevents the stopping process
   // from completing, so this ensures StartNetLog() is received before
   // |file_writer_| finishes stopping, which means this should be a
   // no-op.
-  file_writer_.StartNetLog(base::FilePath(), net::NetLogCaptureMode::Default(),
-                           kMaxLogSizeBytes, base::CommandLine::StringType(),
-                           kChannelString, URLRequestContextGetterList());
+  file_writer()->StartNetLog(
+      base::FilePath(), net::NetLogCaptureMode::Default(), kMaxLogSizeBytes,
+      base::CommandLine::StringType(), kChannelString, network_context());
 
   // Now run the main message loop. Make sure the last StartNetLog() was
   // ignored by checking that the next two states are "stopping-log" followed by
   // "not-logging". Also make sure the capture mode matches that of the first
   // StartNetLog() call (called by StartThenVerifyState()).
   std::unique_ptr<base::DictionaryValue> state =
-      test_state_observer_.WaitForNewState();
+      test_state_observer()->WaitForNewState();
   ASSERT_TRUE(VerifyState(std::move(state), kStateStoppingLogString));
-  state = test_state_observer_.WaitForNewState();
+  state = test_state_observer()->WaitForNewState();
   ASSERT_TRUE(VerifyState(std::move(state), kStateNotLoggingString, true, true,
                           kCaptureModeIncludeSocketBytesString));
 }
 
+TEST_F(NetExportFileWriterTest, HandleCrash) {
+  FakeNetworkContext fake_network_context;
+
+  ASSERT_TRUE(InitializeThenVerifyNewState(true, false));
+  ASSERT_TRUE(StartThenVerifyNewState(
+      base::FilePath(), net::NetLogCaptureMode::IncludeSocketBytes(),
+      kCaptureModeIncludeSocketBytesString, &fake_network_context));
+
+  // Break the pipe, as if network service crashed.
+  fake_network_context.Disconnect();
+
+  std::unique_ptr<base::DictionaryValue> state =
+      test_state_observer()->WaitForNewState();
+  ASSERT_TRUE(VerifyState(std::move(state), kStateNotLoggingString));
+}
+
 }  // namespace net_log
