@@ -4,14 +4,14 @@
 
 #include "components/ntp_tiles/icon_cacher_impl.h"
 
-#include <set>
+#include <memory>
 #include <utility>
 
+#include "base/containers/flat_set.h"
 #include "base/files/scoped_temp_dir.h"
-#include "base/memory/ptr_util.h"
 #include "base/path_service.h"
 #include "base/run_loop.h"
-#include "base/test/histogram_tester.h"
+#include "base/test/metrics/histogram_tester.h"
 #include "base/test/mock_callback.h"
 #include "base/test/scoped_task_environment.h"
 #include "base/test/test_simple_task_runner.h"
@@ -19,12 +19,14 @@
 #include "components/favicon/core/favicon_client.h"
 #include "components/favicon/core/favicon_service_impl.h"
 #include "components/favicon/core/favicon_util.h"
-#include "components/favicon/core/large_icon_service.h"
+#include "components/favicon/core/large_icon_service_impl.h"
 #include "components/favicon_base/favicon_types.h"
 #include "components/history/core/browser/history_database_params.h"
 #include "components/history/core/browser/history_service.h"
 #include "components/image_fetcher/core/image_decoder.h"
 #include "components/image_fetcher/core/image_fetcher.h"
+#include "components/image_fetcher/core/mock_image_decoder.h"
+#include "components/image_fetcher/core/mock_image_fetcher.h"
 #include "components/image_fetcher/core/request_metadata.h"
 #include "net/traffic_annotation/network_traffic_annotation_test_helper.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -34,6 +36,7 @@
 #include "ui/gfx/image/image_unittest_util.h"
 
 using base::Bucket;
+using ::image_fetcher::MockImageFetcher;
 using ::testing::_;
 using ::testing::ElementsAre;
 using ::testing::Eq;
@@ -46,31 +49,7 @@ using ::testing::ReturnArg;
 
 namespace ntp_tiles {
 namespace {
-
-class MockImageFetcher : public image_fetcher::ImageFetcher {
- public:
-  MOCK_METHOD1(SetImageFetcherDelegate,
-               void(image_fetcher::ImageFetcherDelegate* delegate));
-  MOCK_METHOD1(SetDataUseServiceName,
-               void(image_fetcher::ImageFetcher::DataUseServiceName name));
-  MOCK_METHOD1(SetImageDownloadLimit,
-               void(base::Optional<int64_t> max_download_bytes));
-  MOCK_METHOD4(StartOrQueueNetworkRequest,
-               void(const std::string& id,
-                    const GURL& image_url,
-                    const ImageFetcherCallback& callback,
-                    const net::NetworkTrafficAnnotationTag&));
-  MOCK_METHOD1(SetDesiredImageFrameSize, void(const gfx::Size&));
-  MOCK_METHOD0(GetImageDecoder, image_fetcher::ImageDecoder*());
-};
-
-class MockImageDecoder : public image_fetcher::ImageDecoder {
- public:
-  MOCK_METHOD3(DecodeImage,
-               void(const std::string& image_data,
-                    const gfx::Size& desired_image_frame_size,
-                    const image_fetcher::ImageDecodedCallback& callback));
-};
+using MockImageDecoder = image_fetcher::MockImageDecoder;
 
 // This class provides methods to inject an image resource where a real resource
 // would be necessary otherwise. All other methods have return values that allow
@@ -104,19 +83,20 @@ class MockResourceDelegate : public ui::ResourceBundle::Delegate {
 
 ACTION(FailFetch) {
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE,
-      base::Bind(arg2, arg0, gfx::Image(), image_fetcher::RequestMetadata()));
+      FROM_HERE, base::BindOnce(std::move(*arg3), arg0, gfx::Image(),
+                                image_fetcher::RequestMetadata()));
 }
 
 ACTION_P2(DecodeSuccessfully, width, height) {
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::Bind(arg2, gfx::test::CreateImage(width, height)));
+      FROM_HERE, base::BindOnce(arg2, gfx::test::CreateImage(width, height)));
 }
 
 ACTION_P2(PassFetch, width, height) {
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::Bind(arg2, arg0, gfx::test::CreateImage(width, height),
-                            image_fetcher::RequestMetadata()));
+      FROM_HERE, base::BindOnce(std::move(*arg3), arg0,
+                                gfx::test::CreateImage(width, height),
+                                image_fetcher::RequestMetadata()));
 }
 
 ACTION_P(Quit, run_loop) {
@@ -140,7 +120,7 @@ class IconCacherTestBase : public ::testing::Test {
                    favicon_base::IconType icon_type,
                    int width,
                    int height) {
-    favicon_service_.SetFavicons(url, icon_url, icon_type,
+    favicon_service_.SetFavicons({url}, icon_url, icon_type,
                                  gfx::test::CreateImage(width, height));
   }
 
@@ -193,7 +173,7 @@ class IconCacherTestPopularSites : public IconCacherTestBase {
               GURL("http://url.google/"),
               GURL("http://url.google/icon.png"),
               GURL("http://url.google/favicon.ico"),
-              GURL()),  // thumbnail, unused
+              TileTitleSource::UNKNOWN),  // title_source, unused
         image_fetcher_(new ::testing::StrictMock<MockImageFetcher>),
         image_decoder_(new ::testing::StrictMock<MockImageDecoder>) {}
 
@@ -207,7 +187,7 @@ class IconCacherTestPopularSites : public IconCacherTestBase {
         .WillByDefault(ReturnArg<0>());
     ui::ResourceBundle::InitSharedInstanceWithLocale(
         "en-US", &mock_resource_delegate_,
-        ui::ResourceBundle::LOAD_COMMON_RESOURCES);
+        ui::ResourceBundle::DO_NOT_LOAD_COMMON_RESOURCES);
   }
 
   void TearDown() override {
@@ -216,13 +196,13 @@ class IconCacherTestPopularSites : public IconCacherTestBase {
     }
     base::FilePath pak_path;
 #if defined(OS_ANDROID)
-    PathService::Get(ui::DIR_RESOURCE_PAKS_ANDROID, &pak_path);
+    base::PathService::Get(ui::DIR_RESOURCE_PAKS_ANDROID, &pak_path);
 #else
-    PathService::Get(base::DIR_MODULE, &pak_path);
+    base::PathService::Get(base::DIR_MODULE, &pak_path);
 #endif
 
     base::FilePath ui_test_pak_path;
-    ASSERT_TRUE(PathService::Get(ui::UI_TEST_PAK, &ui_test_pak_path));
+    ASSERT_TRUE(base::PathService::Get(ui::UI_TEST_PAK, &ui_test_pak_path));
     ui::ResourceBundle::InitSharedInstanceWithPakPath(ui_test_pak_path);
 
     ui::ResourceBundle::GetSharedInstance().AddDataPackFromPath(
@@ -248,13 +228,13 @@ TEST_F(IconCacherTestPopularSites, LargeCached) {
                     data_use_measurement::DataUseUserData::NTP_TILES));
     EXPECT_CALL(*image_fetcher_, SetDesiredImageFrameSize(gfx::Size(128, 128)));
   }
-  PreloadIcon(site_.url, site_.large_icon_url, favicon_base::TOUCH_ICON, 128,
-              128);
+  PreloadIcon(site_.url, site_.large_icon_url,
+              favicon_base::IconType::kTouchIcon, 128, 128);
   IconCacherImpl cacher(&favicon_service_, nullptr, std::move(image_fetcher_));
   cacher.StartFetchPopularSites(site_, done.Get(), done.Get());
   WaitForMainThreadTasksToFinish();
-  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::FAVICON));
-  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::TOUCH_ICON));
+  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::IconType::kFavicon));
+  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::IconType::kTouchIcon));
   EXPECT_THAT(histogram_tester.GetAllSamples(
                   "NewTabPage.TileFaviconFetchSuccess.Popular"),
               IsEmpty());
@@ -271,7 +251,7 @@ TEST_F(IconCacherTestPopularSites, LargeNotCachedAndFetchSucceeded) {
                     data_use_measurement::DataUseUserData::NTP_TILES));
     EXPECT_CALL(*image_fetcher_, SetDesiredImageFrameSize(gfx::Size(128, 128)));
     EXPECT_CALL(*image_fetcher_,
-                StartOrQueueNetworkRequest(_, site_.large_icon_url, _, _))
+                FetchImageAndData_(_, site_.large_icon_url, _, _, _))
         .WillOnce(PassFetch(128, 128));
     EXPECT_CALL(done, Run()).WillOnce(Quit(&loop));
   }
@@ -279,8 +259,8 @@ TEST_F(IconCacherTestPopularSites, LargeNotCachedAndFetchSucceeded) {
   IconCacherImpl cacher(&favicon_service_, nullptr, std::move(image_fetcher_));
   cacher.StartFetchPopularSites(site_, done.Get(), done.Get());
   loop.Run();
-  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::FAVICON));
-  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::TOUCH_ICON));
+  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::IconType::kFavicon));
+  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::IconType::kTouchIcon));
   EXPECT_THAT(histogram_tester.GetAllSamples(
                   "NewTabPage.TileFaviconFetchSuccess.Popular"),
               ElementsAre(Bucket(/*bucket=*/true, /*count=*/1)));
@@ -298,7 +278,7 @@ TEST_F(IconCacherTestPopularSites, SmallNotCachedAndFetchSucceeded) {
                     data_use_measurement::DataUseUserData::NTP_TILES));
     EXPECT_CALL(*image_fetcher_, SetDesiredImageFrameSize(gfx::Size(128, 128)));
     EXPECT_CALL(*image_fetcher_,
-                StartOrQueueNetworkRequest(_, site_.favicon_url, _, _))
+                FetchImageAndData_(_, site_.favicon_url, _, _, _))
         .WillOnce(PassFetch(128, 128));
     EXPECT_CALL(done, Run()).WillOnce(Quit(&loop));
   }
@@ -306,8 +286,8 @@ TEST_F(IconCacherTestPopularSites, SmallNotCachedAndFetchSucceeded) {
   IconCacherImpl cacher(&favicon_service_, nullptr, std::move(image_fetcher_));
   cacher.StartFetchPopularSites(site_, done.Get(), done.Get());
   loop.Run();
-  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::FAVICON));
-  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::TOUCH_ICON));
+  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::IconType::kFavicon));
+  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::IconType::kTouchIcon));
 }
 
 TEST_F(IconCacherTestPopularSites, LargeNotCachedAndFetchFailed) {
@@ -321,39 +301,37 @@ TEST_F(IconCacherTestPopularSites, LargeNotCachedAndFetchFailed) {
                     data_use_measurement::DataUseUserData::NTP_TILES));
     EXPECT_CALL(*image_fetcher_, SetDesiredImageFrameSize(gfx::Size(128, 128)));
     EXPECT_CALL(*image_fetcher_,
-                StartOrQueueNetworkRequest(_, site_.large_icon_url, _, _))
+                FetchImageAndData_(_, site_.large_icon_url, _, _, _))
         .WillOnce(FailFetch());
   }
 
   IconCacherImpl cacher(&favicon_service_, nullptr, std::move(image_fetcher_));
   cacher.StartFetchPopularSites(site_, done.Get(), done.Get());
   WaitForMainThreadTasksToFinish();
-  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::FAVICON));
-  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::TOUCH_ICON));
-  EXPECT_THAT(
-      histogram_tester.GetAllSamples(
-          "NewTabPage.TileFaviconFetchSuccess.Popular"),
-      ElementsAre(Bucket(/*bucket=*/false, /*count=*/1)));
+  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::IconType::kFavicon));
+  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::IconType::kTouchIcon));
+  EXPECT_THAT(histogram_tester.GetAllSamples(
+                  "NewTabPage.TileFaviconFetchSuccess.Popular"),
+              ElementsAre(Bucket(/*bucket=*/false, /*count=*/1)));
 }
 
 TEST_F(IconCacherTestPopularSites, HandlesEmptyCallbacksNicely) {
   base::HistogramTester histogram_tester;
   EXPECT_CALL(*image_fetcher_, SetDataUseServiceName(_));
   EXPECT_CALL(*image_fetcher_, SetDesiredImageFrameSize(_));
-  EXPECT_CALL(*image_fetcher_, StartOrQueueNetworkRequest(_, _, _, _))
+  EXPECT_CALL(*image_fetcher_, FetchImageAndData_(_, _, _, _, _))
       .WillOnce(PassFetch(128, 128));
   IconCacherImpl cacher(&favicon_service_, nullptr, std::move(image_fetcher_));
   cacher.StartFetchPopularSites(site_, base::Closure(), base::Closure());
   WaitForHistoryThreadTasksToFinish();  // Writing the icon into the DB.
   WaitForMainThreadTasksToFinish();     // Finishing tasks after the DB write.
   // Even though the callbacks are not called, the icon gets written out.
-  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::FAVICON));
-  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::TOUCH_ICON));
+  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::IconType::kFavicon));
+  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::IconType::kTouchIcon));
   // The histogram gets reported despite empty callbacks.
-  EXPECT_THAT(
-      histogram_tester.GetAllSamples(
-          "NewTabPage.TileFaviconFetchSuccess.Popular"),
-      ElementsAre(Bucket(/*bucket=*/true, /*count=*/1)));
+  EXPECT_THAT(histogram_tester.GetAllSamples(
+                  "NewTabPage.TileFaviconFetchSuccess.Popular"),
+              ElementsAre(Bucket(/*bucket=*/true, /*count=*/1)));
 }
 
 TEST_F(IconCacherTestPopularSites, ProvidesDefaultIconAndSucceedsWithFetching) {
@@ -378,7 +356,7 @@ TEST_F(IconCacherTestPopularSites, ProvidesDefaultIconAndSucceedsWithFetching) {
                     data_use_measurement::DataUseUserData::NTP_TILES));
     EXPECT_CALL(*image_fetcher_, SetDesiredImageFrameSize(gfx::Size(128, 128)));
     EXPECT_CALL(*image_fetcher_,
-                StartOrQueueNetworkRequest(_, site_.large_icon_url, _, _))
+                FetchImageAndData_(_, site_.large_icon_url, _, _, _))
         .WillOnce(PassFetch(128, 128));
 
     // Both callback are called async after the request but preliminary has to
@@ -394,19 +372,20 @@ TEST_F(IconCacherTestPopularSites, ProvidesDefaultIconAndSucceedsWithFetching) {
                                 preliminary_icon_available.Get());
 
   default_loop.Run();  // Wait for the default image.
-  EXPECT_THAT(GetCachedIconFor(site_.url, favicon_base::TOUCH_ICON).Size(),
-              Eq(gfx::Size(64, 64)));  // Compares dimensions, not objects.
+  EXPECT_THAT(
+      GetCachedIconFor(site_.url, favicon_base::IconType::kTouchIcon).Size(),
+      Eq(gfx::Size(64, 64)));  // Compares dimensions, not objects.
 
   // Let the fetcher continue and wait for the second call of the callback.
   fetch_loop.Run();  // Wait for the updated image.
-  EXPECT_THAT(GetCachedIconFor(site_.url, favicon_base::TOUCH_ICON).Size(),
-              Eq(gfx::Size(128, 128)));  // Compares dimensions, not objects.
+  EXPECT_THAT(
+      GetCachedIconFor(site_.url, favicon_base::IconType::kTouchIcon).Size(),
+      Eq(gfx::Size(128, 128)));  // Compares dimensions, not objects.
   // The histogram gets reported only once (for the downloaded icon, not for the
   // default one).
-  EXPECT_THAT(
-      histogram_tester.GetAllSamples(
-          "NewTabPage.TileFaviconFetchSuccess.Popular"),
-      ElementsAre(Bucket(/*bucket=*/true, /*count=*/1)));
+  EXPECT_THAT(histogram_tester.GetAllSamples(
+                  "NewTabPage.TileFaviconFetchSuccess.Popular"),
+              ElementsAre(Bucket(/*bucket=*/true, /*count=*/1)));
 }
 
 TEST_F(IconCacherTestPopularSites, LargeNotCachedAndFetchPerformedOnlyOnce) {
@@ -420,7 +399,7 @@ TEST_F(IconCacherTestPopularSites, LargeNotCachedAndFetchPerformedOnlyOnce) {
                     data_use_measurement::DataUseUserData::NTP_TILES));
     EXPECT_CALL(*image_fetcher_, SetDesiredImageFrameSize(gfx::Size(128, 128)));
     EXPECT_CALL(*image_fetcher_,
-                StartOrQueueNetworkRequest(_, site_.large_icon_url, _, _))
+                FetchImageAndData_(_, site_.large_icon_url, _, _, _))
         .WillOnce(PassFetch(128, 128));
     // Success will be notified to both requests.
     EXPECT_CALL(done, Run()).WillOnce(Return()).WillOnce(Quit(&loop));
@@ -430,19 +409,17 @@ TEST_F(IconCacherTestPopularSites, LargeNotCachedAndFetchPerformedOnlyOnce) {
   cacher.StartFetchPopularSites(site_, done.Get(), done.Get());
   cacher.StartFetchPopularSites(site_, done.Get(), done.Get());
   loop.Run();
-  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::FAVICON));
-  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::TOUCH_ICON));
+  EXPECT_FALSE(IconIsCachedFor(site_.url, favicon_base::IconType::kFavicon));
+  EXPECT_TRUE(IconIsCachedFor(site_.url, favicon_base::IconType::kTouchIcon));
 }
 
 class IconCacherTestMostLikely : public IconCacherTestBase {
  protected:
   IconCacherTestMostLikely()
-      : large_icon_service_background_task_runner_(
-            new base::TestSimpleTaskRunner()),
-        fetcher_for_large_icon_service_(
-            base::MakeUnique<::testing::StrictMock<MockImageFetcher>>()),
+      : fetcher_for_large_icon_service_(
+            std::make_unique<::testing::StrictMock<MockImageFetcher>>()),
         fetcher_for_icon_cacher_(
-            base::MakeUnique<::testing::StrictMock<MockImageFetcher>>()) {
+            std::make_unique<::testing::StrictMock<MockImageFetcher>>()) {
     // Expect uninteresting calls here, |fetcher_for_icon_cacher_| is not
     // related to these tests. Keep it strict to make sure we do not use it in
     // any other way.
@@ -453,8 +430,6 @@ class IconCacherTestMostLikely : public IconCacherTestBase {
                 SetDesiredImageFrameSize(gfx::Size(128, 128)));
   }
 
-  scoped_refptr<base::TestSimpleTaskRunner>
-      large_icon_service_background_task_runner_;
   std::unique_ptr<MockImageFetcher> fetcher_for_large_icon_service_;
   std::unique_ptr<MockImageFetcher> fetcher_for_icon_cacher_;
 };
@@ -464,21 +439,20 @@ TEST_F(IconCacherTestMostLikely, Cached) {
   base::HistogramTester histogram_tester;
 
   GURL icon_url("http://www.site.com/favicon.png");
-  PreloadIcon(page_url, icon_url, favicon_base::TOUCH_ICON, 128, 128);
+  PreloadIcon(page_url, icon_url, favicon_base::IconType::kTouchIcon, 128, 128);
 
-  favicon::LargeIconService large_icon_service(
-      &favicon_service_, large_icon_service_background_task_runner_,
-      std::move(fetcher_for_large_icon_service_));
+  favicon::LargeIconServiceImpl large_icon_service(
+      &favicon_service_, std::move(fetcher_for_large_icon_service_));
   IconCacherImpl cacher(&favicon_service_, &large_icon_service,
                         std::move(fetcher_for_icon_cacher_));
 
   base::MockCallback<base::Closure> done;
   EXPECT_CALL(done, Run()).Times(0);
   cacher.StartFetchMostLikely(page_url, done.Get());
-  WaitForMainThreadTasksToFinish();
+  scoped_task_environment_.RunUntilIdle();
 
-  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::FAVICON));
-  EXPECT_TRUE(IconIsCachedFor(page_url, favicon_base::TOUCH_ICON));
+  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::IconType::kFavicon));
+  EXPECT_TRUE(IconIsCachedFor(page_url, favicon_base::IconType::kTouchIcon));
   EXPECT_THAT(histogram_tester.GetAllSamples(
                   "NewTabPage.TileFaviconFetchStatus.Server"),
               IsEmpty());
@@ -496,14 +470,13 @@ TEST_F(IconCacherTestMostLikely, NotCachedAndFetchSucceeded) {
                 SetDataUseServiceName(
                     data_use_measurement::DataUseUserData::LARGE_ICON_SERVICE));
     EXPECT_CALL(*fetcher_for_large_icon_service_,
-                StartOrQueueNetworkRequest(_, _, _, _))
+                FetchImageAndData_(_, _, _, _, _))
         .WillOnce(PassFetch(128, 128));
     EXPECT_CALL(done, Run()).WillOnce(Quit(&loop));
   }
 
-  favicon::LargeIconService large_icon_service(
-      &favicon_service_, large_icon_service_background_task_runner_,
-      std::move(fetcher_for_large_icon_service_));
+  favicon::LargeIconServiceImpl large_icon_service(
+      &favicon_service_, std::move(fetcher_for_large_icon_service_));
   IconCacherImpl cacher(&favicon_service_, &large_icon_service,
                         std::move(fetcher_for_icon_cacher_));
 
@@ -511,11 +484,11 @@ TEST_F(IconCacherTestMostLikely, NotCachedAndFetchSucceeded) {
   // Both these task runners need to be flushed in order to get |done| called by
   // running the main loop.
   WaitForHistoryThreadTasksToFinish();
-  large_icon_service_background_task_runner_->RunUntilIdle();
+  scoped_task_environment_.RunUntilIdle();
 
   loop.Run();
-  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::FAVICON));
-  EXPECT_TRUE(IconIsCachedFor(page_url, favicon_base::TOUCH_ICON));
+  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::IconType::kFavicon));
+  EXPECT_TRUE(IconIsCachedFor(page_url, favicon_base::IconType::kTouchIcon));
   EXPECT_THAT(histogram_tester.GetAllSamples(
                   "NewTabPage.TileFaviconFetchStatus.Server"),
               ElementsAre(Bucket(
@@ -535,14 +508,13 @@ TEST_F(IconCacherTestMostLikely, NotCachedAndFetchFailed) {
                 SetDataUseServiceName(
                     data_use_measurement::DataUseUserData::LARGE_ICON_SERVICE));
     EXPECT_CALL(*fetcher_for_large_icon_service_,
-                StartOrQueueNetworkRequest(_, _, _, _))
+                FetchImageAndData_(_, _, _, _, _))
         .WillOnce(FailFetch());
     EXPECT_CALL(done, Run()).Times(0);
   }
 
-  favicon::LargeIconService large_icon_service(
-      &favicon_service_, large_icon_service_background_task_runner_,
-      std::move(fetcher_for_large_icon_service_));
+  favicon::LargeIconServiceImpl large_icon_service(
+      &favicon_service_, std::move(fetcher_for_large_icon_service_));
   IconCacherImpl cacher(&favicon_service_, &large_icon_service,
                         std::move(fetcher_for_icon_cacher_));
 
@@ -550,11 +522,10 @@ TEST_F(IconCacherTestMostLikely, NotCachedAndFetchFailed) {
   // Both these task runners need to be flushed before flushing the main thread
   // queue in order to finish the work.
   WaitForHistoryThreadTasksToFinish();
-  large_icon_service_background_task_runner_->RunUntilIdle();
-  WaitForMainThreadTasksToFinish();
+  scoped_task_environment_.RunUntilIdle();
 
-  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::FAVICON));
-  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::TOUCH_ICON));
+  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::IconType::kFavicon));
+  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::IconType::kTouchIcon));
   EXPECT_THAT(histogram_tester.GetAllSamples(
                   "NewTabPage.TileFaviconFetchStatus.Server"),
               ElementsAre(Bucket(
@@ -569,25 +540,30 @@ TEST_F(IconCacherTestMostLikely, HandlesEmptyCallbacksNicely) {
 
   EXPECT_CALL(*fetcher_for_large_icon_service_, SetDataUseServiceName(_));
   EXPECT_CALL(*fetcher_for_large_icon_service_,
-              StartOrQueueNetworkRequest(_, _, _, _))
+              FetchImageAndData_(_, _, _, _, _))
       .WillOnce(PassFetch(128, 128));
 
-  favicon::LargeIconService large_icon_service(
-      &favicon_service_, large_icon_service_background_task_runner_,
-      std::move(fetcher_for_large_icon_service_));
+  favicon::LargeIconServiceImpl large_icon_service(
+      &favicon_service_, std::move(fetcher_for_large_icon_service_));
   IconCacherImpl cacher(&favicon_service_, &large_icon_service,
                         std::move(fetcher_for_icon_cacher_));
 
   cacher.StartFetchMostLikely(page_url, base::Closure());
-  // Both these task runners need to be flushed before flushing the main thread
-  // queue in order to finish the work.
+
+  // Finish the posted tasks on the main and the history thread to find out if
+  // we can write the icon.
+  scoped_task_environment_.RunUntilIdle();
   WaitForHistoryThreadTasksToFinish();
-  large_icon_service_background_task_runner_->RunUntilIdle();
-  WaitForMainThreadTasksToFinish();
+  // Continue with the work in large icon service - fetch and decode the data.
+  scoped_task_environment_.RunUntilIdle();
+  // Do the work on the history thread to write down the icon
+  WaitForHistoryThreadTasksToFinish();
+  // Finish the tasks on the main thread.
+  scoped_task_environment_.RunUntilIdle();
 
   // Even though the callbacks are not called, the icon gets written out.
-  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::FAVICON));
-  EXPECT_TRUE(IconIsCachedFor(page_url, favicon_base::TOUCH_ICON));
+  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::IconType::kFavicon));
+  EXPECT_TRUE(IconIsCachedFor(page_url, favicon_base::IconType::kTouchIcon));
 }
 
 TEST_F(IconCacherTestMostLikely, NotCachedAndFetchPerformedOnlyOnce) {
@@ -602,28 +578,33 @@ TEST_F(IconCacherTestMostLikely, NotCachedAndFetchPerformedOnlyOnce) {
                 SetDataUseServiceName(
                     data_use_measurement::DataUseUserData::LARGE_ICON_SERVICE));
     EXPECT_CALL(*fetcher_for_large_icon_service_,
-                StartOrQueueNetworkRequest(_, _, _, _))
+                FetchImageAndData_(_, _, _, _, _))
         .WillOnce(PassFetch(128, 128));
     // Success will be notified to both requests.
     EXPECT_CALL(done, Run()).WillOnce(Return()).WillOnce(Quit(&loop));
   }
 
-  favicon::LargeIconService large_icon_service(
-      &favicon_service_, large_icon_service_background_task_runner_,
-      std::move(fetcher_for_large_icon_service_));
+  favicon::LargeIconServiceImpl large_icon_service(
+      &favicon_service_, std::move(fetcher_for_large_icon_service_));
   IconCacherImpl cacher(&favicon_service_, &large_icon_service,
                         std::move(fetcher_for_icon_cacher_));
 
   cacher.StartFetchMostLikely(page_url, done.Get());
   cacher.StartFetchMostLikely(page_url, done.Get());
-  // Both these task runners need to be flushed in order to get |done| called by
-  // running the main loop.
-  WaitForHistoryThreadTasksToFinish();
-  large_icon_service_background_task_runner_->RunUntilIdle();
 
+  // Finish the posted tasks on the main and the history thread to find out if
+  // we can write the icon.
+  scoped_task_environment_.RunUntilIdle();
+  WaitForHistoryThreadTasksToFinish();
+  // Continue with the work in large icon service - fetch and decode the data.
+  scoped_task_environment_.RunUntilIdle();
+  // Do the work on the history thread to write down the icon
+  WaitForHistoryThreadTasksToFinish();
+  // Finish the tasks on the main thread.
   loop.Run();
-  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::FAVICON));
-  EXPECT_TRUE(IconIsCachedFor(page_url, favicon_base::TOUCH_ICON));
+
+  EXPECT_FALSE(IconIsCachedFor(page_url, favicon_base::IconType::kFavicon));
+  EXPECT_TRUE(IconIsCachedFor(page_url, favicon_base::IconType::kTouchIcon));
 }
 
 }  // namespace
