@@ -6,7 +6,8 @@
 
 namespace ntp_tiles {
 
-NTPTile::NTPTile() : source(TileSource::TOP_SITES) {}
+NTPTile::NTPTile()
+    : title_source(TileTitleSource::UNKNOWN), source(TileSource::TOP_SITES) {}
 
 NTPTile::NTPTile(const NTPTile&) = default;
 
@@ -14,8 +15,8 @@ NTPTile::~NTPTile() {}
 
 bool operator==(const NTPTile& a, const NTPTile& b) {
   return (a.title == b.title) && (a.url == b.url) && (a.source == b.source) &&
+         (a.title_source == b.title_source) &&
          (a.whitelist_icon_path == b.whitelist_icon_path) &&
-         (a.thumbnail_url == b.thumbnail_url) &&
          (a.favicon_url == b.favicon_url);
 }
 
