@@ -10,7 +10,9 @@
 #include "base/files/file_path.h"
 #include "base/macros.h"
 #include "base/strings/string16.h"
+#include "base/time/time.h"
 #include "components/ntp_tiles/tile_source.h"
+#include "components/ntp_tiles/tile_title_source.h"
 #include "url/gurl.h"
 
 namespace ntp_tiles {
@@ -19,6 +21,7 @@ namespace ntp_tiles {
 struct NTPTile {
   base::string16 title;
   GURL url;
+  TileTitleSource title_source;
   TileSource source;
 
   // Empty unless whitelists are enabled and this site is in a whitelist.
@@ -26,12 +29,13 @@ struct NTPTile {
   // is also available from another, higher-priority source.
   base::FilePath whitelist_icon_path;
 
-  // Only valid for source == SUGGESTIONS_SERVICE (empty otherwise).
-  // May point to a local chrome:// URL or to a remote one. May be empty.
-  GURL thumbnail_url;
   // This won't be empty, but might 404 etc.
   GURL favicon_url;
 
+  // Timestamp representing when the tile was originally generated (produced by
+  // a ranking algorithm).
+  base::Time data_generation_time;
+
   NTPTile();
   NTPTile(const NTPTile&);
   ~NTPTile();
