@@ -5,20 +5,23 @@
 #include "components/ntp_tiles/popular_sites_impl.h"
 
 #include <stddef.h>
+#include <map>
+#include <memory>
 #include <utility>
 
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/feature_list.h"
+#include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
+#include "base/strings/utf_string_conversions.h"
 #include "base/time/time.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "components/data_use_measurement/core/data_use_user_data.h"
-#include "components/google/core/browser/google_util.h"
+#include "components/google/core/common/google_util.h"
 #include "components/ntp_tiles/constants.h"
-#include "components/ntp_tiles/field_trial.h"
 #include "components/ntp_tiles/pref_names.h"
 #include "components/ntp_tiles/switches.h"
 #include "components/pref_registry/pref_registry_syncable.h"
@@ -30,6 +33,9 @@
 #include "net/base/load_flags.h"
 #include "net/http/http_status_code.h"
 #include "net/traffic_annotation/network_traffic_annotation.h"
+#include "services/network/public/cpp/resource_request.h"
+#include "services/network/public/cpp/shared_url_loader_factory.h"
+#include "services/network/public/cpp/simple_url_loader.h"
 
 #if defined(OS_ANDROID) || defined(OS_IOS)
 #include "base/json/json_reader.h"
@@ -41,7 +47,6 @@
 #include "components/ntp_tiles/country_code_ios.h"
 #endif
 
-using net::URLFetcher;
 using variations::VariationsService;
 
 namespace ntp_tiles {
@@ -53,12 +58,9 @@ const char kPopularSitesURLFormat[] =
 const char kPopularSitesDefaultDirectory[] = "chrome/ntp/";
 const char kPopularSitesDefaultCountryCode[] = "DEFAULT";
 const char kPopularSitesDefaultVersion[] = "5";
+const int kSitesExplorationStartVersion = 6;
 const int kPopularSitesRedownloadIntervalHours = 24;
 
-const char kPopularSitesLastDownloadPref[] = "popular_sites_last_download";
-const char kPopularSitesURLPref[] = "popular_sites_url";
-const char kPopularSitesJsonPref[] = "suggested_sites_json";
-
 GURL GetPopularSitesURL(const std::string& directory,
                         const std::string& country,
                         const std::string& version) {
@@ -107,6 +109,11 @@ std::string GetVariationVersion() {
                                             "version");
 }
 
+std::string GetVariationDirectory() {
+  return variations::GetVariationParamValue(kPopularSitesFieldTrialName,
+                                            "directory");
+}
+
 PopularSites::SitesVector ParseSiteList(const base::ListValue& list) {
   PopularSites::SitesVector sites;
   for (size_t i = 0; i < list.GetSize(); i++) {
@@ -119,19 +126,81 @@ PopularSites::SitesVector ParseSiteList(const base::ListValue& list) {
       continue;
     std::string favicon_url;
     item->GetString("favicon_url", &favicon_url);
-    std::string thumbnail_url;
-    item->GetString("thumbnail_url", &thumbnail_url);
     std::string large_icon_url;
     item->GetString("large_icon_url", &large_icon_url);
 
+    TileTitleSource title_source = TileTitleSource::UNKNOWN;
+    int title_source_int;
+    if (!item->GetInteger("title_source", &title_source_int)) {
+      // Only v6 and later have "title_source". Earlier versions use title tags.
+      title_source = TileTitleSource::TITLE_TAG;
+    } else if (title_source_int <= static_cast<int>(TileTitleSource::LAST) &&
+               title_source_int >= 0) {
+      title_source = static_cast<TileTitleSource>(title_source_int);
+    }
+
     sites.emplace_back(title, GURL(url), GURL(favicon_url),
-                       GURL(large_icon_url), GURL(thumbnail_url));
+                       GURL(large_icon_url), title_source);
     item->GetInteger("default_icon_resource",
                      &sites.back().default_icon_resource);
+    item->GetBoolean("baked_in", &sites.back().baked_in);
   }
   return sites;
 }
 
+std::map<SectionType, PopularSites::SitesVector> ParseVersion5(
+    const base::ListValue& list) {
+  return {{SectionType::PERSONALIZED, ParseSiteList(list)}};
+}
+
+std::map<SectionType, PopularSites::SitesVector> ParseVersion6OrAbove(
+    const base::ListValue& list) {
+  // Valid lists would have contained at least the PERSONALIZED section.
+  std::map<SectionType, PopularSites::SitesVector> sections = {
+      std::make_pair(SectionType::PERSONALIZED, PopularSites::SitesVector{})};
+  for (size_t i = 0; i < list.GetSize(); i++) {
+    const base::DictionaryValue* item;
+    if (!list.GetDictionary(i, &item)) {
+      LOG(WARNING) << "Parsed SitesExploration list contained an invalid "
+                   << "section at position " << i << ".";
+      continue;
+    }
+    int section;
+    if (!item->GetInteger("section", &section) || section < 0 ||
+        section > static_cast<int>(SectionType::LAST)) {
+      LOG(WARNING) << "Parsed SitesExploration list contained a section with "
+                   << "invalid ID (" << section << ")";
+      continue;
+    }
+    SectionType section_type = static_cast<SectionType>(section);
+    if (section_type == SectionType::UNKNOWN) {
+      LOG(WARNING) << "Dropped an unknown section in SitesExploration list.";
+      continue;
+    }
+    const base::ListValue* sites_list;
+    if (!item->GetList("sites", &sites_list)) {
+      continue;
+    }
+    sections[section_type] = ParseSiteList(*sites_list);
+  }
+  if (!base::FeatureList::IsEnabled(kSiteExplorationUiFeature)) {
+    // New versions of popular sites that should act like old versions will
+    // mimic having only the personalized list.
+    return {std::make_pair(SectionType::PERSONALIZED,
+                           std::move(sections[SectionType::PERSONALIZED]))};
+  }
+  return sections;
+}
+
+std::map<SectionType, PopularSites::SitesVector> ParseSites(
+    const base::ListValue& list,
+    int version) {
+  if (version >= kSitesExplorationStartVersion) {
+    return ParseVersion6OrAbove(list);
+  }
+  return ParseVersion5(list);
+}
+
 #if defined(GOOGLE_CHROME_BUILD) && (defined(OS_ANDROID) || defined(OS_IOS))
 void SetDefaultResourceForSite(int index,
                                int resource_id,
@@ -147,16 +216,20 @@ void SetDefaultResourceForSite(int index,
 // Creates the list of popular sites based on a snapshot available for mobile.
 std::unique_ptr<base::ListValue> DefaultPopularSites() {
 #if !defined(OS_ANDROID) && !defined(OS_IOS)
-  return base::MakeUnique<base::ListValue>();
+  return std::make_unique<base::ListValue>();
 #else
   if (!base::FeatureList::IsEnabled(kPopularSitesBakedInContentFeature)) {
-    return base::MakeUnique<base::ListValue>();
+    return std::make_unique<base::ListValue>();
   }
   std::unique_ptr<base::ListValue> sites =
       base::ListValue::From(base::JSONReader::Read(
-          ResourceBundle::GetSharedInstance().GetRawDataResource(
+          ui::ResourceBundle::GetSharedInstance().GetRawDataResource(
               IDR_DEFAULT_POPULAR_SITES_JSON)));
   DCHECK(sites);
+  for (base::Value& site : *sites) {
+    base::DictionaryValue& dict = static_cast<base::DictionaryValue&>(site);
+    dict.SetBoolean("baked_in", true);
+  }
 #if defined(GOOGLE_CHROME_BUILD)
   int index = 0;
   for (int icon_resource :
@@ -177,12 +250,13 @@ PopularSites::Site::Site(const base::string16& title,
                          const GURL& url,
                          const GURL& favicon_url,
                          const GURL& large_icon_url,
-                         const GURL& thumbnail_url)
+                         TileTitleSource title_source)
     : title(title),
       url(url),
       favicon_url(favicon_url),
       large_icon_url(large_icon_url),
-      thumbnail_url(thumbnail_url),
+      title_source(title_source),
+      baked_in(false),
       default_icon_resource(-1) {}
 
 PopularSites::Site::Site(const Site& other) = default;
@@ -193,15 +267,17 @@ PopularSitesImpl::PopularSitesImpl(
     PrefService* prefs,
     const TemplateURLService* template_url_service,
     VariationsService* variations_service,
-    net::URLRequestContextGetter* download_context,
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
     ParseJSONCallback parse_json)
     : prefs_(prefs),
       template_url_service_(template_url_service),
       variations_(variations_service),
-      download_context_(download_context),
+      url_loader_factory_(std::move(url_loader_factory)),
       parse_json_(std::move(parse_json)),
       is_fallback_(false),
-      sites_(ParseSiteList(*prefs->GetList(kPopularSitesJsonPref))),
+      sections_(
+          ParseSites(*prefs->GetList(prefs::kPopularSitesJsonPref),
+                     prefs_->GetInteger(prefs::kPopularSitesVersionPref))),
       weak_ptr_factory_(this) {}
 
 PopularSitesImpl::~PopularSitesImpl() {}
@@ -212,7 +288,7 @@ bool PopularSitesImpl::MaybeStartFetch(bool force_download,
   callback_ = callback;
 
   const base::Time last_download_time = base::Time::FromInternalValue(
-      prefs_->GetInt64(kPopularSitesLastDownloadPref));
+      prefs_->GetInt64(prefs::kPopularSitesLastDownloadPref));
   const base::TimeDelta time_since_last_download =
       base::Time::Now() - last_download_time;
   const base::TimeDelta redownload_interval =
@@ -221,7 +297,7 @@ bool PopularSitesImpl::MaybeStartFetch(bool force_download,
 
   pending_url_ = GetURLToFetch();
   const bool url_changed =
-      pending_url_.spec() != prefs_->GetString(kPopularSitesURLPref);
+      pending_url_.spec() != prefs_->GetString(prefs::kPopularSitesURLPref);
 
   // Download forced, or we need to download a new file.
   if (force_download || download_time_is_future ||
@@ -232,12 +308,13 @@ bool PopularSitesImpl::MaybeStartFetch(bool force_download,
   return false;
 }
 
-const PopularSites::SitesVector& PopularSitesImpl::sites() const {
-  return sites_;
+const std::map<SectionType, PopularSitesImpl::SitesVector>&
+PopularSitesImpl::sections() const {
+  return sections_;
 }
 
 GURL PopularSitesImpl::GetLastURLFetched() const {
-  return GURL(prefs_->GetString(kPopularSitesURLPref));
+  return GURL(prefs_->GetString(prefs::kPopularSitesURLPref));
 }
 
 GURL PopularSitesImpl::GetURLToFetch() {
@@ -245,6 +322,18 @@ GURL PopularSitesImpl::GetURLToFetch() {
   const std::string country = GetCountryToFetch();
   const std::string version = GetVersionToFetch();
 
+  if (!base::StringToInt(version, &version_in_pending_url_)) {
+    // Parses the leading digits as version. Defaults to 0 if that failed.
+    if (version_in_pending_url_ <= 0) {
+      bool success = base::StringToInt(kPopularSitesDefaultVersion,
+                                       &version_in_pending_url_);
+      DLOG(WARNING) << "The set version \"" << version << "\" does not start "
+                    << "with a valid version number. Default version was used "
+                    << "instead (" << kPopularSitesDefaultVersion << ").";
+      DCHECK(success);
+    }
+  }
+
   const GURL override_url =
       GURL(prefs_->GetString(ntp_tiles::prefs::kPopularSitesOverrideURL));
   return override_url.is_valid()
@@ -256,6 +345,9 @@ std::string PopularSitesImpl::GetDirectoryToFetch() {
   std::string directory =
       prefs_->GetString(ntp_tiles::prefs::kPopularSitesOverrideDirectory);
 
+  if (directory.empty())
+    directory = GetVariationDirectory();
+
   if (directory.empty())
     directory = kPopularSitesDefaultDirectory;
 
@@ -311,7 +403,7 @@ std::string PopularSitesImpl::GetVersionToFetch() {
 }
 
 const base::ListValue* PopularSitesImpl::GetCachedJson() {
-  return prefs_->GetList(kPopularSitesJsonPref);
+  return prefs_->GetList(prefs::kPopularSitesJsonPref);
 }
 
 // static
@@ -326,9 +418,13 @@ void PopularSitesImpl::RegisterProfilePrefs(
   user_prefs->RegisterStringPref(ntp_tiles::prefs::kPopularSitesOverrideVersion,
                                  std::string());
 
-  user_prefs->RegisterInt64Pref(kPopularSitesLastDownloadPref, 0);
-  user_prefs->RegisterStringPref(kPopularSitesURLPref, std::string());
-  user_prefs->RegisterListPref(kPopularSitesJsonPref, DefaultPopularSites());
+  user_prefs->RegisterInt64Pref(prefs::kPopularSitesLastDownloadPref, 0);
+  user_prefs->RegisterStringPref(prefs::kPopularSitesURLPref, std::string());
+  user_prefs->RegisterListPref(prefs::kPopularSitesJsonPref,
+                               DefaultPopularSites());
+  int version;
+  base::StringToInt(kPopularSitesDefaultVersion, &version);
+  user_prefs->RegisterIntegerPref(prefs::kPopularSitesVersionPref, version);
 }
 
 void PopularSitesImpl::FetchPopularSites() {
@@ -347,35 +443,38 @@ void PopularSitesImpl::FetchPopularSites() {
           destination: GOOGLE_OWNED_SERVICE
         }
         policy {
-          cookies_allowed: false
+          cookies_allowed: NO
           setting: "This feature cannot be disabled in settings."
           policy_exception_justification:
             "Not implemented, considered not useful."
         })");
-  fetcher_ = URLFetcher::Create(pending_url_, URLFetcher::GET, this,
-                                traffic_annotation);
-  data_use_measurement::DataUseUserData::AttachToFetcher(
-      fetcher_.get(), data_use_measurement::DataUseUserData::NTP_TILES);
-  fetcher_->SetRequestContext(download_context_);
-  fetcher_->SetLoadFlags(net::LOAD_DO_NOT_SEND_COOKIES |
-                         net::LOAD_DO_NOT_SAVE_COOKIES);
-  fetcher_->SetAutomaticallyRetryOnNetworkChanges(1);
-  fetcher_->Start();
+  auto resource_request = std::make_unique<network::ResourceRequest>();
+  resource_request->url = pending_url_;
+  resource_request->load_flags =
+      net::LOAD_DO_NOT_SEND_COOKIES | net::LOAD_DO_NOT_SAVE_COOKIES;
+  // TODO(https://crbug.com/808498): Re-add data use measurement once
+  // SimpleURLLoader supports it.
+  // ID=data_use_measurement::DataUseUserData::NTP_TILES
+  simple_url_loader_ = network::SimpleURLLoader::Create(
+      std::move(resource_request), traffic_annotation);
+  simple_url_loader_->SetRetryOptions(
+      1, network::SimpleURLLoader::RETRY_ON_NETWORK_CHANGE);
+  simple_url_loader_->DownloadToStringOfUnboundedSizeUntilCrashAndDie(
+      url_loader_factory_.get(),
+      base::BindOnce(&PopularSitesImpl::OnSimpleLoaderComplete,
+                     base::Unretained(this)));
 }
 
-void PopularSitesImpl::OnURLFetchComplete(const net::URLFetcher* source) {
-  DCHECK_EQ(fetcher_.get(), source);
-  std::unique_ptr<net::URLFetcher> free_fetcher = std::move(fetcher_);
+void PopularSitesImpl::OnSimpleLoaderComplete(
+    std::unique_ptr<std::string> response_body) {
+  simple_url_loader_.reset();
 
-  std::string json_string;
-  if (!(source->GetStatus().is_success() &&
-        source->GetResponseCode() == net::HTTP_OK &&
-        source->GetResponseAsString(&json_string))) {
+  if (!response_body) {
     OnDownloadFailed();
     return;
   }
 
-  parse_json_.Run(json_string,
+  parse_json_.Run(*response_body,
                   base::Bind(&PopularSitesImpl::OnJsonParsed,
                              weak_ptr_factory_.GetWeakPtr()),
                   base::Bind(&PopularSitesImpl::OnJsonParseFailed,
@@ -390,13 +489,13 @@ void PopularSitesImpl::OnJsonParsed(std::unique_ptr<base::Value> json) {
     OnDownloadFailed();
     return;
   }
-
-  prefs_->Set(kPopularSitesJsonPref, *list);
-  prefs_->SetInt64(kPopularSitesLastDownloadPref,
+  prefs_->Set(prefs::kPopularSitesJsonPref, *list);
+  prefs_->SetInt64(prefs::kPopularSitesLastDownloadPref,
                    base::Time::Now().ToInternalValue());
-  prefs_->SetString(kPopularSitesURLPref, pending_url_.spec());
+  prefs_->SetInteger(prefs::kPopularSitesVersionPref, version_in_pending_url_);
+  prefs_->SetString(prefs::kPopularSitesURLPref, pending_url_.spec());
 
-  sites_ = ParseSiteList(*list);
+  sections_ = ParseSites(*list, version_in_pending_url_);
   callback_.Run(true);
 }
 
