@@ -4,22 +4,24 @@
 
 #include "components/offline_pages/core/background/pick_request_task.h"
 
-#include <deque>
 #include <memory>
 #include <set>
+#include <vector>
 
 #include "base/bind.h"
-#include "base/test/test_simple_task_runner.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/containers/circular_deque.h"
 #include "base/time/time.h"
 #include "components/offline_pages/core/background/device_conditions.h"
 #include "components/offline_pages/core/background/offliner_policy.h"
 #include "components/offline_pages/core/background/request_coordinator.h"
 #include "components/offline_pages/core/background/request_coordinator_event_logger.h"
 #include "components/offline_pages/core/background/request_notifier.h"
-#include "components/offline_pages/core/background/request_queue_in_memory_store.h"
 #include "components/offline_pages/core/background/request_queue_store.h"
+#include "components/offline_pages/core/background/request_queue_task_test_base.h"
 #include "components/offline_pages/core/background/save_page_request.h"
+#include "components/offline_pages/core/background/test_request_queue_store.h"
+#include "components/offline_pages/core/client_policy_controller.h"
+#include "components/offline_pages/core/offline_clock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace offline_pages {
@@ -50,7 +52,6 @@ const SavePageRequest kEmptyRequest(0UL,
                                     ClientId("", ""),
                                     base::Time(),
                                     true);
-}  // namespace
 
 // Helper class needed by the PickRequestTask
 class RequestNotifierStub : public RequestNotifier {
@@ -88,22 +89,23 @@ class RequestNotifierStub : public RequestNotifier {
   int32_t total_expired_requests_;
 };
 
-class PickRequestTaskTest : public testing::Test {
+class PickRequestTaskTest : public RequestQueueTaskTestBase {
  public:
-  PickRequestTaskTest();
-
-  ~PickRequestTaskTest() override;
+  PickRequestTaskTest() {}
+  ~PickRequestTaskTest() override {}
 
   void SetUp() override;
 
-  void PumpLoop();
-
   void AddRequestDone(ItemActionStatus status);
 
-  void RequestPicked(const SavePageRequest& request, bool cleanup_needed);
+  void RequestPicked(
+      const SavePageRequest& request,
+      const std::unique_ptr<std::vector<SavePageRequest>> available_requests,
+      bool cleanup_needed);
 
   void RequestNotPicked(const bool non_user_requested_tasks_remaining,
-                        bool cleanup_needed);
+                        bool cleanup_needed,
+                        base::Time available_time);
 
   void RequestCountCallback(size_t total_count, size_t available_count);
 
@@ -120,36 +122,23 @@ class PickRequestTaskTest : public testing::Test {
   void TaskCompletionCallback(Task* completed_task);
 
  protected:
-  void InitializeStoreDone(bool success);
-
-  std::unique_ptr<RequestQueueStore> store_;
   std::unique_ptr<RequestNotifierStub> notifier_;
   std::unique_ptr<SavePageRequest> last_picked_;
   std::unique_ptr<OfflinerPolicy> policy_;
+  ClientPolicyController policy_controller_;
   RequestCoordinatorEventLogger event_logger_;
   std::set<int64_t> disabled_requests_;
-  std::deque<int64_t> prioritized_requests_;
+  base::circular_deque<int64_t> prioritized_requests_;
   std::unique_ptr<PickRequestTask> task_;
   bool request_queue_not_picked_called_;
   bool cleanup_needed_;
   size_t total_request_count_;
   size_t available_request_count_;
   bool task_complete_called_;
-
- private:
-  scoped_refptr<base::TestSimpleTaskRunner> task_runner_;
-  base::ThreadTaskRunnerHandle task_runner_handle_;
 };
 
-PickRequestTaskTest::PickRequestTaskTest()
-    : task_runner_(new base::TestSimpleTaskRunner),
-      task_runner_handle_(task_runner_) {}
-
-PickRequestTaskTest::~PickRequestTaskTest() {}
-
 void PickRequestTaskTest::SetUp() {
   DeviceConditions conditions;
-  store_.reset(new RequestQueueInMemoryStore());
   policy_.reset(new OfflinerPolicy());
   notifier_.reset(new RequestNotifierStub());
   MakePickRequestTask();
@@ -160,30 +149,28 @@ void PickRequestTaskTest::SetUp() {
   last_picked_.reset();
   cleanup_needed_ = false;
 
-  store_->Initialize(base::Bind(&PickRequestTaskTest::InitializeStoreDone,
-                                base::Unretained(this)));
+  InitializeStore();
   PumpLoop();
 }
 
-void PickRequestTaskTest::PumpLoop() {
-  task_runner_->RunUntilIdle();
-}
-
 void PickRequestTaskTest::TaskCompletionCallback(Task* completed_task) {
   task_complete_called_ = true;
 }
 
 void PickRequestTaskTest::AddRequestDone(ItemActionStatus status) {}
 
-void PickRequestTaskTest::RequestPicked(const SavePageRequest& request,
-                                        const bool cleanup_needed) {
+void PickRequestTaskTest::RequestPicked(
+    const SavePageRequest& request,
+    std::unique_ptr<std::vector<SavePageRequest>> available_requests,
+    const bool cleanup_needed) {
   last_picked_.reset(new SavePageRequest(request));
   cleanup_needed_ = cleanup_needed;
 }
 
 void PickRequestTaskTest::RequestNotPicked(
     const bool non_user_requested_tasks_remaining,
-    const bool cleanup_needed) {
+    const bool cleanup_needed,
+    base::Time available_time) {
   request_queue_not_picked_called_ = true;
 }
 
@@ -199,10 +186,12 @@ void PickRequestTaskTest::QueueRequests(const SavePageRequest& request1,
   DeviceConditions conditions;
   std::set<int64_t> disabled_requests;
   // Add test requests on the Queue.
-  store_->AddRequest(request1, base::Bind(&PickRequestTaskTest::AddRequestDone,
-                                          base::Unretained(this)));
-  store_->AddRequest(request2, base::Bind(&PickRequestTaskTest::AddRequestDone,
-                                          base::Unretained(this)));
+  store_.AddRequest(request1,
+                    base::BindOnce(&PickRequestTaskTest::AddRequestDone,
+                                   base::Unretained(this)));
+  store_.AddRequest(request2,
+                    base::BindOnce(&PickRequestTaskTest::AddRequestDone,
+                                   base::Unretained(this)));
 
   // Pump the loop to give the async queue the opportunity to do the adds.
   PumpLoop();
@@ -211,21 +200,17 @@ void PickRequestTaskTest::QueueRequests(const SavePageRequest& request1,
 void PickRequestTaskTest::MakePickRequestTask() {
   DeviceConditions conditions;
   task_.reset(new PickRequestTask(
-      store_.get(), policy_.get(),
-      base::Bind(&PickRequestTaskTest::RequestPicked, base::Unretained(this)),
-      base::Bind(&PickRequestTaskTest::RequestNotPicked,
-                 base::Unretained(this)),
-      base::Bind(&PickRequestTaskTest::RequestCountCallback,
-                 base::Unretained(this)),
-      conditions, disabled_requests_, prioritized_requests_));
+      &store_, policy_.get(), &policy_controller_,
+      base::BindOnce(&PickRequestTaskTest::RequestPicked,
+                     base::Unretained(this)),
+      base::BindOnce(&PickRequestTaskTest::RequestNotPicked,
+                     base::Unretained(this)),
+      base::BindOnce(&PickRequestTaskTest::RequestCountCallback,
+                     base::Unretained(this)),
+      conditions, disabled_requests_, &prioritized_requests_));
   task_->SetTaskCompletionCallbackForTesting(
-      task_runner_.get(),
-      base::Bind(&PickRequestTaskTest::TaskCompletionCallback,
-                 base::Unretained(this)));
-}
-
-void PickRequestTaskTest::InitializeStoreDone(bool success) {
-  ASSERT_TRUE(success);
+      base::BindRepeating(&PickRequestTaskTest::TaskCompletionCallback,
+                          base::Unretained(this)));
 }
 
 TEST_F(PickRequestTaskTest, PickFromEmptyQueue) {
@@ -250,7 +235,7 @@ TEST_F(PickRequestTaskTest, ChooseRequestWithHigherRetryCount) {
       kMaxCompletedTries + 1, kBackgroundProcessingTimeBudgetSeconds));
   MakePickRequestTask();
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time,
@@ -271,8 +256,8 @@ TEST_F(PickRequestTaskTest, ChooseRequestWithHigherRetryCount) {
 
 TEST_F(PickRequestTaskTest, ChooseRequestWithSameRetryCountButEarlier) {
   base::Time creation_time1 =
-      base::Time::Now() - base::TimeDelta::FromSeconds(10);
-  base::Time creation_time2 = base::Time::Now();
+      OfflineTimeNow() - base::TimeDelta::FromSeconds(10);
+  base::Time creation_time2 = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time1,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time2,
@@ -296,8 +281,8 @@ TEST_F(PickRequestTaskTest, ChooseEarlierRequest) {
   MakePickRequestTask();
 
   base::Time creation_time1 =
-      base::Time::Now() - base::TimeDelta::FromSeconds(10);
-  base::Time creation_time2 = base::Time::Now();
+      OfflineTimeNow() - base::TimeDelta::FromSeconds(10);
+  base::Time creation_time2 = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time1,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time2,
@@ -321,7 +306,7 @@ TEST_F(PickRequestTaskTest, ChooseSameTimeRequestWithHigherRetryCount) {
       kMaxCompletedTries + 1, kBackgroundProcessingTimeBudgetSeconds));
   MakePickRequestTask();
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time,
@@ -345,7 +330,7 @@ TEST_F(PickRequestTaskTest, ChooseRequestWithLowerRetryCount) {
       kMaxCompletedTries + 1, kBackgroundProcessingTimeBudgetSeconds));
   MakePickRequestTask();
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time,
@@ -370,8 +355,8 @@ TEST_F(PickRequestTaskTest, ChooseLaterRequest) {
   MakePickRequestTask();
 
   base::Time creation_time1 =
-      base::Time::Now() - base::TimeDelta::FromSeconds(10);
-  base::Time creation_time2 = base::Time::Now();
+      OfflineTimeNow() - base::TimeDelta::FromSeconds(10);
+  base::Time creation_time2 = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time1,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time2,
@@ -388,7 +373,7 @@ TEST_F(PickRequestTaskTest, ChooseLaterRequest) {
 }
 
 TEST_F(PickRequestTaskTest, ChooseNonExpiredRequest) {
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   base::Time expired_time =
       creation_time - base::TimeDelta::FromSeconds(
                           policy_->GetRequestExpirationTimeInSeconds() + 60);
@@ -412,8 +397,8 @@ TEST_F(PickRequestTaskTest, ChooseNonExpiredRequest) {
 
 TEST_F(PickRequestTaskTest, ChooseRequestThatHasNotExceededStartLimit) {
   base::Time creation_time1 =
-      base::Time::Now() - base::TimeDelta::FromSeconds(1);
-  base::Time creation_time2 = base::Time::Now();
+      OfflineTimeNow() - base::TimeDelta::FromSeconds(1);
+  base::Time creation_time2 = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time1,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time2,
@@ -438,8 +423,8 @@ TEST_F(PickRequestTaskTest, ChooseRequestThatHasNotExceededStartLimit) {
 
 TEST_F(PickRequestTaskTest, ChooseRequestThatHasNotExceededCompletionLimit) {
   base::Time creation_time1 =
-      base::Time::Now() - base::TimeDelta::FromSeconds(1);
-  base::Time creation_time2 = base::Time::Now();
+      OfflineTimeNow() - base::TimeDelta::FromSeconds(1);
+  base::Time creation_time2 = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time1,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time2,
@@ -470,7 +455,7 @@ TEST_F(PickRequestTaskTest, ChooseRequestThatIsNotDisabled) {
   disabled_requests_.insert(kRequestId2);
   MakePickRequestTask();
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time,
@@ -499,7 +484,7 @@ TEST_F(PickRequestTaskTest, ChoosePrioritizedRequests) {
   prioritized_requests_.push_back(kRequestId2);
   MakePickRequestTask();
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, creation_time,
                            kUserRequested);
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time,
@@ -538,7 +523,7 @@ TEST_F(PickRequestTaskTest, ChooseFromTwoPrioritizedRequests) {
   // Making request 1 more attractive to be picked not considering the
   // prioritizing issues with older creation time, fewer attempt count and it's
   // earlier in the request queue.
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   base::Time older_creation_time =
       creation_time - base::TimeDelta::FromMinutes(10);
   SavePageRequest request1(kRequestId1, kUrl1, kClientId1, older_creation_time,
@@ -565,4 +550,6 @@ TEST_F(PickRequestTaskTest, ChooseFromTwoPrioritizedRequests) {
   EXPECT_TRUE(task_complete_called_);
   EXPECT_EQ(2UL, prioritized_requests_.size());
 }
+
+}  // namespace
 }  // namespace offline_pages
