@@ -5,17 +5,18 @@
 #include "components/offline_pages/core/background/request_queue_store.h"
 
 #include <memory>
+#include <string>
+#include <utility>
 
 #include "base/bind.h"
 #include "base/files/file_path.h"
 #include "base/files/scoped_temp_dir.h"
-#include "base/test/test_simple_task_runner.h"
+#include "base/test/test_mock_time_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "components/offline_pages/core/background/request_queue.h"
-#include "components/offline_pages/core/background/request_queue_in_memory_store.h"
-#include "components/offline_pages/core/background/request_queue_store_sql.h"
 #include "components/offline_pages/core/background/save_page_request.h"
-#include "sql/connection.h"
+#include "components/offline_pages/core/offline_clock.h"
+#include "sql/database.h"
 #include "sql/statement.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -34,7 +35,7 @@ const GURL kUrl2("http://another-example.com");
 const ClientId kClientId("bookmark", "1234");
 const ClientId kClientId2("async", "5678");
 const bool kUserRequested = true;
-const std::string kRequestOrigin = "abc.xyz";
+const char kRequestOrigin[] = "abc.xyz";
 
 enum class LastResult {
   RESULT_NONE,
@@ -42,25 +43,44 @@ enum class LastResult {
   RESULT_TRUE,
 };
 
+SavePageRequest GetTestRequest() {
+  SavePageRequest request(kRequestId, kUrl, kClientId,
+                          base::Time::FromDeltaSinceWindowsEpoch(
+                              base::TimeDelta::FromSeconds(1000)),
+                          kUserRequested);
+  // Set fields to non-default values.
+  request.set_fail_state(offline_items_collection::FailState::FILE_NO_SPACE);
+  request.set_started_attempt_count(2);
+  request.set_completed_attempt_count(3);
+  request.set_last_attempt_time(base::Time::FromDeltaSinceWindowsEpoch(
+      base::TimeDelta::FromSeconds(400)));
+  request.set_request_origin("http://www.origin.com");
+  // Note: pending_state is not stored.
+  request.set_original_url(kUrl2);
+  request.set_auto_fetch_notification_state(
+      SavePageRequest::AutoFetchNotificationState::kShown);
+  return request;
+}
+
 void BuildTestStoreWithSchemaFromM57(const base::FilePath& file) {
-  sql::Connection connection;
+  sql::Database connection;
   ASSERT_TRUE(
       connection.Open(file.Append(FILE_PATH_LITERAL("RequestQueue.db"))));
   ASSERT_TRUE(connection.is_open());
   ASSERT_TRUE(connection.BeginTransaction());
-  ASSERT_TRUE(connection.Execute(
-      "CREATE TABLE " REQUEST_QUEUE_TABLE_NAME
-      " (request_id INTEGER PRIMARY KEY NOT NULL,"
-      " creation_time INTEGER NOT NULL,"
-      " activation_time INTEGER NOT NULL DEFAULT 0,"
-      " last_attempt_time INTEGER NOT NULL DEFAULT 0,"
-      " started_attempt_count INTEGER NOT NULL,"
-      " completed_attempt_count INTEGER NOT NULL,"
-      " state INTEGER NOT NULL DEFAULT 0,"
-      " url VARCHAR NOT NULL,"
-      " client_namespace VARCHAR NOT NULL,"
-      " client_id VARCHAR NOT NULL"
-      ")"));
+  ASSERT_TRUE(
+      connection.Execute("CREATE TABLE " REQUEST_QUEUE_TABLE_NAME
+                         " (request_id INTEGER PRIMARY KEY NOT NULL,"
+                         " creation_time INTEGER NOT NULL,"
+                         " activation_time INTEGER NOT NULL DEFAULT 0,"
+                         " last_attempt_time INTEGER NOT NULL DEFAULT 0,"
+                         " started_attempt_count INTEGER NOT NULL,"
+                         " completed_attempt_count INTEGER NOT NULL,"
+                         " state INTEGER NOT NULL DEFAULT 0,"
+                         " url VARCHAR NOT NULL,"
+                         " client_namespace VARCHAR NOT NULL,"
+                         " client_id VARCHAR NOT NULL"
+                         ")"));
 
   ASSERT_TRUE(connection.CommitTransaction());
   sql::Statement statement(connection.GetUniqueStatement(
@@ -88,7 +108,7 @@ void BuildTestStoreWithSchemaFromM57(const base::FilePath& file) {
 }
 
 void BuildTestStoreWithSchemaFromM58(const base::FilePath& file) {
-  sql::Connection connection;
+  sql::Database connection;
   ASSERT_TRUE(
       connection.Open(file.Append(FILE_PATH_LITERAL("RequestQueue.db"))));
   ASSERT_TRUE(connection.is_open());
@@ -134,7 +154,104 @@ void BuildTestStoreWithSchemaFromM58(const base::FilePath& file) {
       connection.DoesColumnExist(REQUEST_QUEUE_TABLE_NAME, "request_origin"));
 }
 
-}  // namespace
+void BuildTestStoreWithSchemaFromM61(const base::FilePath& file) {
+  sql::Database connection;
+  ASSERT_TRUE(
+      connection.Open(file.Append(FILE_PATH_LITERAL("RequestQueue.db"))));
+  ASSERT_TRUE(connection.is_open());
+  ASSERT_TRUE(connection.BeginTransaction());
+  ASSERT_TRUE(
+      connection.Execute("CREATE TABLE " REQUEST_QUEUE_TABLE_NAME
+                         " (request_id INTEGER PRIMARY KEY NOT NULL,"
+                         " creation_time INTEGER NOT NULL,"
+                         " activation_time INTEGER NOT NULL DEFAULT 0,"
+                         " last_attempt_time INTEGER NOT NULL DEFAULT 0,"
+                         " started_attempt_count INTEGER NOT NULL,"
+                         " completed_attempt_count INTEGER NOT NULL,"
+                         " state INTEGER NOT NULL DEFAULT 0,"
+                         " url VARCHAR NOT NULL,"
+                         " client_namespace VARCHAR NOT NULL,"
+                         " client_id VARCHAR NOT NULL,"
+                         " original_url VARCHAR NOT NULL DEFAULT '',"
+                         " request_origin VARCHAR NOT NULL DEFAULT ''"
+                         ")"));
+
+  ASSERT_TRUE(connection.CommitTransaction());
+  sql::Statement statement(connection.GetUniqueStatement(
+      "INSERT OR IGNORE INTO " REQUEST_QUEUE_TABLE_NAME
+      " (request_id, creation_time, activation_time,"
+      " last_attempt_time, started_attempt_count, completed_attempt_count,"
+      " state, url, client_namespace, client_id, original_url, request_origin)"
+      " VALUES "
+      " (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"));
+
+  statement.BindInt64(0, kRequestId);
+  statement.BindInt64(1, 0);
+  statement.BindInt64(2, 0);
+  statement.BindInt64(3, 0);
+  statement.BindInt64(4, 0);
+  statement.BindInt64(5, 0);
+  statement.BindInt64(6, 0);
+  statement.BindString(7, kUrl.spec());
+  statement.BindString(8, kClientId.name_space);
+  statement.BindString(9, kClientId.id);
+  statement.BindString(10, kUrl2.spec());
+  statement.BindString(11, kRequestOrigin);
+  ASSERT_TRUE(statement.Run());
+  ASSERT_TRUE(connection.DoesTableExist(REQUEST_QUEUE_TABLE_NAME));
+  ASSERT_FALSE(
+      connection.DoesColumnExist(REQUEST_QUEUE_TABLE_NAME, "fail_state"));
+}
+
+void BuildTestStoreWithSchemaFromM72(const base::FilePath& file) {
+  sql::Database connection;
+  ASSERT_TRUE(
+      connection.Open(file.Append(FILE_PATH_LITERAL("RequestQueue.db"))));
+  ASSERT_TRUE(connection.is_open());
+  ASSERT_TRUE(connection.BeginTransaction());
+  ASSERT_TRUE(
+      connection.Execute("CREATE TABLE " REQUEST_QUEUE_TABLE_NAME
+                         " (request_id INTEGER PRIMARY KEY NOT NULL,"
+                         " creation_time INTEGER NOT NULL,"
+                         " activation_time INTEGER NOT NULL DEFAULT 0,"
+                         " last_attempt_time INTEGER NOT NULL DEFAULT 0,"
+                         " started_attempt_count INTEGER NOT NULL,"
+                         " completed_attempt_count INTEGER NOT NULL,"
+                         " state INTEGER NOT NULL DEFAULT 0,"
+                         " url VARCHAR NOT NULL,"
+                         " client_namespace VARCHAR NOT NULL,"
+                         " client_id VARCHAR NOT NULL,"
+                         " original_url VARCHAR NOT NULL DEFAULT '',"
+                         " request_origin VARCHAR NOT NULL DEFAULT '',"
+                         " fail_state INTEGER NOT NULL DEFAULT 0"
+                         ")"));
+
+  ASSERT_TRUE(connection.CommitTransaction());
+  sql::Statement statement(connection.GetUniqueStatement(
+      "INSERT OR IGNORE INTO " REQUEST_QUEUE_TABLE_NAME
+      " (request_id, creation_time, activation_time,"
+      " last_attempt_time, started_attempt_count, completed_attempt_count,"
+      " state, url, client_namespace, client_id, original_url, request_origin, "
+      " fail_state)"
+      " VALUES "
+      " (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"));
+
+  statement.BindInt64(0, kRequestId);
+  statement.BindInt64(1, 0);
+  statement.BindInt64(2, 0);
+  statement.BindInt64(3, 0);
+  statement.BindInt64(4, 0);
+  statement.BindInt64(5, 0);
+  statement.BindInt64(6, 0);
+  statement.BindString(7, kUrl.spec());
+  statement.BindString(8, kClientId.name_space);
+  statement.BindString(9, kClientId.id);
+  statement.BindString(10, kUrl2.spec());
+  statement.BindString(11, kRequestOrigin);
+  statement.BindInt64(12, 1);
+  ASSERT_TRUE(statement.Run());
+  ASSERT_TRUE(connection.DoesTableExist(REQUEST_QUEUE_TABLE_NAME));
+}
 
 // Class that serves as a base for testing different implementations of the
 // |RequestQueueStore|. Specific implementations extend the templatized version
@@ -157,7 +274,7 @@ class RequestQueueStoreTestBase : public testing::Test {
   // Callback used for add/update request.
   void AddOrUpdateDone(UpdateStatus result);
   void AddRequestDone(ItemActionStatus status);
-  void UpdateRequestDone(std::unique_ptr<UpdateRequestsResult> result);
+  void UpdateRequestDone(UpdateRequestsResult result);
   // Callback used for reset.
   void ResetDone(bool result);
 
@@ -182,7 +299,7 @@ class RequestQueueStoreTestBase : public testing::Test {
   std::unique_ptr<UpdateRequestsResult> last_update_result_;
   std::vector<std::unique_ptr<SavePageRequest>> last_requests_;
 
-  scoped_refptr<base::TestSimpleTaskRunner> task_runner_;
+  scoped_refptr<base::TestMockTimeTaskRunner> task_runner_;
   base::ThreadTaskRunnerHandle task_runner_handle_;
 };
 
@@ -190,7 +307,7 @@ RequestQueueStoreTestBase::RequestQueueStoreTestBase()
     : last_result_(LastResult::RESULT_NONE),
       last_update_status_(UpdateStatus::FAILED),
       last_add_status_(ItemActionStatus::NOT_FOUND),
-      task_runner_(new base::TestSimpleTaskRunner),
+      task_runner_(new base::TestMockTimeTaskRunner),
       task_runner_handle_(task_runner_) {
   EXPECT_TRUE(temp_directory_.CreateUniqueTempDir());
 }
@@ -213,8 +330,8 @@ void RequestQueueStoreTestBase::ClearResults() {
 }
 
 void RequestQueueStoreTestBase::InitializeStore(RequestQueueStore* store) {
-  store->Initialize(base::Bind(&RequestQueueStoreTestBase::InitializeCallback,
-                               base::Unretained(this)));
+  store->Initialize(base::BindOnce(
+      &RequestQueueStoreTestBase::InitializeCallback, base::Unretained(this)));
   PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
   ClearResults();
@@ -239,122 +356,105 @@ void RequestQueueStoreTestBase::AddRequestDone(ItemActionStatus status) {
   last_add_status_ = status;
 }
 
-void RequestQueueStoreTestBase::UpdateRequestDone(
-    std::unique_ptr<UpdateRequestsResult> result) {
-  last_update_result_ = std::move(result);
+void RequestQueueStoreTestBase::UpdateRequestDone(UpdateRequestsResult result) {
+  last_update_result_ =
+      std::make_unique<UpdateRequestsResult>(std::move(result));
 }
 
 void RequestQueueStoreTestBase::ResetDone(bool result) {
   last_result_ = result ? LastResult::RESULT_TRUE : LastResult::RESULT_FALSE;
 }
 
-// Defines interface for the store factory.
-class RequestQueueStoreFactory {
- public:
-  virtual RequestQueueStore* BuildStore(const base::FilePath& path) = 0;
-  virtual RequestQueueStore* BuildStoreWithOldSchema(
-      const base::FilePath& path, int version) = 0;
-};
-
-// Implements a store factory for in memory store.
-class RequestQueueInMemoryStoreFactory : public RequestQueueStoreFactory {
- public:
-  RequestQueueStore* BuildStore(const base::FilePath& path) override {
-    RequestQueueStore* store = new RequestQueueInMemoryStore();
-    return store;
-  }
-
-  RequestQueueStore* BuildStoreWithOldSchema(const base::FilePath& path,
-                                             int version) override {
-    return nullptr;
-  }
-};
-
-// Implements a store factory for SQLite based implementation of the store.
-class RequestQueueStoreSQLFactory : public RequestQueueStoreFactory {
- public:
-  RequestQueueStore* BuildStore(const base::FilePath& path) override {
-    RequestQueueStore* store =
-        new RequestQueueStoreSQL(base::ThreadTaskRunnerHandle::Get(), path);
-    return store;
-  }
-
-  RequestQueueStore* BuildStoreWithOldSchema(const base::FilePath& path,
-                                             int version) override {
-    if (version == 57) {
-      BuildTestStoreWithSchemaFromM57(path);
-    } else if (version == 58) {
-      BuildTestStoreWithSchemaFromM58(path);
-    }
-
-    RequestQueueStore* store =
-        new RequestQueueStoreSQL(base::ThreadTaskRunnerHandle::Get(), path);
-    return store;
-  }
-};
-
 // Defines a store test fixture templatized by the store factory.
-template <typename T>
 class RequestQueueStoreTest : public RequestQueueStoreTestBase {
  public:
-  std::unique_ptr<RequestQueueStore> BuildStore();
-  std::unique_ptr<RequestQueueStore> BuildStoreWithOldSchema(int version);
+  std::unique_ptr<RequestQueueStore> BuildStore() {
+    return std::make_unique<RequestQueueStore>(
+        base::ThreadTaskRunnerHandle::Get(), temp_directory_.GetPath());
+  }
+  std::unique_ptr<RequestQueueStore> BuildStoreWithOldSchema(int version) {
+    if (version == 57) {
+      BuildTestStoreWithSchemaFromM57(temp_directory_.GetPath());
+    } else if (version == 58) {
+      BuildTestStoreWithSchemaFromM58(temp_directory_.GetPath());
+    } else if (version == 61) {
+      BuildTestStoreWithSchemaFromM61(temp_directory_.GetPath());
+    } else if (version == 72) {
+      BuildTestStoreWithSchemaFromM72(temp_directory_.GetPath());
+    } else {
+      LOG(ERROR) << "Version " << version << " not implemented";
+      return nullptr;
+    }
+
+    return std::make_unique<RequestQueueStore>(
+        base::ThreadTaskRunnerHandle::Get(), temp_directory_.GetPath());
+  }
+
+  // Performs checks on the database to verify it works after upgrading.
+  void PostUpgradeChecks(RequestQueueStore* store) {
+    // First, remove all requests.
+    {
+      store->GetRequests(base::BindOnce(
+          &RequestQueueStoreTestBase::GetRequestsDone, base::Unretained(this)));
+      PumpLoop();
+      ASSERT_EQ(LastResult::RESULT_TRUE, last_result());
+
+      std::vector<int64_t> request_ids;
+      for (const std::unique_ptr<SavePageRequest>& request : last_requests()) {
+        request_ids.push_back(request->request_id());
+      }
+
+      store->RemoveRequests(
+          request_ids,
+          base::BindOnce(&RequestQueueStoreTestBase::UpdateRequestDone,
+                         base::Unretained(this)));
+      PumpLoop();
+    }
+
+    // Verify a request can be added and retrieved.
+    SavePageRequest request(kRequestId, kUrl, kClientId, OfflineTimeNow(),
+                            kUserRequested);
+    store->AddRequest(request,
+                      base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                     base::Unretained(this)));
+    store->GetRequests(base::BindOnce(
+        &RequestQueueStoreTestBase::GetRequestsDone, base::Unretained(this)));
+    PumpLoop();
+    ASSERT_EQ(ItemActionStatus::SUCCESS, last_add_status());
+    ASSERT_EQ(LastResult::RESULT_TRUE, last_result());
+    ASSERT_EQ(1ul, last_requests().size());
+    EXPECT_EQ(request, *this->last_requests()[0]);
+  }
 
  protected:
-  T factory_;
 };
 
-template <typename T>
-std::unique_ptr<RequestQueueStore> RequestQueueStoreTest<T>::BuildStore() {
-  std::unique_ptr<RequestQueueStore> store(
-      factory_.BuildStore(temp_directory_.GetPath()));
-  return store;
-}
-
-template <typename T>
-std::unique_ptr<RequestQueueStore>
-RequestQueueStoreTest<T>::BuildStoreWithOldSchema(int version) {
-  std::unique_ptr<RequestQueueStore> store(
-      factory_.BuildStoreWithOldSchema(temp_directory_.GetPath(), version));
-  return store;
-}
-
-// |StoreTypes| lists all factories, based on which the tests will be created.
-typedef testing::Types<RequestQueueInMemoryStoreFactory,
-                       RequestQueueStoreSQLFactory>
-    StoreTypes;
-
 // This portion causes test fixtures to be defined.
 // Notice that in the store we are using "this->" to refer to the methods
 // defined on the |RequestQuieueStoreBaseTest| class. That's by design.
-TYPED_TEST_CASE(RequestQueueStoreTest, StoreTypes);
 
-TYPED_TEST(RequestQueueStoreTest, UpgradeFromVersion57Store) {
-  std::unique_ptr<RequestQueueStore> store(this->BuildStoreWithOldSchema(57));
-  // In-memory store does not support upgrading.
-  if (!store)
-    return;
+TEST_F(RequestQueueStoreTest, UpgradeFromVersion57Store) {
+  std::unique_ptr<RequestQueueStore> store = BuildStoreWithOldSchema(57);
   this->InitializeStore(store.get());
 
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
   ASSERT_EQ(1u, this->last_requests().size());
   EXPECT_EQ(kRequestId, this->last_requests()[0]->request_id());
   EXPECT_EQ(kUrl, this->last_requests()[0]->url());
   EXPECT_EQ(GURL(), this->last_requests()[0]->original_url());
+
+  PostUpgradeChecks(store.get());
 }
 
-TYPED_TEST(RequestQueueStoreTest, UpgradeFromVersion58Store) {
-  std::unique_ptr<RequestQueueStore> store(this->BuildStoreWithOldSchema(58));
-  // In-memory store does not support upgrading.
-  if (!store)
-    return;
+TEST_F(RequestQueueStoreTest, UpgradeFromVersion58Store) {
+  std::unique_ptr<RequestQueueStore> store(BuildStoreWithOldSchema(58));
   this->InitializeStore(store.get());
 
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
   ASSERT_EQ(1u, this->last_requests().size());
@@ -362,42 +462,83 @@ TYPED_TEST(RequestQueueStoreTest, UpgradeFromVersion58Store) {
   EXPECT_EQ(kUrl, this->last_requests()[0]->url());
   EXPECT_EQ(kUrl2, this->last_requests()[0]->original_url());
   EXPECT_EQ("", this->last_requests()[0]->request_origin());
+
+  PostUpgradeChecks(store.get());
 }
 
-TYPED_TEST(RequestQueueStoreTest, GetRequestsEmpty) {
+TEST_F(RequestQueueStoreTest, UpgradeFromVersion61Store) {
+  std::unique_ptr<RequestQueueStore> store(BuildStoreWithOldSchema(61));
+  this->InitializeStore(store.get());
+
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
+  this->PumpLoop();
+  ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
+  ASSERT_EQ(1u, this->last_requests().size());
+  EXPECT_EQ(kRequestId, this->last_requests()[0]->request_id());
+  EXPECT_EQ(kUrl, this->last_requests()[0]->url());
+  EXPECT_EQ(kUrl2, this->last_requests()[0]->original_url());
+  EXPECT_EQ(kRequestOrigin, this->last_requests()[0]->request_origin());
+  EXPECT_EQ(0, static_cast<int>(this->last_requests()[0]->fail_state()));
+
+  PostUpgradeChecks(store.get());
+}
+
+TEST_F(RequestQueueStoreTest, UpgradeFromVersion72Store) {
+  std::unique_ptr<RequestQueueStore> store(BuildStoreWithOldSchema(72));
+  this->InitializeStore(store.get());
+
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
+  this->PumpLoop();
+  ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
+  const std::vector<std::unique_ptr<SavePageRequest>>& requests =
+      this->last_requests();
+  ASSERT_EQ(1u, requests.size());
+  EXPECT_EQ(kRequestId, requests[0]->request_id());
+  EXPECT_EQ(kUrl, requests[0]->url());
+  EXPECT_EQ(kUrl2, requests[0]->original_url());
+  EXPECT_EQ(kRequestOrigin, requests[0]->request_origin());
+  EXPECT_EQ(1, static_cast<int>(requests[0]->fail_state()));
+  EXPECT_EQ(0, static_cast<int>(requests[0]->auto_fetch_notification_state()));
+
+  PostUpgradeChecks(store.get());
+}
+
+TEST_F(RequestQueueStoreTest, GetRequestsEmpty) {
   std::unique_ptr<RequestQueueStore> store(this->BuildStore());
   this->InitializeStore(store.get());
 
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   ASSERT_EQ(LastResult::RESULT_NONE, this->last_result());
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
   ASSERT_TRUE(this->last_requests().empty());
 }
 
-TYPED_TEST(RequestQueueStoreTest, GetRequestsByIds) {
+TEST_F(RequestQueueStoreTest, GetRequestsByIds) {
   std::unique_ptr<RequestQueueStore> store(this->BuildStore());
   this->InitializeStore(store.get());
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest request1(kRequestId, kUrl, kClientId, creation_time,
                            kUserRequested);
   store->AddRequest(request1,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time,
                            kUserRequested);
   store->AddRequest(request2,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   this->PumpLoop();
   this->ClearResults();
 
   std::vector<int64_t> request_ids{kRequestId, kRequestId2};
   store->GetRequestsByIds(
-      request_ids, base::Bind(&RequestQueueStoreTestBase::UpdateRequestDone,
-                              base::Unretained(this)));
+      request_ids, base::BindOnce(&RequestQueueStoreTestBase::UpdateRequestDone,
+                                  base::Unretained(this)));
 
   ASSERT_FALSE(this->last_update_result());
   this->PumpLoop();
@@ -420,8 +561,8 @@ TYPED_TEST(RequestQueueStoreTest, GetRequestsByIds) {
   request_ids.push_back(kRequestId);
 
   store->GetRequestsByIds(
-      request_ids, base::Bind(&RequestQueueStoreTestBase::UpdateRequestDone,
-                              base::Unretained(this)));
+      request_ids, base::BindOnce(&RequestQueueStoreTestBase::UpdateRequestDone,
+                                  base::Unretained(this)));
 
   ASSERT_FALSE(this->last_update_result());
   this->PumpLoop();
@@ -437,26 +578,26 @@ TYPED_TEST(RequestQueueStoreTest, GetRequestsByIds) {
   EXPECT_EQ(request1, this->last_update_result()->updated_items.at(0));
 }
 
-TYPED_TEST(RequestQueueStoreTest, AddRequest) {
+TEST_F(RequestQueueStoreTest, AddRequest) {
   std::unique_ptr<RequestQueueStore> store(this->BuildStore());
   this->InitializeStore(store.get());
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest request(kRequestId, kUrl, kClientId, creation_time,
                           kUserRequested);
   request.set_original_url(kUrl2);
 
   store->AddRequest(request,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   ASSERT_EQ(ItemActionStatus::NOT_FOUND, this->last_add_status());
   this->PumpLoop();
   ASSERT_EQ(ItemActionStatus::SUCCESS, this->last_add_status());
 
   // Verifying get reqeust results after a request was added.
   this->ClearResults();
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   ASSERT_EQ(LastResult::RESULT_NONE, this->last_result());
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
@@ -466,32 +607,49 @@ TYPED_TEST(RequestQueueStoreTest, AddRequest) {
   // Verify it is not possible to add the same request twice.
   this->ClearResults();
   store->AddRequest(request,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   ASSERT_EQ(ItemActionStatus::NOT_FOUND, this->last_add_status());
   this->PumpLoop();
   ASSERT_EQ(ItemActionStatus::ALREADY_EXISTS, this->last_add_status());
 
   // Check that there is still only one item in the store.
   this->ClearResults();
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   ASSERT_EQ(LastResult::RESULT_NONE, this->last_result());
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
   ASSERT_EQ(1ul, this->last_requests().size());
 }
 
-TYPED_TEST(RequestQueueStoreTest, UpdateRequest) {
+TEST_F(RequestQueueStoreTest, AddAndGetRequestsMatch) {
+  std::unique_ptr<RequestQueueStore> store(this->BuildStore());
+  this->InitializeStore(store.get());
+  const SavePageRequest request = GetTestRequest();
+  store->AddRequest(request,
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
+  this->PumpLoop();
+
+  ASSERT_EQ(ItemActionStatus::SUCCESS, this->last_add_status());
+  ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
+  ASSERT_EQ(1ul, this->last_requests().size());
+  EXPECT_EQ(request.ToString(), this->last_requests()[0]->ToString());
+}
+
+TEST_F(RequestQueueStoreTest, UpdateRequest) {
   std::unique_ptr<RequestQueueStore> store(this->BuildStore());
   this->InitializeStore(store.get());
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest original_request(kRequestId, kUrl, kClientId, creation_time,
                                    kUserRequested);
   store->AddRequest(original_request,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   this->PumpLoop();
   this->ClearResults();
 
@@ -509,8 +667,8 @@ TYPED_TEST(RequestQueueStoreTest, UpdateRequest) {
                                                   updated_request2};
   store->UpdateRequests(
       requests_to_update,
-      base::Bind(&RequestQueueStoreTestBase::UpdateRequestDone,
-                 base::Unretained(this)));
+      base::BindOnce(&RequestQueueStoreTestBase::UpdateRequestDone,
+                     base::Unretained(this)));
   ASSERT_FALSE(this->last_update_result());
   this->PumpLoop();
   ASSERT_TRUE(this->last_update_result());
@@ -522,13 +680,15 @@ TYPED_TEST(RequestQueueStoreTest, UpdateRequest) {
   EXPECT_EQ(ItemActionStatus::NOT_FOUND,
             this->last_update_result()->item_statuses[1].second);
   EXPECT_EQ(1UL, this->last_update_result()->updated_items.size());
+  EXPECT_EQ(updated_request.ToString(),
+            this->last_update_result()->updated_items.begin()->ToString());
   EXPECT_EQ(updated_request,
             *(this->last_update_result()->updated_items.begin()));
 
   // Verifying get reqeust results after a request was updated.
   this->ClearResults();
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   ASSERT_EQ(LastResult::RESULT_NONE, this->last_result());
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
@@ -536,28 +696,28 @@ TYPED_TEST(RequestQueueStoreTest, UpdateRequest) {
   ASSERT_EQ(updated_request, *(this->last_requests()[0].get()));
 }
 
-TYPED_TEST(RequestQueueStoreTest, RemoveRequests) {
+TEST_F(RequestQueueStoreTest, RemoveRequests) {
   std::unique_ptr<RequestQueueStore> store(this->BuildStore());
   this->InitializeStore(store.get());
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest request1(kRequestId, kUrl, kClientId, creation_time,
                            kUserRequested);
   store->AddRequest(request1,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   SavePageRequest request2(kRequestId2, kUrl2, kClientId2, creation_time,
                            kUserRequested);
   store->AddRequest(request2,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   this->PumpLoop();
   this->ClearResults();
 
   std::vector<int64_t> request_ids{kRequestId, kRequestId2};
   store->RemoveRequests(
-      request_ids, base::Bind(&RequestQueueStoreTestBase::UpdateRequestDone,
-                              base::Unretained(this)));
+      request_ids, base::BindOnce(&RequestQueueStoreTestBase::UpdateRequestDone,
+                                  base::Unretained(this)));
 
   ASSERT_FALSE(this->last_update_result());
   this->PumpLoop();
@@ -574,8 +734,8 @@ TYPED_TEST(RequestQueueStoreTest, RemoveRequests) {
   EXPECT_EQ(request2, this->last_update_result()->updated_items.at(1));
   this->ClearResults();
 
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
   ASSERT_TRUE(this->last_requests().empty());
@@ -583,8 +743,8 @@ TYPED_TEST(RequestQueueStoreTest, RemoveRequests) {
 
   // Try to remove a request that is not in the queue.
   store->RemoveRequests(
-      request_ids, base::Bind(&RequestQueueStoreTestBase::UpdateRequestDone,
-                              base::Unretained(this)));
+      request_ids, base::BindOnce(&RequestQueueStoreTestBase::UpdateRequestDone,
+                                  base::Unretained(this)));
   ASSERT_FALSE(this->last_update_result());
   this->PumpLoop();
   ASSERT_TRUE(this->last_update_result());
@@ -600,49 +760,46 @@ TYPED_TEST(RequestQueueStoreTest, RemoveRequests) {
   EXPECT_EQ(0UL, this->last_update_result()->updated_items.size());
 }
 
-TYPED_TEST(RequestQueueStoreTest, ResetStore) {
+TEST_F(RequestQueueStoreTest, ResetStore) {
   std::unique_ptr<RequestQueueStore> store(this->BuildStore());
   this->InitializeStore(store.get());
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest original_request(kRequestId, kUrl, kClientId, creation_time,
                                    kUserRequested);
   store->AddRequest(original_request,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   this->PumpLoop();
   this->ClearResults();
 
-  store->Reset(base::Bind(&RequestQueueStoreTestBase::ResetDone,
-                          base::Unretained(this)));
+  store->Reset(base::BindOnce(&RequestQueueStoreTestBase::ResetDone,
+                              base::Unretained(this)));
   ASSERT_EQ(LastResult::RESULT_NONE, this->last_result());
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
   this->ClearResults();
 
   this->InitializeStore(store.get());
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
   ASSERT_TRUE(this->last_requests().empty());
 }
 
-class RequestQueueStoreSQLTest
-    : public RequestQueueStoreTest<RequestQueueStoreSQLFactory> {};
-
 // Makes sure that persistent DB is actually persisting requests across store
 // restarts.
-TEST_F(RequestQueueStoreSQLTest, SaveCloseReopenRead) {
+TEST_F(RequestQueueStoreTest, SaveCloseReopenRead) {
   std::unique_ptr<RequestQueueStore> store(BuildStore());
   this->InitializeStore(store.get());
 
-  base::Time creation_time = base::Time::Now();
+  base::Time creation_time = OfflineTimeNow();
   SavePageRequest original_request(kRequestId, kUrl, kClientId, creation_time,
                                    kUserRequested);
   store->AddRequest(original_request,
-                    base::Bind(&RequestQueueStoreTestBase::AddRequestDone,
-                               base::Unretained(this)));
+                    base::BindOnce(&RequestQueueStoreTestBase::AddRequestDone,
+                                   base::Unretained(this)));
   PumpLoop();
   ClearResults();
 
@@ -652,8 +809,8 @@ TEST_F(RequestQueueStoreSQLTest, SaveCloseReopenRead) {
   store = BuildStore();
   this->InitializeStore(store.get());
 
-  store->GetRequests(base::Bind(&RequestQueueStoreTestBase::GetRequestsDone,
-                                base::Unretained(this)));
+  store->GetRequests(base::BindOnce(&RequestQueueStoreTestBase::GetRequestsDone,
+                                    base::Unretained(this)));
   ASSERT_EQ(LastResult::RESULT_NONE, this->last_result());
   this->PumpLoop();
   ASSERT_EQ(LastResult::RESULT_TRUE, this->last_result());
@@ -661,4 +818,5 @@ TEST_F(RequestQueueStoreSQLTest, SaveCloseReopenRead) {
   ASSERT_TRUE(original_request == *(this->last_requests().at(0).get()));
 }
 
-}  // offline_pages
+}  // namespace
+}  // namespace offline_pages
