@@ -10,16 +10,16 @@
 #include "base/hash.h"
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/memory/ptr_util.h"
 #include "base/strings/stringprintf.h"
-#include "base/task_runner.h"
-#include "base/task_runner_util.h"
-#include "chromeos/system/devicetype.h"
+#include "chromeos/constants/devicetype.h"
 #include "components/pairing/bluetooth_pairing_constants.h"
 #include "components/pairing/pairing_api.pb.h"
 #include "components/pairing/proto_decoder.h"
 #include "device/bluetooth/bluetooth_adapter_factory.h"
+#include "mojo/public/cpp/bindings/interface_request.h"
 #include "net/base/io_buffer.h"
+#include "services/device/public/mojom/constants.mojom.h"
+#include "services/service_manager/public/cpp/connector.h"
 
 namespace pairing_chromeos {
 
@@ -79,19 +79,15 @@ pairing_api::HostStatusParameters::EnrollmentStatus PairingApiEnrollmentStatus(
   }
 }
 
-std::vector<BluetoothHostPairingController::InputDeviceInfo> GetDevices() {
-  std::vector<BluetoothHostPairingController::InputDeviceInfo> devices;
-  if (device::InputServiceLinux::HasInstance())
-    device::InputServiceLinux::GetInstance()->GetDevices(&devices);
-  return devices;
-}
-
 }  // namespace
 
 BluetoothHostPairingController::BluetoothHostPairingController(
-    scoped_refptr<base::TaskRunner> input_service_task_runner)
-    : proto_decoder_(base::MakeUnique<ProtoDecoder>(this)),
-      input_service_task_runner_(std::move(input_service_task_runner)) {}
+    service_manager::Connector* connector)
+    : proto_decoder_(std::make_unique<ProtoDecoder>(this)) {
+  DCHECK(connector);
+  connector->BindInterface(device::mojom::kServiceName,
+                           mojo::MakeRequest(&input_device_manager_));
+}
 
 BluetoothHostPairingController::~BluetoothHostPairingController() {
   Reset();
@@ -326,10 +322,10 @@ void BluetoothHostPairingController::OnSendError(
 }
 
 void BluetoothHostPairingController::PowerOffAdapterIfApplicable(
-    const std::vector<InputDeviceInfo>& devices) {
+    std::vector<InputDeviceInfoPtr> devices) {
   bool use_bluetooth = false;
-  for (const auto& device : devices) {
-    if (device.type == InputDeviceInfo::TYPE_BLUETOOTH) {
+  for (auto& device : devices) {
+    if (device->type == device::mojom::InputDeviceType::TYPE_BLUETOOTH) {
       use_bluetooth = true;
       break;
     }
@@ -365,14 +361,12 @@ void BluetoothHostPairingController::OnForget() {
 
   if (adapter_.get()) {
     if (adapter_->IsDiscoverable()) {
-      adapter_->SetDiscoverable(false, base::Bind(&base::DoNothing),
-                                base::Bind(&base::DoNothing));
+      adapter_->SetDiscoverable(false, base::DoNothing(), base::DoNothing());
     }
 
-    base::PostTaskAndReplyWithResult(
-        input_service_task_runner_.get(), FROM_HERE, base::Bind(&GetDevices),
-        base::Bind(&BluetoothHostPairingController::PowerOffAdapterIfApplicable,
-                   ptr_factory_.GetWeakPtr()));
+    input_device_manager_->GetDevices(base::BindOnce(
+        &BluetoothHostPairingController::PowerOffAdapterIfApplicable,
+        ptr_factory_.GetWeakPtr()));
   }
   ChangeStage(STAGE_NONE);
 }
@@ -477,8 +471,8 @@ void BluetoothHostPairingController::StartPairing() {
   }
 
   device::BluetoothAdapterFactory::GetAdapter(
-      base::Bind(&BluetoothHostPairingController::OnGetAdapter,
-                 ptr_factory_.GetWeakPtr()));
+      base::BindOnce(&BluetoothHostPairingController::OnGetAdapter,
+                     ptr_factory_.GetWeakPtr()));
 }
 
 std::string BluetoothHostPairingController::GetDeviceName() {
