@@ -8,93 +8,257 @@
 #include <set>
 #include <utility>
 
+#include "base/feature_list.h"
 #include "base/strings/utf_string_conversions.h"
 #include "components/autofill/core/browser/autofill_country.h"
 #include "components/autofill/core/browser/autofill_data_util.h"
 #include "components/autofill/core/browser/autofill_profile.h"
 #include "components/autofill/core/browser/credit_card.h"
 #include "components/autofill/core/browser/personal_data_manager.h"
+#include "components/payments/content/content_payment_request_delegate.h"
+#include "components/payments/content/payment_manifest_web_data_service.h"
 #include "components/payments/content/payment_response_helper.h"
+#include "components/payments/content/service_worker_payment_instrument.h"
 #include "components/payments/core/autofill_payment_instrument.h"
+#include "components/payments/core/features.h"
 #include "components/payments/core/journey_logger.h"
 #include "components/payments/core/payment_instrument.h"
 #include "components/payments/core/payment_request_data_util.h"
-#include "components/payments/core/payment_request_delegate.h"
+#include "content/public/common/content_features.h"
 
 namespace payments {
 
 PaymentRequestState::PaymentRequestState(
+    content::WebContents* web_contents,
+    const GURL& top_level_origin,
+    const GURL& frame_origin,
     PaymentRequestSpec* spec,
     Delegate* delegate,
     const std::string& app_locale,
     autofill::PersonalDataManager* personal_data_manager,
-    PaymentRequestDelegate* payment_request_delegate,
+    ContentPaymentRequestDelegate* payment_request_delegate,
     JourneyLogger* journey_logger)
     : is_ready_to_pay_(false),
+      get_all_instruments_finished_(true),
       is_waiting_for_merchant_validation_(false),
       app_locale_(app_locale),
       spec_(spec),
       delegate_(delegate),
       personal_data_manager_(personal_data_manager),
       journey_logger_(journey_logger),
+      are_requested_methods_supported_(
+          !spec_->supported_card_networks().empty()),
       selected_shipping_profile_(nullptr),
       selected_shipping_option_error_profile_(nullptr),
       selected_contact_profile_(nullptr),
+      invalid_shipping_profile_(nullptr),
+      invalid_contact_profile_(nullptr),
       selected_instrument_(nullptr),
+      number_of_pending_sw_payment_instruments_(0),
       payment_request_delegate_(payment_request_delegate),
-      profile_comparator_(app_locale, *spec) {
-  PopulateProfileCache();
-  SetDefaultProfileSelections();
+      profile_comparator_(app_locale, *spec),
+      weak_ptr_factory_(this) {
+  if (base::FeatureList::IsEnabled(::features::kServiceWorkerPaymentApps)) {
+    DCHECK(web_contents);
+    get_all_instruments_finished_ = false;
+    ServiceWorkerPaymentAppFactory::GetInstance()->GetAllPaymentApps(
+        web_contents,
+        payment_request_delegate_->GetPaymentManifestWebDataService(),
+        spec_->method_data(),
+        /*may_crawl_for_installable_payment_apps=*/
+        !spec_->supports_basic_card(),
+        base::BindOnce(&PaymentRequestState::GetAllPaymentAppsCallback,
+                       weak_ptr_factory_.GetWeakPtr(), web_contents,
+                       top_level_origin, frame_origin),
+        base::BindOnce([]() {
+          /* Nothing needs to be done after writing cache. This callback is used
+           * only in tests. */
+        }));
+  } else {
+    PopulateProfileCache();
+    SetDefaultProfileSelections();
+  }
   spec_->AddObserver(this);
 }
+
 PaymentRequestState::~PaymentRequestState() {}
 
+void PaymentRequestState::GetAllPaymentAppsCallback(
+    content::WebContents* web_contents,
+    const GURL& top_level_origin,
+    const GURL& frame_origin,
+    content::PaymentAppProvider::PaymentApps apps,
+    ServiceWorkerPaymentAppFactory::InstallablePaymentApps installable_apps) {
+  number_of_pending_sw_payment_instruments_ =
+      apps.size() + installable_apps.size();
+  if (number_of_pending_sw_payment_instruments_ == 0U) {
+    FinishedGetAllSWPaymentInstruments();
+    return;
+  }
+
+  for (auto& app : apps) {
+    std::unique_ptr<ServiceWorkerPaymentInstrument> instrument =
+        std::make_unique<ServiceWorkerPaymentInstrument>(
+            web_contents->GetBrowserContext(), top_level_origin, frame_origin,
+            spec_, std::move(app.second), payment_request_delegate_);
+    instrument->ValidateCanMakePayment(
+        base::BindOnce(&PaymentRequestState::OnSWPaymentInstrumentValidated,
+                       weak_ptr_factory_.GetWeakPtr()));
+    available_instruments_.push_back(std::move(instrument));
+  }
+
+  for (auto& installable_app : installable_apps) {
+    std::unique_ptr<ServiceWorkerPaymentInstrument> instrument =
+        std::make_unique<ServiceWorkerPaymentInstrument>(
+            web_contents, top_level_origin, frame_origin, spec_,
+            std::move(installable_app.second), installable_app.first.spec(),
+            payment_request_delegate_);
+    instrument->ValidateCanMakePayment(
+        base::BindOnce(&PaymentRequestState::OnSWPaymentInstrumentValidated,
+                       weak_ptr_factory_.GetWeakPtr()));
+    available_instruments_.push_back(std::move(instrument));
+  }
+}
+
+void PaymentRequestState::OnSWPaymentInstrumentValidated(
+    ServiceWorkerPaymentInstrument* instrument,
+    bool result) {
+  // Remove service worker payment instruments failed on validation.
+  if (!result) {
+    for (size_t i = 0; i < available_instruments_.size(); i++) {
+      if (available_instruments_[i].get() == instrument) {
+        available_instruments_.erase(available_instruments_.begin() + i);
+        break;
+      }
+    }
+  }
+
+  if (--number_of_pending_sw_payment_instruments_ > 0)
+    return;
+
+  FinishedGetAllSWPaymentInstruments();
+}
+
+void PaymentRequestState::FinishedGetAllSWPaymentInstruments() {
+  PopulateProfileCache();
+  SetDefaultProfileSelections();
+
+  get_all_instruments_finished_ = true;
+  are_requested_methods_supported_ |= !available_instruments_.empty();
+  NotifyOnGetAllPaymentInstrumentsFinished();
+
+  // Fulfill the pending CanMakePayment call.
+  if (can_make_payment_callback_)
+    CheckCanMakePayment(std::move(can_make_payment_callback_));
+
+  // Fulfill the pending HasEnrolledInstrument call.
+  if (has_enrolled_instrument_callback_)
+    CheckHasEnrolledInstrument(std::move(has_enrolled_instrument_callback_));
+
+  // Fulfill the pending AreRequestedMethodsSupported call.
+  if (are_requested_methods_supported_callback_)
+    CheckRequestedMethodsSupported(
+        std::move(are_requested_methods_supported_callback_));
+}
+
 void PaymentRequestState::OnPaymentResponseReady(
     mojom::PaymentResponsePtr payment_response) {
   delegate_->OnPaymentResponseAvailable(std::move(payment_response));
 }
 
-void PaymentRequestState::OnAddressNormalized(
-    const autofill::AutofillProfile& normalized_profile) {
-  delegate_->OnShippingAddressSelected(
-      PaymentResponseHelper::GetMojomPaymentAddressFromAutofillProfile(
-          normalized_profile, app_locale_));
-}
-
-void PaymentRequestState::OnCouldNotNormalize(
-    const autofill::AutofillProfile& profile) {
-  // Since the phone number is formatted in either case, this profile should be
-  // used.
-  OnAddressNormalized(profile);
-}
-
 void PaymentRequestState::OnSpecUpdated() {
+  autofill::AutofillProfile* selected_shipping_profile =
+      selected_shipping_profile_;
+  autofill::AutofillProfile* selected_contact_profile =
+      selected_contact_profile_;
+
+  if (spec_->current_update_reason() ==
+      PaymentRequestSpec::UpdateReason::RETRY) {
+    if (spec_->has_shipping_address_error() && selected_shipping_profile) {
+      invalid_shipping_profile_ = selected_shipping_profile;
+      selected_shipping_profile_ = nullptr;
+    }
+
+    if (spec_->has_payer_error() && selected_contact_profile) {
+      invalid_contact_profile_ = selected_contact_profile;
+      selected_contact_profile_ = nullptr;
+    }
+  }
+
   if (spec_->selected_shipping_option_error().empty()) {
     selected_shipping_option_error_profile_ = nullptr;
   } else {
-    selected_shipping_option_error_profile_ = selected_shipping_profile_;
+    selected_shipping_option_error_profile_ = selected_shipping_profile;
     selected_shipping_profile_ = nullptr;
+    if (spec_->has_shipping_address_error() && selected_shipping_profile) {
+      invalid_shipping_profile_ = selected_shipping_profile;
+    }
   }
+
   is_waiting_for_merchant_validation_ = false;
   UpdateIsReadyToPayAndNotifyObservers();
 }
 
-bool PaymentRequestState::CanMakePayment() const {
-  for (const auto& instrument : available_instruments_) {
-    if (instrument->IsValidForCanMakePayment()) {
-      // AddAutofillPaymentInstrument() filters out available instruments based
-      // on supported card networks (visa, amex) and types (credit, debit).
-      DCHECK(spec_->supported_card_networks_set().find(
-                 instrument->method_name()) !=
-             spec_->supported_card_networks_set().end());
-      return true;
-    }
+void PaymentRequestState::CanMakePayment(StatusCallback callback) {
+  if (!get_all_instruments_finished_) {
+    DCHECK(!can_make_payment_callback_);
+    can_make_payment_callback_ = std::move(callback);
+    return;
   }
-  return false;
+
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE,
+      base::BindOnce(&PaymentRequestState::CheckCanMakePayment,
+                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)));
 }
 
-bool PaymentRequestState::AreRequestedMethodsSupported() const {
-  return !spec_->supported_card_networks().empty();
+void PaymentRequestState::CheckCanMakePayment(StatusCallback callback) {
+  DCHECK(get_all_instruments_finished_);
+  std::move(callback).Run(are_requested_methods_supported_);
+}
+
+void PaymentRequestState::HasEnrolledInstrument(StatusCallback callback) {
+  if (!get_all_instruments_finished_) {
+    DCHECK(!has_enrolled_instrument_callback_);
+    has_enrolled_instrument_callback_ = std::move(callback);
+    return;
+  }
+
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE,
+      base::BindOnce(&PaymentRequestState::CheckHasEnrolledInstrument,
+                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)));
+}
+
+void PaymentRequestState::CheckHasEnrolledInstrument(StatusCallback callback) {
+  bool has_enrolled_instrument_value = false;
+  for (const auto& instrument : available_instruments_) {
+    if (instrument->IsValidForCanMakePayment()) {
+      has_enrolled_instrument_value = true;
+      break;
+    }
+  }
+  std::move(callback).Run(has_enrolled_instrument_value);
+}
+
+void PaymentRequestState::AreRequestedMethodsSupported(
+    StatusCallback callback) {
+  if (!get_all_instruments_finished_) {
+    are_requested_methods_supported_callback_ = std::move(callback);
+    return;
+  }
+
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE,
+      base::BindOnce(&PaymentRequestState::CheckRequestedMethodsSupported,
+                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)));
+}
+
+void PaymentRequestState::CheckRequestedMethodsSupported(
+    StatusCallback callback) {
+  DCHECK(get_all_instruments_finished_);
+
+  std::move(callback).Run(are_requested_methods_supported_);
 }
 
 std::string PaymentRequestState::GetAuthenticatedEmail() const {
@@ -114,7 +278,7 @@ void PaymentRequestState::GeneratePaymentResponse() {
   DCHECK(is_ready_to_pay());
 
   // Once the response is ready, will call back into OnPaymentResponseReady.
-  response_helper_ = base::MakeUnique<PaymentResponseHelper>(
+  response_helper_ = std::make_unique<PaymentResponseHelper>(
       app_locale_, spec_, selected_instrument_, payment_request_delegate_,
       selected_shipping_profile_, selected_contact_profile_, this);
 }
@@ -163,7 +327,7 @@ void PaymentRequestState::AddAutofillPaymentInstrument(
 
   // AutofillPaymentInstrument makes a copy of |card| so it is effectively
   // owned by this object.
-  auto instrument = base::MakeUnique<AutofillPaymentInstrument>(
+  auto instrument = std::make_unique<AutofillPaymentInstrument>(
       basic_card_network, card, matches_merchant_card_type_exactly,
       shipping_profiles_, app_locale_, payment_request_delegate_);
   available_instruments_.push_back(std::move(instrument));
@@ -176,7 +340,7 @@ void PaymentRequestState::AddAutofillShippingProfile(
     bool selected,
     const autofill::AutofillProfile& profile) {
   profile_cache_.push_back(
-      base::MakeUnique<autofill::AutofillProfile>(profile));
+      std::make_unique<autofill::AutofillProfile>(profile));
   // TODO(tmartino): Implement deduplication rules specific to shipping
   // profiles.
   autofill::AutofillProfile* new_cached_profile = profile_cache_.back().get();
@@ -190,7 +354,7 @@ void PaymentRequestState::AddAutofillContactProfile(
     bool selected,
     const autofill::AutofillProfile& profile) {
   profile_cache_.push_back(
-      base::MakeUnique<autofill::AutofillProfile>(profile));
+      std::make_unique<autofill::AutofillProfile>(profile));
   autofill::AutofillProfile* new_cached_profile = profile_cache_.back().get();
   contact_profiles_.push_back(new_cached_profile);
 
@@ -213,24 +377,35 @@ void PaymentRequestState::SetSelectedShippingProfile(
       PaymentRequestSpec::UpdateReason::SHIPPING_ADDRESS);
   selected_shipping_profile_ = profile;
 
+  // Changing the shipping address clears shipping address validation errors
+  // from retry().
+  invalid_shipping_profile_ = nullptr;
+
   // The user should not be able to click on pay until the callback from the
   // merchant.
   is_waiting_for_merchant_validation_ = true;
-  UpdateIsReadyToPayAndNotifyObservers();
 
   // Start the normalization of the shipping address.
-  // Use the country code from the profile if it is set, otherwise infer it
-  // from the |app_locale_|.
-  std::string country_code = data_util::GetCountryCodeWithFallback(
-      selected_shipping_profile_, app_locale_);
-  payment_request_delegate_->GetAddressNormalizer()->StartAddressNormalization(
-      *selected_shipping_profile_, country_code, /*timeout_seconds=*/2, this);
+  payment_request_delegate_->GetAddressNormalizer()->NormalizeAddressAsync(
+      *selected_shipping_profile_, /*timeout_seconds=*/2,
+      base::BindOnce(&PaymentRequestState::OnAddressNormalized,
+                     weak_ptr_factory_.GetWeakPtr()));
 }
 
 void PaymentRequestState::SetSelectedContactProfile(
     autofill::AutofillProfile* profile) {
   selected_contact_profile_ = profile;
+
+  // Changing the contact information clears contact information validation
+  // errors from retry().
+  invalid_contact_profile_ = nullptr;
+
   UpdateIsReadyToPayAndNotifyObservers();
+
+  if (IsPaymentAppInvoked()) {
+    delegate_->OnPayerInfoSelected(
+        response_helper_->GeneratePayerDetail(profile));
+  }
 }
 
 void PaymentRequestState::SetSelectedInstrument(PaymentInstrument* instrument) {
@@ -254,7 +429,7 @@ bool PaymentRequestState::IsPaymentAppInvoked() const {
   return !!response_helper_;
 }
 
-AddressNormalizer* PaymentRequestState::GetAddressNormalizer() {
+autofill::AddressNormalizer* PaymentRequestState::GetAddressNormalizer() {
   return payment_request_delegate_->GetAddressNormalizer();
 }
 
@@ -270,7 +445,7 @@ void PaymentRequestState::PopulateProfileCache() {
   // whenever Profiles are requested.
   for (size_t i = 0; i < profiles.size(); i++) {
     profile_cache_.push_back(
-        base::MakeUnique<autofill::AutofillProfile>(*profiles[i]));
+        std::make_unique<autofill::AutofillProfile>(*profiles[i]));
     raw_profiles_for_filtering.push_back(profile_cache_.back().get());
   }
 
@@ -304,18 +479,11 @@ void PaymentRequestState::PopulateProfileCache() {
   // Create the list of available instruments. A copy of each card will be made
   // by their respective AutofillPaymentInstrument.
   const std::vector<autofill::CreditCard*>& cards =
-      personal_data_manager_->GetCreditCardsToSuggest();
+      personal_data_manager_->GetCreditCardsToSuggest(
+          /*include_server_cards=*/base::FeatureList::IsEnabled(
+              payments::features::kReturnGooglePayInBasicCard));
   for (autofill::CreditCard* card : cards)
     AddAutofillPaymentInstrument(/*selected=*/false, *card);
-
-  bool has_complete_instrument =
-      available_instruments().empty()
-          ? false
-          : available_instruments()[0]->IsCompleteForPayment();
-
-  journey_logger_->SetNumberOfSuggestionsShown(
-      JourneyLogger::Section::SECTION_PAYMENT_METHOD,
-      available_instruments().size(), has_complete_instrument);
 }
 
 void PaymentRequestState::SetDefaultProfileSelections() {
@@ -348,6 +516,15 @@ void PaymentRequestState::SetDefaultProfileSelections() {
                              ? nullptr
                              : first_complete_instrument->get();
   UpdateIsReadyToPayAndNotifyObservers();
+
+  bool has_complete_instrument =
+      available_instruments().empty()
+          ? false
+          : available_instruments()[0]->IsCompleteForPayment();
+
+  journey_logger_->SetNumberOfSuggestionsShown(
+      JourneyLogger::Section::SECTION_PAYMENT_METHOD,
+      available_instruments().size(), has_complete_instrument);
 }
 
 void PaymentRequestState::UpdateIsReadyToPayAndNotifyObservers() {
@@ -356,6 +533,11 @@ void PaymentRequestState::UpdateIsReadyToPayAndNotifyObservers() {
   NotifyOnSelectedInformationChanged();
 }
 
+void PaymentRequestState::NotifyOnGetAllPaymentInstrumentsFinished() {
+  for (auto& observer : observers_)
+    observer.OnGetAllPaymentInstrumentsFinished();
+}
+
 void PaymentRequestState::NotifyOnSelectedInformationChanged() {
   for (auto& observer : observers_)
     observer.OnSelectedInformationChanged();
@@ -381,4 +563,12 @@ bool PaymentRequestState::ArePaymentOptionsSatisfied() {
   return profile_comparator()->IsContactInfoComplete(selected_contact_profile_);
 }
 
+void PaymentRequestState::OnAddressNormalized(
+    bool success,
+    const autofill::AutofillProfile& normalized_profile) {
+  delegate_->OnShippingAddressSelected(
+      data_util::GetPaymentAddressFromAutofillProfile(normalized_profile,
+                                                      app_locale_));
+}
+
 }  // namespace payments
