@@ -10,24 +10,23 @@
 #include <utility>
 
 #include "base/bind.h"
+#include "base/compiler_specific.h"
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
 #include "base/location.h"
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
 #include "base/metrics/histogram_samples.h"
 #include "base/path_service.h"
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
+#include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/test/histogram_tester.h"
+#include "base/synchronization/waitable_event.h"
+#include "base/test/metrics/histogram_tester.h"
 #include "base/test/scoped_task_environment.h"
-#include "base/test/simple_test_clock.h"
 #include "base/threading/sequenced_task_runner_handle.h"
-#include "base/threading/sequenced_worker_pool.h"
 #include "base/threading/thread.h"
 #include "base/values.h"
 #include "components/prefs/persistent_pref_store_unittest.h"
@@ -59,12 +58,6 @@ const char kWriteGolden[] =
      "\"some_directory\":\"/usr/sbin/\","
      "\"tabs\":{\"max_tabs\":10,\"new_windows_in_tabs\":false}}";
 
-// Set the time on the given SimpleTestClock to the given time in minutes.
-void SetCurrentTimeInMinutes(double minutes, base::SimpleTestClock* clock) {
-  const int32_t kBaseTimeMins = 100;
-  clock->SetNow(base::Time::FromDoubleT((kBaseTimeMins + minutes) * 60));
-}
-
 // A PrefFilter that will intercept all calls to FilterOnLoad() and hold on
 // to the |prefs| until explicitly asked to release them.
 class InterceptingPrefFilter : public PrefFilter {
@@ -82,8 +75,9 @@ class InterceptingPrefFilter : public PrefFilter {
       base::DictionaryValue* pref_store_contents) override {
     return on_write_callback_pair_;
   }
+  void OnStoreDeletionFromDisk() override {}
 
-  bool has_intercepted_prefs() const { return intercepted_prefs_ != NULL; }
+  bool has_intercepted_prefs() const { return intercepted_prefs_ != nullptr; }
 
   // Finalize an intercepted read, handing |intercepted_prefs_| back to its
   // JsonPrefStore.
@@ -131,26 +125,60 @@ class MockReadErrorDelegate : public PersistentPrefStore::ReadErrorDelegate {
 };
 
 enum class CommitPendingWriteMode {
+  // Basic mode.
   WITHOUT_CALLBACK,
+  // With reply callback.
   WITH_CALLBACK,
+  // With synchronous notify callback (synchronous after the write -- shouldn't
+  // require pumping messages to observe).
+  WITH_SYNCHRONOUS_CALLBACK,
 };
 
+base::test::ScopedTaskEnvironment::ExecutionMode GetExecutionMode(
+    CommitPendingWriteMode commit_mode) {
+  switch (commit_mode) {
+    case CommitPendingWriteMode::WITHOUT_CALLBACK:
+      FALLTHROUGH;
+    case CommitPendingWriteMode::WITH_CALLBACK:
+      return base::test::ScopedTaskEnvironment::ExecutionMode::QUEUED;
+    case CommitPendingWriteMode::WITH_SYNCHRONOUS_CALLBACK:
+      // Synchronous callbacks require async tasks to run on their own.
+      return base::test::ScopedTaskEnvironment::ExecutionMode::ASYNC;
+  }
+}
+
 void CommitPendingWrite(
     JsonPrefStore* pref_store,
     CommitPendingWriteMode commit_pending_write_mode,
     base::test::ScopedTaskEnvironment* scoped_task_environment) {
-  if (commit_pending_write_mode == CommitPendingWriteMode::WITHOUT_CALLBACK) {
-    pref_store->CommitPendingWrite(OnceClosure());
-    scoped_task_environment->RunUntilIdle();
-  } else {
-    TestCommitPendingWriteWithCallback(pref_store);
+  switch (commit_pending_write_mode) {
+    case CommitPendingWriteMode::WITHOUT_CALLBACK: {
+      pref_store->CommitPendingWrite();
+      scoped_task_environment->RunUntilIdle();
+      break;
+    }
+    case CommitPendingWriteMode::WITH_CALLBACK: {
+      TestCommitPendingWriteWithCallback(pref_store, scoped_task_environment);
+      break;
+    }
+    case CommitPendingWriteMode::WITH_SYNCHRONOUS_CALLBACK: {
+      base::WaitableEvent written;
+      pref_store->CommitPendingWrite(
+          base::OnceClosure(),
+          base::BindOnce(&base::WaitableEvent::Signal, Unretained(&written)));
+      written.Wait();
+      break;
+    }
   }
 }
 
 class JsonPrefStoreTest
     : public testing::TestWithParam<CommitPendingWriteMode> {
  public:
-  JsonPrefStoreTest() = default;
+  JsonPrefStoreTest()
+      : scoped_task_environment_(
+            base::test::ScopedTaskEnvironment::MainThreadType::DEFAULT,
+            GetExecutionMode(GetParam())) {}
 
  protected:
   void SetUp() override {
@@ -168,7 +196,7 @@ class JsonPrefStoreTest
 }  // namespace
 
 // Test fallback behavior for a nonexistent file.
-TEST_F(JsonPrefStoreTest, NonExistentFile) {
+TEST_P(JsonPrefStoreTest, NonExistentFile) {
   base::FilePath bogus_input_file = temp_dir_.GetPath().AppendASCII("read.txt");
   ASSERT_FALSE(PathExists(bogus_input_file));
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(bogus_input_file);
@@ -178,10 +206,10 @@ TEST_F(JsonPrefStoreTest, NonExistentFile) {
 }
 
 // Test fallback behavior for an invalid file.
-TEST_F(JsonPrefStoreTest, InvalidFile) {
+TEST_P(JsonPrefStoreTest, InvalidFile) {
   base::FilePath invalid_file = temp_dir_.GetPath().AppendASCII("invalid.json");
-  ASSERT_LT(0, base::WriteFile(invalid_file,
-                               kInvalidJson, arraysize(kInvalidJson) - 1));
+  ASSERT_LT(0, base::WriteFile(invalid_file, kInvalidJson,
+                               base::size(kInvalidJson) - 1));
 
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(invalid_file);
   EXPECT_EQ(PersistentPrefStore::PREF_READ_ERROR_JSON_PARSE,
@@ -227,7 +255,7 @@ void RunBasicJsonPrefStoreTest(
   base::FilePath some_path(FILE_PATH_LITERAL("/usr/sbin/"));
 
   pref_store->SetValue(kSomeDirectory,
-                       base::MakeUnique<Value>(some_path.value()),
+                       std::make_unique<Value>(some_path.value()),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   EXPECT_TRUE(pref_store->GetValue(kSomeDirectory, &actual));
   EXPECT_TRUE(actual->GetAsString(&path));
@@ -239,7 +267,7 @@ void RunBasicJsonPrefStoreTest(
   EXPECT_TRUE(actual->GetAsBoolean(&boolean));
   EXPECT_TRUE(boolean);
 
-  pref_store->SetValue(kNewWindowsInTabs, base::MakeUnique<Value>(false),
+  pref_store->SetValue(kNewWindowsInTabs, std::make_unique<Value>(false),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   EXPECT_TRUE(pref_store->GetValue(kNewWindowsInTabs, &actual));
   EXPECT_TRUE(actual->GetAsBoolean(&boolean));
@@ -249,7 +277,7 @@ void RunBasicJsonPrefStoreTest(
   int integer = 0;
   EXPECT_TRUE(actual->GetAsInteger(&integer));
   EXPECT_EQ(20, integer);
-  pref_store->SetValue(kMaxTabs, base::MakeUnique<Value>(10),
+  pref_store->SetValue(kMaxTabs, std::make_unique<Value>(10),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   EXPECT_TRUE(pref_store->GetValue(kMaxTabs, &actual));
   EXPECT_TRUE(actual->GetAsInteger(&integer));
@@ -257,7 +285,7 @@ void RunBasicJsonPrefStoreTest(
 
   pref_store->SetValue(
       kLongIntPref,
-      base::MakeUnique<Value>(base::Int64ToString(214748364842LL)),
+      std::make_unique<Value>(base::Int64ToString(214748364842LL)),
       WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   EXPECT_TRUE(pref_store->GetValue(kLongIntPref, &actual));
   EXPECT_TRUE(actual->GetAsString(&string_value));
@@ -277,8 +305,8 @@ void RunBasicJsonPrefStoreTest(
 
 TEST_P(JsonPrefStoreTest, Basic) {
   base::FilePath input_file = temp_dir_.GetPath().AppendASCII("write.json");
-  ASSERT_LT(0, base::WriteFile(input_file,
-                               kReadJson, arraysize(kReadJson) - 1));
+  ASSERT_LT(0,
+            base::WriteFile(input_file, kReadJson, base::size(kReadJson) - 1));
 
   // Test that the persistent value can be loaded.
   ASSERT_TRUE(PathExists(input_file));
@@ -303,8 +331,8 @@ TEST_P(JsonPrefStoreTest, Basic) {
 
 TEST_P(JsonPrefStoreTest, BasicAsync) {
   base::FilePath input_file = temp_dir_.GetPath().AppendASCII("write.json");
-  ASSERT_LT(0, base::WriteFile(input_file,
-                               kReadJson, arraysize(kReadJson) - 1));
+  ASSERT_LT(0,
+            base::WriteFile(input_file, kReadJson, base::size(kReadJson) - 1));
 
   // Test that the persistent value can be loaded.
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(input_file);
@@ -346,9 +374,9 @@ TEST_P(JsonPrefStoreTest, PreserveEmptyValues) {
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(pref_file);
 
   // Set some keys with empty values.
-  pref_store->SetValue("list", base::MakeUnique<base::ListValue>(),
+  pref_store->SetValue("list", std::make_unique<base::ListValue>(),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
-  pref_store->SetValue("dict", base::MakeUnique<base::DictionaryValue>(),
+  pref_store->SetValue("dict", std::make_unique<base::DictionaryValue>(),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
 
   // Write to file.
@@ -360,7 +388,7 @@ TEST_P(JsonPrefStoreTest, PreserveEmptyValues) {
   ASSERT_FALSE(pref_store->ReadOnly());
 
   // Check values.
-  const Value* result = NULL;
+  const Value* result = nullptr;
   EXPECT_TRUE(pref_store->GetValue("list", &result));
   EXPECT_TRUE(ListValue().Equals(result));
   EXPECT_TRUE(pref_store->GetValue("dict", &result));
@@ -369,7 +397,7 @@ TEST_P(JsonPrefStoreTest, PreserveEmptyValues) {
 
 // This test is just documenting some potentially non-obvious behavior. It
 // shouldn't be taken as normative.
-TEST_F(JsonPrefStoreTest, RemoveClearsEmptyParent) {
+TEST_P(JsonPrefStoreTest, RemoveClearsEmptyParent) {
   FilePath pref_file = temp_dir_.GetPath().AppendASCII("empty_values.json");
 
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(pref_file);
@@ -382,13 +410,13 @@ TEST_F(JsonPrefStoreTest, RemoveClearsEmptyParent) {
   pref_store->RemoveValue("dict.key",
                           WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
 
-  const base::Value* retrieved_dict = NULL;
+  const base::Value* retrieved_dict = nullptr;
   bool has_dict = pref_store->GetValue("dict", &retrieved_dict);
   EXPECT_FALSE(has_dict);
 }
 
 // Tests asynchronous reading of the file when there is no file.
-TEST_F(JsonPrefStoreTest, AsyncNonExistingFile) {
+TEST_P(JsonPrefStoreTest, AsyncNonExistingFile) {
   base::FilePath bogus_input_file = temp_dir_.GetPath().AppendASCII("read.txt");
   ASSERT_FALSE(PathExists(bogus_input_file));
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(bogus_input_file);
@@ -409,16 +437,15 @@ TEST_F(JsonPrefStoreTest, AsyncNonExistingFile) {
 
 TEST_P(JsonPrefStoreTest, ReadWithInterceptor) {
   base::FilePath input_file = temp_dir_.GetPath().AppendASCII("write.json");
-  ASSERT_LT(0, base::WriteFile(input_file,
-                               kReadJson, arraysize(kReadJson) - 1));
+  ASSERT_LT(0,
+            base::WriteFile(input_file, kReadJson, base::size(kReadJson) - 1));
 
   std::unique_ptr<InterceptingPrefFilter> intercepting_pref_filter(
       new InterceptingPrefFilter());
   InterceptingPrefFilter* raw_intercepting_pref_filter_ =
       intercepting_pref_filter.get();
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(
-      input_file, base::CreateSequencedTaskRunnerWithTraits({base::MayBlock()}),
-      std::move(intercepting_pref_filter));
+      input_file, std::move(intercepting_pref_filter));
 
   ASSERT_EQ(PersistentPrefStore::PREF_READ_ERROR_ASYNCHRONOUS_TASK_INCOMPLETE,
             pref_store->ReadPrefs());
@@ -428,13 +455,13 @@ TEST_P(JsonPrefStoreTest, ReadWithInterceptor) {
   // returns.
   EXPECT_TRUE(raw_intercepting_pref_filter_->has_intercepted_prefs());
   EXPECT_FALSE(pref_store->IsInitializationComplete());
-  EXPECT_FALSE(pref_store->GetValue(kHomePage, NULL));
+  EXPECT_FALSE(pref_store->GetValue(kHomePage, nullptr));
 
   raw_intercepting_pref_filter_->ReleasePrefs();
 
   EXPECT_FALSE(raw_intercepting_pref_filter_->has_intercepted_prefs());
   EXPECT_TRUE(pref_store->IsInitializationComplete());
-  EXPECT_TRUE(pref_store->GetValue(kHomePage, NULL));
+  EXPECT_TRUE(pref_store->GetValue(kHomePage, nullptr));
 
   // The JSON file looks like this:
   // {
@@ -452,16 +479,15 @@ TEST_P(JsonPrefStoreTest, ReadWithInterceptor) {
 
 TEST_P(JsonPrefStoreTest, ReadAsyncWithInterceptor) {
   base::FilePath input_file = temp_dir_.GetPath().AppendASCII("write.json");
-  ASSERT_LT(0, base::WriteFile(input_file,
-                               kReadJson, arraysize(kReadJson) - 1));
+  ASSERT_LT(0,
+            base::WriteFile(input_file, kReadJson, base::size(kReadJson) - 1));
 
   std::unique_ptr<InterceptingPrefFilter> intercepting_pref_filter(
       new InterceptingPrefFilter());
   InterceptingPrefFilter* raw_intercepting_pref_filter_ =
       intercepting_pref_filter.get();
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(
-      input_file, base::CreateSequencedTaskRunnerWithTraits({base::MayBlock()}),
-      std::move(intercepting_pref_filter));
+      input_file, std::move(intercepting_pref_filter));
 
   MockPrefStoreObserver mock_observer;
   pref_store->AddObserver(&mock_observer);
@@ -480,7 +506,7 @@ TEST_P(JsonPrefStoreTest, ReadAsyncWithInterceptor) {
     EXPECT_FALSE(pref_store->ReadOnly());
     EXPECT_TRUE(raw_intercepting_pref_filter_->has_intercepted_prefs());
     EXPECT_FALSE(pref_store->IsInitializationComplete());
-    EXPECT_FALSE(pref_store->GetValue(kHomePage, NULL));
+    EXPECT_FALSE(pref_store->GetValue(kHomePage, nullptr));
   }
 
   {
@@ -493,7 +519,7 @@ TEST_P(JsonPrefStoreTest, ReadAsyncWithInterceptor) {
     EXPECT_FALSE(pref_store->ReadOnly());
     EXPECT_FALSE(raw_intercepting_pref_filter_->has_intercepted_prefs());
     EXPECT_TRUE(pref_store->IsInitializationComplete());
-    EXPECT_TRUE(pref_store->GetValue(kHomePage, NULL));
+    EXPECT_TRUE(pref_store->GetValue(kHomePage, nullptr));
   }
 
   pref_store->RemoveObserver(&mock_observer);
@@ -512,155 +538,6 @@ TEST_P(JsonPrefStoreTest, ReadAsyncWithInterceptor) {
                             &scoped_task_environment_);
 }
 
-TEST_F(JsonPrefStoreTest, WriteCountHistogramTestBasic) {
-  base::HistogramTester histogram_tester;
-
-  SimpleTestClock* test_clock = new SimpleTestClock;
-  SetCurrentTimeInMinutes(0, test_clock);
-  JsonPrefStore::WriteCountHistogram histogram(
-      base::TimeDelta::FromSeconds(10),
-      base::FilePath(FILE_PATH_LITERAL("/tmp/Local State")),
-      std::unique_ptr<base::Clock>(test_clock));
-  int32_t report_interval =
-      JsonPrefStore::WriteCountHistogram::kHistogramWriteReportIntervalMins;
-
-  histogram.RecordWriteOccured();
-
-  SetCurrentTimeInMinutes(1.5 * report_interval, test_clock);
-  histogram.ReportOutstandingWrites();
-  std::unique_ptr<HistogramSamples> samples =
-      histogram.GetHistogram()->SnapshotSamples();
-
-  std::string histogram_name = histogram.GetHistogram()->histogram_name();
-  histogram_tester.ExpectBucketCount(histogram_name, 1, 1);
-  histogram_tester.ExpectTotalCount(histogram_name, 1);
-
-  ASSERT_EQ("Settings.JsonDataWriteCount.Local_State",
-            histogram.GetHistogram()->histogram_name());
-  ASSERT_TRUE(histogram.GetHistogram()->HasConstructionArguments(1, 30, 31));
-}
-
-TEST_F(JsonPrefStoreTest, WriteCountHistogramTestSinglePeriod) {
-  base::HistogramTester histogram_tester;
-
-  SimpleTestClock* test_clock = new SimpleTestClock;
-  SetCurrentTimeInMinutes(0, test_clock);
-  JsonPrefStore::WriteCountHistogram histogram(
-      base::TimeDelta::FromSeconds(10),
-      base::FilePath(FILE_PATH_LITERAL("/tmp/Local State")),
-      std::unique_ptr<base::Clock>(test_clock));
-  int32_t report_interval =
-      JsonPrefStore::WriteCountHistogram::kHistogramWriteReportIntervalMins;
-
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(0.5 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(0.7 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-
-  // Nothing should be recorded until the report period has elapsed.
-  std::string histogram_name = histogram.GetHistogram()->histogram_name();
-  histogram_tester.ExpectTotalCount(histogram_name, 0);
-
-  SetCurrentTimeInMinutes(1.3 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-
-  // Now the report period has elapsed.
-  histogram_tester.ExpectBucketCount(histogram_name, 3, 1);
-  histogram_tester.ExpectTotalCount(histogram_name, 1);
-
-  // The last write won't be recorded because the second count period hasn't
-  // fully elapsed.
-  SetCurrentTimeInMinutes(1.5 * report_interval, test_clock);
-  histogram.ReportOutstandingWrites();
-
-  histogram_tester.ExpectBucketCount(histogram_name, 3, 1);
-  histogram_tester.ExpectTotalCount(histogram_name, 1);
-}
-
-TEST_F(JsonPrefStoreTest, WriteCountHistogramTestMultiplePeriods) {
-  base::HistogramTester histogram_tester;
-
-  SimpleTestClock* test_clock = new SimpleTestClock;
-  SetCurrentTimeInMinutes(0, test_clock);
-  JsonPrefStore::WriteCountHistogram histogram(
-      base::TimeDelta::FromSeconds(10),
-      base::FilePath(FILE_PATH_LITERAL("/tmp/Local State")),
-      std::unique_ptr<base::Clock>(test_clock));
-  int32_t report_interval =
-      JsonPrefStore::WriteCountHistogram::kHistogramWriteReportIntervalMins;
-
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(0.5 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(0.7 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(1.3 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(1.5 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(2.1 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(2.5 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(2.7 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(3.3 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-
-  // The last write won't be recorded because the second count period hasn't
-  // fully elapsed
-  SetCurrentTimeInMinutes(3.5 * report_interval, test_clock);
-  histogram.ReportOutstandingWrites();
-  std::string histogram_name = histogram.GetHistogram()->histogram_name();
-  histogram_tester.ExpectBucketCount(histogram_name, 3, 2);
-  histogram_tester.ExpectBucketCount(histogram_name, 2, 1);
-  histogram_tester.ExpectTotalCount(histogram_name, 3);
-}
-
-TEST_F(JsonPrefStoreTest, WriteCountHistogramTestPeriodWithGaps) {
-  base::HistogramTester histogram_tester;
-
-  SimpleTestClock* test_clock = new SimpleTestClock;
-  SetCurrentTimeInMinutes(0, test_clock);
-  JsonPrefStore::WriteCountHistogram histogram(
-      base::TimeDelta::FromSeconds(10),
-      base::FilePath(FILE_PATH_LITERAL("/tmp/Local State")),
-      std::unique_ptr<base::Clock>(test_clock));
-  int32_t report_interval =
-      JsonPrefStore::WriteCountHistogram::kHistogramWriteReportIntervalMins;
-
-  // 1 write in the first period.
-  histogram.RecordWriteOccured();
-
-  // No writes in the second and third periods.
-
-  // 2 writes in the fourth period.
-  SetCurrentTimeInMinutes(3.1 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(3.3 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-
-  // No writes in the fifth period.
-
-  // 3 writes in the sixth period.
-  SetCurrentTimeInMinutes(5.1 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(5.3 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-  SetCurrentTimeInMinutes(5.5 * report_interval, test_clock);
-  histogram.RecordWriteOccured();
-
-  SetCurrentTimeInMinutes(6.1 * report_interval, test_clock);
-  histogram.ReportOutstandingWrites();
-  std::string histogram_name = histogram.GetHistogram()->histogram_name();
-  histogram_tester.ExpectBucketCount(histogram_name, 0, 3);
-  histogram_tester.ExpectBucketCount(histogram_name, 1, 1);
-  histogram_tester.ExpectBucketCount(histogram_name, 2, 1);
-  histogram_tester.ExpectBucketCount(histogram_name, 3, 1);
-  histogram_tester.ExpectTotalCount(histogram_name, 6);
-}
-
 INSTANTIATE_TEST_CASE_P(
     WithoutCallback,
     JsonPrefStoreTest,
@@ -669,6 +546,10 @@ INSTANTIATE_TEST_CASE_P(
     WithCallback,
     JsonPrefStoreTest,
     ::testing::Values(CommitPendingWriteMode::WITH_CALLBACK));
+INSTANTIATE_TEST_CASE_P(
+    WithSynchronousCallback,
+    JsonPrefStoreTest,
+    ::testing::Values(CommitPendingWriteMode::WITH_SYNCHRONOUS_CALLBACK));
 
 class JsonPrefStoreLossyWriteTest : public JsonPrefStoreTest {
  public:
@@ -704,13 +585,13 @@ class JsonPrefStoreLossyWriteTest : public JsonPrefStoreTest {
   DISALLOW_COPY_AND_ASSIGN(JsonPrefStoreLossyWriteTest);
 };
 
-TEST_F(JsonPrefStoreLossyWriteTest, LossyWriteBasic) {
+TEST_P(JsonPrefStoreLossyWriteTest, LossyWriteBasic) {
   scoped_refptr<JsonPrefStore> pref_store = CreatePrefStore();
   ImportantFileWriter* file_writer = GetImportantFileWriter(pref_store.get());
 
   // Set a normal pref and check that it gets scheduled to be written.
   ASSERT_FALSE(file_writer->HasPendingWrite());
-  pref_store->SetValue("normal", base::MakeUnique<base::Value>("normal"),
+  pref_store->SetValue("normal", std::make_unique<base::Value>("normal"),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   ASSERT_TRUE(file_writer->HasPendingWrite());
   file_writer->DoScheduledWrite();
@@ -719,14 +600,14 @@ TEST_F(JsonPrefStoreLossyWriteTest, LossyWriteBasic) {
 
   // Set a lossy pref and check that it is not scheduled to be written.
   // SetValue/RemoveValue.
-  pref_store->SetValue("lossy", base::MakeUnique<base::Value>("lossy"),
+  pref_store->SetValue("lossy", std::make_unique<base::Value>("lossy"),
                        WriteablePrefStore::LOSSY_PREF_WRITE_FLAG);
   ASSERT_FALSE(file_writer->HasPendingWrite());
   pref_store->RemoveValue("lossy", WriteablePrefStore::LOSSY_PREF_WRITE_FLAG);
   ASSERT_FALSE(file_writer->HasPendingWrite());
 
   // SetValueSilently/RemoveValueSilently.
-  pref_store->SetValueSilently("lossy", base::MakeUnique<base::Value>("lossy"),
+  pref_store->SetValueSilently("lossy", std::make_unique<base::Value>("lossy"),
                                WriteablePrefStore::LOSSY_PREF_WRITE_FLAG);
   ASSERT_FALSE(file_writer->HasPendingWrite());
   pref_store->RemoveValueSilently("lossy",
@@ -734,7 +615,7 @@ TEST_F(JsonPrefStoreLossyWriteTest, LossyWriteBasic) {
   ASSERT_FALSE(file_writer->HasPendingWrite());
 
   // ReportValueChanged.
-  pref_store->SetValue("lossy", base::MakeUnique<base::Value>("lossy"),
+  pref_store->SetValue("lossy", std::make_unique<base::Value>("lossy"),
                        WriteablePrefStore::LOSSY_PREF_WRITE_FLAG);
   ASSERT_FALSE(file_writer->HasPendingWrite());
   pref_store->ReportValueChanged("lossy",
@@ -749,18 +630,18 @@ TEST_F(JsonPrefStoreLossyWriteTest, LossyWriteBasic) {
             GetTestFileContents());
 }
 
-TEST_F(JsonPrefStoreLossyWriteTest, LossyWriteMixedLossyFirst) {
+TEST_P(JsonPrefStoreLossyWriteTest, LossyWriteMixedLossyFirst) {
   scoped_refptr<JsonPrefStore> pref_store = CreatePrefStore();
   ImportantFileWriter* file_writer = GetImportantFileWriter(pref_store.get());
 
   // Set a lossy pref and check that it is not scheduled to be written.
   ASSERT_FALSE(file_writer->HasPendingWrite());
-  pref_store->SetValue("lossy", base::MakeUnique<base::Value>("lossy"),
+  pref_store->SetValue("lossy", std::make_unique<base::Value>("lossy"),
                        WriteablePrefStore::LOSSY_PREF_WRITE_FLAG);
   ASSERT_FALSE(file_writer->HasPendingWrite());
 
   // Set a normal pref and check that it is scheduled to be written.
-  pref_store->SetValue("normal", base::MakeUnique<base::Value>("normal"),
+  pref_store->SetValue("normal", std::make_unique<base::Value>("normal"),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   ASSERT_TRUE(file_writer->HasPendingWrite());
 
@@ -771,18 +652,18 @@ TEST_F(JsonPrefStoreLossyWriteTest, LossyWriteMixedLossyFirst) {
   ASSERT_FALSE(file_writer->HasPendingWrite());
 }
 
-TEST_F(JsonPrefStoreLossyWriteTest, LossyWriteMixedLossySecond) {
+TEST_P(JsonPrefStoreLossyWriteTest, LossyWriteMixedLossySecond) {
   scoped_refptr<JsonPrefStore> pref_store = CreatePrefStore();
   ImportantFileWriter* file_writer = GetImportantFileWriter(pref_store.get());
 
   // Set a normal pref and check that it is scheduled to be written.
   ASSERT_FALSE(file_writer->HasPendingWrite());
-  pref_store->SetValue("normal", base::MakeUnique<base::Value>("normal"),
+  pref_store->SetValue("normal", std::make_unique<base::Value>("normal"),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   ASSERT_TRUE(file_writer->HasPendingWrite());
 
   // Set a lossy pref and check that the write is still scheduled.
-  pref_store->SetValue("lossy", base::MakeUnique<base::Value>("lossy"),
+  pref_store->SetValue("lossy", std::make_unique<base::Value>("lossy"),
                        WriteablePrefStore::LOSSY_PREF_WRITE_FLAG);
   ASSERT_TRUE(file_writer->HasPendingWrite());
 
@@ -793,12 +674,12 @@ TEST_F(JsonPrefStoreLossyWriteTest, LossyWriteMixedLossySecond) {
   ASSERT_FALSE(file_writer->HasPendingWrite());
 }
 
-TEST_F(JsonPrefStoreLossyWriteTest, ScheduleLossyWrite) {
+TEST_P(JsonPrefStoreLossyWriteTest, ScheduleLossyWrite) {
   scoped_refptr<JsonPrefStore> pref_store = CreatePrefStore();
   ImportantFileWriter* file_writer = GetImportantFileWriter(pref_store.get());
 
   // Set a lossy pref and check that it is not scheduled to be written.
-  pref_store->SetValue("lossy", base::MakeUnique<base::Value>("lossy"),
+  pref_store->SetValue("lossy", std::make_unique<base::Value>("lossy"),
                        WriteablePrefStore::LOSSY_PREF_WRITE_FLAG);
   ASSERT_FALSE(file_writer->HasPendingWrite());
 
@@ -813,6 +694,19 @@ TEST_F(JsonPrefStoreLossyWriteTest, ScheduleLossyWrite) {
   ASSERT_EQ("{\"lossy\":\"lossy\"}", GetTestFileContents());
 }
 
+INSTANTIATE_TEST_CASE_P(
+    WithoutCallback,
+    JsonPrefStoreLossyWriteTest,
+    ::testing::Values(CommitPendingWriteMode::WITHOUT_CALLBACK));
+INSTANTIATE_TEST_CASE_P(
+    WithReply,
+    JsonPrefStoreLossyWriteTest,
+    ::testing::Values(CommitPendingWriteMode::WITH_CALLBACK));
+INSTANTIATE_TEST_CASE_P(
+    WithNotify,
+    JsonPrefStoreLossyWriteTest,
+    ::testing::Values(CommitPendingWriteMode::WITH_SYNCHRONOUS_CALLBACK));
+
 class SuccessfulWriteReplyObserver {
  public:
   SuccessfulWriteReplyObserver() = default;
@@ -910,13 +804,13 @@ WriteCallbacksObserver::GetAndResetPostWriteObservationState() {
   return state;
 }
 
-class JsonPrefStoreCallbackTest : public JsonPrefStoreTest {
+class JsonPrefStoreCallbackTest : public testing::Test {
  public:
   JsonPrefStoreCallbackTest() = default;
 
  protected:
   void SetUp() override {
-    JsonPrefStoreTest::SetUp();
+    ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
     test_file_ = temp_dir_.GetPath().AppendASCII("test.json");
   }
 
@@ -941,33 +835,33 @@ class JsonPrefStoreCallbackTest : public JsonPrefStoreTest {
   SuccessfulWriteReplyObserver successful_write_reply_observer_;
   WriteCallbacksObserver write_callback_observer_;
 
+ protected:
+  base::test::ScopedTaskEnvironment scoped_task_environment_{
+      base::test::ScopedTaskEnvironment::MainThreadType::DEFAULT,
+      base::test::ScopedTaskEnvironment::ExecutionMode::QUEUED};
+
+  base::ScopedTempDir temp_dir_;
+
  private:
   base::FilePath test_file_;
 
   DISALLOW_COPY_AND_ASSIGN(JsonPrefStoreCallbackTest);
 };
 
-// Flaky on Linux TSAN. http://crbug.com/732445.
-#if defined(OS_LINUX) && defined(THREAD_SANITIZER)
-#define MAYBE_TestSerializeDataCallbacks DISABLED_TestSerializeDataCallbacks
-#else
-#define MAYBE_TestSerializeDataCallbacks TestSerializeDataCallbacks
-#endif
-TEST_F(JsonPrefStoreCallbackTest, MAYBE_TestSerializeDataCallbacks) {
+TEST_F(JsonPrefStoreCallbackTest, TestSerializeDataCallbacks) {
   base::FilePath input_file = temp_dir_.GetPath().AppendASCII("write.json");
   ASSERT_LT(0,
-            base::WriteFile(input_file, kReadJson, arraysize(kReadJson) - 1));
+            base::WriteFile(input_file, kReadJson, base::size(kReadJson) - 1));
 
   std::unique_ptr<InterceptingPrefFilter> intercepting_pref_filter(
       new InterceptingPrefFilter(write_callback_observer_.GetCallbackPair()));
   auto pref_store = base::MakeRefCounted<JsonPrefStore>(
-      input_file, base::CreateSequencedTaskRunnerWithTraits({base::MayBlock()}),
-      std::move(intercepting_pref_filter));
+      input_file, std::move(intercepting_pref_filter));
   ImportantFileWriter* file_writer = GetImportantFileWriter(pref_store.get());
 
   EXPECT_EQ(NOT_CALLED,
             write_callback_observer_.GetAndResetPostWriteObservationState());
-  pref_store->SetValue("normal", base::MakeUnique<base::Value>("normal"),
+  pref_store->SetValue("normal", std::make_unique<base::Value>("normal"),
                        WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   file_writer->DoScheduledWrite();
 
@@ -991,7 +885,7 @@ TEST_F(JsonPrefStoreCallbackTest, TestPostWriteCallbacks) {
   // RegisterOnNextSuccessfulWriteReply.
   successful_write_reply_observer_.ObserveNextWriteCallback(pref_store.get());
   write_callback_observer_.ObserveNextWriteCallback(pref_store.get());
-  file_writer->WriteNow(MakeUnique<std::string>("foo"));
+  file_writer->WriteNow(std::make_unique<std::string>("foo"));
   scoped_task_environment_.RunUntilIdle();
   EXPECT_TRUE(successful_write_reply_observer_.GetAndResetObservationState());
   EXPECT_TRUE(write_callback_observer_.GetAndResetPreWriteObservationState());
@@ -1002,7 +896,7 @@ TEST_F(JsonPrefStoreCallbackTest, TestPostWriteCallbacks) {
   // RegisterOnNextWriteSynchronousCallbacks.
   successful_write_reply_observer_.ObserveNextWriteCallback(pref_store.get());
   write_callback_observer_.ObserveNextWriteCallback(pref_store.get());
-  file_writer->WriteNow(MakeUnique<std::string>("foo"));
+  file_writer->WriteNow(std::make_unique<std::string>("foo"));
   scoped_task_environment_.RunUntilIdle();
   EXPECT_TRUE(successful_write_reply_observer_.GetAndResetObservationState());
   EXPECT_TRUE(write_callback_observer_.GetAndResetPreWriteObservationState());
@@ -1011,7 +905,7 @@ TEST_F(JsonPrefStoreCallbackTest, TestPostWriteCallbacks) {
 
   // Test RegisterOnNextSuccessfulWriteReply only.
   successful_write_reply_observer_.ObserveNextWriteCallback(pref_store.get());
-  file_writer->WriteNow(MakeUnique<std::string>("foo"));
+  file_writer->WriteNow(std::make_unique<std::string>("foo"));
   scoped_task_environment_.RunUntilIdle();
   EXPECT_TRUE(successful_write_reply_observer_.GetAndResetObservationState());
   EXPECT_FALSE(write_callback_observer_.GetAndResetPreWriteObservationState());
@@ -1020,7 +914,7 @@ TEST_F(JsonPrefStoreCallbackTest, TestPostWriteCallbacks) {
 
   // Test RegisterOnNextWriteSynchronousCallbacks only.
   write_callback_observer_.ObserveNextWriteCallback(pref_store.get());
-  file_writer->WriteNow(MakeUnique<std::string>("foo"));
+  file_writer->WriteNow(std::make_unique<std::string>("foo"));
   scoped_task_environment_.RunUntilIdle();
   EXPECT_FALSE(successful_write_reply_observer_.GetAndResetObservationState());
   EXPECT_TRUE(write_callback_observer_.GetAndResetPreWriteObservationState());
@@ -1064,7 +958,7 @@ TEST_F(JsonPrefStoreCallbackTest, TestPostWriteCallbacksWithFakeFailure) {
   // Do a real write, and confirm that the successful observer was invoked after
   // being set by |PostWriteCallback| by the last TriggerFakeWriteCallback.
   ImportantFileWriter* file_writer = GetImportantFileWriter(pref_store.get());
-  file_writer->WriteNow(MakeUnique<std::string>("foo"));
+  file_writer->WriteNow(std::make_unique<std::string>("foo"));
   scoped_task_environment_.RunUntilIdle();
   EXPECT_TRUE(successful_write_reply_observer_.GetAndResetObservationState());
   EXPECT_EQ(NOT_CALLED,
@@ -1083,7 +977,7 @@ TEST_F(JsonPrefStoreCallbackTest, TestPostWriteCallbacksDuringProfileDeath) {
         soon_out_of_scope_pref_store.get());
     write_callback_observer_.ObserveNextWriteCallback(
         soon_out_of_scope_pref_store.get());
-    file_writer->WriteNow(MakeUnique<std::string>("foo"));
+    file_writer->WriteNow(std::make_unique<std::string>("foo"));
   }
   scoped_task_environment_.RunUntilIdle();
   EXPECT_FALSE(successful_write_reply_observer_.GetAndResetObservationState());
