@@ -5,6 +5,7 @@
 #include "components/prefs/pref_service.h"
 
 #include <algorithm>
+#include <map>
 #include <utility>
 
 #include "base/bind.h"
@@ -23,21 +24,23 @@
 #include "components/prefs/default_pref_store.h"
 #include "components/prefs/pref_notifier_impl.h"
 #include "components/prefs/pref_registry.h"
-#include "components/prefs/pref_value_store.h"
 
 namespace {
 
 class ReadErrorHandler : public PersistentPrefStore::ReadErrorDelegate {
  public:
-  ReadErrorHandler(base::Callback<void(PersistentPrefStore::PrefReadError)> cb)
-      : callback_(cb) {}
+  using ErrorCallback =
+      base::RepeatingCallback<void(PersistentPrefStore::PrefReadError)>;
+  explicit ReadErrorHandler(ErrorCallback cb) : callback_(cb) {}
 
   void OnError(PersistentPrefStore::PrefReadError error) override {
     callback_.Run(error);
   }
 
  private:
-  base::Callback<void(PersistentPrefStore::PrefReadError)> callback_;
+  ErrorCallback callback_;
+
+  DISALLOW_COPY_AND_ASSIGN(ReadErrorHandler);
 };
 
 // Returns the WriteablePrefStore::PrefWriteFlags for the pref with the given
@@ -53,40 +56,54 @@ uint32_t GetWriteFlags(const PrefService::Preference* pref) {
   return write_flags;
 }
 
+// For prefs names in |pref_store| that are not presented in |pref_changed_map|,
+// check if their values differ from those in pref_service->FindPreference() and
+// add the result into |pref_changed_map|.
+void CheckForNewPrefChangesInPrefStore(
+    std::map<std::string, bool>* pref_changed_map,
+    PrefStore* pref_store,
+    PrefService* pref_service) {
+  if (!pref_store)
+    return;
+  auto values = pref_store->GetValues();
+  for (const auto& item : values->DictItems()) {
+    // If the key already presents, skip it as a store with higher precedence
+    // already sets the entry.
+    if (pref_changed_map->find(item.first) != pref_changed_map->end())
+      continue;
+    const PrefService::Preference* pref =
+        pref_service->FindPreference(item.first);
+    if (!pref)
+      continue;
+    pref_changed_map->emplace(item.first, *(pref->GetValue()) != item.second);
+  }
+}
+
 }  // namespace
 
 PrefService::PrefService(
-    PrefNotifierImpl* pref_notifier,
-    PrefValueStore* pref_value_store,
-    PersistentPrefStore* user_prefs,
-    PrefRegistry* pref_registry,
-    base::Callback<void(PersistentPrefStore::PrefReadError)>
+    std::unique_ptr<PrefNotifierImpl> pref_notifier,
+    std::unique_ptr<PrefValueStore> pref_value_store,
+    scoped_refptr<PersistentPrefStore> user_prefs,
+    scoped_refptr<PrefRegistry> pref_registry,
+    base::RepeatingCallback<void(PersistentPrefStore::PrefReadError)>
         read_error_callback,
     bool async)
-    : pref_notifier_(pref_notifier),
-      pref_value_store_(pref_value_store),
-      pref_registry_(pref_registry),
-      user_pref_store_(user_prefs),
-      read_error_callback_(read_error_callback) {
+    : pref_notifier_(std::move(pref_notifier)),
+      pref_value_store_(std::move(pref_value_store)),
+      user_pref_store_(std::move(user_prefs)),
+      read_error_callback_(std::move(read_error_callback)),
+      pref_registry_(std::move(pref_registry)) {
   pref_notifier_->SetPrefService(this);
 
-  // TODO(battre): This is a check for crbug.com/435208 to make sure that
-  // access violations are caused by a use-after-free bug and not by an
-  // initialization bug.
-  CHECK(pref_registry_);
-  CHECK(pref_value_store_);
+  DCHECK(pref_registry_);
+  DCHECK(pref_value_store_);
 
   InitFromStorage(async);
 }
 
 PrefService::~PrefService() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-
-  // Reset pointers so accesses after destruction reliably crash.
-  pref_value_store_.reset();
-  pref_registry_ = NULL;
-  user_pref_store_ = NULL;
-  pref_notifier_.reset();
 }
 
 void PrefService::InitFromStorage(bool async) {
@@ -98,18 +115,17 @@ void PrefService::InitFromStorage(bool async) {
     // Guarantee that initialization happens after this function returned.
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
-        base::Bind(&PersistentPrefStore::ReadPrefsAsync, user_pref_store_,
-                   new ReadErrorHandler(read_error_callback_)));
+        base::BindOnce(&PersistentPrefStore::ReadPrefsAsync, user_pref_store_,
+                       new ReadErrorHandler(read_error_callback_)));
   }
 }
 
-void PrefService::CommitPendingWrite() {
-  CommitPendingWrite(base::OnceClosure());
-}
-
-void PrefService::CommitPendingWrite(base::OnceClosure done_callback) {
+void PrefService::CommitPendingWrite(
+    base::OnceClosure reply_callback,
+    base::OnceClosure synchronous_done_callback) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  user_pref_store_->CommitPendingWrite(std::move(done_callback));
+  user_pref_store_->CommitPendingWrite(std::move(reply_callback),
+                                       std::move(synchronous_done_callback));
 }
 
 void PrefService::SchedulePendingLossyWrites() {
@@ -225,15 +241,16 @@ std::unique_ptr<base::DictionaryValue> PrefService::GetPreferenceValues(
 const PrefService::Preference* PrefService::FindPreference(
     const std::string& pref_name) const {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  PreferenceMap::iterator it = prefs_map_.find(pref_name);
+  auto it = prefs_map_.find(pref_name);
   if (it != prefs_map_.end())
     return &(it->second);
-  const base::Value* default_value = NULL;
+  const base::Value* default_value = nullptr;
   if (!pref_registry_->defaults()->GetValue(pref_name, &default_value))
-    return NULL;
-  it = prefs_map_.insert(
-      std::make_pair(pref_name, Preference(
-          this, pref_name, default_value->GetType()))).first;
+    return nullptr;
+  it = prefs_map_
+           .insert(std::make_pair(
+               pref_name, Preference(this, pref_name, default_value->type())))
+           .first;
   return &(it->second);
 }
 
@@ -256,6 +273,14 @@ PrefService::PrefInitializationStatus PrefService::GetInitializationStatus()
   }
 }
 
+PrefService::PrefInitializationStatus
+PrefService::GetAllPrefStoresInitializationStatus() const {
+  if (!pref_value_store_->IsInitializationComplete())
+    return INITIALIZATION_STATUS_WAITING;
+
+  return GetInitializationStatus();
+}
+
 bool PrefService::IsManagedPreference(const std::string& pref_name) const {
   const Preference* pref = FindPreference(pref_name);
   return pref && pref->IsManaged();
@@ -273,6 +298,17 @@ bool PrefService::IsUserModifiablePreference(
   return pref && pref->IsUserModifiable();
 }
 
+const base::Value* PrefService::Get(const std::string& path) const {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
+
+  const base::Value* value = GetPreferenceValue(path);
+  if (!value) {
+    NOTREACHED() << "Trying to read an unregistered pref: " << path;
+    return nullptr;
+  }
+  return value;
+}
+
 const base::DictionaryValue* PrefService::GetDictionary(
     const std::string& path) const {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
@@ -280,11 +316,11 @@ const base::DictionaryValue* PrefService::GetDictionary(
   const base::Value* value = GetPreferenceValue(path);
   if (!value) {
     NOTREACHED() << "Trying to read an unregistered pref: " << path;
-    return NULL;
+    return nullptr;
   }
-  if (value->GetType() != base::Value::Type::DICTIONARY) {
+  if (value->type() != base::Value::Type::DICTIONARY) {
     NOTREACHED();
-    return NULL;
+    return nullptr;
   }
   return static_cast<const base::DictionaryValue*>(value);
 }
@@ -296,38 +332,36 @@ const base::Value* PrefService::GetUserPrefValue(
   const Preference* pref = FindPreference(path);
   if (!pref) {
     NOTREACHED() << "Trying to get an unregistered pref: " << path;
-    return NULL;
+    return nullptr;
   }
 
   // Look for an existing preference in the user store. If it doesn't
   // exist, return NULL.
-  base::Value* value = NULL;
+  base::Value* value = nullptr;
   if (!user_pref_store_->GetMutableValue(path, &value))
-    return NULL;
+    return nullptr;
 
-  if (!value->IsType(pref->GetType())) {
+  if (value->type() != pref->GetType()) {
     NOTREACHED() << "Pref value type doesn't match registered type.";
-    return NULL;
+    return nullptr;
   }
 
   return value;
 }
 
 void PrefService::SetDefaultPrefValue(const std::string& path,
-                                      base::Value* value) {
+                                      base::Value value) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  pref_registry_->SetDefaultPrefValue(path, value);
+  pref_registry_->SetDefaultPrefValue(path, std::move(value));
 }
 
 const base::Value* PrefService::GetDefaultPrefValue(
     const std::string& path) const {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   // Lookup the preference in the default store.
-  const base::Value* value = NULL;
-  if (!pref_registry_->defaults()->GetValue(path, &value)) {
-    NOTREACHED() << "Default value missing for pref: " << path;
-    return NULL;
-  }
+  const base::Value* value = nullptr;
+  bool has_value = pref_registry_->defaults()->GetValue(path, &value);
+  DCHECK(has_value) << "Default value missing for pref: " << path;
   return value;
 }
 
@@ -337,11 +371,11 @@ const base::ListValue* PrefService::GetList(const std::string& path) const {
   const base::Value* value = GetPreferenceValue(path);
   if (!value) {
     NOTREACHED() << "Trying to read an unregistered pref: " << path;
-    return NULL;
+    return nullptr;
   }
-  if (value->GetType() != base::Value::Type::LIST) {
+  if (value->type() != base::Value::Type::LIST) {
     NOTREACHED();
-    return NULL;
+    return nullptr;
   }
   return static_cast<const base::ListValue*>(value);
 }
@@ -355,8 +389,8 @@ void PrefService::RemovePrefObserver(const std::string& path,
   pref_notifier_->RemovePrefObserver(path, obs);
 }
 
-void PrefService::AddPrefInitObserver(base::Callback<void(bool)> obs) {
-  pref_notifier_->AddInitObserver(obs);
+void PrefService::AddPrefInitObserver(base::OnceCallback<void(bool)> obs) {
+  pref_notifier_->AddInitObserver(std::move(obs));
 }
 
 PrefRegistry* PrefService::DeprecatedGetPrefRegistry() {
@@ -378,34 +412,86 @@ void PrefService::ClearMutableValues() {
   user_pref_store_->ClearMutableValues();
 }
 
+void PrefService::OnStoreDeletionFromDisk() {
+  user_pref_store_->OnStoreDeletionFromDisk();
+}
+
+void PrefService::ChangePrefValueStore(
+    PrefStore* managed_prefs,
+    PrefStore* supervised_user_prefs,
+    PrefStore* extension_prefs,
+    PrefStore* recommended_prefs,
+    std::unique_ptr<PrefValueStore::Delegate> delegate) {
+  // Only adding new pref stores are supported.
+  DCHECK(!pref_value_store_->HasPrefStore(PrefValueStore::MANAGED_STORE) ||
+         !managed_prefs);
+  DCHECK(
+      !pref_value_store_->HasPrefStore(PrefValueStore::SUPERVISED_USER_STORE) ||
+      !supervised_user_prefs);
+  DCHECK(!pref_value_store_->HasPrefStore(PrefValueStore::EXTENSION_STORE) ||
+         !extension_prefs);
+  DCHECK(!pref_value_store_->HasPrefStore(PrefValueStore::RECOMMENDED_STORE) ||
+         !recommended_prefs);
+
+  // If some of the stores are already initialized, check for pref value changes
+  // according to store precedence.
+  std::map<std::string, bool> pref_changed_map;
+  CheckForNewPrefChangesInPrefStore(&pref_changed_map, managed_prefs, this);
+  CheckForNewPrefChangesInPrefStore(&pref_changed_map, supervised_user_prefs,
+                                    this);
+  CheckForNewPrefChangesInPrefStore(&pref_changed_map, extension_prefs, this);
+  CheckForNewPrefChangesInPrefStore(&pref_changed_map, recommended_prefs, this);
+
+  pref_value_store_ = pref_value_store_->CloneAndSpecialize(
+      managed_prefs, supervised_user_prefs, extension_prefs,
+      nullptr /* command_line_prefs */, nullptr /* user_prefs */,
+      recommended_prefs, nullptr /* default_prefs */, pref_notifier_.get(),
+      std::move(delegate));
+
+  // Notify |pref_notifier_| on all changed values.
+  for (const auto& kv : pref_changed_map) {
+    if (kv.second)
+      pref_notifier_.get()->OnPreferenceChanged(kv.first);
+  }
+}
+
+void PrefService::AddPrefObserverAllPrefs(PrefObserver* obs) {
+  pref_notifier_->AddPrefObserverAllPrefs(obs);
+}
+
+void PrefService::RemovePrefObserverAllPrefs(PrefObserver* obs) {
+  pref_notifier_->RemovePrefObserverAllPrefs(obs);
+}
+
 void PrefService::Set(const std::string& path, const base::Value& value) {
   SetUserPrefValue(path, value.CreateDeepCopy());
 }
 
 void PrefService::SetBoolean(const std::string& path, bool value) {
-  SetUserPrefValue(path, base::MakeUnique<base::Value>(value));
+  SetUserPrefValue(path, std::make_unique<base::Value>(value));
 }
 
 void PrefService::SetInteger(const std::string& path, int value) {
-  SetUserPrefValue(path, base::MakeUnique<base::Value>(value));
+  SetUserPrefValue(path, std::make_unique<base::Value>(value));
 }
 
 void PrefService::SetDouble(const std::string& path, double value) {
-  SetUserPrefValue(path, base::MakeUnique<base::Value>(value));
+  SetUserPrefValue(path, std::make_unique<base::Value>(value));
 }
 
 void PrefService::SetString(const std::string& path, const std::string& value) {
-  SetUserPrefValue(path, base::MakeUnique<base::Value>(value));
+  SetUserPrefValue(path, std::make_unique<base::Value>(value));
 }
 
 void PrefService::SetFilePath(const std::string& path,
                               const base::FilePath& value) {
-  SetUserPrefValue(path, base::CreateFilePathValue(value));
+  SetUserPrefValue(
+      path, base::Value::ToUniquePtrValue(base::CreateFilePathValue(value)));
 }
 
 void PrefService::SetInt64(const std::string& path, int64_t value) {
   SetUserPrefValue(path,
-                   base::MakeUnique<base::Value>(base::Int64ToString(value)));
+                   std::make_unique<base::Value>(base::Int64ToString(value)));
 }
 
 int64_t PrefService::GetInt64(const std::string& path) const {
@@ -427,7 +513,7 @@ int64_t PrefService::GetInt64(const std::string& path) const {
 
 void PrefService::SetUint64(const std::string& path, uint64_t value) {
   SetUserPrefValue(path,
-                   base::MakeUnique<base::Value>(base::Uint64ToString(value)));
+                   std::make_unique<base::Value>(base::NumberToString(value)));
 }
 
 uint64_t PrefService::GetUint64(const std::string& path) const {
@@ -447,6 +533,23 @@ uint64_t PrefService::GetUint64(const std::string& path) const {
   return val;
 }
 
+void PrefService::SetTime(const std::string& path, base::Time value) {
+  SetInt64(path, value.ToDeltaSinceWindowsEpoch().InMicroseconds());
+}
+
+base::Time PrefService::GetTime(const std::string& path) const {
+  return base::Time::FromDeltaSinceWindowsEpoch(
+      base::TimeDelta::FromMicroseconds(GetInt64(path)));
+}
+
+void PrefService::SetTimeDelta(const std::string& path, base::TimeDelta value) {
+  SetInt64(path, value.InMicroseconds());
+}
+
+base::TimeDelta PrefService::GetTimeDelta(const std::string& path) const {
+  return base::TimeDelta::FromMicroseconds(GetInt64(path));
+}
+
 base::Value* PrefService::GetMutableUserPref(const std::string& path,
                                              base::Value::Type type) {
   CHECK(type == base::Value::Type::DICTIONARY ||
@@ -456,28 +559,26 @@ base::Value* PrefService::GetMutableUserPref(const std::string& path,
   const Preference* pref = FindPreference(path);
   if (!pref) {
     NOTREACHED() << "Trying to get an unregistered pref: " << path;
-    return NULL;
+    return nullptr;
   }
   if (pref->GetType() != type) {
     NOTREACHED() << "Wrong type for GetMutableValue: " << path;
-    return NULL;
+    return nullptr;
   }
 
   // Look for an existing preference in the user store. If it doesn't
-  // exist or isn't the correct type, create a new user preference.
-  base::Value* value = NULL;
-  if (!user_pref_store_->GetMutableValue(path, &value) ||
-      !value->IsType(type)) {
-    if (type == base::Value::Type::DICTIONARY) {
-      value = new base::DictionaryValue;
-    } else if (type == base::Value::Type::LIST) {
-      value = new base::ListValue;
-    } else {
-      NOTREACHED();
-    }
-    user_pref_store_->SetValueSilently(path, base::WrapUnique(value),
-                                       GetWriteFlags(pref));
-  }
+  // exist, create a new user preference.
+  base::Value* value = nullptr;
+  if (user_pref_store_->GetMutableValue(path, &value))
+    return value;
+
+  // If no user preference exists, clone default value.
+  const base::Value* default_value = nullptr;
+  pref_registry_->defaults()->GetValue(path, &default_value);
+  DCHECK_EQ(default_value->type(), type);
+  user_pref_store_->SetValueSilently(path, default_value->CreateDeepCopy(),
+                                     GetWriteFlags(pref));
+  user_pref_store_->GetMutableValue(path, &value);
   return value;
 }
 
@@ -503,10 +604,10 @@ void PrefService::SetUserPrefValue(const std::string& path,
     NOTREACHED() << "Trying to write an unregistered pref: " << path;
     return;
   }
-  if (pref->GetType() != new_value->GetType()) {
-    NOTREACHED() << "Trying to set pref " << path
-                 << " of type " << pref->GetType()
-                 << " to value of type " << new_value->GetType();
+  if (pref->GetType() != new_value->type()) {
+    NOTREACHED() << "Trying to set pref " << path << " of type "
+                 << pref->GetType() << " to value of type "
+                 << new_value->type();
     return;
   }
 
@@ -521,25 +622,17 @@ void PrefService::UpdateCommandLinePrefStore(PrefStore* command_line_store) {
 // PrefService::Preference
 
 PrefService::Preference::Preference(const PrefService* service,
-                                    const std::string& name,
+                                    std::string name,
                                     base::Value::Type type)
-    : name_(name), type_(type), pref_service_(service) {
-  DCHECK(service);
-  // Cache the registration flags at creation time to avoid multiple map lookups
-  // later.
-  registration_flags_ = service->pref_registry_->GetRegistrationFlags(name_);
-}
-
-const std::string PrefService::Preference::name() const {
-  return name_;
-}
-
-base::Value::Type PrefService::Preference::GetType() const {
-  return type_;
-}
+    : name_(std::move(name)),
+      type_(type),
+      // Cache the registration flags at creation time to avoid multiple map
+      // lookups later.
+      registration_flags_(service->pref_registry_->GetRegistrationFlags(name_)),
+      pref_service_(service) {}
 
 const base::Value* PrefService::Preference::GetValue() const {
-  const base::Value* result= pref_service_->GetPreferenceValue(name_);
+  const base::Value* result = pref_service_->GetPreferenceValue(name_);
   DCHECK(result) << "Must register pref before getting its value";
   return result;
 }
@@ -548,14 +641,14 @@ const base::Value* PrefService::Preference::GetRecommendedValue() const {
   DCHECK(pref_service_->FindPreference(name_))
       << "Must register pref before getting its value";
 
-  const base::Value* found_value = NULL;
+  const base::Value* found_value = nullptr;
   if (pref_value_store()->GetRecommendedValue(name_, type_, &found_value)) {
-    DCHECK(found_value->IsType(type_));
+    DCHECK(found_value->type() == type_);
     return found_value;
   }
 
   // The pref has no recommended value.
-  return NULL;
+  return nullptr;
 }
 
 bool PrefService::Preference::IsManaged() const {
@@ -609,12 +702,12 @@ const base::Value* PrefService::GetPreferenceValue(
   CHECK(pref_registry_->defaults());
   CHECK(pref_value_store_);
 
-  const base::Value* default_value = NULL;
+  const base::Value* default_value = nullptr;
   if (pref_registry_->defaults()->GetValue(path, &default_value)) {
-    const base::Value* found_value = NULL;
-    base::Value::Type default_type = default_value->GetType();
+    const base::Value* found_value = nullptr;
+    base::Value::Type default_type = default_value->type();
     if (pref_value_store_->GetValue(path, default_type, &found_value)) {
-      DCHECK(found_value->IsType(default_type));
+      DCHECK(found_value->type() == default_type);
       return found_value;
     } else {
       // Every registered preference has at least a default value.
@@ -622,5 +715,5 @@ const base::Value* PrefService::GetPreferenceValue(
     }
   }
 
-  return NULL;
+  return nullptr;
 }
