@@ -8,6 +8,7 @@
 
 #include <memory>
 #include <set>
+#include <tuple>
 
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
@@ -62,15 +63,18 @@ const std::string ColumnsForVersion(int version, bool concatenated) {
     columns.push_back("logo_id");
   }
   columns.push_back("created_by_policy");
-  columns.push_back("instant_url");
+  if (version <= 75) {
+    // Column removed after version 75.
+    columns.push_back("instant_url");
+  }
   columns.push_back("last_modified");
   columns.push_back("sync_guid");
   if (version >= 47) {
     // Column added in version 47.
     columns.push_back("alternate_urls");
   }
-  if (version >= 49) {
-    // Column added in version 49.
+  if (version >= 49 && version <= 75) {
+    // Column added in version 49 and removed after version 75.
     columns.push_back("search_terms_replacement_key");
   }
   if (version >= 52) {
@@ -78,7 +82,13 @@ const std::string ColumnsForVersion(int version, bool concatenated) {
     columns.push_back("image_url");
     columns.push_back("search_url_post_params");
     columns.push_back("suggest_url_post_params");
+  }
+  if (version >= 52 && version <= 75) {
+    // Column added in version 52 and removed after version 75.
     columns.push_back("instant_url_post_params");
+  }
+  if (version >= 52) {
+    // Column added in version 52.
     columns.push_back("image_url_post_params");
   }
   if (version >= 53) {
@@ -93,7 +103,6 @@ const std::string ColumnsForVersion(int version, bool concatenated) {
   return base::JoinString(columns, std::string(concatenated ? " || " : ", "));
 }
 
-
 // Inserts the data from |data| into |s|.  |s| is assumed to have slots for all
 // the columns in the keyword table.  |id_column| is the slot number to bind
 // |data|'s |id| to; |starting_column| is the slot number of the first of a
@@ -123,25 +132,25 @@ void BindURLToStatement(const TemplateURLData& data,
   s->BindString(starting_column + 5, data.originating_url.is_valid() ?
       history::URLDatabase::GURLToDatabaseURL(data.originating_url) :
       std::string());
-  s->BindInt64(starting_column + 6, data.date_created.ToTimeT());
+  s->BindInt64(starting_column + 6,
+               data.date_created.since_origin().InMicroseconds());
   s->BindInt(starting_column + 7, data.usage_count);
   s->BindString(starting_column + 8,
                 base::JoinString(data.input_encodings, ";"));
   s->BindString(starting_column + 9, data.suggestions_url);
   s->BindInt(starting_column + 10, data.prepopulate_id);
   s->BindBool(starting_column + 11, data.created_by_policy);
-  s->BindString(starting_column + 12, data.instant_url);
-  s->BindInt64(starting_column + 13, data.last_modified.ToTimeT());
-  s->BindString(starting_column + 14, data.sync_guid);
-  s->BindString(starting_column + 15, alternate_urls);
-  s->BindString(starting_column + 16, data.search_terms_replacement_key);
-  s->BindString(starting_column + 17, data.image_url);
-  s->BindString(starting_column + 18, data.search_url_post_params);
-  s->BindString(starting_column + 19, data.suggestions_url_post_params);
-  s->BindString(starting_column + 20, data.instant_url_post_params);
-  s->BindString(starting_column + 21, data.image_url_post_params);
-  s->BindString(starting_column + 22, data.new_tab_url);
-  s->BindInt64(starting_column + 23, data.last_visited.ToTimeT());
+  s->BindInt64(starting_column + 12,
+               data.last_modified.since_origin().InMicroseconds());
+  s->BindString(starting_column + 13, data.sync_guid);
+  s->BindString(starting_column + 14, alternate_urls);
+  s->BindString(starting_column + 15, data.image_url);
+  s->BindString(starting_column + 16, data.search_url_post_params);
+  s->BindString(starting_column + 17, data.suggestions_url_post_params);
+  s->BindString(starting_column + 18, data.image_url_post_params);
+  s->BindString(starting_column + 19, data.new_tab_url);
+  s->BindInt64(starting_column + 20,
+               data.last_visited.since_origin().InMicroseconds());
 }
 
 WebDatabaseTable::TypeKey GetKey() {
@@ -168,32 +177,30 @@ WebDatabaseTable::TypeKey KeywordTable::GetTypeKey() const {
 
 bool KeywordTable::CreateTablesIfNecessary() {
   return db_->DoesTableExist("keywords") ||
-      db_->Execute("CREATE TABLE keywords ("
-          "id INTEGER PRIMARY KEY,"
-          "short_name VARCHAR NOT NULL,"
-          "keyword VARCHAR NOT NULL,"
-          "favicon_url VARCHAR NOT NULL,"
-          "url VARCHAR NOT NULL,"
-          "safe_for_autoreplace INTEGER,"
-          "originating_url VARCHAR,"
-          "date_created INTEGER DEFAULT 0,"
-          "usage_count INTEGER DEFAULT 0,"
-          "input_encodings VARCHAR,"
-          "suggest_url VARCHAR,"
-          "prepopulate_id INTEGER DEFAULT 0,"
-          "created_by_policy INTEGER DEFAULT 0,"
-          "instant_url VARCHAR,"
-          "last_modified INTEGER DEFAULT 0,"
-          "sync_guid VARCHAR,"
-          "alternate_urls VARCHAR,"
-          "search_terms_replacement_key VARCHAR,"
-          "image_url VARCHAR,"
-          "search_url_post_params VARCHAR,"
-          "suggest_url_post_params VARCHAR,"
-          "instant_url_post_params VARCHAR,"
-          "image_url_post_params VARCHAR,"
-          "new_tab_url VARCHAR,"
-          " last_visited INTEGER DEFAULT 0)");
+         db_->Execute(
+             "CREATE TABLE keywords ("
+             "id INTEGER PRIMARY KEY,"
+             "short_name VARCHAR NOT NULL,"
+             "keyword VARCHAR NOT NULL,"
+             "favicon_url VARCHAR NOT NULL,"
+             "url VARCHAR NOT NULL,"
+             "safe_for_autoreplace INTEGER,"
+             "originating_url VARCHAR,"
+             "date_created INTEGER DEFAULT 0,"
+             "usage_count INTEGER DEFAULT 0,"
+             "input_encodings VARCHAR,"
+             "suggest_url VARCHAR,"
+             "prepopulate_id INTEGER DEFAULT 0,"
+             "created_by_policy INTEGER DEFAULT 0,"
+             "last_modified INTEGER DEFAULT 0,"
+             "sync_guid VARCHAR,"
+             "alternate_urls VARCHAR,"
+             "image_url VARCHAR,"
+             "search_url_post_params VARCHAR,"
+             "suggest_url_post_params VARCHAR,"
+             "image_url_post_params VARCHAR,"
+             "new_tab_url VARCHAR,"
+             "last_visited INTEGER DEFAULT 0)");
 }
 
 bool KeywordTable::IsSyncable() {
@@ -215,6 +222,12 @@ bool KeywordTable::MigrateToVersion(int version,
       return MigrateToVersion68RemoveShowInDefaultListColumn();
     case 69:
       return MigrateToVersion69AddLastVisitedColumn();
+    case 76:
+      *update_compatible_version = true;
+      return MigrateToVersion76RemoveInstantColumns();
+    case 77:
+      *update_compatible_version = true;
+      return MigrateToVersion77IncreaseTimePrecision();
   }
 
   return true;
@@ -225,8 +238,7 @@ bool KeywordTable::PerformOperations(const Operations& operations) {
   if (!transaction.Begin())
     return false;
 
-  for (Operations::const_iterator i(operations.begin()); i != operations.end();
-       ++i) {
+  for (auto i(operations.begin()); i != operations.end(); ++i) {
     switch (i->first) {
       case ADD:
         if (!AddKeyword(i->second))
@@ -262,8 +274,7 @@ bool KeywordTable::GetKeywords(Keywords* keywords) {
     }
   }
   bool succeeded = s.Succeeded();
-  for (std::set<TemplateURLID>::const_iterator i(bad_entries.begin());
-       i != bad_entries.end(); ++i)
+  for (auto i(bad_entries.begin()); i != bad_entries.end(); ++i)
     succeeded &= RemoveKeyword(*i);
   return succeeded;
 }
@@ -349,6 +360,82 @@ bool KeywordTable::MigrateToVersion69AddLastVisitedColumn() {
                       "INTEGER DEFAULT 0");
 }
 
+// SQLite does not support DROP COLUMN operation. So a new table is created
+// without the removed columns. Data from all but the dropped columns of the old
+// table is copied into it. After that, the old table is dropped and the new
+// table is renamed to it.
+bool KeywordTable::MigrateToVersion76RemoveInstantColumns() {
+  sql::Transaction transaction(db_);
+  std::string query_str =
+      std::string("INSERT INTO temp_keywords SELECT " +
+                  ColumnsForVersion(76, false) + " FROM keywords");
+  const char* clone_query = query_str.c_str();
+  return transaction.Begin() &&
+         db_->Execute(
+             "CREATE TABLE temp_keywords ("
+             "id INTEGER PRIMARY KEY,"
+             "short_name VARCHAR NOT NULL,"
+             "keyword VARCHAR NOT NULL,"
+             "favicon_url VARCHAR NOT NULL,"
+             "url VARCHAR NOT NULL,"
+             "safe_for_autoreplace INTEGER,"
+             "originating_url VARCHAR,"
+             "date_created INTEGER DEFAULT 0,"
+             "usage_count INTEGER DEFAULT 0,"
+             "input_encodings VARCHAR,"
+             "suggest_url VARCHAR,"
+             "prepopulate_id INTEGER DEFAULT 0,"
+             "created_by_policy INTEGER DEFAULT 0,"
+             "last_modified INTEGER DEFAULT 0,"
+             "sync_guid VARCHAR,"
+             "alternate_urls VARCHAR,"
+             "image_url VARCHAR,"
+             "search_url_post_params VARCHAR,"
+             "suggest_url_post_params VARCHAR,"
+             "image_url_post_params VARCHAR,"
+             "new_tab_url VARCHAR,"
+             "last_visited INTEGER DEFAULT 0)") &&
+         db_->Execute(clone_query) && db_->Execute("DROP TABLE keywords") &&
+         db_->Execute("ALTER TABLE temp_keywords RENAME TO keywords") &&
+         transaction.Commit();
+}
+
+bool KeywordTable::MigrateToVersion77IncreaseTimePrecision() {
+  sql::Transaction transaction(db_);
+  if (!transaction.Begin())
+    return false;
+
+  std::string query(
+      "SELECT id, date_created, last_modified, last_visited FROM keywords");
+  sql::Statement s(db_->GetUniqueStatement(query.c_str()));
+  std::vector<std::tuple<TemplateURLID, Time, Time, Time>> updates;
+  while (s.Step()) {
+    updates.push_back(std::make_tuple(
+        s.ColumnInt64(0), Time::FromTimeT(s.ColumnInt64(1)),
+        Time::FromTimeT(s.ColumnInt64(2)), Time::FromTimeT(s.ColumnInt64(3))));
+  }
+  if (!s.Succeeded())
+    return false;
+
+  for (auto tuple : updates) {
+    sql::Statement update_statement(db_->GetCachedStatement(
+        SQL_FROM_HERE,
+        "UPDATE keywords SET date_created = ?, last_modified = ?, last_visited "
+        "= ? WHERE id = ? "));
+    update_statement.BindInt64(
+        0, std::get<1>(tuple).since_origin().InMicroseconds());
+    update_statement.BindInt64(
+        1, std::get<2>(tuple).since_origin().InMicroseconds());
+    update_statement.BindInt64(
+        2, std::get<3>(tuple).since_origin().InMicroseconds());
+    update_statement.BindInt64(3, std::get<0>(tuple));
+    if (!update_statement.Run()) {
+      return false;
+    }
+  }
+  return transaction.Commit();
+}
+
 // static
 bool KeywordTable::GetKeywordDataFromStatement(const sql::Statement& s,
                                                TemplateURLData* data) {
@@ -364,30 +451,30 @@ bool KeywordTable::GetKeywordDataFromStatement(const sql::Statement& s,
     return false;
   data->SetURL(s.ColumnString(4));
   data->suggestions_url = s.ColumnString(10);
-  data->instant_url = s.ColumnString(13);
-  data->image_url = s.ColumnString(18);
-  data->new_tab_url = s.ColumnString(23);
-  data->search_url_post_params = s.ColumnString(19);
-  data->suggestions_url_post_params = s.ColumnString(20);
-  data->instant_url_post_params = s.ColumnString(21);
-  data->image_url_post_params = s.ColumnString(22);
+  data->image_url = s.ColumnString(16);
+  data->new_tab_url = s.ColumnString(20);
+  data->search_url_post_params = s.ColumnString(17);
+  data->suggestions_url_post_params = s.ColumnString(18);
+  data->image_url_post_params = s.ColumnString(19);
   data->favicon_url = GURL(s.ColumnString(3));
   data->originating_url = GURL(s.ColumnString(6));
   data->safe_for_autoreplace = s.ColumnBool(5);
   data->input_encodings = base::SplitString(
       s.ColumnString(9), ";", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
   data->id = s.ColumnInt64(0);
-  data->date_created = Time::FromTimeT(s.ColumnInt64(7));
-  data->last_modified = Time::FromTimeT(s.ColumnInt64(14));
+  data->date_created =
+      base::Time() + base::TimeDelta::FromMicroseconds(s.ColumnInt64(7));
+  data->last_modified =
+      base::Time() + base::TimeDelta::FromMicroseconds(s.ColumnInt64(13));
   data->created_by_policy = s.ColumnBool(12);
   data->usage_count = s.ColumnInt(8);
   data->prepopulate_id = s.ColumnInt(11);
-  data->sync_guid = s.ColumnString(15);
+  data->sync_guid = s.ColumnString(14);
 
   data->alternate_urls.clear();
   base::JSONReader json_reader;
   std::unique_ptr<base::Value> value(
-      json_reader.ReadToValue(s.ColumnString(16)));
+      json_reader.ReadToValue(s.ColumnString(15)));
   base::ListValue* alternate_urls_value;
   if (value.get() && value->GetAsList(&alternate_urls_value)) {
     std::string alternate_url;
@@ -397,17 +484,17 @@ bool KeywordTable::GetKeywordDataFromStatement(const sql::Statement& s,
     }
   }
 
-  data->search_terms_replacement_key = s.ColumnString(17);
-  data->last_visited = Time::FromTimeT(s.ColumnInt64(24));
+  data->last_visited =
+      base::Time() + base::TimeDelta::FromMicroseconds(s.ColumnInt64(21));
 
   return true;
 }
 
 bool KeywordTable::AddKeyword(const TemplateURLData& data) {
   DCHECK(data.id);
-  std::string query("INSERT INTO keywords (" + GetKeywordColumns() + ") "
-                    "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,"
-                    "        ?,?)");
+  std::string query("INSERT INTO keywords (" + GetKeywordColumns() +
+                    ") "
+                    "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
   sql::Statement s(db_->GetCachedStatement(SQL_FROM_HERE, query.c_str()));
   BindURLToStatement(data, &s, 0, 1);
 
@@ -429,13 +516,11 @@ bool KeywordTable::UpdateKeyword(const TemplateURLData& data) {
       SQL_FROM_HERE,
       "UPDATE keywords SET short_name=?, keyword=?, favicon_url=?, url=?, "
       "safe_for_autoreplace=?, originating_url=?, date_created=?, "
-      "usage_count=?, input_encodings=?, suggest_url=?, "
-      "prepopulate_id=?, created_by_policy=?, instant_url=?, "
-      "last_modified=?, sync_guid=?, alternate_urls=?, "
-      "search_terms_replacement_key=?, image_url=?, search_url_post_params=?, "
-      "suggest_url_post_params=?, instant_url_post_params=?, "
+      "usage_count=?, input_encodings=?, suggest_url=?, prepopulate_id=?, "
+      "created_by_policy=?, last_modified=?, sync_guid=?, alternate_urls=?, "
+      "image_url=?, search_url_post_params=?, suggest_url_post_params=?, "
       "image_url_post_params=?, new_tab_url=?, last_visited=? WHERE id=?"));
-  BindURLToStatement(data, &s, 24, 0);  // "24" binds id() as the last item.
+  BindURLToStatement(data, &s, 21, 0);  // "21" binds id() as the last item.
 
   return s.Run();
 }
