@@ -7,7 +7,6 @@
 
 #include <stddef.h>
 
-#include <list>
 #include <map>
 #include <memory>
 #include <set>
@@ -18,24 +17,23 @@
 #include "base/callback_list.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
+#include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
-#include "base/time/clock.h"
+#include "base/time/default_clock.h"
 #include "components/google/core/browser/google_url_tracker.h"
 #include "components/keyed_service/core/keyed_service.h"
 #include "components/prefs/pref_change_registrar.h"
 #include "components/search_engines/default_search_manager.h"
 #include "components/search_engines/keyword_web_data_service.h"
+#include "components/search_engines/search_host_to_urls_map.h"
+#include "components/search_engines/search_terms_data.h"
 #include "components/search_engines/template_url.h"
-#include "components/search_engines/template_url_id.h"
 #include "components/sync/model/sync_change.h"
 #include "components/sync/model/syncable_service.h"
 #include "components/webdata/common/web_data_service_consumer.h"
 
 class GURL;
 class PrefService;
-class SearchHostToURLsMap;
-class SearchTermsData;
-class TemplateURL;
 class TemplateURLServiceClient;
 class TemplateURLServiceObserver;
 struct TemplateURLData;
@@ -77,8 +75,8 @@ class TemplateURLService : public WebDataServiceConsumer,
                            public syncer::SyncableService {
  public:
   using QueryTerms = std::map<std::string, std::string>;
-  using TemplateURLVector = std::vector<TemplateURL*>;
-  using OwnedTemplateURLVector = std::vector<std::unique_ptr<TemplateURL>>;
+  using TemplateURLVector = TemplateURL::TemplateURLVector;
+  using OwnedTemplateURLVector = TemplateURL::OwnedTemplateURLVector;
   using SyncDataMap = std::map<std::string, syncer::SyncData>;
   using Subscription = base::CallbackList<void(void)>::Subscription;
 
@@ -110,7 +108,7 @@ class TemplateURLService : public WebDataServiceConsumer,
       std::unique_ptr<TemplateURLServiceClient> client,
       GoogleURLTracker* google_url_tracker,
       rappor::RapporServiceImpl* rappor_service,
-      const base::Closure& dsp_change_callback);
+      const base::RepeatingClosure& dsp_change_callback);
   // The following is for testing.
   TemplateURLService(const Initializer* initializers, const int count);
   ~TemplateURLService() override;
@@ -118,10 +116,6 @@ class TemplateURLService : public WebDataServiceConsumer,
   // Register Profile preferences in |registry|.
   static void RegisterProfilePrefs(user_prefs::PrefRegistrySyncable* registry);
 
-  // Removes any unnecessary characters from a user input keyword.
-  // This removes the leading scheme, "www." and any trailing slash.
-  static base::string16 CleanUserInputKeyword(const base::string16& keyword);
-
   // Returns true if there is no TemplateURL that conflicts with the
   // keyword/url pair, or there is one but it can be replaced. If there is an
   // existing keyword that can be replaced and template_url_to_replace is
@@ -131,7 +125,7 @@ class TemplateURLService : public WebDataServiceConsumer,
   // a keyword for hosts already associated with a manually-edited keyword.
   bool CanAddAutogeneratedKeyword(const base::string16& keyword,
                                   const GURL& url,
-                                  TemplateURL** template_url_to_replace);
+                                  const TemplateURL** template_url_to_replace);
 
   // Returns whether the engine is a "pre-existing" engine, either from the
   // prepopulate list or created by policy.
@@ -181,9 +175,16 @@ class TemplateURLService : public WebDataServiceConsumer,
   TemplateURL* GetTemplateURLForHost(const std::string& host);
   const TemplateURL* GetTemplateURLForHost(const std::string& host) const;
 
-  // Adds |template_url| to this model.  Returns a raw pointer to |template_url|
-  // if the addition succeeded, or null on failure.  (Many callers need still
-  // need a raw pointer to the TemplateURL so they can access it later.)
+  // Adds a new TemplateURL to this model.
+  //
+  // This function guarantees that on return the model will not have two non-
+  // extension TemplateURLs with the same keyword.  If that means that it cannot
+  // add the provided argument, it will return null.  Otherwise it will return
+  // the raw pointer to the TemplateURL.
+  //
+  // Returns a raw pointer to |template_url| if the addition succeeded, or null
+  // on failure.  (Many callers need still need a raw pointer to the TemplateURL
+  // so they can access it later.)
   TemplateURL* Add(std::unique_ptr<TemplateURL> template_url);
 
   // Like Add(), but overwrites the |template_url|'s values with the provided
@@ -195,7 +196,7 @@ class TemplateURLService : public WebDataServiceConsumer,
 
   // Removes the keyword from the model. This deletes the supplied TemplateURL.
   // This fails if the supplied template_url is the default search provider.
-  void Remove(TemplateURL* template_url);
+  void Remove(const TemplateURL* template_url);
 
   // Removes any TemplateURL of the specified |type| associated with
   // |extension_id|. Unlike with Remove(), this can be called when the
@@ -244,6 +245,11 @@ class TemplateURLService : public WebDataServiceConsumer,
                         const base::string16& keyword,
                         const std::string& search_url);
 
+  // Updates any search providers matching |potential_search_url| with the new
+  // favicon location |favicon_url|.
+  void UpdateProviderFavicons(const GURL& potential_search_url,
+                              const GURL& favicon_url);
+
   // Return true if the given |url| can be made the default. This returns false
   // regardless of |url| if the default search provider is managed by policy or
   // controlled by an extension.
@@ -306,7 +312,7 @@ class TemplateURLService : public WebDataServiceConsumer,
   //
   // If the service has already loaded, this function does nothing.
   std::unique_ptr<Subscription> RegisterOnLoadedCallback(
-      const base::Closure& callback);
+      const base::RepeatingClosure& callback);
 
 #if defined(UNIT_TEST)
   void set_loaded(bool value) { loaded_ = value; }
@@ -347,7 +353,7 @@ class TemplateURLService : public WebDataServiceConsumer,
   // data. This may send notifications if local search engines are added,
   // updated or removed.
   syncer::SyncError ProcessSyncChanges(
-      const tracked_objects::Location& from_here,
+      const base::Location& from_here,
       const syncer::SyncChangeList& change_list) override;
   // Merge initial search engine data from Sync and push any local changes up
   // to Sync. This may send notifications if local search engines are added,
@@ -364,7 +370,7 @@ class TemplateURLService : public WebDataServiceConsumer,
   // This may send a new SyncChange to the cloud. If our model has not yet been
   // associated with Sync, or if this is triggered by a Sync change, then this
   // does nothing.
-  void ProcessTemplateURLChange(const tracked_objects::Location& from_here,
+  void ProcessTemplateURLChange(const base::Location& from_here,
                                 const TemplateURL* turl,
                                 syncer::SyncChange::SyncChangeType type);
 
@@ -427,9 +433,10 @@ class TemplateURLService : public WebDataServiceConsumer,
                            ResolveSyncKeywordConflict);
   FRIEND_TEST_ALL_PREFIXES(TemplateURLServiceSyncTest, PreSyncDeletes);
   FRIEND_TEST_ALL_PREFIXES(TemplateURLServiceSyncTest, MergeInSyncTemplateURL);
-  FRIEND_TEST_ALL_PREFIXES(ToolbarModelTest, GoogleBaseURL);
+  FRIEND_TEST_ALL_PREFIXES(LocationBarModelTest, GoogleBaseURL);
 
   friend class InstantUnitTestBase;
+  friend class Scoper;
   friend class TemplateURLServiceTestUtil;
   friend class TemplateUrlServiceAndroid;
 
@@ -483,6 +490,10 @@ class TemplateURLService : public WebDataServiceConsumer,
   // keywords which begin with a prefix.
   class LessWithPrefix;
 
+  // Used to defer notifications until the last Scoper is destroyed by leaving
+  // the scope of a code block.
+  class Scoper;
+
   void Init(const Initializer* initializers, int num_initializers);
 
   // Given two engines with the same keyword, returns which should take
@@ -497,7 +508,7 @@ class TemplateURLService : public WebDataServiceConsumer,
   // Removes |template_url| from various internal maps
   // (|keyword_to_turl_and_length_|, |keyword_domain_to_turl_and_length_|,
   // |guid_to_turl_|, |provider_map_|).
-  void RemoveFromMaps(TemplateURL* template_url);
+  void RemoveFromMaps(const TemplateURL* template_url);
 
   // Adds |template_url| to various internal maps
   // (|keyword_to_turl_and_length_|, |keyword_domain_to_turl_and_length_|,
@@ -525,11 +536,6 @@ class TemplateURLService : public WebDataServiceConsumer,
   // Transitions to the loaded state.
   void ChangeToLoadedState();
 
-  // Called by DefaultSearchManager when the effective default search engine has
-  // changed.
-  void OnDefaultSearchChange(const TemplateURLData* new_dse_data,
-                             DefaultSearchManager::Source source);
-
   // Applies a DSE change and reports metrics if appropriate.
   void ApplyDefaultSearchChange(const TemplateURLData* new_dse_data,
                                 DefaultSearchManager::Source source);
@@ -555,17 +561,11 @@ class TemplateURLService : public WebDataServiceConsumer,
       const base::string16& keyword);
 
   // Updates the information in |existing_turl| using the information from
-  // |new_values|, but the ID for |existing_turl| is retained.  Notifying
-  // observers is the responsibility of the caller.  Returns whether
+  // |new_values|, but the ID for |existing_turl| is retained. Returns whether
   // |existing_turl| was found in |template_urls_| and thus could be updated.
   //
   // NOTE: This should not be called with an extension keyword as there are no
   // updates needed in that case.
-  bool UpdateNoNotify(TemplateURL* existing_turl,
-                      const TemplateURL& new_values);
-
-  // Calls UpdateNoNotify() and NotifyObservers() if update succeeds.
-  // Returns the result of UpdateNoNotify().
   bool Update(TemplateURL* existing_turl, const TemplateURL& new_values);
 
   // If the TemplateURL comes from a prepopulated URL available in the current
@@ -576,9 +576,9 @@ class TemplateURLService : public WebDataServiceConsumer,
                                               PrefService* prefs);
 
   // If the TemplateURL's sync GUID matches the kSyncedDefaultSearchProviderGUID
-  // preference it will be used to update the DSE in memory and as persisted in
-  // preferences.
-  void MaybeUpdateDSEAfterSync(TemplateURL* synced_turl);
+  // preference it will be used to update the DSE in prefs.
+  // OnDefaultSearchChange may be triggered as a result.
+  void MaybeUpdateDSEViaPrefs(TemplateURL* synced_turl);
 
   // Iterates through the TemplateURLs to see if one matches the visited url.
   // For each TemplateURL whose url matches the visited url
@@ -608,26 +608,13 @@ class TemplateURLService : public WebDataServiceConsumer,
   // This function guarantees that on return the model will not have two non-
   // extension TemplateURLs with the same keyword.  If that means that it cannot
   // add the provided argument, it will return null.  Otherwise it will return
-  // the raw pointer to the TemplateURL.  The caller is responsible for
-  // notifying observers if this function succeeds.
-  TemplateURL* AddNoNotify(std::unique_ptr<TemplateURL> template_url,
-                           bool newly_adding);
-
-  // Removes the keyword from the model. This deletes the supplied TemplateURL.
-  // This fails if the supplied template_url is the default search provider.
-  // Caller is responsible for notifying observers.
-  void RemoveNoNotify(TemplateURL* template_url);
-
-  // Like ResetTemplateURL(), but instead of notifying observers, returns
-  // whether anything has changed.
-  bool ResetTemplateURLNoNotify(TemplateURL* url,
-                                const base::string16& title,
-                                const base::string16& keyword,
-                                const std::string& search_url);
-
-  // Notify the observers that the model has changed.  This is done only if the
-  // model is loaded.
-  void NotifyObservers();
+  // the raw pointer to the TemplateURL.
+  //
+  // Returns a raw pointer to |template_url| if the addition succeeded, or null
+  // on failure.  (Many callers need still need a raw pointer to the TemplateURL
+  // so they can access it later.)
+  TemplateURL* Add(std::unique_ptr<TemplateURL> template_url,
+                   bool newly_adding);
 
   // Updates |template_urls| so that the only "created by policy" entry is
   // |default_from_prefs|. |default_from_prefs| may be NULL if there is no
@@ -730,24 +717,24 @@ class TemplateURLService : public WebDataServiceConsumer,
   bool HasDuplicateKeywords() const;
 
   // ---------- Browser state related members ---------------------------------
-  PrefService* prefs_;
+  PrefService* prefs_ = nullptr;
 
-  std::unique_ptr<SearchTermsData> search_terms_data_;
+  std::unique_ptr<SearchTermsData> search_terms_data_ =
+      std::make_unique<SearchTermsData>();
 
   // ---------- Dependencies on other components ------------------------------
   // Service used to store entries.
-  scoped_refptr<KeywordWebDataService> web_data_service_;
+  scoped_refptr<KeywordWebDataService> web_data_service_ = nullptr;
 
   std::unique_ptr<TemplateURLServiceClient> client_;
 
-  GoogleURLTracker* google_url_tracker_;
+  GoogleURLTracker* google_url_tracker_ = nullptr;
 
   // ---------- Metrics related members ---------------------------------------
-  rappor::RapporServiceImpl* rappor_service_;
+  rappor::RapporServiceImpl* rappor_service_ = nullptr;
 
   // This closure is run when the default search provider is set to Google.
-  base::Closure dsp_change_callback_;
-
+  base::RepeatingClosure dsp_change_callback_;
 
   PrefChangeRegistrar pref_change_registrar_;
 
@@ -770,26 +757,25 @@ class TemplateURLService : public WebDataServiceConsumer,
 
   OwnedTemplateURLVector template_urls_;
 
-  base::ObserverList<TemplateURLServiceObserver> model_observers_;
+  base::ObserverList<TemplateURLServiceObserver>::Unchecked model_observers_;
 
   // Maps from host to set of TemplateURLs whose search url host is host.
-  // NOTE: This is always non-NULL; we use a std::unique_ptr<> to avoid circular
-  // header dependencies.
-  std::unique_ptr<SearchHostToURLsMap> provider_map_;
+  std::unique_ptr<SearchHostToURLsMap> provider_map_ =
+      std::make_unique<SearchHostToURLsMap>();
 
   // Whether the keywords have been loaded.
-  bool loaded_;
+  bool loaded_ = false;
 
   // Set when the web data service fails to load properly.  This prevents
   // further communication with sync or writing to prefs, so we don't persist
   // inconsistent state data anywhere.
-  bool load_failed_;
+  bool load_failed_ = false;
 
   // Whether Load() is disabled. True only in testing contexts.
-  bool disable_load_;
+  bool disable_load_ = false;
 
   // If non-zero, we're waiting on a load.
-  KeywordWebDataService::Handle load_handle_;
+  KeywordWebDataService::Handle load_handle_ = 0;
 
   // All visits that occurred before we finished loading. Once loaded
   // UpdateKeywordSearchTermsForURL is invoked for each element of the vector.
@@ -797,7 +783,7 @@ class TemplateURLService : public WebDataServiceConsumer,
 
   // Once loaded, the default search provider.  This is a pointer to a
   // TemplateURL owned by |template_urls_|.
-  TemplateURL* default_search_provider_;
+  TemplateURL* default_search_provider_ = nullptr;
 
   // A temporary location for the DSE until Web Data has been loaded and it can
   // be merged into |template_urls_|.
@@ -808,20 +794,20 @@ class TemplateURLService : public WebDataServiceConsumer,
 
   // ID assigned to next TemplateURL added to this model. This is an ever
   // increasing integer that is initialized from the database.
-  TemplateURLID next_id_;
+  TemplateURLID next_id_ = kInvalidTemplateURLID + 1;
 
   // Used to retrieve the current time, in base::Time units.
-  std::unique_ptr<base::Clock> clock_;
+  std::unique_ptr<base::Clock> clock_ = std::make_unique<base::DefaultClock>();
 
   // Do we have an active association between the TemplateURLs and sync models?
   // Set in MergeDataAndStartSyncing, reset in StopSyncing. While this is not
   // set, we ignore any local search engine changes (when we start syncing we
   // will look up the most recent values anyways).
-  bool models_associated_;
+  bool models_associated_ = false;
 
   // Whether we're currently processing changes from the syncer. While this is
   // true, we ignore any local search engine changes, since we triggered them.
-  bool processing_syncer_changes_;
+  bool processing_syncer_changes_ = false;
 
   // Sync's syncer::SyncChange handler. We push all our changes through this.
   std::unique_ptr<syncer::SyncChangeProcessor> sync_processor_;
@@ -839,7 +825,7 @@ class TemplateURLService : public WebDataServiceConsumer,
   // This is used to log the origin of changes to the default search provider.
   // We set this value to increasingly specific values when we know what is the
   // cause/origin of a default search change.
-  DefaultSearchChangeOrigin dsp_change_origin_;
+  DefaultSearchChangeOrigin dsp_change_origin_ = DSP_CHANGE_OTHER;
 
   // Stores a list of callbacks to be run after TemplateURLService has loaded.
   base::CallbackList<void(void)> on_loaded_callbacks_;
@@ -850,6 +836,16 @@ class TemplateURLService : public WebDataServiceConsumer,
   std::unique_ptr<GoogleURLTracker::Subscription>
       google_url_updated_subscription_;
 
+  // This tracks how many Scoper handles exist. When the number of handles drops
+  // to zero, a notification is made to observers if
+  // |model_mutated_notification_pending_| is true.
+  int outstanding_scoper_handles_ = 0;
+
+  // Used to track if a notification is necessary due to the model being
+  // mutated. The outermost Scoper handles, can be used to defer notifications,
+  // but if no model mutation occurs, the deferred notification can be skipped.
+  bool model_mutated_notification_pending_ = false;
+
   DISALLOW_COPY_AND_ASSIGN(TemplateURLService);
 };
 
