@@ -7,19 +7,19 @@
 #include "base/base_paths.h"
 #include "base/command_line.h"
 #include "base/i18n/case_conversion.h"
-#include "base/macros.h"
+#include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "components/google/core/browser/google_util.h"
-#include "components/metrics/proto/omnibox_event.pb.h"
-#include "components/metrics/proto/omnibox_input_type.pb.h"
+#include "components/google/core/common/google_util.h"
 #include "components/search_engines/search_engines_switches.h"
 #include "components/search_engines/search_terms_data.h"
 #include "components/search_engines/template_url.h"
 #include "components/search_engines/testing_search_terms_data.h"
 #include "net/base/url_util.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/metrics_proto/omnibox_event.pb.h"
+#include "third_party/metrics_proto/omnibox_input_type.pb.h"
 
 using base::ASCIIToUTF16;
 
@@ -89,7 +89,7 @@ TEST_F(TemplateURLTest, URLRefTestSearchTerms) {
     { "http://en.wikipedia.org/{searchTerms}", ASCIIToUTF16("wiki/?"),
       "http://en.wikipedia.org/wiki/%3F" }
   };
-  for (size_t i = 0; i < arraysize(search_term_cases); ++i) {
+  for (size_t i = 0; i < base::size(search_term_cases); ++i) {
     const SearchTermsCase& value = search_term_cases[i];
     TemplateURLData data;
     data.SetURL(value.url);
@@ -221,9 +221,8 @@ TEST_F(TemplateURLTest, URLRefTestImageURLWithPOST) {
   const TemplateURLRef::PostParams& post_params =
       url.image_url_ref().post_params_;
   EXPECT_EQ(7U, post_params.size());
-  for (TemplateURLRef::PostParams::const_iterator i = post_params.begin();
-       i != post_params.end(); ++i) {
-    TemplateURLRef::Replacements::const_iterator j = replacements.begin();
+  for (auto i = post_params.begin(); i != post_params.end(); ++i) {
+    auto j = replacements.begin();
     for (; j != replacements.end(); ++j) {
       if (j->is_post_param && j->index ==
           static_cast<size_t>(i - post_params.begin())) {
@@ -263,6 +262,21 @@ TEST_F(TemplateURLTest, URLRefTestImageURLWithPOST) {
   }
 }
 
+TEST_F(TemplateURLTest, ImageURLWithGetShouldNotCrash) {
+  TemplateURLData data;
+  data.SetURL("http://foo/?q={searchTerms}&t={google:imageThumbnail}");
+  TemplateURL url(data);
+  EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
+  ASSERT_TRUE(url.url_ref().SupportsReplacement(search_terms_data_));
+
+  TemplateURLRef::SearchTermsArgs search_args(ASCIIToUTF16("X"));
+  search_args.image_thumbnail_content = "dummy-image-thumbnail";
+  GURL result(
+      url.url_ref().ReplaceSearchTerms(search_args, search_terms_data_));
+  ASSERT_TRUE(result.is_valid());
+  EXPECT_EQ("http://foo/?q=X&t=dummy-image-thumbnail", result.spec());
+}
+
 // Test that setting the prepopulate ID from TemplateURL causes the stored
 // TemplateURLRef to handle parsing the URL parameters differently.
 TEST_F(TemplateURLTest, SetPrepopulatedAndParse) {
@@ -271,16 +285,17 @@ TEST_F(TemplateURLTest, SetPrepopulatedAndParse) {
   TemplateURL url(data);
   TemplateURLRef::Replacements replacements;
   bool valid = false;
-  EXPECT_EQ("http://foo{fhqwhgads}bar", url.url_ref().ParseURL(
-      "http://foo{fhqwhgads}bar", &replacements, NULL, &valid));
+  EXPECT_EQ("http://foo{fhqwhgads}bar",
+            url.url_ref().ParseURL("http://foo{fhqwhgads}bar", &replacements,
+                                   nullptr, &valid));
   EXPECT_TRUE(replacements.empty());
   EXPECT_TRUE(valid);
 
   data.prepopulate_id = 123;
   TemplateURL url2(data);
-  EXPECT_EQ("http://foobar", url2.url_ref().ParseURL("http://foo{fhqwhgads}bar",
-                                                     &replacements, NULL,
-                                                     &valid));
+  EXPECT_EQ("http://foobar",
+            url2.url_ref().ParseURL("http://foo{fhqwhgads}bar", &replacements,
+                                    nullptr, &valid));
   EXPECT_TRUE(replacements.empty());
   EXPECT_TRUE(valid);
 }
@@ -291,7 +306,6 @@ TEST_F(TemplateURLTest, SetPrepopulatedAndReplace) {
   TemplateURLData data;
   data.SetURL("http://foo{fhqwhgads}search/?q={searchTerms}");
   data.suggestions_url = "http://foo{fhqwhgads}suggest/?q={searchTerms}";
-  data.instant_url = "http://foo{fhqwhgads}instant/";
   data.image_url = "http://foo{fhqwhgads}image/";
   data.new_tab_url = "http://foo{fhqwhgads}newtab/";
   data.contextual_search_url = "http://foo{fhqwhgads}context/";
@@ -310,8 +324,6 @@ TEST_F(TemplateURLTest, SetPrepopulatedAndReplace) {
             url.url_refs()[1].ReplaceSearchTerms(args, stdata));
   EXPECT_EQ("http://foo%7Bfhqwhgads%7Dsuggest/?q=X",
             url.suggestions_url_ref().ReplaceSearchTerms(args, stdata));
-  EXPECT_EQ("http://foo{fhqwhgads}instant/",
-            url.instant_url_ref().ReplaceSearchTerms(args, stdata));
   EXPECT_EQ("http://foo{fhqwhgads}image/",
             url.image_url_ref().ReplaceSearchTerms(args, stdata));
   EXPECT_EQ("http://foo{fhqwhgads}newtab/",
@@ -329,8 +341,6 @@ TEST_F(TemplateURLTest, SetPrepopulatedAndReplace) {
             url2.url_refs()[1].ReplaceSearchTerms(args, stdata));
   EXPECT_EQ("http://foosuggest/?q=X",
             url2.suggestions_url_ref().ReplaceSearchTerms(args, stdata));
-  EXPECT_EQ("http://fooinstant/",
-            url2.instant_url_ref().ReplaceSearchTerms(args, stdata));
   EXPECT_EQ("http://fooimage/",
             url2.image_url_ref().ReplaceSearchTerms(args, stdata));
   EXPECT_EQ("http://foonewtab/",
@@ -377,14 +387,15 @@ TEST_F(TemplateURLTest, URLRefTestSearchTermsUsingTermsData) {
 
   TestingSearchTermsData search_terms_data("http://example.com/e/");
   TemplateURLData data;
-  for (size_t i = 0; i < arraysize(search_term_cases); ++i) {
+  for (size_t i = 0; i < base::size(search_term_cases); ++i) {
     const SearchTermsCase& value = search_term_cases[i];
     data.SetURL(value.url);
     TemplateURL url(data);
     EXPECT_TRUE(url.url_ref().IsValid(search_terms_data));
     ASSERT_TRUE(url.url_ref().SupportsReplacement(search_terms_data));
     GURL result(url.url_ref().ReplaceSearchTerms(
-        TemplateURLRef::SearchTermsArgs(value.terms), search_terms_data, NULL));
+        TemplateURLRef::SearchTermsArgs(value.terms), search_terms_data,
+        nullptr));
     ASSERT_TRUE(result.is_valid());
     EXPECT_EQ(value.output, result.spec());
   }
@@ -418,7 +429,7 @@ TEST_F(TemplateURLTest, URLRefTermToWide) {
   TemplateURL url(data);
   EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
   ASSERT_TRUE(url.url_ref().SupportsReplacement(search_terms_data_));
-  for (size_t i = 0; i < arraysize(to_wide_cases); i++) {
+  for (size_t i = 0; i < base::size(to_wide_cases); i++) {
     EXPECT_EQ(to_wide_cases[i].expected_decoded_term,
               url.url_ref().SearchTermToString16(
                   to_wide_cases[i].encoded_search_term));
@@ -440,7 +451,7 @@ TEST_F(TemplateURLTest, DisplayURLToURLRef) {
       ASCIIToUTF16("http://foo%s{language}") },
   };
   TemplateURLData data;
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     data.SetURL(test_data[i].url);
     TemplateURL url(data);
     EXPECT_EQ(test_data[i].expected_result,
@@ -483,7 +494,7 @@ TEST_F(TemplateURLTest, ReplaceSearchTerms) {
   };
   TemplateURLData data;
   data.input_encodings.push_back("UTF-8");
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     data.SetURL(test_data[i].url);
     TemplateURL url(data);
     EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
@@ -510,21 +521,17 @@ TEST_F(TemplateURLTest, ReplaceArbitrarySearchTerms) {
     const std::string url;
     const std::string expected_result;
   } test_data[] = {
-    { "BIG5",  base::WideToUTF16(L"\x60BD"),
-      "http://foo/?{searchTerms}{inputEncoding}",
-      "http://foo/?%B1~BIG5" },
-    { "UTF-8", ASCIIToUTF16("blah"),
-      "http://foo/?{searchTerms}{inputEncoding}",
-      "http://foo/?blahUTF-8" },
-    { "Shift_JIS", base::UTF8ToUTF16("\xe3\x81\x82"),
-      "http://foo/{searchTerms}/bar",
-      "http://foo/%82%A0/bar"},
-    { "Shift_JIS", base::UTF8ToUTF16("\xe3\x81\x82 \xe3\x81\x84"),
-      "http://foo/{searchTerms}/bar",
-      "http://foo/%82%A0%20%82%A2/bar"},
+      {"BIG5", base::WideToUTF16(L"\x60BD"),
+       "http://foo/?{searchTerms}{inputEncoding}", "http://foo/?%B1~BIG5"},
+      {"UTF-8", ASCIIToUTF16("blah"),
+       "http://foo/?{searchTerms}{inputEncoding}", "http://foo/?blahUTF-8"},
+      {"Shift_JIS", base::UTF8ToUTF16("\xe3\x81\x82"),
+       "http://foo/{searchTerms}/bar", "http://foo/%82%A0/bar"},
+      {"Shift_JIS", base::UTF8ToUTF16("\xe3\x81\x82 \xe3\x81\x84"),
+       "http://foo/{searchTerms}/bar", "http://foo/%82%A0%20%82%A2/bar"},
   };
   TemplateURLData data;
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     data.SetURL(test_data[i].url);
     data.input_encodings.clear();
     data.input_encodings.push_back(test_data[i].encoding);
@@ -539,6 +546,57 @@ TEST_F(TemplateURLTest, ReplaceArbitrarySearchTerms) {
   }
 }
 
+// Test that encoding with several optional codepages works as intended.
+// Codepages are tried in order, fallback is UTF-8.
+TEST_F(TemplateURLTest, ReplaceSearchTermsMultipleEncodings) {
+  struct TestData {
+    const std::vector<std::string> encodings;
+    const base::string16 search_term;
+    const std::string url;
+    const std::string expected_result;
+  } test_data[] = {
+      // First and third encodings are valid. First is used.
+      {{"windows-1251", "cp-866", "UTF-8"},
+       base::UTF8ToUTF16("\xD1\x8F"),
+       "http://foo/?{searchTerms}{inputEncoding}",
+       "http://foo/?%FFwindows-1251"},
+      // Second and third encodings are valid, second is used.
+      {{"cp-866", "GB2312", "UTF-8"},
+       base::UTF8ToUTF16("\xE7\x8B\x97"),
+       "http://foo/?{searchTerms}{inputEncoding}",
+       "http://foo/?%B9%B7GB2312"},
+      // Second and third encodings are valid in another order, second is used.
+      {{"cp-866", "UTF-8", "GB2312"},
+       base::UTF8ToUTF16("\xE7\x8B\x97"),
+       "http://foo/?{searchTerms}{inputEncoding}",
+       "http://foo/?%E7%8B%97UTF-8"},
+      // Both encodings are invalid, fallback to UTF-8.
+      {{"cp-866", "windows-1251"},
+       base::UTF8ToUTF16("\xE7\x8B\x97"),
+       "http://foo/?{searchTerms}{inputEncoding}",
+       "http://foo/?%E7%8B%97UTF-8"},
+      // No encodings are given, fallback to UTF-8.
+      {{},
+       base::UTF8ToUTF16("\xE7\x8B\x97"),
+       "http://foo/?{searchTerms}{inputEncoding}",
+       "http://foo/?%E7%8B%97UTF-8"},
+  };
+
+  TemplateURLData data;
+  for (size_t i = 0; i < base::size(test_data); ++i) {
+    data.SetURL(test_data[i].url);
+    data.input_encodings = test_data[i].encodings;
+    TemplateURL url(data);
+    EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
+    ASSERT_TRUE(url.url_ref().SupportsReplacement(search_terms_data_));
+    GURL result(url.url_ref().ReplaceSearchTerms(
+        TemplateURLRef::SearchTermsArgs(test_data[i].search_term),
+        search_terms_data_));
+    ASSERT_TRUE(result.is_valid());
+    EXPECT_EQ(test_data[i].expected_result, result.spec());
+  }
+}
+
 // Tests replacing assisted query stats (AQS) in various scenarios.
 TEST_F(TemplateURLTest, ReplaceAssistedQueryStats) {
   struct TestData {
@@ -581,7 +639,7 @@ TEST_F(TemplateURLTest, ReplaceAssistedQueryStats) {
   };
   TemplateURLData data;
   data.input_encodings.push_back("UTF-8");
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     data.SetURL(test_data[i].url);
     TemplateURL url(data);
     EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
@@ -619,7 +677,7 @@ TEST_F(TemplateURLTest, ReplaceCursorPosition) {
   };
   TemplateURLData data;
   data.input_encodings.push_back("UTF-8");
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     data.SetURL(test_data[i].url);
     TemplateURL url(data);
     EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
@@ -637,22 +695,20 @@ TEST_F(TemplateURLTest, ReplaceCursorPosition) {
 TEST_F(TemplateURLTest, ReplaceInputType) {
   struct TestData {
     const base::string16 search_term;
-    metrics::OmniboxInputType::Type input_type;
+    metrics::OmniboxInputType input_type;
     const std::string url;
     const std::string expected_result;
   } test_data[] = {
-    { ASCIIToUTF16("foo"),
-      metrics::OmniboxInputType::UNKNOWN,
-      "{google:baseURL}?{searchTerms}&{google:inputType}",
-      "http://www.google.com/?foo&oit=1&" },
-    { ASCIIToUTF16("foo"),
-      metrics::OmniboxInputType::URL,
-      "{google:baseURL}?{searchTerms}&{google:inputType}",
-      "http://www.google.com/?foo&oit=3&" },
+      {ASCIIToUTF16("foo"), metrics::OmniboxInputType::UNKNOWN,
+       "{google:baseURL}?{searchTerms}&{google:inputType}",
+       "http://www.google.com/?foo&oit=1&"},
+      {ASCIIToUTF16("foo"), metrics::OmniboxInputType::URL,
+       "{google:baseURL}?{searchTerms}&{google:inputType}",
+       "http://www.google.com/?foo&oit=3&"},
   };
   TemplateURLData data;
   data.input_encodings.push_back("UTF-8");
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     data.SetURL(test_data[i].url);
     TemplateURL url(data);
     EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
@@ -689,7 +745,7 @@ TEST_F(TemplateURLTest, ReplaceCurrentPageUrl) {
   };
   TemplateURLData data;
   data.input_encodings.push_back("UTF-8");
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     data.SetURL(test_data[i].url);
     TemplateURL url(data);
     EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
@@ -727,7 +783,7 @@ TEST_F(TemplateURLTest, Suggestions) {
   TemplateURL url(data);
   EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
   ASSERT_TRUE(url.url_ref().SupportsReplacement(search_terms_data_));
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     TemplateURLRef::SearchTermsArgs search_terms_args(
         ASCIIToUTF16("foobar"));
     search_terms_args.accepted_suggestion = test_data[i].accepted_suggestion;
@@ -797,7 +853,7 @@ TEST_F(TemplateURLTest, HostAndSearchTermKey) {
       {"http://blah/?q=stock:{searchTerms}", "blah", "/", "q"},
   };
 
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     TemplateURLData data;
     data.SetURL(test_data[i].url);
     TemplateURL url(data);
@@ -842,7 +898,7 @@ TEST_F(TemplateURLTest, SearchTermKeyLocation) {
        "", "", ""},
   };
 
-  for (size_t i = 0; i < arraysize(test_data); ++i) {
+  for (size_t i = 0; i < base::size(test_data); ++i) {
     TemplateURLData data;
     data.SetURL(test_data[i].url);
     TemplateURL url(data);
@@ -871,7 +927,7 @@ TEST_F(TemplateURLTest, GoogleBaseSuggestURL) {
     { "http://google.com/intl/xx/", "http://google.com/complete/", },
   };
 
-  for (size_t i = 0; i < arraysize(data); ++i)
+  for (size_t i = 0; i < base::size(data); ++i)
     CheckSuggestBaseURL(data[i].base_url, data[i].base_suggest_url);
 }
 
@@ -914,8 +970,8 @@ TEST_F(TemplateURLTest, ParseURLEmpty) {
   TemplateURL url((TemplateURLData()));
   TemplateURLRef::Replacements replacements;
   bool valid = false;
-  EXPECT_EQ(std::string(),
-            url.url_ref().ParseURL(std::string(), &replacements, NULL, &valid));
+  EXPECT_EQ(std::string(), url.url_ref().ParseURL(std::string(), &replacements,
+                                                  nullptr, &valid));
   EXPECT_TRUE(replacements.empty());
   EXPECT_TRUE(valid);
 }
@@ -926,8 +982,8 @@ TEST_F(TemplateURLTest, ParseURLNoTemplateEnd) {
   TemplateURL url(data);
   TemplateURLRef::Replacements replacements;
   bool valid = false;
-  EXPECT_EQ(std::string(), url.url_ref().ParseURL("{", &replacements, NULL,
-                                                  &valid));
+  EXPECT_EQ(std::string(),
+            url.url_ref().ParseURL("{", &replacements, nullptr, &valid));
   EXPECT_TRUE(replacements.empty());
   EXPECT_FALSE(valid);
 }
@@ -938,7 +994,7 @@ TEST_F(TemplateURLTest, ParseURLNoKnownParameters) {
   TemplateURL url(data);
   TemplateURLRef::Replacements replacements;
   bool valid = false;
-  EXPECT_EQ("{}", url.url_ref().ParseURL("{}", &replacements, NULL, &valid));
+  EXPECT_EQ("{}", url.url_ref().ParseURL("{}", &replacements, nullptr, &valid));
   EXPECT_TRUE(replacements.empty());
   EXPECT_TRUE(valid);
 }
@@ -949,9 +1005,8 @@ TEST_F(TemplateURLTest, ParseURLTwoParameters) {
   TemplateURL url(data);
   TemplateURLRef::Replacements replacements;
   bool valid = false;
-  EXPECT_EQ("{}{}",
-            url.url_ref().ParseURL("{}{{searchTerms}}", &replacements, NULL,
-                                   &valid));
+  EXPECT_EQ("{}{}", url.url_ref().ParseURL("{}{{searchTerms}}", &replacements,
+                                           nullptr, &valid));
   ASSERT_EQ(1U, replacements.size());
   EXPECT_EQ(3U, replacements[0].index);
   EXPECT_EQ(TemplateURLRef::SEARCH_TERMS, replacements[0].type);
@@ -964,9 +1019,8 @@ TEST_F(TemplateURLTest, ParseURLNestedParameter) {
   TemplateURL url(data);
   TemplateURLRef::Replacements replacements;
   bool valid = false;
-  EXPECT_EQ("{",
-            url.url_ref().ParseURL("{{searchTerms}", &replacements, NULL,
-                                   &valid));
+  EXPECT_EQ("{", url.url_ref().ParseURL("{{searchTerms}", &replacements,
+                                        nullptr, &valid));
   ASSERT_EQ(1U, replacements.size());
   EXPECT_EQ(1U, replacements[0].index);
   EXPECT_EQ(TemplateURLRef::SEARCH_TERMS, replacements[0].type);
@@ -1001,24 +1055,9 @@ TEST_F(TemplateURLTest, SearchClient) {
   EXPECT_EQ("http://google.com/?foobar&client=search_client&", result_2.spec());
 }
 
-TEST_F(TemplateURLTest, GetURLNoInstantURL) {
-  TemplateURLData data;
-  data.SetURL("http://google.com/?q={searchTerms}");
-  data.suggestions_url = "http://google.com/suggest?q={searchTerms}";
-  data.alternate_urls.push_back("http://google.com/alt?q={searchTerms}");
-  data.alternate_urls.push_back("{google:baseURL}/alt/#q={searchTerms}");
-  TemplateURL url(data);
-  const std::vector<TemplateURLRef>& url_refs = url.url_refs();
-  ASSERT_EQ(3U, url_refs.size());
-  EXPECT_EQ("http://google.com/alt?q={searchTerms}", url_refs[0].GetURL());
-  EXPECT_EQ("{google:baseURL}/alt/#q={searchTerms}", url_refs[1].GetURL());
-  EXPECT_EQ("http://google.com/?q={searchTerms}", url_refs[2].GetURL());
-}
-
 TEST_F(TemplateURLTest, GetURLNoSuggestionsURL) {
   TemplateURLData data;
   data.SetURL("http://google.com/?q={searchTerms}");
-  data.instant_url = "http://google.com/instant#q={searchTerms}";
   data.alternate_urls.push_back("http://google.com/alt?q={searchTerms}");
   data.alternate_urls.push_back("{google:baseURL}/alt/#q={searchTerms}");
   TemplateURL url(data);
@@ -1041,7 +1080,6 @@ TEST_F(TemplateURLTest, GetURLOnlyOneURL) {
 TEST_F(TemplateURLTest, ExtractSearchTermsFromURL) {
   TemplateURLData data;
   data.SetURL("http://google.com/?q={searchTerms}");
-  data.instant_url = "http://google.com/instant#q={searchTerms}";
   data.alternate_urls.push_back("http://google.com/alt/#q={searchTerms}");
   data.alternate_urls.push_back(
       "http://google.com/alt/?ext=foo&q={searchTerms}#ref=bar");
@@ -1053,18 +1091,15 @@ TEST_F(TemplateURLTest, ExtractSearchTermsFromURL) {
   EXPECT_EQ(ASCIIToUTF16("something"), result);
 
   EXPECT_TRUE(url.ExtractSearchTermsFromURL(
-      GURL("http://google.com/?espv&q=something"),
-      search_terms_data_, &result));
+      GURL("http://google.com/?q=something"), search_terms_data_, &result));
   EXPECT_EQ(ASCIIToUTF16("something"), result);
 
   EXPECT_TRUE(url.ExtractSearchTermsFromURL(
-      GURL("http://google.com/?espv=1&q=something"),
-      search_terms_data_, &result));
+      GURL("http://google.com/?q=something"), search_terms_data_, &result));
   EXPECT_EQ(ASCIIToUTF16("something"), result);
 
   EXPECT_TRUE(url.ExtractSearchTermsFromURL(
-      GURL("http://google.com/?espv=0&q=something"),
-      search_terms_data_, &result));
+      GURL("http://google.com/?q=something"), search_terms_data_, &result));
   EXPECT_EQ(ASCIIToUTF16("something"), result);
 
   EXPECT_TRUE(url.ExtractSearchTermsFromURL(
@@ -1073,18 +1108,15 @@ TEST_F(TemplateURLTest, ExtractSearchTermsFromURL) {
   EXPECT_EQ(ASCIIToUTF16("something"), result);
 
   EXPECT_TRUE(url.ExtractSearchTermsFromURL(
-      GURL("http://google.com/alt/#espv&q=something"),
-      search_terms_data_, &result));
+      GURL("http://google.com/alt/#q=something"), search_terms_data_, &result));
   EXPECT_EQ(ASCIIToUTF16("something"), result);
 
   EXPECT_TRUE(url.ExtractSearchTermsFromURL(
-      GURL("http://google.com/alt/#espv=1&q=something"),
-      search_terms_data_, &result));
+      GURL("http://google.com/alt/#q=something"), search_terms_data_, &result));
   EXPECT_EQ(ASCIIToUTF16("something"), result);
 
   EXPECT_TRUE(url.ExtractSearchTermsFromURL(
-      GURL("http://google.com/alt/#espv=0&q=something"),
-      search_terms_data_, &result));
+      GURL("http://google.com/alt/#q=something"), search_terms_data_, &result));
   EXPECT_EQ(ASCIIToUTF16("something"), result);
 
   EXPECT_FALSE(url.ExtractSearchTermsFromURL(
@@ -1293,92 +1325,9 @@ TEST_F(TemplateURLTest, ExtractSearchTermsWithPrefixAndSuffix) {
       GURL("http://www.example.com/?q=google"), search_terms_data_, &result));
 }
 
-TEST_F(TemplateURLTest, HasSearchTermsReplacementKey) {
-  TemplateURLData data;
-  data.SetURL("http://google.com/?q={searchTerms}");
-  data.instant_url = "http://google.com/instant#q={searchTerms}";
-  data.alternate_urls.push_back("http://google.com/alt/#q={searchTerms}");
-  data.alternate_urls.push_back(
-      "http://google.com/alt/?ext=foo&q={searchTerms}#ref=bar");
-  data.search_terms_replacement_key = "espv";
-  TemplateURL url(data);
-
-  // Test with instant enabled required.
-  EXPECT_FALSE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?espv")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/#espv")));
-
-  EXPECT_FALSE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?q=something&espv")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?q=something&espv=1")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?q=something&espv=0")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?espv&q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?espv=1&q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?espv=0&q=something")));
-
-  EXPECT_FALSE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/alt/#q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/alt/#q=something&espv")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/alt/#q=something&espv=1")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/alt/#q=something&espv=0")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/alt/#espv&q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/alt/#espv=1&q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/alt/#espv=0&q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?espv#q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?espv=1#q=something")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?q=something#espv")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://google.com/?q=something#espv=1")));
-
-  // This does not ensure the domain matches.
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://bing.com/?espv")));
-
-  EXPECT_TRUE(url.HasSearchTermsReplacementKey(
-      GURL("http://bing.com/#espv")));
-}
-
 TEST_F(TemplateURLTest, ReplaceSearchTermsInURL) {
   TemplateURLData data;
   data.SetURL("http://google.com/?q={searchTerms}");
-  data.instant_url = "http://google.com/instant#q={searchTerms}";
   data.alternate_urls.push_back("http://google.com/alt/#q={searchTerms}");
   data.alternate_urls.push_back(
       "http://google.com/alt/?ext=foo&q={searchTerms}#ref=bar");
@@ -1549,7 +1498,7 @@ TEST_F(TemplateURLTest, ReplaceSearchTermsInNonUTF8URL) {
             result);
 }
 
-// Test the |suggest_query_params| field of SearchTermsArgs.
+// Test the |additional_query_params| field of SearchTermsArgs.
 TEST_F(TemplateURLTest, SuggestQueryParams) {
   TemplateURLData data;
   // Pick a URL with replacements before, during, and after the query, to ensure
@@ -1558,27 +1507,29 @@ TEST_F(TemplateURLTest, SuggestQueryParams) {
       "#{google:originalQueryForSuggestion}x");
   TemplateURL url(data);
 
-  // Baseline: no |suggest_query_params| field.
+  // Baseline: no |additional_query_params| field.
   TemplateURLRef::SearchTermsArgs search_terms(ASCIIToUTF16("abc"));
   search_terms.original_query = ASCIIToUTF16("def");
   search_terms.accepted_suggestion = 0;
   EXPECT_EQ("http://www.google.com/search?q=abc#oq=def&x",
             url.url_ref().ReplaceSearchTerms(search_terms, search_terms_data_));
 
-  // Set the suggest_query_params.
-  search_terms.suggest_query_params = "pq=xyz";
+  // Set the |additional_query_params|.
+  search_terms.additional_query_params = "pq=xyz";
   EXPECT_EQ("http://www.google.com/search?pq=xyz&q=abc#oq=def&x",
             url.url_ref().ReplaceSearchTerms(search_terms, search_terms_data_));
 
-  // Add extra_query_params in the mix, and ensure it works.
-  search_terms.append_extra_query_params = true;
+  // Add |append_extra_query_params_from_command_line| into the mix, and ensure
+  // it works.
+  search_terms.append_extra_query_params_from_command_line = true;
   base::CommandLine::ForCurrentProcess()->AppendSwitchASCII(
       switches::kExtraSearchQueryParams, "a=b");
   EXPECT_EQ("http://www.google.com/search?a=b&pq=xyz&q=abc#oq=def&x",
             url.url_ref().ReplaceSearchTerms(search_terms, search_terms_data_));
 }
 
-// Test the |append_extra_query_params| field of SearchTermsArgs.
+// Test the |search_terms.append_extra_query_params_from_command_line| field of
+// SearchTermsArgs.
 TEST_F(TemplateURLTest, ExtraQueryParams) {
   TemplateURLData data;
   // Pick a URL with replacements before, during, and after the query, to ensure
@@ -1587,7 +1538,8 @@ TEST_F(TemplateURLTest, ExtraQueryParams) {
       "#{google:originalQueryForSuggestion}x");
   TemplateURL url(data);
 
-  // Baseline: no command-line args, no |append_extra_query_params| flag.
+  // Baseline: no command-line args, no
+  // |search_terms.append_extra_query_params_from_command_line| flag.
   TemplateURLRef::SearchTermsArgs search_terms(ASCIIToUTF16("abc"));
   search_terms.original_query = ASCIIToUTF16("def");
   search_terms.accepted_suggestion = 0;
@@ -1596,7 +1548,7 @@ TEST_F(TemplateURLTest, ExtraQueryParams) {
 
   // Set the flag.  Since there are no command-line args, this should have no
   // effect.
-  search_terms.append_extra_query_params = true;
+  search_terms.append_extra_query_params_from_command_line = true;
   EXPECT_EQ("http://www.google.com/search?q=abc#oq=def&x",
             url.url_ref().ReplaceSearchTerms(search_terms, search_terms_data_));
 
@@ -1607,7 +1559,7 @@ TEST_F(TemplateURLTest, ExtraQueryParams) {
             url.url_ref().ReplaceSearchTerms(search_terms, search_terms_data_));
 
   // Turn off the flag.  Now the command-line arg should be ignored again.
-  search_terms.append_extra_query_params = false;
+  search_terms.append_extra_query_params_from_command_line = false;
   EXPECT_EQ("http://www.google.com/search?q=abc#oq=def&x",
             url.url_ref().ReplaceSearchTerms(search_terms, search_terms_data_));
 }
@@ -1642,7 +1594,6 @@ TEST_F(TemplateURLTest, ReplacePageClassification) {
 TEST_F(TemplateURLTest, IsSearchResults) {
   TemplateURLData data;
   data.SetURL("http://bar/search?q={searchTerms}");
-  data.instant_url = "http://bar/instant#q={searchTerms}";
   data.new_tab_url = "http://bar/newtab";
   data.alternate_urls.push_back("http://bar/?q={searchTerms}");
   data.alternate_urls.push_back("http://bar/#q={searchTerms}");
@@ -1666,7 +1617,7 @@ TEST_F(TemplateURLTest, IsSearchResults) {
     { "http://bar/newtab", false, },
   };
 
-  for (size_t i = 0; i < arraysize(url_data); ++i) {
+  for (size_t i = 0; i < base::size(url_data); ++i) {
     EXPECT_EQ(url_data[i].result,
               search_provider.IsSearchURL(GURL(url_data[i].url),
                                           search_terms_data_));
@@ -1717,9 +1668,10 @@ TEST_F(TemplateURLTest, ContextualSearchParameters) {
                                                         search_terms_data_);
   EXPECT_EQ("http://bar/_/contextualsearch?", result);
 
-  // Test the current common case, which uses no home country.
-  TemplateURLRef::SearchTermsArgs::ContextualSearchParams params(2, 1,
-                                                                 std::string());
+  // Test the current common case, which uses no home country or previous
+  // event.
+  TemplateURLRef::SearchTermsArgs::ContextualSearchParams params(
+      2, 1, std::string(), 0, 0);
   search_terms_args.contextual_search_params = params;
   result = url.url_ref().ReplaceSearchTerms(search_terms_args,
                                             search_terms_data_);
@@ -1729,9 +1681,10 @@ TEST_F(TemplateURLTest, ContextualSearchParameters) {
       "ctxsl_coca=1",
       result);
 
-  // Test the home country case.
+  // Test the home country and non-zero event data case.
   search_terms_args.contextual_search_params =
-      TemplateURLRef::SearchTermsArgs::ContextualSearchParams(2, 2, "CH");
+      TemplateURLRef::SearchTermsArgs::ContextualSearchParams(2, 2, "CH",
+                                                              1657713458, 5);
   result =
       url.url_ref().ReplaceSearchTerms(search_terms_args, search_terms_data_);
 
@@ -1739,7 +1692,9 @@ TEST_F(TemplateURLTest, ContextualSearchParameters) {
       "http://bar/_/contextualsearch?"
       "ctxs=2&"
       "ctxsl_coca=2&"
-      "ctxs_hc=CH",
+      "ctxs_hc=CH&"
+      "ctxsl_pid=1657713458&"
+      "ctxsl_per=5",
       result);
 }
 
@@ -1779,7 +1734,7 @@ TEST_F(TemplateURLTest, GenerateSearchURL) {
       "http://foo/blah.blah.blah.blah.blah" }
   };
 
-  for (size_t i = 0; i < arraysize(generate_url_cases); ++i) {
+  for (size_t i = 0; i < base::size(generate_url_cases); ++i) {
     TemplateURLData data;
     data.SetURL(generate_url_cases[i].url);
     TemplateURL t_url(data);
@@ -1817,7 +1772,6 @@ TEST_F(TemplateURLTest, InvalidateCachedValues) {
   TemplateURLData data;
   data.SetURL("{google:baseURL}search?q={searchTerms}");
   data.suggestions_url = "{google:baseSuggestURL}search?q={searchTerms}";
-  data.instant_url = "{google:baseURL}webhp";
   data.image_url = "{google:baseURL}searchbyimage/upload";
   data.new_tab_url = "{google:baseURL}_/chrome/newtab";
   data.contextual_search_url = "{google:baseURL}_/contextualsearch";
@@ -1839,9 +1793,6 @@ TEST_F(TemplateURLTest, InvalidateCachedValues) {
   EXPECT_EQ("http://www.google.com/complete/search?q=X",
             url.suggestions_url_ref().ReplaceSearchTerms(search_terms_args,
                                                          search_terms_data_));
-  EXPECT_EQ("http://www.google.com/webhp",
-            url.instant_url_ref().ReplaceSearchTerms(search_terms_args,
-                                                     search_terms_data_));
   EXPECT_EQ("http://www.google.com/searchbyimage/upload",
             url.image_url_ref().ReplaceSearchTerms(search_terms_args,
                                                    search_terms_data_));
@@ -1877,9 +1828,6 @@ TEST_F(TemplateURLTest, InvalidateCachedValues) {
   EXPECT_EQ("https://www.foo.org/complete/search?q=X",
             url.suggestions_url_ref().ReplaceSearchTerms(search_terms_args,
                                                          search_terms_data_));
-  EXPECT_EQ("https://www.foo.org/webhp",
-            url.instant_url_ref().ReplaceSearchTerms(search_terms_args,
-                                                     search_terms_data_));
   EXPECT_EQ("https://www.foo.org/searchbyimage/upload",
             url.image_url_ref().ReplaceSearchTerms(search_terms_args,
                                                    search_terms_data_));
@@ -1902,42 +1850,53 @@ TEST_F(TemplateURLTest, InvalidateCachedValues) {
   search_terms_data_.set_google_base_url("http://www.google.com/");
 }
 
-// search_terms_replacement_key param of TemplateURLData with value of
-// "{google:instantExtendedEnabledKey}" is replaced inside TemplateUrl
-// constructor so must be handled specially inside MatchesData.
-// Test that TemplateURL object created with such param matches correctly its
-// TemplateURLData.
-TEST_F(TemplateURLTest, MatchesData) {
+// Tests the use of wildcards in the path to ensure both extracting search terms
+// and generating a search URL work correctly.
+TEST_F(TemplateURLTest, PathWildcard) {
   TemplateURLData data;
-  data.search_terms_replacement_key =
-      google_util::kGoogleInstantExtendedEnabledKeyFull;
+  data.SetURL(
+      "https://www.google.com/search{google:pathWildcard}?q={searchTerms}");
   TemplateURL url(data);
-  EXPECT_NE(google_util::kGoogleInstantExtendedEnabledKeyFull,
-            url.search_terms_replacement_key());
-  EXPECT_TRUE(TemplateURL::MatchesData(&url, &data, search_terms_data_));
-}
 
-// Test for correct replacement of GoogleInstantExtendedEnabledKey param.
-TEST_F(TemplateURLTest, GoogleInstantExtendedEnabledReplacement) {
-  TemplateURLData data;
-  data.SetURL(std::string("https://www.google.com?") +
-              google_util::kGoogleInstantExtendedEnabledKeyFull +
-              "&q={searchTerms}");
-  data.SetShortName(ASCIIToUTF16("Google"));
-  data.SetKeyword(ASCIIToUTF16("google.com"));
-  data.search_terms_replacement_key =
-      google_util::kGoogleInstantExtendedEnabledKeyFull;
-  TemplateURL turl(data);
-  EXPECT_TRUE(TemplateURL::MatchesData(&turl, &data, search_terms_data_));
-  // Expect that replacement of search_terms_replacement_key in TemplateURL
-  // constructor is correct.
-  EXPECT_EQ(google_util::kInstantExtendedAPIParam,
-            turl.search_terms_replacement_key());
-  // Expect that replacement of {google:instantExtendedEnabledKey} in search url
-  // is correct.
-  GURL search_generated = turl.GenerateSearchURL(search_terms_data_);
-  EXPECT_TRUE(turl.HasSearchTermsReplacementKey(search_generated));
-  net::QueryIterator it(search_generated);
-  ASSERT_FALSE(it.IsAtEnd());
-  EXPECT_EQ(google_util::kInstantExtendedAPIParam, it.GetKey());
+  // Test extracting search terms from a URL.
+  base::string16 search_terms;
+  url.ExtractSearchTermsFromURL(GURL("https://www.google.com/search?q=testing"),
+                                search_terms_data_, &search_terms);
+  EXPECT_EQ(base::ASCIIToUTF16("testing"), search_terms);
+  url.ExtractSearchTermsFromURL(
+      GURL("https://www.google.com/search;_this_is_a_test;_?q=testing"),
+      search_terms_data_, &search_terms);
+  EXPECT_EQ(base::ASCIIToUTF16("testing"), search_terms);
+
+  // Tests overlapping prefix/suffix.
+  data.SetURL(
+      "https://www.google.com/search{google:pathWildcard}rch?q={searchTerms}");
+  TemplateURL overlap_url(data);
+  overlap_url.ExtractSearchTermsFromURL(
+      GURL("https://www.google.com/search?q=testing"), search_terms_data_,
+      &search_terms);
+  EXPECT_TRUE(search_terms.empty());
+
+  // Tests wildcard at beginning of path so we only have a suffix.
+  data.SetURL(
+      "https://www.google.com/{google:pathWildcard}rch?q={searchTerms}");
+  TemplateURL suffix_url(data);
+  suffix_url.ExtractSearchTermsFromURL(
+      GURL("https://www.google.com/search?q=testing"), search_terms_data_,
+      &search_terms);
+  EXPECT_EQ(base::ASCIIToUTF16("testing"), search_terms);
+
+  // Tests wildcard between prefix/suffix.
+  overlap_url.ExtractSearchTermsFromURL(
+      GURL("https://www.google.com/search_testing_rch?q=testing"),
+      search_terms_data_, &search_terms);
+  EXPECT_EQ(base::ASCIIToUTF16("testing"), search_terms);
+
+  // Test generating a URL.
+  TemplateURLRef::SearchTermsArgs search_terms_args(ASCIIToUTF16("foo"));
+  GURL generated_url;
+  url.ReplaceSearchTermsInURL(url.GenerateSearchURL(search_terms_data_),
+                              search_terms_args, search_terms_data_,
+                              &generated_url);
+  EXPECT_EQ("https://www.google.com/search?q=foo", generated_url.spec());
 }
