@@ -4,13 +4,14 @@
 
 #include "components/security_state/content/content_utils.h"
 
+#include <memory>
 #include <string>
 #include <vector>
 
-#include "base/memory/ptr_util.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
+#include "components/security_state/content/ssl_status_input_event_data.h"
 #include "components/security_state/core/security_state.h"
 #include "components/strings/grit/components_chromium_strings.h"
 #include "components/strings/grit/components_strings.h"
@@ -19,11 +20,12 @@
 #include "content/public/browser/security_style_explanations.h"
 #include "content/public/browser/ssl_status.h"
 #include "content/public/browser/web_contents.h"
+#include "content/public/common/url_constants.h"
 #include "net/base/net_errors.h"
 #include "net/cert/x509_certificate.h"
 #include "net/ssl/ssl_cipher_suite_names.h"
 #include "net/ssl/ssl_connection_status_flags.h"
-#include "third_party/WebKit/public/platform/WebMixedContentContextType.h"
+#include "third_party/blink/public/platform/web_mixed_content_context_type.h"
 #include "third_party/boringssl/src/include/openssl/ssl.h"
 #include "ui/base/l10n/l10n_util.h"
 
@@ -39,14 +41,15 @@ blink::WebSecurityStyle SecurityLevelToSecurityStyle(
     case security_state::NONE:
     case security_state::HTTP_SHOW_WARNING:
       return blink::kWebSecurityStyleNeutral;
-    case security_state::SECURITY_WARNING:
     case security_state::SECURE_WITH_POLICY_INSTALLED_CERT:
-      return blink::kWebSecurityStyleWarning;
     case security_state::EV_SECURE:
     case security_state::SECURE:
       return blink::kWebSecurityStyleSecure;
     case security_state::DANGEROUS:
       return blink::kWebSecurityStyleInsecure;
+    case security_state::SECURITY_LEVEL_COUNT:
+      NOTREACHED();
+      return blink::kWebSecurityStyleNeutral;
   }
 
   NOTREACHED();
@@ -56,20 +59,17 @@ blink::WebSecurityStyle SecurityLevelToSecurityStyle(
 void ExplainHTTPSecurity(
     const security_state::SecurityInfo& security_info,
     content::SecurityStyleExplanations* security_style_explanations) {
-  if (security_info.security_level == security_state::HTTP_SHOW_WARNING) {
-    if (security_info.displayed_password_field_on_http ||
-        security_info.displayed_credit_card_field_on_http) {
-      security_style_explanations->neutral_explanations.push_back(
+  // If the page triggers an HTTP-Bad dangerous warning, then override the main
+  // summary for the page and add a bullet describing the issue.
+  if (security_info.security_level == security_state::DANGEROUS &&
+      !security_info.scheme_is_cryptographic) {
+    security_style_explanations->summary =
+        l10n_util::GetStringUTF8(IDS_EDITED_NONSECURE_SUMMARY);
+    if (security_info.insecure_input_events.insecure_field_edited) {
+      security_style_explanations->insecure_explanations.push_back(
           content::SecurityStyleExplanation(
-              l10n_util::GetStringUTF8(IDS_PRIVATE_USER_DATA_INPUT),
-              l10n_util::GetStringUTF8(
-                  IDS_PRIVATE_USER_DATA_INPUT_DESCRIPTION)));
-    }
-    if (security_info.incognito_downgraded_security_level) {
-      security_style_explanations->neutral_explanations.push_back(
-          content::SecurityStyleExplanation(
-              l10n_util::GetStringUTF8(IDS_INCOGNITO_NONSECURE),
-              l10n_util::GetStringUTF8(IDS_INCOGNITO_NONSECURE_DESCRIPTION)));
+              l10n_util::GetStringUTF8(IDS_EDITED_NONSECURE),
+              l10n_util::GetStringUTF8(IDS_EDITED_NONSECURE_DESCRIPTION)));
     }
   }
 }
@@ -77,31 +77,46 @@ void ExplainHTTPSecurity(
 void ExplainSafeBrowsingSecurity(
     const security_state::SecurityInfo& security_info,
     content::SecurityStyleExplanations* security_style_explanations) {
-  if (security_info.malicious_content_status !=
+  if (security_info.malicious_content_status ==
       security_state::MALICIOUS_CONTENT_STATUS_NONE) {
-    security_style_explanations->summary =
-        l10n_util::GetStringUTF8(IDS_SAFEBROWSING_WARNING);
+    return;
   }
+  // Override the main summary for the page.
+  security_style_explanations->summary =
+      l10n_util::GetStringUTF8(IDS_SAFEBROWSING_WARNING);
+  // Add a bullet describing the issue.
+  content::SecurityStyleExplanation explanation(
+      l10n_util::GetStringUTF8(IDS_SAFEBROWSING_WARNING_SUMMARY),
+      l10n_util::GetStringUTF8(IDS_SAFEBROWSING_WARNING_DESCRIPTION));
+  security_style_explanations->insecure_explanations.push_back(explanation);
 }
 
 void ExplainCertificateSecurity(
     const security_state::SecurityInfo& security_info,
     content::SecurityStyleExplanations* security_style_explanations) {
   if (security_info.sha1_in_chain) {
-    security_style_explanations->neutral_explanations.push_back(
-        content::SecurityStyleExplanation(
-            l10n_util::GetStringUTF8(IDS_SHA1),
-            l10n_util::GetStringUTF8(IDS_SHA1_DESCRIPTION),
-            !!security_info.certificate,
-            blink::WebMixedContentContextType::kNotMixedContent));
+    content::SecurityStyleExplanation explanation(
+        l10n_util::GetStringUTF8(IDS_CERTIFICATE_TITLE),
+        l10n_util::GetStringUTF8(IDS_SHA1),
+        l10n_util::GetStringUTF8(IDS_SHA1_DESCRIPTION),
+        security_info.certificate,
+        blink::WebMixedContentContextType::kNotMixedContent);
+    // The impact of SHA1 on the certificate status depends on
+    // the EnableSHA1ForLocalAnchors policy.
+    if (security_info.cert_status & net::CERT_STATUS_WEAK_SIGNATURE_ALGORITHM) {
+      security_style_explanations->insecure_explanations.push_back(explanation);
+    } else {
+      security_style_explanations->neutral_explanations.push_back(explanation);
+    }
   }
 
   if (security_info.cert_missing_subject_alt_name) {
     security_style_explanations->insecure_explanations.push_back(
         content::SecurityStyleExplanation(
+            l10n_util::GetStringUTF8(IDS_CERTIFICATE_TITLE),
             l10n_util::GetStringUTF8(IDS_SUBJECT_ALT_NAME_MISSING),
             l10n_util::GetStringUTF8(IDS_SUBJECT_ALT_NAME_MISSING_DESCRIPTION),
-            !!security_info.certificate,
+            security_info.certificate,
             blink::WebMixedContentContextType::kNotMixedContent));
   }
 
@@ -114,10 +129,11 @@ void ExplainCertificateSecurity(
         net::MapCertStatusToNetError(security_info.cert_status)));
 
     content::SecurityStyleExplanation explanation(
+        l10n_util::GetStringUTF8(IDS_CERTIFICATE_TITLE),
         l10n_util::GetStringUTF8(IDS_CERTIFICATE_CHAIN_ERROR),
         l10n_util::GetStringFUTF8(
             IDS_CERTIFICATE_CHAIN_ERROR_DESCRIPTION_FORMAT, error_string),
-        !!security_info.certificate,
+        security_info.certificate,
         blink::WebMixedContentContextType::kNotMixedContent);
 
     if (is_cert_status_minor_error) {
@@ -145,10 +161,11 @@ void ExplainCertificateSecurity(
     if (!security_info.sha1_in_chain) {
       security_style_explanations->secure_explanations.push_back(
           content::SecurityStyleExplanation(
+              l10n_util::GetStringUTF8(IDS_CERTIFICATE_TITLE),
               l10n_util::GetStringUTF8(IDS_VALID_SERVER_CERTIFICATE),
               l10n_util::GetStringFUTF8(
                   IDS_VALID_SERVER_CERTIFICATE_DESCRIPTION, issuer_name),
-              !!security_info.certificate,
+              security_info.certificate,
               blink::WebMixedContentContextType::kNotMixedContent));
     }
   }
@@ -157,10 +174,23 @@ void ExplainCertificateSecurity(
   if (security_info.pkp_bypassed) {
     security_style_explanations->info_explanations.push_back(
         content::SecurityStyleExplanation(
+            l10n_util::GetStringUTF8(IDS_CERTIFICATE_TITLE),
             l10n_util::GetStringUTF8(IDS_PRIVATE_KEY_PINNING_BYPASSED),
             l10n_util::GetStringUTF8(
                 IDS_PRIVATE_KEY_PINNING_BYPASSED_DESCRIPTION)));
   }
+
+  if (security_info.certificate &&
+      !security_info.certificate->valid_expiry().is_null() &&
+      (security_info.certificate->valid_expiry() - base::Time::Now())
+              .InHours() < 48 &&
+      (security_info.certificate->valid_expiry() > base::Time::Now())) {
+    security_style_explanations->info_explanations.push_back(
+        content::SecurityStyleExplanation(
+            l10n_util::GetStringUTF8(IDS_CERTIFICATE_EXPIRING_SOON),
+            l10n_util::GetStringUTF8(
+                IDS_CERTIFICATE_EXPIRING_SOON_DESCRIPTION)));
+  }
 }
 
 void ExplainConnectionSecurity(
@@ -186,12 +216,13 @@ void ExplainConnectionSecurity(
       net::SSLConnectionStatusToCipherSuite(security_info.connection_status);
   net::SSLCipherSuiteToStrings(&key_exchange, &cipher, &mac, &is_aead,
                                &is_tls13, cipher_suite);
-  base::string16 protocol_name = base::ASCIIToUTF16(protocol);
-  const base::string16 cipher_name =
-      (mac == NULL) ? base::ASCIIToUTF16(cipher)
-                    : l10n_util::GetStringFUTF16(IDS_CIPHER_WITH_MAC,
-                                                 base::ASCIIToUTF16(cipher),
-                                                 base::ASCIIToUTF16(mac));
+  const base::string16 protocol_name = base::ASCIIToUTF16(protocol);
+  const base::string16 cipher_name = base::ASCIIToUTF16(cipher);
+  const base::string16 cipher_full_name =
+      (mac == nullptr) ? cipher_name
+                       : l10n_util::GetStringFUTF16(IDS_CIPHER_WITH_MAC,
+                                                    base::ASCIIToUTF16(cipher),
+                                                    base::ASCIIToUTF16(mac));
 
   // Include the key exchange group (previously known as curve) if specified.
   base::string16 key_exchange_name;
@@ -207,42 +238,43 @@ void ExplainConnectionSecurity(
     key_exchange_name = base::ASCIIToUTF16(key_exchange);
   }
 
-  if (security_info.obsolete_ssl_status == net::OBSOLETE_SSL_NONE) {
-    security_style_explanations->secure_explanations.push_back(
-        content::SecurityStyleExplanation(
-            l10n_util::GetStringUTF8(IDS_STRONG_SSL_SUMMARY),
-            l10n_util::GetStringFUTF8(IDS_STRONG_SSL_DESCRIPTION, protocol_name,
-                                      key_exchange_name, cipher_name)));
+  int status = security_info.obsolete_ssl_status;
+  if (status == net::OBSOLETE_SSL_NONE) {
+    security_style_explanations->secure_explanations.emplace_back(
+        l10n_util::GetStringUTF8(IDS_SSL_CONNECTION_TITLE),
+        l10n_util::GetStringUTF8(IDS_SECURE_SSL_SUMMARY),
+        l10n_util::GetStringFUTF8(IDS_SSL_DESCRIPTION, protocol_name,
+                                  key_exchange_name, cipher_full_name));
     return;
   }
 
-  std::vector<base::string16> description_replacements;
-  int status = security_info.obsolete_ssl_status;
-  int str_id;
+  std::vector<std::string> recommendations;
+  if (status & net::OBSOLETE_SSL_MASK_PROTOCOL) {
+    recommendations.push_back(
+        l10n_util::GetStringFUTF8(IDS_SSL_RECOMMEND_PROTOCOL, protocol_name));
+  }
+  if (status & net::OBSOLETE_SSL_MASK_KEY_EXCHANGE) {
+    recommendations.push_back(
+        l10n_util::GetStringUTF8(IDS_SSL_RECOMMEND_KEY_EXCHANGE));
+  }
+  if (status & net::OBSOLETE_SSL_MASK_CIPHER) {
+    // The problems with obsolete encryption come from the cipher portion rather
+    // than the MAC, so use the shorter |cipher_name| rather than
+    // |cipher_full_name|.
+    recommendations.push_back(
+        l10n_util::GetStringFUTF8(IDS_SSL_RECOMMEND_CIPHER, cipher_name));
+  }
+  if (status & net::OBSOLETE_SSL_MASK_SIGNATURE) {
+    recommendations.push_back(
+        l10n_util::GetStringUTF8(IDS_SSL_RECOMMEND_SIGNATURE));
+  }
 
-  str_id = (status & net::OBSOLETE_SSL_MASK_PROTOCOL)
-               ? IDS_SSL_AN_OBSOLETE_PROTOCOL
-               : IDS_SSL_A_STRONG_PROTOCOL;
-  description_replacements.push_back(protocol_name);
-  description_replacements.push_back(l10n_util::GetStringUTF16(str_id));
-
-  str_id = (status & net::OBSOLETE_SSL_MASK_KEY_EXCHANGE)
-               ? IDS_SSL_AN_OBSOLETE_KEY_EXCHANGE
-               : IDS_SSL_A_STRONG_KEY_EXCHANGE;
-  description_replacements.push_back(key_exchange_name);
-  description_replacements.push_back(l10n_util::GetStringUTF16(str_id));
-
-  str_id = (status & net::OBSOLETE_SSL_MASK_CIPHER) ? IDS_SSL_AN_OBSOLETE_CIPHER
-                                                    : IDS_SSL_A_STRONG_CIPHER;
-  description_replacements.push_back(cipher_name);
-  description_replacements.push_back(l10n_util::GetStringUTF16(str_id));
-
-  security_style_explanations->info_explanations.push_back(
-      content::SecurityStyleExplanation(
-          l10n_util::GetStringUTF8(IDS_OBSOLETE_SSL_SUMMARY),
-          base::UTF16ToUTF8(
-              l10n_util::GetStringFUTF16(IDS_OBSOLETE_SSL_DESCRIPTION,
-                                         description_replacements, nullptr))));
+  security_style_explanations->info_explanations.emplace_back(
+      l10n_util::GetStringUTF8(IDS_SSL_CONNECTION_TITLE),
+      l10n_util::GetStringUTF8(IDS_OBSOLETE_SSL_SUMMARY),
+      l10n_util::GetStringFUTF8(IDS_SSL_DESCRIPTION, protocol_name,
+                                key_exchange_name, cipher_full_name),
+      std::move(recommendations));
 }
 
 void ExplainContentSecurity(
@@ -254,21 +286,49 @@ void ExplainContentSecurity(
       SecurityLevelToSecurityStyle(
           security_state::kDisplayedInsecureContentLevel);
 
-  // Record the presence of mixed content (HTTP subresources on an HTTPS
-  // page).
+  // Add the secure explanation unless there is an issue.
+  bool add_secure_explanation = true;
+
   security_style_explanations->ran_mixed_content =
       security_info.mixed_content_status ==
           security_state::CONTENT_STATUS_RAN ||
       security_info.mixed_content_status ==
           security_state::CONTENT_STATUS_DISPLAYED_AND_RAN;
+  if (security_style_explanations->ran_mixed_content) {
+    add_secure_explanation = false;
+    security_style_explanations->insecure_explanations.push_back(
+        content::SecurityStyleExplanation(
+            l10n_util::GetStringUTF8(IDS_RESOURCE_SECURITY_TITLE),
+            l10n_util::GetStringUTF8(IDS_MIXED_ACTIVE_CONTENT_SUMMARY),
+            l10n_util::GetStringUTF8(IDS_MIXED_ACTIVE_CONTENT_DESCRIPTION),
+            nullptr, blink::WebMixedContentContextType::kBlockable));
+  }
+
   security_style_explanations->displayed_mixed_content =
       security_info.mixed_content_status ==
           security_state::CONTENT_STATUS_DISPLAYED ||
       security_info.mixed_content_status ==
           security_state::CONTENT_STATUS_DISPLAYED_AND_RAN;
+  if (security_style_explanations->displayed_mixed_content) {
+    add_secure_explanation = false;
+    security_style_explanations->neutral_explanations.push_back(
+        content::SecurityStyleExplanation(
+            l10n_util::GetStringUTF8(IDS_RESOURCE_SECURITY_TITLE),
+            l10n_util::GetStringUTF8(IDS_MIXED_PASSIVE_CONTENT_SUMMARY),
+            l10n_util::GetStringUTF8(IDS_MIXED_PASSIVE_CONTENT_DESCRIPTION),
+            nullptr, blink::WebMixedContentContextType::kOptionallyBlockable));
+  }
 
   security_style_explanations->contained_mixed_form =
       security_info.contained_mixed_form;
+  if (security_style_explanations->contained_mixed_form) {
+    add_secure_explanation = false;
+    security_style_explanations->neutral_explanations.push_back(
+        content::SecurityStyleExplanation(
+            l10n_util::GetStringUTF8(IDS_RESOURCE_SECURITY_TITLE),
+            l10n_util::GetStringUTF8(IDS_NON_SECURE_FORM_SUMMARY),
+            l10n_util::GetStringUTF8(IDS_NON_SECURE_FORM_DESCRIPTION)));
+  }
 
   // If the main resource was loaded with no certificate errors or only minor
   // certificate errors, then record the presence of subresources with
@@ -285,11 +345,39 @@ void ExplainContentSecurity(
             security_state::CONTENT_STATUS_RAN ||
         security_info.content_with_cert_errors_status ==
             security_state::CONTENT_STATUS_DISPLAYED_AND_RAN;
+    if (security_style_explanations->ran_content_with_cert_errors) {
+      add_secure_explanation = false;
+      security_style_explanations->insecure_explanations.push_back(
+          content::SecurityStyleExplanation(
+              l10n_util::GetStringUTF8(IDS_RESOURCE_SECURITY_TITLE),
+              l10n_util::GetStringUTF8(IDS_CERT_ERROR_ACTIVE_CONTENT_SUMMARY),
+              l10n_util::GetStringUTF8(
+                  IDS_CERT_ERROR_ACTIVE_CONTENT_DESCRIPTION)));
+    }
+
     security_style_explanations->displayed_content_with_cert_errors =
         security_info.content_with_cert_errors_status ==
             security_state::CONTENT_STATUS_DISPLAYED ||
         security_info.content_with_cert_errors_status ==
             security_state::CONTENT_STATUS_DISPLAYED_AND_RAN;
+    if (security_style_explanations->displayed_content_with_cert_errors) {
+      add_secure_explanation = false;
+      security_style_explanations->neutral_explanations.push_back(
+          content::SecurityStyleExplanation(
+              l10n_util::GetStringUTF8(IDS_RESOURCE_SECURITY_TITLE),
+              l10n_util::GetStringUTF8(IDS_CERT_ERROR_PASSIVE_CONTENT_SUMMARY),
+              l10n_util::GetStringUTF8(
+                  IDS_CERT_ERROR_PASSIVE_CONTENT_DESCRIPTION)));
+    }
+  }
+
+  if (add_secure_explanation) {
+    DCHECK(security_info.scheme_is_cryptographic);
+    security_style_explanations->secure_explanations.push_back(
+        content::SecurityStyleExplanation(
+            l10n_util::GetStringUTF8(IDS_RESOURCE_SECURITY_TITLE),
+            l10n_util::GetStringUTF8(IDS_SECURE_RESOURCES_SUMMARY),
+            l10n_util::GetStringUTF8(IDS_SECURE_RESOURCES_DESCRIPTION)));
   }
 }
 
@@ -297,21 +385,27 @@ void ExplainContentSecurity(
 
 std::unique_ptr<security_state::VisibleSecurityState> GetVisibleSecurityState(
     content::WebContents* web_contents) {
-  auto state = base::MakeUnique<security_state::VisibleSecurityState>();
+  auto state = std::make_unique<security_state::VisibleSecurityState>();
 
   content::NavigationEntry* entry =
       web_contents->GetController().GetVisibleEntry();
-  if (!entry || !entry->GetSSL().initialized)
+  if (!entry)
     return state;
-
-  state->connection_info_initialized = true;
+  // Set fields that are not dependent on the connection info.
+  state->is_error_page = entry->GetPageType() == content::PAGE_TYPE_ERROR;
+  state->is_view_source =
+      entry->GetVirtualURL().SchemeIs(content::kViewSourceScheme);
   state->url = entry->GetURL();
+
+  if (!entry->GetSSL().initialized)
+    return state;
+  state->connection_info_initialized = true;
   const content::SSLStatus& ssl = entry->GetSSL();
   state->certificate = ssl.certificate;
   state->cert_status = ssl.cert_status;
   state->connection_status = ssl.connection_status;
   state->key_exchange_group = ssl.key_exchange_group;
-  state->security_bits = ssl.security_bits;
+  state->peer_signature_algorithm = ssl.peer_signature_algorithm;
   state->pkp_bypassed = ssl.pkp_bypassed;
   state->displayed_mixed_content =
       !!(ssl.content_status & content::SSLStatus::DISPLAYED_INSECURE_CONTENT);
@@ -325,12 +419,12 @@ std::unique_ptr<security_state::VisibleSecurityState> GetVisibleSecurityState(
   state->contained_mixed_form =
       !!(ssl.content_status &
          content::SSLStatus::DISPLAYED_FORM_WITH_INSECURE_ACTION);
-  state->displayed_password_field_on_http =
-      !!(ssl.content_status &
-         content::SSLStatus::DISPLAYED_PASSWORD_FIELD_ON_HTTP);
-  state->displayed_credit_card_field_on_http =
-      !!(ssl.content_status &
-         content::SSLStatus::DISPLAYED_CREDIT_CARD_FIELD_ON_HTTP);
+
+  SSLStatusInputEventData* input_events =
+      static_cast<SSLStatusInputEventData*>(ssl.user_data.get());
+
+  if (input_events)
+    state->insecure_input_events = *input_events->input_events();
 
   return state;
 }
