@@ -8,12 +8,13 @@
 #include <string>
 
 #include "base/command_line.h"
-#include "base/message_loop/message_loop.h"
 #include "base/strings/stringprintf.h"
+#include "base/test/scoped_task_environment.h"
 #include "components/content_settings/core/browser/cookie_settings.h"
+#include "components/prefs/pref_member.h"
+#include "components/signin/core/browser/account_consistency_method.h"
 #include "components/signin/core/browser/chrome_connected_header_helper.h"
-#include "components/signin/core/common/profile_management_switches.h"
-#include "components/signin/core/common/signin_features.h"
+#include "components/signin/core/browser/signin_buildflags.h"
 #include "components/sync_preferences/testing_pref_service_syncable.h"
 #include "google_apis/gaia/gaia_urls.h"
 #include "net/traffic_annotation/network_traffic_annotation_test_helper.h"
@@ -25,6 +26,10 @@
 #include "components/signin/core/browser/dice_header_helper.h"
 #endif
 
+namespace {
+constexpr char kTestDeviceId[] = "DeviceID";
+}
+
 namespace signin {
 
 class SigninHeaderHelperTest : public testing::Test {
@@ -35,7 +40,8 @@ class SigninHeaderHelperTest : public testing::Test {
 
     settings_map_ = new HostContentSettingsMap(
         &prefs_, false /* incognito_profile */, false /* guest_profile */,
-        false /* store_last_modified */);
+        false /* store_last_modified */,
+        false /* migrate_requesting_and_top_level_origin_settings */);
     cookie_settings_ =
         new content_settings::CookieSettings(settings_map_.get(), &prefs_, "");
   }
@@ -45,21 +51,26 @@ class SigninHeaderHelperTest : public testing::Test {
   void CheckMirrorCookieRequest(const GURL& url,
                                 const std::string& account_id,
                                 const std::string& expected_request) {
-    EXPECT_EQ(
-        BuildMirrorRequestCookieIfPossible(
-            url, account_id, cookie_settings_.get(), PROFILE_MODE_DEFAULT),
-        expected_request);
+    EXPECT_EQ(BuildMirrorRequestCookieIfPossible(
+                  url, account_id, account_consistency_, cookie_settings_.get(),
+                  PROFILE_MODE_DEFAULT),
+              expected_request);
   }
 
-  std::unique_ptr<net::URLRequest> CreateRequest(const GURL& url,
-                                                 const std::string& account_id,
-                                                 bool sync_enabled) {
+  std::unique_ptr<net::URLRequest> CreateRequest(
+      const GURL& url,
+      const std::string& account_id) {
     std::unique_ptr<net::URLRequest> url_request =
         url_request_context_.CreateRequest(url, net::DEFAULT_PRIORITY, nullptr,
                                            TRAFFIC_ANNOTATION_FOR_TESTS);
-    AppendOrRemoveAccountConsistentyRequestHeader(
-        url_request.get(), GURL(), account_id, sync_enabled,
+    RequestAdapter request_adapter(url_request.get());
+    AppendOrRemoveMirrorRequestHeader(
+        &request_adapter, GURL(), account_id, account_consistency_,
         cookie_settings_.get(), PROFILE_MODE_DEFAULT);
+    AppendOrRemoveDiceRequestHeader(&request_adapter, GURL(), account_id,
+                                    sync_enabled_, sync_has_auth_error_,
+                                    account_consistency_,
+                                    cookie_settings_.get(), device_id_);
     return url_request;
   }
 
@@ -71,7 +82,8 @@ class SigninHeaderHelperTest : public testing::Test {
     std::string request;
     EXPECT_EQ(
         url_request->extra_request_headers().GetHeader(header_name, &request),
-        expected_result);
+        expected_result)
+        << header_name << ": " << request;
     if (expected_result) {
       EXPECT_EQ(expected_request, request);
     }
@@ -81,7 +93,7 @@ class SigninHeaderHelperTest : public testing::Test {
                                 const std::string& account_id,
                                 const std::string& expected_request) {
     std::unique_ptr<net::URLRequest> url_request =
-        CreateRequest(url, account_id, false /* sync_enabled */);
+        CreateRequest(url, account_id);
     CheckAccountConsistencyHeaderRequest(
         url_request.get(), kChromeConnectedHeader, expected_request);
   }
@@ -89,11 +101,10 @@ class SigninHeaderHelperTest : public testing::Test {
 #if BUILDFLAG(ENABLE_DICE_SUPPORT)
   void CheckDiceHeaderRequest(const GURL& url,
                               const std::string& account_id,
-                              bool sync_enabled,
                               const std::string& expected_mirror_request,
                               const std::string& expected_dice_request) {
     std::unique_ptr<net::URLRequest> url_request =
-        CreateRequest(url, account_id, sync_enabled);
+        CreateRequest(url, account_id);
     CheckAccountConsistencyHeaderRequest(
         url_request.get(), kChromeConnectedHeader, expected_mirror_request);
     CheckAccountConsistencyHeaderRequest(url_request.get(), kDiceRequestHeader,
@@ -101,7 +112,13 @@ class SigninHeaderHelperTest : public testing::Test {
   }
 #endif
 
-  base::MessageLoop loop_;
+  base::test::ScopedTaskEnvironment task_environment_;
+
+  bool sync_enabled_ = false;
+  bool sync_has_auth_error_ = false;
+  std::string device_id_ = kTestDeviceId;
+  AccountConsistencyMethod account_consistency_ =
+      AccountConsistencyMethod::kDisabled;
 
   sync_preferences::TestingPrefServiceSyncable prefs_;
   net::TestURLRequestContext url_request_context_;
@@ -110,20 +127,30 @@ class SigninHeaderHelperTest : public testing::Test {
   scoped_refptr<content_settings::CookieSettings> cookie_settings_;
 };
 
-// Tests that no Mirror request is returned when the user is not signed in (no
+#if defined(OS_CHROMEOS)
+// Tests that Mirror request is returned on Chrome OS for Public Sessions (no
 // account id).
+TEST_F(SigninHeaderHelperTest, TestMirrorRequestNoAccountIdChromeOS) {
+  account_consistency_ = AccountConsistencyMethod::kMirror;
+  CheckMirrorHeaderRequest(GURL("https://docs.google.com"), "",
+                           "mode=0,enable_account_consistency=true");
+  CheckMirrorCookieRequest(GURL("https://docs.google.com"), "",
+                           "mode=0:enable_account_consistency=true");
+}
+#else  // !defined(OS_CHROMEOS)
+// Tests that no Mirror request is returned when the user is not signed in (no
+// account id), for non Chrome OS platforms.
 TEST_F(SigninHeaderHelperTest, TestNoMirrorRequestNoAccountId) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   CheckMirrorHeaderRequest(GURL("https://docs.google.com"), "", "");
   CheckMirrorCookieRequest(GURL("https://docs.google.com"), "", "");
 }
+#endif
 
 // Tests that no Mirror request is returned when the cookies aren't allowed to
 // be set.
 TEST_F(SigninHeaderHelperTest, TestNoMirrorRequestCookieSettingBlocked) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   cookie_settings_->SetDefaultCookieSetting(CONTENT_SETTING_BLOCK);
   CheckMirrorHeaderRequest(GURL("https://docs.google.com"), "0123456789", "");
   CheckMirrorCookieRequest(GURL("https://docs.google.com"), "0123456789", "");
@@ -131,8 +158,7 @@ TEST_F(SigninHeaderHelperTest, TestNoMirrorRequestCookieSettingBlocked) {
 
 // Tests that no Mirror request is returned when the target is a non-Google URL.
 TEST_F(SigninHeaderHelperTest, TestNoMirrorRequestExternalURL) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   CheckMirrorHeaderRequest(GURL("https://foo.com"), "0123456789", "");
   CheckMirrorCookieRequest(GURL("https://foo.com"), "0123456789", "");
 }
@@ -140,8 +166,7 @@ TEST_F(SigninHeaderHelperTest, TestNoMirrorRequestExternalURL) {
 // Tests that the Mirror request is returned without the GAIA Id when the target
 // is a google TLD domain.
 TEST_F(SigninHeaderHelperTest, TestMirrorRequestGoogleTLD) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   CheckMirrorHeaderRequest(GURL("https://google.fr"), "0123456789",
                            "mode=0,enable_account_consistency=true");
   CheckMirrorCookieRequest(GURL("https://google.de"), "0123456789",
@@ -151,8 +176,7 @@ TEST_F(SigninHeaderHelperTest, TestMirrorRequestGoogleTLD) {
 // Tests that the Mirror request is returned when the target is the domain
 // google.com, and that the GAIA Id is only attached for the cookie.
 TEST_F(SigninHeaderHelperTest, TestMirrorRequestGoogleCom) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   CheckMirrorHeaderRequest(GURL("https://www.google.com"), "0123456789",
                            "mode=0,enable_account_consistency=true");
   CheckMirrorCookieRequest(
@@ -160,6 +184,36 @@ TEST_F(SigninHeaderHelperTest, TestMirrorRequestGoogleCom) {
       "id=0123456789:mode=0:enable_account_consistency=true");
 }
 
+// Tests that no header sent when mirror account consistency is nor requested.
+TEST_F(SigninHeaderHelperTest, TestMirrorRequestGoogleComNoProfileConsistency) {
+  std::unique_ptr<net::URLRequest> url_request =
+      url_request_context_.CreateRequest(GURL("https://www.google.com"),
+                                         net::DEFAULT_PRIORITY, nullptr,
+                                         TRAFFIC_ANNOTATION_FOR_TESTS);
+  RequestAdapter request_adapter(url_request.get());
+  AppendOrRemoveMirrorRequestHeader(
+      &request_adapter, GURL(), "0123456789", account_consistency_,
+      cookie_settings_.get(), PROFILE_MODE_DEFAULT);
+  CheckAccountConsistencyHeaderRequest(url_request.get(),
+                                       kChromeConnectedHeader, "");
+}
+
+// Tests that header sent when mirror account consistency is requested.
+TEST_F(SigninHeaderHelperTest, TestMirrorRequestGoogleComProfileConsistency) {
+  account_consistency_ = AccountConsistencyMethod::kMirror;
+  std::unique_ptr<net::URLRequest> url_request =
+      url_request_context_.CreateRequest(GURL("https://www.google.com"),
+                                         net::DEFAULT_PRIORITY, nullptr,
+                                         TRAFFIC_ANNOTATION_FOR_TESTS);
+  RequestAdapter request_adapter(url_request.get());
+  AppendOrRemoveMirrorRequestHeader(
+      &request_adapter, GURL(), "0123456789", account_consistency_,
+      cookie_settings_.get(), PROFILE_MODE_DEFAULT);
+  CheckAccountConsistencyHeaderRequest(
+      url_request.get(), kChromeConnectedHeader,
+      "mode=0,enable_account_consistency=true");
+}
+
 // Mirror is always enabled on Android and iOS, so these tests are only relevant
 // on Desktop.
 #if BUILDFLAG(ENABLE_DICE_SUPPORT)
@@ -167,7 +221,6 @@ TEST_F(SigninHeaderHelperTest, TestMirrorRequestGoogleCom) {
 // Tests that the Mirror request is returned when the target is a Gaia URL, even
 // if account consistency is disabled.
 TEST_F(SigninHeaderHelperTest, TestMirrorRequestGaiaURL) {
-  ASSERT_FALSE(switches::IsAccountConsistencyMirrorEnabled());
   CheckMirrorHeaderRequest(GURL("https://accounts.google.com"), "0123456789",
                            "mode=0,enable_account_consistency=false");
   CheckMirrorCookieRequest(
@@ -177,46 +230,106 @@ TEST_F(SigninHeaderHelperTest, TestMirrorRequestGaiaURL) {
 
 // Tests Dice requests.
 TEST_F(SigninHeaderHelperTest, TestDiceRequest) {
-  switches::EnableAccountConsistencyDiceForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kDice;
   // ChromeConnected but no Dice for Docs URLs.
   CheckDiceHeaderRequest(
-      GURL("https://docs.google.com"), "0123456789", false /* sync_enabled */,
+      GURL("https://docs.google.com"), "0123456789",
       "id=0123456789,mode=0,enable_account_consistency=false", "");
 
   // ChromeConnected and Dice for Gaia URLs.
   // Sync disabled.
   std::string client_id = GaiaUrls::GetInstance()->oauth2_chrome_client_id();
   ASSERT_FALSE(client_id.empty());
-  CheckDiceHeaderRequest(GURL("https://accounts.google.com"), "0123456789",
-                         false /* sync_enabled */,
-                         "mode=0,enable_account_consistency=false",
-                         "client_id=" + client_id);
-  // Sync enabled: check that the Dice header has the Sync account ID and that
-  // the mirror header is not modified.
   CheckDiceHeaderRequest(
       GURL("https://accounts.google.com"), "0123456789",
-      true /* sync_enabled */, "mode=0,enable_account_consistency=false",
-      "client_id=" + client_id + ",sync_account_id=0123456789");
+      "mode=0,enable_account_consistency=false",
+      base::StringPrintf(
+          "version=%s,client_id=%s,device_id=DeviceID,signin_mode=all_accounts,"
+          "signout_mode=show_confirmation",
+          kDiceProtocolVersion, client_id.c_str()));
+  // Sync enabled: check that the Dice header has the Sync account ID and that
+  // the mirror header is not modified.
+  sync_enabled_ = true;
+  CheckDiceHeaderRequest(
+      GURL("https://accounts.google.com"), "0123456789",
+      "mode=0,enable_account_consistency=false",
+      base::StringPrintf("version=%s,client_id=%s,device_id=DeviceID,"
+                         "sync_account_id=0123456789,signin_mode=all_accounts,"
+                         "signout_mode=show_confirmation",
+                         kDiceProtocolVersion, client_id.c_str()));
+  sync_enabled_ = false;
 
   // No ChromeConnected and no Dice for other URLs.
-  CheckDiceHeaderRequest(GURL("https://www.google.com"), "0123456789",
-                         false /* sync_enabled */, "", "");
+  CheckDiceHeaderRequest(GURL("https://www.google.com"), "0123456789", "", "");
+}
+
+// When cookies are blocked, only the Dice header is sent.
+TEST_F(SigninHeaderHelperTest, DiceCookiesBlocked) {
+  account_consistency_ = AccountConsistencyMethod::kDice;
+  cookie_settings_->SetDefaultCookieSetting(CONTENT_SETTING_BLOCK);
+
+  std::string client_id = GaiaUrls::GetInstance()->oauth2_chrome_client_id();
+  ASSERT_FALSE(client_id.empty());
+  CheckDiceHeaderRequest(
+      GURL("https://accounts.google.com"), "0123456789", "",
+      base::StringPrintf(
+          "version=%s,client_id=%s,device_id=DeviceID,signin_mode=all_accounts,"
+          "signout_mode=show_confirmation",
+          kDiceProtocolVersion, client_id.c_str()));
 }
 
 // Tests that no Dice request is returned when Dice is not enabled.
 TEST_F(SigninHeaderHelperTest, TestNoDiceRequestWhenDisabled) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   CheckDiceHeaderRequest(GURL("https://accounts.google.com"), "0123456789",
-                         false /* sync_enabled */,
                          "mode=0,enable_account_consistency=true", "");
 }
 
+TEST_F(SigninHeaderHelperTest, TestDiceEmptyDeviceID) {
+  account_consistency_ = AccountConsistencyMethod::kDiceMigration;
+  std::string client_id = GaiaUrls::GetInstance()->oauth2_chrome_client_id();
+  ASSERT_FALSE(client_id.empty());
+
+  device_id_.clear();
+
+  CheckDiceHeaderRequest(
+      GURL("https://accounts.google.com"), "0123456789",
+      "mode=0,enable_account_consistency=false",
+      base::StringPrintf("version=%s,client_id=%s,signin_mode=all_accounts,"
+                         "signout_mode=no_confirmation",
+                         kDiceProtocolVersion, client_id.c_str()));
+}
+
+// Tests that the signout confirmation is requested iff the Dice migration is
+// complete.
+TEST_F(SigninHeaderHelperTest, TestDiceMigration) {
+  account_consistency_ = AccountConsistencyMethod::kDiceMigration;
+  std::string client_id = GaiaUrls::GetInstance()->oauth2_chrome_client_id();
+  ASSERT_FALSE(client_id.empty());
+
+  // No signout confirmation by default.
+  CheckDiceHeaderRequest(
+      GURL("https://accounts.google.com"), "0123456789",
+      "mode=0,enable_account_consistency=false",
+      base::StringPrintf(
+          "version=%s,client_id=%s,device_id=DeviceID,signin_mode=all_accounts,"
+          "signout_mode=no_confirmation",
+          kDiceProtocolVersion, client_id.c_str()));
+
+  // Signout confirmation after the migration is complete.
+  account_consistency_ = AccountConsistencyMethod::kDice;
+  CheckDiceHeaderRequest(
+      GURL("https://accounts.google.com"), "0123456789",
+      "mode=0,enable_account_consistency=false",
+      base::StringPrintf(
+          "version=%s,client_id=%s,device_id=DeviceID,signin_mode=all_accounts,"
+          "signout_mode=show_confirmation",
+          kDiceProtocolVersion, client_id.c_str()));
+}
+
 // Tests that the Mirror request is returned with the GAIA Id on Drive origin,
 // even if account consistency is disabled.
 TEST_F(SigninHeaderHelperTest, TestMirrorRequestDrive) {
-  ASSERT_FALSE(switches::IsAccountConsistencyMirrorEnabled());
   CheckMirrorHeaderRequest(
       GURL("https://docs.google.com/document"), "0123456789",
       "id=0123456789,mode=0,enable_account_consistency=false");
@@ -225,8 +338,7 @@ TEST_F(SigninHeaderHelperTest, TestMirrorRequestDrive) {
       "id=0123456789:mode=0:enable_account_consistency=false");
 
   // Enable Account Consistency will override the disable.
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   CheckMirrorHeaderRequest(
       GURL("https://docs.google.com/document"), "0123456789",
       "id=0123456789,mode=0,enable_account_consistency=true");
@@ -238,6 +350,8 @@ TEST_F(SigninHeaderHelperTest, TestMirrorRequestDrive) {
 TEST_F(SigninHeaderHelperTest, TestDiceInvalidResponseParams) {
   DiceResponseParams params = BuildDiceSigninResponseParams("blah");
   EXPECT_EQ(DiceAction::NONE, params.user_intention);
+  params = BuildDiceSignoutResponseParams("blah");
+  EXPECT_EQ(DiceAction::NONE, params.user_intention);
 }
 
 TEST_F(SigninHeaderHelperTest, TestBuildDiceResponseParams) {
@@ -253,10 +367,24 @@ TEST_F(SigninHeaderHelperTest, TestBuildDiceResponseParams) {
             "action=SIGNIN,id=%s,email=%s,authuser=%i,authorization_code=%s",
             kGaiaID, kEmail, kSessionIndex, kAuthorizationCode));
     EXPECT_EQ(DiceAction::SIGNIN, params.user_intention);
-    EXPECT_EQ(kGaiaID, params.signin_info.gaia_id);
-    EXPECT_EQ(kEmail, params.signin_info.email);
-    EXPECT_EQ(kSessionIndex, params.signin_info.session_index);
-    EXPECT_EQ(kAuthorizationCode, params.signin_info.authorization_code);
+    ASSERT_TRUE(params.signin_info);
+    EXPECT_EQ(kGaiaID, params.signin_info->account_info.gaia_id);
+    EXPECT_EQ(kEmail, params.signin_info->account_info.email);
+    EXPECT_EQ(kSessionIndex, params.signin_info->account_info.session_index);
+    EXPECT_EQ(kAuthorizationCode, params.signin_info->authorization_code);
+  }
+
+  {
+    // EnableSync response.
+    DiceResponseParams params = BuildDiceSigninResponseParams(
+        base::StringPrintf("action=ENABLE_SYNC,id=%s,email=%s,authuser=%i",
+                           kGaiaID, kEmail, kSessionIndex));
+    EXPECT_EQ(DiceAction::ENABLE_SYNC, params.user_intention);
+    ASSERT_TRUE(params.enable_sync_info);
+    EXPECT_EQ(kGaiaID, params.enable_sync_info->account_info.gaia_id);
+    EXPECT_EQ(kEmail, params.enable_sync_info->account_info.email);
+    EXPECT_EQ(kSessionIndex,
+              params.enable_sync_info->account_info.session_index);
   }
 
   {
@@ -267,12 +395,12 @@ TEST_F(SigninHeaderHelperTest, TestBuildDiceResponseParams) {
         base::StringPrintf("email=\"%s\", sessionindex=%i, obfuscatedid=\"%s\"",
                            kEmail, kSessionIndex, kGaiaID));
     ASSERT_EQ(DiceAction::SIGNOUT, params.user_intention);
-    EXPECT_EQ(1u, params.signout_info.gaia_id.size());
-    EXPECT_EQ(1u, params.signout_info.email.size());
-    EXPECT_EQ(1u, params.signout_info.session_index.size());
-    EXPECT_EQ(kGaiaID, params.signout_info.gaia_id[0]);
-    EXPECT_EQ(kEmail, params.signout_info.email[0]);
-    EXPECT_EQ(kSessionIndex, params.signout_info.session_index[0]);
+    ASSERT_TRUE(params.signout_info);
+    EXPECT_EQ(1u, params.signout_info->account_infos.size());
+    EXPECT_EQ(kGaiaID, params.signout_info->account_infos[0].gaia_id);
+    EXPECT_EQ(kEmail, params.signout_info->account_infos[0].email);
+    EXPECT_EQ(kSessionIndex,
+              params.signout_info->account_infos[0].session_index);
   }
 
   {
@@ -286,15 +414,16 @@ TEST_F(SigninHeaderHelperTest, TestBuildDiceResponseParams) {
             "email=\"%s\", sessionindex=%i, obfuscatedid=\"%s\"",
             kEmail, kSessionIndex, kGaiaID, kEmail2, kSessionIndex2, kGaiaID2));
     ASSERT_EQ(DiceAction::SIGNOUT, params.user_intention);
-    EXPECT_EQ(2u, params.signout_info.gaia_id.size());
-    EXPECT_EQ(2u, params.signout_info.email.size());
-    EXPECT_EQ(2u, params.signout_info.session_index.size());
-    EXPECT_EQ(kGaiaID, params.signout_info.gaia_id[0]);
-    EXPECT_EQ(kEmail, params.signout_info.email[0]);
-    EXPECT_EQ(kSessionIndex, params.signout_info.session_index[0]);
-    EXPECT_EQ(kGaiaID2, params.signout_info.gaia_id[1]);
-    EXPECT_EQ(kEmail2, params.signout_info.email[1]);
-    EXPECT_EQ(kSessionIndex2, params.signout_info.session_index[1]);
+    ASSERT_TRUE(params.signout_info);
+    EXPECT_EQ(2u, params.signout_info->account_infos.size());
+    EXPECT_EQ(kGaiaID, params.signout_info->account_infos[0].gaia_id);
+    EXPECT_EQ(kEmail, params.signout_info->account_infos[0].email);
+    EXPECT_EQ(kSessionIndex,
+              params.signout_info->account_infos[0].session_index);
+    EXPECT_EQ(kGaiaID2, params.signout_info->account_infos[1].gaia_id);
+    EXPECT_EQ(kEmail2, params.signout_info->account_infos[1].email);
+    EXPECT_EQ(kSessionIndex2,
+              params.signout_info->account_infos[1].session_index);
   }
 
   {
@@ -329,16 +458,16 @@ TEST_F(SigninHeaderHelperTest, TestBuildDiceResponseParams) {
 // Tests that the Mirror header request is returned normally when the redirect
 // URL is eligible.
 TEST_F(SigninHeaderHelperTest, TestMirrorHeaderEligibleRedirectURL) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   const GURL url("https://docs.google.com/document");
   const GURL redirect_url("https://www.google.com");
   const std::string account_id = "0123456789";
   std::unique_ptr<net::URLRequest> url_request =
       url_request_context_.CreateRequest(url, net::DEFAULT_PRIORITY, nullptr,
                                          TRAFFIC_ANNOTATION_FOR_TESTS);
-  AppendOrRemoveAccountConsistentyRequestHeader(
-      url_request.get(), redirect_url, account_id, false /* sync_enabled */,
+  RequestAdapter request_adapter(url_request.get());
+  AppendOrRemoveMirrorRequestHeader(
+      &request_adapter, redirect_url, account_id, account_consistency_,
       cookie_settings_.get(), PROFILE_MODE_DEFAULT);
   EXPECT_TRUE(
       url_request->extra_request_headers().HasHeader(kChromeConnectedHeader));
@@ -347,16 +476,16 @@ TEST_F(SigninHeaderHelperTest, TestMirrorHeaderEligibleRedirectURL) {
 // Tests that the Mirror header request is stripped when the redirect URL is not
 // eligible.
 TEST_F(SigninHeaderHelperTest, TestMirrorHeaderNonEligibleRedirectURL) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   const GURL url("https://docs.google.com/document");
   const GURL redirect_url("http://www.foo.com");
   const std::string account_id = "0123456789";
   std::unique_ptr<net::URLRequest> url_request =
       url_request_context_.CreateRequest(url, net::DEFAULT_PRIORITY, nullptr,
                                          TRAFFIC_ANNOTATION_FOR_TESTS);
-  AppendOrRemoveAccountConsistentyRequestHeader(
-      url_request.get(), redirect_url, account_id, false /* sync_enabled */,
+  RequestAdapter request_adapter(url_request.get());
+  AppendOrRemoveMirrorRequestHeader(
+      &request_adapter, redirect_url, account_id, account_consistency_,
       cookie_settings_.get(), PROFILE_MODE_DEFAULT);
   EXPECT_FALSE(
       url_request->extra_request_headers().HasHeader(kChromeConnectedHeader));
@@ -365,8 +494,7 @@ TEST_F(SigninHeaderHelperTest, TestMirrorHeaderNonEligibleRedirectURL) {
 // Tests that the Mirror header, whatever its value is, is untouched when both
 // the current and the redirect URL are non-eligible.
 TEST_F(SigninHeaderHelperTest, TestIgnoreMirrorHeaderNonEligibleURLs) {
-  switches::EnableAccountConsistencyMirrorForTesting(
-      base::CommandLine::ForCurrentProcess());
+  account_consistency_ = AccountConsistencyMethod::kMirror;
   const GURL url("https://www.bar.com");
   const GURL redirect_url("http://www.foo.com");
   const std::string account_id = "0123456789";
@@ -376,8 +504,9 @@ TEST_F(SigninHeaderHelperTest, TestIgnoreMirrorHeaderNonEligibleURLs) {
                                          TRAFFIC_ANNOTATION_FOR_TESTS);
   url_request->SetExtraRequestHeaderByName(kChromeConnectedHeader, fake_header,
                                            false);
-  AppendOrRemoveAccountConsistentyRequestHeader(
-      url_request.get(), redirect_url, account_id, false /* sync_enabled */,
+  RequestAdapter request_adapter(url_request.get());
+  AppendOrRemoveMirrorRequestHeader(
+      &request_adapter, redirect_url, account_id, account_consistency_,
       cookie_settings_.get(), PROFILE_MODE_DEFAULT);
   std::string header;
   EXPECT_TRUE(url_request->extra_request_headers().GetHeader(
