@@ -15,39 +15,32 @@
 #include "components/prefs/pref_service.h"
 #include "components/signin/core/browser/account_tracker_service.h"
 #include "components/signin/core/browser/gaia_cookie_manager_service.h"
-#include "components/signin/core/browser/signin_client.h"
+#include "components/signin/core/browser/identity_utils.h"
 #include "components/signin/core/browser/signin_internals_util.h"
 #include "components/signin/core/browser/signin_metrics.h"
-#include "components/signin/core/common/signin_pref_names.h"
+#include "components/signin/core/browser/signin_pref_names.h"
 #include "google_apis/gaia/gaia_auth_util.h"
 #include "google_apis/gaia/gaia_constants.h"
 #include "google_apis/gaia/gaia_urls.h"
 #include "google_apis/gaia/google_service_auth_error.h"
 #include "third_party/icu/source/i18n/unicode/regex.h"
 
-using namespace signin_internals_util;
-
-SigninManager::SigninManager(SigninClient* client,
-                             ProfileOAuth2TokenService* token_service,
-                             AccountTrackerService* account_tracker_service,
-                             GaiaCookieManagerService* cookie_manager_service)
-    : SigninManagerBase(client, account_tracker_service),
-      prohibit_signout_(false),
+SigninManager::SigninManager(
+    SigninClient* client,
+    ProfileOAuth2TokenService* token_service,
+    AccountTrackerService* account_tracker_service,
+    GaiaCookieManagerService* cookie_manager_service,
+    signin::AccountConsistencyMethod account_consistency)
+    : SigninManagerBase(client, token_service, account_tracker_service),
       type_(SIGNIN_TYPE_NONE),
-      client_(client),
-      token_service_(token_service),
       cookie_manager_service_(cookie_manager_service),
+      account_consistency_(account_consistency),
       signin_manager_signed_in_(false),
       user_info_fetched_by_account_tracker_(false),
       weak_pointer_factory_(this) {}
 
 SigninManager::~SigninManager() {}
 
-void SigninManager::InitTokenService() {
-  if (token_service_)
-    token_service_->LoadCredentials(GetAuthenticatedAccountId());
-}
-
 std::string SigninManager::SigninTypeToString(SigninManager::SigninType type) {
   switch (type) {
     case SIGNIN_TYPE_NONE:
@@ -92,7 +85,6 @@ bool SigninManager::PrepareForSignin(SigninType type,
   password_.assign(password);
   signin_manager_signed_in_ = false;
   user_info_fetched_by_account_tracker_ = false;
-  NotifyDiagnosticsObservers(SIGNIN_STARTED, SigninTypeToString(type));
   return true;
 }
 
@@ -101,7 +93,7 @@ void SigninManager::StartSignInWithRefreshToken(
     const std::string& gaia_id,
     const std::string& username,
     const std::string& password,
-    const OAuthTokenFetchedCallback& callback) {
+    OAuthTokenFetchedCallback callback) {
   DCHECK(!IsAuthenticated());
   SigninType signin_type = refresh_token.empty()
                                ? SIGNIN_TYPE_WITHOUT_REFRESH_TOKEN
@@ -115,7 +107,7 @@ void SigninManager::StartSignInWithRefreshToken(
 
   if (!callback.is_null()) {
     // Callback present, let the caller complete the pending sign-in.
-    callback.Run(temp_refresh_token_);
+    std::move(callback).Run(temp_refresh_token_);
   } else {
     // No callback, so just complete the pending signin.
     CompletePendingSignin();
@@ -129,7 +121,7 @@ void SigninManager::CopyCredentialsFrom(const SigninManager& source) {
   possibly_invalid_email_ = source.possibly_invalid_email_;
   temp_refresh_token_ = source.temp_refresh_token_;
   password_ = source.password_;
-  source.client_->AfterCredentialsCopied();
+  source.signin_client()->AfterCredentialsCopied();
 }
 
 void SigninManager::ClearTransientSigninData() {
@@ -153,15 +145,43 @@ void SigninManager::HandleAuthError(const GoogleServiceAuthError& error) {
 void SigninManager::SignOut(
     signin_metrics::ProfileSignout signout_source_metric,
     signin_metrics::SignoutDelete signout_delete_metric) {
-  client_->PreSignOut(
-      base::Bind(&SigninManager::DoSignOut, base::Unretained(this),
-                 signout_source_metric, signout_delete_metric),
+  RemoveAccountsOption remove_option =
+      (account_consistency_ == signin::AccountConsistencyMethod::kDice)
+          ? RemoveAccountsOption::kRemoveAuthenticatedAccountIfInError
+          : RemoveAccountsOption::kRemoveAllAccounts;
+  StartSignOut(signout_source_metric, signout_delete_metric, remove_option);
+}
+
+void SigninManager::SignOutAndRemoveAllAccounts(
+    signin_metrics::ProfileSignout signout_source_metric,
+    signin_metrics::SignoutDelete signout_delete_metric) {
+  StartSignOut(signout_source_metric, signout_delete_metric,
+               RemoveAccountsOption::kRemoveAllAccounts);
+}
+
+void SigninManager::SignOutAndKeepAllAccounts(
+    signin_metrics::ProfileSignout signout_source_metric,
+    signin_metrics::SignoutDelete signout_delete_metric) {
+  StartSignOut(signout_source_metric, signout_delete_metric,
+               RemoveAccountsOption::kKeepAllAccounts);
+}
+
+void SigninManager::StartSignOut(
+    signin_metrics::ProfileSignout signout_source_metric,
+    signin_metrics::SignoutDelete signout_delete_metric,
+    RemoveAccountsOption remove_option) {
+  signin_client()->PreSignOut(
+      base::BindOnce(&SigninManager::OnSignoutDecisionReached,
+                     base::Unretained(this), signout_source_metric,
+                     signout_delete_metric, remove_option),
       signout_source_metric);
 }
 
-void SigninManager::DoSignOut(
+void SigninManager::OnSignoutDecisionReached(
     signin_metrics::ProfileSignout signout_source_metric,
-    signin_metrics::SignoutDelete signout_delete_metric) {
+    signin_metrics::SignoutDelete signout_delete_metric,
+    RemoveAccountsOption remove_option,
+    SigninClient::SignoutDecision signout_decision) {
   DCHECK(IsInitialized());
 
   signin_metrics::LogSignout(signout_source_metric, signout_delete_metric);
@@ -181,46 +201,61 @@ void SigninManager::DoSignOut(
     return;
   }
 
-  if (prohibit_signout_) {
-    DVLOG(1) << "Ignoring attempt to sign out while signout is prohibited";
+  // TODO(crbug.com/887756): Consider moving this higher up, or document why
+  // the above blocks are exempt from the |signout_decision| early return.
+  if (signout_decision == SigninClient::SignoutDecision::DISALLOW_SIGNOUT) {
+    DVLOG(1) << "Ignoring attempt to sign out while signout disallowed";
     return;
   }
 
   ClearTransientSigninData();
 
+  AccountInfo account_info = GetAuthenticatedAccountInfo();
   const std::string account_id = GetAuthenticatedAccountId();
-  const std::string username = GetAuthenticatedAccountInfo().email;
+  const std::string username = account_info.email;
   const base::Time signin_time =
-      base::Time::FromInternalValue(
-          client_->GetPrefs()->GetInt64(prefs::kSignedInTime));
-  clear_authenticated_user();
-  client_->GetPrefs()->ClearPref(prefs::kGoogleServicesHostedDomain);
-  client_->GetPrefs()->ClearPref(prefs::kGoogleServicesAccountId);
-  client_->GetPrefs()->ClearPref(prefs::kGoogleServicesUserAccountId);
-  client_->GetPrefs()->ClearPref(prefs::kSignedInTime);
-  client_->SignOut();
+      base::Time::FromDeltaSinceWindowsEpoch(base::TimeDelta::FromMicroseconds(
+          signin_client()->GetPrefs()->GetInt64(prefs::kSignedInTime)));
+  ClearAuthenticatedAccountId();
+  signin_client()->GetPrefs()->ClearPref(prefs::kGoogleServicesHostedDomain);
+  signin_client()->GetPrefs()->ClearPref(prefs::kGoogleServicesAccountId);
+  signin_client()->GetPrefs()->ClearPref(prefs::kGoogleServicesUserAccountId);
+  signin_client()->GetPrefs()->ClearPref(prefs::kSignedInTime);
 
   // Determine the duration the user was logged in and log that to UMA.
   if (!signin_time.is_null()) {
     base::TimeDelta signed_in_duration = base::Time::Now() - signin_time;
-    UMA_HISTOGRAM_COUNTS("Signin.SignedInDurationBeforeSignout",
-                         signed_in_duration.InMinutes());
+    UMA_HISTOGRAM_COUNTS_1M("Signin.SignedInDurationBeforeSignout",
+                            signed_in_duration.InMinutes());
   }
 
   // Revoke all tokens before sending signed_out notification, because there
   // may be components that don't listen for token service events when the
   // profile is not connected to an account.
-  LOG(WARNING) << "Revoking refresh token on server. Reason: sign out, "
-               << "IsSigninAllowed: " << IsSigninAllowed();
-  token_service_->RevokeAllCredentials();
+  switch (remove_option) {
+    case RemoveAccountsOption::kRemoveAllAccounts:
+      VLOG(0) << "Revoking all refresh tokens on server. Reason: sign out, "
+              << "IsSigninAllowed: " << IsSigninAllowed();
+      token_service()->RevokeAllCredentials(
+          signin_metrics::SourceForRefreshTokenOperation::
+              kSigninManager_ClearPrimaryAccount);
+      break;
+    case RemoveAccountsOption::kRemoveAuthenticatedAccountIfInError:
+      if (token_service()->RefreshTokenHasError(account_id))
+        token_service()->RevokeCredentials(
+            account_id, signin_metrics::SourceForRefreshTokenOperation::
+                            kSigninManager_ClearPrimaryAccount);
+      break;
+    case RemoveAccountsOption::kKeepAllAccounts:
+      // Do nothing.
+      break;
+  }
 
-  for (auto& observer : observer_list_)
-    observer.GoogleSignedOut(account_id, username);
+  FireGoogleSignedOut(account_info);
 }
 
-void SigninManager::Initialize(PrefService* local_state) {
-  SigninManagerBase::Initialize(local_state);
-
+void SigninManager::FinalizeInitBeforeLoadingRefreshTokens(
+    PrefService* local_state) {
   // local_state can be null during unit tests.
   if (local_state) {
     local_state_pref_registrar_.Init(local_state);
@@ -229,31 +264,45 @@ void SigninManager::Initialize(PrefService* local_state) {
         base::Bind(&SigninManager::OnGoogleServicesUsernamePatternChanged,
                    weak_pointer_factory_.GetWeakPtr()));
   }
-  signin_allowed_.Init(prefs::kSigninAllowed,
-                       client_->GetPrefs(),
+  signin_allowed_.Init(prefs::kSigninAllowed, signin_client()->GetPrefs(),
                        base::Bind(&SigninManager::OnSigninAllowedPrefChanged,
                                   base::Unretained(this)));
 
   std::string account_id =
-      client_->GetPrefs()->GetString(prefs::kGoogleServicesAccountId);
+      signin_client()->GetPrefs()->GetString(prefs::kGoogleServicesAccountId);
   std::string user = account_id.empty() ? std::string() :
       account_tracker_service()->GetAccountInfo(account_id).email;
-  if ((!account_id.empty() && !IsAllowedUsername(user)) || !IsSigninAllowed()) {
-    // User is signed in, but the username is invalid - the administrator must
-    // have changed the policy since the last signin, so sign out the user.
-    SignOut(signin_metrics::SIGNIN_PREF_CHANGED_DURING_SIGNIN,
-            signin_metrics::SignoutDelete::IGNORE_METRIC);
+  if (!account_id.empty() && (!IsAllowedUsername(user) || !IsSigninAllowed())) {
+    // User is signed in, but the username is invalid or signin is no longer
+    // allowed, so the user must be sign out.
+    //
+    // This may happen in the following cases:
+    //   a. The user has toggled off signin allowed in settings.
+    //   b. The administrator changed the policy since the last signin.
+    //
+    // Note: The token service has not yet loaded its credentials, so accounts
+    // cannot be revoked here.
+    //
+    // On desktop, when SigninManager is initializing, the profile was not yet
+    // marked with sign out allowed. Therefore sign out is not allowed and all
+    // calls to SignOut methods are no-op.
+    //
+    // TODO(msarda): SignOut methods do not gurantee that sign out can actually
+    // be done (this depends on whether sign out is allowed). Add a check here
+    // on desktop to make it clear that SignOut does not do anything.
+    SignOutAndKeepAllAccounts(signin_metrics::SIGNIN_PREF_CHANGED_DURING_SIGNIN,
+                              signin_metrics::SignoutDelete::IGNORE_METRIC);
   }
 
-  if (account_tracker_service()->GetMigrationState() ==
-      AccountTrackerService::MIGRATION_IN_PROGRESS) {
-    token_service_->AddObserver(this);
-  }
-  InitTokenService();
   account_tracker_service()->AddObserver(this);
+
+  // It is important to only load credentials after starting to observe the
+  // token service.
+  token_service()->AddObserver(this);
 }
 
 void SigninManager::Shutdown() {
+  token_service()->RemoveObserver(this);
   account_tracker_service()->RemoveObserver(this);
   local_state_pref_registrar_.RemoveAll();
   SigninManagerBase::Shutdown();
@@ -273,6 +322,10 @@ bool SigninManager::IsSigninAllowed() const {
   return signin_allowed_.GetValue();
 }
 
+void SigninManager::SetSigninAllowed(bool allowed) {
+  signin_allowed_.SetValue(allowed);
+}
+
 void SigninManager::OnSigninAllowedPrefChanged() {
   if (!IsSigninAllowed() && (IsAuthenticated() || AuthInProgress()))
     SignOut(signin_metrics::SIGNOUT_PREF_CHANGED,
@@ -280,36 +333,9 @@ void SigninManager::OnSigninAllowedPrefChanged() {
 }
 
 // static
-bool SigninManager::IsUsernameAllowedByPolicy(const std::string& username,
-                                              const std::string& policy) {
-  if (policy.empty())
-    return true;
-
-  // Patterns like "*@foo.com" are not accepted by our regex engine (since they
-  // are not valid regular expressions - they should instead be ".*@foo.com").
-  // For convenience, detect these patterns and insert a "." character at the
-  // front.
-  base::string16 pattern = base::UTF8ToUTF16(policy);
-  if (pattern[0] == L'*')
-    pattern.insert(pattern.begin(), L'.');
-
-  // See if the username matches the policy-provided pattern.
-  UErrorCode status = U_ZERO_ERROR;
-  const icu::UnicodeString icu_pattern(FALSE, pattern.data(), pattern.length());
-  icu::RegexMatcher matcher(icu_pattern, UREGEX_CASE_INSENSITIVE, status);
-  if (!U_SUCCESS(status)) {
-    LOG(ERROR) << "Invalid login regex: " << pattern << ", status: " << status;
-    // If an invalid pattern is provided, then prohibit *all* logins (better to
-    // break signin than to quietly allow users to sign in).
-    return false;
-  }
-  // The default encoding is UTF-8 in Chromium's ICU.
-  icu::UnicodeString icu_input(username.data());
-  matcher.reset(icu_input);
-  status = U_ZERO_ERROR;
-  UBool match = matcher.matches(status);
-  DCHECK(U_SUCCESS(status));
-  return !!match;  // !! == convert from UBool to bool.
+SigninManager* SigninManager::FromSigninManagerBase(
+    SigninManagerBase* manager) {
+  return static_cast<SigninManager*>(manager);
 }
 
 bool SigninManager::IsAllowedUsername(const std::string& username) const {
@@ -319,7 +345,7 @@ bool SigninManager::IsAllowedUsername(const std::string& username) const {
 
   std::string pattern =
       local_state->GetString(prefs::kGoogleServicesUsernamePattern);
-  return IsUsernameAllowedByPolicy(username, pattern);
+  return identity::IsUsernameAllowedByPattern(username, pattern);
 }
 
 bool SigninManager::AuthInProgress() const {
@@ -330,27 +356,26 @@ const std::string& SigninManager::GetAccountIdForAuthInProgress() const {
   return possibly_invalid_account_id_;
 }
 
+const std::string& SigninManager::GetGaiaIdForAuthInProgress() const {
+  return possibly_invalid_gaia_id_;
+}
+
 const std::string& SigninManager::GetUsernameForAuthInProgress() const {
   return possibly_invalid_email_;
 }
 
-void SigninManager::DisableOneClickSignIn(PrefService* prefs) {
-  prefs->SetBoolean(prefs::kReverseAutologinEnabled, false);
-}
-
 void SigninManager::MergeSigninCredentialIntoCookieJar() {
-  if (!client_->ShouldMergeSigninCredentialsIntoCookieJar())
+  if (account_consistency_ == signin::AccountConsistencyMethod::kMirror)
     return;
 
   if (!IsAuthenticated())
     return;
 
   cookie_manager_service_->AddAccountToCookie(GetAuthenticatedAccountId(),
-                                              "ChromiumSigninManager");
+                                              gaia::GaiaSource::kSigninManager);
 }
 
 void SigninManager::CompletePendingSignin() {
-  NotifyDiagnosticsObservers(SIGNIN_COMPLETED, "Successful");
   DCHECK(!possibly_invalid_account_id_.empty());
   OnSignedIn();
 
@@ -358,7 +383,10 @@ void SigninManager::CompletePendingSignin() {
 
   if (!temp_refresh_token_.empty()) {
     std::string account_id = GetAuthenticatedAccountId();
-    token_service_->UpdateCredentials(account_id, temp_refresh_token_);
+    token_service()->UpdateCredentials(
+        account_id, temp_refresh_token_,
+        signin_metrics::SourceForRefreshTokenOperation::
+            kSigninManager_LegacyPreDiceSigninFlow);
     temp_refresh_token_.clear();
   }
   MergeSigninCredentialIntoCookieJar();
@@ -376,8 +404,12 @@ void SigninManager::OnExternalSigninCompleted(const std::string& username) {
 }
 
 void SigninManager::OnSignedIn() {
-  client_->GetPrefs()->SetInt64(prefs::kSignedInTime,
-                                base::Time::Now().ToInternalValue());
+  bool reauth_in_progress = IsAuthenticated();
+
+  signin_client()->GetPrefs()->SetInt64(
+      prefs::kSignedInTime,
+      base::Time::Now().ToDeltaSinceWindowsEpoch().InMicroseconds());
+
   SetAuthenticatedAccountInfo(possibly_invalid_gaia_id_,
                               possibly_invalid_email_);
   const std::string gaia_id = possibly_invalid_gaia_id_;
@@ -387,32 +419,35 @@ void SigninManager::OnSignedIn() {
   possibly_invalid_email_.clear();
   signin_manager_signed_in_ = true;
 
-  for (auto& observer : observer_list_) {
-    observer.GoogleSigninSucceeded(GetAuthenticatedAccountId(),
-                                   GetAuthenticatedAccountInfo().email);
+  if (!reauth_in_progress)
+    FireGoogleSigninSucceeded();
 
-    observer.GoogleSigninSucceededWithPassword(
-        GetAuthenticatedAccountId(), GetAuthenticatedAccountInfo().email,
-        password_);
-  }
-
-  client_->OnSignedIn(GetAuthenticatedAccountId(), gaia_id,
-                      GetAuthenticatedAccountInfo().email, password_);
-
-  signin_metrics::LogSigninProfile(client_->IsFirstRun(),
-                                   client_->GetInstallDate());
-
-  DisableOneClickSignIn(client_->GetPrefs());  // Don't ever offer again.
+  signin_metrics::LogSigninProfile(signin_client()->IsFirstRun(),
+                                   signin_client()->GetInstallDate());
 
   PostSignedIn();
 }
 
+void SigninManager::FireGoogleSigninSucceeded() {
+  const AccountInfo account_info = GetAuthenticatedAccountInfo();
+  for (auto& observer : observer_list_) {
+    observer.GoogleSigninSucceeded(account_info);
+    observer.GoogleSigninSucceededWithPassword(account_info, password_);
+  }
+}
+
+void SigninManager::FireGoogleSignedOut(const AccountInfo& account_info) {
+  for (auto& observer : observer_list_) {
+    observer.GoogleSignedOut(account_info);
+  }
+}
+
 void SigninManager::PostSignedIn() {
   if (!signin_manager_signed_in_ || !user_info_fetched_by_account_tracker_)
     return;
 
-  client_->PostSignedIn(GetAuthenticatedAccountId(),
-                        GetAuthenticatedAccountInfo().email, password_);
+  signin_client()->PostSignedIn(GetAuthenticatedAccountId(),
+                                GetAuthenticatedAccountInfo().email, password_);
   password_.clear();
 }
 
@@ -430,15 +465,24 @@ void SigninManager::OnAccountUpdateFailed(const std::string& account_id) {
 }
 
 void SigninManager::OnRefreshTokensLoaded() {
+  token_service()->RemoveObserver(this);
+
   if (account_tracker_service()->GetMigrationState() ==
       AccountTrackerService::MIGRATION_IN_PROGRESS) {
     account_tracker_service()->SetMigrationDone();
-    token_service_->RemoveObserver(this);
+  }
+
+  // Remove account information from the account tracker service if needed.
+  if (token_service()->HasLoadCredentialsFinishedWithNoErrors()) {
+    std::vector<AccountInfo> accounts_in_tracker_service =
+        account_tracker_service()->GetAccounts();
+    for (const auto& account : accounts_in_tracker_service) {
+      if (GetAuthenticatedAccountId() != account.account_id &&
+          !token_service()->RefreshTokenIsAvailable(account.account_id)) {
+        DVLOG(0) << "Removed account from account tracker service: "
+                 << account.account_id;
+        account_tracker_service()->RemoveAccount(account.account_id);
+      }
+    }
   }
 }
-
-void SigninManager::ProhibitSignout(bool prohibit_signout) {
-  prohibit_signout_ = prohibit_signout;
-}
-
-bool SigninManager::IsSignoutProhibited() const { return prohibit_signout_; }
