@@ -10,9 +10,10 @@
 #import "base/mac/foundation_util.h"
 #include "base/macros.h"
 #include "base/strings/sys_string_conversions.h"
-#include "components/google/core/browser/google_util.h"
-#include "components/pref_registry/pref_registry_syncable.h"
+#include "components/google/core/common/google_util.h"
+#include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/scoped_user_pref_update.h"
+#include "components/signin/core/browser/account_consistency_method.h"
 #include "components/signin/core/browser/account_reconcilor.h"
 #include "components/signin/core/browser/signin_client.h"
 #include "components/signin/core/browser/signin_header_helper.h"
@@ -56,7 +57,8 @@ class AccountConsistencyHandler : public web::WebStatePolicyDecider {
 
  private:
   // web::WebStatePolicyDecider override
-  bool ShouldAllowResponse(NSURLResponse* response) override;
+  bool ShouldAllowResponse(NSURLResponse* response,
+                           bool for_main_frame) override;
   void WebStateDestroyed() override;
 
   AccountConsistencyService* account_consistency_service_;  // Weak.
@@ -75,7 +77,8 @@ AccountConsistencyHandler::AccountConsistencyHandler(
       account_reconcilor_(account_reconcilor),
       delegate_(delegate) {}
 
-bool AccountConsistencyHandler::ShouldAllowResponse(NSURLResponse* response) {
+bool AccountConsistencyHandler::ShouldAllowResponse(NSURLResponse* response,
+                                                    bool for_main_frame) {
   NSHTTPURLResponse* http_response =
       base::mac::ObjCCast<NSHTTPURLResponse>(response);
   if (!http_response)
@@ -196,35 +199,44 @@ AccountConsistencyService::CookieRequest::CreateAddCookieRequest(
 
 AccountConsistencyService::CookieRequest
 AccountConsistencyService::CookieRequest::CreateRemoveCookieRequest(
-    const std::string& domain) {
+    const std::string& domain,
+    base::OnceClosure callback) {
   AccountConsistencyService::CookieRequest cookie_request;
   cookie_request.request_type = REMOVE_CHROME_CONNECTED_COOKIE;
   cookie_request.domain = domain;
+  cookie_request.callback = std::move(callback);
   return cookie_request;
 }
 
+AccountConsistencyService::CookieRequest::CookieRequest() = default;
+
+AccountConsistencyService::CookieRequest::~CookieRequest() = default;
+
+AccountConsistencyService::CookieRequest::CookieRequest(
+    AccountConsistencyService::CookieRequest&&) = default;
+
 AccountConsistencyService::AccountConsistencyService(
     web::BrowserState* browser_state,
     AccountReconcilor* account_reconcilor,
     scoped_refptr<content_settings::CookieSettings> cookie_settings,
     GaiaCookieManagerService* gaia_cookie_manager_service,
     SigninClient* signin_client,
-    SigninManager* signin_manager)
+    identity::IdentityManager* identity_manager)
     : browser_state_(browser_state),
       account_reconcilor_(account_reconcilor),
       cookie_settings_(cookie_settings),
       gaia_cookie_manager_service_(gaia_cookie_manager_service),
       signin_client_(signin_client),
-      signin_manager_(signin_manager),
+      identity_manager_(identity_manager),
       applying_cookie_requests_(false) {
   gaia_cookie_manager_service_->AddObserver(this);
-  signin_manager_->AddObserver(this);
-  web::BrowserState::GetActiveStateManager(browser_state_)->AddObserver(this);
+  identity_manager_->AddObserver(this);
+  ActiveStateManager::FromBrowserState(browser_state_)->AddObserver(this);
   LoadFromPrefs();
-  if (signin_manager_->IsAuthenticated()) {
+  if (identity_manager_->HasPrimaryAccount()) {
     AddChromeConnectedCookies();
   } else {
-    RemoveChromeConnectedCookies();
+    RemoveChromeConnectedCookies(base::OnceClosure());
   }
 }
 
@@ -234,8 +246,7 @@ AccountConsistencyService::~AccountConsistencyService() {
 }
 
 // static
-void AccountConsistencyService::RegisterPrefs(
-    user_prefs::PrefRegistrySyncable* registry) {
+void AccountConsistencyService::RegisterPrefs(PrefRegistrySimple* registry) {
   registry->RegisterDictionaryPref(
       AccountConsistencyService::kDomainsWithCookiePref);
 }
@@ -271,6 +282,25 @@ bool AccountConsistencyService::ShouldAddChromeConnectedCookieToDomain(
          base::TimeDelta::FromHours(kHoursThresholdToReAddCookie);
 }
 
+void AccountConsistencyService::RemoveChromeConnectedCookies(
+    base::OnceClosure callback) {
+  DCHECK(!browser_state_->IsOffTheRecord());
+  if (last_cookie_update_map_.empty()) {
+    if (!callback.is_null())
+      std::move(callback).Run();
+    return;
+  }
+  std::map<std::string, base::Time> last_cookie_update_map =
+      last_cookie_update_map_;
+  auto iter_last_item = std::prev(last_cookie_update_map.end());
+  for (auto iter = last_cookie_update_map.begin(); iter != iter_last_item;
+       iter++) {
+    RemoveChromeConnectedCookieFromDomain(iter->first, base::OnceClosure());
+  }
+  RemoveChromeConnectedCookieFromDomain(iter_last_item->first,
+                                        std::move(callback));
+}
+
 void AccountConsistencyService::AddChromeConnectedCookieToDomain(
     const std::string& domain,
     bool force_update_if_too_old) {
@@ -284,13 +314,12 @@ void AccountConsistencyService::AddChromeConnectedCookieToDomain(
 }
 
 void AccountConsistencyService::RemoveChromeConnectedCookieFromDomain(
-    const std::string& domain) {
-  if (last_cookie_update_map_.count(domain) == 0) {
-    // Cookie is not on the domain. Nothing to do.
-    return;
-  }
+    const std::string& domain,
+    base::OnceClosure callback) {
+  DCHECK_NE(0ul, last_cookie_update_map_.count(domain));
   last_cookie_update_map_.erase(domain);
-  cookie_requests_.push_back(CookieRequest::CreateRemoveCookieRequest(domain));
+  cookie_requests_.push_back(
+      CookieRequest::CreateRemoveCookieRequest(domain, std::move(callback)));
   ApplyCookieRequests();
 }
 
@@ -304,9 +333,8 @@ void AccountConsistencyService::LoadFromPrefs() {
 
 void AccountConsistencyService::Shutdown() {
   gaia_cookie_manager_service_->RemoveObserver(this);
-  signin_manager_->RemoveObserver(this);
-  web::BrowserState::GetActiveStateManager(browser_state_)
-      ->RemoveObserver(this);
+  identity_manager_->RemoveObserver(this);
+  ActiveStateManager::FromBrowserState(browser_state_)->RemoveObserver(this);
   ResetWKWebView();
   web_state_handlers_.clear();
 }
@@ -320,7 +348,7 @@ void AccountConsistencyService::ApplyCookieRequests() {
   if (cookie_requests_.empty()) {
     return;
   }
-  if (!web::BrowserState::GetActiveStateManager(browser_state_)->IsActive()) {
+  if (!ActiveStateManager::FromBrowserState(browser_state_)->IsActive()) {
     // Web view usage isn't active for now, ignore cookie requests for now and
     // wait to be notified that it became active again.
     return;
@@ -335,8 +363,9 @@ void AccountConsistencyService::ApplyCookieRequests() {
   switch (cookie_requests_.front().request_type) {
     case ADD_CHROME_CONNECTED_COOKIE:
       cookie_value = signin::BuildMirrorRequestCookieIfPossible(
-          url, signin_manager_->GetAuthenticatedAccountInfo().gaia,
-          cookie_settings_.get(), signin::PROFILE_MODE_DEFAULT);
+          url, identity_manager_->GetPrimaryAccountInfo().gaia,
+          signin::AccountConsistencyMethod::kMirror, cookie_settings_.get(),
+          signin::PROFILE_MODE_DEFAULT);
       if (cookie_value.empty()) {
         // Don't add the cookie. Tentatively correct |last_cookie_update_map_|.
         last_cookie_update_map_.erase(cookie_requests_.front().domain);
@@ -364,8 +393,8 @@ void AccountConsistencyService::ApplyCookieRequests() {
 
 void AccountConsistencyService::FinishedApplyingCookieRequest(bool success) {
   DCHECK(!cookie_requests_.empty());
+  CookieRequest& request = cookie_requests_.front();
   if (success) {
-    const CookieRequest& request = cookie_requests_.front();
     DictionaryPrefUpdate update(
         signin_client_->GetPrefs(),
         AccountConsistencyService::kDomainsWithCookiePref);
@@ -373,7 +402,7 @@ void AccountConsistencyService::FinishedApplyingCookieRequest(bool success) {
       case ADD_CHROME_CONNECTED_COOKIE:
         // Add request.domain to prefs, use |true| as a dummy value (that is
         // never used), as the dictionary is used as a set.
-        update->SetBooleanWithoutPathExpansion(request.domain, true);
+        update->SetKey(request.domain, base::Value(true));
         break;
       case REMOVE_CHROME_CONNECTED_COOKIE:
         // Remove request.domain from prefs.
@@ -381,13 +410,17 @@ void AccountConsistencyService::FinishedApplyingCookieRequest(bool success) {
         break;
     }
   }
+  base::OnceClosure callback(std::move(request.callback));
   cookie_requests_.pop_front();
   applying_cookie_requests_ = false;
   ApplyCookieRequests();
+  if (!callback.is_null()) {
+    std::move(callback).Run();
+  }
 }
 
 WKWebView* AccountConsistencyService::GetWKWebView() {
-  if (!web::BrowserState::GetActiveStateManager(browser_state_)->IsActive()) {
+  if (!ActiveStateManager::FromBrowserState(browser_state_)->IsActive()) {
     // |browser_state_| is not active, WKWebView linked to this browser state
     // should not exist or be created.
     return nil;
@@ -425,26 +458,23 @@ void AccountConsistencyService::AddChromeConnectedCookies() {
                                    false /* force_update_if_too_old */);
 }
 
-void AccountConsistencyService::RemoveChromeConnectedCookies() {
-  DCHECK(!browser_state_->IsOffTheRecord());
-  std::map<std::string, base::Time> last_cookie_update_map =
-      last_cookie_update_map_;
-  for (const auto& domain : last_cookie_update_map) {
-    RemoveChromeConnectedCookieFromDomain(domain.first);
-  }
-}
-
 void AccountConsistencyService::OnBrowsingDataRemoved() {
   // CHROME_CONNECTED cookies have been removed, update internal state
   // accordingly.
   ResetWKWebView();
+  for (auto& cookie_request : cookie_requests_) {
+    base::OnceClosure callback(std::move(cookie_request.callback));
+    if (!callback.is_null()) {
+      std::move(callback).Run();
+    }
+  }
   cookie_requests_.clear();
   last_cookie_update_map_.clear();
   base::DictionaryValue dict;
   signin_client_->GetPrefs()->Set(kDomainsWithCookiePref, dict);
 
   // APISID cookie has been removed, notify the GCMS.
-  gaia_cookie_manager_service_->ForceOnCookieChangedProcessing();
+  gaia_cookie_manager_service_->ForceOnCookieChangeProcessing();
 }
 
 void AccountConsistencyService::OnAddAccountToCookieCompleted(
@@ -460,15 +490,16 @@ void AccountConsistencyService::OnGaiaAccountsInCookieUpdated(
   AddChromeConnectedCookies();
 }
 
-void AccountConsistencyService::GoogleSigninSucceeded(
-    const std::string& account_id,
-    const std::string& username) {
+void AccountConsistencyService::OnPrimaryAccountSet(
+    const AccountInfo& account_info) {
   AddChromeConnectedCookies();
 }
 
-void AccountConsistencyService::GoogleSignedOut(const std::string& account_id,
-                                                const std::string& username) {
-  RemoveChromeConnectedCookies();
+void AccountConsistencyService::OnPrimaryAccountCleared(
+    const AccountInfo& previous_account_info) {
+  // There is not need to remove CHROME_CONNECTED cookies on |GoogleSignedOut|
+  // events as these cookies will be removed by the GaiaCookieManagerServer
+  // right before fetching the Gaia logout request.
 }
 
 void AccountConsistencyService::OnActive() {
