@@ -4,14 +4,14 @@
 
 #include "components/ssl_errors/error_classification.h"
 
+#include <memory>
+
 #include "base/files/file_path.h"
-#include "base/memory/ptr_util.h"
-#include "base/message_loop/message_loop.h"
 #include "base/strings/string_split.h"
-#include "base/test/histogram_tester.h"
+#include "base/test/metrics/histogram_tester.h"
+#include "base/test/scoped_task_environment.h"
 #include "base/test/simple_test_clock.h"
 #include "base/test/simple_test_tick_clock.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/default_clock.h"
 #include "base/time/default_tick_clock.h"
 #include "components/network_time/network_time_test_utils.h"
@@ -25,7 +25,9 @@
 #include "net/test/embedded_test_server/http_response.h"
 #include "net/test/test_certificate_data.h"
 #include "net/test/test_data_directory.h"
-#include "net/url_request/url_request_test_util.h"
+#include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
+#include "services/network/test/test_shared_url_loader_factory.h"
+#include "services/network/test/test_url_loader_factory.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/gurl.h"
@@ -52,6 +54,9 @@ class SSLErrorClassificationTest : public ::testing::Test {
     return field_trial_test_.get();
   }
 
+ protected:
+  network::TestURLLoaderFactory test_url_loader_factory_;
+
  private:
   std::unique_ptr<network_time::FieldTrialTest> field_trial_test_;
 };
@@ -213,7 +218,7 @@ TEST_F(SSLErrorClassificationTest, TestPrivateURL) {
   EXPECT_TRUE(ssl_errors::IsHostnameNonUniqueOrDotless("foo.blah"));
 }
 
-TEST(ErrorClassification, LevenshteinDistance) {
+TEST_F(SSLErrorClassificationTest, LevenshteinDistance) {
   EXPECT_EQ(0u, ssl_errors::GetLevenshteinDistance("banana", "banana"));
 
   EXPECT_EQ(2u, ssl_errors::GetLevenshteinDistance("ab", "ba"));
@@ -247,12 +252,11 @@ TEST_F(SSLErrorClassificationTest, GetClockState) {
   histograms.ExpectTotalCount(kNetworkTimeHistogram, 0);
   TestingPrefServiceSimple pref_service;
   network_time::NetworkTimeTracker::RegisterPrefs(pref_service.registry());
-  base::MessageLoop loop;
   network_time::NetworkTimeTracker network_time_tracker(
-      base::MakeUnique<base::DefaultClock>(),
-      base::MakeUnique<base::DefaultTickClock>(), &pref_service,
-      new net::TestURLRequestContextGetter(
-          base::ThreadTaskRunnerHandle::Get()));
+      std::make_unique<base::DefaultClock>(),
+      std::make_unique<base::DefaultTickClock>(), &pref_service,
+      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
+          &test_url_loader_factory_));
 
   ssl_errors::SetBuildTimeForTesting(base::Time::Now());
   EXPECT_EQ(
@@ -358,10 +362,11 @@ TEST_F(SSLErrorClassificationTest, GetClockState) {
 // Tests that all possible NetworkClockState histogram values are recorded
 // appropriately.
 TEST_F(SSLErrorClassificationTest, NetworkClockStateHistogram) {
-  base::Thread io_thread("IO thread");
-  base::Thread::Options thread_options;
-  thread_options.message_loop_type = base::MessageLoop::TYPE_IO;
-  EXPECT_TRUE(io_thread.StartWithOptions(thread_options));
+  base::test::ScopedTaskEnvironment task_environment(
+      base::test::ScopedTaskEnvironment::MainThreadType::IO);
+
+  scoped_refptr<network::TestSharedURLLoaderFactory> shared_url_loader_factory =
+      base::MakeRefCounted<network::TestSharedURLLoaderFactory>();
 
   net::EmbeddedTestServer test_server;
   ASSERT_TRUE(test_server.InitializeAndListen());
@@ -376,12 +381,10 @@ TEST_F(SSLErrorClassificationTest, NetworkClockStateHistogram) {
   clock->Advance(base::TimeDelta::FromDays(111));
   tick_clock->Advance(base::TimeDelta::FromDays(222));
 
-  base::MessageLoop loop;
   network_time::NetworkTimeTracker network_time_tracker(
       std::unique_ptr<base::Clock>(clock),
-      std::unique_ptr<base::TickClock>(tick_clock), &pref_service,
-      new net::TestURLRequestContextGetter(io_thread.task_runner()));
-  network_time_tracker.SetTimeServerURLForTesting(test_server.GetURL("/"));
+      std::unique_ptr<const base::TickClock>(tick_clock), &pref_service,
+      shared_url_loader_factory);
   field_trial_test()->SetNetworkQueriesWithVariationsService(
       true, 0.0,
       network_time::NetworkTimeTracker::FETCHES_IN_BACKGROUND_AND_ON_DEMAND);
@@ -474,6 +477,4 @@ TEST_F(SSLErrorClassificationTest, NetworkClockStateHistogram) {
   histograms.ExpectBucketCount(
       kNetworkTimeHistogram, ssl_errors::NETWORK_CLOCK_STATE_UNKNOWN_SYNC_LOST,
       1);
-
-  io_thread.Stop();
 }
