@@ -9,30 +9,35 @@
 #include <tuple>
 #include <utility>
 
+#include "base/command_line.h"
 #include "base/logging.h"
-#include "base/memory/ptr_util.h"
-#include "base/message_loop/message_loop.h"
+#include "base/message_loop/message_loop_current.h"
 #include "base/run_loop.h"
 #include "base/strings/stringprintf.h"
-#include "base/test/histogram_tester.h"
+#include "base/test/metrics/histogram_tester.h"
+#include "base/test/scoped_feature_list.h"
 #include "base/test/test_simple_task_runner.h"
+#include "base/time/time.h"
 #include "components/subresource_filter/content/browser/async_document_subresource_filter.h"
+#include "components/subresource_filter/content/browser/subresource_filter_client.h"
 #include "components/subresource_filter/content/browser/subresource_filter_observer_manager.h"
 #include "components/subresource_filter/content/common/subresource_filter_messages.h"
-#include "components/subresource_filter/core/common/activation_level.h"
-#include "components/subresource_filter/core/common/activation_state.h"
+#include "components/subresource_filter/content/mojom/subresource_filter_agent.mojom.h"
+#include "components/subresource_filter/core/common/common_features.h"
 #include "components/subresource_filter/core/common/test_ruleset_creator.h"
 #include "components/subresource_filter/core/common/test_ruleset_utils.h"
+#include "components/subresource_filter/core/mojom/subresource_filter.mojom.h"
 #include "components/url_pattern_index/proto/rules.pb.h"
 #include "content/public/browser/navigation_handle.h"
 #include "content/public/browser/navigation_throttle.h"
 #include "content/public/browser/web_contents.h"
-#include "content/public/common/browser_side_navigation_policy.h"
 #include "content/public/test/mock_render_process_host.h"
 #include "content/public/test/navigation_simulator.h"
 #include "content/public/test/test_renderer_host.h"
+#include "content/public/test/test_utils.h"
 #include "net/base/net_errors.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/blink/public/common/associated_interfaces/associated_interface_provider.h"
 #include "url/url_constants.h"
 
 namespace subresource_filter {
@@ -53,6 +58,46 @@ enum PageActivationNotificationTiming {
   WILL_PROCESS_RESPONSE,
 };
 
+class FakeSubresourceFilterAgent : public mojom::SubresourceFilterAgent {
+ public:
+  FakeSubresourceFilterAgent() : binding_(this) {}
+  ~FakeSubresourceFilterAgent() override = default;
+
+  void OnSubresourceFilterAgentRequest(
+      mojo::ScopedInterfaceEndpointHandle handle) {
+    binding_.Bind(
+        mojo::AssociatedInterfaceRequest<mojom::SubresourceFilterAgent>(
+            std::move(handle)));
+  }
+
+  // mojom::SubresourceFilterAgent:
+  void ActivateForNextCommittedLoad(
+      mojom::ActivationStatePtr activation_state,
+      blink::mojom::AdFrameType ad_frame_type =
+          blink::mojom::AdFrameType::kNonAd) override {
+    last_activation_ = std::move(activation_state);
+    is_ad_subframe_ = ad_frame_type != blink::mojom::AdFrameType::kNonAd;
+  }
+
+  // These methods reset state back to default when they are called.
+  bool LastAdSubframe() {
+    bool is_ad_subframe = is_ad_subframe_;
+    is_ad_subframe_ = false;
+    return is_ad_subframe;
+  }
+  bool LastActivated() {
+    bool activated = last_activation_ && last_activation_->activation_level !=
+                                             mojom::ActivationLevel::kDisabled;
+    last_activation_.reset();
+    return activated;
+  }
+
+ private:
+  mojom::ActivationStatePtr last_activation_;
+  bool is_ad_subframe_ = false;
+  mojo::AssociatedBinding<mojom::SubresourceFilterAgent> binding_;
+};
+
 // Simple throttle that sends page-level activation to the manager for a
 // specific set of URLs.
 class MockPageStateActivationThrottle : public content::NavigationThrottle {
@@ -63,14 +108,17 @@ class MockPageStateActivationThrottle : public content::NavigationThrottle {
       : content::NavigationThrottle(navigation_handle),
         activation_throttle_state_(activation_throttle_state) {
     // Add some default activations.
-    mock_page_activations_[GURL(kTestURLWithActivation)] =
-        ActivationState(ActivationLevel::ENABLED);
-    mock_page_activations_[GURL(kTestURLWithActivation2)] =
-        ActivationState(ActivationLevel::ENABLED);
-    mock_page_activations_[GURL(kTestURLWithDryRun)] =
-        ActivationState(ActivationLevel::DRYRUN);
+    mojom::ActivationState enabled_state;
+    enabled_state.activation_level = mojom::ActivationLevel::kEnabled;
+
+    mojom::ActivationState dry_run_state;
+    dry_run_state.activation_level = mojom::ActivationLevel::kDryRun;
+
+    mock_page_activations_[GURL(kTestURLWithActivation)] = enabled_state;
+    mock_page_activations_[GURL(kTestURLWithActivation2)] = enabled_state;
+    mock_page_activations_[GURL(kTestURLWithDryRun)] = dry_run_state;
     mock_page_activations_[GURL(kTestURLWithNoActivation)] =
-        ActivationState(ActivationLevel::DISABLED);
+        mojom::ActivationState();
   }
   ~MockPageStateActivationThrottle() override {}
 
@@ -96,14 +144,13 @@ class MockPageStateActivationThrottle : public content::NavigationThrottle {
         // The throttle manager does not use the activation decision.
         SubresourceFilterObserverManager::FromWebContents(
             navigation_handle()->GetWebContents())
-            ->NotifyPageActivationComputed(
-                navigation_handle(), ActivationDecision::UNKNOWN, it->second);
+            ->NotifyPageActivationComputed(navigation_handle(), it->second);
       }
     }
     return content::NavigationThrottle::PROCEED;
   }
 
-  std::map<GURL, ActivationState> mock_page_activations_;
+  std::map<GURL, mojom::ActivationState> mock_page_activations_;
   PageActivationNotificationTiming activation_throttle_state_;
 
   DISALLOW_COPY_AND_ASSIGN(MockPageStateActivationThrottle);
@@ -112,16 +159,18 @@ class MockPageStateActivationThrottle : public content::NavigationThrottle {
 class ContentSubresourceFilterThrottleManagerTest
     : public content::RenderViewHostTestHarness,
       public content::WebContentsObserver,
-      public ContentSubresourceFilterThrottleManager::Delegate,
+      public SubresourceFilterClient,
       public ::testing::WithParamInterface<PageActivationNotificationTiming> {
  public:
-  ContentSubresourceFilterThrottleManagerTest()
-      : ContentSubresourceFilterThrottleManager::Delegate() {}
+  ContentSubresourceFilterThrottleManagerTest() {}
   ~ContentSubresourceFilterThrottleManagerTest() override {}
 
   // content::RenderViewHostTestHarness:
   void SetUp() override {
     content::RenderViewHostTestHarness::SetUp();
+    content::WebContents* web_contents =
+        RenderViewHostTestHarness::web_contents();
+    CreateAgentForHost(web_contents->GetMainFrame());
 
     NavigateAndCommit(GURL("https://example.first"));
 
@@ -137,16 +186,16 @@ class ContentSubresourceFilterThrottleManagerTest
     // Make the blocking task runner run on the current task runner for the
     // tests, to ensure that the NavigationSimulator properly runs all necessary
     // tasks while waiting for throttle checks to finish.
-    dealer_handle_ = base::MakeUnique<VerifiedRulesetDealer::Handle>(
-        base::MessageLoop::current()->task_runner());
-    dealer_handle_->SetRulesetFile(
-        testing::TestRuleset::Open(test_ruleset_pair_.indexed));
+    dealer_handle_ = std::make_unique<VerifiedRulesetDealer::Handle>(
+        base::MessageLoopCurrent::Get()->task_runner());
+    dealer_handle_->TryOpenAndSetRulesetFile(test_ruleset_pair_.indexed.path,
+                                             /*expected_checksum=*/0,
+                                             base::DoNothing());
 
     throttle_manager_ =
-        base::MakeUnique<ContentSubresourceFilterThrottleManager>(
-            this, dealer_handle_.get(),
-            RenderViewHostTestHarness::web_contents());
-    Observe(RenderViewHostTestHarness::web_contents());
+        std::make_unique<ContentSubresourceFilterThrottleManager>(
+            this, dealer_handle_.get(), web_contents);
+    Observe(web_contents);
   }
 
   void TearDown() override {
@@ -157,20 +206,15 @@ class ContentSubresourceFilterThrottleManagerTest
   }
 
   void ExpectActivationSignalForFrame(content::RenderFrameHost* rfh,
-                                      bool expect_activation) {
-    content::MockRenderProcessHost* render_process_host =
-        static_cast<content::MockRenderProcessHost*>(rfh->GetProcess());
-    const IPC::Message* message =
-        render_process_host->sink().GetFirstMessageMatching(
-            SubresourceFilterMsg_ActivateForNextCommittedLoad::ID);
-    ASSERT_EQ(expect_activation, !!message);
-    if (expect_activation) {
-      std::tuple<ActivationState> args;
-      SubresourceFilterMsg_ActivateForNextCommittedLoad::Read(message, &args);
-      ActivationLevel level = std::get<0>(args).activation_level;
-      EXPECT_NE(ActivationLevel::DISABLED, level);
-    }
-    render_process_host->sink().ClearMessages();
+                                      bool expect_activation,
+                                      bool expect_is_ad_subframe = false) {
+    // In some cases we need to verify that messages were _not_ sent, in which
+    // case using a Wait() idiom would cause hangs. RunUntilIdle instead to
+    // ensure mojo calls make it to the fake agent.
+    base::RunLoop().RunUntilIdle();
+    FakeSubresourceFilterAgent* agent = agent_map_[rfh].get();
+    EXPECT_EQ(expect_activation, agent->LastActivated());
+    EXPECT_EQ(expect_is_ad_subframe, agent->LastAdSubframe());
   }
 
   // Helper methods:
@@ -184,45 +228,47 @@ class ContentSubresourceFilterThrottleManagerTest
             url, render_frame_host);
   }
 
-  void CreateSubframeWithTestNavigation(const GURL& url,
-                                        content::RenderFrameHost* parent) {
+  content::RenderFrameHost* CreateSubframeWithTestNavigation(
+      const GURL& url,
+      content::RenderFrameHost* parent) {
     content::RenderFrameHost* subframe =
         content::RenderFrameHostTester::For(parent)->AppendChild(
             base::StringPrintf("subframe-%s", url.spec().c_str()));
     CreateTestNavigation(url, subframe);
+    return subframe;
   }
 
   void SimulateStartAndExpectResult(
-      content::NavigationThrottle::ThrottleCheckResult expect_result) {
+      content::NavigationThrottle::ThrottleAction expect_result) {
     navigation_simulator_->Start();
     content::NavigationThrottle::ThrottleCheckResult result =
         navigation_simulator_->GetLastThrottleCheckResult();
     EXPECT_EQ(expect_result, result);
-    if (result != content::NavigationThrottle::PROCEED)
+    if (result.action() != content::NavigationThrottle::PROCEED)
       navigation_simulator_.reset();
   }
 
   void SimulateRedirectAndExpectResult(
       const GURL& new_url,
-      content::NavigationThrottle::ThrottleCheckResult expect_result) {
+      content::NavigationThrottle::ThrottleAction expect_result) {
     navigation_simulator_->Redirect(new_url);
     content::NavigationThrottle::ThrottleCheckResult result =
         navigation_simulator_->GetLastThrottleCheckResult();
     EXPECT_EQ(expect_result, result);
-    if (result != content::NavigationThrottle::PROCEED)
+    if (result.action() != content::NavigationThrottle::PROCEED)
       navigation_simulator_.reset();
   }
 
   // Returns the RenderFrameHost that the navigation commit in.
   content::RenderFrameHost* SimulateCommitAndExpectResult(
-      content::NavigationThrottle::ThrottleCheckResult expect_result) {
+      content::NavigationThrottle::ThrottleAction expect_result) {
     navigation_simulator_->Commit();
     content::NavigationThrottle::ThrottleCheckResult result =
         navigation_simulator_->GetLastThrottleCheckResult();
     EXPECT_EQ(expect_result, result);
 
     auto scoped_simulator = std::move(navigation_simulator_);
-    if (result == content::NavigationThrottle::PROCEED)
+    if (result.action() == content::NavigationThrottle::PROCEED)
       return scoped_simulator->GetFinalRenderFrameHost();
     return nullptr;
   }
@@ -254,6 +300,14 @@ class ContentSubresourceFilterThrottleManagerTest
 
  protected:
   // content::WebContentsObserver
+  void RenderFrameCreated(content::RenderFrameHost* new_host) override {
+    CreateAgentForHost(new_host);
+  }
+
+  void FrameDeleted(content::RenderFrameHost* host) override {
+    agent_map_.erase(host);
+  }
+
   void DidStartNavigation(
       content::NavigationHandle* navigation_handle) override {
     if (navigation_handle->IsSameDocument())
@@ -265,7 +319,7 @@ class ContentSubresourceFilterThrottleManagerTest
         ::testing::UnitTest::GetInstance()->current_test_info()->value_param()
             ? GetParam()
             : WILL_PROCESS_RESPONSE;
-    throttles.push_back(base::MakeUnique<MockPageStateActivationThrottle>(
+    throttles.push_back(std::make_unique<MockPageStateActivationThrottle>(
         navigation_handle, state));
     throttle_manager_->MaybeAppendNavigationThrottles(navigation_handle,
                                                       &throttles);
@@ -274,9 +328,27 @@ class ContentSubresourceFilterThrottleManagerTest
     }
   }
 
-  // ContentSubresourceFilterThrottleManager::Delegate:
-  void OnFirstSubresourceLoadDisallowed() override {
-    ++disallowed_notification_count_;
+  void CreateAgentForHost(content::RenderFrameHost* host) {
+    auto new_agent = std::make_unique<FakeSubresourceFilterAgent>();
+    host->GetRemoteAssociatedInterfaces()->OverrideBinderForTesting(
+        mojom::SubresourceFilterAgent::Name_,
+        base::BindRepeating(
+            &FakeSubresourceFilterAgent::OnSubresourceFilterAgentRequest,
+            base::Unretained(new_agent.get())));
+    agent_map_[host] = std::move(new_agent);
+  }
+
+  // SubresourceFilterClient:
+  void ShowNotification() override { ++disallowed_notification_count_; }
+  mojom::ActivationLevel OnPageActivationComputed(
+      content::NavigationHandle* navigation_handle,
+      mojom::ActivationLevel effective_activation_level,
+      ActivationDecision* decision) override {
+    return effective_activation_level;
+  }
+
+  ContentSubresourceFilterThrottleManager* throttle_manager() {
+    return throttle_manager_.get();
   }
 
  private:
@@ -287,6 +359,10 @@ class ContentSubresourceFilterThrottleManagerTest
 
   std::unique_ptr<ContentSubresourceFilterThrottleManager> throttle_manager_;
 
+  std::map<content::RenderFrameHost*,
+           std::unique_ptr<FakeSubresourceFilterAgent>>
+      agent_map_;
+
   std::unique_ptr<content::NavigationSimulator> navigation_simulator_;
 
   // Incremented on every OnFirstSubresourceLoadDisallowed call.
@@ -295,7 +371,7 @@ class ContentSubresourceFilterThrottleManagerTest
   DISALLOW_COPY_AND_ASSIGN(ContentSubresourceFilterThrottleManagerTest);
 };
 
-INSTANTIATE_TEST_CASE_P(PageActivationNotificationTiming,
+INSTANTIATE_TEST_CASE_P(,
                         ContentSubresourceFilterThrottleManagerTest,
                         ::testing::Values(WILL_START_REQUEST,
                                           WILL_PROCESS_RESPONSE));
@@ -316,6 +392,10 @@ TEST_P(ContentSubresourceFilterThrottleManagerTest,
 }
 
 TEST_P(ContentSubresourceFilterThrottleManagerTest, NoPageActivation) {
+  // This test assumes that we're not in DryRun mode.
+  base::test::ScopedFeatureList scoped_feature;
+  scoped_feature.InitAndDisableFeature(kAdTagging);
+
   // Commit a navigation that triggers page level activation.
   NavigateAndCommitMainFrame(GURL(kTestURLWithNoActivation));
   ExpectActivationSignalForFrame(main_rfh(), false /* expect_activation */);
@@ -342,7 +422,8 @@ TEST_P(ContentSubresourceFilterThrottleManagerTest,
   content::RenderFrameHost* child =
       SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
   // But it should still be activated.
-  ExpectActivationSignalForFrame(child, true /* expect_activation */);
+  ExpectActivationSignalForFrame(child, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
 
   EXPECT_EQ(0, disallowed_notification_count());
 }
@@ -358,12 +439,10 @@ TEST_P(ContentSubresourceFilterThrottleManagerTest,
   CreateSubframeWithTestNavigation(
       GURL("https://www.example.com/before-redirect.html"), main_rfh());
   SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
-  content::NavigationThrottle::ThrottleCheckResult expected_result =
-      content::IsBrowserSideNavigationEnabled()
-          ? content::NavigationThrottle::BLOCK_REQUEST_AND_COLLAPSE
-          : content::NavigationThrottle::CANCEL;
+  content::NavigationThrottle::ThrottleAction expected_action =
+      content::NavigationThrottle::BLOCK_REQUEST_AND_COLLAPSE;
   SimulateRedirectAndExpectResult(
-      GURL("https://www.example.com/disallowed.html"), expected_result);
+      GURL("https://www.example.com/disallowed.html"), expected_action);
 
   EXPECT_EQ(1, disallowed_notification_count());
 }
@@ -469,6 +548,10 @@ TEST_P(ContentSubresourceFilterThrottleManagerTest,
 
 TEST_P(ContentSubresourceFilterThrottleManagerTest,
        DoNotFilterForInactiveFrame) {
+  // This test assumes that we're not in DryRun mode.
+  base::test::ScopedFeatureList scoped_feature;
+  scoped_feature.InitAndDisableFeature(kAdTagging);
+
   NavigateAndCommitMainFrame(GURL("https://do-not-activate.html"));
   ExpectActivationSignalForFrame(main_rfh(), false /* expect_activation */);
 
@@ -515,6 +598,10 @@ TEST_P(ContentSubresourceFilterThrottleManagerTest, RulesetHandleRegeneration) {
 
 TEST_P(ContentSubresourceFilterThrottleManagerTest,
        SameSiteNavigation_RulesetGoesAway) {
+  // This test assumes that we're not in DryRun mode.
+  base::test::ScopedFeatureList scoped_feature;
+  scoped_feature.InitAndDisableFeature(kAdTagging);
+
   GURL same_site_inactive_url =
       GURL(base::StringPrintf("%sinactive.html", kTestURLWithActivation));
 
@@ -659,6 +746,10 @@ TEST_P(ContentSubresourceFilterThrottleManagerTest, ActivationPropagation2) {
 // Same-site navigations within a single RFH do not persist activation.
 TEST_P(ContentSubresourceFilterThrottleManagerTest,
        SameSiteNavigationStopsActivation) {
+  // This test assumes that we're not in DryRun mode.
+  base::test::ScopedFeatureList scoped_feature;
+  scoped_feature.InitAndDisableFeature(kAdTagging);
+
   NavigateAndCommitMainFrame(GURL(kTestURLWithActivation));
   ExpectActivationSignalForFrame(main_rfh(), true /* expect_activation */);
 
@@ -679,20 +770,27 @@ TEST_P(ContentSubresourceFilterThrottleManagerTest,
 }
 
 TEST_F(ContentSubresourceFilterThrottleManagerTest, LogActivation) {
+  // This test assumes that we're not in DryRun mode.
+  base::test::ScopedFeatureList scoped_feature;
+  scoped_feature.InitAndDisableFeature(kAdTagging);
+
   base::HistogramTester tester;
   const char kActivationStateHistogram[] =
       "SubresourceFilter.PageLoad.ActivationState";
   NavigateAndCommitMainFrame(GURL(kTestURLWithDryRun));
   tester.ExpectBucketCount(kActivationStateHistogram,
-                           static_cast<int>(ActivationLevel::DRYRUN), 1);
+                           static_cast<int>(mojom::ActivationLevel::kDryRun),
+                           1);
 
   NavigateAndCommitMainFrame(GURL(kTestURLWithNoActivation));
   tester.ExpectBucketCount(kActivationStateHistogram,
-                           static_cast<int>(ActivationLevel::DISABLED), 1);
+                           static_cast<int>(mojom::ActivationLevel::kDisabled),
+                           1);
 
   NavigateAndCommitMainFrame(GURL(kTestURLWithActivation));
   tester.ExpectBucketCount(kActivationStateHistogram,
-                           static_cast<int>(ActivationLevel::ENABLED), 1);
+                           static_cast<int>(mojom::ActivationLevel::kEnabled),
+                           1);
 
   // Navigate a subframe that is not filtered, but should still activate.
   CreateSubframeWithTestNavigation(GURL("https://whitelist.com"), main_rfh());
@@ -705,8 +803,223 @@ TEST_F(ContentSubresourceFilterThrottleManagerTest, LogActivation) {
   // Only those with page level activation do ruleset lookups.
   tester.ExpectTotalCount("SubresourceFilter.PageLoad.Activation.WallDuration",
                           2);
+  // The *.CPUDuration histograms are recorded only if base::ThreadTicks is
+  // supported.
   tester.ExpectTotalCount("SubresourceFilter.PageLoad.Activation.CPUDuration",
-                          2);
+                          base::ThreadTicks::IsSupported() ? 2 : 0);
+}
+
+// Check to make sure we don't send an IPC with the ad tag bit for ad frames
+// that are successfully filtered.
+TEST_P(ContentSubresourceFilterThrottleManagerTest,
+       ActivateMainFrameAndFilterSubframeNavigationTaggedAsAd) {
+  // Commit a navigation that triggers page level activation.
+  NavigateAndCommitMainFrame(GURL(kTestURLWithActivation));
+  ExpectActivationSignalForFrame(main_rfh(), true /* expect_activation */,
+                                 false /* is_ad_subframe */);
+
+  // A disallowed subframe navigation should be successfully filtered.
+  CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/disallowed.html"), main_rfh());
+
+  SimulateStartAndExpectResult(
+      content::NavigationThrottle::BLOCK_REQUEST_AND_COLLAPSE);
+
+  EXPECT_EQ(1, disallowed_notification_count());
+}
+
+// If the RenderFrame determines that the frame is an ad, then any navigation
+// for that frame should be considered an ad.
+TEST_P(ContentSubresourceFilterThrottleManagerTest,
+       SubframeNavigationTaggedAsAdByRenderer) {
+  NavigateAndCommitMainFrame(GURL(kTestURLWithDryRun));
+  ExpectActivationSignalForFrame(main_rfh(), true /* expect_activation */,
+                                 false /* is_ad_subframe */);
+
+  content::RenderFrameHost* subframe = CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/allowed.html"), main_rfh());
+
+  EXPECT_FALSE(throttle_manager()->IsFrameTaggedAsAdForTesting(subframe));
+  throttle_manager()->OnFrameIsAdSubframe(subframe);
+  EXPECT_TRUE(throttle_manager()->IsFrameTaggedAsAdForTesting(subframe));
+
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+
+  subframe =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  EXPECT_TRUE(subframe);
+
+  ExpectActivationSignalForFrame(subframe, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
+
+  // A non-ad navigation for the same frame should be considered an ad
+  // subframe as well.
+  CreateTestNavigation(GURL("https://example.com/allowed2.html"), subframe);
+  SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  ExpectActivationSignalForFrame(subframe, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
+}
+
+// If the RenderFrame determines that the frame is an ad, and the frame changes
+// processes, then the new frame host should still be considered an ad.
+TEST_P(ContentSubresourceFilterThrottleManagerTest,
+       AdTagCarriesAcrossProcesses) {
+  content::IsolateAllSitesForTesting(base::CommandLine::ForCurrentProcess());
+
+  NavigateAndCommitMainFrame(GURL(kTestURLWithDryRun));
+  ExpectActivationSignalForFrame(main_rfh(), true /* expect_activation */,
+                                 false /* is_ad_subframe */);
+
+  // Create a subframe to a different site. It will start as a same-process
+  // frame but transition to a cross-process frame just before commit (after
+  // the throttle has marked the frame as an ad.)
+  content::RenderFrameHost* initial_subframe = CreateSubframeWithTestNavigation(
+      GURL("https://www.example2.com/allowed.html"), main_rfh());
+
+  // Simulate the render process telling the manager that the frame is an ad.
+  throttle_manager()->OnFrameIsAdSubframe(initial_subframe);
+  EXPECT_TRUE(
+      throttle_manager()->IsFrameTaggedAsAdForTesting(initial_subframe));
+
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+
+  content::RenderFrameHost* final_subframe =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  EXPECT_TRUE(final_subframe);
+  EXPECT_NE(initial_subframe, final_subframe);
+
+  EXPECT_TRUE(throttle_manager()->IsFrameTaggedAsAdForTesting(final_subframe));
+  EXPECT_FALSE(
+      throttle_manager()->IsFrameTaggedAsAdForTesting(initial_subframe));
+  ExpectActivationSignalForFrame(final_subframe, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
+}
+
+// If the RenderFrame determines that the frame is an ad, then its child frames
+// should also be considered ads.
+TEST_P(ContentSubresourceFilterThrottleManagerTest,
+       GrandchildNavigationTaggedAsAdByRenderer) {
+  NavigateAndCommitMainFrame(GURL(kTestURLWithDryRun));
+  ExpectActivationSignalForFrame(main_rfh(), true /* expect_activation */,
+                                 false /* is_ad_subframe */);
+
+  // Create a subframe that's marked as an ad by the render process.
+  content::RenderFrameHost* subframe = CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/allowed.html"), main_rfh());
+
+  // Simulate the render process telling the manager that the frame is an ad.
+  throttle_manager()->OnFrameIsAdSubframe(subframe);
+
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+
+  subframe =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  ExpectActivationSignalForFrame(subframe, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
+
+  // Create a grandchild frame that is marked as an ad because its parent is.
+  content::RenderFrameHost* grandchild_frame = CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/foo/allowed.html"), subframe);
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+  grandchild_frame =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  ExpectActivationSignalForFrame(grandchild_frame, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
+  EXPECT_TRUE(
+      throttle_manager()->IsFrameTaggedAsAdForTesting(grandchild_frame));
+}
+
+TEST_P(ContentSubresourceFilterThrottleManagerTest,
+       DryRun_FrameTaggingDeleted) {
+  NavigateAndCommitMainFrame(GURL(kTestURLWithDryRun));
+  ExpectActivationSignalForFrame(main_rfh(), true /* expect_activation */);
+}
+
+TEST_P(ContentSubresourceFilterThrottleManagerTest,
+       DryRun_FrameTaggingAsAdPropagatesToChildFrame) {
+  NavigateAndCommitMainFrame(GURL(kTestURLWithDryRun));
+  ExpectActivationSignalForFrame(main_rfh(), true /* expect_activation */);
+
+  // A disallowed subframe navigation should not be filtered in dry-run mode.
+  CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/disallowed.html"), main_rfh());
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+  content::RenderFrameHost* child =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  EXPECT_TRUE(child);
+
+  // But it should still be activated.
+  ExpectActivationSignalForFrame(child, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
+  EXPECT_TRUE(throttle_manager()->IsFrameTaggedAsAdForTesting(child));
+
+  // Create a subframe which is allowed as per ruleset but should still be
+  // tagged as ad because of its parent.
+  CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/allowed_by_ruleset.html"), child);
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+  content::RenderFrameHost* grandchild =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  EXPECT_TRUE(grandchild);
+  ExpectActivationSignalForFrame(grandchild, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
+  EXPECT_TRUE(throttle_manager()->IsFrameTaggedAsAdForTesting(grandchild));
+
+  // Verify that a 2nd level nested frame should also be tagged.
+  CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/great_grandchild_allowed_by_ruleset.html"),
+      child);
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+  content::RenderFrameHost* greatGrandchild =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  EXPECT_TRUE(greatGrandchild);
+  ExpectActivationSignalForFrame(greatGrandchild, true /* expect_activation */,
+                                 true /* is_ad_subframe */);
+  EXPECT_TRUE(throttle_manager()->IsFrameTaggedAsAdForTesting(greatGrandchild));
+
+  EXPECT_EQ(0, disallowed_notification_count());
+}
+
+TEST_P(ContentSubresourceFilterThrottleManagerTest,
+       DryRun_AllowedFrameNotTaggedAsAd) {
+  NavigateAndCommitMainFrame(GURL(kTestURLWithDryRun));
+  ExpectActivationSignalForFrame(main_rfh(), true /* expect_activation */);
+
+  CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/allowed_by_ruleset.html"), main_rfh());
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+  content::RenderFrameHost* child =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  EXPECT_TRUE(child);
+  ExpectActivationSignalForFrame(child, true /* expect_activation */,
+                                 false /* is_ad_subframe */);
+  EXPECT_FALSE(throttle_manager()->IsFrameTaggedAsAdForTesting(child));
+
+  // Create a subframe which is allowed as per ruleset and should not be tagged
+  // as ad because its parent is not tagged as well.
+  CreateSubframeWithTestNavigation(
+      GURL("https://www.example.com/also_allowed_by_ruleset.html"), child);
+  SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
+  content::RenderFrameHost* grandchild =
+      SimulateCommitAndExpectResult(content::NavigationThrottle::PROCEED);
+  EXPECT_TRUE(grandchild);
+  ExpectActivationSignalForFrame(grandchild, true /* expect_activation */,
+                                 false /* is_ad_subframe */);
+  EXPECT_FALSE(throttle_manager()->IsFrameTaggedAsAdForTesting(grandchild));
+
+  EXPECT_EQ(0, disallowed_notification_count());
+}
+
+TEST_P(ContentSubresourceFilterThrottleManagerTest,
+       FirstDisallowedLoadCalledOutOfOrder) {
+  NavigateAndCommitMainFrame(GURL(kTestURLWithActivation));
+  NavigateAndCommitMainFrame(GURL(kTestURLWithNoActivation));
+
+  // Simulate the previous navigation sending an IPC that a load was disallowed.
+  // This could happen e.g. for cross-process navigations, which have no
+  // ordering guarantees.
+  throttle_manager()->DidDisallowFirstSubresource();
+  EXPECT_EQ(0, disallowed_notification_count());
 }
 
 // TODO(csharrison): Make sure the following conditions are exercised in tests:
