@@ -61,6 +61,13 @@ const char kProfileEverInitialized[] = "profile_ever_initialized";
 // attempted.
 const char kMinimalMigrationAttempted[] = "minimal_migration_attempted";
 
+// Key of the boolean flag telling if user session requires policy.
+const char kProfileRequiresPolicy[] = "profile_requires_policy";
+
+// Key of the boolean flag telling if user is ephemeral and should be removed
+// from the local state on logout.
+const char kIsEphemeral[] = "is_ephemeral";
+
 PrefService* GetLocalState() {
   if (!UserManager::IsInitialized())
     return nullptr;
@@ -135,9 +142,11 @@ bool FindPrefs(const AccountId& account_id,
     return false;
 
   // UserManager is usually NULL in unit tests.
-  if (UserManager::IsInitialized() &&
-      UserManager::Get()->IsUserNonCryptohomeDataEphemeral(account_id))
+  if (account_id.GetAccountType() != AccountType::ACTIVE_DIRECTORY &&
+      UserManager::IsInitialized() &&
+      UserManager::Get()->IsUserNonCryptohomeDataEphemeral(account_id)) {
     return false;
+  }
 
   const base::ListValue* known_users = local_state->GetList(kKnownUsers);
   for (size_t i = 0; i < known_users->GetSize(); ++i) {
@@ -162,9 +171,11 @@ void UpdatePrefs(const AccountId& account_id,
     return;
 
   // UserManager is usually NULL in unit tests.
-  if (UserManager::IsInitialized() &&
-      UserManager::Get()->IsUserNonCryptohomeDataEphemeral(account_id))
+  if (account_id.GetAccountType() != AccountType::ACTIVE_DIRECTORY &&
+      UserManager::IsInitialized() &&
+      UserManager::Get()->IsUserNonCryptohomeDataEphemeral(account_id)) {
     return;
+  }
 
   ListPrefUpdate update(local_state, kKnownUsers);
   for (size_t i = 0; i < update->GetSize(); ++i) {
@@ -394,6 +405,24 @@ void SetGaiaIdMigrationStatusDone(const AccountId& account_id,
                  true);
 }
 
+void SaveKnownUser(const AccountId& account_id) {
+  const bool is_ephemeral =
+      UserManager::IsInitialized() &&
+      UserManager::Get()->IsUserNonCryptohomeDataEphemeral(account_id);
+  if (is_ephemeral &&
+      account_id.GetAccountType() != AccountType::ACTIVE_DIRECTORY) {
+    return;
+  }
+  UpdateId(account_id);
+  GetLocalState()->CommitPendingWrite();
+}
+
+void SetIsEphemeralUser(const AccountId& account_id, bool is_ephemeral) {
+  if (account_id.GetAccountType() != AccountType::ACTIVE_DIRECTORY)
+    return;
+  SetBooleanPref(account_id, kIsEphemeral, is_ephemeral);
+}
+
 void UpdateGaiaID(const AccountId& account_id, const std::string& gaia_id) {
   SetStringPref(account_id, kGAIAIdKey, gaia_id);
   SetStringPref(account_id, kAccountTypeKey,
@@ -461,25 +490,33 @@ bool IsUsingSAML(const AccountId& account_id) {
 
 bool WasProfileEverInitialized(const AccountId& account_id) {
   bool profile_ever_initialized;
-  const bool pref_set = GetBooleanPref(account_id, kProfileEverInitialized,
-                                       &profile_ever_initialized);
-  // TODO(atwilson): Remove migration code below once this UMA stat reports
-  // that migration is completed - crbug.com/736760.
-  UMA_HISTOGRAM_BOOLEAN("UserManager.ProfileEverInitializedMigrationCompleted",
-                        pref_set);
-  if (pref_set)
+  if (GetBooleanPref(account_id, kProfileEverInitialized,
+                     &profile_ever_initialized)) {
     return profile_ever_initialized;
-
-  // Sessions created before we started setting the session_initialized flag
-  // should default to "initialized = true".
-  LOG(WARNING) << "Treating unmigrated user as profile_ever_initialized=true";
-  return true;
+  }
+  return false;
 }
 
 void SetProfileEverInitialized(const AccountId& account_id, bool initialized) {
   SetBooleanPref(account_id, kProfileEverInitialized, initialized);
 }
 
+void SetProfileRequiresPolicy(const AccountId& account_id,
+                              ProfileRequiresPolicy required) {
+  DCHECK_NE(required, ProfileRequiresPolicy::kUnknown);
+  SetBooleanPref(account_id, kProfileRequiresPolicy,
+                 required == ProfileRequiresPolicy::kPolicyRequired);
+}
+
+ProfileRequiresPolicy GetProfileRequiresPolicy(const AccountId& account_id) {
+  bool requires_policy;
+  if (GetBooleanPref(account_id, kProfileRequiresPolicy, &requires_policy)) {
+    return requires_policy ? ProfileRequiresPolicy::kPolicyRequired
+                           : ProfileRequiresPolicy::kNoPolicyRequired;
+  }
+  return ProfileRequiresPolicy::kUnknown;
+}
+
 void UpdateReauthReason(const AccountId& account_id, const int reauth_reason) {
   SetIntegerPref(account_id, kReauthReasonKey, reauth_reason);
 }
@@ -525,9 +562,29 @@ void RemovePrefs(const AccountId& account_id) {
   }
 }
 
-// Exported so tests can call this from other components.
-void RemovePrefsForTesting(const AccountId& account_id) {
-  RemovePrefs(account_id);
+void CleanEphemeralUsers() {
+  PrefService* local_state = GetLocalState();
+
+  // Local State may not be initialized in tests.
+  if (!local_state)
+    return;
+
+  ListPrefUpdate update(local_state, kKnownUsers);
+  auto& list_storage = update->GetList();
+  for (auto it = list_storage.begin(); it < list_storage.end();) {
+    bool remove = false;
+    base::DictionaryValue* element = nullptr;
+    if (update->GetDictionary(std::distance(list_storage.begin(), it),
+                              &element)) {
+      base::Value* is_ephemeral = element->FindKey(kIsEphemeral);
+      if (is_ephemeral && is_ephemeral->GetBool())
+        remove = true;
+    }
+    if (remove)
+      it = list_storage.erase(it);
+    else
+      it++;
+  }
 }
 
 void RegisterPrefs(PrefRegistrySimple* registry) {
