@@ -11,6 +11,7 @@
 #include <vector>
 
 #include "base/macros.h"
+#include "base/stl_util.h"
 #include "base/strings/string_split.h"
 #include "components/variations/client_filterable_state.h"
 #include "components/variations/processed_study.h"
@@ -35,26 +36,31 @@ Study::Experiment* AddExperiment(const std::string& name,
   return experiment;
 }
 
+std::vector<std::string> SplitFilterString(const std::string& input) {
+  return base::SplitString(input, ",", base::TRIM_WHITESPACE,
+                           base::SPLIT_WANT_ALL);
+}
+
 }  // namespace
 
 TEST(VariationsStudyFilteringTest, CheckStudyChannel) {
   const Study::Channel channels[] = {
       Study::CANARY, Study::DEV, Study::BETA, Study::STABLE,
   };
-  bool channel_added[arraysize(channels)] = { 0 };
+  bool channel_added[base::size(channels)] = {0};
 
   Study::Filter filter;
 
-  // Check in the forwarded order. The loop cond is <= arraysize(channels)
+  // Check in the forwarded order. The loop cond is <= base::size(channels)
   // instead of < so that the result of adding the last channel gets checked.
-  for (size_t i = 0; i <= arraysize(channels); ++i) {
-    for (size_t j = 0; j < arraysize(channels); ++j) {
+  for (size_t i = 0; i <= base::size(channels); ++i) {
+    for (size_t j = 0; j < base::size(channels); ++j) {
       const bool expected = channel_added[j] || filter.channel_size() == 0;
       const bool result = internal::CheckStudyChannel(filter, channels[j]);
       EXPECT_EQ(expected, result) << "Case " << i << "," << j << " failed!";
     }
 
-    if (i < arraysize(channels)) {
+    if (i < base::size(channels)) {
       filter.add_channel(channels[i]);
       channel_added[i] = true;
     }
@@ -63,15 +69,15 @@ TEST(VariationsStudyFilteringTest, CheckStudyChannel) {
   // Do the same check in the reverse order.
   filter.clear_channel();
   memset(&channel_added, 0, sizeof(channel_added));
-  for (size_t i = 0; i <= arraysize(channels); ++i) {
-    for (size_t j = 0; j < arraysize(channels); ++j) {
+  for (size_t i = 0; i <= base::size(channels); ++i) {
+    for (size_t j = 0; j < base::size(channels); ++j) {
       const bool expected = channel_added[j] || filter.channel_size() == 0;
       const bool result = internal::CheckStudyChannel(filter, channels[j]);
       EXPECT_EQ(expected, result) << "Case " << i << "," << j << " failed!";
     }
 
-    if (i < arraysize(channels)) {
-      const int index = arraysize(channels) - i - 1;
+    if (i < base::size(channels)) {
+      const int index = base::size(channels) - i - 1;
       filter.add_channel(channels[index]);
       channel_added[index] = true;
     }
@@ -84,13 +90,13 @@ TEST(VariationsStudyFilteringTest, CheckStudyFormFactor) {
   };
 
   ASSERT_EQ(Study::FormFactor_ARRAYSIZE,
-            static_cast<int>(arraysize(form_factors)));
+            static_cast<int>(base::size(form_factors)));
 
-  bool form_factor_added[arraysize(form_factors)] = { 0 };
+  bool form_factor_added[base::size(form_factors)] = {0};
   Study::Filter filter;
 
-  for (size_t i = 0; i <= arraysize(form_factors); ++i) {
-    for (size_t j = 0; j < arraysize(form_factors); ++j) {
+  for (size_t i = 0; i <= base::size(form_factors); ++i) {
+    for (size_t j = 0; j < base::size(form_factors); ++j) {
       const bool expected = form_factor_added[j] ||
                             filter.form_factor_size() == 0;
       const bool result = internal::CheckStudyFormFactor(filter,
@@ -99,7 +105,7 @@ TEST(VariationsStudyFilteringTest, CheckStudyFormFactor) {
                                   << " failed!";
     }
 
-    if (i < arraysize(form_factors)) {
+    if (i < base::size(form_factors)) {
       filter.add_form_factor(form_factors[i]);
       form_factor_added[i] = true;
     }
@@ -108,8 +114,8 @@ TEST(VariationsStudyFilteringTest, CheckStudyFormFactor) {
   // Do the same check in the reverse order.
   filter.clear_form_factor();
   memset(&form_factor_added, 0, sizeof(form_factor_added));
-  for (size_t i = 0; i <= arraysize(form_factors); ++i) {
-    for (size_t j = 0; j < arraysize(form_factors); ++j) {
+  for (size_t i = 0; i <= base::size(form_factors); ++i) {
+    for (size_t j = 0; j < base::size(form_factors); ++j) {
       const bool expected = form_factor_added[j] ||
                             filter.form_factor_size() == 0;
       const bool result = internal::CheckStudyFormFactor(filter,
@@ -118,8 +124,8 @@ TEST(VariationsStudyFilteringTest, CheckStudyFormFactor) {
                                   << " failed!";
     }
 
-    if (i < arraysize(form_factors)) {
-      const int index = arraysize(form_factors) - i - 1;
+    if (i < base::size(form_factors)) {
+      const int index = base::size(form_factors) - i - 1;
       filter.add_form_factor(form_factors[index]);
       form_factor_added[index] = true;
     }
@@ -127,9 +133,9 @@ TEST(VariationsStudyFilteringTest, CheckStudyFormFactor) {
 
   // Test exclude_form_factors, forward order.
   filter.clear_form_factor();
-  bool form_factor_excluded[arraysize(form_factors)] = { 0 };
-  for (size_t i = 0; i <= arraysize(form_factors); ++i) {
-    for (size_t j = 0; j < arraysize(form_factors); ++j) {
+  bool form_factor_excluded[base::size(form_factors)] = {0};
+  for (size_t i = 0; i <= base::size(form_factors); ++i) {
+    for (size_t j = 0; j < base::size(form_factors); ++j) {
       const bool expected = filter.exclude_form_factor_size() == 0 ||
                             !form_factor_excluded[j];
       const bool result = internal::CheckStudyFormFactor(filter,
@@ -138,7 +144,7 @@ TEST(VariationsStudyFilteringTest, CheckStudyFormFactor) {
                                   << j << " failed!";
     }
 
-    if (i < arraysize(form_factors)) {
+    if (i < base::size(form_factors)) {
       filter.add_exclude_form_factor(form_factors[i]);
       form_factor_excluded[i] = true;
     }
@@ -147,8 +153,8 @@ TEST(VariationsStudyFilteringTest, CheckStudyFormFactor) {
   // Test exclude_form_factors, reverse order.
   filter.clear_exclude_form_factor();
   memset(&form_factor_excluded, 0, sizeof(form_factor_excluded));
-  for (size_t i = 0; i <= arraysize(form_factors); ++i) {
-    for (size_t j = 0; j < arraysize(form_factors); ++j) {
+  for (size_t i = 0; i <= base::size(form_factors); ++i) {
+    for (size_t j = 0; j < base::size(form_factors); ++j) {
       const bool expected = filter.exclude_form_factor_size() == 0 ||
                             !form_factor_excluded[j];
       const bool result = internal::CheckStudyFormFactor(filter,
@@ -157,8 +163,8 @@ TEST(VariationsStudyFilteringTest, CheckStudyFormFactor) {
                                   << j << " failed!";
     }
 
-    if (i < arraysize(form_factors)) {
-      const int index = arraysize(form_factors) - i - 1;
+    if (i < base::size(form_factors)) {
+      const int index = base::size(form_factors) - i - 1;
       filter.add_exclude_form_factor(form_factors[index]);
       form_factor_excluded[index] = true;
     }
@@ -191,22 +197,16 @@ TEST(VariationsStudyFilteringTest, CheckStudyLocale) {
       {"", "fr-CA", true, true, true},
   };
 
-  for (size_t i = 0; i < arraysize(test_cases); ++i) {
+  for (const auto& test : test_cases) {
     Study::Filter filter;
-    for (const std::string& locale : base::SplitString(
-             test_cases[i].filter_locales, ",",
-             base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL))
+    for (const std::string& locale : SplitFilterString(test.filter_locales))
       filter.add_locale(locale);
-    for (const std::string& exclude_locale :
-         base::SplitString(test_cases[i].exclude_locales, ",",
-                           base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL))
-      filter.add_exclude_locale(exclude_locale);
-    EXPECT_EQ(test_cases[i].en_us_result,
-              internal::CheckStudyLocale(filter, "en-US"));
-    EXPECT_EQ(test_cases[i].en_ca_result,
-              internal::CheckStudyLocale(filter, "en-CA"));
-    EXPECT_EQ(test_cases[i].fr_result,
-              internal::CheckStudyLocale(filter, "fr"));
+    for (const std::string& locale : SplitFilterString(test.exclude_locales))
+      filter.add_exclude_locale(locale);
+
+    EXPECT_EQ(test.en_us_result, internal::CheckStudyLocale(filter, "en-US"));
+    EXPECT_EQ(test.en_ca_result, internal::CheckStudyLocale(filter, "en-CA"));
+    EXPECT_EQ(test.fr_result, internal::CheckStudyLocale(filter, "fr"));
   }
 }
 
@@ -215,23 +215,22 @@ TEST(VariationsStudyFilteringTest, CheckStudyPlatform) {
       Study::PLATFORM_WINDOWS,         Study::PLATFORM_MAC,
       Study::PLATFORM_LINUX,           Study::PLATFORM_CHROMEOS,
       Study::PLATFORM_ANDROID,         Study::PLATFORM_IOS,
-      Study::PLATFORM_ANDROID_WEBVIEW,
-  };
-  ASSERT_EQ(Study::Platform_ARRAYSIZE, static_cast<int>(arraysize(platforms)));
-  bool platform_added[arraysize(platforms)] = { 0 };
+      Study::PLATFORM_ANDROID_WEBVIEW, Study::PLATFORM_FUCHSIA};
+  ASSERT_EQ(Study::Platform_ARRAYSIZE, static_cast<int>(base::size(platforms)));
+  bool platform_added[base::size(platforms)] = {0};
 
   Study::Filter filter;
 
-  // Check in the forwarded order. The loop cond is <= arraysize(platforms)
+  // Check in the forwarded order. The loop cond is <= base::size(platforms)
   // instead of < so that the result of adding the last channel gets checked.
-  for (size_t i = 0; i <= arraysize(platforms); ++i) {
-    for (size_t j = 0; j < arraysize(platforms); ++j) {
+  for (size_t i = 0; i <= base::size(platforms); ++i) {
+    for (size_t j = 0; j < base::size(platforms); ++j) {
       const bool expected = platform_added[j] || filter.platform_size() == 0;
       const bool result = internal::CheckStudyPlatform(filter, platforms[j]);
       EXPECT_EQ(expected, result) << "Case " << i << "," << j << " failed!";
     }
 
-    if (i < arraysize(platforms)) {
+    if (i < base::size(platforms)) {
       filter.add_platform(platforms[i]);
       platform_added[i] = true;
     }
@@ -240,15 +239,15 @@ TEST(VariationsStudyFilteringTest, CheckStudyPlatform) {
   // Do the same check in the reverse order.
   filter.clear_platform();
   memset(&platform_added, 0, sizeof(platform_added));
-  for (size_t i = 0; i <= arraysize(platforms); ++i) {
-    for (size_t j = 0; j < arraysize(platforms); ++j) {
+  for (size_t i = 0; i <= base::size(platforms); ++i) {
+    for (size_t j = 0; j < base::size(platforms); ++j) {
       const bool expected = platform_added[j] || filter.platform_size() == 0;
       const bool result = internal::CheckStudyPlatform(filter, platforms[j]);
       EXPECT_EQ(expected, result) << "Case " << i << "," << j << " failed!";
     }
 
-    if (i < arraysize(platforms)) {
-      const int index = arraysize(platforms) - i - 1;
+    if (i < base::size(platforms)) {
+      const int index = base::size(platforms) - i - 1;
       filter.add_platform(platforms[index]);
       platform_added[index] = true;
     }
@@ -290,7 +289,7 @@ TEST(VariationsStudyFilteringTest, CheckStudyStartDate) {
   // Start date not set should result in true.
   EXPECT_TRUE(internal::CheckStudyStartDate(filter, now));
 
-  for (size_t i = 0; i < arraysize(start_test_cases); ++i) {
+  for (size_t i = 0; i < base::size(start_test_cases); ++i) {
     filter.set_start_date(TimeToProtoTime(start_test_cases[i].start_date));
     const bool result = internal::CheckStudyStartDate(filter, now);
     EXPECT_EQ(start_test_cases[i].expected_result, result)
@@ -313,7 +312,7 @@ TEST(VariationsStudyFilteringTest, CheckStudyEndDate) {
   // End date not set should result in true.
   EXPECT_TRUE(internal::CheckStudyEndDate(filter, now));
 
-  for (size_t i = 0; i < arraysize(start_test_cases); ++i) {
+  for (size_t i = 0; i < base::size(start_test_cases); ++i) {
     filter.set_end_date(TimeToProtoTime(start_test_cases[i].end_date));
     const bool result = internal::CheckStudyEndDate(filter, now);
     EXPECT_EQ(start_test_cases[i].expected_result, result) << "Case " << i
@@ -368,7 +367,7 @@ TEST(VariationsStudyFilteringTest, CheckStudyVersion) {
   // Min/max version not set should result in true.
   EXPECT_TRUE(internal::CheckStudyVersion(filter, base::Version("1.2.3")));
 
-  for (size_t i = 0; i < arraysize(min_test_cases); ++i) {
+  for (size_t i = 0; i < base::size(min_test_cases); ++i) {
     filter.set_min_version(min_test_cases[i].min_version);
     const bool result = internal::CheckStudyVersion(
         filter, base::Version(min_test_cases[i].version));
@@ -377,7 +376,7 @@ TEST(VariationsStudyFilteringTest, CheckStudyVersion) {
   }
   filter.clear_min_version();
 
-  for (size_t i = 0; i < arraysize(max_test_cases); ++i) {
+  for (size_t i = 0; i < base::size(max_test_cases); ++i) {
     filter.set_max_version(max_test_cases[i].max_version);
     const bool result = internal::CheckStudyVersion(
         filter, base::Version(max_test_cases[i].version));
@@ -386,8 +385,8 @@ TEST(VariationsStudyFilteringTest, CheckStudyVersion) {
   }
 
   // Check intersection semantics.
-  for (size_t i = 0; i < arraysize(min_test_cases); ++i) {
-    for (size_t j = 0; j < arraysize(max_test_cases); ++j) {
+  for (size_t i = 0; i < base::size(min_test_cases); ++i) {
+    for (size_t j = 0; j < base::size(max_test_cases); ++j) {
       filter.set_min_version(min_test_cases[i].min_version);
       filter.set_max_version(max_test_cases[j].max_version);
 
@@ -413,54 +412,46 @@ TEST(VariationsStudyFilteringTest, CheckStudyHardwareClass) {
     const char* actual_hardware_class;
     bool expected_result;
   } test_cases[] = {
-    // Neither filtered nor excluded set:
-    // True since empty is always a match.
-    {"", "", "fancy INTEL pear device", true},
-    {"", "", "", true},
+      // Neither filtered nor excluded set:
+      // True since empty is always a match.
+      {"", "", "fancy INTEL pear device", true},
+      {"", "", "", true},
 
-    // Filtered set:
-    {"apple,pear,orange", "", "apple", true},
-    {"apple,pear,orange", "", "fancy INTEL pear device", true},
-    {"apple,pear,orange", "", "fancy INTEL GRAPE device", false},
-    // Somehow tagged as both, but still valid.
-    {"apple,pear,orange", "", "fancy INTEL pear GRAPE device", true},
-    // Substring, so still valid.
-    {"apple,pear,orange", "", "fancy INTEL SNapple device", true},
-    // No issues with doubling.
-    {"apple,pear,orange", "", "fancy orange orange device", true},
-    // Empty, which is what would happen for non ChromeOS platforms.
-    {"apple,pear,orange", "", "", false},
+      // Filtered set:
+      {"apple,pear,orange", "", "apple", true},
+      {"apple,pear,orange", "", "aPPle", true},
+      {"apple,pear,orange", "", "fancy INTEL pear device", false},
+      {"apple,pear,orange", "", "fancy INTEL GRAPE device", false},
+      // Somehow tagged as both, but still valid.
+      {"apple,pear,orange", "", "fancy INTEL pear GRAPE device", false},
+      // Substring, which should not match.
+      {"apple,pear,orange", "", "fancy INTEL SNapple device", false},
+      // Empty, which is what would happen for non ChromeOS platforms.
+      {"apple,pear,orange", "", "", false},
 
-    // Excluded set:
-    {"", "apple,pear,orange", "apple", false},
-    {"", "apple,pear,orange", "fancy INTEL pear device", false},
-    {"", "apple,pear,orange", "fancy INTEL GRAPE device", true},
-    // Somehow tagged as both. Very excluded!
-    {"", "apple,pear,orange", "fancy INTEL pear GRAPE device", false},
-    // Substring, so still invalid.
-    {"", "apple,pear,orange", "fancy INTEL SNapple device", false},
-    // Empty.
-    {"", "apple,pear,orange", "", true},
+      // Excluded set:
+      {"", "apple,pear,orange", "apple", false},
+      {"", "apple,pear,orange", "fancy INTEL pear device", true},
+      {"", "apple,pear,orange", "fancy INTEL GRAPE device", true},
+      // Empty.
+      {"", "apple,pear,orange", "", true},
 
-    // Not testing when both are set as it should never occur and should be
-    // considered undefined.
+      // Not testing when both are set as it should never occur and should be
+      // considered undefined.
   };
 
-  for (size_t i = 0; i < arraysize(test_cases); ++i) {
+  for (const auto& test : test_cases) {
     Study::Filter filter;
-    for (const std::string& cur : base::SplitString(
-             test_cases[i].hardware_class, ",",
-             base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL))
-      filter.add_hardware_class(cur);
+    for (const auto& hw_class : SplitFilterString(test.hardware_class))
+      filter.add_hardware_class(hw_class);
+    for (const auto& hw_class : SplitFilterString(test.exclude_hardware_class))
+      filter.add_exclude_hardware_class(hw_class);
 
-    for (const std::string& cur : base::SplitString(
-             test_cases[i].exclude_hardware_class, ",",
-             base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL))
-      filter.add_exclude_hardware_class(cur);
-
-    EXPECT_EQ(test_cases[i].expected_result,
-              internal::CheckStudyHardwareClass(
-                  filter, test_cases[i].actual_hardware_class));
+    EXPECT_EQ(test.expected_result, internal::CheckStudyHardwareClass(
+                                        filter, test.actual_hardware_class))
+        << "hardware_class=" << test.hardware_class << " "
+        << "exclude_hardware_class=" << test.exclude_hardware_class << " "
+        << "actual_hardware_class=" << test.actual_hardware_class;
   }
 }
 
@@ -498,14 +489,10 @@ TEST(VariationsStudyFilteringTest, CheckStudyCountry) {
 
   for (const auto& test : test_cases) {
     Study::Filter filter;
-    for (const std::string& country : base::SplitString(
-             test.country, ",", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL))
+    for (const std::string& country : SplitFilterString(test.country))
       filter.add_country(country);
-
-    for (const std::string& exclude_country : base::SplitString(
-             test.exclude_country, ",",
-             base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL))
-      filter.add_exclude_country(exclude_country);
+    for (const std::string& country : SplitFilterString(test.exclude_country))
+      filter.add_exclude_country(country);
 
     EXPECT_EQ(test.expected_result,
               internal::CheckStudyCountry(filter, test.actual_country));
@@ -649,7 +636,7 @@ TEST(VariationsStudyFilteringTest, IsStudyExpired) {
   // Expiry date not set should result in false.
   EXPECT_FALSE(internal::IsStudyExpired(study, now));
 
-  for (size_t i = 0; i < arraysize(expiry_test_cases); ++i) {
+  for (size_t i = 0; i < base::size(expiry_test_cases); ++i) {
     study.set_expiry_date(TimeToProtoTime(expiry_test_cases[i].expiry_date));
     const bool result = internal::IsStudyExpired(study, now);
     EXPECT_EQ(expiry_test_cases[i].expected_result, result)
