@@ -7,15 +7,15 @@
 
 #include "base/base64url.h"
 #include "base/logging.h"
-#include "base/macros.h"
+#include "base/stl_util.h"
 #include "components/webcrypto/algorithm_dispatch.h"
 #include "components/webcrypto/algorithms/test_helpers.h"
 #include "components/webcrypto/crypto_data.h"
 #include "components/webcrypto/jwk.h"
 #include "components/webcrypto/status.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/WebKit/public/platform/WebCryptoAlgorithmParams.h"
-#include "third_party/WebKit/public/platform/WebCryptoKeyAlgorithm.h"
+#include "third_party/blink/public/platform/web_crypto_algorithm_params.h"
+#include "third_party/blink/public/platform/web_crypto_key_algorithm.h"
 
 namespace webcrypto {
 
@@ -72,7 +72,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithNoAlg) {
   ASSERT_EQ(
       Status::Success(),
       ImportKeyJwkFromDict(
-          *jwk.get(),
+          *jwk,
           CreateRsaHashedImportAlgorithm(blink::kWebCryptoAlgorithmIdRsaOaep,
                                          blink::kWebCryptoAlgorithmIdSha1),
           true, blink::kWebCryptoKeyUsageEncrypt, &public_key));
@@ -86,7 +86,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithMatchingAlg) {
   ASSERT_EQ(
       Status::Success(),
       ImportKeyJwkFromDict(
-          *jwk.get(),
+          *jwk,
           CreateRsaHashedImportAlgorithm(blink::kWebCryptoAlgorithmIdRsaOaep,
                                          blink::kWebCryptoAlgorithmIdSha1),
           true, blink::kWebCryptoKeyUsageEncrypt, &public_key));
@@ -100,7 +100,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithMismatchedAlgFails) {
   ASSERT_EQ(
       Status::ErrorJwkAlgorithmInconsistent(),
       ImportKeyJwkFromDict(
-          *jwk.get(),
+          *jwk,
           CreateRsaHashedImportAlgorithm(blink::kWebCryptoAlgorithmIdRsaOaep,
                                          blink::kWebCryptoAlgorithmIdSha1),
           true, blink::kWebCryptoKeyUsageEncrypt, &public_key));
@@ -115,7 +115,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithMismatchedTypeFails) {
   ASSERT_EQ(
       Status::ErrorJwkUnexpectedKty("RSA"),
       ImportKeyJwkFromDict(
-          *jwk.get(),
+          *jwk,
           CreateRsaHashedImportAlgorithm(blink::kWebCryptoAlgorithmIdRsaOaep,
                                          blink::kWebCryptoAlgorithmIdSha1),
           true, blink::kWebCryptoKeyUsageEncrypt, &public_key));
@@ -129,7 +129,7 @@ TEST_F(WebCryptoRsaOaepTest, ExportPublicJwk) {
                    {blink::kWebCryptoAlgorithmIdSha256, "RSA-OAEP-256"},
                    {blink::kWebCryptoAlgorithmIdSha384, "RSA-OAEP-384"},
                    {blink::kWebCryptoAlgorithmIdSha512, "RSA-OAEP-512"}};
-  for (size_t i = 0; i < arraysize(kTestData); ++i) {
+  for (size_t i = 0; i < base::size(kTestData); ++i) {
     const TestData& test_data = kTestData[i];
     SCOPED_TRACE(test_data.expected_jwk_alg);
 
@@ -140,7 +140,7 @@ TEST_F(WebCryptoRsaOaepTest, ExportPublicJwk) {
     blink::WebCryptoKey public_key;
     ASSERT_EQ(Status::Success(),
               ImportKeyJwkFromDict(
-                  *jwk.get(),
+                  *jwk,
                   CreateRsaHashedImportAlgorithm(
                       blink::kWebCryptoAlgorithmIdRsaOaep, test_data.hash_alg),
                   true, blink::kWebCryptoKeyUsageEncrypt, &public_key));
@@ -162,7 +162,7 @@ TEST_F(WebCryptoRsaOaepTest, EncryptDecryptKnownAnswerTest) {
   for (size_t test_index = 0; test_index < tests->GetSize(); ++test_index) {
     SCOPED_TRACE(test_index);
 
-    base::DictionaryValue* test = NULL;
+    base::DictionaryValue* test = nullptr;
     ASSERT_TRUE(tests->GetDictionary(test_index, &test));
 
     blink::WebCryptoAlgorithm digest_algorithm =
@@ -213,7 +213,7 @@ TEST_F(WebCryptoRsaOaepTest, EncryptWithLargeMessageFails) {
   blink::WebCryptoKey public_key;
   ASSERT_EQ(Status::Success(),
             ImportKeyJwkFromDict(
-                *jwk.get(),
+                *jwk,
                 CreateRsaHashedImportAlgorithm(
                     blink::kWebCryptoAlgorithmIdRsaOaep, kHash),
                 true, blink::kWebCryptoKeyUsageEncrypt, &public_key));
@@ -269,7 +269,7 @@ TEST_F(WebCryptoRsaOaepTest, EncryptWithLargeDigestFails) {
   blink::WebCryptoKey public_key;
   ASSERT_EQ(Status::Success(),
             ImportKeyJwkFromDict(
-                *jwk.get(),
+                *jwk,
                 CreateRsaHashedImportAlgorithm(
                     blink::kWebCryptoAlgorithmIdRsaOaep, kHash),
                 true, blink::kWebCryptoKeyUsageEncrypt, &public_key));
@@ -479,7 +479,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportExportJwkRsaPublicKey) {
       {blink::kWebCryptoAlgorithmIdSha512, blink::kWebCryptoKeyUsageEncrypt,
        "RSA-OAEP-512"}};
 
-  for (size_t test_index = 0; test_index < arraysize(kTests); ++test_index) {
+  for (size_t test_index = 0; test_index < base::size(kTests); ++test_index) {
     SCOPED_TRACE(test_index);
     const TestCase& test = kTests[test_index];
 
