@@ -8,7 +8,6 @@
 #include "base/files/scoped_temp_dir.h"
 #include "base/guid.h"
 #include "base/macros.h"
-#include "base/message_loop/message_loop.h"
 #include "base/path_service.h"
 #include "base/stl_util.h"
 #include "base/strings/string16.h"
@@ -25,7 +24,6 @@
 #include "components/autofill/core/browser/webdata/autofill_entry.h"
 #include "components/autofill/core/browser/webdata/autofill_table.h"
 #include "components/autofill/core/common/autofill_constants.h"
-#include "components/password_manager/core/browser/webdata/logins_table.h"
 #include "components/search_engines/keyword_table.h"
 #include "components/signin/core/browser/webdata/token_service_table.h"
 #include "components/webdata/common/web_database.h"
@@ -65,13 +63,11 @@ class WebDatabaseMigrationTest : public testing::Test {
   void DoMigration() {
     AutofillTable autofill_table;
     KeywordTable keyword_table;
-    LoginsTable logins_table;
     TokenServiceTable token_service_table;
 
     WebDatabase db;
     db.AddTable(&autofill_table);
     db.AddTable(&keyword_table);
-    db.AddTable(&logins_table);
     db.AddTable(&token_service_table);
 
     // This causes the migration to occur.
@@ -96,7 +92,7 @@ class WebDatabaseMigrationTest : public testing::Test {
   // Returns true if the file exists and is read successfully, false otherwise.
   bool GetWebDatabaseData(const base::FilePath& file, std::string* contents) {
     base::FilePath source_path;
-    PathService::Get(base::DIR_SOURCE_ROOT, &source_path);
+    base::PathService::Get(base::DIR_SOURCE_ROOT, &source_path);
     source_path = source_path.AppendASCII("components");
     source_path = source_path.AppendASCII("test");
     source_path = source_path.AppendASCII("data");
@@ -106,7 +102,7 @@ class WebDatabaseMigrationTest : public testing::Test {
            base::ReadFileToString(source_path, contents);
   }
 
-  static int VersionFromConnection(sql::Connection* connection) {
+  static int VersionFromConnection(sql::Database* connection) {
     // Get version.
     sql::Statement s(connection->GetUniqueStatement(
         "SELECT value FROM meta WHERE key='version'"));
@@ -130,14 +126,14 @@ class WebDatabaseMigrationTest : public testing::Test {
   DISALLOW_COPY_AND_ASSIGN(WebDatabaseMigrationTest);
 };
 
-const int WebDatabaseMigrationTest::kCurrentTestedVersionNumber = 74;
+const int WebDatabaseMigrationTest::kCurrentTestedVersionNumber = 81;
 
 void WebDatabaseMigrationTest::LoadDatabase(
     const base::FilePath::StringType& file) {
   std::string contents;
   ASSERT_TRUE(GetWebDatabaseData(base::FilePath(file), &contents));
 
-  sql::Connection connection;
+  sql::Database connection;
   ASSERT_TRUE(connection.Open(GetDatabasePath()));
   ASSERT_TRUE(connection.Execute(contents.data()));
 }
@@ -146,7 +142,7 @@ void WebDatabaseMigrationTest::LoadDatabase(
 // schema as migrating from an empty database.
 TEST_F(WebDatabaseMigrationTest, VersionXxSqlFilesAreGolden) {
   DoMigration();
-  sql::Connection connection;
+  sql::Database connection;
   ASSERT_TRUE(connection.Open(GetDatabasePath()));
   const std::string& expected_schema = RemoveQuotes(connection.GetSchema());
   for (int i = WebDatabase::kDeprecatedVersionNumber + 1;
@@ -177,7 +173,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateEmptyToCurrent) {
   // Verify post-conditions.  These are expectations for current version of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
 
     // Check version.
@@ -191,8 +187,6 @@ TEST_F(WebDatabaseMigrationTest, MigrateEmptyToCurrent) {
     EXPECT_TRUE(connection.DoesTableExist("autofill_profiles"));
     EXPECT_TRUE(connection.DoesTableExist("credit_cards"));
     EXPECT_TRUE(connection.DoesTableExist("keywords"));
-    // The logins table is obsolete. (We used to store saved passwords here.)
-    EXPECT_FALSE(connection.DoesTableExist("logins"));
     EXPECT_TRUE(connection.DoesTableExist("meta"));
     EXPECT_TRUE(connection.DoesTableExist("token_service"));
     // The web_apps and web_apps_icons tables are obsolete as of version 58.
@@ -212,7 +206,7 @@ TEST_F(WebDatabaseMigrationTest, RazeDeprecatedVersionAndReinit) {
   // Verify pre-conditions.  These are expectations for version 50 of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -224,8 +218,6 @@ TEST_F(WebDatabaseMigrationTest, RazeDeprecatedVersionAndReinit) {
         connection.DoesColumnExist("keywords", "search_url_post_params"));
     ASSERT_FALSE(
         connection.DoesColumnExist("keywords", "suggest_url_post_params"));
-    ASSERT_FALSE(
-        connection.DoesColumnExist("keywords", "instant_url_post_params"));
     ASSERT_FALSE(
         connection.DoesColumnExist("keywords", "image_url_post_params"));
   }
@@ -235,7 +227,7 @@ TEST_F(WebDatabaseMigrationTest, RazeDeprecatedVersionAndReinit) {
   // Verify post-conditions.  These are expectations for current version of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -248,8 +240,6 @@ TEST_F(WebDatabaseMigrationTest, RazeDeprecatedVersionAndReinit) {
         connection.DoesColumnExist("keywords", "search_url_post_params"));
     EXPECT_TRUE(
         connection.DoesColumnExist("keywords", "suggest_url_post_params"));
-    EXPECT_TRUE(
-        connection.DoesColumnExist("keywords", "instant_url_post_params"));
     EXPECT_TRUE(
         connection.DoesColumnExist("keywords", "image_url_post_params"));
   }
@@ -263,7 +253,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion52ToCurrent) {
   // Verify pre-conditions.  These are expectations for version 52 of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -278,7 +268,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion52ToCurrent) {
   // Verify post-conditions.  These are expectations for current version of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -302,7 +292,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion53ToCurrent) {
   // Verify pre-conditions.  These are expectations for version 53 of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
 
     EXPECT_TRUE(
@@ -324,7 +314,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion53ToCurrent) {
   // Verify post-conditions.  These are expectations for current version of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -458,7 +448,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion54ToCurrent) {
   // Verify pre-conditions.  These are expectations for version 54 of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
 
     EXPECT_TRUE(connection.DoesTableExist("autofill_dates"));
@@ -540,7 +530,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion54ToCurrent) {
   // Verify post-conditions.  These are expectations for current version of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -612,7 +602,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion55ToCurrent) {
   // Verify pre-conditions. These are expectations for version 55 of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -625,7 +615,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion55ToCurrent) {
   // Verify post-conditions. These are expectations for current version of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -674,7 +664,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion56ToCurrent) {
   // Verify pre-conditions. These are expectations for version 56 of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -697,7 +687,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion56ToCurrent) {
   // Verify post-conditions. These are expectations for current version of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -735,7 +725,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion57ToCurrent) {
   // Verify pre-conditions. These are expectations for version 57 of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -750,7 +740,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion57ToCurrent) {
   // Verify post-conditions. These are expectations for current version of the
   // database.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -775,7 +765,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion58ToCurrent) {
       "?q={searchTerms}'";
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -795,7 +785,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion58ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -828,7 +818,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion59ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -844,7 +834,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion59ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -864,7 +854,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion60ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -881,7 +871,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion60ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -901,7 +891,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion61ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -918,7 +908,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion61ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -938,7 +928,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion64ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -959,7 +949,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion64ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -983,7 +973,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion65ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1001,7 +991,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion65ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1026,7 +1016,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion66ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1045,7 +1035,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion66ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1066,7 +1056,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion67ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1080,7 +1070,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion67ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1098,7 +1088,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion68ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1112,7 +1102,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion68ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1129,7 +1119,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion69ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1144,7 +1134,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion69ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1164,7 +1154,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion70ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1181,7 +1171,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion70ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1241,7 +1231,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion71ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1260,7 +1250,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion71ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1285,7 +1275,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion72ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1300,7 +1290,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion72ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1324,7 +1314,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion73ToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1342,7 +1332,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion73ToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1369,7 +1359,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion73WithTypeColumnToCurrent) {
 
   // Verify pre-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1388,7 +1378,7 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion73WithTypeColumnToCurrent) {
 
   // Verify post-conditions.
   {
-    sql::Connection connection;
+    sql::Database connection;
     ASSERT_TRUE(connection.Open(GetDatabasePath()));
     ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
 
@@ -1408,3 +1398,363 @@ TEST_F(WebDatabaseMigrationTest, MigrateVersion73WithTypeColumnToCurrent) {
     EXPECT_EQ(2, s_masked_cards.ColumnInt(0));
   }
 }
+
+// Tests adding "validity_bitfield" column for the "autofill_profiles" table.
+TEST_F(WebDatabaseMigrationTest, MigrateVersion74ToCurrent) {
+  ASSERT_NO_FATAL_FAILURE(LoadDatabase(FILE_PATH_LITERAL("version_74.sql")));
+
+  // Verify pre-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    sql::MetaTable meta_table;
+    ASSERT_TRUE(meta_table.Init(&connection, 74, 74));
+
+    EXPECT_FALSE(
+        connection.DoesColumnExist("autofill_profiles", "validity_bitfield"));
+  }
+
+  DoMigration();
+
+  // Verify post-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    // Check version.
+    EXPECT_EQ(kCurrentTestedVersionNumber, VersionFromConnection(&connection));
+
+    EXPECT_TRUE(
+        connection.DoesColumnExist("autofill_profiles", "validity_bitfield"));
+
+    // Data should have been preserved. Validity bitfield should have been set
+    // to 0.
+    sql::Statement s_profiles(connection.GetUniqueStatement(
+        "SELECT guid, company_name, street_address, dependent_locality,"
+        " city, state, zipcode, sorting_code, country_code, date_modified,"
+        " origin, language_code, validity_bitfield "
+        "FROM autofill_profiles"));
+
+    ASSERT_TRUE(s_profiles.Step());
+    EXPECT_EQ("00000000-0000-0000-0000-000000000001",
+              s_profiles.ColumnString(0));
+    EXPECT_EQ(ASCIIToUTF16("Google Inc"), s_profiles.ColumnString16(1));
+    EXPECT_EQ(ASCIIToUTF16("340 Main St"), s_profiles.ColumnString16(2));
+    EXPECT_EQ(base::string16(), s_profiles.ColumnString16(3));
+    EXPECT_EQ(ASCIIToUTF16("Los Angeles"), s_profiles.ColumnString16(4));
+    EXPECT_EQ(ASCIIToUTF16("CA"), s_profiles.ColumnString16(5));
+    EXPECT_EQ(ASCIIToUTF16("90291"), s_profiles.ColumnString16(6));
+    EXPECT_EQ(base::string16(), s_profiles.ColumnString16(7));
+    EXPECT_EQ(ASCIIToUTF16("US"), s_profiles.ColumnString16(8));
+    EXPECT_EQ(1395948829, s_profiles.ColumnInt(9));
+    EXPECT_EQ(ASCIIToUTF16(autofill::kSettingsOrigin),
+              s_profiles.ColumnString16(10));
+    EXPECT_EQ("en", s_profiles.ColumnString(11));
+    // The new validity bitfield should have the default value of 0.
+    EXPECT_EQ(0, s_profiles.ColumnInt(12));
+
+    // No more entries expected.
+    ASSERT_FALSE(s_profiles.Step());
+  }
+}
+
+// Tests deletion of Instant-related columns in keywords table.
+TEST_F(WebDatabaseMigrationTest, MigrateVersion75ToCurrent) {
+  ASSERT_NO_FATAL_FAILURE(LoadDatabase(FILE_PATH_LITERAL("version_75.sql")));
+
+  // Verify pre-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    sql::MetaTable meta_table;
+    ASSERT_TRUE(meta_table.Init(&connection, 75, 75));
+
+    EXPECT_TRUE(connection.DoesColumnExist("keywords", "instant_url"));
+    EXPECT_TRUE(
+        connection.DoesColumnExist("keywords", "instant_url_post_params"));
+    EXPECT_TRUE(
+        connection.DoesColumnExist("keywords", "search_terms_replacement_key"));
+  }
+
+  DoMigration();
+
+  // Verify post-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    // Check version.
+    EXPECT_EQ(kCurrentTestedVersionNumber, VersionFromConnection(&connection));
+
+    EXPECT_FALSE(connection.DoesColumnExist("keywords", "instant_url"));
+    EXPECT_FALSE(
+        connection.DoesColumnExist("keywords", "instant_url_post_params"));
+    EXPECT_FALSE(
+        connection.DoesColumnExist("keywords", "search_terms_replacement_key"));
+  }
+}
+
+// Tests changing format of three timestamp columns inside keywords.
+TEST_F(WebDatabaseMigrationTest, MigrateVersion76ToCurrent) {
+  ASSERT_NO_FATAL_FAILURE(LoadDatabase(FILE_PATH_LITERAL("version_76.sql")));
+
+  // Verify pre-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    sql::MetaTable meta_table;
+    ASSERT_TRUE(meta_table.Init(&connection, 76, 76));
+
+    sql::Statement s(connection.GetUniqueStatement(
+        "SELECT id, date_created, last_modified, last_visited FROM keywords"));
+    ASSERT_TRUE(s.Step());
+    EXPECT_EQ(2, s.ColumnInt64(0));
+    EXPECT_EQ(123, s.ColumnInt64(1));
+    EXPECT_EQ(456, s.ColumnInt64(2));
+    EXPECT_EQ(789, s.ColumnInt64(3));
+  }
+
+  DoMigration();
+
+  // Verify post-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    // Check version.
+    EXPECT_EQ(kCurrentTestedVersionNumber, VersionFromConnection(&connection));
+
+    sql::Statement s(connection.GetUniqueStatement(
+        "SELECT id, date_created, last_modified, last_visited FROM keywords"));
+    ASSERT_TRUE(s.Step());
+    EXPECT_EQ(2, s.ColumnInt64(0));
+    EXPECT_EQ(11644473723000000, s.ColumnInt64(1));
+    EXPECT_EQ(11644474056000000, s.ColumnInt64(2));
+    EXPECT_EQ(11644474389000000, s.ColumnInt64(3));
+  }
+}
+
+// Tests adding model_type columns into autofill_sync_metadata and
+// autofill_model_type_state.
+TEST_F(WebDatabaseMigrationTest, MigrateVersion77ToCurrent) {
+  ASSERT_NO_FATAL_FAILURE(LoadDatabase(FILE_PATH_LITERAL("version_77.sql")));
+
+  // Verify pre-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    sql::MetaTable meta_table;
+    ASSERT_TRUE(meta_table.Init(&connection, 77, 77));
+
+    sql::Statement s1(connection.GetUniqueStatement(
+        "SELECT storage_key, value FROM autofill_sync_metadata"));
+    ASSERT_TRUE(s1.Step());
+    EXPECT_EQ("storage_key1", s1.ColumnString(0));
+    EXPECT_EQ("blob1", s1.ColumnString(1));
+    ASSERT_TRUE(s1.Step());
+    EXPECT_EQ("storage_key2", s1.ColumnString(0));
+    EXPECT_EQ("blob2", s1.ColumnString(1));
+
+    sql::Statement s2(connection.GetUniqueStatement(
+        "SELECT id, value FROM autofill_model_type_state"));
+    ASSERT_TRUE(s2.Step());
+    EXPECT_EQ(1, s2.ColumnInt(0));
+    EXPECT_EQ("state", s2.ColumnString(1));
+  }
+
+  DoMigration();
+
+  // Verify post-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    // Check version.
+    EXPECT_EQ(kCurrentTestedVersionNumber, VersionFromConnection(&connection));
+
+    // Note: The migration to version 78 (which added the model_type column)
+    // used the wrong integer ID for the model_type. As a consequence, the later
+    // migration to version 81 deletes all the badly-migrated data (at that
+    // point, it'll most likely have been re-downloaded and stored under the
+    // correct ID, so no point in trying to salvage anything). As a consequence,
+    // there should now be no sync metadata in the database.
+    sql::Statement s1(connection.GetUniqueStatement(
+        "SELECT model_type, storage_key, value FROM autofill_sync_metadata"));
+    EXPECT_FALSE(s1.Step());
+
+    // The same applies for model type state.
+    sql::Statement s2(connection.GetUniqueStatement(
+        "SELECT model_type, value FROM autofill_model_type_state"));
+    EXPECT_FALSE(s2.Step());
+  }
+}
+
+TEST_F(WebDatabaseMigrationTest, MigrateVersion78ToCurrent) {
+  ASSERT_NO_FATAL_FAILURE(LoadDatabase(FILE_PATH_LITERAL("version_78.sql")));
+
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+
+    ASSERT_TRUE(connection.DoesTableExist("ie7_logins"));
+    ASSERT_TRUE(connection.DoesTableExist("logins"));
+  }
+
+  DoMigration();
+
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+
+    ASSERT_FALSE(connection.DoesTableExist("ie7_logins"));
+    ASSERT_FALSE(connection.DoesTableExist("logins"));
+  }
+}
+
+// Tests adding "is_client_validity_states_updated" column for the
+// "autofill_profiles" table.
+TEST_F(WebDatabaseMigrationTest, MigrateVersion79ToCurrent) {
+  ASSERT_NO_FATAL_FAILURE(LoadDatabase(FILE_PATH_LITERAL("version_79.sql")));
+
+  // Verify pre-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+    sql::MetaTable meta_table;
+    ASSERT_TRUE(meta_table.Init(&connection, 79, 79));
+    EXPECT_FALSE(connection.DoesColumnExist(
+        "autofill_profiles", "is_client_validity_states_updated"));
+  }
+  DoMigration();
+  // Verify post-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+    // Check version.
+    EXPECT_EQ(kCurrentTestedVersionNumber, VersionFromConnection(&connection));
+    EXPECT_TRUE(connection.DoesColumnExist(
+        "autofill_profiles", "is_client_validity_states_updated"));
+    // Data should have been preserved. Validity
+    // is_client_validity_states_updated should have been set to false.
+    sql::Statement s_profiles(connection.GetUniqueStatement(
+        "SELECT guid, company_name, street_address, dependent_locality,"
+        " city, state, zipcode, sorting_code, country_code, date_modified,"
+        " origin, language_code, validity_bitfield, "
+        " is_client_validity_states_updated "
+        " FROM autofill_profiles"));
+    ASSERT_TRUE(s_profiles.Step());
+    EXPECT_EQ("00000000-0000-0000-0000-000000000001",
+              s_profiles.ColumnString(0));
+    EXPECT_EQ(ASCIIToUTF16("Google Inc"), s_profiles.ColumnString16(1));
+    EXPECT_EQ(ASCIIToUTF16("340 Main St"), s_profiles.ColumnString16(2));
+    EXPECT_EQ(base::string16(), s_profiles.ColumnString16(3));
+    EXPECT_EQ(ASCIIToUTF16("Los Angeles"), s_profiles.ColumnString16(4));
+    EXPECT_EQ(ASCIIToUTF16("CA"), s_profiles.ColumnString16(5));
+    EXPECT_EQ(ASCIIToUTF16("90291"), s_profiles.ColumnString16(6));
+    EXPECT_EQ(base::string16(), s_profiles.ColumnString16(7));
+    EXPECT_EQ(ASCIIToUTF16("US"), s_profiles.ColumnString16(8));
+    EXPECT_EQ(1395948829, s_profiles.ColumnInt(9));
+    EXPECT_EQ(ASCIIToUTF16(autofill::kSettingsOrigin),
+              s_profiles.ColumnString16(10));
+    EXPECT_EQ("en", s_profiles.ColumnString(11));
+    EXPECT_EQ(1365, s_profiles.ColumnInt(12));
+    // The new is_client_validity_states_updated should have the default value
+    // of FALSE.
+    EXPECT_FALSE(s_profiles.ColumnBool(13));
+
+    // No more entries expected.
+    ASSERT_FALSE(s_profiles.Step());
+  }
+}
+
+TEST_F(WebDatabaseMigrationTest, MigrateVersion80ToCurrent) {
+  ASSERT_NO_FATAL_FAILURE(LoadDatabase(FILE_PATH_LITERAL("version_80.sql")));
+
+  // Verify pre-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    sql::MetaTable meta_table;
+    ASSERT_TRUE(meta_table.Init(&connection, 80, 79));
+
+    sql::Statement s1(connection.GetUniqueStatement(
+        "SELECT model_type, storage_key, value FROM autofill_sync_metadata"));
+    ASSERT_TRUE(s1.Step());
+    // Note: This is the *wrong* ID for AUTOFILL, simulating the botched
+    // migration in version 78. See crbug.com/895826.
+    ASSERT_EQ(syncer::ModelTypeToHistogramInt(syncer::AUTOFILL),
+              s1.ColumnInt(0));
+    ASSERT_EQ("storage_key1", s1.ColumnString(1));
+    ASSERT_EQ("blob1", s1.ColumnString(2));
+
+    ASSERT_TRUE(s1.Step());
+    // Note: This is the *correct* ID for AUTOFILL, simulating the data that got
+    // redownloaded after the bad migration, and stored under the correct ID.
+    ASSERT_EQ(syncer::ModelTypeToStableIdentifier(syncer::AUTOFILL),
+              s1.ColumnInt(0));
+    ASSERT_EQ("storage_key2", s1.ColumnString(1));
+    ASSERT_EQ("blob2", s1.ColumnString(2));
+
+    sql::Statement s2(connection.GetUniqueStatement(
+        "SELECT model_type, value FROM autofill_model_type_state"));
+    ASSERT_TRUE(s2.Step());
+    // Like above: Bad value.
+    ASSERT_EQ(syncer::ModelTypeToHistogramInt(syncer::AUTOFILL),
+              s2.ColumnInt(0));
+    ASSERT_EQ("state1", s2.ColumnString(1));
+    ASSERT_TRUE(s2.Step());
+    // Good value.
+    ASSERT_EQ(syncer::ModelTypeToStableIdentifier(syncer::AUTOFILL),
+              s2.ColumnInt(0));
+    ASSERT_EQ("state2", s2.ColumnString(1));
+  }
+
+  DoMigration();
+
+  // Verify post-conditions.
+  {
+    sql::Database connection;
+    ASSERT_TRUE(connection.Open(GetDatabasePath()));
+    ASSERT_TRUE(sql::MetaTable::DoesTableExist(&connection));
+
+    // Check version.
+    EXPECT_EQ(kCurrentTestedVersionNumber, VersionFromConnection(&connection));
+
+    // Check that the badly-migrated AUTOFILL data from version 78 is gone, but
+    // the correct redownloaded data is still here.
+    sql::Statement s1(connection.GetUniqueStatement(
+        "SELECT model_type, storage_key, value FROM autofill_sync_metadata"));
+    ASSERT_TRUE(s1.Step());
+    EXPECT_EQ(syncer::ModelTypeToStableIdentifier(syncer::AUTOFILL),
+              s1.ColumnInt(0));
+    EXPECT_EQ("storage_key2", s1.ColumnString(1));
+    EXPECT_EQ("blob2", s1.ColumnString(2));
+    EXPECT_FALSE(s1.Step());
+
+    // Check that the badly-migrated AUTOFILL model type state from version 78
+    // is gone, but the correct redownloaded state is still here.
+    sql::Statement s2(connection.GetUniqueStatement(
+        "SELECT model_type, value FROM autofill_model_type_state"));
+    ASSERT_TRUE(s2.Step());
+    EXPECT_EQ(syncer::ModelTypeToStableIdentifier(syncer::AUTOFILL),
+              s2.ColumnInt(0));
+    EXPECT_EQ("state2", s2.ColumnString(1));
+    EXPECT_FALSE(s2.Step());
+  }
+}
