@@ -11,6 +11,7 @@
 #include <memory>
 #include <set>
 #include <string>
+#include <tuple>
 #include <utility>
 
 #include "base/bind.h"
@@ -18,39 +19,35 @@
 #include "base/guid.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
-#include "base/message_loop/message_loop.h"
+#include "base/optional.h"
 #include "base/run_loop.h"
 #include "base/stl_util.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
+#include "components/download/public/common/download_create_info.h"
+#include "components/download/public/common/download_file_factory.h"
+#include "components/download/public/common/download_interrupt_reasons.h"
+#include "components/download/public/common/download_item.h"
+#include "components/download/public/common/download_item_factory.h"
+#include "components/download/public/common/download_item_impl.h"
+#include "components/download/public/common/download_item_impl_delegate.h"
+#include "components/download/public/common/download_request_handle_interface.h"
+#include "components/download/public/common/mock_download_file.h"
+#include "components/download/public/common/mock_download_item_impl.h"
 #include "content/browser/byte_stream.h"
-#include "content/browser/download/download_create_info.h"
-#include "content/browser/download/download_file_factory.h"
-#include "content/browser/download/download_item_factory.h"
-#include "content/browser/download/download_item_impl.h"
-#include "content/browser/download/download_item_impl_delegate.h"
-#include "content/browser/download/download_request_handle.h"
-#include "content/browser/download/mock_download_file.h"
-#include "content/browser/download/mock_download_item_impl.h"
+#include "content/browser/download/byte_stream_input_stream.h"
 #include "content/public/browser/browser_context.h"
-#include "content/public/browser/download_interrupt_reasons.h"
-#include "content/public/browser/download_item.h"
 #include "content/public/browser/download_manager_delegate.h"
-#include "content/public/test/mock_download_item.h"
+#include "content/public/browser/storage_partition.h"
 #include "content/public/test/test_browser_context.h"
-#include "content/public/test/test_browser_thread.h"
-#include "net/log/net_log_with_source.h"
+#include "content/public/test/test_browser_thread_bundle.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gmock_mutant.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/origin.h"
 
-#if !defined(OS_ANDROID)
-#include "content/public/browser/zoom_level_delegate.h"
-#endif  // !defined(OS_ANDROID)
-
 using ::testing::AllOf;
 using ::testing::DoAll;
 using ::testing::Eq;
@@ -65,7 +62,7 @@ using ::testing::_;
 ACTION_TEMPLATE(RunCallback,
                 HAS_1_TEMPLATE_PARAMS(int, k),
                 AND_1_VALUE_PARAMS(p0)) {
-  return ::std::tr1::get<k>(args).Run(p0);
+  return std::get<k>(args).Run(p0);
 }
 
 namespace content {
@@ -73,27 +70,26 @@ class ByteStreamReader;
 
 namespace {
 
-// Matches a DownloadCreateInfo* that points to the same object as |info| and
-// has a |default_download_directory| that matches |download_directory|.
-MATCHER_P2(DownloadCreateInfoWithDefaultPath, info, download_directory, "") {
-  return arg == info && arg->default_download_directory == download_directory;
+bool URLAlwaysSafe(int render_process_id, const GURL& url) {
+  return true;
 }
 
 class MockDownloadManagerDelegate : public DownloadManagerDelegate {
  public:
   MockDownloadManagerDelegate();
-  virtual ~MockDownloadManagerDelegate();
+  ~MockDownloadManagerDelegate() override;
 
   MOCK_METHOD0(Shutdown, void());
   MOCK_METHOD1(GetNextId, void(const DownloadIdCallback&));
   MOCK_METHOD2(DetermineDownloadTarget,
-               bool(DownloadItem* item,
+               bool(download::DownloadItem* item,
                     const DownloadTargetCallback&));
   MOCK_METHOD1(ShouldOpenFileBasedOnExtension, bool(const base::FilePath&));
   MOCK_METHOD2(ShouldCompleteDownload,
-               bool(DownloadItem*, const base::Closure&));
+               bool(download::DownloadItem*, const base::Closure&));
   MOCK_METHOD2(ShouldOpenDownload,
-               bool(DownloadItem*, const DownloadOpenDelayedCallback&));
+               bool(download::DownloadItem*,
+                    const DownloadOpenDelayedCallback&));
   MOCK_METHOD4(GetSaveDir, void(BrowserContext*,
                                 base::FilePath*, base::FilePath*, bool*));
   MOCK_METHOD5(ChooseSavePath, void(
@@ -107,7 +103,7 @@ MockDownloadManagerDelegate::MockDownloadManagerDelegate() {}
 MockDownloadManagerDelegate::~MockDownloadManagerDelegate() {}
 
 class MockDownloadItemFactory
-    : public DownloadItemFactory,
+    : public download::DownloadItemFactory,
       public base::SupportsWeakPtr<MockDownloadItemFactory> {
  public:
   MockDownloadItemFactory();
@@ -118,19 +114,19 @@ class MockDownloadItemFactory
   // functionality if it's ever needed by consumers.
 
   // Returns NULL if no item of that id is present.
-  MockDownloadItemImpl* GetItem(int id);
+  download::MockDownloadItemImpl* GetItem(int id);
 
   // Remove and return an item made by the factory.
   // Generally used during teardown.
-  MockDownloadItemImpl* PopItem();
+  download::MockDownloadItemImpl* PopItem();
 
   // Should be called when the item of this id is removed so that
   // we don't keep dangling pointers.
   void RemoveItem(int id);
 
   // Overridden methods from DownloadItemFactory.
-  DownloadItemImpl* CreatePersistedItem(
-      DownloadItemImplDelegate* delegate,
+  download::DownloadItemImpl* CreatePersistedItem(
+      download::DownloadItemImplDelegate* delegate,
       const std::string& guid,
       uint32_t download_id,
       const base::FilePath& current_path,
@@ -140,6 +136,7 @@ class MockDownloadItemFactory
       const GURL& site_url,
       const GURL& tab_url,
       const GURL& tab_referrer_url,
+      const base::Optional<url::Origin>& request_initiator,
       const std::string& mime_type,
       const std::string& original_mime_type,
       base::Time start_time,
@@ -149,52 +146,56 @@ class MockDownloadItemFactory
       int64_t received_bytes,
       int64_t total_bytes,
       const std::string& hash,
-      DownloadItem::DownloadState state,
-      DownloadDangerType danger_type,
-      DownloadInterruptReason interrupt_reason,
+      download::DownloadItem::DownloadState state,
+      download::DownloadDangerType danger_type,
+      download::DownloadInterruptReason interrupt_reason,
       bool opened,
       base::Time last_access_time,
       bool transient,
-      const std::vector<DownloadItem::ReceivedSlice>& received_slices,
-      const net::NetLogWithSource& net_log) override;
-  DownloadItemImpl* CreateActiveItem(
-      DownloadItemImplDelegate* delegate,
+      const std::vector<download::DownloadItem::ReceivedSlice>& received_slices)
+      override;
+  download::DownloadItemImpl* CreateActiveItem(
+      download::DownloadItemImplDelegate* delegate,
       uint32_t download_id,
-      const DownloadCreateInfo& info,
-      const net::NetLogWithSource& net_log) override;
-  DownloadItemImpl* CreateSavePageItem(
-      DownloadItemImplDelegate* delegate,
+      const download::DownloadCreateInfo& info) override;
+  download::DownloadItemImpl* CreateSavePageItem(
+      download::DownloadItemImplDelegate* delegate,
       uint32_t download_id,
       const base::FilePath& path,
       const GURL& url,
       const std::string& mime_type,
-      std::unique_ptr<DownloadRequestHandleInterface> request_handle,
-      const net::NetLogWithSource& net_log) override;
+      std::unique_ptr<download::DownloadRequestHandleInterface> request_handle)
+      override;
+
+  void set_is_download_persistent(bool is_download_persistent) {
+    is_download_persistent_ = is_download_persistent;
+  }
 
  private:
-  std::map<uint32_t, MockDownloadItemImpl*> items_;
-  DownloadItemImplDelegate item_delegate_;
+  std::map<uint32_t, download::MockDownloadItemImpl*> items_;
+  download::DownloadItemImplDelegate item_delegate_;
+  bool is_download_persistent_;
 
   DISALLOW_COPY_AND_ASSIGN(MockDownloadItemFactory);
 };
 
-MockDownloadItemFactory::MockDownloadItemFactory() {}
+MockDownloadItemFactory::MockDownloadItemFactory()
+    : is_download_persistent_(false) {}
 
 MockDownloadItemFactory::~MockDownloadItemFactory() {}
 
-MockDownloadItemImpl* MockDownloadItemFactory::GetItem(int id) {
+download::MockDownloadItemImpl* MockDownloadItemFactory::GetItem(int id) {
   if (items_.find(id) == items_.end())
-    return NULL;
+    return nullptr;
   return items_[id];
 }
 
-MockDownloadItemImpl* MockDownloadItemFactory::PopItem() {
+download::MockDownloadItemImpl* MockDownloadItemFactory::PopItem() {
   if (items_.empty())
-    return NULL;
+    return nullptr;
 
-  std::map<uint32_t, MockDownloadItemImpl*>::iterator first_item =
-      items_.begin();
-  MockDownloadItemImpl* result = first_item->second;
+  auto first_item = items_.begin();
+  download::MockDownloadItemImpl* result = first_item->second;
   items_.erase(first_item);
   return result;
 }
@@ -204,8 +205,8 @@ void MockDownloadItemFactory::RemoveItem(int id) {
   items_.erase(id);
 }
 
-DownloadItemImpl* MockDownloadItemFactory::CreatePersistedItem(
-    DownloadItemImplDelegate* delegate,
+download::DownloadItemImpl* MockDownloadItemFactory::CreatePersistedItem(
+    download::DownloadItemImplDelegate* delegate,
     const std::string& guid,
     uint32_t download_id,
     const base::FilePath& current_path,
@@ -215,6 +216,7 @@ DownloadItemImpl* MockDownloadItemFactory::CreatePersistedItem(
     const GURL& site_url,
     const GURL& tab_url,
     const GURL& tab_referrer_url,
+    const base::Optional<url::Origin>& request_initiator,
     const std::string& mime_type,
     const std::string& original_mime_type,
     base::Time start_time,
@@ -224,17 +226,16 @@ DownloadItemImpl* MockDownloadItemFactory::CreatePersistedItem(
     int64_t received_bytes,
     int64_t total_bytes,
     const std::string& hash,
-    DownloadItem::DownloadState state,
-    DownloadDangerType danger_type,
-    DownloadInterruptReason interrupt_reason,
+    download::DownloadItem::DownloadState state,
+    download::DownloadDangerType danger_type,
+    download::DownloadInterruptReason interrupt_reason,
     bool opened,
     base::Time last_access_time,
     bool transient,
-    const std::vector<DownloadItem::ReceivedSlice>& received_slices,
-    const net::NetLogWithSource& net_log) {
+    const std::vector<download::DownloadItem::ReceivedSlice>& received_slices) {
   DCHECK(items_.find(download_id) == items_.end());
-  MockDownloadItemImpl* result =
-      new StrictMock<MockDownloadItemImpl>(&item_delegate_);
+  download::MockDownloadItemImpl* result =
+      new StrictMock<download::MockDownloadItemImpl>(&item_delegate_);
   EXPECT_CALL(*result, GetId())
       .WillRepeatedly(Return(download_id));
   EXPECT_CALL(*result, GetGuid()).WillRepeatedly(ReturnRefOfCopy(guid));
@@ -242,20 +243,60 @@ DownloadItemImpl* MockDownloadItemFactory::CreatePersistedItem(
   return result;
 }
 
-DownloadItemImpl* MockDownloadItemFactory::CreateActiveItem(
-    DownloadItemImplDelegate* delegate,
+download::DownloadItemImpl* MockDownloadItemFactory::CreateActiveItem(
+    download::DownloadItemImplDelegate* delegate,
     uint32_t download_id,
-    const DownloadCreateInfo& info,
-    const net::NetLogWithSource& net_log) {
+    const download::DownloadCreateInfo& info) {
   DCHECK(items_.find(download_id) == items_.end());
 
-  MockDownloadItemImpl* result =
-      new StrictMock<MockDownloadItemImpl>(&item_delegate_);
+  download::MockDownloadItemImpl* result =
+      new StrictMock<download::MockDownloadItemImpl>(&item_delegate_);
   EXPECT_CALL(*result, GetId())
       .WillRepeatedly(Return(download_id));
   EXPECT_CALL(*result, GetGuid())
+      .WillRepeatedly(ReturnRefOfCopy(base::GenerateGUID()));
+  EXPECT_CALL(*result, GetUrlChain())
+      .WillRepeatedly(ReturnRefOfCopy(std::vector<GURL>()));
+  EXPECT_CALL(*result, GetReferrerUrl())
+      .WillRepeatedly(ReturnRefOfCopy(GURL()));
+  EXPECT_CALL(*result, GetTabUrl()).WillRepeatedly(ReturnRefOfCopy(GURL()));
+  EXPECT_CALL(*result, GetTabReferrerUrl())
+      .WillRepeatedly(ReturnRefOfCopy(GURL()));
+  EXPECT_CALL(*result, GetETag())
+      .WillRepeatedly(ReturnRefOfCopy(std::string()));
+  EXPECT_CALL(*result, GetLastModifiedTime())
+      .WillRepeatedly(ReturnRefOfCopy(std::string()));
+  EXPECT_CALL(*result, GetMimeType()).WillRepeatedly(Return(std::string()));
+  EXPECT_CALL(*result, GetOriginalMimeType())
+      .WillRepeatedly(Return(std::string()));
+  EXPECT_CALL(*result, GetTotalBytes()).WillRepeatedly(Return(0));
+  EXPECT_CALL(*result, GetFullPath())
       .WillRepeatedly(
-          ReturnRefOfCopy(base::ToUpperASCII(base::GenerateGUID())));
+          ReturnRefOfCopy(base::FilePath(FILE_PATH_LITERAL("foo"))));
+  EXPECT_CALL(*result, GetTargetFilePath())
+      .WillRepeatedly(
+          ReturnRefOfCopy(base::FilePath(FILE_PATH_LITERAL("foo"))));
+  EXPECT_CALL(*result, GetReceivedBytes()).WillRepeatedly(Return(0));
+  EXPECT_CALL(*result, GetStartTime()).WillRepeatedly(Return(base::Time()));
+  EXPECT_CALL(*result, GetEndTime()).WillRepeatedly(Return(base::Time()));
+  EXPECT_CALL(*result, GetReceivedSlices())
+      .WillRepeatedly(ReturnRefOfCopy(
+          std::vector<download::DownloadItem::ReceivedSlice>()));
+  EXPECT_CALL(*result, GetHash())
+      .WillRepeatedly(ReturnRefOfCopy(std::string()));
+  EXPECT_CALL(*result, GetState())
+      .WillRepeatedly(Return(download::DownloadItem::IN_PROGRESS));
+  EXPECT_CALL(*result, GetDangerType())
+      .WillRepeatedly(Return(download::DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS));
+  EXPECT_CALL(*result, GetLastReason())
+      .WillRepeatedly(Return(download::DOWNLOAD_INTERRUPT_REASON_NONE));
+  EXPECT_CALL(*result, IsPaused()).WillRepeatedly(Return(false));
+  EXPECT_CALL(*result, IsTemporary()).WillRepeatedly(Return(false));
+
+  if (is_download_persistent_) {
+    EXPECT_CALL(*result, RemoveObserver(_));
+    EXPECT_CALL(*result, AddObserver(_));
+  }
   items_[download_id] = result;
 
   // Active items are created and then immediately are called to start
@@ -265,18 +306,17 @@ DownloadItemImpl* MockDownloadItemFactory::CreateActiveItem(
   return result;
 }
 
-DownloadItemImpl* MockDownloadItemFactory::CreateSavePageItem(
-    DownloadItemImplDelegate* delegate,
+download::DownloadItemImpl* MockDownloadItemFactory::CreateSavePageItem(
+    download::DownloadItemImplDelegate* delegate,
     uint32_t download_id,
     const base::FilePath& path,
     const GURL& url,
     const std::string& mime_type,
-    std::unique_ptr<DownloadRequestHandleInterface> request_handle,
-    const net::NetLogWithSource& net_log) {
+    std::unique_ptr<download::DownloadRequestHandleInterface> request_handle) {
   DCHECK(items_.find(download_id) == items_.end());
 
-  MockDownloadItemImpl* result =
-      new StrictMock<MockDownloadItemImpl>(&item_delegate_);
+  download::MockDownloadItemImpl* result =
+      new StrictMock<download::MockDownloadItemImpl>(&item_delegate_);
   EXPECT_CALL(*result, GetId())
       .WillRepeatedly(Return(download_id));
   items_[download_id] = result;
@@ -285,98 +325,76 @@ DownloadItemImpl* MockDownloadItemFactory::CreateSavePageItem(
 }
 
 class MockDownloadFileFactory
-    : public DownloadFileFactory,
+    : public download::DownloadFileFactory,
       public base::SupportsWeakPtr<MockDownloadFileFactory> {
  public:
   MockDownloadFileFactory() {}
-  virtual ~MockDownloadFileFactory() {}
+  ~MockDownloadFileFactory() override {}
 
   // Overridden method from DownloadFileFactory
   MOCK_METHOD2(MockCreateFile,
-               MockDownloadFile*(const DownloadSaveInfo&, ByteStreamReader*));
+               download::MockDownloadFile*(const download::DownloadSaveInfo&,
+                                           download::InputStream*));
 
-  DownloadFile* CreateFile(
-      std::unique_ptr<DownloadSaveInfo> save_info,
+  download::DownloadFile* CreateFile(
+      std::unique_ptr<download::DownloadSaveInfo> save_info,
       const base::FilePath& default_download_directory,
-      std::unique_ptr<ByteStreamReader> byte_stream,
-      const net::NetLogWithSource& net_log,
-      base::WeakPtr<DownloadDestinationObserver> observer) override {
-    return MockCreateFile(*save_info, byte_stream.get());
+      std::unique_ptr<download::InputStream> stream,
+      uint32_t download_id,
+      base::WeakPtr<download::DownloadDestinationObserver> observer) override {
+    return MockCreateFile(*save_info, stream.get());
   }
 };
 
-class MockBrowserContext : public BrowserContext {
- public:
-  MockBrowserContext() {
-    content::BrowserContext::Initialize(this, base::FilePath());
-  }
-  ~MockBrowserContext() {}
-
-  MOCK_CONST_METHOD0(GetPath, base::FilePath());
-#if !defined(OS_ANDROID)
-  MOCK_METHOD1(CreateZoomLevelDelegateMock,
-               ZoomLevelDelegate*(const base::FilePath&));
-#endif  // !defined(OS_ANDROID)
-  MOCK_CONST_METHOD0(IsOffTheRecord, bool());
-  MOCK_METHOD0(GetResourceContext, ResourceContext*());
-  MOCK_METHOD0(GetDownloadManagerDelegate, DownloadManagerDelegate*());
-  MOCK_METHOD0(GetGuestManager, BrowserPluginGuestManager* ());
-  MOCK_METHOD0(GetSpecialStoragePolicy, storage::SpecialStoragePolicy*());
-  MOCK_METHOD0(GetPushMessagingService, PushMessagingService*());
-  MOCK_METHOD0(GetSSLHostStateDelegate, SSLHostStateDelegate*());
-  MOCK_METHOD0(GetPermissionManager, PermissionManager*());
-  MOCK_METHOD0(GetBackgroundSyncController, BackgroundSyncController*());
-  MOCK_METHOD0(GetBrowsingDataRemoverDelegate, BrowsingDataRemoverDelegate*());
-  MOCK_METHOD0(CreateMediaRequestContext,
-               net::URLRequestContextGetter*());
-  MOCK_METHOD2(CreateMediaRequestContextForStoragePartition,
-               net::URLRequestContextGetter*(
-                   const base::FilePath& partition_path, bool in_memory));
-
-  // Define these two methods to avoid a
-  // 'cannot access private member declared in class
-  // URLRequestInterceptorScopedVector'
-  // build error if they're put in MOCK_METHOD.
-  net::URLRequestContextGetter* CreateRequestContext(
-      ProtocolHandlerMap* protocol_handlers,
-      URLRequestInterceptorScopedVector request_interceptors) override {
-    return nullptr;
-  }
-
-  net::URLRequestContextGetter* CreateRequestContextForStoragePartition(
-      const base::FilePath& partition_path,
-      bool in_memory,
-      ProtocolHandlerMap* protocol_handlers,
-      URLRequestInterceptorScopedVector request_interceptors) override {
-    return nullptr;
-  }
-#if !defined(OS_ANDROID)
-  std::unique_ptr<ZoomLevelDelegate> CreateZoomLevelDelegate(
-      const base::FilePath& path) override {
-    return std::unique_ptr<ZoomLevelDelegate>(
-        CreateZoomLevelDelegateMock(path));
-  }
-#endif  // !defined(OS_ANDROID)
-};
-
 class MockDownloadManagerObserver : public DownloadManager::Observer {
  public:
   MockDownloadManagerObserver() {}
-  ~MockDownloadManagerObserver() {}
-  MOCK_METHOD2(OnDownloadCreated, void(
-        DownloadManager*, DownloadItem*));
+  ~MockDownloadManagerObserver() override {}
+  MOCK_METHOD2(OnDownloadCreated,
+               void(DownloadManager*, download::DownloadItem*));
   MOCK_METHOD1(ManagerGoingDown, void(DownloadManager*));
   MOCK_METHOD2(SelectFileDialogDisplayed, void(DownloadManager*, int32_t));
 };
 
 class MockByteStreamReader : public ByteStreamReader {
  public:
-  virtual ~MockByteStreamReader() {}
+  ~MockByteStreamReader() override {}
   MOCK_METHOD2(Read, StreamState(scoped_refptr<net::IOBuffer>*, size_t*));
   MOCK_CONST_METHOD0(GetStatus, int());
   MOCK_METHOD1(RegisterCallback, void(const base::Closure&));
 };
 
+class TestInProgressManager : public download::InProgressDownloadManager {
+ public:
+  TestInProgressManager();
+  ~TestInProgressManager() override = default;
+
+  std::vector<std::unique_ptr<download::DownloadItemImpl>>
+  TakeInProgressDownloads() override;
+
+  void AddDownloadItem(std::unique_ptr<download::DownloadItemImpl> item);
+
+ private:
+  std::vector<std::unique_ptr<download::DownloadItemImpl>> download_items_;
+};
+
+TestInProgressManager::TestInProgressManager()
+    : download::InProgressDownloadManager(
+          nullptr,
+          base::FilePath(),
+          download::InProgressDownloadManager::IsOriginSecureCallback(),
+          base::BindRepeating(&URLAlwaysSafe)) {}
+
+void TestInProgressManager::AddDownloadItem(
+    std::unique_ptr<download::DownloadItemImpl> item) {
+  download_items_.emplace_back(std::move(item));
+}
+
+std::vector<std::unique_ptr<download::DownloadItemImpl>>
+TestInProgressManager::TakeInProgressDownloads() {
+  return std::move(download_items_);
+}
+
 }  // namespace
 
 class DownloadManagerTest : public testing::Test {
@@ -386,11 +404,10 @@ class DownloadManagerTest : public testing::Test {
 
   DownloadManagerTest()
       : callback_called_(false),
-        target_disposition_(DownloadItem::TARGET_DISPOSITION_OVERWRITE),
-        danger_type_(DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS),
-        interrupt_reason_(DOWNLOAD_INTERRUPT_REASON_NONE),
-        ui_thread_(BrowserThread::UI, &message_loop_),
-        file_thread_(BrowserThread::FILE, &message_loop_),
+        target_disposition_(
+            download::DownloadItem::TARGET_DISPOSITION_OVERWRITE),
+        danger_type_(download::DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS),
+        interrupt_reason_(download::DOWNLOAD_INTERRUPT_REASON_NONE),
         next_download_id_(0) {}
 
   // We tear down everything in TearDown().
@@ -408,17 +425,13 @@ class DownloadManagerTest : public testing::Test {
         new StrictMock<MockDownloadManagerDelegate>);
     EXPECT_CALL(*mock_download_manager_delegate_.get(), Shutdown())
         .WillOnce(Return());
-    mock_browser_context_.reset(new StrictMock<MockBrowserContext>);
-    EXPECT_CALL(*mock_browser_context_.get(), IsOffTheRecord())
-        .WillRepeatedly(Return(false));
-
-    download_manager_.reset(new DownloadManagerImpl(
-                                NULL, mock_browser_context_.get()));
+    browser_context_ = std::make_unique<TestBrowserContext>();
+    download_manager_.reset(new DownloadManagerImpl(browser_context_.get()));
     download_manager_->SetDownloadItemFactoryForTesting(
-        std::unique_ptr<DownloadItemFactory>(
+        std::unique_ptr<download::DownloadItemFactory>(
             mock_download_item_factory_.get()));
     download_manager_->SetDownloadFileFactoryForTesting(
-        std::unique_ptr<DownloadFileFactory>(
+        std::unique_ptr<download::DownloadFileFactory>(
             mock_download_file_factory_.get()));
     observer_.reset(new MockDownloadManagerObserver());
     download_manager_->AddObserver(observer_.get());
@@ -430,10 +443,10 @@ class DownloadManagerTest : public testing::Test {
   }
 
   void TearDown() override {
-    while (MockDownloadItemImpl*
-           item = mock_download_item_factory_->PopItem()) {
+    while (download::MockDownloadItemImpl* item =
+               mock_download_item_factory_->PopItem()) {
       EXPECT_CALL(*item, GetState())
-          .WillOnce(Return(DownloadItem::CANCELLED));
+          .WillOnce(Return(download::DownloadItem::CANCELLED));
     }
     EXPECT_CALL(GetMockObserver(), ManagerGoingDown(download_manager_.get()))
         .WillOnce(Return());
@@ -443,35 +456,36 @@ class DownloadManagerTest : public testing::Test {
     base::RunLoop().RunUntilIdle();
     ASSERT_FALSE(mock_download_item_factory_);
     mock_download_manager_delegate_.reset();
-    mock_browser_context_.reset();
     download_urls_.clear();
   }
 
   // Returns download id.
-  MockDownloadItemImpl& AddItemToManager() {
-    DownloadCreateInfo info;
+  download::MockDownloadItemImpl& AddItemToManager() {
+    download::DownloadCreateInfo info;
 
     // Args are ignored except for download id, so everything else can be
     // null.
     uint32_t id = next_download_id_;
     ++next_download_id_;
-    info.request_handle.reset(new DownloadRequestHandle);
     download_manager_->CreateActiveItem(id, info);
     DCHECK(mock_download_item_factory_->GetItem(id));
-    MockDownloadItemImpl& item(*mock_download_item_factory_->GetItem(id));
+    download::MockDownloadItemImpl& item(
+        *mock_download_item_factory_->GetItem(id));
     // Satisfy expectation.  If the item is created in StartDownload(),
     // we call Start on it immediately, so we need to set that expectation
     // in the factory.
-    std::unique_ptr<DownloadRequestHandleInterface> req_handle;
-    item.Start(std::unique_ptr<DownloadFile>(), std::move(req_handle), info);
+    std::unique_ptr<download::DownloadRequestHandleInterface> req_handle;
+    item.Start(std::unique_ptr<download::DownloadFile>(), std::move(req_handle),
+               info, nullptr, nullptr);
     DCHECK(id < download_urls_.size());
     EXPECT_CALL(item, GetURL()).WillRepeatedly(ReturnRef(download_urls_[id]));
 
     return item;
   }
 
-  MockDownloadItemImpl& GetMockDownloadItem(int id) {
-    MockDownloadItemImpl* itemp = mock_download_item_factory_->GetItem(id);
+  download::MockDownloadItemImpl& GetMockDownloadItem(int id) {
+    download::MockDownloadItemImpl* itemp =
+        mock_download_item_factory_->GetItem(id);
 
     DCHECK(itemp);
     return *itemp;
@@ -492,10 +506,10 @@ class DownloadManagerTest : public testing::Test {
 
   void DownloadTargetDeterminedCallback(
       const base::FilePath& target_path,
-      DownloadItem::TargetDisposition disposition,
-      DownloadDangerType danger_type,
+      download::DownloadItem::TargetDisposition disposition,
+      download::DownloadDangerType danger_type,
       const base::FilePath& intermediate_path,
-      DownloadInterruptReason interrupt_reason) {
+      download::DownloadInterruptReason interrupt_reason) {
     callback_called_ = true;
     target_path_ = target_path;
     target_disposition_ = disposition;
@@ -504,36 +518,48 @@ class DownloadManagerTest : public testing::Test {
     interrupt_reason_ = interrupt_reason;
   }
 
-  void DetermineDownloadTarget(DownloadItemImpl* item) {
+  void DetermineDownloadTarget(download::DownloadItemImpl* item) {
     download_manager_->DetermineDownloadTarget(
         item, base::Bind(
             &DownloadManagerTest::DownloadTargetDeterminedCallback,
             base::Unretained(this)));
   }
 
+  void OnInProgressDownloadManagerInitialized() {
+    download_manager_->OnInProgressDownloadManagerInitialized();
+  }
+
+  void OnHistoryDBInitialized() {
+    download_manager_->PostInitialization(
+        DownloadManager::DOWNLOAD_INITIALIZATION_DEPENDENCY_HISTORY_DB);
+  }
+
+  void SetInProgressDownloadManager(
+      std::unique_ptr<download::InProgressDownloadManager> manager) {
+    download_manager_->in_progress_manager_ = std::move(manager);
+  }
+
  protected:
   // Key test variable; we'll keep it available to sub-classes.
   std::unique_ptr<DownloadManagerImpl> download_manager_;
   base::WeakPtr<MockDownloadFileFactory> mock_download_file_factory_;
+  base::WeakPtr<MockDownloadItemFactory> mock_download_item_factory_;
 
   // Target detetermined callback.
   bool callback_called_;
   base::FilePath target_path_;
-  DownloadItem::TargetDisposition target_disposition_;
-  DownloadDangerType danger_type_;
+  download::DownloadItem::TargetDisposition target_disposition_;
+  download::DownloadDangerType danger_type_;
   base::FilePath intermediate_path_;
-  DownloadInterruptReason interrupt_reason_;
+  download::DownloadInterruptReason interrupt_reason_;
 
   std::vector<GURL> download_urls_;
 
  private:
-  base::MessageLoopForUI message_loop_;
-  TestBrowserThread ui_thread_;
-  TestBrowserThread file_thread_;
-  base::WeakPtr<MockDownloadItemFactory> mock_download_item_factory_;
+  TestBrowserThreadBundle thread_bundle_;
   std::unique_ptr<MockDownloadManagerDelegate> mock_download_manager_delegate_;
-  std::unique_ptr<MockBrowserContext> mock_browser_context_;
   std::unique_ptr<MockDownloadManagerObserver> observer_;
+  std::unique_ptr<TestBrowserContext> browser_context_;
   uint32_t next_download_id_;
 
   DISALLOW_COPY_AND_ASSIGN(DownloadManagerTest);
@@ -541,10 +567,14 @@ class DownloadManagerTest : public testing::Test {
 
 // Confirm the appropriate invocations occur when you start a download.
 TEST_F(DownloadManagerTest, StartDownload) {
-  std::unique_ptr<DownloadCreateInfo> info(new DownloadCreateInfo);
+  std::unique_ptr<download::DownloadCreateInfo> info(
+      new download::DownloadCreateInfo);
   std::unique_ptr<ByteStreamReader> stream(new MockByteStreamReader);
-  uint32_t local_id(5);  // Random value
+  // Random value, a non 0 value means history db is properly loaded, and new
+  // downloads should be persisted to the in-progress db.
+  uint32_t local_id(5);
   base::FilePath download_path(FILE_PATH_LITERAL("download/path"));
+  OnInProgressDownloadManagerInitialized();
 
   EXPECT_FALSE(download_manager_->GetDownload(local_id));
 
@@ -553,30 +583,70 @@ TEST_F(DownloadManagerTest, StartDownload) {
   EXPECT_CALL(GetMockDownloadManagerDelegate(), GetNextId(_))
       .WillOnce(RunCallback<0>(local_id));
 
-#if !defined(USE_X11) || defined(OS_CHROMEOS)
+#if !defined(USE_X11)
   // Doing nothing will set the default download directory to null.
   EXPECT_CALL(GetMockDownloadManagerDelegate(), GetSaveDir(_, _, _, _));
 #endif
   EXPECT_CALL(GetMockDownloadManagerDelegate(),
               ApplicationClientIdForFileScanning())
       .WillRepeatedly(Return("client-id"));
-  MockDownloadFile* mock_file = new MockDownloadFile;
+  download::MockDownloadFile* mock_file = new download::MockDownloadFile;
+  auto input_stream =
+      std::make_unique<ByteStreamInputStream>(std::move(stream));
   EXPECT_CALL(*mock_download_file_factory_.get(),
-              MockCreateFile(Ref(*info->save_info.get()), stream.get()))
+              MockCreateFile(Ref(*info->save_info.get()), input_stream.get()))
       .WillOnce(Return(mock_file));
 
-  download_manager_->StartDownload(std::move(info), std::move(stream),
-                                   DownloadUrlParameters::OnStartedCallback());
+  mock_download_item_factory_->set_is_download_persistent(true);
+  download_manager_->StartDownload(
+      std::move(info), std::move(input_stream), nullptr,
+      download::DownloadUrlParameters::OnStartedCallback());
   EXPECT_TRUE(download_manager_->GetDownload(local_id));
 }
 
+// Test the case that a new download is started when history db failed to
+// initialize.
+TEST_F(DownloadManagerTest, StartDownloadWithoutHistoryDB) {
+  std::unique_ptr<download::DownloadCreateInfo> info(
+      new download::DownloadCreateInfo);
+  std::unique_ptr<ByteStreamReader> stream(new MockByteStreamReader);
+  base::FilePath download_path(FILE_PATH_LITERAL("download/path"));
+  OnInProgressDownloadManagerInitialized();
+  EXPECT_FALSE(download_manager_->GetDownload(1));
+
+  EXPECT_CALL(GetMockObserver(), OnDownloadCreated(download_manager_.get(), _))
+      .WillOnce(Return());
+  // Returning kInvalidId to indicate that the history db failed.
+  EXPECT_CALL(GetMockDownloadManagerDelegate(), GetNextId(_))
+      .WillOnce(RunCallback<0>(download::DownloadItem::kInvalidId));
+
+#if !defined(USE_X11)
+  // Doing nothing will set the default download directory to null.
+  EXPECT_CALL(GetMockDownloadManagerDelegate(), GetSaveDir(_, _, _, _));
+#endif
+  EXPECT_CALL(GetMockDownloadManagerDelegate(),
+              ApplicationClientIdForFileScanning())
+      .WillRepeatedly(Return("client-id"));
+  download::MockDownloadFile* mock_file = new download::MockDownloadFile;
+  auto input_stream =
+      std::make_unique<ByteStreamInputStream>(std::move(stream));
+  EXPECT_CALL(*mock_download_file_factory_.get(),
+              MockCreateFile(Ref(*info->save_info.get()), input_stream.get()))
+      .WillOnce(Return(mock_file));
+
+  download_manager_->StartDownload(
+      std::move(info), std::move(input_stream), nullptr,
+      download::DownloadUrlParameters::OnStartedCallback());
+  EXPECT_TRUE(download_manager_->GetDownload(1));
+}
+
 // Confirm that calling DetermineDownloadTarget behaves properly if the delegate
 // blocks starting.
 TEST_F(DownloadManagerTest, DetermineDownloadTarget_True) {
   // Put a mock we have a handle to on the download manager.
-  MockDownloadItemImpl& item(AddItemToManager());
+  download::MockDownloadItemImpl& item(AddItemToManager());
   EXPECT_CALL(item, GetState())
-      .WillRepeatedly(Return(DownloadItem::IN_PROGRESS));
+      .WillRepeatedly(Return(download::DownloadItem::IN_PROGRESS));
 
   EXPECT_CALL(GetMockDownloadManagerDelegate(),
               DetermineDownloadTarget(&item, _))
@@ -588,7 +658,7 @@ TEST_F(DownloadManagerTest, DetermineDownloadTarget_True) {
 // allows starting.  This also tests OnDownloadTargetDetermined.
 TEST_F(DownloadManagerTest, DetermineDownloadTarget_False) {
   // Put a mock we have a handle to on the download manager.
-  MockDownloadItemImpl& item(AddItemToManager());
+  download::MockDownloadItemImpl& item(AddItemToManager());
 
   base::FilePath path(FILE_PATH_LITERAL("random_filepath.txt"));
   EXPECT_CALL(GetMockDownloadManagerDelegate(),
@@ -601,33 +671,41 @@ TEST_F(DownloadManagerTest, DetermineDownloadTarget_False) {
   DetermineDownloadTarget(&item);
   EXPECT_TRUE(callback_called_);
   EXPECT_EQ(path, target_path_);
-  EXPECT_EQ(DownloadItem::TARGET_DISPOSITION_OVERWRITE, target_disposition_);
-  EXPECT_EQ(DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS, danger_type_);
+  EXPECT_EQ(download::DownloadItem::TARGET_DISPOSITION_OVERWRITE,
+            target_disposition_);
+  EXPECT_EQ(download::DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS, danger_type_);
   EXPECT_EQ(path, intermediate_path_);
-  EXPECT_EQ(DOWNLOAD_INTERRUPT_REASON_NONE, interrupt_reason_);
+  EXPECT_EQ(download::DOWNLOAD_INTERRUPT_REASON_NONE, interrupt_reason_);
 }
 
 TEST_F(DownloadManagerTest, GetDownloadByGuid) {
   for (uint32_t i = 0; i < 4; ++i)
     AddItemToManager();
 
-  MockDownloadItemImpl& item = GetMockDownloadItem(0);
-  DownloadItem* result = download_manager_->GetDownloadByGuid(item.GetGuid());
+  download::MockDownloadItemImpl& item = GetMockDownloadItem(0);
+  download::DownloadItem* result =
+      download_manager_->GetDownloadByGuid(item.GetGuid());
   ASSERT_TRUE(result);
-  ASSERT_EQ(static_cast<DownloadItem*>(&item), result);
+  ASSERT_EQ(static_cast<download::DownloadItem*>(&item), result);
 
   ASSERT_FALSE(download_manager_->GetDownloadByGuid(""));
 
   const char kGuid[] = "8DF158E8-C980-4618-BB03-EBA3242EB48B";
-  DownloadItem* persisted_item = download_manager_->CreateDownloadItem(
-      kGuid, 10, base::FilePath(), base::FilePath(), std::vector<GURL>(),
-      GURL("http://example.com/a"), GURL("http://example.com/a"),
-      GURL("http://example.com/a"), GURL("http://example.com/a"),
-      "application/octet-stream", "application/octet-stream", base::Time::Now(),
-      base::Time::Now(), std::string(), std::string(), 10, 10, std::string(),
-      DownloadItem::INTERRUPTED, DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS,
-      DOWNLOAD_INTERRUPT_REASON_SERVER_FAILED, false, base::Time::Now(), true,
-      std::vector<DownloadItem::ReceivedSlice>());
+  std::vector<GURL> url_chain;
+  url_chain.emplace_back("http://example.com/1.zip");
+  download::DownloadItem* persisted_item =
+      download_manager_->CreateDownloadItem(
+          kGuid, 10, base::FilePath(), base::FilePath(), url_chain,
+          GURL("http://example.com/a"), GURL("http://example.com/a"),
+          GURL("http://example.com/a"), GURL("http://example.com/a"),
+          url::Origin::Create(GURL("http://example.com/")),
+          "application/octet-stream", "application/octet-stream",
+          base::Time::Now(), base::Time::Now(), std::string(), std::string(),
+          10, 10, std::string(), download::DownloadItem::INTERRUPTED,
+          download::DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS,
+          download::DOWNLOAD_INTERRUPT_REASON_SERVER_FAILED, false,
+          base::Time::Now(), true,
+          std::vector<download::DownloadItem::ReceivedSlice>());
   ASSERT_TRUE(persisted_item);
 
   ASSERT_EQ(persisted_item, download_manager_->GetDownloadByGuid(kGuid));
@@ -646,10 +724,10 @@ base::Callback<bool(const GURL&)> GetSingleURLFilter(const GURL& url) {
 TEST_F(DownloadManagerTest, RemoveDownloadsByURL) {
   base::Time now(base::Time::Now());
   for (uint32_t i = 0; i < 2; ++i) {
-    MockDownloadItemImpl& item(AddItemToManager());
+    download::MockDownloadItemImpl& item(AddItemToManager());
     EXPECT_CALL(item, GetStartTime()).WillRepeatedly(Return(now));
     EXPECT_CALL(item, GetState())
-        .WillRepeatedly(Return(DownloadItem::COMPLETE));
+        .WillRepeatedly(Return(download::DownloadItem::COMPLETE));
   }
 
   EXPECT_CALL(GetMockDownloadItem(0), Remove());
@@ -658,8 +736,42 @@ TEST_F(DownloadManagerTest, RemoveDownloadsByURL) {
   base::Callback<bool(const GURL&)> url_filter =
       GetSingleURLFilter(download_urls_[0]);
   int remove_count = download_manager_->RemoveDownloadsByURLAndTime(
-      url_filter, base::Time(), base::Time::Max());
+      std::move(url_filter), base::Time(), base::Time::Max());
   EXPECT_EQ(remove_count, 1);
 }
 
+// Confirm that in-progress downloads will be taken and managed by
+// DownloadManager.
+TEST_F(DownloadManagerTest, OnInProgressDownloadsLoaded) {
+  auto in_progress_manager = std::make_unique<TestInProgressManager>();
+  const char kGuid[] = "8DF158E8-C980-4618-BB03-EBA3242EB48B";
+  std::vector<GURL> url_chain;
+  url_chain.emplace_back("http://example.com/1.zip");
+  auto in_progress_item = std::make_unique<download::DownloadItemImpl>(
+      in_progress_manager.get(), kGuid, 10, base::FilePath(), base::FilePath(),
+      url_chain, GURL("http://example.com/a"), GURL("http://example.com/a"),
+      GURL("http://example.com/a"), GURL("http://example.com/a"),
+      url::Origin::Create(GURL("http://example.com")),
+      "application/octet-stream", "application/octet-stream", base::Time::Now(),
+      base::Time::Now(), std::string(), std::string(), 10, 10, 0, std::string(),
+      download::DownloadItem::INTERRUPTED,
+      download::DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS,
+      download::DOWNLOAD_INTERRUPT_REASON_SERVER_FAILED, false, false, false,
+      base::Time::Now(), true,
+      std::vector<download::DownloadItem::ReceivedSlice>());
+  in_progress_manager->AddDownloadItem(std::move(in_progress_item));
+  SetInProgressDownloadManager(std::move(in_progress_manager));
+  EXPECT_CALL(GetMockObserver(), OnDownloadCreated(download_manager_.get(), _))
+      .WillOnce(Return());
+  OnInProgressDownloadManagerInitialized();
+  ASSERT_FALSE(download_manager_->GetDownloadByGuid(kGuid));
+
+  OnHistoryDBInitialized();
+  ASSERT_TRUE(download_manager_->GetDownloadByGuid(kGuid));
+  download::DownloadItem* download =
+      download_manager_->GetDownloadByGuid(kGuid);
+  download->Remove();
+  ASSERT_FALSE(download_manager_->GetDownloadByGuid(kGuid));
+}
+
 }  // namespace content
