@@ -29,10 +29,11 @@
 #include "base/memory/ptr_util.h"
 #include "base/path_service.h"
 #include "base/strings/stringprintf.h"
+#include "base/task/post_task.h"
+#include "base/threading/scoped_blocking_call.h"
 #include "base/threading/thread_checker.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/threading/thread_task_runner_handle.h"
-#include "base/threading/worker_pool.h"
 #include "build/build_config.h"
 #include "crypto/nss_crypto_module_delegate.h"
 #include "crypto/nss_util_internal.h"
@@ -72,7 +73,7 @@ std::string GetNSSErrorMessage() {
 #if !defined(OS_CHROMEOS)
 base::FilePath GetDefaultConfigDirectory() {
   base::FilePath dir;
-  PathService::Get(base::DIR_HOME, &dir);
+  base::PathService::Get(base::DIR_HOME, &dir);
   if (dir.empty()) {
     LOG(ERROR) << "Failed to get home directory.";
     return dir;
@@ -85,7 +86,7 @@ base::FilePath GetDefaultConfigDirectory() {
   DVLOG(2) << "DefaultConfigDirectory: " << dir.value();
   return dir;
 }
-#endif  // !defined(IS_CHROMEOS)
+#endif  // !defined(OS_CHROMEOS)
 
 // On non-Chrome OS platforms, return the default config directory. On Chrome OS
 // test images, return a read-only directory with fake root CA certs (which are
@@ -135,15 +136,10 @@ class NSPRInitSingleton {
     PR_Init(PR_USER_THREAD, PR_PRIORITY_NORMAL, 0);
   }
 
-  // NOTE(willchan): We don't actually execute this code since we leak NSS to
-  // prevent non-joinable threads from using NSS after it's already been shut
-  // down.
-  ~NSPRInitSingleton() {
-    PL_ArenaFinish();
-    PRStatus prstatus = PR_Cleanup();
-    if (prstatus != PR_SUCCESS)
-      LOG(ERROR) << "PR_Cleanup failed; was NSPR initialized on wrong thread?";
-  }
+  // NOTE(willchan): We don't actually cleanup on destruction since we leak NSS
+  // to prevent non-joinable threads from using NSS after it's already been
+  // shut down.
+  ~NSPRInitSingleton() = delete;
 };
 
 base::LazyInstance<NSPRInitSingleton>::Leaky
@@ -180,11 +176,11 @@ class ChromeOSUserData {
   }
 
   ScopedPK11Slot GetPrivateSlot(
-      const base::Callback<void(ScopedPK11Slot)>& callback) {
+      base::OnceCallback<void(ScopedPK11Slot)> callback) {
     if (private_slot_)
       return ScopedPK11Slot(PK11_ReferenceSlot(private_slot_.get()));
     if (!callback.is_null())
-      tpm_ready_callback_list_.push_back(callback);
+      tpm_ready_callback_list_.push_back(std::move(callback));
     return ScopedPK11Slot();
   }
 
@@ -197,7 +193,8 @@ class ChromeOSUserData {
     for (SlotReadyCallbackList::iterator i = callback_list.begin();
          i != callback_list.end();
          ++i) {
-      (*i).Run(ScopedPK11Slot(PK11_ReferenceSlot(private_slot_.get())));
+      std::move(*i).Run(
+          ScopedPK11Slot(PK11_ReferenceSlot(private_slot_.get())));
     }
   }
 
@@ -215,7 +212,7 @@ class ChromeOSUserData {
 
   bool private_slot_initialization_started_;
 
-  typedef std::vector<base::Callback<void(ScopedPK11Slot)> >
+  typedef std::vector<base::OnceCallback<void(ScopedPK11Slot)>>
       SlotReadyCallbackList;
   SlotReadyCallbackList tpm_ready_callback_list_;
 };
@@ -296,7 +293,7 @@ class NSSInitSingleton {
 
   void InitializeTPMTokenAndSystemSlot(
       int system_slot_id,
-      const base::Callback<void(bool)>& callback) {
+      base::OnceCallback<void(bool)> callback) {
     DCHECK(thread_checker_.CalledOnValidThread());
     // Should not be called while there is already an initialization in
     // progress.
@@ -304,7 +301,7 @@ class NSSInitSingleton {
     // If EnableTPMTokenForNSS hasn't been called, return false.
     if (!tpm_token_enabled_for_nss_) {
       base::ThreadTaskRunnerHandle::Get()->PostTask(
-          FROM_HERE, base::Bind(callback, false));
+          FROM_HERE, base::BindOnce(std::move(callback), false));
       return;
     }
 
@@ -312,8 +309,8 @@ class NSSInitSingleton {
     // Note that only |tpm_slot_| is checked, since |chaps_module_| could be
     // nullptr in tests while |tpm_slot_| has been set to the test DB.
     if (tpm_slot_) {
-      base::ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                                    base::Bind(callback, true));
+      base::ThreadTaskRunnerHandle::Get()->PostTask(
+          FROM_HERE, base::BindOnce(std::move(callback), true));
       return;
     }
 
@@ -322,25 +319,28 @@ class NSSInitSingleton {
     std::unique_ptr<TPMModuleAndSlot> tpm_args(
         new TPMModuleAndSlot(chaps_module_));
     TPMModuleAndSlot* tpm_args_ptr = tpm_args.get();
-    if (base::WorkerPool::PostTaskAndReply(
-            FROM_HERE,
-            base::Bind(&NSSInitSingleton::InitializeTPMTokenOnWorkerThread,
+    base::PostTaskWithTraitsAndReply(
+        FROM_HERE,
+        {base::MayBlock(), base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
+        base::BindOnce(&NSSInitSingleton::InitializeTPMTokenInThreadPool,
                        system_slot_id, tpm_args_ptr),
-            base::Bind(&NSSInitSingleton::OnInitializedTPMTokenAndSystemSlot,
+        base::BindOnce(&NSSInitSingleton::OnInitializedTPMTokenAndSystemSlot,
                        base::Unretained(this),  // NSSInitSingleton is leaky
-                       callback, base::Passed(&tpm_args)),
-            true /* task_is_slow */)) {
-      initializing_tpm_token_ = true;
-    } else {
-      base::ThreadTaskRunnerHandle::Get()->PostTask(
-          FROM_HERE, base::Bind(callback, false));
-    }
+                       std::move(callback), std::move(tpm_args)));
+    initializing_tpm_token_ = true;
   }
 
-  static void InitializeTPMTokenOnWorkerThread(CK_SLOT_ID token_slot_id,
-                                               TPMModuleAndSlot* tpm_args) {
-    // This tries to load the Chaps module so NSS can talk to the hardware
-    // TPM.
+  static void InitializeTPMTokenInThreadPool(CK_SLOT_ID token_slot_id,
+                                             TPMModuleAndSlot* tpm_args) {
+    // NSS functions may reenter //net via extension hooks. If the reentered
+    // code needs to synchronously wait for a task to run but the thread pool in
+    // which that task must run doesn't have enough threads to schedule it, a
+    // deadlock occurs. To prevent that, the base::ScopedBlockingCall below
+    // increments the thread pool capacity for the duration of the TPM
+    // initialization.
+    base::ScopedBlockingCall scoped_blocking_call(
+        base::BlockingType::WILL_BLOCK);
+
     if (!tpm_args->chaps_module) {
       ScopedChapsLoadFixup chaps_loader;
 
@@ -357,12 +357,12 @@ class NSSInitSingleton {
     }
     if (tpm_args->chaps_module) {
       tpm_args->tpm_slot =
-          GetTPMSlotForIdOnWorkerThread(tpm_args->chaps_module, token_slot_id);
+          GetTPMSlotForIdInThreadPool(tpm_args->chaps_module, token_slot_id);
     }
   }
 
   void OnInitializedTPMTokenAndSystemSlot(
-      const base::Callback<void(bool)>& callback,
+      base::OnceCallback<void(bool)> callback,
       std::unique_ptr<TPMModuleAndSlot> tpm_args) {
     DCHECK(thread_checker_.CalledOnValidThread());
     DVLOG(2) << "Loaded chaps: " << !!tpm_args->chaps_module
@@ -380,7 +380,7 @@ class NSSInitSingleton {
     if (tpm_slot_)
       RunAndClearTPMReadyCallbackList();
 
-    callback.Run(!!tpm_slot_);
+    std::move(callback).Run(!!tpm_slot_);
   }
 
   void RunAndClearTPMReadyCallbackList() {
@@ -389,11 +389,11 @@ class NSSInitSingleton {
     for (TPMReadyCallbackList::iterator i = callback_list.begin();
          i != callback_list.end();
          ++i) {
-      i->Run();
+      std::move(*i).Run();
     }
   }
 
-  bool IsTPMTokenReady(const base::Closure& callback) {
+  bool IsTPMTokenReady(base::OnceClosure callback) {
     if (!callback.is_null()) {
       // Cannot DCHECK in the general case yet, but since the callback is
       // a new addition to the API, DCHECK to make sure at least the new uses
@@ -409,7 +409,7 @@ class NSSInitSingleton {
       return true;
 
     if (!callback.is_null())
-      tpm_ready_callback_list_.push_back(callback);
+      tpm_ready_callback_list_.push_back(std::move(callback));
 
     return false;
   }
@@ -417,7 +417,7 @@ class NSSInitSingleton {
   // Note that CK_SLOT_ID is an unsigned long, but cryptohome gives us the slot
   // id as an int. This should be safe since this is only used with chaps, which
   // we also control.
-  static crypto::ScopedPK11Slot GetTPMSlotForIdOnWorkerThread(
+  static crypto::ScopedPK11Slot GetTPMSlotForIdInThreadPool(
       SECMODModule* chaps_module,
       CK_SLOT_ID slot_id) {
     DCHECK(chaps_module);
@@ -447,7 +447,7 @@ class NSSInitSingleton {
         "%s %s", kUserNSSDatabaseName, username_hash.c_str());
     ScopedPK11Slot public_slot(OpenPersistentNSSDBForPath(db_name, path));
     chromeos_user_map_[username_hash] =
-        base::MakeUnique<ChromeOSUserData>(std::move(public_slot));
+        std::make_unique<ChromeOSUserData>(std::move(public_slot));
     return true;
   }
 
@@ -482,14 +482,14 @@ class NSSInitSingleton {
     std::unique_ptr<TPMModuleAndSlot> tpm_args(
         new TPMModuleAndSlot(chaps_module_));
     TPMModuleAndSlot* tpm_args_ptr = tpm_args.get();
-    base::WorkerPool::PostTaskAndReply(
+    base::PostTaskWithTraitsAndReply(
         FROM_HERE,
-        base::Bind(&NSSInitSingleton::InitializeTPMTokenOnWorkerThread, slot_id,
-                   tpm_args_ptr),
-        base::Bind(&NSSInitSingleton::OnInitializedTPMForChromeOSUser,
-                   base::Unretained(this),  // NSSInitSingleton is leaky
-                   username_hash, base::Passed(&tpm_args)),
-        true /* task_is_slow */);
+        {base::MayBlock(), base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
+        base::BindOnce(&NSSInitSingleton::InitializeTPMTokenInThreadPool,
+                       slot_id, tpm_args_ptr),
+        base::BindOnce(&NSSInitSingleton::OnInitializedTPMForChromeOSUser,
+                       base::Unretained(this),  // NSSInitSingleton is leaky
+                       username_hash, std::move(tpm_args)));
   }
 
   void OnInitializedTPMForChromeOSUser(
@@ -510,6 +510,12 @@ class NSSInitSingleton {
     DCHECK(chromeos_user_map_[username_hash]->
                private_slot_initialization_started());
 
+    if (prepared_test_private_slot_) {
+      chromeos_user_map_[username_hash]->SetPrivateSlot(
+          std::move(prepared_test_private_slot_));
+      return;
+    }
+
     chromeos_user_map_[username_hash]->SetPrivateSlot(
         chromeos_user_map_[username_hash]->GetPublicSlot());
   }
@@ -532,21 +538,22 @@ class NSSInitSingleton {
 
   ScopedPK11Slot GetPrivateSlotForChromeOSUser(
       const std::string& username_hash,
-      const base::Callback<void(ScopedPK11Slot)>& callback) {
+      base::OnceCallback<void(ScopedPK11Slot)> callback) {
     DCHECK(thread_checker_.CalledOnValidThread());
 
     if (username_hash.empty()) {
       DVLOG(2) << "empty username_hash";
       if (!callback.is_null()) {
         base::ThreadTaskRunnerHandle::Get()->PostTask(
-            FROM_HERE, base::Bind(callback, base::Passed(ScopedPK11Slot())));
+            FROM_HERE, base::BindOnce(std::move(callback), ScopedPK11Slot()));
       }
       return ScopedPK11Slot();
     }
 
     DCHECK(chromeos_user_map_.find(username_hash) != chromeos_user_map_.end());
 
-    return chromeos_user_map_[username_hash]->GetPrivateSlot(callback);
+    return chromeos_user_map_[username_hash]->GetPrivateSlot(
+        std::move(callback));
   }
 
   void CloseChromeOSUserForTesting(const std::string& username_hash) {
@@ -557,6 +564,8 @@ class NSSInitSingleton {
   }
 
   void SetSystemKeySlotForTesting(ScopedPK11Slot slot) {
+    DCHECK(thread_checker_.CalledOnValidThread());
+
     // Ensure that a previous value of test_system_slot_ is not overwritten.
     // Unsetting, i.e. setting a nullptr, however is allowed.
     DCHECK(!slot || !test_system_slot_);
@@ -568,6 +577,15 @@ class NSSInitSingleton {
       tpm_slot_.reset();
     }
   }
+
+  void SetPrivateSoftwareSlotForChromeOSUserForTesting(ScopedPK11Slot slot) {
+    DCHECK(thread_checker_.CalledOnValidThread());
+
+    // Ensure that a previous value of prepared_test_private_slot_ is not
+    // overwritten. Unsetting, i.e. setting a nullptr, however is allowed.
+    DCHECK(!slot || !prepared_test_private_slot_);
+    prepared_test_private_slot_ = std::move(slot);
+  }
 #endif  // defined(OS_CHROMEOS)
 
 #if !defined(OS_CHROMEOS)
@@ -584,26 +602,26 @@ class NSSInitSingleton {
 
 #if defined(OS_CHROMEOS)
   void GetSystemNSSKeySlotCallback(
-      const base::Callback<void(ScopedPK11Slot)>& callback) {
-    callback.Run(ScopedPK11Slot(PK11_ReferenceSlot(tpm_slot_.get())));
+      base::OnceCallback<void(ScopedPK11Slot)> callback) {
+    std::move(callback).Run(
+        ScopedPK11Slot(PK11_ReferenceSlot(tpm_slot_.get())));
   }
 
   ScopedPK11Slot GetSystemNSSKeySlot(
-      const base::Callback<void(ScopedPK11Slot)>& callback) {
+      base::OnceCallback<void(ScopedPK11Slot)> callback) {
     DCHECK(thread_checker_.CalledOnValidThread());
     // TODO(mattm): chromeos::TPMTokenloader always calls
     // InitializeTPMTokenAndSystemSlot with slot 0.  If the system slot is
     // disabled, tpm_slot_ will be the first user's slot instead. Can that be
     // detected and return nullptr instead?
 
-    base::Closure wrapped_callback;
+    base::OnceClosure wrapped_callback;
     if (!callback.is_null()) {
-      wrapped_callback =
-          base::Bind(&NSSInitSingleton::GetSystemNSSKeySlotCallback,
-                     base::Unretained(this) /* singleton is leaky */,
-                     callback);
+      wrapped_callback = base::BindOnce(
+          &NSSInitSingleton::GetSystemNSSKeySlotCallback,
+          base::Unretained(this) /* singleton is leaky */, std::move(callback));
     }
-    if (IsTPMTokenReady(wrapped_callback))
+    if (IsTPMTokenReady(std::move(wrapped_callback)))
       return ScopedPK11Slot(PK11_ReferenceSlot(tpm_slot_.get()));
     return ScopedPK11Slot();
   }
@@ -617,22 +635,24 @@ class NSSInitSingleton {
         initializing_tpm_token_(false),
         chaps_module_(nullptr),
         root_(nullptr) {
+    // Initializing NSS causes us to do blocking IO.
+    // Temporarily allow it until we fix
+    //   http://code.google.com/p/chromium/issues/detail?id=59847
+    base::ThreadRestrictions::ScopedAllowIO allow_io;
+
     // It's safe to construct on any thread, since LazyInstance will prevent any
     // other threads from accessing until the constructor is done.
     thread_checker_.DetachFromThread();
 
     EnsureNSPRInit();
 
-    // We *must* have NSS >= 3.14.3.
-    static_assert(
-        (NSS_VMAJOR == 3 && NSS_VMINOR == 14 && NSS_VPATCH >= 3) ||
-        (NSS_VMAJOR == 3 && NSS_VMINOR > 14) ||
-        (NSS_VMAJOR > 3),
-        "nss version check failed");
+    // We *must* have NSS >= 3.26 at compile time.
+    static_assert((NSS_VMAJOR == 3 && NSS_VMINOR >= 26) || (NSS_VMAJOR > 3),
+                  "nss version check failed");
     // Also check the run-time NSS version.
     // NSS_VersionCheck is a >= check, not strict equality.
-    if (!NSS_VersionCheck("3.14.3")) {
-      LOG(FATAL) << "NSS_VersionCheck(\"3.14.3\") failed. NSS >= 3.14.3 is "
+    if (!NSS_VersionCheck("3.26")) {
+      LOG(FATAL) << "NSS_VersionCheck(\"3.26\") failed. NSS >= 3.26 is "
                     "required. Please upgrade to the latest NSS, and if you "
                     "still get this error, contact your distribution "
                     "maintainer.";
@@ -688,32 +708,10 @@ class NSSInitSingleton {
                            0, NSS_USE_ALG_IN_CERT_SIGNATURE);
   }
 
-  // NOTE(willchan): We don't actually execute this code since we leak NSS to
-  // prevent non-joinable threads from using NSS after it's already been shut
-  // down.
-  ~NSSInitSingleton() {
-#if defined(OS_CHROMEOS)
-    chromeos_user_map_.clear();
-#endif
-    tpm_slot_.reset();
-    if (root_) {
-      SECMOD_UnloadUserModule(root_);
-      SECMOD_DestroyModule(root_);
-      root_ = nullptr;
-    }
-    if (chaps_module_) {
-      SECMOD_UnloadUserModule(chaps_module_);
-      SECMOD_DestroyModule(chaps_module_);
-      chaps_module_ = nullptr;
-    }
-
-    SECStatus status = NSS_Shutdown();
-    if (status != SECSuccess) {
-      // We VLOG(1) because this failure is relatively harmless (leaking, but
-      // we're shutting down anyway).
-      VLOG(1) << "NSS_Shutdown failed; see http://crbug.com/4609";
-    }
-  }
+  // NOTE(willchan): We don't actually cleanup on destruction since we leak NSS
+  // to prevent non-joinable threads from using NSS after it's already been
+  // shut down.
+  ~NSSInitSingleton() = delete;
 
   // Load nss's built-in root certs.
   SECMODModule* InitDefaultRootCerts() {
@@ -757,7 +755,7 @@ class NSSInitSingleton {
 
   bool tpm_token_enabled_for_nss_;
   bool initializing_tpm_token_;
-  typedef std::vector<base::Closure> TPMReadyCallbackList;
+  typedef std::vector<base::OnceClosure> TPMReadyCallbackList;
   TPMReadyCallbackList tpm_ready_callback_list_;
   SECMODModule* chaps_module_;
   crypto::ScopedPK11Slot tpm_slot_;
@@ -765,6 +763,7 @@ class NSSInitSingleton {
 #if defined(OS_CHROMEOS)
   std::map<std::string, std::unique_ptr<ChromeOSUserData>> chromeos_user_map_;
   ScopedPK11Slot test_system_slot_;
+  ScopedPK11Slot prepared_test_private_slot_;
 #endif
 
   base::ThreadChecker thread_checker_;
@@ -796,10 +795,6 @@ void EnsureNSPRInit() {
 }
 
 void EnsureNSSInit() {
-  // Initializing SSL causes us to do blocking IO.
-  // Temporarily allow it until we fix
-  //   http://code.google.com/p/chromium/issues/detail?id=59847
-  base::ThreadRestrictions::ScopedAllowIO allow_io;
   g_nss_singleton.Get();
 }
 
@@ -818,8 +813,8 @@ AutoSECMODListReadLock::~AutoSECMODListReadLock() {
 
 #if defined(OS_CHROMEOS)
 ScopedPK11Slot GetSystemNSSKeySlot(
-    const base::Callback<void(ScopedPK11Slot)>& callback) {
-  return g_nss_singleton.Get().GetSystemNSSKeySlot(callback);
+    base::OnceCallback<void(ScopedPK11Slot)> callback) {
+  return g_nss_singleton.Get().GetSystemNSSKeySlot(std::move(callback));
 }
 
 void SetSystemKeySlotForTesting(ScopedPK11Slot slot) {
@@ -834,15 +829,14 @@ bool IsTPMTokenEnabledForNSS() {
   return g_nss_singleton.Get().IsTPMTokenEnabledForNSS();
 }
 
-bool IsTPMTokenReady(const base::Closure& callback) {
-  return g_nss_singleton.Get().IsTPMTokenReady(callback);
+bool IsTPMTokenReady(base::OnceClosure callback) {
+  return g_nss_singleton.Get().IsTPMTokenReady(std::move(callback));
 }
 
-void InitializeTPMTokenAndSystemSlot(
-    int token_slot_id,
-    const base::Callback<void(bool)>& callback) {
+void InitializeTPMTokenAndSystemSlot(int token_slot_id,
+                                     base::OnceCallback<void(bool)> callback) {
   g_nss_singleton.Get().InitializeTPMTokenAndSystemSlot(token_slot_id,
-                                                        callback);
+                                                        std::move(callback));
 }
 
 bool InitializeNSSForChromeOSUser(const std::string& username_hash,
@@ -878,14 +872,19 @@ ScopedPK11Slot GetPublicSlotForChromeOSUser(const std::string& username_hash) {
 
 ScopedPK11Slot GetPrivateSlotForChromeOSUser(
     const std::string& username_hash,
-    const base::Callback<void(ScopedPK11Slot)>& callback) {
-  return g_nss_singleton.Get().GetPrivateSlotForChromeOSUser(username_hash,
-                                                             callback);
+    base::OnceCallback<void(ScopedPK11Slot)> callback) {
+  return g_nss_singleton.Get().GetPrivateSlotForChromeOSUser(
+      username_hash, std::move(callback));
 }
 
 void CloseChromeOSUserForTesting(const std::string& username_hash) {
   g_nss_singleton.Get().CloseChromeOSUserForTesting(username_hash);
 }
+
+void SetPrivateSoftwareSlotForChromeOSUserForTesting(ScopedPK11Slot slot) {
+  g_nss_singleton.Get().SetPrivateSoftwareSlotForChromeOSUserForTesting(
+      std::move(slot));
+}
 #endif  // defined(OS_CHROMEOS)
 
 base::Time PRTimeToBaseTime(PRTime prtime) {
