@@ -4,10 +4,14 @@
 
 #include "extensions/browser/guest_view/web_view/web_view_apitest.h"
 
+#include <memory>
+#include <string>
 #include <utility>
 
 #include "base/command_line.h"
+#include "base/feature_list.h"
 #include "base/macros.h"
+#include "base/memory/scoped_refptr.h"
 #include "base/path_service.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
@@ -17,9 +21,13 @@
 #include "components/guest_view/browser/guest_view_manager_delegate.h"
 #include "components/guest_view/browser/guest_view_manager_factory.h"
 #include "components/guest_view/browser/test_guest_view_manager.h"
+#include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
+#include "content/public/browser/render_view_host.h"
+#include "content/public/browser/render_widget_host_view.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/test/browser_test_utils.h"
+#include "content/public/test/hit_test_region_observer.h"
 #include "content/public/test/test_renderer_host.h"
 #include "content/public/test/test_utils.h"
 #include "extensions/browser/api/extensions_api_client.h"
@@ -35,12 +43,19 @@
 #include "extensions/shell/browser/shell_extension_system.h"
 #include "extensions/shell/test/shell_test.h"
 #include "extensions/test/extension_test_message_listener.h"
+#include "extensions/test/result_catcher.h"
 #include "net/base/filename_util.h"
+#include "net/test/embedded_test_server/default_handlers.h"
 #include "net/test/embedded_test_server/embedded_test_server.h"
 #include "net/test/embedded_test_server/http_request.h"
 #include "net/test/embedded_test_server/http_response.h"
+#include "services/network/public/cpp/features.h"
 #include "ui/display/display_switches.h"
 
+#if defined(USE_AURA)
+#include "third_party/blink/public/platform/web_mouse_event.h"
+#endif
+
 using guest_view::GuestViewManager;
 using guest_view::TestGuestViewManager;
 
@@ -84,9 +99,11 @@ class WebContentsHiddenObserver : public content::WebContentsObserver {
   }
 
   // WebContentsObserver.
-  void WasHidden() override {
-    hidden_observed_ = true;
-    hidden_callback_.Run();
+  void OnVisibilityChanged(content::Visibility visibility) override {
+    if (visibility == content::Visibility::HIDDEN) {
+      hidden_observed_ = true;
+      hidden_callback_.Run();
+    }
   }
 
   bool hidden_observed() { return hidden_observed_; }
@@ -136,9 +153,9 @@ WebViewAPITest::WebViewAPITest() {
 }
 
 void WebViewAPITest::LaunchApp(const std::string& app_location) {
-  base::ThreadRestrictions::ScopedAllowIO allow_io;
+  base::ScopedAllowBlockingForTesting allow_blocking;
   base::FilePath test_data_dir;
-  PathService::Get(DIR_TEST_DATA, &test_data_dir);
+  base::PathService::Get(DIR_TEST_DATA, &test_data_dir);
   test_data_dir = test_data_dir.AppendASCII(app_location.c_str());
 
   const Extension* extension = extension_system_->LoadApp(test_data_dir);
@@ -160,16 +177,26 @@ content::WebContents* WebViewAPITest::GetFirstAppWindowWebContents() {
 }
 
 void WebViewAPITest::RunTest(const std::string& test_name,
-                             const std::string& app_location) {
+                             const std::string& app_location,
+                             bool ad_hoc_framework) {
   LaunchApp(app_location);
 
-  ExtensionTestMessageListener done_listener("TEST_PASSED", false);
-  done_listener.set_failure_message("TEST_FAILED");
-  ASSERT_TRUE(content::ExecuteScript(
-      embedder_web_contents_,
-      base::StringPrintf("runTest('%s')", test_name.c_str())))
-      << "Unable to start test.";
-  ASSERT_TRUE(done_listener.WaitUntilSatisfied());
+  if (ad_hoc_framework) {
+    ExtensionTestMessageListener done_listener("TEST_PASSED", false);
+    done_listener.set_failure_message("TEST_FAILED");
+    ASSERT_TRUE(content::ExecuteScript(
+        embedder_web_contents_,
+        base::StringPrintf("runTest('%s')", test_name.c_str())))
+        << "Unable to start test.";
+    ASSERT_TRUE(done_listener.WaitUntilSatisfied());
+  } else {
+    ResultCatcher catcher;
+    ASSERT_TRUE(content::ExecuteScript(
+        embedder_web_contents_,
+        base::StringPrintf("runTest('%s')", test_name.c_str())))
+        << "Unable to start test.";
+    ASSERT_TRUE(catcher.GetNextResult()) << catcher.message();
+  }
 }
 
 void WebViewAPITest::SetUpCommandLine(base::CommandLine* command_line) {
@@ -192,9 +219,9 @@ void WebViewAPITest::StartTestServer(const std::string& app_location) {
 
   test_config_.SetInteger(kTestServerPort, embedded_test_server()->port());
 
-  base::ThreadRestrictions::ScopedAllowIO allow_io;
+  base::ScopedAllowBlockingForTesting allow_blocking;
   base::FilePath test_data_dir;
-  PathService::Get(DIR_TEST_DATA, &test_data_dir);
+  base::PathService::Get(DIR_TEST_DATA, &test_data_dir);
   test_data_dir = test_data_dir.AppendASCII(app_location.c_str());
   embedded_test_server()->ServeFilesFromDirectory(test_data_dir);
 
@@ -212,6 +239,8 @@ void WebViewAPITest::StartTestServer(const std::string& app_location) {
           kUserAgentRedirectResponsePath,
           embedded_test_server()->GetURL(kRedirectResponseFullPath)));
 
+  net::test_server::RegisterDefaultHandlers(embedded_test_server());
+
   embedded_test_server()->StartAcceptingConnections();
 }
 
@@ -286,53 +315,18 @@ content::WebContents* WebViewAPITest::GetGuestWebContents() {
   return GetGuestViewManager()->WaitForSingleGuestCreated();
 }
 
-// Occasionally hits NOTIMPLEMENTED on Linux.  https://crbug.com/422998
-#if defined(OS_LINUX)
-#define MAYBE_AcceptTouchEvents DISABLED_AcceptTouchEvents
-#else
-#define MAYBE_AcceptTouchEvents AcceptTouchEvents
-#endif
-IN_PROC_BROWSER_TEST_F(WebViewAPITest, MAYBE_AcceptTouchEvents) {
-  LaunchApp("web_view/accept_touch_events");
-
-  content::RenderViewHost* embedder_rvh =
-      GetEmbedderWebContents()->GetRenderViewHost();
-
-  bool embedder_has_touch_handler =
-      content::RenderViewHostTester::HasTouchEventHandler(embedder_rvh);
-  EXPECT_FALSE(embedder_has_touch_handler);
-
-  SendMessageToGuestAndWait("install-touch-handler", "installed-touch-handler");
-
-  // Note that we need to wait for the installed/registered touch handler to
-  // appear in browser process before querying |embedder_rvh|.
-  // In practice, since we do a roundrtip from browser process to guest and
-  // back, this is sufficient.
-  embedder_has_touch_handler =
-      content::RenderViewHostTester::HasTouchEventHandler(embedder_rvh);
-  EXPECT_TRUE(embedder_has_touch_handler);
-
-  SendMessageToGuestAndWait("uninstall-touch-handler",
-                            "uninstalled-touch-handler");
-  // Same as the note above about waiting.
-  embedder_has_touch_handler =
-      content::RenderViewHostTester::HasTouchEventHandler(embedder_rvh);
-  EXPECT_FALSE(embedder_has_touch_handler);
-}
-
 // This test verifies that hiding the embedder also hides the guest.
 IN_PROC_BROWSER_TEST_F(WebViewAPITest, EmbedderVisibilityChanged) {
   LaunchApp("web_view/visibility_changed");
 
-  scoped_refptr<content::MessageLoopRunner> loop_runner(
-      new content::MessageLoopRunner);
+  base::RunLoop run_loop;
   WebContentsHiddenObserver observer(GetGuestWebContents(),
-                                     loop_runner->QuitClosure());
+                                     run_loop.QuitClosure());
 
   // Handled in web_view/visibility_changed/main.js
   SendMessageToEmbedder("hide-embedder");
   if (!observer.hidden_observed())
-    loop_runner->Run();
+    run_loop.Run();
 }
 
 // Test for http://crbug.com/419611.
@@ -356,15 +350,14 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, DisplayNoneSetSrc) {
 IN_PROC_BROWSER_TEST_F(WebViewAPITest, GuestVisibilityChanged) {
   LaunchApp("web_view/visibility_changed");
 
-  scoped_refptr<content::MessageLoopRunner> loop_runner(
-      new content::MessageLoopRunner);
+  base::RunLoop run_loop;
   WebContentsHiddenObserver observer(GetGuestWebContents(),
-                                     loop_runner->QuitClosure());
+                                     run_loop.QuitClosure());
 
   // Handled in web_view/visibility_changed/main.js
   SendMessageToEmbedder("hide-guest");
   if (!observer.hidden_observed())
-    loop_runner->Run();
+    run_loop.Run();
 }
 
 // This test ensures that closing app window on 'loadcommit' does not crash.
@@ -404,6 +397,10 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestAPIMethodExistence) {
   RunTest("testAPIMethodExistence", "web_view/apitest");
 }
 
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestCustomElementCallbacksInaccessible) {
+  RunTest("testCustomElementCallbacksInaccessible", "web_view/apitest");
+}
+
 IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestAssignSrcAfterCrash) {
   RunTest("testAssignSrcAfterCrash", "web_view/apitest");
 }
@@ -458,6 +455,39 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestContentLoadEvent) {
   RunTest("testContentLoadEvent", "web_view/apitest");
 }
 
+#if defined(USE_AURA)
+// Verifies that trying to show the context menu doesn't crash
+// (https://crbug.com/820604).
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestContextMenu) {
+  // Launch some test app that displays a webview.
+  LaunchApp("web_view/visibility_changed");
+
+  // Ensure the webview's surface is ready for hit testing.
+  content::WebContents* guest_web_contents = GetGuestWebContents();
+  content::WaitForHitTestDataOrGuestSurfaceReady(guest_web_contents);
+
+  // Register a ContextMenuFilter to wait for the context menu event to be sent.
+  content::RenderProcessHost* guest_process_host =
+      guest_web_contents->GetMainFrame()->GetProcess();
+  auto context_menu_filter = base::MakeRefCounted<content::ContextMenuFilter>();
+  guest_process_host->AddFilter(context_menu_filter.get());
+
+  // Trigger the context menu. AppShell doesn't show a context menu; this is
+  // just a sanity check that nothing breaks.
+  content::WebContents* root_web_contents =
+      guest_web_contents->GetOutermostWebContents();
+  content::RenderWidgetHostView* guest_view =
+      guest_web_contents->GetRenderWidgetHostView();
+  gfx::Point guest_context_menu_position(5, 5);
+  gfx::Point root_context_menu_position =
+      guest_view->TransformPointToRootCoordSpace(guest_context_menu_position);
+  content::SimulateRoutedMouseClickAt(
+      root_web_contents, blink::WebInputEvent::kNoModifiers,
+      blink::WebMouseEvent::Button::kRight, root_context_menu_position);
+  context_menu_filter->Wait();
+}
+#endif
+
 IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestDeclarativeWebRequestAPI) {
   std::string app_location = "web_view/apitest";
   StartTestServer(app_location);
@@ -505,7 +535,9 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestDisplayNoneWebviewLoad) {
   RunTest("testDisplayNoneWebviewLoad", "web_view/apitest");
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestDisplayNoneWebviewRemoveChild) {
+// Flaky. See http://crbug.com/769467.
+IN_PROC_BROWSER_TEST_F(WebViewAPITest,
+                       DISABLED_TestDisplayNoneWebviewRemoveChild) {
   RunTest("testDisplayNoneWebviewRemoveChild", "web_view/apitest");
 }
 
@@ -586,11 +618,25 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestLoadAbortNonWebSafeScheme) {
   RunTest("testLoadAbortNonWebSafeScheme", "web_view/apitest");
 }
 
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestLoadAbortUnknownScheme) {
+  RunTest("testLoadAbortUnknownScheme", "web_view/apitest");
+}
+
 IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestLoadProgressEvent) {
   RunTest("testLoadProgressEvent", "web_view/apitest");
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestLoadStartLoadRedirect) {
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestCanGoBack) {
+  RunTest("testCanGoBack", "web_view/apitest");
+}
+
+// Crashes on Win only.  http://crbug.com/805903
+#if defined(OS_WIN)
+#define MAYBE_TestLoadStartLoadRedirect DISABLED_TestLoadStartLoadRedirect
+#else
+#define MAYBE_TestLoadStartLoadRedirect TestLoadStartLoadRedirect
+#endif
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, MAYBE_TestLoadStartLoadRedirect) {
   std::string app_location = "web_view/apitest";
   StartTestServer(app_location);
   RunTest("testLoadStartLoadRedirect", app_location);
@@ -601,10 +647,6 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestNavigateAfterResize) {
   RunTest("testNavigateAfterResize", "web_view/apitest");
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestNavigationToExternalProtocol) {
-  RunTest("testNavigationToExternalProtocol", "web_view/apitest");
-}
-
 IN_PROC_BROWSER_TEST_F(WebViewAPITest,
                        TestContentInitiatedNavigationToDataUrlBlocked) {
   RunTest("testContentInitiatedNavigationToDataUrlBlocked", "web_view/apitest");
@@ -681,7 +723,8 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestRemoveWebviewOnExit) {
                                      "runTest('testRemoveWebviewOnExit')"));
 
   content::WebContents* guest_web_contents = GetGuestWebContents();
-  EXPECT_TRUE(guest_web_contents->GetRenderProcessHost()->IsForGuestsOnly());
+  EXPECT_TRUE(
+      guest_web_contents->GetMainFrame()->GetProcess()->IsForGuestsOnly());
   ASSERT_TRUE(guest_loaded_listener.WaitUntilSatisfied());
 
   content::WebContentsDestroyedWatcher destroyed_watcher(guest_web_contents);
@@ -711,7 +754,13 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestRemoveWebviewAfterNavigation) {
   RunTest("testRemoveWebviewAfterNavigation", "web_view/apitest");
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestResizeWebviewResizesContent) {
+#if defined(OS_WIN)
+#define MAYBE_TestResizeWebviewResizesContent \
+  DISABLED_TestResizeWebviewResizesContent
+#else
+#define MAYBE_TestResizeWebviewResizesContent TestResizeWebviewResizesContent
+#endif
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, MAYBE_TestResizeWebviewResizesContent) {
   RunTest("testResizeWebviewResizesContent", "web_view/apitest");
 }
 
@@ -726,13 +775,26 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestWebRequestAPI) {
   StopTestServer();
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestWebRequestAPIWithHeaders) {
+// Crashes on Win only.  http://crbug.com/805903
+#if defined(OS_WIN)
+#define MAYBE_TestWebRequestAPIWithHeaders DISABLED_TestWebRequestAPIWithHeaders
+#else
+#define MAYBE_TestWebRequestAPIWithHeaders TestWebRequestAPIWithHeaders
+#endif
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, MAYBE_TestWebRequestAPIWithHeaders) {
   std::string app_location = "web_view/apitest";
   StartTestServer(app_location);
   RunTest("testWebRequestAPIWithHeaders", app_location);
   StopTestServer();
 }
 
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestWebRequestAPIWithExtraHeaders) {
+  std::string app_location = "web_view/apitest";
+  StartTestServer(app_location);
+  RunTest("testWebRequestAPIWithExtraHeaders", app_location);
+  StopTestServer();
+}
+
 IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestLoadEventsSameDocumentNavigation) {
   std::string app_location = "web_view/apitest";
   StartTestServer(app_location);
@@ -759,4 +821,30 @@ IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestCaptureVisibleRegion) {
   RunTest("testCaptureVisibleRegion", "web_view/apitest");
 }
 
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestNoUserCodeCreate) {
+  RunTest("testCreate", "web_view/no_internal_calls_to_user_code", false);
+}
+
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestNoUserCodeSetOnEventProperty) {
+  RunTest("testSetOnEventProperty", "web_view/no_internal_calls_to_user_code",
+          false);
+}
+
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestNoUserCodeGetSetAttributes) {
+  RunTest("testGetSetAttributes", "web_view/no_internal_calls_to_user_code",
+          false);
+}
+
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestNoUserCodeBackForward) {
+  RunTest("testBackForward", "web_view/no_internal_calls_to_user_code", false);
+}
+
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestNoUserCodeFocus) {
+  RunTest("testFocus", "web_view/no_internal_calls_to_user_code", false);
+}
+
+IN_PROC_BROWSER_TEST_F(WebViewAPITest, TestClosedShadowRoot) {
+  RunTest("testClosedShadowRoot", "web_view/apitest");
+}
+
 }  // namespace extensions
