@@ -7,11 +7,8 @@
 #include <utility>
 
 #include "base/json/json_writer.h"
-#include "base/memory/ptr_util.h"
 #include "base/values.h"
 
-using content::BrowserThread;
-
 namespace {
 
 // To save typing ValueStore::DEFAULTS everywhere.
@@ -57,13 +54,13 @@ testing::AssertionResult SettingsEq(
     const char* _1, const char* _2,
     const base::DictionaryValue& expected,
     ValueStore::ReadResult actual_result) {
-  if (!actual_result->status().ok()) {
-    return testing::AssertionFailure() << "Result has error: "
-                                       << actual_result->status().message;
+  if (!actual_result.status().ok()) {
+    return testing::AssertionFailure()
+           << "Result has error: " << actual_result.status().message;
   }
 
   std::string error;
-  if (!ValuesEqual(&expected, &actual_result->settings(), &error)) {
+  if (!ValuesEqual(&expected, &actual_result.settings(), &error)) {
     return testing::AssertionFailure() << error;
   }
 
@@ -76,12 +73,12 @@ testing::AssertionResult ChangesEq(
     const char* _1, const char* _2,
     const ValueStoreChangeList& expected,
     ValueStore::WriteResult actual_result) {
-  if (!actual_result->status().ok()) {
-    return testing::AssertionFailure() << "Result has error: "
-                                       << actual_result->status().message;
+  if (!actual_result.status().ok()) {
+    return testing::AssertionFailure()
+           << "Result has error: " << actual_result.status().message;
   }
 
-  const ValueStoreChangeList& actual = actual_result->changes();
+  const ValueStoreChangeList& actual = actual_result.changes();
   if (expected.size() != actual.size()) {
     return testing::AssertionFailure() <<
         "Actual has wrong size, expecting " << expected.size() <<
@@ -90,12 +87,11 @@ testing::AssertionResult ChangesEq(
 
   std::map<std::string, std::unique_ptr<ValueStoreChange>> expected_as_map;
   for (const ValueStoreChange& change : expected)
-    expected_as_map[change.key()] = base::MakeUnique<ValueStoreChange>(change);
+    expected_as_map[change.key()] = std::make_unique<ValueStoreChange>(change);
 
   std::set<std::string> keys_seen;
 
-  for (ValueStoreChangeList::const_iterator it = actual.begin();
-      it != actual.end(); ++it) {
+  for (auto it = actual.cbegin(); it != actual.cend(); ++it) {
     if (keys_seen.count(it->key())) {
       return testing::AssertionFailure() <<
           "Multiple changes seen for key: " << it->key();
@@ -130,9 +126,7 @@ ValueStoreTest::ValueStoreTest()
       dict1_(new base::DictionaryValue()),
       dict3_(new base::DictionaryValue()),
       dict12_(new base::DictionaryValue()),
-      dict123_(new base::DictionaryValue()),
-      ui_thread_(BrowserThread::UI, base::MessageLoop::current()),
-      file_thread_(BrowserThread::FILE, base::MessageLoop::current()) {
+      dict123_(new base::DictionaryValue()) {
   val1_.reset(new base::Value(key1_ + "Value"));
   val2_.reset(new base::Value(key2_ + "Value"));
   val3_.reset(new base::Value(key3_ + "Value"));
