@@ -11,9 +11,10 @@
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
 #include "base/json/json_string_value_serializer.h"
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
+#include "base/optional.h"
 #include "base/path_service.h"
+#include "base/stl_util.h"
+#include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
@@ -33,6 +34,13 @@ namespace extensions {
 
 namespace {
 
+const char manifest_content[] =
+    "{\n"
+    "  \"name\": \"Underscore folder test\",\n"
+    "  \"version\": \"1.0\",\n"
+    "  \"manifest_version\": 2\n"
+    "}\n";
+
 scoped_refptr<Extension> LoadExtensionManifest(
     const base::DictionaryValue& manifest,
     const base::FilePath& manifest_dir,
@@ -54,11 +62,51 @@ scoped_refptr<Extension> LoadExtensionManifest(
   std::unique_ptr<base::Value> result = deserializer.Deserialize(NULL, error);
   if (!result.get())
     return NULL;
-  CHECK_EQ(base::Value::Type::DICTIONARY, result->GetType());
+  CHECK_EQ(base::Value::Type::DICTIONARY, result->type());
   return LoadExtensionManifest(*base::DictionaryValue::From(std::move(result)),
                                manifest_dir, location, extra_flags, error);
 }
 
+void RunUnderscoreDirectoriesTest(
+    const std::vector<std::string>& underscore_directories) {
+  base::ScopedTempDir temp;
+  ASSERT_TRUE(temp.CreateUniqueTempDir());
+
+  base::FilePath ext_path = temp.GetPath();
+  ASSERT_TRUE(base::CreateDirectory(ext_path));
+
+  for (const auto& dir : underscore_directories)
+    ASSERT_TRUE(base::CreateDirectory(ext_path.AppendASCII(dir)));
+
+  ASSERT_EQ(static_cast<int>(strlen(manifest_content)),
+            base::WriteFile(ext_path.AppendASCII("manifest.json"),
+                            manifest_content, strlen(manifest_content)));
+
+  std::string error;
+  scoped_refptr<Extension> extension = file_util::LoadExtension(
+      ext_path, Manifest::UNPACKED, Extension::NO_FLAGS, &error);
+  ASSERT_TRUE(extension) << error;
+  EXPECT_TRUE(error.empty());
+
+  const std::vector<InstallWarning>& warnings = extension->install_warnings();
+  ASSERT_EQ(1u, warnings.size());
+
+  // The warning should report any one of the illegal underscore directories.
+  bool warning_matched = false;
+  for (const auto& dir : underscore_directories) {
+    std::string expected_warning = base::StringPrintf(
+        "Cannot load extension with file or directory name %s. Filenames "
+        "starting with \"_\" are reserved for use by the system.",
+        dir.c_str());
+    if (expected_warning == warnings[0].message)
+      warning_matched = true;
+  }
+
+  EXPECT_TRUE(warning_matched)
+      << "Correct warning not generated for an unpacked extension with "
+      << base::JoinString(underscore_directories, ",") << " directories.";
+}
+
 }  // namespace
 
 typedef testing::Test FileUtilTest;
@@ -122,9 +170,21 @@ TEST_F(FileUtilTest, InstallUninstallGarbageCollect) {
   ASSERT_TRUE(base::DirectoryExists(all_extensions));
 }
 
+TEST_F(FileUtilTest, LoadExtensionWithMetadataFolder) {
+  RunUnderscoreDirectoriesTest({"_metadata"});
+}
+
+TEST_F(FileUtilTest, LoadExtensionWithUnderscoreFolder) {
+  RunUnderscoreDirectoriesTest({"_badfolder"});
+}
+
+TEST_F(FileUtilTest, LoadExtensionWithUnderscoreAndMetadataFolder) {
+  RunUnderscoreDirectoriesTest({"_metadata", "_badfolder"});
+}
+
 TEST_F(FileUtilTest, LoadExtensionWithValidLocales) {
   base::FilePath install_dir;
-  ASSERT_TRUE(PathService::Get(DIR_TEST_DATA, &install_dir));
+  ASSERT_TRUE(base::PathService::Get(DIR_TEST_DATA, &install_dir));
   install_dir = install_dir.AppendASCII("extension_with_locales");
 
   std::string error;
@@ -136,7 +196,7 @@ TEST_F(FileUtilTest, LoadExtensionWithValidLocales) {
 
 TEST_F(FileUtilTest, LoadExtensionWithoutLocalesFolder) {
   base::FilePath install_dir;
-  ASSERT_TRUE(PathService::Get(DIR_TEST_DATA, &install_dir));
+  ASSERT_TRUE(base::PathService::Get(DIR_TEST_DATA, &install_dir));
   install_dir = install_dir.AppendASCII("extension_without_locales");
 
   std::string error;
@@ -165,10 +225,10 @@ TEST_F(FileUtilTest, CheckIllegalFilenamesOnlyReserved) {
   base::ScopedTempDir temp;
   ASSERT_TRUE(temp.CreateUniqueTempDir());
 
-  const base::FilePath::CharType* folders[] = {
-      extensions::kLocaleFolder, extensions::kPlatformSpecificFolder};
+  static const base::FilePath::CharType* const folders[] = {
+      kLocaleFolder, kPlatformSpecificFolder};
 
-  for (size_t i = 0; i < arraysize(folders); i++) {
+  for (size_t i = 0; i < base::size(folders); i++) {
     base::FilePath src_path = temp.GetPath().Append(folders[i]);
     ASSERT_TRUE(base::CreateDirectory(src_path));
   }
@@ -181,7 +241,7 @@ TEST_F(FileUtilTest, CheckIllegalFilenamesReservedAndIllegal) {
   base::ScopedTempDir temp;
   ASSERT_TRUE(temp.CreateUniqueTempDir());
 
-  base::FilePath src_path = temp.GetPath().Append(extensions::kLocaleFolder);
+  base::FilePath src_path = temp.GetPath().Append(kLocaleFolder);
   ASSERT_TRUE(base::CreateDirectory(src_path));
 
   src_path = temp.GetPath().AppendASCII("_some_dir");
@@ -228,7 +288,7 @@ TEST_F(FileUtilTest,
 
 TEST_F(FileUtilTest, LoadExtensionGivesHelpfullErrorOnMissingManifest) {
   base::FilePath install_dir;
-  ASSERT_TRUE(PathService::Get(DIR_TEST_DATA, &install_dir));
+  ASSERT_TRUE(base::PathService::Get(DIR_TEST_DATA, &install_dir));
   install_dir =
       install_dir.AppendASCII("file_util").AppendASCII("missing_manifest");
 
@@ -237,12 +297,12 @@ TEST_F(FileUtilTest, LoadExtensionGivesHelpfullErrorOnMissingManifest) {
       install_dir, Manifest::UNPACKED, Extension::NO_FLAGS, &error));
   ASSERT_TRUE(extension.get() == NULL);
   ASSERT_FALSE(error.empty());
-  ASSERT_STREQ("Manifest file is missing or unreadable.", error.c_str());
+  ASSERT_EQ(manifest_errors::kManifestUnreadable, error);
 }
 
 TEST_F(FileUtilTest, LoadExtensionGivesHelpfullErrorOnBadManifest) {
   base::FilePath install_dir;
-  ASSERT_TRUE(PathService::Get(DIR_TEST_DATA, &install_dir));
+  ASSERT_TRUE(base::PathService::Get(DIR_TEST_DATA, &install_dir));
   install_dir =
       install_dir.AppendASCII("file_util").AppendASCII("bad_manifest");
 
@@ -251,10 +311,9 @@ TEST_F(FileUtilTest, LoadExtensionGivesHelpfullErrorOnBadManifest) {
       install_dir, Manifest::UNPACKED, Extension::NO_FLAGS, &error));
   ASSERT_TRUE(extension.get() == NULL);
   ASSERT_FALSE(error.empty());
-  ASSERT_STREQ(
-      "Manifest is not valid JSON.  "
-      "Line: 2, column: 16, Syntax error.",
-      error.c_str());
+  ASSERT_EQ(manifest_errors::kManifestParseError +
+                std::string("  Line: 2, column: 16, Syntax error."),
+            error);
 }
 
 TEST_F(FileUtilTest, ValidateThemeUTF8) {
@@ -277,7 +336,7 @@ TEST_F(FileUtilTest, ValidateThemeUTF8) {
       kManifest, temp.GetPath(), Manifest::UNPACKED, 0, &error);
   ASSERT_TRUE(extension.get()) << error;
 
-  std::vector<extensions::InstallWarning> warnings;
+  std::vector<InstallWarning> warnings;
   EXPECT_TRUE(file_util::ValidateExtension(extension.get(), &error, &warnings))
       << error;
   EXPECT_EQ(0U, warnings.size());
@@ -290,14 +349,14 @@ TEST_F(FileUtilTest, BackgroundScriptsMustExist) {
   std::unique_ptr<base::DictionaryValue> value(new base::DictionaryValue());
   value->SetString("name", "test");
   value->SetString("version", "1");
-  value->SetInteger("manifest_version", 1);
+  value->SetInteger("manifest_version", 2);
 
   base::ListValue* scripts =
-      value->SetList("background.scripts", base::MakeUnique<base::ListValue>());
+      value->SetList("background.scripts", std::make_unique<base::ListValue>());
   scripts->AppendString("foo.js");
 
   std::string error;
-  std::vector<extensions::InstallWarning> warnings;
+  std::vector<InstallWarning> warnings;
   scoped_refptr<Extension> extension = LoadExtensionManifest(
       *value, temp.GetPath(), Manifest::UNPACKED, 0, &error);
   ASSERT_TRUE(extension.get()) << error;
@@ -354,20 +413,20 @@ TEST_F(FileUtilTest, FindPrivateKeyFiles) {
   base::FilePath src_path = temp.GetPath().AppendASCII("some_dir");
   ASSERT_TRUE(base::CreateDirectory(src_path));
 
-  ASSERT_EQ(static_cast<int>(arraysize(private_key)),
+  ASSERT_EQ(static_cast<int>(base::size(private_key)),
             base::WriteFile(src_path.AppendASCII("a_key.pem"), private_key,
-                            arraysize(private_key)));
-  ASSERT_EQ(static_cast<int>(arraysize(private_key)),
+                            base::size(private_key)));
+  ASSERT_EQ(static_cast<int>(base::size(private_key)),
             base::WriteFile(src_path.AppendASCII("second_key.pem"), private_key,
-                            arraysize(private_key)));
+                            base::size(private_key)));
   // Shouldn't find a key with a different extension.
-  ASSERT_EQ(static_cast<int>(arraysize(private_key)),
+  ASSERT_EQ(static_cast<int>(base::size(private_key)),
             base::WriteFile(src_path.AppendASCII("key.diff_ext"), private_key,
-                            arraysize(private_key)));
+                            base::size(private_key)));
   // Shouldn't find a key that isn't parsable.
-  ASSERT_EQ(static_cast<int>(arraysize(private_key)) - 30,
+  ASSERT_EQ(static_cast<int>(base::size(private_key)) - 30,
             base::WriteFile(src_path.AppendASCII("unparsable_key.pem"),
-                            private_key, arraysize(private_key) - 30));
+                            private_key, base::size(private_key) - 30));
   std::vector<base::FilePath> private_keys =
       file_util::FindPrivateKeyFiles(temp.GetPath());
   EXPECT_EQ(2U, private_keys.size());
@@ -409,7 +468,7 @@ TEST_F(FileUtilTest, WarnOnPrivateKey) {
   ASSERT_EQ(1u, extension->install_warnings().size());
   EXPECT_THAT(extension->install_warnings(),
               testing::ElementsAre(testing::Field(
-                  &extensions::InstallWarning::message,
+                  &InstallWarning::message,
                   testing::ContainsRegex(
                       "extension includes the key file.*ext_root.a_key.pem"))));
 
@@ -428,7 +487,7 @@ TEST_F(FileUtilTest, WarnOnPrivateKey) {
 // Try to install an extension with a zero-length icon file.
 TEST_F(FileUtilTest, CheckZeroLengthAndMissingIconFile) {
   base::FilePath install_dir;
-  ASSERT_TRUE(PathService::Get(DIR_TEST_DATA, &install_dir));
+  ASSERT_TRUE(base::PathService::Get(DIR_TEST_DATA, &install_dir));
 
   base::FilePath ext_dir =
       install_dir.AppendASCII("file_util").AppendASCII("bad_icon");
@@ -442,7 +501,7 @@ TEST_F(FileUtilTest, CheckZeroLengthAndMissingIconFile) {
 // Try to install an unpacked extension with a zero-length icon file.
 TEST_F(FileUtilTest, CheckZeroLengthAndMissingIconFileUnpacked) {
   base::FilePath install_dir;
-  ASSERT_TRUE(PathService::Get(DIR_TEST_DATA, &install_dir));
+  ASSERT_TRUE(base::PathService::Get(DIR_TEST_DATA, &install_dir));
 
   base::FilePath ext_dir =
       install_dir.AppendASCII("file_util").AppendASCII("bad_icon");
@@ -454,6 +513,45 @@ TEST_F(FileUtilTest, CheckZeroLengthAndMissingIconFileUnpacked) {
   EXPECT_EQ("Could not load extension icon 'missing-icon.png'.", error);
 }
 
+// Try to install an unpacked extension with an invisible icon. This
+// should fail.
+TEST_F(FileUtilTest, CheckInvisibleIconFileUnpacked) {
+  base::FilePath install_dir;
+  ASSERT_TRUE(base::PathService::Get(DIR_TEST_DATA, &install_dir));
+
+  base::FilePath ext_dir =
+      install_dir.AppendASCII("file_util").AppendASCII("invisible_icon");
+
+  // Set the flag that enables the error.
+  file_util::SetReportErrorForInvisibleIconForTesting(true);
+  std::string error;
+  scoped_refptr<Extension> extension(file_util::LoadExtension(
+      ext_dir, Manifest::UNPACKED, Extension::NO_FLAGS, &error));
+  file_util::SetReportErrorForInvisibleIconForTesting(false);
+  EXPECT_FALSE(extension);
+  EXPECT_EQ("The icon is not sufficiently visible 'invisible_icon.png'.",
+            error);
+}
+
+// Try to install a packed extension with an invisible icon. This should
+// succeed.
+TEST_F(FileUtilTest, CheckInvisibleIconFilePacked) {
+  base::FilePath install_dir;
+  ASSERT_TRUE(base::PathService::Get(DIR_TEST_DATA, &install_dir));
+
+  base::FilePath ext_dir =
+      install_dir.AppendASCII("file_util").AppendASCII("invisible_icon");
+
+  // Set the flag that enables the error.
+  file_util::SetReportErrorForInvisibleIconForTesting(true);
+  std::string error;
+  scoped_refptr<Extension> extension(file_util::LoadExtension(
+      ext_dir, Manifest::INTERNAL, Extension::NO_FLAGS, &error));
+  file_util::SetReportErrorForInvisibleIconForTesting(false);
+  EXPECT_TRUE(extension);
+  EXPECT_TRUE(error.empty());
+}
+
 TEST_F(FileUtilTest, ExtensionURLToRelativeFilePath) {
 #define URL_PREFIX "chrome-extension://extension-id/"
   struct TestCase {
@@ -477,12 +575,11 @@ TEST_F(FileUtilTest, ExtensionURLToRelativeFilePath) {
   };
 #undef URL_PREFIX
 
-  for (size_t i = 0; i < arraysize(test_cases); ++i) {
+  for (size_t i = 0; i < base::size(test_cases); ++i) {
     GURL url(test_cases[i].url);
     base::FilePath expected_path =
         base::FilePath::FromUTF8Unsafe(test_cases[i].expected_relative_path);
-    base::FilePath actual_path =
-        extensions::file_util::ExtensionURLToRelativeFilePath(url);
+    base::FilePath actual_path = file_util::ExtensionURLToRelativeFilePath(url);
     EXPECT_FALSE(actual_path.IsAbsolute()) <<
       " For the path " << actual_path.value();
     EXPECT_EQ(expected_path.value(), actual_path.value()) <<
