@@ -8,8 +8,9 @@
 
 #include <ostream>
 
-#include "base/macros.h"
+#include "base/stl_util.h"
 #include "base/strings/pattern.h"
+#include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
@@ -17,6 +18,7 @@
 #include "content/public/common/url_constants.h"
 #include "extensions/common/constants.h"
 #include "net/base/registry_controlled_domains/registry_controlled_domain.h"
+#include "net/base/url_util.h"
 #include "url/gurl.h"
 #include "url/url_util.h"
 
@@ -24,14 +26,14 @@ const char URLPattern::kAllUrlsPattern[] = "<all_urls>";
 
 namespace {
 
-// TODO(aa): What about more obscure schemes like data: and javascript: ?
+// TODO(aa): What about more obscure schemes like javascript: ?
 // Note: keep this array in sync with kValidSchemeMasks.
 const char* const kValidSchemes[] = {
     url::kHttpScheme,         url::kHttpsScheme,
     url::kFileScheme,         url::kFtpScheme,
     content::kChromeUIScheme, extensions::kExtensionScheme,
     url::kFileSystemScheme,   url::kWsScheme,
-    url::kWssScheme,
+    url::kWssScheme,          url::kDataScheme,
 };
 
 const int kValidSchemeMasks[] = {
@@ -39,10 +41,10 @@ const int kValidSchemeMasks[] = {
     URLPattern::SCHEME_FILE,       URLPattern::SCHEME_FTP,
     URLPattern::SCHEME_CHROMEUI,   URLPattern::SCHEME_EXTENSION,
     URLPattern::SCHEME_FILESYSTEM, URLPattern::SCHEME_WS,
-    URLPattern::SCHEME_WSS,
+    URLPattern::SCHEME_WSS,        URLPattern::SCHEME_DATA,
 };
 
-static_assert(arraysize(kValidSchemes) == arraysize(kValidSchemeMasks),
+static_assert(base::size(kValidSchemes) == base::size(kValidSchemeMasks),
               "must keep these arrays in sync");
 
 const char kParseSuccess[] = "Success.";
@@ -68,7 +70,8 @@ const char* const kParseResultMessages[] = {
   kParseErrorInvalidHost,
 };
 
-static_assert(URLPattern::NUM_PARSE_RESULTS == arraysize(kParseResultMessages),
+static_assert(static_cast<int>(URLPattern::ParseResult::kNumParseResults) ==
+                  base::size(kParseResultMessages),
               "must add message for each parse result");
 
 const char kPathSeparator[] = "/";
@@ -122,7 +125,7 @@ base::StringPiece CanonicalizeHostForMatching(base::StringPiece host_piece) {
 
 // static
 bool URLPattern::IsValidSchemeForExtensions(base::StringPiece scheme) {
-  for (size_t i = 0; i < arraysize(kValidSchemes); ++i) {
+  for (size_t i = 0; i < base::size(kValidSchemes); ++i) {
     if (scheme == kValidSchemes[i])
       return true;
   }
@@ -132,7 +135,7 @@ bool URLPattern::IsValidSchemeForExtensions(base::StringPiece scheme) {
 // static
 int URLPattern::GetValidSchemeMaskForExtensions() {
   int result = 0;
-  for (size_t i = 0; i < arraysize(kValidSchemeMasks); ++i)
+  for (size_t i = 0; i < base::size(kValidSchemeMasks); ++i)
     result |= kValidSchemeMasks[i];
   return result;
 }
@@ -160,8 +163,15 @@ URLPattern::URLPattern(int valid_schemes, base::StringPiece pattern)
       match_effective_tld_(true),
       port_("*") {
   ParseResult result = Parse(pattern);
-  if (PARSE_SUCCESS != result)
-    NOTREACHED() << "URLPattern invalid: " << pattern << " result " << result;
+  if (result != ParseResult::kSuccess) {
+    const char* error_string = GetParseResultString(result);
+    // Temporarily add more logging to investigate why this code path is
+    // reached. For http://crbug.com/856948
+    LOG(ERROR) << "Invalid pattern was given " << pattern << " result "
+               << error_string;
+    NOTREACHED() << "URLPattern invalid: '" << pattern
+                 << "'; error: " << error_string;
+  }
 }
 
 URLPattern::URLPattern(const URLPattern& other) = default;
@@ -200,7 +210,7 @@ URLPattern::ParseResult URLPattern::Parse(base::StringPiece pattern,
   // Special case pattern to match every valid URL.
   if (pattern == kAllUrlsPattern) {
     SetMatchAllURLs(true);
-    return PARSE_SUCCESS;
+    return ParseResult::kSuccess;
   }
 
   // Parse out the scheme.
@@ -214,20 +224,20 @@ URLPattern::ParseResult URLPattern::Parse(base::StringPiece pattern,
   }
 
   if (scheme_end_pos == base::StringPiece::npos)
-    return PARSE_ERROR_MISSING_SCHEME_SEPARATOR;
+    return ParseResult::kMissingSchemeSeparator;
 
   if (!SetScheme(pattern.substr(0, scheme_end_pos)))
-    return PARSE_ERROR_INVALID_SCHEME;
+    return ParseResult::kInvalidScheme;
 
   bool standard_scheme = IsStandardScheme(scheme_);
   if (standard_scheme != has_standard_scheme_separator)
-    return PARSE_ERROR_WRONG_SCHEME_SEPARATOR;
+    return ParseResult::kWrongSchemeSeparator;
 
   // Advance past the scheme separator.
   scheme_end_pos +=
       (standard_scheme ? strlen(url::kStandardSchemeSeparator) : 1);
   if (scheme_end_pos >= pattern.size())
-    return PARSE_ERROR_EMPTY_HOST;
+    return ParseResult::kEmptyHost;
 
   // Parse out the host and path.
   size_t host_start_pos = scheme_end_pos;
@@ -252,29 +262,56 @@ URLPattern::ParseResult URLPattern::Parse(base::StringPiece pattern,
 
     // Host is required.
     if (host_start_pos == host_end_pos)
-      return PARSE_ERROR_EMPTY_HOST;
+      return ParseResult::kEmptyHost;
 
     if (host_end_pos == base::StringPiece::npos)
-      return PARSE_ERROR_EMPTY_PATH;
+      return ParseResult::kEmptyPath;
 
-    // TODO(devlin): This whole series is expensive. Luckily we don't do it
-    // *too* often, but it could be optimized.
-    pattern.substr(host_start_pos, host_end_pos - host_start_pos)
-        .CopyToString(&host_);
+    base::StringPiece host_and_port =
+        pattern.substr(host_start_pos, host_end_pos - host_start_pos);
+
+    size_t port_separator_pos = base::StringPiece::npos;
+    if (host_and_port[0] != '[') {
+      // Not IPv6 (either IPv4 or just a normal address).
+      port_separator_pos = host_and_port.find(':');
+    } else {  // IPv6.
+      size_t host_end_pos = host_and_port.find(']');
+      if (host_end_pos == base::StringPiece::npos)
+        return ParseResult::kInvalidHost;
+      if (host_end_pos == 1)
+        return ParseResult::kEmptyHost;
+
+      if (host_end_pos < host_and_port.length() - 1) {
+        // The host isn't the only component. Check for a port. This would
+        // require a ':' to follow the closing ']' from the host.
+        if (host_and_port[host_end_pos + 1] != ':')
+          return ParseResult::kInvalidHost;
+
+        port_separator_pos = host_end_pos + 1;
+      }
+    }
+
+    if (port_separator_pos != base::StringPiece::npos &&
+        !SetPort(host_and_port.substr(port_separator_pos + 1))) {
+      return ParseResult::kInvalidPort;
+    }
+
+    // Note: this substr() will be the entire string if the port position
+    // wasn't found.
+    base::StringPiece host_piece = host_and_port.substr(0, port_separator_pos);
 
     // The first component can optionally be '*' to match all subdomains.
     std::vector<base::StringPiece> host_components = base::SplitStringPiece(
-        host_, ".", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
+        host_piece, ".", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
 
     // Could be empty if the host only consists of whitespace characters.
     if (host_components.empty() ||
         (host_components.size() == 1 && host_components[0].empty()))
-      return PARSE_ERROR_EMPTY_HOST;
+      return ParseResult::kEmptyHost;
 
     if (host_components[0] == "*") {
       match_subdomains_ = true;
-      host_components.erase(host_components.begin(),
-                            host_components.begin() + 1);
+      host_components.erase(host_components.begin());
     }
 
     // If explicitly allowed, the last component can optionally be '*' to
@@ -291,27 +328,34 @@ URLPattern::ParseResult URLPattern::Parse(base::StringPiece pattern,
 
   SetPath(pattern.substr(path_start_pos));
 
-  size_t port_pos = host_.find(':');
-  if (port_pos != std::string::npos) {
-    if (!SetPort(host_.substr(port_pos + 1)))
-      return PARSE_ERROR_INVALID_PORT;
-    host_ = host_.substr(0, port_pos);
-  }
-
   // No other '*' can occur in the host, though. This isn't necessary, but is
   // done as a convenience to developers who might otherwise be confused and
   // think '*' works as a glob in the host.
   if (host_.find('*') != std::string::npos)
-    return PARSE_ERROR_INVALID_HOST_WILDCARD;
+    return ParseResult::kInvalidHostWildcard;
+
+  if (!host_.empty()) {
+    // If |host_| is present (i.e., isn't a wildcard), we need to canonicalize
+    // it.
+    url::CanonHostInfo host_info;
+    host_ = net::CanonicalizeHost(host_, &host_info);
+    // net::CanonicalizeHost() returns an empty string on failure.
+    if (host_.empty())
+      return ParseResult::kInvalidHost;
+  }
 
   // Null characters are not allowed in hosts.
   if (host_.find('\0') != std::string::npos)
-    return PARSE_ERROR_INVALID_HOST;
+    return ParseResult::kInvalidHost;
 
-  return PARSE_SUCCESS;
+  return ParseResult::kSuccess;
 }
 
 void URLPattern::SetValidSchemes(int valid_schemes) {
+  // TODO(devlin): Should we check that valid_schemes agrees with |scheme_|
+  // here? Otherwise, valid_schemes_ and schemes_ may stop agreeing with each
+  // other (e.g., in the case of `*://*/*`, where the scheme should only be
+  // http or https).
   spec_.clear();
   valid_schemes_ = valid_schemes;
 }
@@ -358,7 +402,7 @@ bool URLPattern::IsValidScheme(base::StringPiece scheme) const {
   if (valid_schemes_ == SCHEME_ALL)
     return true;
 
-  for (size_t i = 0; i < arraysize(kValidSchemes); ++i) {
+  for (size_t i = 0; i < base::size(kValidSchemes); ++i) {
     if (scheme == kValidSchemes[i] && (valid_schemes_ & kValidSchemeMasks[i]))
       return true;
   }
@@ -393,6 +437,8 @@ bool URLPattern::MatchesURL(const GURL& test) const {
     test_url = test.inner_url();
   }
 
+  // Ensure the scheme matches first, since <all_urls> may not match this URL if
+  // the scheme is excluded.
   if (!MatchesScheme(test_url->scheme_piece()))
     return false;
 
@@ -436,7 +482,10 @@ bool URLPattern::MatchesScheme(base::StringPiece test) const {
 }
 
 bool URLPattern::MatchesHost(base::StringPiece host) const {
-  // TODO(devlin): This is a bit sad. Parsing urls is expensive.
+  // TODO(devlin): This is a bit sad. Parsing urls is expensive. However, it's
+  // important that we do this conversion to a GURL in order to canonicalize the
+  // host (the pattern's host_ already is canonicalized from Parse()). We can't
+  // just do string comparison.
   return MatchesHost(
       GURL(base::StringPrintf("%s%s%s/", url::kHttpScheme,
                               url::kStandardSchemeSeparator, host.data())));
@@ -485,41 +534,39 @@ bool URLPattern::MatchesHost(const GURL& test) const {
   return test_host[test_host.length() - pattern_host.length() - 1] == '.';
 }
 
-bool URLPattern::ImpliesAllHosts() const {
+bool URLPattern::MatchesEffectiveTld(
+    net::registry_controlled_domains::PrivateRegistryFilter private_filter,
+    net::registry_controlled_domains::UnknownRegistryFilter unknown_filter)
+    const {
   // Check if it matches all urls or is a pattern like http://*/*.
-  if (match_all_urls_ ||
-      (match_subdomains_ && host_.empty() && port_ == "*" && path_ == "/*")) {
+  if (match_all_urls_ || (match_subdomains_ && host_.empty()))
     return true;
-  }
 
-  // If this doesn't even match subdomains, it can't possibly imply all hosts.
+  // If this doesn't even match subdomains, it can't possibly be a TLD wildcard.
   if (!match_subdomains_)
     return false;
 
   // If there was more than just a TLD in the host (e.g., *.foobar.com), it
-  // doesn't imply all hosts. We don't include private TLDs, so that, e.g.,
-  // *.appspot.com does not imply all hosts.
+  // doesn't match all hosts in an effective TLD.
   if (net::registry_controlled_domains::HostHasRegistryControlledDomain(
-          host_, net::registry_controlled_domains::EXCLUDE_UNKNOWN_REGISTRIES,
-          net::registry_controlled_domains::EXCLUDE_PRIVATE_REGISTRIES))
+          host_, unknown_filter, private_filter)) {
     return false;
+  }
 
   // At this point the host could either be just a TLD ("com") or some unknown
   // TLD-like string ("notatld"). To disambiguate between them construct a
   // fake URL, and check the registry.
   //
   // If we recognized this TLD, then this is a pattern like *.com, and it
-  // should imply all hosts.
+  // matches an effective TLD.
   return net::registry_controlled_domains::HostHasRegistryControlledDomain(
-      "notatld." + host_,
-      net::registry_controlled_domains::EXCLUDE_UNKNOWN_REGISTRIES,
-      net::registry_controlled_domains::EXCLUDE_PRIVATE_REGISTRIES);
+      "notatld." + host_, unknown_filter, private_filter);
 }
 
 bool URLPattern::MatchesSingleOrigin() const {
   // Strictly speaking, the port is part of the origin, but in URLPattern it
   // defaults to *. It's not very interesting anyway, so leave it out.
-  return !ImpliesAllHosts() && scheme_ != "*" && !match_subdomains_;
+  return !MatchesEffectiveTld() && scheme_ != "*" && !match_subdomains_;
 }
 
 bool URLPattern::MatchesPath(base::StringPiece test) const {
@@ -591,8 +638,14 @@ bool URLPattern::OverlapsWith(const URLPattern& other) const {
 }
 
 bool URLPattern::Contains(const URLPattern& other) const {
-  if (match_all_urls())
+  // Important: it's not enough to just check match_all_urls(); we also need to
+  // make sure that the schemes in this pattern are a superset of those in
+  // |other|.
+  if (match_all_urls() &&
+      (valid_schemes_ & other.valid_schemes_) == other.valid_schemes_) {
     return true;
+  }
+
   return MatchesAllSchemes(other.GetExplicitSchemes()) &&
          MatchesHost(other.host()) &&
          (!other.match_subdomains_ || match_subdomains_) &&
@@ -600,10 +653,113 @@ bool URLPattern::Contains(const URLPattern& other) const {
          MatchesPath(StripTrailingWildcard(other.path()));
 }
 
+base::Optional<URLPattern> URLPattern::CreateIntersection(
+    const URLPattern& other) const {
+  DCHECK(match_effective_tld_);
+  DCHECK(other.match_effective_tld_);
+
+  // Easy case: Schemes don't overlap. Return nullopt.
+  int intersection_schemes = URLPattern::SCHEME_NONE;
+  if (valid_schemes_ == URLPattern::SCHEME_ALL)
+    intersection_schemes = other.valid_schemes_;
+  else if (other.valid_schemes_ == URLPattern::SCHEME_ALL)
+    intersection_schemes = valid_schemes_;
+  else
+    intersection_schemes = valid_schemes_ & other.valid_schemes_;
+
+  if (intersection_schemes == URLPattern::SCHEME_NONE)
+    return base::nullopt;
+
+  {
+    // In a few cases, we can (mostly) return a copy of one of the patterns.
+    // This can happen when either:
+    // - The URLPattern's are identical (possibly excluding valid_schemes_)
+    // - One of the patterns has match_all_urls() equal to true.
+    // NOTE(devlin): Theoretically, we could use Contains() instead of
+    // match_all_urls() here. However, Contains() strips the trailing wildcard
+    // from the path, which could yield the incorrect result.
+    const URLPattern* copy_source = nullptr;
+    if (*this == other || other.match_all_urls())
+      copy_source = this;
+    else if (match_all_urls())
+      copy_source = &other;
+
+    if (copy_source) {
+      // NOTE: equality checks don't take into account valid_schemes_, and
+      // schemes can be different in the case of match_all_urls() as well, so
+      // we can't always just return *copy_source.
+      if (intersection_schemes == copy_source->valid_schemes_)
+        return *copy_source;
+      URLPattern result(intersection_schemes);
+      ParseResult parse_result = result.Parse(copy_source->GetAsString());
+      CHECK_EQ(ParseResult::kSuccess, parse_result);
+      return result;
+    }
+  }
+
+  // No more easy cases. Go through component by component to find the patterns
+  // that intersect.
+
+  // Note: Alias the function type (rather than using auto) because
+  // MatchesHost() is overloaded.
+  using match_function_type = bool (URLPattern::*)(base::StringPiece) const;
+
+  auto get_intersection = [this, &other](base::StringPiece own_str,
+                                         base::StringPiece other_str,
+                                         match_function_type match_function,
+                                         base::StringPiece* out) {
+    if ((this->*match_function)(other_str)) {
+      *out = other_str;
+      return true;
+    }
+    if ((other.*match_function)(own_str)) {
+      *out = own_str;
+      return true;
+    }
+    return false;
+  };
+
+  base::StringPiece scheme;
+  base::StringPiece host;
+  base::StringPiece port;
+  base::StringPiece path;
+  // If any pieces fail to overlap, then there is no intersection.
+  if (!get_intersection(scheme_, other.scheme_, &URLPattern::MatchesScheme,
+                        &scheme) ||
+      !get_intersection(host_, other.host_, &URLPattern::MatchesHost, &host) ||
+      !get_intersection(port_, other.port_, &URLPattern::MatchesPortPattern,
+                        &port) ||
+      !get_intersection(path_, other.path_, &URLPattern::MatchesPath, &path)) {
+    return base::nullopt;
+  }
+
+  // Only match subdomains if both patterns match subdomains.
+  base::StringPiece subdomains;
+  if (match_subdomains_ && other.match_subdomains_) {
+    // The host may be empty (e.g., in the case of *://*/* - in that case, only
+    // append '*' instead of '*.'.
+    subdomains = host.empty() ? "*" : "*.";
+  }
+
+  base::StringPiece scheme_separator =
+      IsStandardScheme(scheme) ? url::kStandardSchemeSeparator : ":";
+
+  std::string pattern_str = base::StrCat(
+      {scheme, scheme_separator, subdomains, host, ":", port, path});
+
+  URLPattern pattern(intersection_schemes);
+  ParseResult result = pattern.Parse(pattern_str);
+  // TODO(devlin): I don't think there's any way this should ever fail, but
+  // use a CHECK() to flush any cases out. If nothing crops up, downgrade this
+  // to a DCHECK in M72.
+  CHECK_EQ(ParseResult::kSuccess, result);
+
+  return pattern;
+}
+
 bool URLPattern::MatchesAnyScheme(
     const std::vector<std::string>& schemes) const {
-  for (std::vector<std::string>::const_iterator i = schemes.begin();
-       i != schemes.end(); ++i) {
+  for (auto i = schemes.cbegin(); i != schemes.cend(); ++i) {
     if (MatchesScheme(*i))
       return true;
   }
@@ -613,8 +769,7 @@ bool URLPattern::MatchesAnyScheme(
 
 bool URLPattern::MatchesAllSchemes(
     const std::vector<std::string>& schemes) const {
-  for (std::vector<std::string>::const_iterator i = schemes.begin();
-       i != schemes.end(); ++i) {
+  for (auto i = schemes.cbegin(); i != schemes.cend(); ++i) {
     if (!MatchesScheme(*i))
       return false;
   }
@@ -645,7 +800,7 @@ std::vector<std::string> URLPattern::GetExplicitSchemes() const {
     return result;
   }
 
-  for (size_t i = 0; i < arraysize(kValidSchemes); ++i) {
+  for (size_t i = 0; i < base::size(kValidSchemes); ++i) {
     if (MatchesScheme(kValidSchemes[i])) {
       result.push_back(kValidSchemes[i]);
     }
@@ -672,5 +827,5 @@ std::vector<URLPattern> URLPattern::ConvertToExplicitSchemes() const {
 // static
 const char* URLPattern::GetParseResultString(
     URLPattern::ParseResult parse_result) {
-  return kParseResultMessages[parse_result];
+  return kParseResultMessages[static_cast<int>(parse_result)];
 }
