@@ -8,7 +8,6 @@
 
 #include "base/bind.h"
 #include "base/logging.h"
-#include "base/memory/ptr_util.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/values.h"
@@ -25,7 +24,7 @@
 #include "gin/arguments.h"
 #include "gin/handle.h"
 #include "gin/per_context_data.h"
-#include "third_party/WebKit/public/web/WebUserGestureIndicator.h"
+#include "third_party/blink/public/web/web_user_gesture_indicator.h"
 
 namespace extensions {
 
@@ -62,17 +61,38 @@ std::string GetJSEnumEntryName(const std::string& original) {
   return result;
 }
 
-bool IsContextValid(v8::Local<v8::Context> context) {
-  // If the given context has been disposed, the per-context data has been
-  // deleted, and the context is no longer valid. The APIBinding (which owns
-  // various necessary pieces) should outlive all contexts, so if the context
-  // is valid, associated callbacks should be safe.
-  return gin::PerContextData::From(context) != nullptr;
+struct SignaturePair {
+  std::unique_ptr<APISignature> method_signature;
+  std::unique_ptr<APISignature> callback_signature;
+};
+
+SignaturePair GetAPISignatureFromDictionary(const base::DictionaryValue* dict) {
+  const base::ListValue* params = nullptr;
+  CHECK(dict->GetList("parameters", &params));
+
+  SignaturePair result;
+  result.method_signature = std::make_unique<APISignature>(*params);
+  // If response validation is enabled, parse the callback signature. Otherwise,
+  // there's no reason to, so don't bother.
+  if (result.method_signature->has_callback() &&
+      binding::IsResponseValidationEnabled()) {
+    const base::Value* callback_params = params->GetList().back().FindKeyOfType(
+        "parameters", base::Value::Type::LIST);
+    if (callback_params) {
+      const base::ListValue* params_as_list = nullptr;
+      callback_params->GetAsList(&params_as_list);
+      result.callback_signature =
+          std::make_unique<APISignature>(*params_as_list);
+    }
+  }
+
+  return result;
 }
 
-void CallbackHelper(const v8::FunctionCallbackInfo<v8::Value>& info) {
+void RunAPIBindingHandlerCallback(
+    const v8::FunctionCallbackInfo<v8::Value>& info) {
   gin::Arguments args(info);
-  if (!IsContextValid(args.isolate()->GetCurrentContext()))
+  if (!binding::IsContextValidOrThrowError(args.isolate()->GetCurrentContext()))
     return;
 
   v8::Local<v8::External> external;
@@ -183,8 +203,8 @@ APIBinding::APIBinding(const std::string& api_name,
                        const base::ListValue* type_definitions,
                        const base::ListValue* event_definitions,
                        const base::DictionaryValue* property_definitions,
-                       const CreateCustomType& create_custom_type,
-                       const OnSilentRequest& on_silent_request,
+                       CreateCustomType create_custom_type,
+                       OnSilentRequest on_silent_request,
                        std::unique_ptr<APIBindingHooks> binding_hooks,
                        APITypeReferenceMap* type_refs,
                        APIRequestHandler* request_handler,
@@ -192,8 +212,8 @@ APIBinding::APIBinding(const std::string& api_name,
                        BindingAccessChecker* access_checker)
     : api_name_(api_name),
       property_definitions_(property_definitions),
-      create_custom_type_(create_custom_type),
-      on_silent_request_(on_silent_request),
+      create_custom_type_(std::move(create_custom_type)),
+      on_silent_request_(std::move(on_silent_request)),
       binding_hooks_(std::move(binding_hooks)),
       type_refs_(type_refs),
       request_handler_(request_handler),
@@ -213,20 +233,23 @@ APIBinding::APIBinding(const std::string& api_name,
       std::string name;
       CHECK(func_dict->GetString("name", &name));
 
-      const base::ListValue* params = nullptr;
-      CHECK(func_dict->GetList("parameters", &params));
+      SignaturePair signatures = GetAPISignatureFromDictionary(func_dict);
 
       bool for_io_thread = false;
       func_dict->GetBoolean("forIOThread", &for_io_thread);
 
-      auto signature = base::MakeUnique<APISignature>(*params);
       std::string full_name =
           base::StringPrintf("%s.%s", api_name_.c_str(), name.c_str());
-      methods_[name] = base::MakeUnique<MethodData>(
-          full_name, signature.get(),
+      methods_[name] = std::make_unique<MethodData>(
+          full_name, signatures.method_signature.get(),
           for_io_thread ? binding::RequestThread::IO
                         : binding::RequestThread::UI);
-      type_refs->AddAPIMethodSignature(full_name, std::move(signature));
+      type_refs->AddAPIMethodSignature(full_name,
+                                       std::move(signatures.method_signature));
+      if (signatures.callback_signature) {
+        type_refs->AddCallbackSignature(
+            full_name, std::move(signatures.callback_signature));
+      }
     }
   }
 
@@ -236,7 +259,7 @@ APIBinding::APIBinding(const std::string& api_name,
       CHECK(type.GetAsDictionary(&type_dict));
       std::string id;
       CHECK(type_dict->GetString("id", &id));
-      auto argument_spec = base::MakeUnique<ArgumentSpec>(*type_dict);
+      auto argument_spec = std::make_unique<ArgumentSpec>(*type_dict);
       const std::set<std::string>& enum_values = argument_spec->enum_values();
       if (!enum_values.empty()) {
         // Type names may be prefixed by the api name. If so, remove the prefix.
@@ -262,11 +285,16 @@ APIBinding::APIBinding(const std::string& api_name,
           std::string function_name;
           CHECK(func_dict->GetString("name", &function_name));
 
-          const base::ListValue* params = nullptr;
-          CHECK(func_dict->GetList("parameters", &params));
+          SignaturePair signatures = GetAPISignatureFromDictionary(func_dict);
+
+          std::string full_name =
+              base::StringPrintf("%s.%s", id.c_str(), function_name.c_str());
           type_refs->AddTypeMethodSignature(
-              base::StringPrintf("%s.%s", id.c_str(), function_name.c_str()),
-              base::MakeUnique<APISignature>(*params));
+              full_name, std::move(signatures.method_signature));
+          if (signatures.callback_signature) {
+            type_refs->AddCallbackSignature(
+                full_name, std::move(signatures.callback_signature));
+          }
         }
       }
     }
@@ -330,7 +358,7 @@ APIBinding::APIBinding(const std::string& api_name,
         }
       }
 
-      events_.push_back(base::MakeUnique<EventData>(
+      events_.push_back(std::make_unique<EventData>(
           std::move(name), std::move(full_name), supports_filters,
           supports_rules, supports_lazy_listeners, max_listeners,
           notify_on_change, std::move(rule_actions), std::move(rule_conditions),
@@ -343,7 +371,7 @@ APIBinding::~APIBinding() {}
 
 v8::Local<v8::Object> APIBinding::CreateInstance(
     v8::Local<v8::Context> context) {
-  DCHECK(IsContextValid(context));
+  DCHECK(binding::IsContextValid(context));
   v8::Isolate* isolate = context->GetIsolate();
   if (object_template_.IsEmpty())
     InitializeTemplate(isolate);
@@ -375,6 +403,8 @@ v8::Local<v8::Object> APIBinding::CreateInstance(
     }
   }
 
+  binding_hooks_->InitializeInstance(context, object);
+
   return object;
 }
 
@@ -387,12 +417,12 @@ void APIBinding::InitializeTemplate(v8::Isolate* isolate) {
     MethodData& method = *key_value.second;
     DCHECK(method.callback.is_null());
     method.callback =
-        base::Bind(&APIBinding::HandleCall, weak_factory_.GetWeakPtr(),
-                   method.full_name, method.signature, method.thread);
+        base::BindRepeating(&APIBinding::HandleCall, weak_factory_.GetWeakPtr(),
+                            method.full_name, method.signature, method.thread);
 
     object_template->Set(
         gin::StringToSymbol(isolate, key_value.first),
-        v8::FunctionTemplate::New(isolate, &CallbackHelper,
+        v8::FunctionTemplate::New(isolate, &RunAPIBindingHandlerCallback,
                                   v8::External::New(isolate, &method.callback),
                                   v8::Local<v8::Signature>(), 0,
                                   v8::ConstructorBehavior::kThrow));
@@ -465,7 +495,7 @@ void APIBinding::DecorateTemplateWithProperties(
     if (dict->GetString("$ref", &ref)) {
       const base::ListValue* property_values = nullptr;
       CHECK(dict->GetList("value", &property_values));
-      auto property_data = base::MakeUnique<CustomPropertyData>(
+      auto property_data = std::make_unique<CustomPropertyData>(
           ref, iter.key(), property_values, create_custom_type_);
       object_template->SetLazyDataProperty(
           v8_key, &APIBinding::GetCustomPropertyObject,
@@ -513,7 +543,7 @@ void APIBinding::GetEventObject(
   v8::Isolate* isolate = info.GetIsolate();
   v8::HandleScope handle_scope(isolate);
   v8::Local<v8::Context> context = info.Holder()->CreationContext();
-  if (!IsContextValid(context))
+  if (!binding::IsContextValidOrThrowError(context))
     return;
 
   CHECK(info.Data()->IsExternal());
@@ -545,7 +575,7 @@ void APIBinding::GetCustomPropertyObject(
   v8::Isolate* isolate = info.GetIsolate();
   v8::HandleScope handle_scope(isolate);
   v8::Local<v8::Context> context = info.Holder()->CreationContext();
-  if (!IsContextValid(context))
+  if (!binding::IsContextValid(context))
     return;
 
   v8::Context::Scope context_scope(context);
@@ -614,7 +644,8 @@ void APIBinding::HandleCall(const std::string& name,
 
         return;  // Our work here is done.
       case APIBindingHooks::RequestResult::ARGUMENTS_UPDATED:
-        updated_args = true;  // Intentional fall-through.
+        updated_args = true;
+        FALLTHROUGH;
       case APIBindingHooks::RequestResult::NOT_HANDLED:
         break;  // Handle in the default manner.
     }
