@@ -5,8 +5,7 @@
 #include "extensions/renderer/bindings/api_bindings_system.h"
 
 #include "base/bind.h"
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
+#include "base/bind_helpers.h"
 #include "base/stl_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/values.h"
@@ -94,11 +93,6 @@ bool AllowAllAPIs(v8::Local<v8::Context> context, const std::string& name) {
   return true;
 }
 
-void DoNothingWithSilentRequest(
-    v8::Local<v8::Context> context,
-    const std::string& call_name,
-    const std::vector<v8::Local<v8::Value>>& arguments) {}
-
 }  // namespace
 
 APIBindingsSystemTest::APIBindingsSystemTest() {}
@@ -115,20 +109,26 @@ void APIBindingsSystemTest::SetUp() {
     api_schemas_[api.name] = std::move(api_schema);
   }
 
-  binding::AddConsoleError add_console_error(base::Bind(
+  binding::AddConsoleError add_console_error(base::BindRepeating(
       &APIBindingsSystemTest::AddConsoleError, base::Unretained(this)));
-  bindings_system_ = base::MakeUnique<APIBindingsSystem>(
-      base::Bind(&RunFunctionOnGlobalAndIgnoreResult),
-      base::Bind(&RunFunctionOnGlobalAndReturnHandle),
-      base::Bind(&APIBindingsSystemTest::GetAPISchema, base::Unretained(this)),
-      base::Bind(&AllowAllAPIs),
-      base::Bind(&APIBindingsSystemTest::OnAPIRequest, base::Unretained(this)),
-      base::Bind(&APIBindingsSystemTest::OnEventListenersChanged,
-                 base::Unretained(this)),
-      base::Bind(&DoNothingWithSilentRequest), add_console_error,
-      APILastError(base::Bind(&APIBindingsSystemTest::GetLastErrorParent,
+  auto get_context_owner = [](v8::Local<v8::Context>) {
+    return std::string("context");
+  };
+  bindings_system_ = std::make_unique<APIBindingsSystem>(
+      base::BindRepeating(&APIBindingsSystemTest::GetAPISchema,
+                          base::Unretained(this)),
+      base::BindRepeating(&AllowAllAPIs),
+      base::BindRepeating(&APIBindingsSystemTest::OnAPIRequest,
+                          base::Unretained(this)),
+      base::BindRepeating(&GetTestUserActivationState),
+      base::BindRepeating(&APIBindingsSystemTest::OnEventListenersChanged,
+                          base::Unretained(this)),
+      base::BindRepeating(get_context_owner), base::DoNothing(),
+      add_console_error,
+      APILastError(
+          base::BindRepeating(&APIBindingsSystemTest::GetLastErrorParent,
                               base::Unretained(this)),
-                   add_console_error));
+          add_console_error));
 }
 
 void APIBindingsSystemTest::TearDown() {
@@ -271,7 +271,7 @@ TEST_F(APIBindingsSystemTest, TestInitializationAndCallbacks) {
     v8::Local<v8::Function> function = FunctionFromString(context, kTestCall);
     v8::Local<v8::Value> args[] = {alpha_api};
     RunFunctionAndExpectError(
-        function, context, arraysize(args), args,
+        function, context, base::size(args), args,
         "Uncaught TypeError: " +
             api_errors::InvocationError(
                 "alpha.functionWithEnum", "alpha.enumRef e",
@@ -327,7 +327,7 @@ TEST_F(APIBindingsSystemTest, TestCustomHooks) {
       return result;
     }
     std::string argument;
-    EXPECT_EQ("foo", gin::V8ToString(arguments->at(0)));
+    EXPECT_EQ("foo", gin::V8ToString(context->GetIsolate(), arguments->at(0)));
     if (!arguments->at(1)->IsFunction()) {
       EXPECT_TRUE(arguments->at(1)->IsFunction());
       return result;
@@ -340,9 +340,9 @@ TEST_F(APIBindingsSystemTest, TestCustomHooks) {
     return result;
   };
 
-  auto test_hooks = base::MakeUnique<APIBindingHooksTestDelegate>();
+  auto test_hooks = std::make_unique<APIBindingHooksTestDelegate>();
   test_hooks->AddHandler("alpha.functionWithCallback",
-                         base::Bind(hook, &did_call));
+                         base::BindRepeating(hook, &did_call));
   APIBindingHooks* binding_hooks =
       bindings_system()->GetHooksForAPI(kAlphaAPIName);
   binding_hooks->SetDelegate(std::move(test_hooks));
@@ -392,7 +392,7 @@ TEST_F(APIBindingsSystemTest, TestSetCustomCallback) {
   v8::Local<v8::Object> js_hooks = hooks->GetJSHookInterface(context);
   v8::Local<v8::Function> function = FunctionFromString(context, kHook);
   v8::Local<v8::Value> args[] = {js_hooks};
-  RunFunctionOnGlobal(function, context, arraysize(args), args);
+  RunFunctionOnGlobal(function, context, base::size(args), args);
 
   {
     const char kTestCall[] =
@@ -446,7 +446,6 @@ TEST_F(APIBindingsSystemTest, TestCustomEvent) {
   v8::Local<v8::Context> context = MainContext();
 
   auto create_custom_event = [](v8::Local<v8::Context> context,
-                                const binding::RunJSFunctionSync& run_js,
                                 const std::string& event_name) {
     v8::Isolate* isolate = context->GetIsolate();
     v8::Local<v8::Object> ret = v8::Object::New(isolate);
@@ -456,8 +455,8 @@ TEST_F(APIBindingsSystemTest, TestCustomEvent) {
     return ret.As<v8::Value>();
   };
 
-  auto test_hooks = base::MakeUnique<APIBindingHooksTestDelegate>();
-  test_hooks->SetCustomEvent(base::Bind(create_custom_event));
+  auto test_hooks = std::make_unique<APIBindingHooksTestDelegate>();
+  test_hooks->SetCustomEvent(base::BindRepeating(create_custom_event));
   APIBindingHooks* binding_hooks =
       bindings_system()->GetHooksForAPI(kAlphaAPIName);
   binding_hooks->SetDelegate(std::move(test_hooks));
