@@ -17,13 +17,12 @@
 #include "base/callback_forward.h"
 #include "base/compiler_specific.h"
 #include "base/logging.h"
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/metrics/histogram_base.h"
 #include "base/metrics/histogram_samples.h"
 #include "base/metrics/statistics_recorder.h"
 #include "base/run_loop.h"
+#include "base/stl_util.h"
 #include "base/values.h"
 #include "components/prefs/testing_pref_store.h"
 #include "mojo/public/cpp/bindings/binding_set.h"
@@ -252,10 +251,12 @@ std::unique_ptr<base::DictionaryValue> MockPrefHashStore::ComputeSplitMacs(
     const std::string& path,
     const base::DictionaryValue* split_values) {
   std::unique_ptr<base::DictionaryValue> macs_dict(new base::DictionaryValue);
+  if (!split_values)
+    return macs_dict;
   for (base::DictionaryValue::Iterator it(*split_values); !it.IsAtEnd();
        it.Advance()) {
-    macs_dict->SetStringWithoutPathExpansion(
-        it.key(), "split mac for: " + path + "/" + it.key());
+    macs_dict->SetKey(it.key(),
+                      base::Value("split mac for: " + path + "/" + it.key()));
   }
   return macs_dict;
 };
@@ -401,18 +402,12 @@ class MockHashStoreContents : public HashStoreContents {
 
   // Records calls to this mock's SetMac/SetSplitMac methods.
   void RecordSetMac(const std::string& path, const std::string& mac) {
-    dictionary_.SetStringWithoutPathExpansion(path, mac);
+    dictionary_.SetKey(path, base::Value(mac));
   }
   void RecordSetSplitMac(const std::string& path,
                          const std::string& split_path,
                          const std::string& mac) {
-    base::DictionaryValue* mac_dict = nullptr;
-    dictionary_.GetDictionaryWithoutPathExpansion(path, &mac_dict);
-    if (!mac_dict) {
-      mac_dict = dictionary_.SetDictionaryWithoutPathExpansion(
-          path, base::MakeUnique<base::DictionaryValue>());
-    }
-    mac_dict->SetStringWithoutPathExpansion(split_path, mac);
+    dictionary_.SetPath({path, split_path}, base::Value(mac));
   }
 
   // Records a call to this mock's RemoveEntry method.
@@ -555,7 +550,6 @@ class PrefHashFilterTest : public testing::TestWithParam<EnforcementLevel>,
         reset_recorded_(false) {}
 
   void SetUp() override {
-    base::StatisticsRecorder::Initialize();
     Reset();
   }
 
@@ -592,7 +586,7 @@ class PrefHashFilterTest : public testing::TestWithParam<EnforcementLevel>,
             std::move(temp_mock_external_validation_pref_hash_store),
             std::move(temp_mock_external_validation_hash_store_contents)),
         std::move(configuration), std::move(reset_on_load_observer),
-        &mock_validation_delegate_, arraysize(kTestTrackedPrefs), true));
+        &mock_validation_delegate_, base::size(kTestTrackedPrefs), true));
   }
 
   // Verifies whether a reset was reported by the PrefHashFiler. Also verifies
@@ -652,13 +646,13 @@ class PrefHashFilterTest : public testing::TestWithParam<EnforcementLevel>,
 TEST_P(PrefHashFilterTest, EmptyAndUnchanged) {
   DoFilterOnLoad(false);
   // All paths checked.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_pref_hash_store_->checked_paths_count());
   // No paths stored, since they all return |UNCHANGED|.
   ASSERT_EQ(0u, mock_pref_hash_store_->stored_paths_count());
   // Since there was nothing in |pref_store_contents_| the checked value should
   // have been NULL for all tracked preferences.
-  for (size_t i = 0; i < arraysize(kTestTrackedPrefs); ++i) {
+  for (size_t i = 0; i < base::size(kTestTrackedPrefs); ++i) {
     ASSERT_EQ(
         NULL,
         mock_pref_hash_store_->checked_value(kTestTrackedPrefs[i].name).first);
@@ -667,9 +661,9 @@ TEST_P(PrefHashFilterTest, EmptyAndUnchanged) {
   VerifyRecordedReset(false);
 
   // Delegate saw all paths, and all unchanged.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->recorded_validations_count());
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->CountValidationsOfState(
                 ValueState::UNCHANGED));
 }
@@ -702,6 +696,27 @@ TEST_P(PrefHashFilterTest, FilterTrackedPrefUpdate) {
   VerifyRecordedReset(false);
 }
 
+TEST_P(PrefHashFilterTest, FilterTrackedPrefClearing) {
+  base::DictionaryValue root_dict;
+  // We don't actually add the pref's value to root_dict to simulate that
+  // it was just cleared in the PrefStore.
+
+  // No path should be stored on FilterUpdate.
+  pref_hash_filter_->FilterUpdate(kAtomicPref);
+  ASSERT_EQ(0u, mock_pref_hash_store_->stored_paths_count());
+
+  // One path should be stored on FilterSerializeData, with no value.
+  pref_hash_filter_->FilterSerializeData(&root_dict);
+  ASSERT_EQ(1u, mock_pref_hash_store_->stored_paths_count());
+  MockPrefHashStore::ValuePtrStrategyPair stored_value =
+      mock_pref_hash_store_->stored_value(kAtomicPref);
+  ASSERT_EQ(nullptr, stored_value.first);
+  ASSERT_EQ(PrefTrackingStrategy::ATOMIC, stored_value.second);
+
+  ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
+  VerifyRecordedReset(false);
+}
+
 TEST_P(PrefHashFilterTest, ReportSuperMacValidity) {
   // Do this once just to force the histogram to be defined.
   DoFilterOnLoad(false);
@@ -741,7 +756,7 @@ TEST_P(PrefHashFilterTest, ReportSuperMacValidity) {
 TEST_P(PrefHashFilterTest, FilterSplitPrefUpdate) {
   base::DictionaryValue root_dict;
   base::DictionaryValue* dict_value = root_dict.SetDictionary(
-      kSplitPref, base::MakeUnique<base::DictionaryValue>());
+      kSplitPref, std::make_unique<base::DictionaryValue>());
   dict_value->SetString("a", "foo");
   dict_value->SetInteger("b", 1234);
 
@@ -761,6 +776,27 @@ TEST_P(PrefHashFilterTest, FilterSplitPrefUpdate) {
   VerifyRecordedReset(false);
 }
 
+TEST_P(PrefHashFilterTest, FilterTrackedSplitPrefClearing) {
+  base::DictionaryValue root_dict;
+  // We don't actually add the pref's value to root_dict to simulate that
+  // it was just cleared in the PrefStore.
+
+  // No path should be stored on FilterUpdate.
+  pref_hash_filter_->FilterUpdate(kSplitPref);
+  ASSERT_EQ(0u, mock_pref_hash_store_->stored_paths_count());
+
+  // One path should be stored on FilterSerializeData, with no value.
+  pref_hash_filter_->FilterSerializeData(&root_dict);
+  ASSERT_EQ(1u, mock_pref_hash_store_->stored_paths_count());
+  MockPrefHashStore::ValuePtrStrategyPair stored_value =
+      mock_pref_hash_store_->stored_value(kSplitPref);
+  ASSERT_EQ(nullptr, stored_value.first);
+  ASSERT_EQ(PrefTrackingStrategy::SPLIT, stored_value.second);
+
+  ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
+  VerifyRecordedReset(false);
+}
+
 TEST_P(PrefHashFilterTest, FilterUntrackedPrefUpdate) {
   base::DictionaryValue root_dict;
   root_dict.SetString("untracked", "some value");
@@ -785,7 +821,7 @@ TEST_P(PrefHashFilterTest, MultiplePrefsFilterSerializeData) {
   root_dict.SetInteger(kAtomicPref3, 3);
   root_dict.SetInteger("untracked", 4);
   base::DictionaryValue* dict_value = root_dict.SetDictionary(
-      kSplitPref, base::MakeUnique<base::DictionaryValue>());
+      kSplitPref, std::make_unique<base::DictionaryValue>());
   dict_value->SetBoolean("a", true);
 
   // Only update kAtomicPref, kAtomicPref3, and kSplitPref.
@@ -828,7 +864,7 @@ TEST_P(PrefHashFilterTest, UnknownNullValue) {
   mock_pref_hash_store_->SetCheckResult(kSplitPref,
                                         ValueState::TRUSTED_NULL_VALUE);
   DoFilterOnLoad(false);
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_pref_hash_store_->checked_paths_count());
   ASSERT_EQ(2u, mock_pref_hash_store_->stored_paths_count());
   ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
@@ -844,11 +880,11 @@ TEST_P(PrefHashFilterTest, UnknownNullValue) {
   ASSERT_EQ(PrefTrackingStrategy::SPLIT, stored_split_value.second);
 
   // Delegate saw all prefs, two of which had the expected value_state.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->recorded_validations_count());
   ASSERT_EQ(2u, mock_validation_delegate_record_->CountValidationsOfState(
                     ValueState::TRUSTED_NULL_VALUE));
-  ASSERT_EQ(arraysize(kTestTrackedPrefs) - 2u,
+  ASSERT_EQ(base::size(kTestTrackedPrefs) - 2u,
             mock_validation_delegate_record_->CountValidationsOfState(
                 ValueState::UNCHANGED));
 
@@ -867,7 +903,7 @@ TEST_P(PrefHashFilterTest, InitialValueUnknown) {
       pref_store_contents_->SetString(kAtomicPref, "string value");
 
   base::DictionaryValue* dict_value = pref_store_contents_->SetDictionary(
-      kSplitPref, base::MakeUnique<base::DictionaryValue>());
+      kSplitPref, std::make_unique<base::DictionaryValue>());
   dict_value->SetString("a", "foo");
   dict_value->SetInteger("b", 1234);
 
@@ -880,17 +916,17 @@ TEST_P(PrefHashFilterTest, InitialValueUnknown) {
                                         ValueState::UNTRUSTED_UNKNOWN_VALUE);
   // If we are enforcing, expect this to report changes.
   DoFilterOnLoad(GetParam() >= EnforcementLevel::ENFORCE_ON_LOAD);
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_pref_hash_store_->checked_paths_count());
   ASSERT_EQ(2u, mock_pref_hash_store_->stored_paths_count());
   ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
 
   // Delegate saw all prefs, two of which had the expected value_state.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->recorded_validations_count());
   ASSERT_EQ(2u, mock_validation_delegate_record_->CountValidationsOfState(
                     ValueState::UNTRUSTED_UNKNOWN_VALUE));
-  ASSERT_EQ(arraysize(kTestTrackedPrefs) - 2u,
+  ASSERT_EQ(base::size(kTestTrackedPrefs) - 2u,
             mock_validation_delegate_record_->CountValidationsOfState(
                 ValueState::UNCHANGED));
 
@@ -932,7 +968,7 @@ TEST_P(PrefHashFilterTest, InitialValueTrustedUnknown) {
       pref_store_contents_->SetString(kAtomicPref, "test");
 
   auto* dict_value = pref_store_contents_->SetDictionary(
-      kSplitPref, base::MakeUnique<base::DictionaryValue>());
+      kSplitPref, std::make_unique<base::DictionaryValue>());
   dict_value->SetString("a", "foo");
   dict_value->SetInteger("b", 1234);
 
@@ -944,17 +980,17 @@ TEST_P(PrefHashFilterTest, InitialValueTrustedUnknown) {
   mock_pref_hash_store_->SetCheckResult(kSplitPref,
                                         ValueState::TRUSTED_UNKNOWN_VALUE);
   DoFilterOnLoad(false);
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_pref_hash_store_->checked_paths_count());
   ASSERT_EQ(2u, mock_pref_hash_store_->stored_paths_count());
   ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
 
   // Delegate saw all prefs, two of which had the expected value_state.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->recorded_validations_count());
   ASSERT_EQ(2u, mock_validation_delegate_record_->CountValidationsOfState(
                     ValueState::TRUSTED_UNKNOWN_VALUE));
-  ASSERT_EQ(arraysize(kTestTrackedPrefs) - 2u,
+  ASSERT_EQ(base::size(kTestTrackedPrefs) - 2u,
             mock_validation_delegate_record_->CountValidationsOfState(
                 ValueState::UNCHANGED));
 
@@ -980,7 +1016,7 @@ TEST_P(PrefHashFilterTest, InitialValueChanged) {
   base::Value* int_value = pref_store_contents_->SetInteger(kAtomicPref, 1234);
 
   base::DictionaryValue* dict_value = pref_store_contents_->SetDictionary(
-      kSplitPref, base::MakeUnique<base::DictionaryValue>());
+      kSplitPref, std::make_unique<base::DictionaryValue>());
   dict_value->SetString("a", "foo");
   dict_value->SetInteger("b", 1234);
   dict_value->SetInteger("c", 56);
@@ -998,7 +1034,7 @@ TEST_P(PrefHashFilterTest, InitialValueChanged) {
   mock_pref_hash_store_->SetInvalidKeysResult(kSplitPref, mock_invalid_keys);
 
   DoFilterOnLoad(GetParam() >= EnforcementLevel::ENFORCE_ON_LOAD);
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_pref_hash_store_->checked_paths_count());
   ASSERT_EQ(2u, mock_pref_hash_store_->stored_paths_count());
   ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
@@ -1055,17 +1091,17 @@ TEST_P(PrefHashFilterTest, EmptyCleared) {
   mock_pref_hash_store_->SetCheckResult(kAtomicPref, ValueState::CLEARED);
   mock_pref_hash_store_->SetCheckResult(kSplitPref, ValueState::CLEARED);
   DoFilterOnLoad(false);
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_pref_hash_store_->checked_paths_count());
   ASSERT_EQ(2u, mock_pref_hash_store_->stored_paths_count());
   ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
 
   // Delegate saw all prefs, two of which had the expected value_state.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->recorded_validations_count());
   ASSERT_EQ(2u, mock_validation_delegate_record_->CountValidationsOfState(
                     ValueState::CLEARED));
-  ASSERT_EQ(arraysize(kTestTrackedPrefs) - 2u,
+  ASSERT_EQ(base::size(kTestTrackedPrefs) - 2u,
             mock_validation_delegate_record_->CountValidationsOfState(
                 ValueState::UNCHANGED));
 
@@ -1089,7 +1125,7 @@ TEST_P(PrefHashFilterTest, InitialValueUnchangedLegacyId) {
       pref_store_contents_->SetString(kAtomicPref, "string value");
 
   base::DictionaryValue* dict_value = pref_store_contents_->SetDictionary(
-      kSplitPref, base::MakeUnique<base::DictionaryValue>());
+      kSplitPref, std::make_unique<base::DictionaryValue>());
   dict_value->SetString("a", "foo");
   dict_value->SetInteger("b", 1234);
 
@@ -1099,16 +1135,16 @@ TEST_P(PrefHashFilterTest, InitialValueUnchangedLegacyId) {
   mock_pref_hash_store_->SetCheckResult(kAtomicPref, ValueState::SECURE_LEGACY);
   mock_pref_hash_store_->SetCheckResult(kSplitPref, ValueState::SECURE_LEGACY);
   DoFilterOnLoad(false);
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_pref_hash_store_->checked_paths_count());
   ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
 
   // Delegate saw all prefs, two of which had the expected value_state.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->recorded_validations_count());
   ASSERT_EQ(2u, mock_validation_delegate_record_->CountValidationsOfState(
                     ValueState::SECURE_LEGACY));
-  ASSERT_EQ(arraysize(kTestTrackedPrefs) - 2u,
+  ASSERT_EQ(base::size(kTestTrackedPrefs) - 2u,
             mock_validation_delegate_record_->CountValidationsOfState(
                 ValueState::UNCHANGED));
 
@@ -1144,7 +1180,7 @@ TEST_P(PrefHashFilterTest, DontResetReportOnly) {
       pref_store_contents_->SetInteger(kReportOnlyPref, 3);
   base::DictionaryValue* report_only_split_val =
       pref_store_contents_->SetDictionary(
-          kReportOnlySplitPref, base::MakeUnique<base::DictionaryValue>());
+          kReportOnlySplitPref, std::make_unique<base::DictionaryValue>());
   report_only_split_val->SetInteger("a", 1234);
 
   ASSERT_TRUE(pref_store_contents_->Get(kAtomicPref, NULL));
@@ -1161,17 +1197,17 @@ TEST_P(PrefHashFilterTest, DontResetReportOnly) {
   DoFilterOnLoad(GetParam() >= EnforcementLevel::ENFORCE_ON_LOAD);
   // All prefs should be checked and a new hash should be stored for each tested
   // pref.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_pref_hash_store_->checked_paths_count());
   ASSERT_EQ(4u, mock_pref_hash_store_->stored_paths_count());
   ASSERT_EQ(1u, mock_pref_hash_store_->transactions_performed());
 
   // Delegate saw all prefs, four of which had the expected value_state.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->recorded_validations_count());
   ASSERT_EQ(4u, mock_validation_delegate_record_->CountValidationsOfState(
                     ValueState::CHANGED));
-  ASSERT_EQ(arraysize(kTestTrackedPrefs) - 4u,
+  ASSERT_EQ(base::size(kTestTrackedPrefs) - 4u,
             mock_validation_delegate_record_->CountValidationsOfState(
                 ValueState::UNCHANGED));
 
@@ -1210,7 +1246,7 @@ TEST_P(PrefHashFilterTest, DontResetReportOnly) {
 
 TEST_P(PrefHashFilterTest, CallFilterSerializeDataCallbacks) {
   base::DictionaryValue root_dict;
-  auto dict_value = base::MakeUnique<base::DictionaryValue>();
+  auto dict_value = std::make_unique<base::DictionaryValue>();
   dict_value->SetBoolean("a", true);
   root_dict.SetInteger(kAtomicPref, 1);
   root_dict.SetInteger(kAtomicPref2, 2);
@@ -1284,7 +1320,7 @@ TEST_P(PrefHashFilterTest, CallFilterSerializeDataCallbacksWithFailure) {
 TEST_P(PrefHashFilterTest, ExternalValidationValueChanged) {
   pref_store_contents_->SetInteger(kAtomicPref, 1234);
 
-  auto dict_value = base::MakeUnique<base::DictionaryValue>();
+  auto dict_value = std::make_unique<base::DictionaryValue>();
   dict_value->SetString("a", "foo");
   dict_value->SetInteger("b", 1234);
   dict_value->SetInteger("c", 56);
@@ -1304,18 +1340,18 @@ TEST_P(PrefHashFilterTest, ExternalValidationValueChanged) {
 
   DoFilterOnLoad(false);
 
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_external_validation_pref_hash_store_->checked_paths_count());
   ASSERT_EQ(2u,
             mock_external_validation_pref_hash_store_->stored_paths_count());
   ASSERT_EQ(
       1u, mock_external_validation_pref_hash_store_->transactions_performed());
 
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->recorded_validations_count());
 
   // Regular validation should not have any CHANGED prefs.
-  ASSERT_EQ(arraysize(kTestTrackedPrefs),
+  ASSERT_EQ(base::size(kTestTrackedPrefs),
             mock_validation_delegate_record_->CountValidationsOfState(
                 ValueState::UNCHANGED));
 
@@ -1323,7 +1359,7 @@ TEST_P(PrefHashFilterTest, ExternalValidationValueChanged) {
   ASSERT_EQ(2u,
             mock_validation_delegate_record_->CountExternalValidationsOfState(
                 ValueState::CHANGED));
-  ASSERT_EQ(arraysize(kTestTrackedPrefs) - 2u,
+  ASSERT_EQ(base::size(kTestTrackedPrefs) - 2u,
             mock_validation_delegate_record_->CountExternalValidationsOfState(
                 ValueState::UNCHANGED));
 }
