@@ -2,8 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "ui/android/resources/resource_factory.h"
-
 #include "jni/ResourceFactory_jni.h"
 #include "ui/android/resources/nine_patch_resource.h"
 #include "ui/gfx/geometry/rect.h"
@@ -12,24 +10,20 @@ using base::android::JavaParamRef;
 
 namespace ui {
 
-bool RegisterResourceFactory(JNIEnv* env) {
-  return RegisterNativesImpl(env);
-}
-
-jlong CreateBitmapResource(JNIEnv* env, const JavaParamRef<jclass>& clazz) {
+jlong JNI_ResourceFactory_CreateBitmapResource(JNIEnv* env) {
   return reinterpret_cast<intptr_t>(new Resource());
 }
 
-jlong CreateNinePatchBitmapResource(JNIEnv* env,
-                                    const JavaParamRef<jclass>& clazz,
-                                    jint padding_left,
-                                    jint padding_top,
-                                    jint padding_right,
-                                    jint padding_bottom,
-                                    jint aperture_left,
-                                    jint aperture_top,
-                                    jint aperture_right,
-                                    jint aperture_bottom) {
+jlong JNI_ResourceFactory_CreateNinePatchBitmapResource(
+    JNIEnv* env,
+    jint padding_left,
+    jint padding_top,
+    jint padding_right,
+    jint padding_bottom,
+    jint aperture_left,
+    jint aperture_top,
+    jint aperture_right,
+    jint aperture_bottom) {
   gfx::Rect padding(padding_left, padding_top, padding_right - padding_left,
                     padding_bottom - padding_top);
   gfx::Rect aperture(aperture_left, aperture_top,
