@@ -28,30 +28,30 @@ const float kDoubleTapDragZoomSpeed = 0.005f;
 
 const char* GetMotionEventActionName(MotionEvent::Action action) {
   switch (action) {
-    case MotionEvent::ACTION_NONE:
-      return "ACTION_NONE";
-    case MotionEvent::ACTION_POINTER_DOWN:
-      return "ACTION_POINTER_DOWN";
-    case MotionEvent::ACTION_POINTER_UP:
-      return "ACTION_POINTER_UP";
-    case MotionEvent::ACTION_DOWN:
-      return "ACTION_DOWN";
-    case MotionEvent::ACTION_UP:
-      return "ACTION_UP";
-    case MotionEvent::ACTION_CANCEL:
-      return "ACTION_CANCEL";
-    case MotionEvent::ACTION_MOVE:
-      return "ACTION_MOVE";
-    case MotionEvent::ACTION_HOVER_ENTER:
-      return "ACTION_HOVER_ENTER";
-    case MotionEvent::ACTION_HOVER_EXIT:
-      return "ACTION_HOVER_EXIT";
-    case MotionEvent::ACTION_HOVER_MOVE:
-      return "ACTION_HOVER_MOVE";
-    case MotionEvent::ACTION_BUTTON_PRESS:
-      return "ACTION_BUTTON_PRESS";
-    case MotionEvent::ACTION_BUTTON_RELEASE:
-      return "ACTION_BUTTON_RELEASE";
+    case MotionEvent::Action::NONE:
+      return "Action::NONE";
+    case MotionEvent::Action::POINTER_DOWN:
+      return "Action::POINTER_DOWN";
+    case MotionEvent::Action::POINTER_UP:
+      return "Action::POINTER_UP";
+    case MotionEvent::Action::DOWN:
+      return "Action::DOWN";
+    case MotionEvent::Action::UP:
+      return "Action::UP";
+    case MotionEvent::Action::CANCEL:
+      return "Action::CANCEL";
+    case MotionEvent::Action::MOVE:
+      return "Action::MOVE";
+    case MotionEvent::Action::HOVER_ENTER:
+      return "Action::HOVER_ENTER";
+    case MotionEvent::Action::HOVER_EXIT:
+      return "Action::HOVER_EXIT";
+    case MotionEvent::Action::HOVER_MOVE:
+      return "Action::HOVER_MOVE";
+    case MotionEvent::Action::BUTTON_PRESS:
+      return "Action::BUTTON_PRESS";
+    case MotionEvent::Action::BUTTON_RELEASE:
+      return "Action::BUTTON_RELEASE";
   }
   return "";
 }
@@ -76,6 +76,12 @@ gfx::RectF ClampBoundingBox(const gfx::RectF& bounds,
 
 }  // namespace
 
+// GestureProviderClient:
+
+bool GestureProviderClient::RequiresDoubleTapGestureEvents() const {
+  return false;
+}
+
 // GestureProvider:::Config
 
 GestureProvider::Config::Config()
@@ -118,7 +124,7 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
       SetIgnoreSingleTap(true);
 
     const MotionEvent::Action action = event.GetAction();
-    if (action == MotionEvent::ACTION_DOWN) {
+    if (action == MotionEvent::Action::DOWN) {
       current_down_time_ = event.GetEventTime();
       current_longpress_time_ = base::TimeTicks();
       ignore_single_tap_ = false;
@@ -129,17 +135,18 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
       tap_down_point_ = gfx::PointF(event.GetX(), event.GetY());
       max_diameter_before_show_press_ = event.GetTouchMajor();
     }
-    gesture_detector_.OnTouchEvent(event);
+    gesture_detector_.OnTouchEvent(event,
+                                   client_->RequiresDoubleTapGestureEvents());
     scale_gesture_detector_.OnTouchEvent(event);
 
-    if (action == MotionEvent::ACTION_UP ||
-        action == MotionEvent::ACTION_CANCEL) {
+    if (action == MotionEvent::Action::UP ||
+        action == MotionEvent::Action::CANCEL) {
       // Note: This call will have no effect if a fling was just generated, as
       // |Fling()| will have already signalled an end to touch-scrolling.
       if (scroll_event_sent_)
         Send(CreateGesture(ET_GESTURE_SCROLL_END, event));
       current_down_time_ = base::TimeTicks();
-    } else if (action == MotionEvent::ACTION_MOVE) {
+    } else if (action == MotionEvent::Action::MOVE) {
       if (!show_press_event_sent_ && !scroll_event_sent_) {
         max_diameter_before_show_press_ =
             std::max(max_diameter_before_show_press_, event.GetTouchMajor());
@@ -157,8 +164,8 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
            gesture.type() == ET_GESTURE_BEGIN ||
            gesture.type() == ET_GESTURE_END);
 
-    if (gesture.primary_tool_type == MotionEvent::TOOL_TYPE_UNKNOWN ||
-        gesture.primary_tool_type == MotionEvent::TOOL_TYPE_FINGER) {
+    if (gesture.primary_tool_type == MotionEvent::ToolType::UNKNOWN ||
+        gesture.primary_tool_type == MotionEvent::ToolType::FINGER) {
       gesture.details.set_bounding_box(
           ClampBoundingBox(gesture.details.bounding_box_f(),
                            config_.min_gesture_bounds_length,
@@ -189,8 +196,10 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
         break;
       case ET_GESTURE_PINCH_BEGIN:
         DCHECK(!pinch_event_sent_);
-        if (!scroll_event_sent_)
+        if (!scroll_event_sent_ &&
+            !scale_gesture_detector_.InAnchoredScaleMode()) {
           Send(GestureEventData(ET_GESTURE_SCROLL_BEGIN, gesture));
+        }
         pinch_event_sent_ = true;
         break;
       case ET_GESTURE_PINCH_END:
@@ -203,6 +212,7 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
         // which case the press should simply be dropped.
         if (pinch_event_sent_ || scroll_event_sent_)
           return;
+        break;
       default:
         break;
     };
@@ -230,7 +240,9 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
                const MotionEvent& e) override {
     if (ignore_multitouch_zoom_events_ && !detector.InAnchoredScaleMode())
       return false;
+    bool first_scale = false;
     if (!pinch_event_sent_) {
+      first_scale = true;
       Send(CreateGesture(ET_GESTURE_PINCH_BEGIN,
                          e.GetPointerId(),
                          e.GetToolType(),
@@ -259,8 +271,13 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
       // For historical reasons, Chrome has instead adopted a scale factor
       // computation that is invariant to the focal distance, where
       // the scale delta remains constant if the touch velocity is constant.
-      float dy =
-          (detector.GetCurrentSpanY() - detector.GetPreviousSpanY()) * 0.5f;
+      // Note: Because we calculate the scale here manually based on the
+      // y-span, but the scale factor accounts for slop in the first previous
+      // span, we manaully reproduce the behavior here for previous span y.
+      float prev_y = first_scale
+                         ? config_.gesture_detector_config.touch_slop * 2
+                         : detector.GetPreviousSpanY();
+      float dy = (detector.GetCurrentSpanY() - prev_y) * 0.5f;
       scale = std::pow(scale > 1 ? 1.0f + kDoubleTapDragZoomSpeed
                                  : 1.0f - kDoubleTapDragZoomSpeed,
                        std::abs(dy));
@@ -333,8 +350,8 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
     if (!scroll_event_sent_) {
       // Note that scroll start hints are in distance traveled, where
       // scroll deltas are in the opposite direction.
-      GestureEventDetails scroll_details(
-          ET_GESTURE_SCROLL_BEGIN, -raw_distance_x, -raw_distance_y);
+      GestureEventDetails scroll_details(ET_GESTURE_SCROLL_BEGIN, -distance_x,
+                                         -distance_y);
       scroll_details.set_device_type(GestureDeviceType::DEVICE_TOUCHSCREEN);
 
       // Scroll focus point always starts with the first touch down point.
@@ -466,7 +483,7 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
       }
     }
 
-    if (e.GetAction() == MotionEvent::ACTION_UP &&
+    if (e.GetAction() == MotionEvent::Action::UP &&
         !current_longpress_time_.is_null() &&
         !IsScaleGestureDetectionInProgress()) {
       GestureEventDetails long_tap_details(ET_GESTURE_LONG_TAP);
@@ -489,11 +506,11 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
 
   bool OnDoubleTapEvent(const MotionEvent& e) override {
     switch (e.GetAction()) {
-      case MotionEvent::ACTION_DOWN:
+      case MotionEvent::Action::DOWN:
         gesture_detector_.set_longpress_enabled(false);
         break;
 
-      case MotionEvent::ACTION_UP:
+      case MotionEvent::Action::UP:
         if (!IsPinchInProgress() && !IsScrollInProgress()) {
           Send(CreateTapGesture(ET_GESTURE_DOUBLE_TAP, e, 1));
           return true;
@@ -568,18 +585,12 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
 
   GestureEventData CreateGesture(const GestureEventDetails& details,
                                  const MotionEvent& event) const {
-    return GestureEventData(details,
-                            event.GetPointerId(),
-                            event.GetToolType(),
-                            event.GetEventTime(),
-                            event.GetX(),
-                            event.GetY(),
-                            event.GetRawX(),
-                            event.GetRawY(),
+    return GestureEventData(details, event.GetPointerId(), event.GetToolType(),
+                            event.GetEventTime(), event.GetX(), event.GetY(),
+                            event.GetRawX(), event.GetRawY(),
                             event.GetPointerCount(),
                             GetBoundingBox(event, details.type()),
-                            event.GetFlags(),
-                            0U);
+                            event.GetFlags(), event.GetUniqueEventId());
   }
 
   GestureEventData CreateGesture(EventType type,
@@ -677,7 +688,8 @@ class GestureProvider::GestureListenerImpl : public ScaleGestureListener,
   }
 
   bool IsDoubleTapEnabled() const {
-    return gesture_detector_.has_doubletap_listener();
+    return gesture_detector_.has_doubletap_listener() &&
+           client_->RequiresDoubleTapGestureEvents();
   }
 
   void SetIgnoreSingleTap(bool value) { ignore_single_tap_ = value; }
@@ -812,9 +824,8 @@ bool GestureProvider::OnTouchEvent(const MotionEvent& event) {
 }
 
 void GestureProvider::ResetDetection() {
-  MotionEventGeneric generic_cancel_event(MotionEvent::ACTION_CANCEL,
-                                          base::TimeTicks::Now(),
-                                          PointerProperties());
+  MotionEventGeneric generic_cancel_event(
+      MotionEvent::Action::CANCEL, base::TimeTicks::Now(), PointerProperties());
   OnTouchEvent(generic_cancel_event);
 }
 
@@ -852,19 +863,20 @@ bool GestureProvider::CanHandle(const MotionEvent& event) const {
   // Aura requires one cancel event per touch point, whereas Android requires
   // one cancel event per touch sequence. Thus we need to allow extra cancel
   // events.
-  return current_down_event_ || event.GetAction() == MotionEvent::ACTION_DOWN ||
-         event.GetAction() == MotionEvent::ACTION_CANCEL;
+  return current_down_event_ ||
+         event.GetAction() == MotionEvent::Action::DOWN ||
+         event.GetAction() == MotionEvent::Action::CANCEL;
 }
 
 void GestureProvider::OnTouchEventHandlingBegin(const MotionEvent& event) {
   switch (event.GetAction()) {
-    case MotionEvent::ACTION_DOWN:
+    case MotionEvent::Action::DOWN:
       current_down_event_ = event.Clone();
       if (gesture_begin_end_types_enabled_)
         gesture_listener_->Send(
             gesture_listener_->CreateGesture(ET_GESTURE_BEGIN, event));
       break;
-    case MotionEvent::ACTION_POINTER_DOWN:
+    case MotionEvent::Action::POINTER_DOWN:
       if (gesture_begin_end_types_enabled_) {
         const int action_index = event.GetActionIndex();
         gesture_listener_->Send(gesture_listener_->CreateGesture(
@@ -881,17 +893,17 @@ void GestureProvider::OnTouchEventHandlingBegin(const MotionEvent& event) {
             event.GetFlags()));
       }
       break;
-    case MotionEvent::ACTION_POINTER_UP:
-    case MotionEvent::ACTION_UP:
-    case MotionEvent::ACTION_CANCEL:
-    case MotionEvent::ACTION_MOVE:
+    case MotionEvent::Action::POINTER_UP:
+    case MotionEvent::Action::UP:
+    case MotionEvent::Action::CANCEL:
+    case MotionEvent::Action::MOVE:
       break;
-    case MotionEvent::ACTION_NONE:
-    case MotionEvent::ACTION_HOVER_ENTER:
-    case MotionEvent::ACTION_HOVER_EXIT:
-    case MotionEvent::ACTION_HOVER_MOVE:
-    case MotionEvent::ACTION_BUTTON_PRESS:
-    case MotionEvent::ACTION_BUTTON_RELEASE:
+    case MotionEvent::Action::NONE:
+    case MotionEvent::Action::HOVER_ENTER:
+    case MotionEvent::Action::HOVER_EXIT:
+    case MotionEvent::Action::HOVER_MOVE:
+    case MotionEvent::Action::BUTTON_PRESS:
+    case MotionEvent::Action::BUTTON_RELEASE:
       NOTREACHED();
       break;
   }
@@ -899,8 +911,8 @@ void GestureProvider::OnTouchEventHandlingBegin(const MotionEvent& event) {
 
 void GestureProvider::OnTouchEventHandlingEnd(const MotionEvent& event) {
   switch (event.GetAction()) {
-    case MotionEvent::ACTION_UP:
-    case MotionEvent::ACTION_CANCEL: {
+    case MotionEvent::Action::UP:
+    case MotionEvent::Action::CANCEL: {
       if (gesture_begin_end_types_enabled_)
         gesture_listener_->Send(
             gesture_listener_->CreateGesture(ET_GESTURE_END, event));
@@ -910,21 +922,21 @@ void GestureProvider::OnTouchEventHandlingEnd(const MotionEvent& event) {
       UpdateDoubleTapDetectionSupport();
       break;
     }
-    case MotionEvent::ACTION_POINTER_UP:
+    case MotionEvent::Action::POINTER_UP:
       if (gesture_begin_end_types_enabled_)
         gesture_listener_->Send(
             gesture_listener_->CreateGesture(ET_GESTURE_END, event));
       break;
-    case MotionEvent::ACTION_DOWN:
-    case MotionEvent::ACTION_POINTER_DOWN:
-    case MotionEvent::ACTION_MOVE:
+    case MotionEvent::Action::DOWN:
+    case MotionEvent::Action::POINTER_DOWN:
+    case MotionEvent::Action::MOVE:
       break;
-    case MotionEvent::ACTION_NONE:
-    case MotionEvent::ACTION_HOVER_ENTER:
-    case MotionEvent::ACTION_HOVER_EXIT:
-    case MotionEvent::ACTION_HOVER_MOVE:
-    case MotionEvent::ACTION_BUTTON_PRESS:
-    case MotionEvent::ACTION_BUTTON_RELEASE:
+    case MotionEvent::Action::NONE:
+    case MotionEvent::Action::HOVER_ENTER:
+    case MotionEvent::Action::HOVER_EXIT:
+    case MotionEvent::Action::HOVER_MOVE:
+    case MotionEvent::Action::BUTTON_PRESS:
+    case MotionEvent::Action::BUTTON_RELEASE:
       NOTREACHED();
       break;
   }
