@@ -14,6 +14,14 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
    */
   var MAX_LOGIN_ATTEMPTS_IN_POD = 3;
 
+  /**
+   * Time after which the sign-in error bubble should be hidden if it is
+   * overlayed over the detachable base change warning bubble (to ensure that
+   * the detachable base warning is not obscured indefinitely).
+   * @const {number}
+   */
+  var SIGNIN_ERROR_OVER_DETACHABLE_BASE_WARNING_TIMEOUT_MS = 5000;
+
   return {
     EXTERNAL_API: [
       'loadUsers',
@@ -30,14 +38,17 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
       'hideUserPodCustomIcon',
       'setUserPodFingerprintIcon',
       'removeUserPodFingerprintIcon',
+      'selectPodForDetachableBaseWarningBubble',
       'setPinEnabledForUser',
       'setAuthType',
-      'setTouchViewState',
+      'setTabletModeState',
+      'setDemoModeState',
       'setPublicSessionDisplayName',
       'setPublicSessionLocales',
       'setPublicSessionKeyboardLayouts',
       'setLockScreenAppsState',
       'setOverlayColors',
+      'togglePodBackground',
     ],
 
     preferredWidth_: 0,
@@ -76,14 +87,23 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
     },
 
     /**
-      * Sets login screen overlay colors based on colors extracted from the
-      * wallpaper.
-      * @param {string} maskColor Color for the gradient mask.
-      * @param {string} scrollColor Color for the small pods container.
-      */
-     setOverlayColors: function(maskColor, scrollColor) {
+     * Sets login screen overlay colors based on colors extracted from the
+     * wallpaper.
+     * @param {string} maskColor Color for the gradient mask.
+     * @param {string} scrollColor Color for the small pods container.
+     */
+    setOverlayColors: function(maskColor, scrollColor) {
       $('pod-row').setOverlayColors(maskColor, scrollColor);
-     },
+    },
+
+    /**
+     * Toggles the background behind user pods.
+     * @param {boolean} showPodBackground Whether to add background behind user
+     *     pods.
+     */
+    togglePodBackground: function(showPodBackground) {
+      $('pod-row').togglePodBackground(showPodBackground);
+    },
 
     /**
      * When the account picker is being used to lock the screen, pressing the
@@ -146,8 +166,10 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
         // again already. If that happens, ignore the onShow() call.
         return;
       }
-      chrome.send('getTouchViewState');
-      if (!this.firstShown_) return;
+      chrome.send('getTabletModeState');
+      chrome.send('getDemoModeState');
+      if (!this.firstShown_)
+        return;
       this.firstShown_ = false;
 
       // Ensure that login is actually visible.
@@ -162,6 +184,7 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
      */
     onBeforeHide: function() {
       $('pod-row').clearFocusedPod();
+      $('bubble-persistent').hide();
       this.showing_ = false;
       chrome.send('loginUIStateChanged', ['account-picker', false]);
       $('login-header-bar').signinUIState = SIGNIN_UI_STATE.HIDDEN;
@@ -194,11 +217,66 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
         }
         // Update the pod row display if incorrect password.
         $('pod-row').setFocusedPodErrorDisplay(true);
-        activatedPod.showBubble(error);
+
+        // If a warning that the detachable base is different than the one
+        // previously used by the user is shown for the pod, make sure that the
+        // sign-in error gets hidden reasonably soon.
+        // If the detachable base was changed maliciously while the user was
+        // away, the attacker might attempt to use the sign-in error but to
+        // obscure the detachable base warning hoping that the user will miss it
+        // when they get back to the device.
+        var timeout = activatedPod.showingDetachableBaseWarningBubble() ?
+            SIGNIN_ERROR_OVER_DETACHABLE_BASE_WARNING_TIMEOUT_MS :
+            undefined;
+        activatedPod.showBubble(error, {timeout: timeout});
       }
     },
 
-   /**
+    /**
+     * Ensures that a user pod is selected and focused, and thus ready to show a
+     * warning bubble for detachable base change. This is needed for two
+     * reasons:
+     *   1. The detachable base state is associated with a user, so a user pod
+     *      has to be selected in order to know for which user the detachable
+     *      base state should be considered (e.g. there might be two large user
+     *      pods in the account picker).
+     *   2. The warning bubble is attached to the pod's auth element, which is
+     *      only shown if the pod is focused. The bubble anchor should be
+     *      visible in order to properly calculate the bubble position.
+     */
+    selectPodForDetachableBaseWarningBubble: function() {
+      $('pod-row').maybePreselectPod();
+    },
+
+    /**
+     * Shows a persistent bubble warning to the user that the current detachable
+     * base is different than the one they were last using, and that it might
+     * not be trusted.
+     *
+     * @param {string} username The username of the user under whose user pod
+     *     the warning should be displayed.
+     * @param {HTMLElement} content The warning bubble content.
+     */
+    showDetachableBaseWarningBubble: function(username, content) {
+      var podRow = $('pod-row');
+      var pod = podRow.pods.find(pod => pod.user.username == username);
+      if (pod)
+        pod.showDetachableBaseWarningBubble(content);
+    },
+
+    /**
+     * Hides the detachable base warning for the user.
+     *
+     * @param {string} username The username that identifies the user pod from
+     *     under which the detachable base warning bubble should be removed.
+     */
+    hideDetachableBaseWarningBubble: function(username) {
+      var pod = $('pod-row').pods.find(pod => pod.user.username == username);
+      if (pod)
+        pod.hideDetachableBaseWarningBubble();
+    },
+
+    /**
      * Loads given users in pod row.
      * @param {array} users Array of user.
      * @param {boolean} showGuest Whether to show guest session button.
@@ -290,9 +368,10 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
      * this function updates the message in the banner. This function is used
      * by the chrome.screenlockPrivate.showMessage API.
      * @param {string} message Text to be displayed or empty to hide the banner.
+     * @param {boolean} isWarning True if the given message is a warning.
      */
-    showBannerMessage: function(message) {
-      $('pod-row').showBannerMessage(message);
+    showBannerMessage: function(message, isWarning) {
+      $('pod-row').showBannerMessage(message, isWarning);
     },
 
     /**
@@ -348,11 +427,19 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
     },
 
     /**
-     * Sets the state of touch view mode.
-     * @param {boolean} isTouchViewEnabled true if the mode is on.
+     * Sets the state of tablet mode.
+     * @param {boolean} isTabletModeEnabled true if the mode is on.
      */
-    setTouchViewState: function(isTouchViewEnabled) {
-      $('pod-row').setTouchViewState(isTouchViewEnabled);
+    setTabletModeState: function(isTabletModeEnabled) {
+      $('pod-row').setTabletModeState(isTabletModeEnabled);
+    },
+
+    /**
+     * Sets whether the device is in demo mode.
+     * @param {boolean} isDeviceInDemoMode true if the device is in demo mode.
+     */
+    setDemoModeState: function(isDeviceInDemoMode) {
+      $('pod-row').setDemoModeState(isDeviceInDemoMode);
     },
 
     /**
@@ -382,10 +469,8 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
      * @param {boolean} multipleRecommendedLocales Whether |locales| contains
      *     two or more recommended locales
      */
-    setPublicSessionLocales: function(userID,
-                                      locales,
-                                      defaultLocale,
-                                      multipleRecommendedLocales) {
+    setPublicSessionLocales: function(
+        userID, locales, defaultLocale, multipleRecommendedLocales) {
       $('pod-row').setPublicSessionLocales(userID,
                                            locales,
                                            defaultLocale,
@@ -418,7 +503,6 @@ login.createScreen('AccountPickerScreen', 'account-picker', function() {
           this.lockScreenAppsState_ === LOCK_SCREEN_APPS_STATE.FOREGROUND;
       this.lockScreenAppsState_ = state;
 
-      $('login-header-bar').lockScreenAppsState = state;
       $('top-header-bar').lockScreenAppsState = state;
 
       // Reset the focused pod if app window is being shown on top of the user
