@@ -199,7 +199,7 @@ void InnerBoundedLabel::SetText(const base::string16& new_text) {
 }
 
 int InnerBoundedLabel::GetTextFlags() {
-  int flags = gfx::Canvas::MULTI_LINE | gfx::Canvas::CHARACTER_BREAK;
+  int flags = gfx::Canvas::MULTI_LINE | gfx::Canvas::CHARACTER_BREAKABLE;
 
   // We can't use subpixel rendering if the background is non-opaque.
   if (SkColorGetA(background_color()) != 0xFF)
@@ -303,6 +303,10 @@ int BoundedLabel::GetLineLimit() const {
   return line_limit_;
 }
 
+const gfx::FontList& BoundedLabel::font_list() const {
+  return label_->font_list();
+}
+
 int BoundedLabel::GetLinesForWidthAndLimit(int width, int limit) {
   return visible() ? label_->GetLinesForWidthAndLimit(width, limit) : 0;
 }
