@@ -4,12 +4,13 @@
 
 #include "ui/views/accessibility/ax_aura_obj_cache.h"
 
-#include "base/memory/singleton.h"
+#include "base/no_destructor.h"
 #include "base/strings/string_util.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/client/focus_client.h"
 #include "ui/aura/window.h"
 #include "ui/views/accessibility/ax_aura_obj_wrapper.h"
+#include "ui/views/accessibility/ax_aura_window_utils.h"
 #include "ui/views/accessibility/ax_view_obj_wrapper.h"
 #include "ui/views/accessibility/ax_widget_obj_wrapper.h"
 #include "ui/views/accessibility/ax_window_obj_wrapper.h"
@@ -18,6 +19,7 @@
 #include "ui/views/widget/widget_delegate.h"
 
 namespace views {
+namespace {
 
 aura::client::FocusClient* GetFocusClient(aura::Window* root_window) {
   if (!root_window)
@@ -25,12 +27,18 @@ aura::client::FocusClient* GetFocusClient(aura::Window* root_window) {
   return aura::client::GetFocusClient(root_window);
 }
 
+}  // namespace
+
 // static
 AXAuraObjCache* AXAuraObjCache::GetInstance() {
-  return base::Singleton<AXAuraObjCache>::get();
+  static base::NoDestructor<AXAuraObjCache> instance;
+  return instance.get();
 }
 
 AXAuraObjWrapper* AXAuraObjCache::GetOrCreate(View* view) {
+  // Avoid problems with transient focus events. https://crbug.com/729449
+  if (!view->GetWidget())
+    return nullptr;
   return CreateInternal<AXViewObjWrapper>(view, view_to_id_map_);
 }
 
@@ -89,10 +97,8 @@ AXAuraObjWrapper* AXAuraObjCache::Get(int32_t id) {
 
 void AXAuraObjCache::GetTopLevelWindows(
     std::vector<AXAuraObjWrapper*>* children) {
-  for (const auto& it : window_to_id_map_) {
-    if (!it.first->parent())
-      children->push_back(GetOrCreate(it.first));
-  }
+  for (aura::Window* root : root_windows_)
+    children->push_back(GetOrCreate(root));
 }
 
 AXAuraObjWrapper* AXAuraObjCache::GetFocus() {
@@ -116,30 +122,34 @@ void AXAuraObjCache::FireEvent(AXAuraObjWrapper* aura_obj,
 
 AXAuraObjCache::AXAuraObjCache() = default;
 
-AXAuraObjCache::~AXAuraObjCache() {
-  is_destroying_ = true;
-  cache_.clear();
-}
+// Never runs because object is leaked.
+AXAuraObjCache::~AXAuraObjCache() = default;
 
 View* AXAuraObjCache::GetFocusedView() {
-  if (root_windows_.empty())
-    return nullptr;
-  aura::client::FocusClient* focus_client =
-      GetFocusClient(*root_windows_.begin());
-  if (!focus_client)
-    return nullptr;
+  Widget* focused_widget = focused_widget_for_testing_;
+  aura::Window* focused_window = nullptr;
+  if (!focused_widget) {
+    if (root_windows_.empty())
+      return nullptr;
+    aura::client::FocusClient* focus_client =
+        GetFocusClient(*root_windows_.begin());
+    if (!focus_client)
+      return nullptr;
 
-  aura::Window* focused_window = focus_client->GetFocusedWindow();
-  if (!focused_window)
-    return nullptr;
-
-  Widget* focused_widget = Widget::GetWidgetForNativeView(focused_window);
-  while (!focused_widget) {
-    focused_window = focused_window->parent();
+    focused_window = focus_client->GetFocusedWindow();
     if (!focused_window)
-      break;
+      return nullptr;
 
-    focused_widget = Widget::GetWidgetForNativeView(focused_window);
+    // SingleProcessMash may need to jump between ash and client windows.
+    AXAuraWindowUtils* window_utils = AXAuraWindowUtils::Get();
+    focused_widget = window_utils->GetWidgetForNativeView(focused_window);
+    while (!focused_widget) {
+      focused_window = focused_window->parent();
+      if (!focused_window)
+        break;
+
+      focused_widget = window_utils->GetWidgetForNativeView(focused_window);
+    }
   }
 
   if (!focused_widget)
@@ -153,7 +163,8 @@ View* AXAuraObjCache::GetFocusedView() {
   if (focused_view)
     return focused_view;
 
-  if (focused_window->GetProperty(
+  if (focused_window &&
+      focused_window->GetProperty(
           aura::client::kAccessibilityFocusFallsbackToWidgetKey)) {
     // If focused widget has non client view, falls back to first child view of
     // its client view. We don't expect that non client view gets keyboard
@@ -199,8 +210,8 @@ AXAuraObjWrapper* AXAuraObjCache::CreateInternal(
   if (it != aura_view_to_id_map.end())
     return Get(it->second);
 
-  auto wrapper = std::make_unique<AuraViewWrapper>(aura_view);
-  int32_t id = wrapper->GetUniqueId().Get();
+  auto wrapper = std::make_unique<AuraViewWrapper>(this, aura_view);
+  int32_t id = wrapper->GetUniqueId();
   aura_view_to_id_map[aura_view] = id;
   cache_[id] = std::move(wrapper);
   return cache_[id].get();
