@@ -4,52 +4,41 @@
 
 #include "ui/views/animation/ink_drop_host_view.h"
 
-#include "base/memory/ptr_util.h"
+#include "build/build_config.h"
 #include "ui/events/event.h"
 #include "ui/events/scoped_target_handler.h"
 #include "ui/gfx/color_palette.h"
 #include "ui/gfx/geometry/size_conversions.h"
+#include "ui/views/animation/flood_fill_ink_drop_ripple.h"
 #include "ui/views/animation/ink_drop.h"
 #include "ui/views/animation/ink_drop_highlight.h"
 #include "ui/views/animation/ink_drop_impl.h"
 #include "ui/views/animation/ink_drop_mask.h"
 #include "ui/views/animation/ink_drop_stub.h"
 #include "ui/views/animation/square_ink_drop_ripple.h"
+#include "ui/views/controls/focus_ring.h"
 #include "ui/views/style/platform_style.h"
+#include "ui/views/view_properties.h"
 
 namespace views {
-namespace {
-
-// The scale factor to compute the large size of the default
-// SquareInkDropRipple.
-const float kLargeInkDropScale = 1.333f;
-
-// Default opacity of the ink drop when it is visible.
-const float kInkDropVisibleOpacity = 0.175f;
-
-}  // namespace
-
-// static
-constexpr int InkDropHostView::kInkDropSmallCornerRadius;
-constexpr int InkDropHostView::kInkDropLargeCornerRadius;
 
 // An EventHandler that is guaranteed to be invoked and is not prone to
 // InkDropHostView descendents who do not call
-// InkDropHostView::OnGestureEvent().  Only one instance of this class can exist
-// at any given time for each ink drop host view.
-//
-// TODO(bruthig): Consider getting rid of this class.
-class InkDropHostView::InkDropGestureHandler : public ui::EventHandler {
+// InkDropHostView::OnGestureEvent() and InkDropHostView::OnMouseEvent().
+// Only one instance of this class can exist at any given time for each ink drop
+// host view.
+class InkDropHostView::InkDropEventHandler : public ui::EventHandler {
  public:
-  explicit InkDropGestureHandler(InkDropHostView* host_view)
-      : target_handler_(new ui::ScopedTargetHandler(host_view, this)),
+  explicit InkDropEventHandler(InkDropHostView* host_view)
+      : target_handler_(
+            std::make_unique<ui::ScopedTargetHandler>(host_view, this)),
         host_view_(host_view) {}
 
-  ~InkDropGestureHandler() override {}
+  ~InkDropEventHandler() override = default;
 
   // ui::EventHandler:
   void OnGestureEvent(ui::GestureEvent* event) override {
-    if (!host_view_->enabled())
+    if (!host_view_->enabled() || host_view_->ink_drop_mode_ != InkDropMode::ON)
       return;
 
     InkDropState current_ink_drop_state =
@@ -62,8 +51,7 @@ class InkDropHostView::InkDropGestureHandler : public ui::EventHandler {
           return;
         ink_drop_state = InkDropState::ACTION_PENDING;
         // The ui::ET_GESTURE_TAP_DOWN event needs to be marked as handled so
-        // that
-        // subsequent events for the gesture are sent to |this|.
+        // that subsequent events for the gesture are sent to |this|.
         event->SetHandled();
         break;
       case ui::ET_GESTURE_LONG_PRESS:
@@ -98,6 +86,23 @@ class InkDropHostView::InkDropGestureHandler : public ui::EventHandler {
     host_view_->AnimateInkDrop(ink_drop_state, event);
   }
 
+  void OnMouseEvent(ui::MouseEvent* event) override {
+    switch (event->type()) {
+      case ui::ET_MOUSE_ENTERED:
+        host_view_->GetInkDrop()->SetHovered(true);
+        break;
+      case ui::ET_MOUSE_EXITED:
+        host_view_->GetInkDrop()->SetHovered(false);
+        break;
+      case ui::ET_MOUSE_DRAGGED:
+        host_view_->GetInkDrop()->SetHovered(
+            host_view_->GetLocalBounds().Contains(event->location()));
+        break;
+      default:
+        break;
+    }
+  }
+
  private:
   // Allows |this| to handle all GestureEvents on |host_view_|.
   std::unique_ptr<ui::ScopedTargetHandler> target_handler_;
@@ -105,23 +110,11 @@ class InkDropHostView::InkDropGestureHandler : public ui::EventHandler {
   // The host view to cache ui::Events to when animating the ink drop.
   InkDropHostView* host_view_;
 
-  DISALLOW_COPY_AND_ASSIGN(InkDropGestureHandler);
+  DISALLOW_COPY_AND_ASSIGN(InkDropEventHandler);
 };
 
-// static
-
-gfx::Size InkDropHostView::CalculateLargeInkDropSize(
-    const gfx::Size& small_size) {
-  return gfx::ScaleToCeiledSize(gfx::Size(small_size), kLargeInkDropScale);
-}
-
 InkDropHostView::InkDropHostView()
-    : ink_drop_mode_(InkDropMode::OFF),
-      ink_drop_(nullptr),
-      ink_drop_visible_opacity_(
-          PlatformStyle::kUseRipples ? kInkDropVisibleOpacity : 0),
-      old_paint_to_layer_(false),
-      destroying_(false) {}
+    : ink_drop_event_handler_(std::make_unique<InkDropEventHandler>(this)) {}
 
 InkDropHostView::~InkDropHostView() {
   // TODO(bruthig): Improve InkDropImpl to be safer about calling back to
@@ -154,53 +147,42 @@ void InkDropHostView::RemoveInkDropLayer(ui::Layer* ink_drop_layer) {
 }
 
 std::unique_ptr<InkDrop> InkDropHostView::CreateInkDrop() {
-  return CreateDefaultInkDropImpl();
+  return CreateDefaultFloodFillInkDropImpl();
 }
 
 std::unique_ptr<InkDropRipple> InkDropHostView::CreateInkDropRipple() const {
-  return CreateDefaultInkDropRipple(
-      GetMirroredRect(GetContentsBounds()).CenterPoint());
+  return std::make_unique<views::FloodFillInkDropRipple>(
+      size(), gfx::Insets(), GetInkDropCenterBasedOnLastEvent(),
+      GetInkDropBaseColor(), ink_drop_visible_opacity());
 }
 
 std::unique_ptr<InkDropHighlight> InkDropHostView::CreateInkDropHighlight()
     const {
-  return CreateDefaultInkDropHighlight(
-      gfx::RectF(GetMirroredRect(GetContentsBounds())).CenterPoint());
-}
+  auto highlight = std::make_unique<views::InkDropHighlight>(
+      size(), 0, gfx::RectF(GetMirroredRect(GetLocalBounds())).CenterPoint(),
+      GetInkDropBaseColor());
+  // TODO(pbos): Once |ink_drop_highlight_opacity_| is either always set or
+  // callers are using the default InkDropHighlight value then make this a
+  // constructor argument to InkDropHighlight.
+  if (ink_drop_highlight_opacity_)
+    highlight->set_visible_opacity(*ink_drop_highlight_opacity_);
 
-std::unique_ptr<InkDropRipple> InkDropHostView::CreateDefaultInkDropRipple(
-    const gfx::Point& center_point,
-    const gfx::Size& size) const {
-  std::unique_ptr<InkDropRipple> ripple(new SquareInkDropRipple(
-      CalculateLargeInkDropSize(size), kInkDropLargeCornerRadius, size,
-      kInkDropSmallCornerRadius, center_point, GetInkDropBaseColor(),
-      ink_drop_visible_opacity()));
-  return ripple;
-}
-
-std::unique_ptr<InkDropHighlight>
-InkDropHostView::CreateDefaultInkDropHighlight(const gfx::PointF& center_point,
-                                               const gfx::Size& size) const {
-  std::unique_ptr<InkDropHighlight> highlight(new InkDropHighlight(
-      size, kInkDropSmallCornerRadius, center_point, GetInkDropBaseColor()));
-  highlight->set_explode_size(gfx::SizeF(CalculateLargeInkDropSize(size)));
   return highlight;
 }
 
+std::unique_ptr<views::InkDropMask> InkDropHostView::CreateInkDropMask() const {
+  return std::make_unique<views::PathInkDropMask>(size(),
+                                                  GetHighlightPath(this));
+}
+
+SkColor InkDropHostView::GetInkDropBaseColor() const {
+  NOTREACHED();
+  return gfx::kPlaceholderColor;
+}
+
 void InkDropHostView::SetInkDropMode(InkDropMode ink_drop_mode) {
   ink_drop_mode_ = ink_drop_mode;
   ink_drop_ = nullptr;
-
-  if (ink_drop_mode_ != InkDropMode::ON)
-    gesture_handler_ = nullptr;
-  else if (!gesture_handler_)
-    gesture_handler_ = std::make_unique<InkDropGestureHandler>(this);
-}
-
-gfx::Point InkDropHostView::GetInkDropCenterBasedOnLastEvent() const {
-  return last_ripple_triggering_event_
-             ? last_ripple_triggering_event_->location()
-             : GetMirroredRect(GetContentsBounds()).CenterPoint();
 }
 
 void InkDropHostView::AnimateInkDrop(InkDropState state,
@@ -221,11 +203,20 @@ void InkDropHostView::AnimateInkDrop(InkDropState state,
   GetInkDrop()->AnimateToState(state);
 }
 
+void InkDropHostView::ViewHierarchyChanged(
+    const ViewHierarchyChangedDetails& details) {
+  // If we're being removed hide the ink-drop so if we're highlighted now the
+  // highlight won't be active if we're added back again.
+  if (!details.is_add && details.child == this && ink_drop_) {
+    GetInkDrop()->SnapToHidden();
+    GetInkDrop()->SetHovered(false);
+  }
+  View::ViewHierarchyChanged(details);
+}
+
 void InkDropHostView::OnBoundsChanged(const gfx::Rect& previous_bounds) {
   if (ink_drop_)
     ink_drop_->HostSizeChanged(size());
-  if (ink_drop_mask_)
-    ink_drop_mask_->UpdateLayerSize(size());
 }
 
 void InkDropHostView::VisibilityChanged(View* starting_from, bool is_visible) {
@@ -246,30 +237,50 @@ void InkDropHostView::OnBlur() {
   GetInkDrop()->SetFocused(false);
 }
 
-void InkDropHostView::OnMouseEvent(ui::MouseEvent* event) {
-  switch (event->type()) {
-    case ui::ET_MOUSE_ENTERED:
-      GetInkDrop()->SetHovered(true);
-      break;
-    case ui::ET_MOUSE_EXITED:
-      GetInkDrop()->SetHovered(false);
-      break;
-    case ui::ET_MOUSE_DRAGGED:
-      GetInkDrop()->SetHovered(GetLocalBounds().Contains(event->location()));
-      break;
-    default:
-      break;
-  }
-  View::OnMouseEvent(event);
+std::unique_ptr<InkDropImpl> InkDropHostView::CreateDefaultInkDropImpl() {
+  auto ink_drop = std::make_unique<InkDropImpl>(this, size());
+  ink_drop->SetAutoHighlightMode(
+      InkDropImpl::AutoHighlightMode::HIDE_ON_RIPPLE);
+  return ink_drop;
 }
 
-SkColor InkDropHostView::GetInkDropBaseColor() const {
-  NOTREACHED();
-  return gfx::kPlaceholderColor;
+std::unique_ptr<InkDropImpl>
+InkDropHostView::CreateDefaultFloodFillInkDropImpl() {
+  auto ink_drop = std::make_unique<InkDropImpl>(this, size());
+  ink_drop->SetAutoHighlightMode(
+      views::InkDropImpl::AutoHighlightMode::SHOW_ON_RIPPLE);
+  return ink_drop;
 }
 
-std::unique_ptr<views::InkDropMask> InkDropHostView::CreateInkDropMask() const {
-  return nullptr;
+std::unique_ptr<InkDropRipple> InkDropHostView::CreateDefaultInkDropRipple(
+    const gfx::Point& center_point,
+    const gfx::Size& size) const {
+  return CreateSquareInkDropRipple(center_point, size);
+}
+
+std::unique_ptr<InkDropRipple> InkDropHostView::CreateSquareInkDropRipple(
+    const gfx::Point& center_point,
+    const gfx::Size& size) const {
+  auto ripple = std::make_unique<SquareInkDropRipple>(
+      CalculateLargeInkDropSize(size), ink_drop_large_corner_radius_, size,
+      ink_drop_small_corner_radius_, center_point, GetInkDropBaseColor(),
+      ink_drop_visible_opacity());
+  return ripple;
+}
+
+std::unique_ptr<InkDropHighlight>
+InkDropHostView::CreateDefaultInkDropHighlight(const gfx::PointF& center_point,
+                                               const gfx::Size& size) const {
+  return CreateSquareInkDropHighlight(center_point, size);
+}
+
+std::unique_ptr<InkDropHighlight> InkDropHostView::CreateSquareInkDropHighlight(
+    const gfx::PointF& center_point,
+    const gfx::Size& size) const {
+  auto highlight = std::make_unique<InkDropHighlight>(
+      size, ink_drop_small_corner_radius_, center_point, GetInkDropBaseColor());
+  highlight->set_explode_size(gfx::SizeF(CalculateLargeInkDropSize(size)));
+  return highlight;
 }
 
 bool InkDropHostView::HasInkDrop() const {
@@ -287,34 +298,29 @@ InkDrop* InkDropHostView::GetInkDrop() {
   return ink_drop_.get();
 }
 
+gfx::Point InkDropHostView::GetInkDropCenterBasedOnLastEvent() const {
+  return last_ripple_triggering_event_
+             ? last_ripple_triggering_event_->location()
+             : GetMirroredRect(GetContentsBounds()).CenterPoint();
+}
+
 void InkDropHostView::InstallInkDropMask(ui::Layer* ink_drop_layer) {
-// Layer masks don't work on Windows. See crbug.com/713359
-#if !defined(OS_WIN)
   ink_drop_mask_ = CreateInkDropMask();
   if (ink_drop_mask_)
     ink_drop_layer->SetMaskLayer(ink_drop_mask_->layer());
-#endif
 }
 
 void InkDropHostView::ResetInkDropMask() {
   ink_drop_mask_.reset();
 }
 
-std::unique_ptr<InkDropImpl> InkDropHostView::CreateDefaultInkDropImpl() {
-  std::unique_ptr<InkDropImpl> ink_drop =
-      std::make_unique<InkDropImpl>(this, size());
-  ink_drop->SetAutoHighlightMode(
-      views::InkDropImpl::AutoHighlightMode::HIDE_ON_RIPPLE);
-  return ink_drop;
-}
-
-std::unique_ptr<InkDropImpl>
-InkDropHostView::CreateDefaultFloodFillInkDropImpl() {
-  std::unique_ptr<views::InkDropImpl> ink_drop =
-      InkDropHostView::CreateDefaultInkDropImpl();
-  ink_drop->SetAutoHighlightMode(
-      views::InkDropImpl::AutoHighlightMode::SHOW_ON_RIPPLE);
-  return ink_drop;
+// static
+gfx::Size InkDropHostView::CalculateLargeInkDropSize(
+    const gfx::Size& small_size) {
+  // The scale factor to compute the large size of the default
+  // SquareInkDropRipple.
+  constexpr float kLargeInkDropScale = 1.333f;
+  return gfx::ScaleToCeiledSize(gfx::Size(small_size), kLargeInkDropScale);
 }
 
 }  // namespace views
