@@ -10,37 +10,37 @@
 
 #include "base/lazy_instance.h"
 #include "base/logging.h"
-#include "base/memory/ptr_util.h"
 #include "build/build_config.h"
+#include "ui/accessibility/ax_node_data.h"
 #include "ui/gfx/animation/throb_animation.h"
 #include "ui/gfx/canvas.h"
 #include "ui/gfx/color_utils.h"
 #include "ui/gfx/font_list.h"
 #include "ui/gfx/geometry/vector2d.h"
 #include "ui/native_theme/native_theme.h"
-#include "ui/views/animation/flood_fill_ink_drop_ripple.h"
 #include "ui/views/animation/ink_drop.h"
-#include "ui/views/animation/ink_drop_highlight.h"
-#include "ui/views/animation/ink_drop_impl.h"
-#include "ui/views/animation/square_ink_drop_ripple.h"
 #include "ui/views/background.h"
 #include "ui/views/controls/button/label_button_border.h"
 #include "ui/views/controls/button/label_button_label.h"
 #include "ui/views/layout/layout_provider.h"
 #include "ui/views/painter.h"
 #include "ui/views/style/platform_style.h"
+#include "ui/views/view_properties.h"
 #include "ui/views/window/dialog_delegate.h"
 
 namespace views {
+namespace {
+// The length of the hover fade animation.
+constexpr int kHoverAnimationDurationMs = 170;
+}  // namespace
 
 // static
-const int LabelButton::kHoverAnimationDurationMs = 170;
 const char LabelButton::kViewClassName[] = "LabelButton";
 
 LabelButton::LabelButton(ButtonListener* listener,
                          const base::string16& text,
                          int button_context)
-    : CustomButton(listener),
+    : Button(listener),
       image_(new ImageView()),
       label_(new LabelButtonLabel(text, button_context)),
       ink_drop_container_(new InkDropContainerView()),
@@ -55,7 +55,7 @@ LabelButton::LabelButton(ButtonListener* listener,
       style_(STYLE_TEXTBUTTON),
       border_is_themed_border_(true),
       image_label_spacing_(LayoutProvider::Get()->GetDistanceMetric(
-          DISTANCE_RELATED_CONTROL_HORIZONTAL)),
+          DISTANCE_RELATED_LABEL_HORIZONTAL)),
       horizontal_alignment_(gfx::ALIGN_LEFT) {
   SetAnimationDuration(kHoverAnimationDurationMs);
   SetTextInternal(text);
@@ -71,9 +71,6 @@ LabelButton::LabelButton(ButtonListener* listener,
   AddChildView(label_);
   label_->SetAutoColorReadabilityEnabled(false);
   label_->SetHorizontalAlignment(gfx::ALIGN_TO_HEAD);
-
-  // Inset the button focus rect from the actual border; roughly match Windows.
-  SetFocusPainter(Painter::CreateDashedFocusPainterWithInsets(gfx::Insets(3)));
 }
 
 LabelButton::~LabelButton() {}
@@ -189,7 +186,7 @@ gfx::Size LabelButton::CalculatePreferredSize() const {
   label.SetLineHeight(label_->line_height());
   label.SetShadows(label_->shadows());
 
-  if (style_ == STYLE_BUTTON && PlatformStyle::kDefaultLabelButtonHasBoldFont) {
+  if (style_ == STYLE_BUTTON) {
     // Some text appears wider when rendered normally than when rendered bold.
     // Accommodate the widest, as buttons may show bold and shouldn't resize.
     const int current_width = label.GetPreferredSize().width();
@@ -199,22 +196,17 @@ gfx::Size LabelButton::CalculatePreferredSize() const {
   }
 
   // Calculate the required size.
-  const gfx::Size image_size(image_->GetPreferredSize());
-  gfx::Size size(label.GetPreferredSize());
-  if (image_size.width() > 0 && size.width() > 0)
-    size.Enlarge(image_label_spacing_, 0);
-  size.SetToMax(gfx::Size(0, image_size.height()));
-  const gfx::Insets insets(GetInsets());
-  size.Enlarge(image_size.width() + insets.width(), insets.height());
+  const gfx::Size preferred_label_size = label.GetPreferredSize();
+  gfx::Size size = GetUnclampedSizeWithoutLabel();
+  size.Enlarge(preferred_label_size.width(), 0);
 
-  // Make the size at least as large as the minimum size needed by the border.
-  size.SetToMax(border() ? border()->GetMinimumSize() : gfx::Size());
+  // Increase the height of the label (with insets) if larger.
+  size.set_height(std::max(preferred_label_size.height() + GetInsets().height(),
+                           size.height()));
 
-  // Increase the minimum size monotonically with the preferred size.
   size.SetToMax(min_size_);
-  min_size_ = size;
 
-  // Return the largest known size clamped to the maximum size (if valid).
+  // Clamp size to max size (if valid).
   if (max_size_.width() > 0)
     size.set_width(std::min(max_size_.width(), size.width()));
   if (max_size_.height() > 0)
@@ -226,20 +218,24 @@ gfx::Size LabelButton::CalculatePreferredSize() const {
   return cached_preferred_size_;
 }
 
-int LabelButton::GetHeightForWidth(int w) const {
-  w -= GetInsets().width();
-  const gfx::Size image_size(image_->GetPreferredSize());
-  w -= image_size.width();
-  if (image_size.width() > 0 && !GetText().empty())
-    w -= image_label_spacing_;
+int LabelButton::GetHeightForWidth(int width) const {
+  const gfx::Size size_without_label = GetUnclampedSizeWithoutLabel();
+  // Get label height for the remaining width.
+  const int label_height_with_insets =
+      label_->GetHeightForWidth(width - size_without_label.width()) +
+      GetInsets().height();
 
-  int height = std::max(image_size.height(), label_->GetHeightForWidth(w));
-  if (border())
-    height = std::max(height, border()->GetMinimumSize().height());
+  // Height is the larger of size without label and label height with insets.
+  int height = std::max(size_without_label.height(), label_height_with_insets);
 
-  height = std::max(height, min_size_.height());
+  // Make sure height respects min_size_.
+  if (height < min_size_.height())
+    height = min_size_.height();
+
+  // Clamp height to the maximum height (if valid).
   if (max_size_.height() > 0)
-    height = std::min(height, max_size_.height());
+    return std::min(max_size_.height(), height);
+
   return height;
 }
 
@@ -274,7 +270,15 @@ void LabelButton::Layout() {
       label_area.height());
 
   gfx::Point image_origin(child_area.origin());
-  image_origin.Offset(0, (child_area.height() - image_size.height()) / 2);
+  if (label_->multi_line()) {
+    // Right now this code currently only works for CheckBox and RadioButton
+    // descendants that have multi-line enabled for their label.
+    image_origin.Offset(
+        0, std::max(
+               0, (label_->font_list().GetHeight() - image_size.height()) / 2));
+  } else {
+    image_origin.Offset(0, (child_area.height() - image_size.height()) / 2);
+  }
   if (horizontal_alignment_ == gfx::ALIGN_CENTER) {
     const int spacing = (image_size.width() > 0 && label_size.width() > 0) ?
         image_label_spacing_ : 0;
@@ -298,7 +302,7 @@ void LabelButton::Layout() {
   }
 
   label_->SetBoundsRect(label_bounds);
-  CustomButton::Layout();
+  Button::Layout();
 }
 
 const char* LabelButton::GetClassName() const {
@@ -306,15 +310,15 @@ const char* LabelButton::GetClassName() const {
 }
 
 void LabelButton::EnableCanvasFlippingForRTLUI(bool flip) {
-  CustomButton::EnableCanvasFlippingForRTLUI(flip);
+  Button::EnableCanvasFlippingForRTLUI(flip);
   image_->EnableCanvasFlippingForRTLUI(flip);
 }
 
 std::unique_ptr<LabelButtonBorder> LabelButton::CreateDefaultBorder() const {
   if (style_ != Button::STYLE_TEXTBUTTON)
-    return base::MakeUnique<LabelButtonAssetBorder>(style_);
+    return std::make_unique<LabelButtonAssetBorder>(style_);
   std::unique_ptr<LabelButtonBorder> border =
-      base::MakeUnique<LabelButtonBorder>();
+      std::make_unique<LabelButtonBorder>();
   border->set_insets(views::LabelButtonAssetBorder::GetDefaultInsetsForStyle(
       style_));
   return border;
@@ -334,18 +338,14 @@ gfx::Rect LabelButton::GetChildAreaBounds() {
   return GetLocalBounds();
 }
 
-bool LabelButton::ShouldUseFloodFillInkDrop() const {
-  return !GetText().empty();
-}
-
 void LabelButton::OnFocus() {
-  CustomButton::OnFocus();
+  Button::OnFocus();
   // Typically the border renders differently when focused.
   SchedulePaint();
 }
 
 void LabelButton::OnBlur() {
-  CustomButton::OnBlur();
+  Button::OnBlur();
   // Typically the border renders differently when focused.
   SchedulePaint();
 }
@@ -374,29 +374,10 @@ void LabelButton::RemoveInkDropLayer(ui::Layer* ink_drop_layer) {
   ink_drop_container_->RemoveInkDropLayer(ink_drop_layer);
 }
 
-std::unique_ptr<InkDrop> LabelButton::CreateInkDrop() {
-  return ShouldUseFloodFillInkDrop() ? CreateDefaultFloodFillInkDropImpl()
-                                     : CustomButton::CreateInkDrop();
-}
-
-std::unique_ptr<views::InkDropRipple> LabelButton::CreateInkDropRipple() const {
-  return ShouldUseFloodFillInkDrop()
-             ? base::MakeUnique<views::FloodFillInkDropRipple>(
-                   size(), GetInkDropCenterBasedOnLastEvent(),
-                   GetInkDropBaseColor(), ink_drop_visible_opacity())
-             : CreateDefaultInkDropRipple(
-                   image()->GetMirroredBounds().CenterPoint());
-}
-
-std::unique_ptr<views::InkDropHighlight> LabelButton::CreateInkDropHighlight()
-    const {
-  return ShouldUseFloodFillInkDrop()
-             ? base::MakeUnique<views::InkDropHighlight>(
-                   size(), kInkDropSmallCornerRadius,
-                   gfx::RectF(GetLocalBounds()).CenterPoint(),
-                   GetInkDropBaseColor())
-             : CreateDefaultInkDropHighlight(
-                   gfx::RectF(image()->GetMirroredBounds()).CenterPoint());
+void LabelButton::GetAccessibleNodeData(ui::AXNodeData* node_data) {
+  if (is_default())
+    node_data->AddState(ax::mojom::State::kDefault);
+  Button::GetAccessibleNodeData(node_data);
 }
 
 void LabelButton::StateChanged(ButtonState old_state) {
@@ -468,19 +449,17 @@ void LabelButton::UpdateStyleToIndicateDefaultStatus() {
   // never be given default status.
   DCHECK_EQ(cached_normal_font_list_.GetFontSize(),
             label()->font_list().GetFontSize());
-  const bool bold =
-      PlatformStyle::kDefaultLabelButtonHasBoldFont && is_default_;
   // TODO(tapted): This should use style::GetFont(), but this part can just be
   // deleted when default buttons no longer go bold. Colors will need updating
   // still.
-  label_->SetFontList(bold ? cached_default_button_font_list_
-                           : cached_normal_font_list_);
+  label_->SetFontList(is_default_ ? cached_default_button_font_list_
+                                  : cached_normal_font_list_);
   InvalidateLayout();
   ResetLabelEnabledColor();
 }
 
 void LabelButton::UpdateImage() {
-  image_->SetImage(GetImage(state()));
+  image_->SetImage(GetImage(GetVisualState()));
   ResetCachedPreferredSize();
 }
 
@@ -546,11 +525,25 @@ void LabelButton::ResetCachedPreferredSize() {
   cached_preferred_size_ = gfx::Size();
 }
 
+gfx::Size LabelButton::GetUnclampedSizeWithoutLabel() const {
+  const gfx::Size image_size = image_->GetPreferredSize();
+  gfx::Size size = image_size;
+  const gfx::Insets insets(GetInsets());
+  size.Enlarge(insets.width(), insets.height());
+
+  // Accommodate for spacing between image and text if both are present.
+  if (!GetText().empty() && image_size.width() > 0)
+    size.Enlarge(image_label_spacing_, 0);
+
+  // Make the size at least as large as the minimum size needed by the border.
+  if (border())
+    size.SetToMax(border()->GetMinimumSize());
+
+  return size;
+}
+
 void LabelButton::ResetLabelEnabledColor() {
-  const SkColor color =
-      explicitly_set_colors_[state()]
-          ? button_state_colors_[state()]
-          : PlatformStyle::TextColorForButton(button_state_colors_, *this);
+  const SkColor color = button_state_colors_[state()];
   if (state() != STATE_DISABLED && label_->enabled_color() != color)
     label_->SetEnabledColor(color);
 }
