@@ -4,7 +4,7 @@
 
 #include <stddef.h>
 
-#include "base/macros.h"
+#include "base/stl_util.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/third_party/mozilla/url_parse.h"
 #include "url/url_canon.h"
@@ -14,7 +14,19 @@
 
 namespace url {
 
-TEST(URLUtilTest, FindAndCompareScheme) {
+class URLUtilTest : public testing::Test {
+ public:
+  URLUtilTest() = default;
+  ~URLUtilTest() override {
+    // Reset any added schemes.
+    Shutdown();
+  }
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(URLUtilTest);
+};
+
+TEST_F(URLUtilTest, FindAndCompareScheme) {
   Component found_scheme;
 
   // Simple case where the scheme is found and matches.
@@ -63,7 +75,7 @@ TEST(URLUtilTest, FindAndCompareScheme) {
   EXPECT_TRUE(found_scheme == Component(1, 11));
 }
 
-TEST(URLUtilTest, IsStandard) {
+TEST_F(URLUtilTest, IsStandard) {
   const char kHTTPScheme[] = "http";
   EXPECT_TRUE(IsStandard(kHTTPScheme, Component(0, strlen(kHTTPScheme))));
 
@@ -71,7 +83,7 @@ TEST(URLUtilTest, IsStandard) {
   EXPECT_FALSE(IsStandard(kFooScheme, Component(0, strlen(kFooScheme))));
 }
 
-TEST(URLUtilTest, IsReferrerScheme) {
+TEST_F(URLUtilTest, IsReferrerScheme) {
   const char kHTTPScheme[] = "http";
   EXPECT_TRUE(IsReferrerScheme(kHTTPScheme, Component(0, strlen(kHTTPScheme))));
 
@@ -79,14 +91,14 @@ TEST(URLUtilTest, IsReferrerScheme) {
   EXPECT_FALSE(IsReferrerScheme(kFooScheme, Component(0, strlen(kFooScheme))));
 }
 
-TEST(URLUtilTest, AddReferrerScheme) {
+TEST_F(URLUtilTest, AddReferrerScheme) {
   const char kFooScheme[] = "foo";
   EXPECT_FALSE(IsReferrerScheme(kFooScheme, Component(0, strlen(kFooScheme))));
-  AddReferrerScheme(kFooScheme, url::SCHEME_WITHOUT_PORT);
+  AddReferrerScheme(kFooScheme, url::SCHEME_WITH_HOST);
   EXPECT_TRUE(IsReferrerScheme(kFooScheme, Component(0, strlen(kFooScheme))));
 }
 
-TEST(URLUtilTest, GetStandardSchemeType) {
+TEST_F(URLUtilTest, GetStandardSchemeType) {
   url::SchemeType scheme_type;
 
   const char kHTTPScheme[] = "http";
@@ -94,23 +106,23 @@ TEST(URLUtilTest, GetStandardSchemeType) {
   EXPECT_TRUE(GetStandardSchemeType(kHTTPScheme,
                                     Component(0, strlen(kHTTPScheme)),
                                     &scheme_type));
-  EXPECT_EQ(url::SCHEME_WITH_PORT, scheme_type);
+  EXPECT_EQ(url::SCHEME_WITH_HOST_PORT_AND_USER_INFORMATION, scheme_type);
 
   const char kFilesystemScheme[] = "filesystem";
-  scheme_type = url::SCHEME_WITH_PORT;
+  scheme_type = url::SCHEME_WITH_HOST_PORT_AND_USER_INFORMATION;
   EXPECT_TRUE(GetStandardSchemeType(kFilesystemScheme,
                                     Component(0, strlen(kFilesystemScheme)),
                                     &scheme_type));
   EXPECT_EQ(url::SCHEME_WITHOUT_AUTHORITY, scheme_type);
 
   const char kFooScheme[] = "foo";
-  scheme_type = url::SCHEME_WITH_PORT;
+  scheme_type = url::SCHEME_WITH_HOST_PORT_AND_USER_INFORMATION;
   EXPECT_FALSE(GetStandardSchemeType(kFooScheme,
                                      Component(0, strlen(kFooScheme)),
                                      &scheme_type));
 }
 
-TEST(URLUtilTest, ReplaceComponents) {
+TEST_F(URLUtilTest, ReplaceComponents) {
   Parsed parsed;
   RawCanonOutputT<char> output;
   Parsed new_parsed;
@@ -153,7 +165,7 @@ static std::string CheckReplaceScheme(const char* base_url,
   return output_string;
 }
 
-TEST(URLUtilTest, ReplaceScheme) {
+TEST_F(URLUtilTest, ReplaceScheme) {
   EXPECT_EQ("https://google.com/",
             CheckReplaceScheme("http://google.com/", "https"));
   EXPECT_EQ("file://google.com/",
@@ -179,63 +191,95 @@ TEST(URLUtilTest, ReplaceScheme) {
   EXPECT_EQ("about://google.com/",
             CheckReplaceScheme("http://google.com/", "about"));
 
-  EXPECT_EQ("http://example.com/%20hello%20# world",
+  EXPECT_EQ("http://example.com/%20hello%20#%20world",
             CheckReplaceScheme("myscheme:example.com/ hello # world ", "http"));
 }
 
-TEST(URLUtilTest, DecodeURLEscapeSequences) {
+TEST_F(URLUtilTest, DecodeURLEscapeSequences) {
   struct DecodeCase {
     const char* input;
     const char* output;
   } decode_cases[] = {
-    {"hello, world", "hello, world"},
-    {"%01%02%03%04%05%06%07%08%09%0a%0B%0C%0D%0e%0f/",
-     "\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0B\x0C\x0D\x0e\x0f/"},
-    {"%10%11%12%13%14%15%16%17%18%19%1a%1B%1C%1D%1e%1f/",
-     "\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1B\x1C\x1D\x1e\x1f/"},
-    {"%20%21%22%23%24%25%26%27%28%29%2a%2B%2C%2D%2e%2f/",
-     " !\"#$%&'()*+,-.//"},
-    {"%30%31%32%33%34%35%36%37%38%39%3a%3B%3C%3D%3e%3f/",
-     "0123456789:;<=>?/"},
-    {"%40%41%42%43%44%45%46%47%48%49%4a%4B%4C%4D%4e%4f/",
-     "@ABCDEFGHIJKLMNO/"},
-    {"%50%51%52%53%54%55%56%57%58%59%5a%5B%5C%5D%5e%5f/",
-     "PQRSTUVWXYZ[\\]^_/"},
-    {"%60%61%62%63%64%65%66%67%68%69%6a%6B%6C%6D%6e%6f/",
-     "`abcdefghijklmno/"},
-    {"%70%71%72%73%74%75%76%77%78%79%7a%7B%7C%7D%7e%7f/",
-     "pqrstuvwxyz{|}~\x7f/"},
-    // Test un-UTF-8-ization.
-    {"%e4%bd%a0%e5%a5%bd", "\xe4\xbd\xa0\xe5\xa5\xbd"},
+      {"hello, world", "hello, world"},
+      {"%01%02%03%04%05%06%07%08%09%0a%0B%0C%0D%0e%0f/",
+       "\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0B\x0C\x0D\x0e\x0f/"},
+      {"%10%11%12%13%14%15%16%17%18%19%1a%1B%1C%1D%1e%1f/",
+       "\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1B\x1C\x1D\x1e\x1f/"},
+      {"%20%21%22%23%24%25%26%27%28%29%2a%2B%2C%2D%2e%2f/",
+       " !\"#$%&'()*+,-.//"},
+      {"%30%31%32%33%34%35%36%37%38%39%3a%3B%3C%3D%3e%3f/",
+       "0123456789:;<=>?/"},
+      {"%40%41%42%43%44%45%46%47%48%49%4a%4B%4C%4D%4e%4f/",
+       "@ABCDEFGHIJKLMNO/"},
+      {"%50%51%52%53%54%55%56%57%58%59%5a%5B%5C%5D%5e%5f/",
+       "PQRSTUVWXYZ[\\]^_/"},
+      {"%60%61%62%63%64%65%66%67%68%69%6a%6B%6C%6D%6e%6f/",
+       "`abcdefghijklmno/"},
+      {"%70%71%72%73%74%75%76%77%78%79%7a%7B%7C%7D%7e%7f/",
+       "pqrstuvwxyz{|}~\x7f/"},
+      {"%e4%bd%a0%e5%a5%bd", "\xe4\xbd\xa0\xe5\xa5\xbd"},
   };
 
-  for (size_t i = 0; i < arraysize(decode_cases); i++) {
+  for (size_t i = 0; i < base::size(decode_cases); i++) {
     const char* input = decode_cases[i].input;
     RawCanonOutputT<base::char16> output;
-    DecodeURLEscapeSequences(input, strlen(input), &output);
+    DecodeURLEscapeSequences(input, strlen(input),
+                             DecodeURLMode::kUTF8OrIsomorphic, &output);
     EXPECT_EQ(decode_cases[i].output,
               base::UTF16ToUTF8(base::string16(output.data(),
                                                output.length())));
+
+    RawCanonOutputT<base::char16> output_utf8;
+    DecodeURLEscapeSequences(input, strlen(input), DecodeURLMode::kUTF8,
+                             &output_utf8);
+    EXPECT_EQ(decode_cases[i].output,
+              base::UTF16ToUTF8(
+                  base::string16(output_utf8.data(), output_utf8.length())));
   }
 
   // Our decode should decode %00
   const char zero_input[] = "%00";
   RawCanonOutputT<base::char16> zero_output;
-  DecodeURLEscapeSequences(zero_input, strlen(zero_input), &zero_output);
+  DecodeURLEscapeSequences(zero_input, strlen(zero_input), DecodeURLMode::kUTF8,
+                           &zero_output);
   EXPECT_NE("%00", base::UTF16ToUTF8(
       base::string16(zero_output.data(), zero_output.length())));
 
   // Test the error behavior for invalid UTF-8.
-  const char invalid_input[] = "%e4%a0%e5%a5%bd";
-  const base::char16 invalid_expected[4] = {0x00e4, 0x00a0, 0x597d, 0};
-  RawCanonOutputT<base::char16> invalid_output;
-  DecodeURLEscapeSequences(invalid_input, strlen(invalid_input),
-                           &invalid_output);
-  EXPECT_EQ(base::string16(invalid_expected),
-            base::string16(invalid_output.data(), invalid_output.length()));
+  struct Utf8DecodeCase {
+    const char* input;
+    std::vector<base::char16> expected_iso;
+    std::vector<base::char16> expected_utf8;
+  } utf8_decode_cases[] = {
+      // %e5%a5%bd is a valid UTF-8 sequence. U+597D
+      {"%e4%a0%e5%a5%bd",
+       {0x00e4, 0x00a0, 0x00e5, 0x00a5, 0x00bd, 0},
+       {0xfffd, 0x597d, 0}},
+      {"%e5%a5%bd%e4%a0",
+       {0x00e5, 0x00a5, 0x00bd, 0x00e4, 0x00a0, 0},
+       {0x597d, 0xfffd, 0}},
+      {"%e4%a0%e5%bd",
+       {0x00e4, 0x00a0, 0x00e5, 0x00bd, 0},
+       {0xfffd, 0xfffd, 0}},
+  };
+
+  for (const auto& test : utf8_decode_cases) {
+    const char* input = test.input;
+    RawCanonOutputT<base::char16> output_iso;
+    DecodeURLEscapeSequences(input, strlen(input),
+                             DecodeURLMode::kUTF8OrIsomorphic, &output_iso);
+    EXPECT_EQ(base::string16(test.expected_iso.data()),
+              base::string16(output_iso.data(), output_iso.length()));
+
+    RawCanonOutputT<base::char16> output_utf8;
+    DecodeURLEscapeSequences(input, strlen(input), DecodeURLMode::kUTF8,
+                             &output_utf8);
+    EXPECT_EQ(base::string16(test.expected_utf8.data()),
+              base::string16(output_utf8.data(), output_utf8.length()));
+  }
 }
 
-TEST(URLUtilTest, TestEncodeURIComponent) {
+TEST_F(URLUtilTest, TestEncodeURIComponent) {
   struct EncodeCase {
     const char* input;
     const char* output;
@@ -259,7 +303,7 @@ TEST(URLUtilTest, TestEncodeURIComponent) {
      "pqrstuvwxyz%7B%7C%7D~%7F"},
   };
 
-  for (size_t i = 0; i < arraysize(encode_cases); i++) {
+  for (size_t i = 0; i < base::size(encode_cases); i++) {
     const char* input = encode_cases[i].input;
     RawCanonOutputT<char> buffer;
     EncodeURIComponent(input, strlen(input), &buffer);
@@ -268,7 +312,7 @@ TEST(URLUtilTest, TestEncodeURIComponent) {
   }
 }
 
-TEST(URLUtilTest, TestResolveRelativeWithNonStandardBase) {
+TEST_F(URLUtilTest, TestResolveRelativeWithNonStandardBase) {
   // This tests non-standard (in the sense that IsStandard() == false)
   // hierarchical schemes.
   struct ResolveRelativeCase {
@@ -278,50 +322,54 @@ TEST(URLUtilTest, TestResolveRelativeWithNonStandardBase) {
     const char* out;
   } resolve_non_standard_cases[] = {
       // Resolving a relative path against a non-hierarchical URL should fail.
-    {"scheme:opaque_data", "/path", false, ""},
+      {"scheme:opaque_data", "/path", false, ""},
       // Resolving a relative path against a non-standard authority-based base
       // URL doesn't alter the authority section.
-    {"scheme://Authority/", "../path", true, "scheme://Authority/path"},
+      {"scheme://Authority/", "../path", true, "scheme://Authority/path"},
       // A non-standard hierarchical base is resolved with path URL
       // canonicalization rules.
-    {"data:/Blah:Blah/", "file.html", true, "data:/Blah:Blah/file.html"},
-    {"data:/Path/../part/part2", "file.html", true,
-      "data:/Path/../part/file.html"},
+      {"data:/Blah:Blah/", "file.html", true, "data:/Blah:Blah/file.html"},
+      {"data:/Path/../part/part2", "file.html", true,
+       "data:/Path/../part/file.html"},
+      {"data://text/html,payload", "//user:pass@host:33////payload22", true,
+       "data://user:pass@host:33////payload22"},
       // Path URL canonicalization rules also apply to non-standard authority-
       // based URLs.
-    {"custom://Authority/", "file.html", true,
-      "custom://Authority/file.html"},
-    {"custom://Authority/", "other://Auth/", true, "other://Auth/"},
-    {"custom://Authority/", "../../file.html", true,
-      "custom://Authority/file.html"},
-    {"custom://Authority/path/", "file.html", true,
-      "custom://Authority/path/file.html"},
-    {"custom://Authority:NoCanon/path/", "file.html", true,
-      "custom://Authority:NoCanon/path/file.html"},
+      {"custom://Authority/", "file.html", true,
+       "custom://Authority/file.html"},
+      {"custom://Authority/", "other://Auth/", true, "other://Auth/"},
+      {"custom://Authority/", "../../file.html", true,
+       "custom://Authority/file.html"},
+      {"custom://Authority/path/", "file.html", true,
+       "custom://Authority/path/file.html"},
+      {"custom://Authority:NoCanon/path/", "file.html", true,
+       "custom://Authority:NoCanon/path/file.html"},
       // It's still possible to get an invalid path URL.
-    {"custom://Invalid:!#Auth/", "file.html", false, ""},
+      {"custom://Invalid:!#Auth/", "file.html", false, ""},
       // A path with an authority section gets canonicalized under standard URL
       // rules, even though the base was non-standard.
-    {"content://content.Provider/", "//other.Provider", true,
-      "content://other.provider/"},
+      {"content://content.Provider/", "//other.Provider", true,
+       "content://other.provider/"},
+
       // Resolving an absolute URL doesn't cause canonicalization of the
       // result.
-    {"about:blank", "custom://Authority", true, "custom://Authority"},
+      {"about:blank", "custom://Authority", true, "custom://Authority"},
       // Fragment URLs can be resolved against a non-standard base.
-    {"scheme://Authority/path", "#fragment", true,
-      "scheme://Authority/path#fragment"},
-    {"scheme://Authority/", "#fragment", true, "scheme://Authority/#fragment"},
+      {"scheme://Authority/path", "#fragment", true,
+       "scheme://Authority/path#fragment"},
+      {"scheme://Authority/", "#fragment", true,
+       "scheme://Authority/#fragment"},
       // Resolving should fail if the base URL is authority-based but is
       // missing a path component (the '/' at the end).
-    {"scheme://Authority", "path", false, ""},
+      {"scheme://Authority", "path", false, ""},
       // Test resolving a fragment (only) against any kind of base-URL.
-    {"about:blank", "#id42", true, "about:blank#id42" },
-    {"about:blank", " #id42", true, "about:blank#id42" },
-    {"about:blank#oldfrag", "#newfrag", true, "about:blank#newfrag" },
+      {"about:blank", "#id42", true, "about:blank#id42"},
+      {"about:blank", " #id42", true, "about:blank#id42"},
+      {"about:blank#oldfrag", "#newfrag", true, "about:blank#newfrag"},
       // A surprising side effect of allowing fragments to resolve against
       // any URL scheme is we might break javascript: URLs by doing so...
-    {"javascript:alert('foo#bar')", "#badfrag", true,
-      "javascript:alert('foo#badfrag" },
+      {"javascript:alert('foo#bar')", "#badfrag", true,
+       "javascript:alert('foo#badfrag"},
       // In this case, the backslashes will not be canonicalized because it's a
       // non-standard URL, but they will be treated as a path separators,
       // giving the base URL here a path of "\".
@@ -330,10 +378,9 @@ TEST(URLUtilTest, TestResolveRelativeWithNonStandardBase) {
       // either "aaa://a\" or "aaa://a/" since the path is being replaced with
       // the "current directory". But in the context of resolving on data URLs,
       // adding the requested dot doesn't seem wrong either.
-    {"aaa://a\\", "aaa:.", true, "aaa://a\\." }
-  };
+      {"aaa://a\\", "aaa:.", true, "aaa://a\\."}};
 
-  for (size_t i = 0; i < arraysize(resolve_non_standard_cases); i++) {
+  for (size_t i = 0; i < base::size(resolve_non_standard_cases); i++) {
     const ResolveRelativeCase& test_data = resolve_non_standard_cases[i];
     Parsed base_parsed;
     ParsePathURL(test_data.base, strlen(test_data.base), false, &base_parsed);
@@ -353,7 +400,7 @@ TEST(URLUtilTest, TestResolveRelativeWithNonStandardBase) {
   }
 }
 
-TEST(URLUtilTest, TestNoRefComponent) {
+TEST_F(URLUtilTest, TestNoRefComponent) {
   // The hash-mark must be ignored when mailto: scheme is parsed,
   // even if the URL has a base and relative part.
   const char* base = "mailto://to/";
@@ -374,7 +421,7 @@ TEST(URLUtilTest, TestNoRefComponent) {
   EXPECT_FALSE(resolved_parsed.ref.is_valid());
 }
 
-TEST(URLUtilTest, PotentiallyDanglingMarkup) {
+TEST_F(URLUtilTest, PotentiallyDanglingMarkup) {
   struct ResolveRelativeCase {
     const char* base;
     const char* rel;
@@ -422,7 +469,7 @@ TEST(URLUtilTest, PotentiallyDanglingMarkup) {
   }
 }
 
-TEST(URLUtilTest, TestDomainIs) {
+TEST_F(URLUtilTest, TestDomainIs) {
   const struct {
     const char* canonicalized_host;
     const char* lower_ascii_domain;
