@@ -146,6 +146,24 @@ void BrowserAccessibilityManagerQt::FireBlinkEvent(ax::mojom::Event event_type,
         break;
     }
 }
+
+void BrowserAccessibilityManagerQt::FireGeneratedEvent(ui::AXEventGenerator::Event event_type,
+                                                       BrowserAccessibility* node)
+{
+    BrowserAccessibilityQt *iface = static_cast<BrowserAccessibilityQt*>(node);
+
+    switch (event_type) {
+    case ui::AXEventGenerator::Event::VALUE_CHANGED:
+        if (iface->role() == QAccessible::EditableText) {
+            QAccessibleTextUpdateEvent event(iface, -1, QString(), QString());
+            QAccessible::updateAccessibility(&event);
+        }
+        break;
+    default:
+        break;
+    }
+}
+
 #endif // QT_NO_ACCESSIBILITY
 
 }
