@@ -119,7 +119,9 @@ void WebChannelTransport::Uninstall(blink::WebLocalFrame *frame, uint worldId)
     if (qtObjectValue.IsEmpty() || !qtObjectValue->IsObject())
         return;
     v8::Local<v8::Object> qtObject = v8::Local<v8::Object>::Cast(qtObjectValue);
-    qtObject->Delete(gin::StringToV8(isolate, "webChannelTransport"));
+    // FIXME: ?
+    auto whocares = qtObject->Delete(context, gin::StringToV8(isolate, "webChannelTransport"));
+    Q_UNUSED(whocares);
 }
 
 void WebChannelTransport::NativeQtSendMessage(gin::Arguments *args)
@@ -137,6 +139,8 @@ void WebChannelTransport::NativeQtSendMessage(gin::Arguments *args)
         args->ThrowTypeError("Missing argument");
         return;
     }
+    v8::Isolate *isolate = blink::MainThreadIsolate();
+    v8::HandleScope handleScope(isolate);
 
     if (!jsonValue->IsString()) {
         args->ThrowTypeError("Expected string");
@@ -144,10 +148,10 @@ void WebChannelTransport::NativeQtSendMessage(gin::Arguments *args)
     }
     v8::Local<v8::String> jsonString = v8::Local<v8::String>::Cast(jsonValue);
 
-    QByteArray json(jsonString->Utf8Length(), 0);
-    jsonString->WriteUtf8(json.data(), json.size(),
-                         nullptr,
-                         v8::String::REPLACE_INVALID_UTF8);
+    QByteArray json(jsonString->Utf8Length(isolate), 0);
+    jsonString->WriteUtf8(isolate,
+                          json.data(), json.size(),
+                          nullptr, v8::String::REPLACE_INVALID_UTF8);
 
     QJsonParseError error;
     QJsonDocument doc = QJsonDocument::fromJson(json, &error);
