@@ -28,6 +28,7 @@
 
 #include "testhandler.h"
 #include "server.h"
+#include "util.h"
 #include <QtWebEngine/private/qquickwebenginedialogrequests_p.h>
 #include <QtWebEngine/private/qquickwebenginecontextmenurequest_p.h>
 #include <QQuickWebEngineProfile>
@@ -43,6 +44,7 @@ class tst_Dialogs : public QObject {
 public:
     tst_Dialogs(){}
 
+
 private slots:
     void initTestCase();
     void init();
@@ -64,7 +66,6 @@ private:
 
 void tst_Dialogs::initTestCase()
 {
-    QtWebEngine::initialize();
     QQuickWebEngineProfile::defaultProfile()->setOffTheRecord(true);
     qmlRegisterType<TestHandler>("io.qt.tester", 1, 0, "TestHandler");
     m_engine.reset(new QQmlApplicationEngine());
@@ -230,5 +231,5 @@ void tst_Dialogs::javaScriptDialogRequested()
 }
 
 #include "tst_dialogs.moc"
-QTEST_MAIN(tst_Dialogs)
+W_QTEST_MAIN(tst_Dialogs)
 
