@@ -172,6 +172,7 @@ private Q_SLOTS:
     void cleanupTestCase();
 
     void jsUrlCanon();
+    void jsUrlRelative();
     void jsUrlOrigin();
     void subdirWithAccess();
     void subdirWithoutAccess();
@@ -281,6 +282,54 @@ void tst_Origins::jsUrlCanon()
              QVariant(QSL("hostportanduserinformationsyntax://a:b@foo/bar")));
 }
 
+// Test relative URL resolution.
+void tst_Origins::jsUrlRelative()
+{
+    QVERIFY(load(QSL("about:blank")));
+
+    // Schemes with hosts, like http, work as expected.
+    QCOMPARE(eval(QSL("new URL('bar', 'http://foo').href")), QVariant(QSL("http://foo/bar")));
+    QCOMPARE(eval(QSL("new URL('baz', 'http://foo/bar').href")), QVariant(QSL("http://foo/baz")));
+    QCOMPARE(eval(QSL("new URL('baz', 'http://foo/bar/').href")), QVariant(QSL("http://foo/bar/baz")));
+    QCOMPARE(eval(QSL("new URL('/baz', 'http://foo/bar/').href")), QVariant(QSL("http://foo/baz")));
+    QCOMPARE(eval(QSL("new URL('./baz', 'http://foo/bar/').href")), QVariant(QSL("http://foo/bar/baz")));
+    QCOMPARE(eval(QSL("new URL('../baz', 'http://foo/bar/').href")), QVariant(QSL("http://foo/baz")));
+    QCOMPARE(eval(QSL("new URL('../../baz', 'http://foo/bar/').href")), QVariant(QSL("http://foo/baz")));
+    QCOMPARE(eval(QSL("new URL('//baz', 'http://foo/bar/').href")), QVariant(QSL("http://baz/")));
+
+    // In the case of schemes without hosts, relative URLs only work if the URL
+    // starts with a single slash -- and canonicalization does not guarantee
+    // this. The following cases all fail with TypeErrors.
+    QCOMPARE(eval(QSL("new URL('bar', 'tst:foo').href")), QVariant());
+    QCOMPARE(eval(QSL("new URL('baz', 'tst:foo/bar').href")), QVariant());
+    QCOMPARE(eval(QSL("new URL('bar', 'tst://foo').href")), QVariant());
+    QCOMPARE(eval(QSL("new URL('bar', 'tst:///foo').href")), QVariant());
+
+    // However, registered custom schemes have been patched to allow relative
+    // URLs even without an initial slash.
+    QCOMPARE(eval(QSL("new URL('bar', 'qrc:foo').href")), QVariant(QSL("qrc:bar")));
+    QCOMPARE(eval(QSL("new URL('baz', 'qrc:foo/bar').href")), QVariant(QSL("qrc:foo/baz")));
+    QCOMPARE(eval(QSL("new URL('bar', 'qrc://foo').href")), QVariant());
+    QCOMPARE(eval(QSL("new URL('bar', 'qrc:///foo').href")), QVariant());
+
+    // With a slash it works the same as http except 'foo' is part of the path and not the host.
+    QCOMPARE(eval(QSL("new URL('bar', 'qrc:/foo').href")), QVariant(QSL("qrc:/bar")));
+    QCOMPARE(eval(QSL("new URL('bar', 'qrc:/foo/').href")), QVariant(QSL("qrc:/foo/bar")));
+    QCOMPARE(eval(QSL("new URL('baz', 'qrc:/foo/bar').href")), QVariant(QSL("qrc:/foo/baz")));
+    QCOMPARE(eval(QSL("new URL('baz', 'qrc:/foo/bar/').href")), QVariant(QSL("qrc:/foo/bar/baz")));
+    QCOMPARE(eval(QSL("new URL('/baz', 'qrc:/foo/bar/').href")), QVariant(QSL("qrc:/baz")));
+    QCOMPARE(eval(QSL("new URL('./baz', 'qrc:/foo/bar/').href")), QVariant(QSL("qrc:/foo/bar/baz")));
+    QCOMPARE(eval(QSL("new URL('../baz', 'qrc:/foo/bar/').href")), QVariant(QSL("qrc:/foo/baz")));
+    QCOMPARE(eval(QSL("new URL('../../baz', 'qrc:/foo/bar/').href")), QVariant(QSL("qrc:/baz")));
+    QCOMPARE(eval(QSL("new URL('../../../baz', 'qrc:/foo/bar/').href")), QVariant(QSL("qrc:/baz")));
+
+    // If the relative URL begins with >= 2 slashes, then the scheme is treated
+    // not as a Syntax::Path scheme but as a Syntax::HostPortAndUserInformation
+    // scheme.
+    QCOMPARE(eval(QSL("new URL('//baz', 'qrc:/foo/bar/').href")), QVariant(QSL("qrc://baz/")));
+    QCOMPARE(eval(QSL("new URL('///baz', 'qrc:/foo/bar/').href")), QVariant(QSL("qrc://baz/")));
+}
+
 // Test origin serialization in Blink, implemented by blink::KURL and
 // blink::SecurityOrigin as opposed to GURL and url::Origin.
 void tst_Origins::jsUrlOrigin()
@@ -301,7 +350,7 @@ void tst_Origins::jsUrlOrigin()
     QCOMPARE(eval(QSL("new URL(\"qrc:/crysis.css\").origin")), QVariant(QSL("qrc://")));
     QCOMPARE(eval(QSL("new URL(\"qrc://foo.com/crysis.css\").origin")), QVariant(QSL("qrc://")));
 
-    // Same with unregistered schemes.
+    // Unregistered schemes behaves like opaque origins.
     QCOMPARE(eval(QSL("new URL(\"tst:/banana\").origin")), QVariant(QSL("tst://")));
     QCOMPARE(eval(QSL("new URL(\"tst://foo.com/banana\").origin")), QVariant(QSL("tst://")));
 
@@ -564,8 +613,6 @@ private:
 // Try opening a WebSocket from pages loaded over various URL schemes.
 void tst_Origins::webSocket()
 {
-    const int kAbnormalClosure = 1006;
-
     EchoServer echoServer;
     QWebChannel channel;
     channel.registerObject(QSL("echoServer"), &echoServer);
@@ -578,9 +625,9 @@ void tst_Origins::webSocket()
     QVERIFY(load(QSL("qrc:/resources/websocket.html")));
     QTRY_COMPARE(eval(QSL("result")), QVariant(QSL("ok")));
 
-    // Only registered schemes can open WebSockets.
+    // Unregistered schemes can also open WebSockets (since Chromium 71)
     QVERIFY(load(QSL("tst:/resources/websocket.html")));
-    QTRY_COMPARE(eval(QSL("result")), QVariant(kAbnormalClosure));
+    QTRY_COMPARE(eval(QSL("result")), QVariant(QSL("ok")));
 
     // Even an insecure registered scheme can open WebSockets.
     QVERIFY(load(QSL("PathSyntax:/resources/websocket.html")));
@@ -599,11 +646,10 @@ void tst_Origins::dedicatedWorker()
     QTRY_VERIFY(eval(QSL("done")).toBool());
     QCOMPARE(eval(QSL("result")), QVariant(42));
 
-    // Unregistered schemes cannot create Workers.
+    // Unregistered schemes can also create Workers (since Chromium 71)
     QVERIFY(load(QSL("tst:/resources/dedicatedWorker.html")));
     QTRY_VERIFY(eval(QSL("done")).toBool());
-    QVERIFY(eval(QSL("error")).toString()
-            .contains(QSL("Access to dedicated workers is denied to origin 'tst://'")));
+    QCOMPARE(eval(QSL("result")), QVariant(42));
 
     // Even an insecure registered scheme can create Workers.
     QVERIFY(load(QSL("PathSyntax:/resources/dedicatedWorker.html")));
@@ -640,10 +686,7 @@ void tst_Origins::sharedWorker()
     QTRY_VERIFY(eval(QSL("done")).toBool());
     QCOMPARE(eval(QSL("result")), QVariant(42));
 
-    // Even unregistered schemes can create SharedWorkers.
-    QVERIFY(load(QSL("tst:/resources/sharedWorker.html")));
-    QTRY_VERIFY(eval(QSL("done")).toBool());
-    QCOMPARE(eval(QSL("result")), QVariant(42));
+    // Unregistered schemes should not create SharedWorkers.
 
     QVERIFY(load(QSL("PathSyntax:/resources/sharedWorker.html")));
     QTRY_VERIFY(eval(QSL("done")).toBool());
@@ -727,15 +770,9 @@ void tst_Origins::createObjectURL()
     QVERIFY(load(QSL("qrc:/resources/createObjectURL.html")));
     QVERIFY(eval(QSL("result")).toString().startsWith(QSL("blob:qrc:")));
 
-    // Illegal for unregistered schemes (renderer gets terminated).
-    qRegisterMetaType<QWebEnginePage::RenderProcessTerminationStatus>("RenderProcessTerminationStatus");
-    QSignalSpy loadFinishedSpy(m_page, &QWebEnginePage::loadFinished);
-    QSignalSpy renderProcessTerminatedSpy(m_page, &QWebEnginePage::renderProcessTerminated);
-    m_page->load(QSL("tst:/resources/createObjectURL.html"));
-    QVERIFY(!renderProcessTerminatedSpy.empty() || renderProcessTerminatedSpy.wait(20000));
-    QVERIFY(renderProcessTerminatedSpy.front().value(0).value<QWebEnginePage::RenderProcessTerminationStatus>()
-            != QWebEnginePage::NormalTerminationStatus);
-    QVERIFY(loadFinishedSpy.empty());
+    // Also legal for unregistered schemes (since Chromium 71)
+    QVERIFY(load(QSL("tst:/resources/createObjectURL.html")));
+    QVERIFY(eval(QSL("result")).toString().startsWith(QSL("blob:tst:")));
 }
 
 QTEST_MAIN(tst_Origins)
